(setq hasMilliSecs (/= (getvar "MILLISECS") nil))
(setvar "LOGFILEMODE" 0)
(vl-load-com)

(if vlisp-optimiser (vlisp-optimiser nil)) ;; disable VLisp-Optimiser, if present

(defun emul-vla-collection2list ( coll / item lst )
  (vlax-for item coll
    (setq lst (cons item lst))
  )
  (reverse lst)
)
(if (not vla-collection->list) (setq vla-collection->list emul-vla-collection2list))

(defun t_vla_fastcom_collection2list ( count fptr /  app cur col time1 time2 time3 time4 )

  ;;--- build data
  (setq app (vlax-get-acad-object))
  (setq cur (vla-get-activedocument app))
  (setq col (vla-get-layouts cur))

  ;;--- now run test : NORMAL
  (if lisp$enablefastcom (lisp$enablefastcom nil)) ;; disable FAST COM, if present
  (setq time1 (getvar "MILLISECS"))
  (if (not time1) (setq time1 (getvar "TDUSRTIMER")))
  (if (not time1) (setq time1 0.0))

  (repeat (* count count)
    (emul-vla-collection2list col)
  )

  (setq time2 (getvar "MILLISECS"))
  (if (not time2) (setq time2 (getvar "TDUSRTIMER")))
  (if (not time2) (setq time2 0.0))

  ;;--- now run test : VLE
  (if lisp$enablefastcom (lisp$enablefastcom t)) ;; enable FAST COM, if present
  (setq time3 (getvar "MILLISECS"))
  (if (not time3) (setq time3 (getvar "TDUSRTIMER")))
  (if (not time3) (setq time3 0.0))

  (repeat (* count count)
    (vla-collection->list col)
  )

  (setq time4 (getvar "MILLISECS"))
  (if (not time4) (setq time4 (getvar "TDUSRTIMER")))
  (if (not time4) (setq time4 0.0))

  ;;--- test results
  (setq bench$secsNorm (if hasMilliSecs (* 0.001 (- time2 time1)) (* 86400.0 (- time2 time1))))
  (setq bench$secsFast (if hasMilliSecs (* 0.001 (- time4 time3)) (* 86400.0 (- time4 time3))))
  (setq bench$seconds (+ bench$secsNorm bench$secsFast))

  (princ (strcat "* VLA-COLLECTION->LIST : " (rtos bench$secsNorm 2 3) "  (fast COM) : " (rtos bench$secsFast 2 3) "\n"))
  (if fptr
    (princ (strcat "* VLA-COLLECTION->LIST : " (rtos bench$secsNorm 2 3) "  (fast COM) : " (rtos bench$secsFast 2 3) "\n") fptr)
  )

  bench$seconds
)

;; to increase/decrease the number of bench loops : > 1 increase < 1 decrease
(if (not $benchcyclesfactor) (setq $benchcyclesfactor 1))

;; to run this loaded test manually
(defun C:Run ()
  (setvar "cmdecho" 0)
  (print)
  (t_vla_fastcom_collection2list (* 300 $benchcyclesfactor) nil)
  (setvar "cmdecho" 1)
  (princ)
)
