(setq hasMilliSecs (/= (getvar "MILLISECS") nil))
(setvar "LOGFILEMODE" 0)
(vl-load-com)

(if (and (not vle-vector-angletoref) (findfile "vle-extension.lsp")) (load "vle-extension.lsp"))

(defun emul-vector-crossproduct ( vec1 vec2 / vX1 vY1 vZ1 vX2 vY2 vZ2 )
  (if (or (/= (type vec1) 'LIST) (/= (type vec2) 'LIST)) (exit))

  (setq vX1 (car vec1) vY1 (cadr vec1))
  (setq vX2 (car vec2) vY2 (cadr vec2))
  (if (or (not vX1) (not vY1) (not vX2) (not vY2)) (exit))

  (setq vZ1 (caddr vec1) vZ2 (caddr vec2))
  (if (not vZ1) (setq vZ1 0.0))
  (if (not vZ2) (setq vZ2 0.0))

  (list (- (* vY1 vZ2) (* vZ1 vY2))
        (- (* vZ1 vX2) (* vX1 vZ2))
        (- (* vX1 vY2) (* vY1 vX2))
  )
)
(defun emul-vector-dotproduct ( vec1 vec2 / vX1 vY1 vZ1 vX2 vY2 vZ2 )
  (if (or (/= (type vec1) 'LIST) (/= (type vec2) 'LIST)) (exit))

  (setq vX1 (car vec1) vY1 (cadr vec1))
  (setq vX2 (car vec2) vY2 (cadr vec2))
  (if (or (not vX1) (not vY1) (not vX2) (not vY2)) (exit))

  (setq vZ1 (caddr vec1) vZ2 (caddr vec2))
  (if (not vZ1) (setq vZ1 0.0))
  (if (not vZ2) (setq vZ2 0.0))

  (+ (* vX1 vX2)
     (* vY1 vY2)
     (* vZ1 vZ2)
  )
)
(defun emul-vector-length ( vec / vX vY vZ )
  (if (/= (type vec) 'LIST) (exit))

  (setq vX (car vec) vY (cadr vec))
  (if (or (not vX) (not vY)) (exit))

  (setq vZ (caddr vec))
  (if (not vZ) (setq vZ 0.0))

  (sqrt (+ (* vX vX) (* vY vY) (* vZ vZ)))
)

(defun emul-vector-angletoref ( vec1 vec2 normal / crossProd dotProd len ang ref )
  (setq crossProd (emul-vector-crossproduct vec1 vec2))
  (setq dotProd (emul-vector-dotproduct vec1 vec2))
  (setq len (emul-vector-length crossProd))
  (setq ang (atan len dotProd))

  (setq ref (emul-vector-dotproduct normal crossProd))
  (if (>= ref 0.0)
    ang
    (- (* 2 PI) ang)
  )
)

(defun t_vle_vector_angletoref ( count fptr / v2d v3d normal time1 time2 time3 time4 )

  ;;--- build data list
  (setq v2d '(2.0 1.0) v3d '(-2.0 -3.0 0) normal '(0 0 ))

  ;;--- now run test : NORMAL
  (setq time1 (getvar "MILLISECS"))
  (if (not time1) (setq time1 (getvar "TDUSRTIMER")))
  (if (not time1) (setq time1 0.0))

  (repeat (* count count)
    (emul-vector-angletoref v2d v3d normal)
    (emul-vector-angletoref v3d v2d normal)
  )

  (setq time2 (getvar "MILLISECS"))
  (if (not time2) (setq time2 (getvar "TDUSRTIMER")))
  (if (not time2) (setq time2 0.0))

  ;;--- now run test : VLE
  (setq time3 (getvar "MILLISECS"))
  (if (not time3) (setq time3 (getvar "TDUSRTIMER")))
  (if (not time3) (setq time3 0.0))

  (repeat (* count count)
    (vle-vector-angletoref v2d v3d normal)
    (vle-vector-angletoref v3d v2d normal)
  )

  (setq time4 (getvar "MILLISECS"))
  (if (not time4) (setq time4 (getvar "TDUSRTIMER")))
  (if (not time4) (setq time4 0.0))

  ;;--- test results
  (setq bench$secsNorm (if hasMilliSecs (* 0.001 (- time2 time1)) (* 86400.0 (- time2 time1))))
  (setq bench$secsFast (if hasMilliSecs (* 0.001 (- time4 time3)) (* 86400.0 (- time4 time3))))
  (setq bench$seconds (+ bench$secsNorm bench$secsFast))

  (princ (strcat "* (Standard) : " (rtos bench$secsNorm 2 3) "  VLE-VECTOR-ANGLETOREF : " (rtos bench$secsFast 2 3) "\n"))
  (if fptr
    (princ (strcat "* (Standard) : " (rtos bench$secsNorm 2 3) "  VLE-VECTOR-ANGLETOREF : " (rtos bench$secsFast 2 3) "\n") fptr)
  )

  bench$seconds
)

;; to increase/decrease the number of bench loops : > 1 increase < 1 decrease
(if (not $benchcyclesfactor) (setq $benchcyclesfactor 1))

;; to run this loaded test manually
(defun C:Run ()
  (setvar "cmdecho" 0)
  (print)
  (t_vle_vector_angletoref (* 300 $benchcyclesfactor) nil)
  (setvar "cmdecho" 1)
  (princ)
)
