
;---SUB_LIST returns a list consisting of the first N1 members of list L1.  [Count starts at 0]

(defun SUB_LIST (L1 N1)
  (cond
    ((null L1) nil)
    ((= N1 -1) nil)
    (t
     (cons (car L1) (SUB_LIST (cdr L1) (1- N1)))
    )
  )
)

;;; Joins strings from list together again.
;;; Takes all types and converts them to strings (safer).
;;; Opposite to std-strsplit.
;;;  (JoinStrings '("1" "2" "3") ",") => "1,2,3"
;;;  (JoinStrings '(1 2 3) ",")       => "1,2,3"
(defun JoinStrings (lst _delim)
  (cond
    ((std-string-list-p lst)
     (substr
       (apply
         (function strcat)      ;force strings
         (mapcar
           (function (lambda (s) (strcat _delim s)))
           lst
         )
       )
       (1+ (strlen _delim))
     )
    )
    (T
     ;(if (not std-princ-to-string))    ;VL optimizer
     (std-%simple-require "STDLISP")

     (substr
       (apply (function strcat)     ;force strings
          (mapcar
              (function (lambda (s)
                          (strcat _delim (std-princ-to-string s))
                        )
              )
              lst
          )
       )
       (1+ (strlen _delim))
     )
    )
  )
)

;;Serge Pashkov

;;; SousSequence - subsequence including start excluding end
;;;   (SousSequence '(0 1 2 3) 1 3) => '(1 2)
;(defun SousSequence (lst start endp)
;  (if (<= 0 start endp (length lst))   ; fixed
;    (std-firstn (- endp start) (std-nthcdr start lst))
;    (std-error (list "SousSequence: "
;            (std-msg "index out of range")
;            " - "
;            start
;            " "
;            endp
;          )
;    )
;  )
;)

;;; (STD-BUTLAST lst) - list without (last lst)
;;; iterative version
(defun STD-BUTLAST (lst)
  (reverse (cdr (reverse lst)))
)

(defun IsNameValid (name)
  (not
    (wcmatch name "*[] `#`@`.`?`*`~`[`,`'!%^&()+={}|`\\:;\"<>/]*")
  )
)

(defun IsCharValid (ch)
  (not
    (wcmatch ch "[] `#`@`.`?`*`~`[`,`'!%^&()+={}|`\\:;\"<>/]")
  )
)

;;;;;;;;;;;;;;
;;;If you take the distances from one point to each of the other three points -
;;;assuming it's a rectangle and not a parallelogram - then the greatest length
;;;will be the diagonal and the other two will be the short and long side
;;;dimensions, regardless of the order of the point list.
;(setq dslst (list (distance pt1 pt2)(distance pt1 pt3)(distance pt1 pt4)))
;(setq dslst (vl-sort  dslst '<)) ;ascending order
;(nth 0 dslst) ;short dimension
;(nth 2 dslst) ;long dimension

;; ! Function : Apply interpolation on a segment of points.
;; !
;; ! Arguments: 'Seg' - List of points to be interpolated
;; !
;; ! Returns  : 'vlist' - The list of interpolated points
;; !
;; ! Comments:  Interpolation is the process of populating zero values lying in
;; !            between two known non-zero values by arriving at these values by
;; !            means of a process of linear interpolation. By this, it means that
;; !            all the intermediate values are obtained by applying a uniform
;; !            rate of change based on distance from the first of the two points
;; !            forming the interpolation boundary.
;; ! Theory:    If A and B are two non-zero elevation points,
;; !            A= x1, y1, z1
;; !            B= x2, y2, z2
;; !            and the distance between then is D,
;; !            then linear interpolation is at any point in between is calculated
;; !            at the rate of (Z2-Z1)/D per unit change of elevation from A

(defun GE_ApplyInterP ( Seg
                        / ZStart ZEnd vlist nlist len Idx PrevPt pt fac CumD Z
                      )
  (setq
    ZStart (nth 2 Seg)
    ZEnd   (nth 3 Seg)
    vlist  (nth 4 Seg)
    len    (GE_PlLen vlist nil T)
    nlist  '()
    Idx    1
    CumD   0.0
    fac    (/ (- ZEnd ZStart) len)
  )

  (foreach pt vlist
    (if (= Idx 1)
      (setq nlist (cons (list (car pt) (cadr pt) ZStart) nlist))
      (setq
            CumD  (+ CumD (distance PrevPt pt))
            Z     (+ ZStart (* CumD fac))
            nlist (cons (list (car pt) (cadr pt) Z) nlist)
      )
    )

    (setq
      PrevPt pt
      Idx    (1+ Idx)
    )
  )
  (reverse nlist)
)

;; ! Function : Determines the mid point of a polyline and draws a point at that
;; !            location
(defun c:MidPoly (/ ent ename entl en oname param len hLen MidPt OS)
  (setq ent (entsel "\nSelect polyline:"))
  (if ent
    (progn
      (setq
        ename (car ent)
        entl  (entget ename)
        en    (cdr (assoc 0 entl))
      )
      (if (member en (list "POLYLINE" "LWPOLYLINE"))
    (progn
      (setq
        oname (vlax-ename->vla-object ename)
        param (vlax-curve-getEndParam oname)
        len   (vlax-curve-getDistAtParam oname param)
        hLen  (* 0.5 len)
        MidPt (vlax-curve-getPointAtDist oname hLen)
      )
      (vlax-release-object oname)
      (setq OS (getvar "OSMODE"))
      (setvar "OSMODE" 0)
      (command "._Point" MidPt)
      (princ "\nPoint object created at mid-point:")
      (princ MidPt)
      (setvar "OSMODE" OS)
    )
    (princ "\nYou must pick a polyline object only.")
      )
    )
  )
  (prin1)
)

;; ! Function : Relocate a number of blocks from one location to new individually specifed locations
(defun c:ReLocate (/ ss ssl tmp ename entl cnt Insp newInsp)
  (princ "\nSelect block objects:")
  (setq ss (ssget (list (cons 0 "INSERT"))))
  (if ss
    (progn
      (setq
         ssl (sslength ss)
         tmp (strcat " of " (itoa ssl))
         cnt 0
      )
      (repeat ssl
        (setq
          ename (ssname ss cnt)
          cnt   (1+ cnt)
          entl  (entget ename)
          Insp  (cdr (assoc 10 entl))
        )
        (initget 1)
        (setq newInsp
               (getpoint Insp
                 (strcat "\nPick new insertion point, block "
                     (itoa cnt)
                     tmp
                     ":"
                 )
               )
        )
        (if (> (distance Insp newInsp) 0.0)
          (command "._Move" ename "" Insp newInsp)
        )
      )
    )
  )
  (prin1)
)

;; ! Function : Returns the first occurence of a DXF dotted pair from a list
;; ! Argument : 'n'     - The DXF code to check
;; !            'alist' - The List to check
;; ! Returns  : The value of the DXF dotted pair, if it exists else returns nil
(defun LI_item (n alist)
  (cdr (assoc n alist))
)

;; ! Function : From a given list of points IN A STRAIGHT LINE, return the two
;; !            most fathest points
;; ! Arguments: 'vlist' - List of points
;; ! Returns  : List of two points which are geometrically the farthest. The first point is to the left of the second.
(defun GE_GetFarthestPoints (vlist / len MinX MaxX vlist1 pt pt1 pt2 FUZZ)
  (setq
    FUZZ  0.0001
    pt1   nil
    pt2   nil
    vlist (GE_DistWeed vlist FUZZ "Horiz")
  )
  (if vlist
    (progn
      (setq
        vlist (car vlist)
        len   (length vlist)
      )
      (if (>= len 2)
    (progn
      (setq
        pt1 (nth 0 vlist)
        pt2 (nth 1 vlist)
      )
      (if (MI_3PUCS pt1 pt2 nil)
        (progn
          (setq
        vlist1 (mapcar '(lambda (x) (trans x 0 1)) vlist)
        MinX   1E20
        MaxX   -1E20
        pt1    nil
        pt2    nil
          )
          (foreach pt vlist1
        (if (< (car pt) MinX)
          (setq
            MinX (car pt)
            pt1  pt
          )
        )

        (if (> (car pt) MaxX)
          (setq
            MaxX (car pt)
            pt2  pt
          )
        )
          )
          (setq
        pt1 (trans pt1 1 0)
        pt2 (trans pt2 1 0)
          )
          (command "._Ucs" "_Previous")
        )
        (setq
          pt1 nil
          pt2 nil
        )
      )
    )
      )
    )
  )

  (if (and pt1 pt2)
    (list pt1 pt2)
    nil
  )
)

;; ! Argument :
;; !            'ss'       - selection set to be checked.
;; !            'ChkXdata' - If T, verify Xdata equivalence also.
;; !            'ChkLyr'   - If T, verify Layer also.
;; !            'Diff'     - Fuzzy tolerance value
;; ! Action   : Checks if there are duplicate objects in the selection set.
;; !            Additionally, the duplication can be detected on the basis of
;; !            Xdata and/or layer match.
;; ! Updated  : April 16, 1998
;; ! Comments : Uses append, not efficient, needs changing over to cons
(defun MI_ChkDup (ss         ChkXdata   ChkLyr     Diff
          /      EllipseLst ImageLst   InsertLst
          LeaderLst  LineLst    MlineLst   MtextLst
          RegionLst  PointLst   PolylineLst
          ShapeLst   SolidLst   SplineLst  TextLst
          TraceLst   3DFaceLst  3DSolidLst ArcLst
          AttdefLst  CircleLst  DimensionLst
          DupSS      xd     Lst    cnt
          en         ename  entl       ETok1
          ETok2      ETok   la     ssl
          vlist      tmp    tmp1
         )
  (setq DupSS (ssadd))
  (if ss
    (progn
      (setq
    Lst      '()
    EllipseLst   '()
    ImageLst     '()
    InsertLst    '()
    LeaderLst    '()
    LineLst      '()
    MlineLst     '()
    MtextLst     '()
    RegionLst    '()
    PointLst     '()
    PolylineLst  '()
    ShapeLst     '()
    SolidLst     '()
    SplineLst    '()
    TextLst      '()
    TraceLst     '()
    3DFaceLst    '()
    3DSolidLst   '()
    ArcLst       '()
    AttdefLst    '()
    CircleLst    '()
    DimensionLst '()
    ssl      (sslength ss)
    tmp      (strcat " of " (itoa ssl))
    cnt      0
      )
      (princ "\n")
      (repeat ssl
    (setq
      ename (ssname ss cnt)
      entl  (entget ename)
      en    (LI_item 0 entl)
      xd    (cdr (assoc -3 (entget ename (list "*"))))
      la    (LI_item 8 entl)
      cnt   (1+ cnt)
    )
    (cond
      ((= en "ELLIPSE")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 40 entl)
            (LI_item 41 entl)
            (LI_item 42 entl)
              )
       )

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and EllipseLst (LI_memb ETok EllipseLst Diff))
         (ssadd ename DupSS)
         (setq EllipseLst (append EllipseLst (list ETok)))
       )
      )
      ((= en "INSERT")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 41 entl)
            (LI_item 42 entl)
            (LI_item 43 entl)
            (MI_r2d (LI_item 50 entl))
              )
       )
       (setq tmp1 (LI_item 2 entl))
       (if (and
         (> (strlen tmp1) 1)
         (= (substr tmp1 1 1) "*")
         (= (substr tmp1 2 1) "U")
           )
         (prin1)
         (setq ETok (cons tmp1 ETok))
       )

       (if (setq Lst (BL_GetAtts ename))
         (setq ETok (append ETok (list Lst)))
       )

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and InsertLst (LI_memb ETok InsertLst Diff))
         (ssadd ename DupSS)
         (setq InsertLst (append InsertLst (list ETok)))
       )
      )
      ((= en "LEADER")
       (setq ETok (list (LI_CodeStrip
                  entl
                  (list -1 0 3 5 8 100 210 211 212 340)
                )
              )
       )

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and LeaderLst (LI_memb ETok LeaderLst Diff))
         (ssadd ename DupSS)
         (setq LeaderLst (append LeaderLst (list ETok)))
       )
      )
      ((= en "LINE")
       (setq
         ETok1 (list (LI_item 10 entl) (LI_item 11 entl))
         ETok2 (list (LI_item 11 entl) (LI_item 10 entl))
       )

       (if ChkLyr
         (setq ETok1 (append ETok1 (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok1 (append ETok1 (list xd)))
       )
       (if ChkLyr
         (setq ETok2 (append ETok2 (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok2 (append ETok2 (list xd)))
       )

       (if (and LineLst
            (or (LI_memb ETok1 LineLst Diff)
            (LI_memb ETok2 LineLst Diff)
            )
           )
         (ssadd ename DupSS)
         (setq
           LineLst (append LineLst (list ETok1))
           LineLst (append LineLst (list ETok2))
         )
       )
      )
      ((= en "MLINE")
       (setq
         ETok (list (LI_CodeStrip entl (list -1 0 2 5 8 100 340)))
       )

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and LeaderLst (LI_memb ETok LeaderLst Diff))
         (ssadd ename DupSS)
         (setq LeaderLst (append LeaderLst (list ETok)))
       )
      )
      ((= en "MTEXT")
       (setq ETok (list
            (LI_item 1 entl)
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 40 entl)
            (LI_item 41 entl)
            (LI_item 67 entl)
            (LI_item 71 entl)
            (LI_item 72 entl)
              )
       )

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and MTextLst (LI_memb ETok MTextLst Diff))
         (ssadd ename DupSS)
         (setq MTextLst (append MTextLst (list ETok)))
       )
      )
      ((= en "REGION")

       (setq ETok (list ename))

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and RegionLst (LI_memb ETok RegionLst Diff))
         (ssadd ename DupSS)
         (setq RegionLst (append RegionLst (list ETok)))
       )
      )
      ((= en "POINT")
       (setq ETok (list (LI_item 10 entl)))

       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and PointLst (LI_memb ETok PointLst Diff))
         (ssadd ename DupSS)
         (setq PointLst (append PointLst (list ETok)))
       )
      )
      ((member en (list "POLYLINE" "LWPOLYLINE"))
       (setq
         vlist (PL_plist ename)
         ETok1 (list vlist)
         ETok2 (list (reverse vlist))
       )

       (if ChkLyr
         (setq ETok1 (append ETok1 (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok1 (append ETok1 (list xd)))
       )
       (if ChkLyr
         (setq ETok2 (append ETok2 (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok2 (append ETok2 (list xd)))
       )

       (if (and PolylineLst
            (or (LI_memb ETok1 PolylineLst Diff)
            (LI_memb ETok2 PolylineLst Diff)
            )
           )
         (ssadd ename DupSS)
         (setq PolylineLst (append PolylineLst (list ETok1 ETok2)))
       )
      )
      ((= en "SHAPE")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 2 entl)
            (LI_item 40 entl)
            (LI_item 41 entl)
            (MI_r2d (LI_item 50 entl))
            (MI_r2d (LI_item 51 entl))
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and ShapeLst (LI_memb ETok ShapeLst Diff))
         (ssadd ename DupSS)
         (setq ShapeLst (append ShapeLst (list ETok)))
       )
      )
      ((= en "SOLID")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 12 entl)
            (LI_item 13 entl)
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and SolidLst (LI_memb ETok SolidLst Diff))
         (ssadd ename DupSS)
         (setq SolidLst (append SolidLst (list ETok)))
       )
      )
      ((= en "SPLINE")
       (setq ETok (list ename))
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and SplineLst (LI_memb ETok SplineLst Diff))
         (ssadd ename DupSS)
         (setq SplineLst (append SplineLst (list ETok)))
       )
      )
      ((= en "TEXT")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 40 entl)
            (LI_item 41 entl)
            (MI_r2d (LI_item 50 entl))
            (LI_item 1 entl)
            (LI_item 71 entl)
            (LI_item 72 entl)
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and TextLst (LI_memb ETok TextLst Diff))
         (ssadd ename DupSS)
         (setq TextLst (append TextLst (list ETok)))
       )
      )
      ((= en "TRACE")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 12 entl)
            (LI_item 13 entl)
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and TraceLst (LI_memb ETok TraceLst Diff))
         (ssadd ename DupSS)
         (setq TraceLst (append TraceLst (list ETok)))
       )
      )
      ((= en "3DFACE")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 12 entl)
            (LI_item 13 entl)
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and 3DFaceLst (LI_memb ETok 3DFaceLst Diff))
         (ssadd ename DupSS)
         (setq 3DFaceLst (append 3DFaceLst (list ETok)))
       )
      )
      ((= en "3DSOLID")
       (setq ETok (list ename))
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and 3DSolidLst (LI_memb ETok 3DSolidLst Diff))
         (ssadd ename DupSS)
         (setq 3DSolidLst (append 3DSolidLst (list ETok)))
       )
      )
      ((= en "ARC")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 40 entl)
            (MI_r2d (LI_item 50 entl))
            (MI_r2d (LI_item 51 entl))
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and ArcLst (LI_memb ETok ArcLst Diff))
         (ssadd ename DupSS)
         (setq ArcLst (append ArcLst (list ETok)))
       )
      )
      ((= en "ATTDEF")
       (setq ETok (list
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 1 entl)
            (LI_item 2 entl)
            (LI_item 3 entl)
            (LI_item 40 entl)
            (LI_item 41 entl)
            (MI_r2d (LI_item 50 entl))
            (MI_r2d (LI_item 51 entl))
            (LI_item 70 entl)
            (LI_item 71 entl)
            (LI_item 72 entl)
            (LI_item 73 entl)
            (LI_item 74 entl)
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and AttdefLst (LI_memb ETok AttdefLst Diff))
         (ssadd ename DupSS)
         (setq AttdefLst (append AttdefLst (list ETok)))
       )
      )
      ((= en "CIRCLE")
       (setq ETok (list (LI_item 10 entl) (LI_item 40 entl)))
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and CircleLst (LI_memb ETok CircleLst Diff))
         (ssadd ename DupSS)
         (setq CircleLst (append CircleLst (list ETok)))
       )
      )
      ((= en "DIMENSION")
       (setq ETok (list
            (LI_item 1 entl)
            (LI_item 2 entl)
            (LI_item 3 entl)
            (LI_item 10 entl)
            (LI_item 11 entl)
            (LI_item 12 entl)
            (LI_item 13 entl)
            (LI_item 14 entl)
            (LI_item 15 entl)
            (LI_item 16 entl)
            (LI_item 70 entl)
            (LI_item 40 entl)
            (MI_r2d (LI_item 50 entl))
            (MI_r2d (LI_item 51 entl))
            (MI_r2d (LI_item 52 entl))
            (MI_r2d (LI_item 53 entl))
              )
       )
       (if ChkLyr
         (setq ETok (append ETok (list la)))
       )
       (if (and xd ChkXdata)
         (setq ETok (append ETok (list xd)))
       )

       (if (and DimensionLst (LI_memb ETok DimensionLst Diff))
         (ssadd ename DupSS)
         (setq DimensionLst (append DimensionLst (list ETok)))
       )
      )
    )
    (princ (strcat "\rChecking for duplicate objects..."
               (itoa cnt)
               tmp
           )
    )
      )
    )
  )
  (if (> (sslength DupSS) 0)
    DupSS
    nil
  )
)

(defun Wiper (/)
  (command "_.undo" "_be")
  (setvar "cmdecho" 0)
  (prompt "\nFrom point: ")
  (command "_.pline" pause "_w" 0 0)
  (while (> (getvar "cmdactive") 0) (command (getpoint (getvar "lastpoint") "\nTo point: ")))
  (command "_.pedit"
       "_l"
       "_c"
       "_x"
       "_.wipeout"
       "_n"
       (entlast)
       "_y"
       "_.wipeout"
       "_f"
       "_off"
       "_.undo"
       "_e"
  )
)

; -- Function VxGetEndPoints
; Returns the endpoints of an object.
; Arguments [Typ]:
;   Obj = Object [VLA-OBJECT]
; Return [Typ]:
;   > Endpoints '((x y z) (x y z)) [LIST]
;   > False if invalid object
; Notes: works on *Polyline, Spline, Ellipse, Line and Arc.
(defun VxGetExtrems (Obj)
  (list
    (trans (vlax-curve-getStartPoint Obj) 0 1)
    (trans (vlax-curve-getEndPoint Obj) 0 1)
  )
)

;In particular, object snaps are a bit of a tricky duck. In the old days,
;you would just set OSMODE to 0 to disable snaps. R14 and A2K added a new way
;to do this: set OSMODE to whatever it is plus 16384. Any value in excess of 16384
;indicates that objects snaps are temporarily disabled. This is not quite the same as OSMODE being 0.
;That can mean you turned them off OR they were never set in the first place.
;Rather than performing tests to determine whether you should add or subtract 16384,
;you can use the BOOLE function to automatically do it for you:
(defun toggle (varname bitcode)  (setvar varname (boole 6 (getvar varname) bitcode)) )

;Use the above code like so: (toggle "osmode" 16384)
;That will disable if below 16384 and enable if above 16384.
;This routine can be used to toggle any system variable that uses bitcodes.
;One example is the AUTOSNAP variable which controls, among other things,
;whether Object Snap Tracking and/or Polar Tracking is enabled:
;(toggle "autosnap" 8); toggles Polar Tracking
;(toggle "autosnap" 16); toggles Object Snap Tracking

;---Remove Duplicate members from a list.
;   Given list = '(1 1 2 3 4 "a" "b" "a")
;   Returns '(1 2 3 4 "a" "b")
(defun Remove_Dup (LST / CL)
  (mapcar
    '(lambda (X)
      (cond
        ((member X LST)
         (cond
           ((not (member X CL))
            (setq CL (cons X CL))
           )
         )
        )
      )
    )
    LST
  )
  (reverse CL)
)

;---The integer padto tells how far to pad to. It returns a string of spaces to the desired pad to integer.
;   If the string is longer than the desired pad to count the function returns a single space in a string.
;   If drapeau T = back, drapeau nil = front.
(defun PadIt (strng padto drapeau / cnt temp)
  (if (and (= (type strng) 'STR)(= (type padto) 'INT))
    (progn
      (setq temp "")
      (if (< (strlen strng) padto)
        (repeat (- padto (strlen strng))
          (setq temp (strcat temp " "))
        )
        (setq temp " ")
      )
      (if drapeau
        (strcat strng temp)     ;back padding
        (strcat temp strng)     ;front padding
      )
    )
    (progn
      (princ (strcat "\nPad Error: The first argument must be a string "
                     "\n           The second must be an integer.\n"))
      (princ)
      nil
    )
  )
)

(defun dxf2 (code ename) (cdr (assoc code (entget ename))))

;;(Setq OKflag T  NomDuProj "DSetUpForm"  ProjName0 "XpressForm"  AutreProj "AALib"  sLeTitreAA "ArchAssist  for AutoCAD 2000-2006"
;;      ProjName2 "Walls"  bOpt nil  TruBlu nil  sAppName4PlTxt "AA_CURVTXT"  sAppNameAA "AA_OPENING"  OnTrace t  AAVer "01.3"
;;      PourAA T  AABuild  "0108"  bImpUnit nil  bMetUnit nil  AcadVer (strcat "R" (substr (getvar "acadver") 1 2))
;;)
;;
;;(if (= (wcmatch (getvar "ACADVER") "*15*") T)
;;    (setq sAcadVer "A")
;;)
;;(if (= (wcmatch (getvar "ACADVER") "*16*") T)
;;    (setq sAcadVer "B")
;;)
;;
;;(if (= (substr (getvar "acadver") 1 4) "16.1") (setq sAcadVer "C"))     ;ACAD2005
;;(if (= (substr (getvar "acadver") 1 4) "16.2") (setq sAcadVer "D"))     ;ACAD2006

(defun TraceIsOn (Aff / )
  (if OnTrace (princ (strcat "\n[" Aff " Checked]")))
  (princ)
)

(defun MontreAA2 ()
  (if (= (Odcl_LoadProject ProjName2 nil) -10)
    (progn
      (WsFlashMess1
        "Echec chargement bote de dialogue. Fichier ArchAssist introuvable.\nSi la location a change, la mise  jour de MetricArchAssist.INI est indispensable."
        "Archivio ArchAssist non trovato. Se il percorso modificasse da ultima installazione,\nl'aggiornamento di MetricArchAssist.INI  indispensabile."
        "Archivo ArchAssist no ha sido encontrado. Si la ubicacion es modificada\ndesde la ultima instalacion, archivo MetricArchAssist.INI tendra que ser actualizado con su nueva ubicacion."
        "File ArchAssist not found. If Path modified since last\ninstallation, update MetricArchAssist.INI file."
        1
        5
      )
      (exit)
    )
  )
)

(defun MontreAA3 ()
  (if (= (Odcl_LoadProject ProjName0 nil) -10)
    (progn
      (WsFlashMess1
        "Echec chargement bote de dialogue XpressForm."
        "Archivio XpressForm non trovato. "
        "Archivo XpressForm no ha sido encontrado."
        "File XpressForm not found. "
        1
        5
      )
      (exit)
    )
  )
)

(defun MontreAA5 ()
  (if (= (Odcl_LoadProject AutreProj nil) -10)
    (progn
      (WsFlashMess1
        "Echec chargement bote de dialogue AALib."
        "Archivio AALib non trovato. "
        "Archivo AALib no ha sido encontrado."
        "File AALib not found. "
        1 5
      )
      (exit)
    )
  )
)

(defun DSUDisp ()
  (if (= (Odcl_LoadProject NomDuProj nil) -10)
    (progn
      (WsFlashMess1
        "Echec chargement bote de dialogue DSetUpForm"
        "Archivio DSetUpForm non trovato. "
        "Archivo DSetUpForm no ha sido encontrado."
        "File DSetUpForm not found. "
        1 5
      )
      (exit)
    )
  )
)

;;(setq Hshfile3 (findfile "ai_utils.lsp"))
;;(if (= Hshfile3 nil)
;;  (princ "\n* ai_utils.lsp is necessary to some routines.\n")
;;  ;;(SendStringToExecute "(load Hshfile3) ")
;;  (load Hshfile3)
;;)

(defun String-p (s) (= (type s) 'STR))

;;; low-level modifier, takes only an integer number
;;; in CL named: char-downcase
(defun 2Minusc (i)
  (if (or (<= 65 i 90)
          (<= 192 i 223)
      )
      (+ i 32)
      i
  )
)

;;; low-level modifier, takes only an integer number
;;; in CL named: char-upcase
(defun 2Majusc (i)
  (if (or (<= 97 i 122)
          (<= 224 i 255)
      )
    (- i 32)
    i
  )
)
;;; AutoCAD independent strcase
;;; Affected by redefinition of 2Majusc in STDLOCAL
(defun StringCase (s)
  (cond
    ((std-list->string
       (mapcar
         (function 2Majusc)
         (StringVersListe s)
       )
     )
    )
    (T "")
  )
)

;;; numbers need less memory space and housekeeping
;;; (StringVersListe "12") => (49 50)
(defun StringVersListe  (s / lst)
  (if (String-p s)
    (while (/= s "")
      (setq lst (cons (ascii (substr s 1 1)) lst)
        s   (substr s 2)
      )
    )
  )
  (reverse lst)
)

;;; numbers need less memory space and housekeeping
;;; unicode support
;;; (ListeVersString '(49 50 9600)) => "12"
(defun ListeVersString (lst / s)
  (if (listp lst)           ; fixed
    (progn
      (setq s "")
      (while lst
    (setq s   (strcat s (chr (car lst)))
          lst (cdr lst)
    )
      )
    )
  )
  s
)

(defun std-list->string (lst) (ListeVersString lst))

;;; (Number2Hex 451) = "1C3"
;;; Base Conversion
(defun Number2Hex (i / s a)
  (setq s "")
  (while (> i 0)
    (setq a (rem i 16)
      i (lsh i -4)
    )
    (setq s (strcat
          (if (< a 10)
        (chr (+ 48 a))      ;(ascii "0") = 48
        (chr (+ 55 a))
          )
          s
        )
    )
  )
)

;;; (Hex2Number "1C") -> 28
;;; strcase needs ADS
(defun Hex2Number (s / i n a)
  (setq i 0
    n 0
    s (StringCase s)
  )                 ;to be ADS independant
  (if (= (substr s 1 2) "0X")
    (setq s (substr s 3))
  )
  (while (< i (strlen s))
    (setq a (substr s (setq i (1+ i)) 1))
    (if (not (<= "0" a "F"))
      (chr a)
    )                   ;force bad argument type error
    (setq n (+ (lsh n 4)
           (- (ascii a)
          (if (<= a "9")
            48
            55
          )
           )
        )
    )
  )
)

;;; (Number2Oct 132) = "204"
(defun Number2Oct (i / s a)
  (setq s "")
  (while (> i 0)
    (setq a (rem i 8)
      i (lsh i -3)
    )
    (setq s (strcat (chr (+ 48 a)) s))
  )
)                   ;(ascii "0") = 48

;;; (Oct2Number "204") = 132
(defun Oct2Number (s / i n a)
  (setq i 0
    n 0
  )
  (while (< i (strlen s))
    (setq a (substr s (setq i (1+ i)) 1))
    (setq n (+ (lsh n 3)
           (- (ascii a) 48)
        )
    )
  )
)

;;; (Number2Bin 52) = (1 1 0 1 0 0)
(defun Number2Bin (i / lst)
  (while (> i 0)
    (setq lst (cons (rem i 2) lst)
      i   (lsh i -1)
    )
  )
  lst
)

;;; (Bin2Number '(1 1 0 0 0 1)) = 49
(defun Bin2Number (numlst / i n)
  (setq n 0)
  (foreach i numlst
    (setq n
       (cond
         ((= i 0) (lsh n 1))
         ((= i 1) (1+ (lsh n 1)))
         (T (chr (chr (+ 48 i))))
       )
    )
  )
)

;----------------------
;---Finalising Install.
;----------------------
;Purpose: Adds a folder to the AutoCAD support search path at the specified position.
;Arguments: A folder path and the positon at which to insert it (0 based).
;Example: (SupportPathAdded 2 "c:\\myFolder")
(defun SupportPathAdded (pos dir / tmp c lst)
  (setq tmp ""
    c   -1
  )
  (if (not (member dir (setq lst (strParse (getenv "ACAD") ";"))))
    (progn (if (not pos)
         (setq tmp (strcat (getenv "ACAD") ";" dir))
         (mapcar '(lambda (x)
            (setq tmp (if (= (setq c (1+ c)) pos)
                    (strcat tmp ";" dir ";" x)
                    (strcat tmp ";" x)
                  )
            )
              )
             lst
         )
       )
       (setenv "ACAD" tmp)
    )
  )
  (princ)
)

;Purpose: Removes the specified folder from the AutoCAD support search path.
;Arguments: The folder to remove
;Example: (RidSupportPath "C:\\Program Files\\ArchAssist")
(defun RidSupportPath (dir / tmp)
  (setq tmp "")
  (mapcar '(lambda (x)
         (if (/= x dir)
           (setq tmp (strcat tmp x ";"))
         )
       )
      (StrParse (getenv "ACAD") ";")
  )
  (setenv "ACAD" (substr tmp 1 (1- (strlen tmp))))
  (princ)
)

;;; Parameters:
;;;     Str     String to parse
;;;     Delimiter   Delimiter to search for
;;;------------------------------------------------------------------
;;; Returns:
;;;     A list strings.
;;;     ex:
;;;     (setq a "Harp,Guiness,Black and Tan")
;;;     (StrParse a ",")
;;;     returns:
;;;     ("Harp" "Guiness" "Black and Tan")
(defun strParse (Str Delimiter / SearchStr StringLen return n char)
  (setq SearchStr Str)
  (setq StringLen (strlen SearchStr))
  (setq return '())
  (while (> StringLen 0)
    (setq n 1)
    (setq char (substr SearchStr 1 1))
    (while (and (/= char Delimiter) (/= char ""))
      (setq n (1+ n))
      (setq char (substr SearchStr n 1))
    )
    (setq return (cons (substr SearchStr 1 (1- n)) return))
    (setq SearchStr (substr SearchStr (1+ n) StringLen))
    (setq StringLen (strlen SearchStr))
  )
  (reverse return)
)

;;Thanks to Bill Kramer for this little beauty.
(defun C:RegTest (/ AcadReg LMAcadReg Name1 Name2)
  (setq AcadReg (vlax-product-key)
    LMAcadReg   (strcat "HKEY_LOCAL_MACHINE\\" AcadReg)
    Name1       (vl-registry-read LMAcadReg "FirstName")
    Name2       (vl-registry-read LMAcadReg "LastName")
  )
  (alert (strcat "Name of this computer is " Name1 " " Name2))
  (princ)
)

;--------------------------------------
;---Partial menu loading, unloading ...
;--------------------------------------
;;; B/Depuis une fonction utiliser la syntaxe suivante:
;;; Utiliser la commande (TerMENU valmnu filmnu posmnu grpmnu nummnu)
;;; Pour informations complmentaires sur les options voir ci-dessous
;;; arguments de la fonction batch.

;;; Alerte en cas d'erreur
(defun mnuend (app msg)
  ;;LCA - COMMENT: *error* defun no longer evaluates as a list.
  (defun *error* (s)
    (if oerrmnu
      (setq *error* oerrmnu)
    )
    (princ)
  )
  (if msg
    (princ (strcat "Application error : " app msg "\n"))
  )
  ;;; Restauration
  (if ofilmnu
    (setvar "filedia" ofilmnu)
  )
  (if ocmdmnu
    (setvar "cmdecho" ocmdmnu)
  )
  (exit)
)

;;; Verification pralable fichiers ncessaires Version Autocad et plateforme

;;(cond
;;  ;;; Plateforme Windows
;;  ((/= (wcmatch (getvar "platform") "*Windows*") T)
;;   (mnuend
;;     "ArchAssist"
;;     (strcat "\n\tApplication ArchAssist needs Windows: \n\tCurrent platform: " (getvar "platform"))
;;   )
;;  )
;;  ;;; Version Autocad 15/16 Windows
;;  ((and (/= (wcmatch (getvar "ACADVER") "*15*") T)
;;        (/= (wcmatch (getvar "ACADVER") "*16*") T)
;;   )
;;   (mnuend
;;     "ArchAssist"
;;     (strcat "\n\tApplication ArchAssist needs AutoCAD 15 or 16: \n\tCurrent version: " (getvar "ACADVER"))
;;   )
;;  )
;;)

;;; Fonction d'erreur
;;; Ne pas modifier SVP la liste ci-dessous
(defun mnuerr (s / errnum)
  (if (not (member s (list "Function cancelled" "interruption de la console")))
    (progn (setq errnum (getvar "errno")) (princ (strcat "\nError number: " (itoa errnum) " = " s)))
  )
  ;;; Restauration
  (reset nil)
  (resset varlst)
  (if oerrmnu
    (setq *error* oerrmnu)
  )
  (princ)
)

;;; Fonction en ligne de commande
;;; =========================================================================
;;; Variables:
;;;     r__mnu  = Rponse utilisateur
;;;     nammnu  = Nom du menu groupe
;;;     oerrmnu = Sauvegarde Ancienne valeur de error
;;;     ofilmnu     = Sauvegarde Ancienne valeur de filedia
;;;     ocmdmnu     = Sauvegarde Ancienne valeur de cmdecho
(defun AAmenu ( Quest / r__mnu nammnu oerrmnu ocmdmnu ofilmnu )
  ;;  (Wallsgather_info)
  (WsFlashMess2 (strcat "\nChargement partiel du menu." copyright)
                (strcat "\nPartial Menu Loading." copyright)
                (strcat "\nPartial Menu Loading." copyright)
                (strcat "\nPartial Menu Loading." copyright)
  )
  (setq oerrmnu *error*)
  (setq *error* mnuerr)
  (setvar "cmdecho" 0)
  ;;; Dfinition du nom du menu srf
  ;;;**************************************************************************
  (cond
    ((= langue "Franais")
     (setq AALinguo "fr"
           nammnu   "ArchAssist"
     )
    )
    ((= langue "Italiano")
     (setq AALinguo "it"
           nammnu   "ArchAssist"
     )
    )
    ((= langue "Espanol")
     (setq AALinguo "es"
           nammnu   "ArchAssist"
     )
    )
    (T
     (setq AALinguo "ASSIST"
           nammnu   "ArchAssist"
     )
    )
  )
  ;;; Verification Version Autocad
  (cond
    ;;; Existance du menu
    ((and (not (findfile (strcat nammnu ".mnc")))
          (not (findfile (strcat nammnu ".mnr")))
          (not (findfile (strcat nammnu ".mns")))
     )
     (cond
       ((not (findfile (strcat nammnu ".mnu")))
    (cond
      ((= langue "Franais")
       (setq AALinguo "ArchAssistfr")
       (mnuend
         "ArchAssist"
         (dos_msgbox
           (strcat "Echec: Chargement du menu.\nFile " (strcase nammnu) ".MNU est introuvable.")
           "Message ArchAssist"
           1
           5
         )
       )
      )
      ((= langue "Italiano")
       (setq AALinguo "ArchAssistit")
       (mnuend
         "ArchAssist"
         (dos_msgbox
           (strcat "Failure on loading menu.\nFile " (strcase nammnu) ".MNU cannot be found.")
           "ArchAssist Message"
           1
           5
         )
       )
      )
      ((= langue "Espanol")
       (setq AALinguo "ArchAssistes")
       (mnuend
         "ArchAssist"
         (dos_msgbox
           (strcat "Failure on loading menu.\nFile " (strcase nammnu) ".MNU cannot be found.")
           "ArchAssist Message"
           1
           5
         )
       )
      )
      (T
       (mnuend
         "ArchAssist"
         (dos_msgbox
           (strcat "Failure on loading menu.\nFile " (strcase nammnu) ".MNU cannot be found.")
           "ArchAssist Message"
           1
           5
         )
       )
      )
      (setq AALinguo "ArchAssist")
    )
       )
     )
    )
  )
  (if (and (= Quest nil) (= r__mnu nil))
    (progn
      (cond ((= langue "Franais")
         (setq r__mnu (ukword 1
                  "Charger Recharger Dcharger"
                  "Charger menu ou [Recharger/Dcharger] "
                  "Charger"
              )
         )
         (if (= r__mnu "Charger")
           (setq r__mnu "Load")
         )
         (if (= r__mnu "Recharger")
           (setq r__mnu "Reload")
         )
         (if (= r__mnu "Dcharger")
           (setq r__mnu "Unload")
         )
        )
        ((= langue "Italiano")
         (setq r__mnu (ukword 1
                  "Charger Recharger Dcharger"
                  "Charger menu ou [Recharger/Dcharger] "
                  "Charger"
              )
         )
         (if (= r__mnu "Charger")
           (setq r__mnu "Load")
         )
         (if (= r__mnu "Recharger")
           (setq r__mnu "Reload")
         )
         (if (= r__mnu "Dcharger")
           (setq r__mnu "Unload")
         )
        )
        ((= langue "Espanol")
         (setq r__mnu (ukword 1
                  "Charger Recharger Dcharger"
                  "Charger menu ou [Recharger/Dcharger] "
                  "Charger"
              )
         )
         (if (= r__mnu "Charger")
           (setq r__mnu "Load")
         )
         (if (= r__mnu "Recharger")
           (setq r__mnu "Reload")
         )
         (if (= r__mnu "Dcharger")
           (setq r__mnu "Unload")
         )
        )
        (T (setq r__mnu (ukword 1 "Load Reload Unload" "Load menu or [Reload/Unload] " "Load")))
      )
    )
    (setq r__mnu Quest)
  )
  ;;; Lancement de la fonction batch
  (cond ((wcmatch (getvar "acadver") "*16*")
     (cond ((= r__mnu "Unload") (srfmnu 0 nammnu "" nammnu AALinguo))   ;(strcase nammnu)
           ((= r__mnu "Load")   (srfmnu 1 nammnu "10" nammnu AALinguo))
           ((= r__mnu "Reload") ;;(srfmnu 0 nammnu "" nammnu AALinguo)  ;disabled 18/7/05
                    (srfmnu 2 nammnu "10" nammnu AALinguo)
           )
     )
    )
    ((wcmatch (getvar "acadver") "*15*")
     (cond ((= r__mnu "Unload") (srfmnu 0 nammnu "" (strcase nammnu) AALinguo))
           ((= r__mnu "Load")   (srfmnu 1 nammnu "10" (strcase nammnu) AALinguo))
           ((= r__mnu "Reload")
        (srfmnu 0 nammnu ""   (strcase nammnu) AALinguo)
        (srfmnu 2 nammnu "10" (strcase nammnu) AALinguo)
           )
     )
    )
  )
  ;;; Restauration
  (if oerrmnu
    (setq *error* oerrmnu)
  )
  (reset nil)
  (resset varlst)
)

;;; Function Batch
;;; =========================================================================
;;; Argument:
;;;     valmnu  = Option de chargement
;;;     0 = Decharger   1 = Charger   2 = Restaurer
;;;     filmnu  = Nom du menu
;;;     posmnu  = Position "NUM"  utiliser dans le pop pour le chargement de "0" a "16" ou "" si dcharger
;;;     grpmnu  = Nom du groupe de menu
;;;     popmnu  = Nom du pop  charger
;;; Variables:  c__mnu  = Contenu de la commande menucmd  excuter
;;; (srfmnu 1 nammnu "10" (strcase nammnu) AALinguo)

(defun srfmnu (valmnu filmnu posmnu grpmnu popmnu / c__mnu)
  ;;; Vrification des arguments
  (cond
    ((or (and (/= valmnu 0) (/= valmnu 1) (/= valmnu 2)) (/= (type filmnu) 'STR))
     (mnuend "ArchAssist" (strcat "\n\tIncorrect arguments passed to a function:\n\tCheck syntax."))
    )
  )
  (cond
    ;;; Reconstitution de la commande menucmd pour chargement
    ((or (= valmnu 1) (= valmnu 2)) (setq c__mnu (strcat "P" posmnu "=+" grpmnu "." popmnu)))
    ((= valmnu 0)
    ;;; Reconstitution de la commande menucmd pour dchargement
     (setq c__mnu (strcat "G" nammnu "." popmnu "=-"))
    )
  )
  ;;; Traitement
  ;;;**************************************************************************
  (cond
    ;;; Chargement
    ;;;**************************************************************************
    ((or (= valmnu 1) (= valmnu 2))
     (cond
       ((= (wcmatch (getvar "ACADVER") "*15*") T)
        (if (= (menugroup grpmnu) nil)
          (progn
            (if (or (= (findfile (strcat filmnu ".mnc")) nil)       ;chargement du menu si menu compil non trouv
                (= (findfile (strcat filmnu ".mnr")) nil)
                (= (findfile (strcat filmnu ".mns")) nil)
            )
              (command "_menuload" (findfile (strcat filmnu ".mnu")))   ;chargement du menu demand
              (progn (cond                      ;chargement du menu compil
                   ((= valmnu 1) (command "_menuload" (findfile (strcat filmnu ".mnc"))))
                   ;;;Restauration du menu source
                   ((= valmnu 2) (command "_menuload" (findfile (strcat filmnu ".mnu"))))
                 )
              )
            )
          )
        )
       )
     )

     (if (= (substr (getvar "acadver") 1 4) "16.2")
       (progn
         (if (= (menugroup grpmnu) nil)
           (progn
             (if (or (= (findfile (strcat filmnu ".cui")) nil)
                     (= (findfile (strcat filmnu ".mnr")) nil)
                     (= (findfile (strcat filmnu ".mns")) nil)
                 )
               ;;; chargement du menu source   si menu compil non trouv
               (command "_menuload" (findfile (strcat filmnu ".cui")))
               ;;; chargement du menu demand
               (progn
                 (cond
                   ;;; chargement du menu compile
                   ((= valmnu 1) (command "_menuload" (findfile (strcat filmnu ".cui"))))
                   ;;; Restauration du menu source
                   ((= valmnu 2) (command "_menuload" (findfile (strcat filmnu ".cui"))))
                 )
               )
             )
           )
         )
       )
       (progn
         (if (= (menugroup grpmnu) nil)
           (progn
             (if (or (= (findfile (strcat filmnu ".mnc")) nil)
                     (= (findfile (strcat filmnu ".mnr")) nil)
                     (= (findfile (strcat filmnu ".mns")) nil)
                 )
               ;;; chargement du menu source   si menu compil non trouv
               (command "_menuload" (findfile (strcat filmnu ".mnu")))
               ;;; chargement du menu demand
               (progn
                 (cond
                   ;;; chargement du menu compile
                   ((= valmnu 1) (command "_menuload" (findfile (strcat filmnu ".mnc"))))
                   ;;; Restauration du menu source
                   ((= valmnu 2) (command "_menuload" (findfile (strcat filmnu ".mnu"))))
                 )
               )
             )
           )
         )
       )
     )
     ;;; chargement du pop
     (menucmd c__mnu)
    )
    ;;; DChargement
    ;;;***************************************************************
    ((= valmnu 0) (menucmd c__mnu) (command "_menuunload" grpmnu))
  )
)

;;;-------------
;;;(SupportPathAdded 2 "C:\\Program Files\\ArchAssist") ;;### TM : VERY DANGEROUS !!!

(defun c:RPM (/)
  (princ "Removing support path, partial menu...")
    (if (setq sPath2 (findfile "ArchAssist.vlx"))
      (progn
       (AAMenu "Unload")
       (RidSupportPath (vl-filename-directory sPath2))
      )
    )
)

;;(if (wcmatch (getvar "ACADVER") "*15*")
;;  (progn
;;    (if (null (member "doslib2k.arx" (arx)))
;;      (arxload "doslib2k.arx" "Doslib2k.arx not found...")
;;    )
;;    (if (null (member "ObjectDCL.arx" (arx)))
;;      (arxload "ObjectDCL.arx" "ObjectDCL.arx not found...")
;;    )
;;    (if (null (member "GeomCal.arx" (arx)))
;;      (arxload "GeomCal.arx" "GeomCal.arx not found...")
;;    )
;;    (if (null (member "Geom3D.arx" (arx)))
;;      (arxload "Geom3D.arx" "Geom3D.arx not found...")
;;    )
;;  )
;;)
;;(if (wcmatch (getvar "ACADVER") "*16*")
;;  (progn
;;    (if (null (member "doslib2004.arx" (arx)))
;;      (arxload "doslib2004.arx" "Doslib2004.arx not found...")
;;    )
;;    (if (null (member "ObjectDCL2004.arx" (arx)))
;;      (arxload "ObjectDCL2004.arx" "ObjectDCL2004.arx not found...")
;;    )
;;    (if (null (member "GeomCal.arx" (arx)))
;;      (arxload "GeomCal.arx" "GeomCal.arx not found...")
;;    )
;;    (if (null (member "Geom3D.arx" (arx)))
;;      (arxload "Geom3D.arx" "Geom3D.arx not found...")
;;    )
;;  )
;;)

;;(defun MyStartup () (princ "\nRajoute mais ou, putain d'Adele?\n"))

;;(defun AAPlugIntoStartup (MyFunc) ; to be called with quoted function name
;;  (eval   (list 'defun
;;        'S::StartUp
;;        ()
;;        (if S::StartUp
;;      (list (list 'quote S::StartUp))
;;        )
;;        (list MyFunc)
;;  )
;;  )
;;)

;;(AAPlugIntoStartup 'MyStartUp)

(defun getexpr (msg) (initget 128) (getint msg))

(defun Cassure (msg /)
  (if TruBlu
    (progn
      (princ (strcat "\n*Breakpoint <" msg "> press Enter to resume execution*"))
      (while (setq expr (getexpr "\n>> ")) (print (eval (read expr))) (princ))
      (princ "\n**Exiting breakpoint. Resuming program execution**\n")
    )
  )
)

;---This is a routine for doing a bubble sort on x, y, or z portion of a point from a list of points.
;   It accepts these arguments:
;   pt_list is a list of points passed.
;   position is a string of x y or z the default is x.
;   bit controls the order of the list being passed back.
;     0 = decending order
;     1 = ascending order
(defun sort_list (pt_list position bit / cnt outer hold hold1)
  (setq outer (length pt_list))
  (while (/= outer 0)
    (setq cnt 0)
    (repeat (length pt_list)
     (if (cond ((= (strcase position) "Z")(> (caddr (nth cnt pt_list))
                                             (caddr (nth (1+ cnt) pt_list))))

               ((= (strcase position) "Y")(> (cadr (nth cnt pt_list))
                                             (cadr (nth (1+ cnt) pt_list))))

                                       (T (> (car  (nth cnt pt_list))
                                             (car  (nth (1+ cnt) pt_list))))
         )

      (if (< cnt (1- (length pt_list)))
       (setq hold    (nth cnt pt_list)
             hold1   (nth (1+ cnt) pt_list)
             pt_list (subst '(0) (nth (1+ cnt) pt_list) pt_list)
             pt_list (subst hold1 (nth cnt pt_list) pt_list)
             pt_list (subst hold '(0) pt_list))
      )
     )
     (setq cnt (1+ cnt))
    )
    (setq outer (1- outer))
  )
  (if (= bit 0)
    (setq pt_list (reverse pt_list))
    (setq pt_list pt_list)
  )
)

;---SWAP takes the contents of two specified locations in a
;   list and causes them to trade places.  N1 and N2 are used
;   in the same manner as (NTH) [starting at 0].
(defun SWAP (DList N1 N2 / TMP1)
  (setq TMP1 (nth N1 DList)
        DLIST (append (sub_list DList (1- N1))
                      (list (nth N2 DList))
                      (reverse (sub_list (reverse DList)
                                         (- (length DList) (+ 2 N1)))))
        DLIST (append (sub_list DList (1- N2))
                      (list TMP1)
                      (reverse (sub_list (reverse DList)
                                         (- (length DList) (+ 2 N2)))))
  )
)

;;; Description:   returns number of times a member is a list.
;;;     ITEM       Member that is to be counted.
;;;     LST        List that member is in.
(defun Mem_Cnt (ITEM LST / List_Mem_In)
  (defun List_Mem_In (ITEM LST / TL)
    (mapcar
      '(lambda (X)
        (cond
;;        ( (= X ITEM))
          ( (equal X ITEM 0.00001)
            (setq TL (cons X TL))
          )
        )
      )
      LST
    )
    (reverse TL)
  )
  (length (List_Mem_In ITEM LST))
)

;;; Description: gets list of all instances of the specified member from given list.
;;;     ITEM       Member that is to be retrieved from list.
;;;     LST        List that member is retrieved from.
(defun List_Mem_In (ITEM LST / TL)
  (mapcar
    '(lambda (X)
      (cond
        ( (= X ITEM)
          (setq TL (cons X TL))
        )
      )
    )
    LST
  )
  (reverse TL)
)

;---If a member is common to both lists, a list is returned with ALL instances of that member removed
;   from the appending of the two lists. The common members of two lists are not in the returned list.
(defun Join_Kill_Common (LST1 LST2 / )
  (defun Cutout (ITEM LST / TL)
    (mapcar
      '(lambda (X)
        (cond
          ( (not (equal X ITEM))
            (setq TL (cons X Tl))
          )
        )
      )
      LST
    )
    (reverse TL)
  )
  (mapcar
    '(lambda (X)
      (cond
        ( (member X LST2)
          (setq LST1 (Cutout X LST1))
          (setq LST2 (Cutout X LST2))
        )
      )
    )
    LST1
  )
  (append LST1 LST2)
)

(defun first_vtx (e)
  (if (= (cdr (assoc 0 (entget e))) "POLYLINE")         ;check for polyline
    (cdr (assoc 10 (entget (entnext e))))               ;if pline get 1st pt
    (progn
     (prompt "\nObject not a POLYLINE.")            ;if not pline tell me
     nil
    )
  )
)

(defun FixObjByLayer ( / cnt color ent ent1)
  (setq  ent  (entnext)
         cnt  0
         cnt1 0
  )
  (while ent
    (if  (or (dxf -2 (entget ent))
         (= "VERTEX" (dxf 0 (entget ent)))
         (= "ATTRIB" (dxf 0 (entget ent)))
     )
      (setq cnt1 (1+ cnt1))
      (command "._CHPROP" ent "" "_C" "_BYLAYER" "")
    )
    (setq cnt  (1+ cnt)
        ent1 (entnext ent)
    )
    (if  ent1
      (setq ent ent1)
      (setq ent nil)
    )
  )
)

(defun WinOrCross (p1 p2 / xmin ymin xmax ymax)
  (setq xmin (car p1)
    ymin (cadr p1)
    xmax (car p2)
    ymax (cadr p2)
  )
  (if (> xmin xmax)
    (command "_.SELECT" "_C" p1 p2 "")
    (command "_.SELECT" "_W" p1 p2 "")
  )
  (setq pBL p1 pTr p2)
)

;If you've ever tried to use the GETCORNER function while in a 3D view,
;you've discovered an unfortunate problem: the box drawn by GETCORNER is plan to
;the view, not the UCS. This has a very disorienting effect since your eyes and your
;cursor don't agree. Here's a solution:
;Functions exactly like getcorner but the box is always drawn plan to the UCS, not the view
;Arguments: A base point and an optional prompt. If you do not want a prompt, pass nil
;Example: (setq pCnr2 (GetCornerEx (setq pCnr1 (getpoint "\nSpecify first corner point: ")) "\nSpecify opposite corner: "))
(defun GetCornerEx (pCnr1 msg / pTest)
  (if msg (prompt msg) )
  (while (= (car (setq pTest (grread T 1))) 5)
    (setq pCnr2 (cadr pTest))
    (redraw)
    (grvecs
      (list
        (list (car pCnr1) (cadr pCnr1))
        (list (car pCnr2) (cadr pCnr1))
        (list (car pCnr2) (cadr pCnr1))
        (list (car pCnr2) (cadr pCnr2))
        (list (car pCnr2) (cadr pCnr2))
        (list (car pCnr1) (cadr pCnr2))
        (list (car pCnr1) (cadr pCnr2))
        (list (car pCnr1) (cadr pCnr1))
      )
    )
  )
  (redraw)
  (if (= (car pTest) 3)(cadr pTest) )

  (WinOrCross pCnr1 pCnr2)

  (setq ss1 (ssget "_P"))
  (setq 1ss (ak_ssget ss1))
  (command "_.SELECT" ss1 pause)
  (setq ss1 (ssget "_P"))
)

;;(setq sttime (* 1440 (getvar "tdusrtimer")) )

;---This routine passes back a true or nil
; It will accept reals, integers, and strings for either number.
; If the NUMB argument is a multiple of the BASE argument a T is returned
; If not a nil is returned.
(defun mod (numb base)
  (if (= (type numb) 'STR)
    (setq numb (atof numb))
  )
  (if (= (type base) 'STR)
    (setq base (atof base))
  )
  (if (= (rem numb base) 0)
    T
    nil
  )
)

;---Melange
(defun melange (instr key mode / cnt)
  (setq cnt 1 newstr "")
  (repeat (strlen instr)
    (cond
      ((= mode 1)                         ;melange
       (setq newstr (strcat newstr (chr (+ (ascii (substr instr cnt 1))(ascii key)))))
      )
      ((= mode 2)                         ;demelange
       (setq newstr (strcat newstr (chr (- (ascii (substr instr cnt 1))(ascii key)))))
      )
    )
    (setq cnt (1+ cnt))
  )
  (eval newstr)
)

;---
(defun dtr (d) (/ (* d pi) 180.0))
(defun rtd (r) (/ (* r 180.0) pi))

;;(setq Hshpi90  (/ pi 2.0)  Hshpi270  (* pi 1.5)  NumDeSec "Substr"  ModDrapeau T
;;      Hshpi360 (* pi 2.0)  tiempo  (- (* 43 0.5) 0.5)  que nil
;;      basta    (* (/ 39.6 9) 5)  putain nil  fna nil  compteur 0   csn nil
;;;;;      ElemUn   "2005032"   ElemUna "2005032"  ElemDe "9.1707"  ElemTr "0642"
;;)

;---Remove all spaces from a string
(defun nospace (astring / ch ret)
  (setq ret "")
  (while (> (strlen astring) 0)
    (setq ch (substr astring 1 1)
          astring (substr astring 2)
    )
    (if (/= ch " ") (setq ret (strcat ret ch)))
  )
  ret
)

;---Adds an element in front of the nth element in the list.
(defun add_n (num lst item / split_list)
  (setq split_list (split_at (- num 1) lst))
  (append
    (reverse (reverse (car split_list)))
    (list item)
    (cadr split_list)
  )
)

(defun Cutout (ITEM LST / TL)
  (mapcar
    '(lambda (X)
      (cond
        ((not (equal X ITEM))
         (setq TL (cons X Tl))
        )
      )
    )
    LST
  )
  (reverse TL)
)

;;; Splits a list into a list of two lists after the number specified.
;;; (split_at 2 '(a b c d e )) returns ((a b c ) (d e)). If
;;; number is invalid for length of list, the original list is returned.
;;; Number index counting base is 0.
(defun split_at (num lst / head tail cnt)
  (setq  head '()
         tail '()
         cnt  0
         num  (1+ num)
  )
  (cond
    ((and (>= num 1) (<= num (length lst)))
     (while (>= (length lst) cnt)
       (cond
         ((< cnt num)
          (setq head (cons (nth cnt lst) head))
         )
         ((>= cnt num)
          (setq tail (cons (nth cnt lst) tail))
         )
       )
       (setq cnt (1+ cnt))
     )
     (list (reverse head) (reverse (cdr tail)))
    )
    (T
     lst
    )
  )
)

;---Removes the nth element from a list. Counting base is 0. If
;   number is invalid for length of list, the original list is returned.
(defun remove_n (num lst / split_list head tail cnt)
  (setq  head '()
         tail '()
         cnt  0
         num  (1+ num)
  )
  (cond
    ((and (>= num 1) (<= num (length lst)))
     (while (>= (length lst) cnt)
       (cond
         ((< cnt num)
          (setq head (cons (nth cnt lst) head))
         )
         ((>= cnt num)
          (setq tail (cons (nth cnt lst) tail))
         )
       )
       (setq cnt (1+ cnt))
     )
     (setq split_list (list (reverse head) (reverse (cdr tail))))
     (append
       (reverse (cdr (reverse (car split_list))))
       (cadr split_list)
     )
    )
    (T lst)
  )
)

;---Routine to replace NUMber element in a list with a new ITEM. Counting
;   base is 0. If number is invalid for length of list, the original list is returned.
;   (Replace_Nth 2 '(a b c d e) bananas) returns  (a b bananas d e)
;   Used when multiple occurrences of an element occur in a list and SUBST
;   would replace all of them, and the list is indexed.
(defun Replace_Nth (NUM NEW LST / SPLIT_LIST HEAD TAIL CNT)
  (setq CNT 0
        NUM (1+ NUM)
  )
  (cond
    ( (and (>= NUM 1) (<= NUM (length LST)))
      (while (>= (length LST) CNT)
        (cond
          ((< CNT NUM)
           (setq HEAD (cons (nth CNT LST) HEAD))
          )
          ((>= CNT NUM)
           (setq TAIL (cons (nth CNT LST) TAIL))
          )
        )
        (setq CNT (1+ CNT))
      )
      (setq SPLIT_LIST (list (reverse HEAD) (reverse (cdr TAIL))))
      (append
        (reverse (cons NEW (cdr (reverse (car SPLIT_LIST)))))
        (cadr SPLIT_LIST)
      )
    )
    (T LST)
  )
)

;;(setq scroll "Loading aborted...  ArchAssist is either corrupted or reverse-engineered.")

;---(strfind "abcdefgh" "c") => ("ab" "defgh")
(defun STRFIND (SRC SRCH / CNT RET)
  (setq CNT 1)
  (while (<= CNT (strlen SRC))
    (if (= (substr SRC CNT (strlen SRCH)) SRCH)
      (setq RET (list
                  (substr SRC 1 (1- CNT))
                  (substr SRC (+ CNT (strlen SRCH)))
                )
            CNT (1+ (strlen SRC))
      )
      (setq CNT (1+ CNT))
    )
  )
  (if (null RET) (list SRC nil) RET)
)

;---(strrep "abcdefgh" "de" "//") => "abc//fgh"
(defun STRREP (SRC SRCH SREP / TMP)
  (setq TMP (STRFIND SRC SRCH))
  (if (null (cadr TMP)) SRC (strcat (car TMP) SREP (cadr TMP)))
)

;---(strrepl-all "12,45,78,55" "," "||") => "12||45||78||55"
(defun RepAllStrings (SRC SRCH SREP)
  (while (/= SRC (setq SRC (STRREP SRC SRCH SREP)))) SRC
)

;---(strrev "1234.56") => "65.4321"
(defun STRREV (SRC)
  (cond
    ((= SRC "") "")
    (t
     (strcat (STRREV (substr SRC 2)) (substr SRC 1 1))
    )
  )
)

;---(str_2_list "12,34,56,78" ",") => ("12" "34" "56" "78")
(defun str_2_list ( big_string delim / str_list piece place ch )
  (setq place 1  str_list (list) )
  (while (<= place (strlen big_string))
    (setq ch (substr big_string place 1))
    (if (= ch delim)
      (setq place (1+ place))
      (progn
        (setq piece "")
        (while (and (/= ch delim)(/= ch ""))
          (setq piece (strcat piece ch)
                place (1+ place)
                ch (substr big_string place 1)
          )
        )
        (setq str_list (cons piece str_list))
      )
    )
  )
  (reverse str_list)
)

;--- (DeStrAListe "12,34,56,78" ",") --> ("12" "34" "56" "78")
(defun DeStrAListe (SRC DLM)
  (read
    (strcat
      "("
      (chr 34)
      (RepAllStrings SRC DLM (strcat (chr 34) " " (chr 34)))
      (chr 34)
      ")"
    )
  )
)

; Function: (Diesel_eval <string> )
; Evaluates a DIESEL expression and returns the result to the
; calling AutoLISP expression.
; Automatically adds the "$M=" prefix required by (menucmd).
(defun Diesel_eval (s)
  (menucmd (strcat "M=" s))
)

; Function: (Diesel_apply <function> <arglist> )
; Applys a DIESEL function to a supplied list of arguments and
; returns the result.
; <function> is a string, which is the name of the DIESEL function
; to be applied, and <arglist> is a list of arguments that are to
; be submitted to the function (Diesel function calls are limited
; to a maximum of TEN arguments).
; All arguments must be of the types STRING, INT, or REAL, and are
; automatically converted to strings before being passed to DIESEL
; for evaluation.  Floating point formatting precision is 16 places.
; Example: (Diesel_apply "edtime" (list (getvar "tdindwg") "DD, hh:mm:ss"))
; Returns the time in the current drawing in days, hours, minutes, and seconds.
(defun Diesel_apply (fun args)
  (Diesel_eval
    (strcat "$(" (strdcat (cons fun (mapcar 'Diesel_format args)) ",") ")")
  )
)

; Function: (strdcat <list> <delimiter> )
; Converts a list of strings into a character-delimited string,
; where each element in the resulting string is delimited by the
; string <delimiter>.
; E.g. (strdcat '("One" "Two" "Three") ",")
; Returns -> "One,Two,Three"
(defun strdcat (l d)
  (apply 'strcat
    (cons
      (car l)
      (mapcar '(lambda (x) (strcat d x)) (cdr l))
    )
  )
)

; Function: (Diesel_format <value> )
; Converts <value> (either a real number, integer, or string) into a
; string to be passed as an argument to a DIESEL expression.
; Note that numeric precision of system variables is not preserved
; by formatting the result of an AutoLISP (getvar) function call, so
; format will allow you to include an expression as well, in the form: "$(getvar,lastpoint)"
; Lists are converted into comma-delimited strings, e.g. (Diesel_format (list 2 3 5))
; Returns -> "2,3,5"
(defun Diesel_format (value)
  ((lambda (vtype)
     (cond
       ((eq vtype 'real) (rtos value 2 16)
       )
       ((eq vtype 'int)  (rtos value 2 0)
       )
       ((eq vtype 'str)
        (cond
          ((eq (substr value 1 2) "$(") value)
          (t (strcat "\"" value "\""))
        )
       )
       ((and value (listp value))
        (strdcat (mapcar 'Diesel_format value) ",")
       )
       ((not value) "")
       (t (princ "\nformat error: unsupported data type."))
     )
   )
   (type value)
  )
)

; Function: (StringIndex <index> <string> )
; Returns the specified element in a comma-delimited string.
; <index> (zero-based) is the position of the desired element.
; E.g. (StringIndex "Zero,One,,Three,four" 3)
; Returns -> "Three"
(defun StringIndex (n s)
  (Diesel_apply "index" (list n s))
)

; Function: (string->list <string> )
; Converts a comma-delimited string into a list.
; E.g., (StringVerslist "Zero,One,Two,,Four")
; Returns -> '("Zero" "One" "Two" NIL "Four")
; Two consecutive commas are interpreted as a NULL element
; in the input string, which yields a value of NIL for the
; corresponding element in the resulting LIST.
(defun StringVerslist (s / tk)
  (cond
    ((eq s "") nil)
    ((eq "" (setq tk (Diesel_apply "index" 0 s)))
     (cons nil (StringVerslist (substr s 2)))
    )
    (t (cons tk (StringVerslist (substr s (+ 2 (strlen tk))))))
  )
)

; Function: (edtime <time> <picture> )
; Provides direct AutoLISP access to the Diesel $(edtime) function:
(defun edtime (time picture)
   (Diesel_apply "edtime" (list time picture))
)

;---Expression TO String
(defun etos (arg / file)
  (if (= 'STR (type arg))
    (setq arg (strcat "\"" arg "\""))
  )
  (setq file (open "$" "w"))
  (princ arg file)
  (close file)
  (setq file (open "$" "r"))
  (setq arg (read-line file))
  (close file)
  (close (open "$" "w"))
  arg
)

;---EQ_TOL:  check numeric equality with an error tolerance
;   important when working with real #'s
;   Syntax (eq_tol <numeric1> <numeric2> <error tolerance>)
;   return T if (abs (- <numeric1> <numeric2>)) is less than <error tolerance>
(defun eq_tol (target observed tolerance)
  (if (> (abs (- target observed)) tolerance)
    nil
    T
  )
)

;---data  - a string, the string to be checked and possibly truncated.
;   len   - an integer, truncate the string if its length is longer than this.
;   break - an integer, truncate the string and append the ellipsis at this position.
;   examples: (truncated "Encyclopedia" 10 7)  returns  "Encyclo..."
;             (truncated "Encyclopedia" 15 7)  returns  "Encyclopedia"
;             (truncated "This is only a test" 12 10)  returns  "This is on..."
(defun truncated (data len break / )
  (if (>= (strlen data) len)
    (strcat (substr data 1 break) "...")
    data
  )
)

(defun WallsMod ( / )

  (setq nutime (* 1440 (getvar "tdusrtimer"))
        nutime (round2dec (- nutime sttime) 10)
  )

  (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
    (progn
      (FlashMess3
        "ArchAssist maintenant Actif."
        "ArchAssist now Active."
        "ArchAssist now Active."
        "ArchAssist now Active."
      )
    )
  )

  (if ModDrapeau
    (progn
      (if (or (mod nutime 1.9)(mod nutime 2.1))
        (dos_msgbox (strcat " Unregistered copy of ArchAssist  1.3\n This is a 21-day Evaluation Demo.\n" copyright) "ArchAssist Message" 1 3)
      )
      (if (or (mod nutime 3.1)(mod nutime 3.3))
        (dos_msgbox (strcat " Esta version no esta bajo licencia de ArchAssist  1.3\n Periodo de evaluacion: 21 dias.\n" copyright) "Mensaje ArchAssist" 1 3)
      )
      (if (or (mod nutime 5.1)(mod nutime 5.3))
        (dos_msgbox (strcat " Ceci est la version non-licencie d'ArchAssist  1.3\n Priode d'valuation: 21 jours.\n" copyright) "Message ArchAssist" 1 3)
      )
    )
  )
)

(defun wlf (ligne file) (write-line ligne file))

;---"This is a long string that will not fit into a 32 character static text box."
;   would display as a 32 character long string as follows:
;   "This is a long...tatic text box."
(defun ReadXNum (str1 maxlen)
  ;;; initialize internals
  (setq tmpstr str1)
  (setq len (strlen tmpstr))

  (if (> len maxlen)
    (progn
       (setq maxlen2 (/ maxlen 2))
       (if (> maxlen (* maxlen2 2))
         (setq maxlen2 (- maxlen2 1))
       )
       (if (is_lead_byte  (substr tmpstr (- maxlen2 2) 1))
         (setq tmpstr1 (substr tmpstr 1 (- maxlen2 3)))
         (setq tmpstr1 (substr tmpstr 1 (- maxlen2 2)))
       )
       (if (is_lead_byte  (substr tmpstr (- len (- maxlen2 1)) 1))
         (setq tmpstr2 (substr tmpstr (- len (- maxlen2 3))))
         (setq tmpstr2 (substr tmpstr (- len (- maxlen2 2))))
       )
       (setq str2 (strcat tmpstr1 "..." tmpstr2))
    )
    (setq str2 (strcat tmpstr))
  )
)

;---OBJECT MANIPULATIONS FOR DCLs.
(defun emod (value bit)
  (if (= bit 62)
    (progn
      (if (= value "BYLAYER") (setq value 256))
      (if (= value "BYBLOCK") (setq value 0))
    )
  )
  (if (setq oldlist (cdr (assoc bit elist)))
    (tempmod value bit nil)
    (setq elist (append elist (list (cons bit value))))
  )
)

(defun WsFlashMess1 (frmess itmess esmess enmess numHsh1 numHsh2)
  (cond
    ((= langue "Franais") (dos_msgbox frmess "Message ArchAssist"  numHsh1 numHsh2))
    ((= langue "Italiano") (dos_msgbox itmess "Messagio ArchAssist" numHsh1 numHsh2))
    ((= langue "Espanol")  (dos_msgbox esmess "Mensaje ArchAssist"  numHsh1 numHsh2))
    (T                     (dos_msgbox enmess "ArchAssist Message"  numHsh1 numHsh2))
  )
)

(defun WsFlashMess2 (frmess itmess esmess enmess)
  (cond
    ((= langue "Franais") (princ frmess))
    ((= langue "Italiano") (princ itmess))
    ((= langue "Espanol")  (princ esmess))
    (T                     (princ enmess))
  )
)

(defun FlashMess3 (frmess itmess esmess enmess)
  (setq ocmdecho (getvar "cmdecho"))(setvar "cmdecho" 0) ;;### TM added to prevent echos
  (cond
    ((= langue "Franais") (command "MODEMACRO" frmess))
    ((= langue "Italiano") (command "MODEMACRO" itmess))
    ((= langue "Espanol")  (command "MODEMACRO" esmess))
    (T                     (command "MODEMACRO" enmess))
  )
  (setvar "cmdecho" ocmdecho) ;;### TM added
)

; Takes point one and performs the operation of the op argument (a string)
; on point two legal operators for the op argument are: - + * /
(defun make_newpt (op pt1 pt2 / 3d p1x p1y p1z p2x p2y p2z newx newy newz)
  (if (and (= 'LIST (type pt1))
           (= 'STR  (type op))
           (= 'LIST (type pt2))
      )
    (progn
      (if (and (= (length pt1) 3)          ; working with a set of 3d points?
               (= (length pt2) 3)
          )
        (setq 3d t)
      )
      (setq p1x (nth 0 pt1)
            p1y (nth 1 pt1)
            p2x (nth 0 pt2)
            p2y (nth 1 pt2)
      )
      (if 3d
        (setq p1z (nth 2 pt1)
              p2z (nth 2 pt2)
        )
      )

      ;; trap for divide by zero
      (if (and (= op "/")(= p2x 0.0))(setq p2x 1.0))
      (if (and (= op "/")(= p2y 0.0))(setq p2y 1.0))
      (if 3d
        (if (and (= op "/")(= p2z 0.0))(setq p2z 1.0))
      )
      (setq newx ((eval (read op)) p1x p2x)
            newy ((eval (read op)) p1y p2y)
      )
      (if 3d
        (progn
          (setq newz ((eval (read op)) p1z p2z))
          (list newx newy newz)
        )
        (progn
          (list newx newy)
        )
      )
    )
    (progn
      (princ "\nError: Bad argument type supplied to make_newpt.")
      nil
    )
  )
)

;;(princ "\nInitialising ArchAssist. Please wait...")

;;(cond
;;  ((= langue "Franais")(setq mess7 "\nChargement de ArchAssist a t annul.") )
;;  ((= langue "Italiano")(setq mess7 "\nCarico di ArchAssist  stato annullato.") )
;;  ((= langue "Espanol") (setq mess7 "\nSe cancel cargamento de ArchAssist.") )
;;  (T                    (setq mess7 "\nLoading of ArchAssist was cancelled.") )
;;)
;;
;;(if (= (wcmatch (getvar "ACADVER") "*15*") T) (setq liege "s"))   ;;Acasskimz.css
;;(if (= (wcmatch (getvar "ACADVER") "*16*") T) (setq liege "s"))    ;;Acasssez.css
;;
;;(setq riri "IDStamp1"  loulou "IDStamp2"  produit "Demo Copy")
;;
;;(setq sAddress (dos_macaddress)            ;"00:A0:D1:D5:70:B5" or Media Access Control number
;;      sLaSerie (dos_serialno)              ;"644DD6AC" or Serial Disk number
;;      sNiuAd   (DeStrAListe sAddress ":")  ;("00" "A0" "D1" "D5" "70" "B5")
;;
;;      ;;;  For testing ***
;;      ;sNewAd "A0D1D570B5"
;;
;;      ;;; For compiling *** "A0D1D570B5"
;;      sNewAd (strcat (nth 1 sNiuAd)(nth 2 sNiuAd)(nth 3 sNiuAd)(nth 4 sNiuAd)(nth 5 sNiuAd))
;;      iChew  (Hex2Number (substr sNewAd 9 2))        ;Dernier 2 caracteres "B5" -->>  181
;;)

(Defun c:TellMe (/)
  (princ (strcat sAddress " - " sLaSerie))
  (princ)
)

(defun c:regWalls ()
  (setq Hshfile2 (findfile "MetricArchAssist.INI"))
  (if (= Hshfile2 nil)
    (progn
      (WsFlashMess1
        "Fichier MetricArchAssist.INI introuvable. Arrt du programme."
        "Archivio MetricArchAssist.INI non sono stati trovati. Carico del programma si conclude."
        "Archivo MetricArchAssist.INI no ha sido encontrado. Carga del programa se termina."
        "File MetricArchAssist.INI not found! Loading terminated.  "
        1 5
      )
      (exit)
    )
    (setq langue   (dos_getini "CustomSettings" "Language" Hshfile2)
          chemin   (dos_getini "CustomSettings" "ProductPath" Hshfile2)
          Srvchmn  (dos_getini "CustomSettings" "ServerPath" Hshfile2)
          sArStyle (dos_getini "CustomSettings" "SMStyle" Hshfile2)
          sUnitSet (dos_getini "CustomSettings" "Units" Hshfile2)
          bKinkOnL (dos_getini "CustomSettings" "OnCutPlane" Hshfile2)
          sNewpath (dos_getini "CustomSettings" "BAKFolder" Hshfile2)
    )
  )

  (TraceIsOn ".1b")

  ;;Command: (dos_macaddress) = "00:D0:09:C9:BC:1E",  (dos_serialno "c:") = "163107E0"

  (defun c:Regist_OnInitialize ()
    ;;;Si H/disk number
    (Odcl_Control_SetProperty Walls_Regist_RequestTextBox "Text" (strcat (getvar "_PKSER") (afaire (dos_serialno))))
    ;;;ou macaddress
    ;(Odcl_Control_SetProperty Walls_Regist_RequestTextBox "Text" (strcat (getvar "_PKSER") sNewAd))

    (Odcl_Control_SetProperty Walls_Regist_LabAcadVersion "Caption" sAcadVer)
    (if (= sAddress nil)
      (Odcl_Control_SetProperty Walls_Regist_LabAloneOrNet "Caption" "1") ;pas de carte de reseau = H/disk number
      (Odcl_Control_SetProperty Walls_Regist_LabAloneOrNet "Caption" "2") ;carte de reseau = macaddress
    )
  )
  (defun c:Regist_reguser_OnEditChanged (sText /)       ;nom de l'entreprise
    ;;(Odcl_MessageBox "To Do: code must be added to c:Regist_reguser_EditChanged" sText)
    (cond
      ((or (= sText " ")                    ;si vide ou nul
           (= sText "?")
           (= sText "nil")
           (= sText "")
       )
       (Odcl_Control_SetProperty ProjName2 "Regist" "reguser" "Text" "?")
       (Odcl_Control_SetProperty ProjName2 "Regist" "RWarn1" "Visible" 1)       ;rendre visible
       (setq rFlag1 nil)        ;flag to stop
      )
      (T
       (setq reguser sText rFlag1 T)
       (Odcl_Control_SetProperty ProjName2 "Regist" "RWarn1" "Visible" 0)       ;rendre invisible
      )
    )
    (princ)
  )
  (defun c:Regist_reguser_OnKillFocus ()
    (setq sText (Odcl_Control_GetProperty
                  ProjName2
                  "Regist"
                  "reguser"
                  "Text"
                )
    )
    (if (or (= sText " ")                   ;si vide ou nul
            (= sText "nil")
            (= sText "")
        )
      (setq sText "?" rFlag1 nil)
      (Odcl_Control_SetProperty ProjName2 "Regist" "reguser" "Text" sText)
    )
    (princ)
  )
  (defun c:Regist_autcode_OnEditChanged (sText /)       ;numero de code
    ;(Odcl_MessageBox "To Do: code must be added to c:Regist_autcode_EditChanged" sText)
    (cond
      ((or (= sText " ")                    ;si blanc ou nil
           (= sText "")
       )
       (Odcl_Control_SetProperty ProjName2 "Regist" "autcode" "Text" "?")
       (Odcl_Control_SetProperty ProjName2 "Regist" "RWarn2" "Visible" 1)       ;rendre visible
       (setq rFlag2 nil)        ;flag to stop
      )
      (T
       (setq autcode sText rFlag2 T)
       (Odcl_Control_SetProperty ProjName2 "Regist" "RWarn2" "Visible" 0)       ;rendre invisible
      )
    )
    (princ)
  )

  (TraceIsOn ".1c")

  (defun c:Regist_Cancel_OnClicked ()
    (Odcl_Form_Close ProjName2 "Regist")
    (princ)
  )

  (defun c:Regist_Accept_OnClicked ()

    (if (and (= rFlag1 T) (= rFlag2 T))
      (progn (Odcl_Form_Close ProjName2 "Regist")
        (WallsWipe)
        (dos_regset
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
              AcadVer
              "\\TroCADero\\"
              Averse
          )
          riri                     ;"Chiffre"
          (substr autcode 1 8)             ;Authorz
        )
        (dos_regset
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
              AcadVer
              "\\TroCADero\\"
              Averse
          )
          dadi
          (melange (substr autcode 9 8) "atoi" 1)      ;serialno *** 10 pas 8 ici, si carte de reseau
        )
        ;;      (setq sRevend (substr autcode 17 1))
        (dos_regset
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
              AcadVer
              "\\TroCADero\\"
              Averse
          )
          loulou
          (melange reguser "atoi" 1)           ;Company
        )
        (Beurre)
      )
    )
  )

  (MontreAA2)
  (Odcl_Form_Show ProjName2 "Regist")
)

;;;;(setq copyright " Copyright  1995-2006, TroCADero P/L.\n")
;;(setq copyright "\n"
;;      Hshfile2 (findfile "MetricArchAssist.ini")
;;)
;;(if (= Hshfile2 nil)
;;  (progn
;;    (WsFlashMess1
;;      "Fichier MetricArchAssist.INI introuvable. Arrt du programme."
;;      "Archivio MetricArchAssist.INI non sono stati trovati. Carico del programma si conclude."
;;      "Archivo MetricArchAssist.INI no ha sido encontrado. Carga del programa se termina."
;;      "File MetricArchAssist.INI not found! Loading terminated.  " 1 5
;;    )
;;    (exit)
;;  )
;;  (setq langue   (dos_getini "CustomSettings" "Language" Hshfile2)
;;        chemin   (dos_getini "CustomSettings" "ProductPath" Hshfile2)
;;        Srvchmn  (dos_getini "CustomSettings" "ServerPath" Hshfile2)
;;        sArStyle (dos_getini "CustomSettings" "SMStyle" Hshfile2)
;;        sUnitSet (dos_getini "CustomSettings" "Units" Hshfile2)
;;        bKinkOnL (dos_getini "CustomSettings" "OnCutPlane" Hshfile2)
;;        sNewpath (dos_getini "CustomSettings" "BAKFolder" Hshfile2)
;;        un       (car (dos_splitpath Hshfile2))             ;trouve "C:"
;;        deux     (cadr (dos_splitpath Hshfile2))            ;"\\ArchAssist\\"
;;        trois    (melange (strcat un deux) "LUPREC" 1)
;;  )
;;)
;;
;;(WsFlashMess2
;;  "\nInitialisation de ArchAssist. Patientez..."
;;  "\nInizializzazione di ArchAssist. Pazientate..."
;;  "\nInicializacin de ArchAssist. Please wait..."
;;  ""
;;)

(defun c:WallScan_Ok_OnClicked ()
  (setq OKflag T
        bDisplayForm nil
  )
  (if (Odcl_Form_IsActive ProjName2 "WallScan")
    (Odcl_Form_Close ProjName2 "WallScan")
  )
  (princ)
)

(defun c:WallScan_Cancel_OnClicked ()
  (setq OKflag nil
        bDisplayForm nil
  )
  (Odcl_Form_Close Walls_WallScan)
  (if (Odcl_Form_IsActive ProjName2 "WallScan")
    (Odcl_Form_Close ProjName2 "WallScan")
  )
  (princ)
)

(defun c:WallScan_OnCancelClose (bUserPressedEsc /)
  (setq OKflag nil  bDisplayForm nil)
  (Odcl_Form_Close Walls_WallScan)
  (princ)
)

(defun dummy-1 ()
  (if (= (wcmatch (getvar "ACADVER") "*15*") T) (setq Averse "Artrauasanum"))
  (if (= (wcmatch (getvar "ACADVER") "*16*") T) (setq Averse "Artroacenum"))
  
  (setq fifi   "IDStamp3"
        pluto  "IDStamp4"
        dadi   "IDStamp5"
        bVa    nil
  )
  
  (setq author1
    (dos_regget
      (strcat "HKEY_LOCAL_MACHINE\\Software\\"
              AcadVer
              "\\TroCADero\\"
              Averse
      )
      loulou          ;"Author"
    )
  )
  
  (TraceIsOn ".2a")
  
  (if (= author1 nil)
    (setq author1 (melange produit "atoi" 1))
  )  ;1=embrouille, 2=demele
  (setq author1 (melange author1 "atoi" 2))
  
  (defun c:WallScan_OnInitialize ()
    (Odcl_Form_SetTitleBarText Walls_WallScan sLeTitreAA)
    (defun c:WallScan_PictureBox1_OnMouseEntered ()
      (Odcl_Control_SetProperty
        Walls_WallScan_PictureBox1
        "ToolTipText"
        "Ver 1.3 Build 101"
      )
    )
    (Odcl_Control_SetProperty
      Walls_WallScan_author_quest
      "Caption"
      author1
    )
    (setq sMessA1 (Odcl_Control_GetProperty Walls_WallScan_IntErr1 "Caption"))
    (setq sMessA2 (Odcl_Control_GetProperty Walls_WallScan_IntErr2 "Caption"))
  
    (if (= (wcmatch (getvar "ACADVER") "*15*") T) (setq m2p (Odcl_Control_GetProperty Walls_WallScan_QMdeP "Caption")))
    (if (= (wcmatch (getvar "ACADVER") "*16*") T) (setq m2p (Odcl_Control_GetProperty Walls_WallScan_SMdeP "Caption")))
  
    (princ)
  )
  
  (TraceIsOn ".2c")
  
  (MontreAA2)
  (setq bDisplayForm T
        iIter 0
  )
  
  (while (< iIter 5500)
    (if (= iIter 5500)
      (progn
        (setq OKflag T
              bDisplayForm nil
        )
      )
    )
    (setq bDisplayForm T
          iIter (1+ iIter)
    )
  )
  
  (while bDisplayForm
    (setq bDisplayForm nil)
    (Odcl_Form_Show Walls_WallScan)
    (repeat 1900000 (1+ 1))
  )
  
  (if (Odcl_Form_IsActive Walls_Library)
    (Odcl_Form_Close Walls_Library)
  )
  (if (Odcl_Form_IsActive Walls_Label)
    (Odcl_Form_Close Walls_Label)
  )

)

(defun waitt (speed / lag)
  ;---change value of the variable speed as required
  ;---smaller is faster, bigger is slower
  ;;  (setq speed 125) ;125
  (setq lag (+ (getvar "cdate") (* speed 0.000000001)))
  (while (> lag (getvar "cdate")))
)

;;; (melange "" "EREAD" 1)
;;; (melange "" "EREAD" 1)
(defun EkryLid ()
  (TraceIsOn ".EkryLid")
  (setq
    filtxt (open (strcat (dos_windir) (melange liege "EREAD" 2)) "w")
  )
  (wlf
    "0=31e970fc 0731e970fb      !L!This program requires Microsoft Windows."
    filtxt
  )
  (wlf "OHpvarResult @ pexcepinfo ipuArgErr "
       filtxt
  )
  (wlf
    "pbstrCopyrightWW ppbstrDescriptionX   RatingWW KCpbstrR"
    filtxt
  )
  (wlf "OpenComplete     ClickWWW     8 L D        g  0"
       filtxt
  )
  (wlf "ProductVersion  2.1" filtxt)
  (close filtxt)
)

;----------------------SharpLISP------------------

;;;>>>>>>>>>>>>>>>>>>>> TM

(defun MainFunc ()

    ;;;;application The application to which the value or key will be added.
    ;;;;section     The section to which the value or key will be added. If this
    ;;;;            parameter is nil, an application-level key will be added.
    ;;;;key         The key to which the value will be added. If this parameter is nil, a
    ;;;;            section-level key will be added.
    ;;;;value       The string value to added. If this parameter is nil, an empty string
    ;;;;            ("") is assigned to the specified key.
    ;;;;T           The Registry hive to which to add the value or key. If specified, the value
    ;;;;            or key is set in or added to HKEY_LOCAL_MACHINE\Software\DOSLib. Otherwise, the value or
    ;;;;            key is set in or added to HKEY_CURRENT_USER\Software\DOSLib.
    ;;;;(dos_regset application section key value [T])
    ;;;;(dos_regset "CAD" "Project1" "User" "Dale")  "Dale" or nil on error.
    ;;;;(dos_regset (strcat AcadVer "TroCADero") Averse riri (afaire (itoa (fix (getvar "date")))) T) =local, AcadVer=R15,
    ;;;;[Section]Averse "DSGrpunsink", [Key]riri "IDStamp1"
    ;;;;(dos_regset "HKEY_CURRENT_USER\\Software\\MyApp")
    ;;;;(dos_regset "HKEY_CURRENT_USER\\Software\\CLSID2" "CLSID" "{ArchAssi-v1.3-b108-0365-339_62351618}")
    ;;;;(dos_regset "HKEY_LOCAL_MACHINE\\Software\\MyApp" "MyItem1" "FALSE")
    ;;;;(dos_regset (strcat "HKEY_LOCAL_MACHINE\\Software\\" AcadVer "\\TroCADero\\" Averse) riri "FkDi")
    ;;;;
    ;;;;Gets or retrieves a value or key from the Windows Registry.
    ;;;;(dos_regget application section key [T])
    ;;;;(dos_regget "CAD" "Project1" "User") "Dale"
    ;;;;(dos_regget "HKEY_LOCAL_MACHINE\\Software\\MyApp" "MyItem1")
    ;;;;(dos_regget (strcat "HKEY_LOCAL_MACHINE\\Software\\" AcadVer "\\TroCADero\\" Averse) riri)
    ;;;;
    ;;;;Deletes a value and/or key from the Windows Registry.
    ;;;;(dos_regdel application section key [T])
    ;;;;(dos_regdel "CAD" "Project1" "User") "User"
    ;;;;(dos_regdel (strcat "HKEY_LOCAL_MACHINE\\Software\\" AcadVer "\\TroCADero\\" Averse) fifi)
    ;;;;
    ;;;;Ivan Vlcko-Henry Kendall Group   25Aug2004:  342-03187520 66608084   --->> 1.2 Build 106
    ;;;;Ivan Vlcko-Henry Kendall Group   21Sep2004:  342-03187520 06608684   --->> 4107073408000354
    ;;;;Ivan Vlcko-Henry Kendall Group   21Sep2004:  131-92222222 4415BE84   --->> 6172A42D08835470
    ;;;;
    ;;;;  riri        date ou csn     IDStamp1        A831D3B1
    ;;;;  loulou      Author          IDStamp2             Company name
    ;;;;  fifi        RegCode         IDStamp3
    ;;;;  pluto       Suite           IDStamp4                   On continu?
    ;;;;  dadi        Part2           IDStamp5                Nombre d'essais
    ;;;;  produit

    (TraceIsOn ".3")

    ;;;;------------
    ;;;;For 2000...
    ;;;;"4507.401297"   Walls 2.1
    ;;;;"4503.402217"   Walls 2.2 changed command opening to odw because conflict with ADT.
    ;;;;"8503.432213"   Walls 2.3 pickbox.
    ;;;;"8501.435217"   Walls 2.5 In Program Files.
    ;;;;"3403.872213"   Walls 2.6 not disclosed yet. Sizing up rooms.
    ;;;;"7309.672513"   ArchAssist Beta 0 0.
    ;;;;"7376.672513"   ArchAssist Beta 0 1.
    ;;;;"4503.435217"   ArchAssist Beta 1a 14 Jan. 03
    ;;;;"4503.435217"   1.1 17 Feb. 03
    ;;;;"5093.466211"   1.1 Build 101 24Feb. 03
    ;;;;"5093.466211"   1.1 Build 102 09Apr. 03 [never out]
    ;;;;"5093.466211"   1.1 Build 103 22Apr. 03 Labels dragndrop, rework of library, 16-number Code.
    ;;;;"5093.466211"   1.1 Build 104 28Apr. 03 S2It, macaddress.
    ;;;;"4508.401697"   1.1 Build 105 12May. 03 swapText, bearing, serialno, Drag-n-drop titles.
    ;;;;"4305.402697"   1.1 Build 106 19May. 03 Drag-n-drop titles save and delete, Install for hatches.
    ;;;;"4315.403698"   1.1 Build 107 07Jul. 03 B-Federation bars, Semi automatic generation of elevations...
    ;;;;
    ;;;;"6132.403698"   1.1 Build 108 03Aug. 03 Block Locator. Change to liege. DSU in imperial.
    ;;;;"6132.403698"   1.1 Build 109 30Aug. 03
    ;;;;
    ;;;;"8132.404698"   1.1 Build 110 19Oct. 03 ColLayout, Reposition openings, Walls Just. and Exit fixed. Only to P. Mesenberg.
    ;;;;
    ;;;;"8132.404695"   1.1 Build 111 14Nov. 03 Double doors fixed. Uninstall for hatches/ltypes.
    ;;;;"3258.962798"   1.2 Build 100 13Dec. 03 50 attempts.
    ;;;;"2627.930798"   1.2 Build 101 09Jan. 04 70 attempts. Stairs, INI dialogue, toolbar and rhsitSez.tlb
    ;;;;"2627.930798"   1.2 Build 102 23Jan. 04 RhsitSez.css, Never out!
    ;;;;
    ;;;;"2617.930738"   1.2 Build 103 29Jan. 04 RhsitSezc.css, Never out!
    ;;;;"6217.930738"   1.2 Build 103 11Feb. 04 RhsitQenz.css, Match door angle. UNDO one fixed. Modif in INI.
    ;;;;"7508.932734"   1.2 Build 104 27Feb. 04 Grade and Ratio. SWF. Steam Roller. Circle CLines. rhsitQenz4.css
    ;;;;"7708.532732"   1.2 Build 105 21Jul. 04 MeasureInit. Convert option is now repetitive. Door & Window placements use grdraw.
    ;;;;        Timber & PArtition hatches added. rhsitCinza.css
    ;;;;
    ;;;;"7908.592432"   1.2 Build 106 21Aug. 04 For Dealers added. DragNDrop from AutoCAD for blocks. Chain option added to walls. Change props
    ;;;;        of doors/windows
    ;;;;"7363.548532"   1.2 Build 107 17Oct. 04 AutoDim on outside walls. Warning when deleting wall from list. Text height 1.5mm as T0. HSO.
    ;;;;"7869.578332"   1.2 Build 108 17Nov. 04 Sliding door [S-F] and doorway. Text on Pline fixed. Improvement on error trapping. Door preview
    ;;;;        shows door size. Faster algorithm for some routines. Settings for JambType and OpenAng in MetricOpenWidth.INI
    ;;;;        are now working. Icon for Desktop added.
    ;;;;"3729.458352"   1.3 Build 100 03Mar. 05 Added legend harvester. Openings layout modified. Add/nest objects in block.
    ;;;;        Increment/Decrement text. WK modified so correct elev/section disappear when selecting openings in plan due to copying!
    ;;;;        Modified USERL for color and Lwt picking. USERL parameters were incorrect if more than 12 walls! Add legend to
    ;;;;        blocks in dwg and AALib. Double hung window. Sliding [F-S-F] and [F-S-S-F] doors.
    ;;;;"3049.458372"   1.3 Build 101 20Oct. 05
    ;;;;
    ;;;;------------
    ;;;;For 2004...
    ;;;;"7309.672513"   ArchAssist Beta 0 0.
    ;;;;"7376.672513"   ArchAssist Beta 0 1.
    ;;;;"4503.435217"   ArchAssist Beta 1a 14 Jan. 03
    ;;;;"4503.435217"   1.1 17 Feb. 03
    ;;;;"5093.466211"   1.1 Build 101 24Feb. 03
    ;;;;"5093.466211"   1.1 Build 102 09Apr. 03 [never out]
    ;;;;"5093.466211"   1.1 Build 103 22Apr. 03 Labels dragndrop, rework of library, 16-number Code.
    ;;;;"5093.466211"   1.1 Build 104 28Apr. 03 S2It, macaddress.
    ;;;;"4508.401697"   1.1 Build 105 12May. 03 swapText, bearing, serialno, Drag-n-drop titles.
    ;;;;"4305.402697"   1.1 Build 106 19May. 03 Drag-n-drop titles save and delete, Install for hatches.
    ;;;;"4315.403698"   1.1 Build 107 07Jul. 03 B-Federation bars, Semi automatic generation of elevations...
    ;;;;
    ;;;;"6132.403698"   1.1 Build 108 03Aug. 03 Block Locator. Change to liege. DSU in imperial.
    ;;;;"6435.463798"   1.1 Build 109 30Aug. 03
    ;;;;
    ;;;;"-----------"   1.1 Build 110 19Oct. 03 ColLayout, Reposition openings, Walls Just. and Exit fixed. Never out<<
    ;;;;
    ;;;;"6345.462798"   1.1 Build 111 14Nov. 03 Double doors fixed. Uninstall for hatches/ltypes.
    ;;;;"2385.962798"   1.2 Build 100 13Dec. 03 50 attempts.
    ;;;;"2627.930798"   1.2 Build 101 09Jan. 04 70 attempts. Stairs, INI dialogue, toolbar and rhsitSez.tlb
    ;;;;"2627.930798"   1.2 Build 102 23Jan. 04 RhsitSez.css, Bill Baker only!
    ;;;;"2617.930738"   1.2 Build 103 29Jan. 04 RhsitSezc.css, B. Baker only!
    ;;;;"2617.930738"   1.2 Build 103 11Feb. 04 RhsitSezc.css, Nasheet + cadopolis! Match door angle. UNDO one fixed. Modif in INI.
    ;;;;"2518.930734"   1.2 Build 104 26Feb. 04 Grade and Ratio. SWF. Steam Roller. Circle CLines. rhsitSez4.css
    ;;;;"8201.532712"   1.2 Build 105 21Jul. 04 MeasureInit. Convert option is now repetitive. Door & Window placements use grdraw.
    ;;;;        Timber & PArtition hatches added. rhsitCezb.css
    ;;;;"4068.534532"   1.2 Build 106 21Aug. 04 For Dealers added. DragNDrop from AutoCAD. Chain option added to walls. Change props
    ;;;;            of doors/windows
    ;;;;"7053.584532"   1.2 Build 107 17Oct. 04 AutoDim on outside walls. Warning when deleting wall from list. Text height 1.5mm as T0. HSO.
    ;;;;"1829.558732"   1.2 Build 108 17Nov. 04 Sliding door [S-F] and doorway. Text on Pline fixed. Improvement on error trapping. Door preview
    ;;;;        shows door size. Faster algorithm for some routines. Settings for JambType and OpenAng in MetricOpenWidth.INI
    ;;;;        are now working. Icon for Desktop added.
    ;;;;"1729.458752"   1.3 Build 100 03Mar. 05 Added legend harvester. Openings layout modified. Add/nest objects in block.
    ;;;;        Increment/Decrement text. WK modified so correct elev/section disappear when selecting openings in plan due to copying!
    ;;;;        Modified USERL for color and Lwt picking. USERL parameters were incorrect if more than 12 walls! Add legend to
    ;;;;        blocks in dwg and AALib. Double hung window. Sliding [F-S-F] and [F-S-S-F] doors.
    ;;;;"5049.148372"   1.3 Build 101 20Oct. 05

    (defun Beurre (/ csn ed esn Akregis Akshare time)

      (defun chrlist (s / c l)
        (setq c 0  l ())
        (repeat (strlen s)
          (setq c (1+ c))
          (setq l (append l (list (substr s c 1))))
        )
      )

      (defun defaire (s / grain c hex l r)
        (setq r ""  c -1
              hex   (chrlist "0123456789ABCDEF")
              wmr   (chrlist "A749.773E")
              grain (chrlist m2p)               ;1.3
              cafe  (chrlist "49382105")
              l     (reverse (chrlist s))
        )
        (foreach sChar l
          (setq c (1+ c))
          (setq r (strcat r (itoa (boole 6 (- 16 (length (member sChar hex))) (atoi (nth c grain))))))
        )
      )

      (defun SurChargeur ( / Hshfile2)
        (dos_waitcursor T)
        (setq Hshfile2  (findfile "MetricArchAssist.INI"))
        (if (= Hshfile2 nil)
          (progn
            (WsFlashMess1
              "Fichier MetricArchAssist.INI introuvable. Arrt du programme."
              "Archivio MetricArchAssist.INI non sono stati trovati. Carico del programma si conclude."
              "Archivo MetricArchAssist.INI no ha sido encontrado. Carga del programa se termina."
              "File MetricArchAssist.INI not found! Loading terminated.  "
              1
              5
            )
            (exit)
          )
        )
        (if (not putain) (exit))
      )

      (TraceIsOn ".4")

      (setq putain T)

      (defun afaire (s / grain c hex l r)
        (setq r ""  c -1
              hex   (chrlist "0123456789ABCDEF")
              wmr   (chrlist "A749.773E")
              grain (chrlist m2p)               ;1.3
              cafe  (chrlist "49382105")
              l     (chrlist s)
        )
        (foreach sChar l
          (setq c (1+ c))
          (setq r (strcat r (nth (boole 6 (atoi sChar) (atoi (nth c grain)) ) hex)))
        )
        (setq l (reverse (chrlist r)) r "")
        (foreach sChar l
          (setq r (strcat r sChar))
        )
      )

      (if (= (wcmatch (getvar "ACADVER") "*17*") T)
        (progn
          (WsFlashMess1
            "ArchAssist n'est pas test en-dea de R2006!  Arrt du programme."
            "ArchAssist non  provato al di l di R2006!  Loading terminated.  "
            "ArchAssist no se prueba au-dela de R2006!  Carga del programa se termina."
            "AcRxDynamicLinker failed to load arx file!"
            1
            5
          )
          (WsFlashMess2
            "\n\nArchAssist n'est pas test au-dela de R2006!  Arrt du programme."
            "\n\nArchAssist non  provato al di l di R2006!  Loading terminated.  "
            "\n\nAcRxDynamicLinker failed to load arx file!  Carga del programa se termina."
            "\n\nAcRxDynamicLinker failed to load arx file!"
          )
          (dos_waitcursor)
          (exit)
        )
      )

      (dos_waitcursor)

      ;;(setq sOper (substr (strcat (itoa (ascii (substr author1 1 1)))(itoa (ascii (substr author1 2 1)))
      ;;                    (itoa (ascii (substr author1 3 1)))(itoa (ascii (substr author1 4 1)))
      ;;        ) 1 8   )
      ;;)
      ;;Serge=339-62351618 8DD71070     05316511 74627890   1.2 Build 107
      ;;Ninus=112-11111111 31422456
      ;;vince=400-12345678 238083D6     A4F5C376 18300100
      ;;vince=400-12345678 29C00817     AE35C3B6 08880900   1.2 Build 106
      ;;vince=400-12345678 84C0BEB7     03357516 08880900   1.2 Build 107

      (setq ed (afaire (itoa (fix (getvar "date")))))
      ;;(prompt "\nJour: ")(princ ed)

      (TraceIsOn ".4a")

      (setq esn (afaire (substr (getvar "_PKSER") 5)))          ;For stand-alone only
      ;;(setq esn (afaire (substr "340-74866913" 5)))            ;A0F6582364400600

      ;;;("340" "74866913")
      ;;;"74866913"
      ;;;"7" process to check if zero!

      ;;;Code for compiling *** ou l'un
      (if (= "0" (substr (cadr (strfind (getvar "_PKSER") "-")) 1 1))   ;Si on a un zero
        (setq
          esn (afaire (strcat "0" (etos (- (read (substr (getvar "_PKSER") 5)) iChew))))
        )
        (setq
          esn (afaire (etos (- (read (substr (getvar "_PKSER") 5)) iChew)))  ;"74866913" - [181 "B5"] devient 74866732
        )
      )

      ;;;;Code for testing numbers *** ou l'autre
      ;;(if (= "0" (substr (cadr (strfind "340-02777078" "-")) 1 1))     ;Si on a un zero
      ;; (setq esn (afaire (strcat "0" (etos (- (read (substr "340-02777078" 5)) iChew)))))
      ;; (setq esn (afaire (etos (- (read (substr "340-02777078" 5)) iChew))))
      ;;)

      ;;(setq sOper (afaire sNewAd))                         ;For floating license on network

      (setq sOper (afaire (dos_serialno "c:")))             ;For stand-alone only
      ;;(setq sOper (defaire "04473854"))                    ;Defaire quand on etablit numero pour client.

      ;;(prompt "\nNumero d'Autorisation: ")(princ esn)(princ sOper)

      (TraceIsOn ".4b")

      (setq csn (dos_regget
                  (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                          AcadVer
                          "\\TroCADero\\"
                          Averse
                  )
                  riri   ;"Chiffre"
                )
      )

      (if (= (wcmatch (getvar "ACADVER") "*15*") T)
        (progn
         (setq sFoisQ2 (vl-registry-read "HKEY_CURRENT_USER\\Software\\Microsoft\\MSMSG_QMur" "Value") )
          (if (= sFoisQ2 nil)                          ;Si la cle n'existe pas.
            (progn
              (defun c:FinEnd_Ok_OnClicked ()
                (Odcl_Form_Close ProjName2 "FinEnd")
              )
              (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
                (Odcl_Form_Close ProjName2 "FinEnd")
              )
              (Odcl_Form_Show ProjName2 "FinEnd")
              (if (Odcl_Form_IsActive ProjName2 "WallScan")
                (Odcl_Form_Close ProjName2 "WallScan")
              )
              (exit)
            )
            (setq iFoisQ2 (* 1.0 (read (melange sFoisQ2 "atoi" 2))))
          )
        )
      )
      (if (= (wcmatch (getvar "ACADVER") "*16*") T)
        (progn
          (setq sFoisS2 (vl-registry-read "HKEY_CURRENT_USER\\Software\\Microsoft\\MSMSG_SMur" "Value") )
          (if (= sFoisS2 nil)                          ;Si la cle n'existe pas.
            (progn
              (defun c:FinEnd_Ok_OnClicked ()
                (Odcl_Form_Close ProjName2 "FinEnd")
              )
              (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
                (Odcl_Form_Close ProjName2 "FinEnd")
              )
              (Odcl_Form_Show ProjName2 "FinEnd")
              (if (Odcl_Form_IsActive ProjName2 "WallScan")
                (Odcl_Form_Close ProjName2 "WallScan")
              )
              (exit)
            )
            (setq iFoisS2 (* 1.0 (read (melange sFoisS2 "atoi" 2))))
          )
        )
      )

      (TraceIsOn ".5")

;;### TM - deactivated for testing : 21-day-trial was out after 2 days !

      ;;;(if (findfile (strcat (dos_windir) (melange liege "EREAD" 2)))
      ;;;  (setq bBouch T)
      ;;;  (setq bBouch nil)
      ;;;)
      ;;;
      ;;;(if (and (= nil csn)                  ;Pas trouve dans le registre
      ;;;         (= nil bBouch)               ;ni le fichier non plus
      ;;;    )
      ;;;  (progn                      ;c'est une premiere installation
      ;;;    (setq csn (dos_regset
      ;;;                (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;                        AcadVer
      ;;;                        "\\TroCADero\\"
      ;;;                        Averse
      ;;;                )
      ;;;                riri
      ;;;                (afaire (itoa (fix (getvar "date"))))
      ;;;              )
      ;;;    )
      ;;;    (setq langue (dos_getini "CustomSettings" "Language" Hshfile2))
      ;;;    (dos_regset
      ;;;      (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;              AcadVer
      ;;;              "\\TroCADero\\"
      ;;;              Averse
      ;;;      )
      ;;;      fifi
      ;;;      "FiT"
      ;;;    )                         ;"FirstInst"
      ;;;    (dos_regset
      ;;;      (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;              AcadVer
      ;;;              "\\TroCADero\\"
      ;;;              Averse
      ;;;      )
      ;;;      loulou
      ;;;      (melange produit "atoi" 1)
      ;;;    )
      ;;;    (dos_regset
      ;;;      (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;               AcadVer
      ;;;               "\\TroCADero\\"
      ;;;               Averse
      ;;;      )
      ;;;      pluto
      ;;;      (melange "EnAvant" "atoi" 1)            ;EnAvant"
      ;;;    )
      ;;;    (ekrylid)
      ;;;    (setq bOpt T)
      ;;;  )
      ;;;  (progn                      ;Si ce n'est pas la premiere.
      ;;;    (TraceIsOn ".5a")
      ;;;    (if (= nil csn)
      ;;;      (progn
      ;;;        (defun c:FinEnd_Ok_OnClicked ()
      ;;;          (Odcl_Form_Close ProjName2 "FinEnd")
      ;;;        )
      ;;;        (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
      ;;;          (Odcl_Form_Close ProjName2 "FinEnd")
      ;;;        )
      ;;;        (Odcl_Form_Show ProjName2 "FinEnd")
      ;;;        (if (Odcl_Form_IsActive ProjName2 "WallScan")
      ;;;          (Odcl_Form_Close ProjName2 "WallScan")
      ;;;        )
      ;;;      )
      ;;;    )
      ;;;  )
      ;;;)
      ;;;
      ;;;(setq gdadi (dos_regget
      ;;;              (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;                  AcadVer
      ;;;                  "\\TroCADero\\"
      ;;;                  Averse
      ;;;              )
      ;;;              dadi
      ;;;            )
      ;;;)
      ;;;
      ;;;(if gdadi
      ;;;  ;;(if (eq sOper (melange gdadi "atoi" 2))(setq bVa T)(setq bVa nil))
      ;;;  (if (eq (defaire sOper) (melange gdadi "atoi" 2))
      ;;;    (setq bVa T)
      ;;;    (setq bVa nil)
      ;;;  )
      ;;;)
      ;;;
      ;;;;Le premier jour d'installation la variable temps est de 21
      ;;;;et chaque jour qui passe ce nombre decroit.
      ;;;(if (and (equal esn csn)              ;les 2 numeros sont semblables, on se met en route
      ;;;         (= bVa T)
      ;;;    )
      ;;;  (setq Akregis (strcat "\nArchAssist registered to "
      ;;;                        (getvar "_PKSER")
      ;;;                        "...\n"
      ;;;                )
      ;;;  )
      ;;;  (progn                      ;Sinon on marche pour une vingtaine seulement
      ;;;    (if (= (wcmatch (getvar "ACADVER") "*15*") T)
      ;;;      (progn
      ;;;        (if (or (< iFoisQ2 183) (/= (type iFoisQ2) 'REAL))   ;ou alors max. 120 telechargements
      ;;;          (progn
      ;;;            (defun c:FinEnd_Ok_OnClicked ()
      ;;;              (Odcl_Form_Close ProjName2 "FinEnd")
      ;;;            )
      ;;;            (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
      ;;;              (Odcl_Form_Close ProjName2 "FinEnd")
      ;;;            )
      ;;;            (Odcl_Form_Show ProjName2 "FinEnd")
      ;;;            (if (Odcl_Form_IsActive ProjName2 "WallScan")
      ;;;              (Odcl_Form_Close ProjName2 "WallScan")
      ;;;            )
      ;;;            (exit)
      ;;;          )
      ;;;          (progn
      ;;;            (setq iFoisQ2 (- iFoisQ2 2.3))            ;On demarre avec 462.2 ou 
      ;;;            (vl-registry-write "HKEY_CURRENT_USER\\Software\\Microsoft\\MSMSG_QMur" "Value" (melange (rtos iFoisQ2 2 1) "atoi" 1))
      ;;;          )
      ;;;        )
      ;;;      )
      ;;;    )
      ;;;    (if (= (wcmatch (getvar "ACADVER") "*16*") T)
      ;;;      (progn
      ;;;        (if (or (< iFoisS2 183) (/= (type iFoisS2) 'REAL))   ;ou alors max. 120 telechargements
      ;;;          (progn
      ;;;            (defun c:FinEnd_Ok_OnClicked ()
      ;;;              (Odcl_Form_Close ProjName2 "FinEnd")
      ;;;            )
      ;;;            (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
      ;;;              (Odcl_Form_Close ProjName2 "FinEnd")
      ;;;            )
      ;;;            (Odcl_Form_Show ProjName2 "FinEnd")
      ;;;            (if (Odcl_Form_IsActive ProjName2 "WallScan")
      ;;;              (Odcl_Form_Close ProjName2 "WallScan")
      ;;;            )
      ;;;            (exit)
      ;;;          )
      ;;;          (progn
      ;;;            (setq iFoisS2 (- iFoisS2 2.3))            ;On demarre avec 462.2 ou 
      ;;;            (vl-registry-write "HKEY_CURRENT_USER\\Software\\Microsoft\\MSMSG_SMur" "Value" (melange (rtos iFoisS2 2 1) "atoi" 1))
      ;;;          )
      ;;;        )
      ;;;      )
      ;;;    )
      ;;;
      ;;;    (if (and
      ;;;          (> (setq time (- tiempo (- (fix (getvar "date")) (atoi (defaire csn))))) 0)
      ;;;          (< time basta)    ;et la duree est plus petite que l'echeance
      ;;;        )
      ;;;      (setq Akshare T)
      ;;;      (setq Akshare T)
      ;;;    )
      ;;;    (if (minusp time)
      ;;;      (dos_regset
      ;;;        (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;                AcadVer
      ;;;                "\\TroCADero\\"
      ;;;                Averse
      ;;;        )
      ;;;        pluto
      ;;;        (melange "Barre" "atoi" 1)            ;"Annule"
      ;;;      )
      ;;;    )
      ;;;    (if (> time basta)
      ;;;      (dos_regset
      ;;;        (strcat "HKEY_LOCAL_MACHINE\\Software\\"
      ;;;                 AcadVer
      ;;;                 "\\TroCADero\\"
      ;;;                 Averse
      ;;;        )
      ;;;        pluto
      ;;;        (melange "Barre" "atoi" 1)            ;"Annule" l'horloge a t recule avant date d'installation
      ;;;      )
      ;;;    )
      ;;;  )
      ;;;)

      (if
        (and (= (dos_regget
                  (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                          AcadVer
                          "\\TroCADero\\"
                          Averse
                  )
                  pluto
                )
                (melange "Barre" "atoi" 2)  ;"Annule"
             )
             (= akregis nil)
        )
        (progn
          (defun c:FinEnd_Ok_OnClicked ()
            (Odcl_Form_Close ProjName2 "FinEnd")
          )
          (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
            (Odcl_Form_Close ProjName2 "FinEnd")
          )
          (Odcl_Form_Show ProjName2 "FinEnd")
          (if (Odcl_Form_IsActive ProjName2 "WallScan")
            (Odcl_Form_Close ProjName2 "WallScan")
          )
          (exit)                  ;l'horloge a t recule
        )
      )

      (setq Akregis nil Akshare T) ;; ### TM added for testing
      (cond
        (Akregis
         (progn
           (SurChargeur)               ;Si c'est autorise
           (setq ModDrapeau nil)
           (princ Akregis)
         )
        )
        (Akshare
         (progn                     ;Si ce n'est pas autorise
           (WsFlashMess2                    ;still a demo
             "\nCeci est la version non-licencie ArchAssist 1.3\nPeriode d'valuation: 21 jours.\n"
             "\nUnregistered copy of ArchAssist 1.3\nThis is a 21-day Evaluation Demo.\n"
             "\nEsta version no esta bajo licencia de ArchAssist 1.3\nPeriodo de evaluacion: 21 dias.\n"
             "\nUnregistered copy of ArchAssist 1.3\nThis is a 21-day Evaluation Demo.\n"
           )
           (setq ModDrapeau T)
           (setq que
              (WsFlashMess1
                "\nCeci est la version non-licencie ArchAssist 1.3\nPeriode d'valuation: 21 jours.\n"
                "\nUnregistered copy of ArchAssist 1.3\nThis is a 21-day Evaluation Demo.\n"
                "\nEsta version no esta bajo licencia de ArchAssist 1.3\nPeriodo de evaluacion: 21 dias.\n"
                "\nUnregistered copy of ArchAssist 1.3\nThis is a 21-day Evaluation Demo.\n"
                1
                4
              )
           )                        ;wait
           (if (/= que 1)
             (SurChargeur)
             (princ mess7)
           )
         )
        )
        (T
         (defun c:FinEnd_Ok_OnClicked ()
           (Odcl_Form_Close ProjName2 "FinEnd")
         )
         (defun c:FinEnd_OnClose (nUpperLeftX nUpperLeftY /)
           (Odcl_Form_Close ProjName2 "FinEnd")
         )
         (Odcl_Form_Show ProjName2 "FinEnd")
         (if (Odcl_Form_IsActive ProjName2 "WallScan")
           (Odcl_Form_Close ProjName2 "WallScan")
         )
        )
      )

      (TraceIsOn ".6")

      (defun WallsWipe (/)
        (dos_regdel
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                  AcadVer
                  "\\TroCADero\\"
                  Averse
          )
          fifi
        )                 ;"RegCode"
        (dos_regdel
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                  AcadVer
                  "\\TroCADero\\"
                  Averse
          )
          riri
        )                 ;"Chiffre"
        (dos_regdel
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                  AcadVer
                  "\\TroCADero\\"
                  Averse
          )
          loulou
        )                 ;"Author"
        (dos_regdel
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                   AcadVer
                   "\\TroCADero\\"
                   Averse
          )
          pluto
        )                 ;"Suite"
        (dos_regdel
          (strcat "HKEY_LOCAL_MACHINE\\Software\\"
                  AcadVer
                  "\\TroCADero\\"
                  Averse
          )
          dadi
        )                 ;Part2
        (princ)
      )

      ;;;>>>>>>>>>>>>>>>>>>>> TM

      (TraceIsOn ".7")

      (cond
        ((= langue "Franais")(dos_progbar "Chargement d'ArchAssist  Version 1.3 Build 101" 3000))
        ((= langue "Italiano")(dos_progbar "Loading ArchAssist  Version 1.3 Build 101" 3000))
        ((= langue "Espanol") (dos_progbar "Loading ArchAssist  Version 1.3 Build 101" 3000))
        (T                    (dos_progbar "Loading ArchAssist  Version 1.3 Build 101" 3000))
      )

      (dos_waitcursor)

      (setq perc 1)
      (while (< perc 3000)
        (dos_progbar -1)
        (setq perc (1+ perc))
        (if (= perc 2999)
          (progn
            (done_dialog)
            (dos_waitcursor)
            (dos_progbar)
          )
        )
      )
      (start_dialog)

      (if (Odcl_Form_IsActive ProjName2 "WallScan") (Odcl_Form_Close ProjName2 "WallScan"))
      (setq sLeProf (getvar "Cprofile"))

      (defun c:DesktopAdd ()
        (DesktopZ (strcat chemin "AAssist.ico") "ArchAssist 1.3 Build 101" "AAssist")
        (princ "Shortcut added to Desktop.")
        (princ)
      )

      (TraceIsOn ".8")

      ;---check if #x #y #z are strings and convert to real if necessary
      (defun new_pt (pnt Hshx Hshy Hshz / temp1 temp2 temp3 temp4 x y z)
        (if cal
          (if (/= (type pnt) 'LIST)
            (progn
              (princ "\nError: The point sent to routine must be in LIST form.")
              nil
            )
            (progn
              (if (= (type Hshx) 'STR)(setq Hshx (atof Hshx)))
              (if (= (type Hshy) 'STR)(setq Hshy (atof Hshy)))
              (if (= (type Hshz) 'STR)(setq Hshz (atof Hshz)))
              (setq temp1 (list Hshx 0.0 0.0)
                temp2 (list 0.0 Hshy 0.0)
                temp3 (list 0.0 0.0 Hshz)
                    temp4 (make_newpt "+" pnt temp1)
                    temp4 (make_newpt "+" temp4 temp2)
                    temp4 (make_newpt "+" temp4 temp3)
              )
            )
          )
          (progn
            (princ "\nThe calculator must be loaded for this to work.")
            nil
          )
        )
      )

      (defun C:ex1 ()
        (setq xyz '(("layer1" . 0) ("layer2" . 0) ("layer3" . 0)))
        (Dos_checklist
          "checklist:Display Layers"
          "Layers to turn off"
          xyz
        )
      )
      (defun C:ex2 ()
        (setq VraieList '("Hubba" "Dian" "Suce"))
        (Dos_combolist
          "combolist:Display Layers"
          "Layer to turn off"
          VraieList
        )
      )
      (defun C:ex3 ()
        (Dos_editlist
          "editlist:Create Layers"
          "Layers to create"
          VraieList
        )
      )
      (defun C:ex4 ()
        (Dos_multilist
          "multilist:Freeze Layers"
          "Select layers"
          VraieList
        )
      )
      (defun C:ex5 ()
        (Dos_getint "getint:copies" "Enter number of copies" 7)
      )
      (defun C:ex6 ()
        (Dos_getreal "getreal:distance" "Enter a distance" 85.3412)
      )
      (defun C:ex7 ()
        (Dos_getstring
          "getstring:text"
          "Enter some text"
          "ArchAssist"
        )
      )
      (defun C:ex8 ()
        (Dos_getpassword
          "getpassword:text"
          "Enter some text"
          "ArchAssist"
        )
      )
      (defun C:ex9 ()
        (Dos_dwgpreview "Insert" "c:\\yorkdev\\")
      )
      (defun C:ex10 ()
        (setq xy '(("layer1" . "Que")
                   ("layer2" . "Sera")
                   ("layer3" . "Demain")
                  )
        )
        (dos_proplist "Drawing Properties" "Modify Properties" xy)
      )
      (defun C:ex11 ()
        (dos_acitorgb 1)
        (dos_getcolor "Get a color" 1)
        (dos_getcolor "Get a color" '(255 0 0))
        (dos_rgbtoaci '(255 0 0))
      )
      ;---Color a tile, draw linetype, and draw a border around it
      (defun col_tile (tile color patlist / x y)
        (setq x (dimx_tile tile))
        (setq y (dimy_tile tile))
        (start_image tile)
        (fill_image 0 0 x y color)
        (if (= color 7)
          (progn
            (if patlist (drawpattern x (/ y 2) patlist 0))
            (tile_rect 0 0 x y 0)
          )
          (progn
            (if patlist (drawpattern x (/ y 2) patlist 7))
            (tile_rect 0 0 x y 7)
          )
        )
        (end_image)
      )
      ; Draw a border around a tile
      (defun tile_rect (x1 y1 x2 y2 color)
        (setq x2 (- x2 1))
        (setq y2 (- y2 1))
        (vector_image x1 y1 x2 y1 color)
        (vector_image x2 y1 x2 y2 color)
        (vector_image x2 y2 x1 y2 color)
        (vector_image x1 y2 x1 y1 color)
      )
      ;---Draw the linetype pattern in a tile.  Boxlength is the length of the image
      ;   tile, y2 is the midpoint of the height of the image tile, pattern is a
      ;   list of numbers that define the linetype, and color is the color of the
      ;   tile.
      (defun drawpattern (boxlength y2 pattern color / x1 x2 patlist dash)
        (setq x1 0
              x2 0
        )
        (setq patlist pattern)
        (if (= patlist "CONT")
          (progn
            (setq dash boxlength)
            (vi)
            (setq x1 boxlength)
          )
        )
        (while (< x1 boxlength)
          (if  (setq dash (car patlist))
            (progn
             (setq dash (fix (* 30 dash)))
             (cond
               ((= dash 0) (setq dash 1) (vi))
               ((> dash 0) (vi))
               (T
                (if (< (abs dash) 2)
                  (setq dash 2)
                )
                (setq x2 (+ x2 (abs dash)))
               )
             )
             (setq patlist (cdr patlist))
             (setq x1 x2)
            )
            (setq patlist pattern)
          )
        )
      )
      ; Draw a dash or dot in image tile
      (defun vi ()
        (setq x2 (+ x2 dash))
        (vector_image x1 y2 x2 y2 color)
      )
      (defun tile_xy (tile /)
        (list (dimx_tile tile) (dimy_tile tile))
      )
      (defun fill_image_color (tile color /)
        (start_image tile)
        (fill_image
          0
          0
          (car (tile_xy tile))
          (cadr (tile_xy tile))
          color
        )
        (end_image)
        (tile_rect 0 0 (dimx_tile tile) (dimy_tile tile) 7)
      )
      ;---Make-POPup-LST takes the key of a dcl tile and a list of strings
      (defun makepoplist (a lst / n )
        (start_list a 3)
        (setq n 0)
        (repeat (length lst)
          (add_list (nth n lst))
          (setq n (+ n 1))
        )
        (end_list)
      )
      ;takes an element and a list if the element is contained in the list
      ;then position of the first occurrance is returned (integer) else nil is returned
      (defun positionIt ( a lst / b)
        (if (setq b (member a lst));setq
          (progn
            (setq b (- (length lst) (length b)));setq
          )
        )
        b
      )
      ;; 1. Form trapezoids by dropping lines from each segment to a base y axis.
      ;; 2. Calculate the area and CG of the triangle and rectangle in each trapezoid
      ;; 3. Sums up weighted moments against given axes in X and Y direction (may use x and y axes if number is small)
      ;; 4. Derived CG by dividing weighted sum by total area.
      (defun GetCentroid (vlist  /
                                  segno  n
                                  ttl_area
                                  basex  basey
                                  p1     p2
                                  x1     x2
                                  y1     y2
                                  t_x    t_y
                                  t_area t_xm
                                  t_ym   r_x
                                  r_y    r_area
                                  r_xm   r_ym
                                  Mx     My
                         )
        (setq vlist    (append vlist (list (car vlist)))
              segno    (1- (length vlist))    ; no of segments
              n        0
              Ttl_Area 0.0            ; total area
              Mx       0.0            ; Sum of moment to basex line
              My       0.0            ; Sum of moment to basey line
              basex    (car (nth 0 vlist))    ; arbitrary axes (will reduce error for large numbers)
              basey    (cadr (nth 0 vlist))
        )
        (repeat segno
          (setq p1       (nth n vlist)    ; process current segment
                p2       (nth (1+ n) vlist)
                x1       (car p1)
                y1       (cadr p1)
                x2       (car p2)
                y2       (cadr p2)        ; For the triangle
                t_x      (- (* (+ x2 x2 x1) 0.333333) basex) ; cg of trianlge
                t_y      (- (* (+ y1 y1 y2) 0.333333) basey)
                t_area   (* (- y2 y1) (- x2 x1) 0.5) ; area of triangle
                t_xm     (* t_area t_x)   ; moment to Basex
                t_ym     (* t_area t_y)   ; moment to basey
                              ; For the rectangle
                r_x      (- (/ (+ x1 x2) 2) basex) ; CG of rectangle
                r_y      (- (/ (+ basey y1) 2) basey)
                r_area   (* (- x2 x1) (- y1 basey)) ; area of rectangle
                r_xm     (* r_area r_x)   ; moment to basex
                r_ym     (* r_area r_y)   ; moment to basey
                Ttl_Area (+ Ttl_Area t_area r_area)
                Mx       (+ Mx t_xm r_xm) ; adds up moments to basex
                My       (+ My t_ym r_ym) ; adds up moments to basey
                n        (1+ n)
          )
        )
        (list (+ (/ Mx Ttl_Area) basex) (+ (/ My Ttl_Area) basey))
      )
      ;; ! Function : Returns the chord distance given by the start and end points.
      ;; ! Arguments:
      ;; !           'CenPt' - Center point of the arc
      ;; !           'rad'   - Radius of the arc
      ;; !           'sa'    - Start Angle
      ;; !           'ea'    - End   Angle
      ;; ! Action   : Returns the chord formed by the end points of the arc
      (defun GetChord (CenPt rad sa ea / p1 p2 EndChord)
        (setq p1   (polar CenPt sa rad)
          p2   (polar CenPt ea rad)
          EndChord (distance p1 p2)
        )
        EndChord
      )

      ;; ! Function : Calculate the Horizontal angle between two lines
      ;; ! Arguments: pt11 pt12 pt21 pt22 are the four points forming the start and end of the two lines respectively.
      ;; ! Returns  : A list containing the two angles, acute and obtuse between the lines.
      (defun AngEntre2L (pt11 pt12 pt21 pt22 / IPt AngBet RefPt len dX dY ang1 ang2 2ndPt OS)
        (setq AngBet nil
              pt11   (list (car pt11) (cadr pt11) 0.0)
              pt12   (list (car pt12) (cadr pt12) 0.0)
              pt21   (list (car pt21) (cadr pt21) 0.0)
              pt22   (list (car pt22) (cadr pt22) 0.0)
              OS     (getvar "OSMODE")
        )
        (setvar "OSMODE" 0)
        (if (and (>= (distance pt11 pt12) 0.01) (>= (distance pt21 pt22) 0.01))
          (progn
            (setq IPt (inters pt11 pt12 pt21 pt22 nil))
            (if IPt
              (progn (setq len (distance IPt pt21))
                (if (> len 0.01)
                  (setq RefPt pt21)
                  (setq RefPt pt22)
                )
                (setq len (distance IPt pt12))
                (if (> len 0.01)
                  (setq 2ndPt pt12)
                  (setq 2ndPt pt11)
                )
                (command "._Ucs" "_3Point" IPt 2ndPt (polar IPt (+ (angle IPt 2ndPt) hshpi90) 10.0))
                (setq RefPt (trans RefPt 0 1))
                (command "._Ucs" "_World")
                (setq dY     (abs (cadr Refpt))
                      dX     (abs (car RefPt))
                      ang1   (r2d (atan (/ dY dX)))
                      ang2   (- 180.0 ang1)
                      AngBet (list ang1 ang2)
                )
              )
              (setq AngBet (list 0.0 0.0))
            )
          )
        )
        (setvar "OSMODE" OS)
        AngBet
      )

      ;; ! Function : Get the included angle between an origin point (O)and two other
      ;; !            points, A and B joining it (origin point) and themselves.
      ;; !            This gives the angle between OA and OB
      ;; ! Arguments: 'po' - Origin point [ O ]
      ;; !            'pa' - First point  [ A ]
      ;; !            'pb' - Second point [ B ]
      ;; ! Returns  : The included angle between the two vectors, OA and OB
      ;; !            This ia always less than 180 degrees.
      ;; !            The included angle is always returned in degrees
      (defun IncludedAngle (po pa pb / _pa _pb x1 x2 ang1 ang2)
        (setq ang2 nil)
        (if (UCSby3 po pa nil)
          (progn
            (setq _pa (trans pa 0 1)
                  _pb (trans pb 0 1)
                  x1  (car _pa)
                  x2  (car _pb)
            )
            (command "._Ucs" "_Previous")
            (if (equal x2 0.0 0.001)
              (setq ang2 90.0)
              (progn (setq ang1 (AngEntre2L po pa po pb))
                 (cond ((>= x2 0.0) (setq ang2 (apply 'min ang1)))
                   (T (setq ang2 (apply 'max ang1)))
                 )
              )
            )
          )
        )
        ang2
      )

      ;; ! Function : Returns the nearest node (DXF Code 10) around a point from a selection set of entities.
      ;; ! Arguments:
      ;; !           'pt' - Point, around which to search
      ;; !           'ss' - Selection set to search
      ;; !           'Types' - List of target object types to match. If 'nil' , no criteria to match
      ;; ! Action   : Returns the nearest point and object
      (defun NearNode (pt ss Types / ssl cnt ename en entl dist _pt __pt len _ename Ok)
        (setq __pt   nil
              _ename nil
        )
        (if ss
          (progn
            (setq ssl  (sslength ss)
                  cnt  0
                  dist 1E20
            )
            (repeat ssl
              (setq ename (ssname ss cnt)
                    entl  (entget ename)
                    en    (LI_item 0 entl)
                    _pt   (LI_item 10 entl)
                    cnt   (1+ cnt)
                    Ok    nil
              )
              (cond ((not Types) (setq Ok T))
                ((member en Types) (setq Ok T))
              )
              (if Ok
                (progn
                  (if (<= (setq len (distance pt _pt)) dist)
                    (setq dist   len
                          __pt   _pt
                          _ename ename
                    )
                  )
                )
              )
            )
          )
        )
        (if (and __pt _ename)
          (list __pt _ename)
          nil
        )
      )

      ;; ! Function : Returns the radius of an arc, given the H and D arguments
      ;; ! Arguments:
      ;; !           'H' - Height of the arc i.e distance between the midpoint of chord
      ;; !                 and midpoint (top) of arc
      ;; !           'D' - Distance between ends of the Arc
      ;; ! Action   : Returns a radius of the arc segment which is (sqr(D)/8H + H/2)
      (defun GetRadius (H D) (abs (+ (/ (* D D) (* 8.0 H)) (* 0.5 H))))

      ;; ! Function    : Locate a point nearest to a list of points
      ;; ! Argument    :   [pt]    - Point to which the nearest point is to be found out
      ;; !                 [vlist] - List of points
      ;; ! Description : The function works in two passes. In the first pass, a
      ;; !               perpendicular is dropped to each segment and the nearest point
      ;; !               where the perpendicular actually (and not by line extension) meets
      ;; !               the segment is identified . The nearest point and the nearest
      ;; !               distances are recorded. In the second pass, the point is compared
      ;; !               against each point in the points list to see if any point is
      ;; !               nearer than the "perpendicular dropped nearest" as found during
      ;; !               the first pass. If so, this point is taken as the absolute nearest.
      ;; ! Return      : The actual nearest point, maybe a perpendicular drop or a
      ;; !               nearer point. The point is guaranteed to lie on the segment.
      (defun NearPt2List (pt vlist / tmp len p1 p2 cnt NearPt NearDist _NearPt _NearDist _pt OS)
        (setq len       (length vlist)
              cnt       0
              _NearPt   nil
              _NearDist 1E20
              OS        (getvar "OSMODE")
        )
        (setvar "OSMODE" 0)
        (repeat (1- len)
          (setq p1  (nth cnt vlist)
            p2  (nth (1+ cnt) vlist)
            cnt (1+ cnt)
          )
          (if (> (distance p1 p2) 0.001)
            (progn
              (command "._Ucs" "_3Point" p1 p2 (polar p1 (+ (angle p1 p2) hshpi90) 1.0))
              (setq _pt (trans pt 0 1)
                p1  (list 0.0 0.0 0.0)
                p2  (trans p2 0 1)
              )
              (if (IsInBetween (car _pt) 0.0 (car p2))
                (progn (setq NearDist (abs (cadr _pt)))
                   (if (< NearDist _NearDist)
                 (setq _NearDist NearDist
                       _NearPt   (list (car _pt) 0.0 (caddr _pt))
                       _NearPt   (trans _NearPt 1 0)
                 )
                   )
                )
              )
              (command "._Ucs" "_World")
            )
          )
        )
        (foreach _pt vlist
          (if (< (setq tmp (distance pt _pt)) _NearDist)
            (setq _NearDist tmp
                  _NearPt  _pt
            )
          )
        )
        (setvar "OSMODE" OS)
        _NearPt
      )

      ;; ! Function : Locate the perpendicular drop point from a point pp to any line
      ;; !            segment defined by two other points pt1 and pt2
      ;; ! Argument : [pp] - Point to drop from
      ;; !            [p1] - First Point of line segment
      ;; !            [p2] - Second Point of line segment
      ;; ! Return   : The drop point on the line segment
      (defun PerpPt2Line (pp p1 p2 / x y x1 x2 x3 x4 y1 y2 y3 y4 m p4)
        (setq x  (car pp)
              y  (cadr pp)
              x1 (car p1)
              y1 (cadr p1)
              x2 (car p2)
              y2 (cadr p2)
        )
        (cond
          ((equal x1 x2 0.0000001)    ; Vertical line
           (setq x4 x1
                 y4 y
                 p4 (list x4 y4 0.0)
           )
          )
          ((equal y1 y2 0.0000001)    ; Horizontal line
           (setq x4 x
                 y4 y1
                 p4 (list x4 y4 0.0)
           )
          )
          (T
           (setq m  (/ (- y2 y1) (- x2 x1)) ; gradient p1- p2
                 x4 (/ (+ (/ x m) (+ y (- (* m x1) y1))) (+ (/ 1 m) m))
                 y4 (+ y1 (* m (- x4 x1)))
                 p4 (list x4 y4 0.0)
           )
          )
        )
        p4
      )

      ;; !            'pt1' - First point (origin) of the UCS
      ;; !            'pt2' - Second point (on X-axis) of the UCS
      ;; !            'pt3' - Third point (defining XY plane) of the UCS
      ;; !            If 'pt3' is nil, the third point will be assumed along the
      ;; !            X axis starting from the first point as the origin
      ;; ! Action   : Create a new 3 Point UCS if the distance between the points are non-zero
      (defun UCSby3 (pt1 pt2 pt3 / Done OS)
        (setq Done nil)
        (if (not pt3)
          (setq pt3 (polar pt1 (+ (angle pt1 pt2) hshpi90) 10.0))
        )
        (if (and (> (distance pt1 pt2) 0.001) (> (distance pt1 pt3) 0.001) (> (distance pt2 pt3) 0.001))
          (progn (command "._Ucs" "_3Point" pt1 pt2 pt3) (setq Done T))
        )
        Done
      )

      ;; ! Arguments: (SortByDist llst pt1 "Horiz")
      ;; !           'vlist' - List of points
      ;; !           'pt1'   - 'Reference point for distance measurement'
      ;; !           'Mode'  - can be either "Slope" or "Horiz"
      ;; !                     Specifies whether the distances are to be measured
      ;; !                     horizontally projected or in slope [actual].
      ;; ! Action   : Sorts a list of points in ascending order by their distance from a given point.
      (defun SortByDist (vlist pt1 Mode / tmpPt)
        (if (= Mode "Horiz")
          (setq tmpPt (list (car pt1) (cadr pt1) 0.0))
          (setq tmpPt pt1)
        )
        (setq vlist (mapcar '(lambda (x)
                   (if (= Mode "Horiz")
                     (list (distance tmpPt (list (car x) (cadr x) 0.0)) x)
                     (list (distance tmpPt x) x)
                   )
                     )
                    vlist
                )
          vlist (l_ssort vlist 0 '>)
          vlist (mapcar 'cadr vlist)
        )
      )

      ; Return selection set of objects after entname
      (defun SSGetFrom (ename / ss1)
        (if (not ename)
          (setq ss1 (ssget "_X"))
          (progn (setq ss1 (ssadd))
            (while (and ename (setq ename (entnext ename))) (ssadd ename ss1))
            (if (zerop (sslength ss1))
              nil
              (progn (command "._Select" ss1 "") (setq ss1 (ssget "_P")))
            )
          )
        )
      )

      ;; ! Function : Checks if a polyline has atleast one ARC segment
      ;; ! Argument : 'ename' - Polyline object name
      ;; ! Returns  : T if the polyline is arc'ed, else nil
      (defun PL_Arced? (ename / flag en entl VxEntl BlgLst)
        (setq BlgLst (LI_MakeUnique (PL_GetBulgeLst ename)))
        (if (and (= (length BlgLst) 1) (zerop (nth 0 BlgLst)))
          nil
          T
        )
      )

      ;; ! Function : Returns the coordinates of a box (window) around a point. The window may be a square or rectangle
      ;; ! Arguments: 'pt'   - Center point of the window
      ;; !            'dX'   - X Dimension of the box
      ;; !            'dY'   - Y Dimension of the box
      ;; !            'Rot'  - Rotation angle of the box (measured in default AutoCAD
      ;; !                     terms - 0 is east and ccw direction is positive)
      ;; ! Action   : Returns a list of 4 points starting from the top left and in clockwise direction.
      (defun BoxItIn (pt dX dY Rot / p1 p2 p3 p4 halfX halfY _Rot)
        (setq
          halfX (* 0.5 dX)
          halfY (* 0.5 dY)
          _Rot  (dtr Rot)
          p1    (polar pt (+ _Rot pi) halfX)
          p1    (polar p1 (+ _Rot hshpi90) halfY)
          p2    (polar p1 _Rot dX)
          p3    (polar p2 (- _Rot hshpi90) dY)
          p4    (polar p3 (+ _Rot pi) dX)
        )
        (list p1 p2 p3 p4)
      )

      ;; ! Function : Strip DXF codes specified in the list from the given list
      ;; ! Argument : 'entl'     - List to check (usually will be return from entget)
      ;; !            'StripLst' - List containing DXF codes to be stripped
      (defun StripDXFCode (entl StripLst / len _entl Code Lst)
        (setq len   (length entl)
             _entl '()
        )
        (foreach Lst entl
          (setq Code (car Lst))
          (if (not (member Code StripLst))
            (setq _entl (append _entl (list Lst)))
          )
        )
        _entl
      )

      ;; ! Function : Groups a linear list into nested sublists
      ;; ! Argument : [Lst]    - the linear list
      ;; !            [LstBeg] - the list begin marker
      ;; !            [LstEnd] - the list end marker
      ;; ! Returns  : a list in which anything between the 'LstBeg' and 'LstEnd' is converted into a sub-list
      (defun GroupListe (Lst LstBeg LstEnd / nested_list Lst1 itm)
        (setq Lst1 '())
        (while Lst
          (setq itm (car Lst))
          (cond
            ((= LstBeg itm)
             (setq nested_list (NestedLoc (cdr Lst) LstBeg LstEnd)
               Lst1        (append Lst1 (list (GroupListe nested_list LstBeg LstEnd)))
             )
             (repeat (1+ (length nested_list)) (setq Lst (cdr Lst)))
            )
            (T (setq Lst1 (append Lst1 (list itm))))
          )
          (setq Lst (cdr Lst))
        )
        Lst1
      )

      ;; ! Function : Finds out the nested part of the list
      ;; ! Argument : [Lst]    - the linear list
      ;; !            [LstBeg] - the list begin marker
      ;; !            [LstEnd] - the list end marker
      ;; ! Returns  : a list in which anything between the 'LstBeg' and 'LstEnd' is converted into a sub-list.
      (defun NestedLoc (Lst LstBeg LstEnd / itm cnt itm Lst1)
        (setq cnt  1
              Lst1 '()
        )
        (while (> cnt 0)
          (setq itm (car lst))
          (cond
            ((= itm LstBeg)
             (setq cnt  (1+ cnt)
               Lst1 (append Lst1 (list itm))
             )
            )
            ((= itm LstEnd)
             (setq cnt (1- cnt))
             (if (> cnt 0)
               (setq Lst1 (append Lst1 (list itm)))
             )
            )
            (T (setq Lst1 (append Lst1 (list itm))))
          )
          (setq Lst (cdr Lst))
        )
        Lst1
      )

      ;; ! SS_SeleBlk
      ;; ! Function : Returns a selection set based on a specified block name.
      ;; ! Argument : [name] - name of block, may take wild card.
      ;; ! Returns  : Selection set of entities matching the blkname.
      ;; !            Returns nil if none is found.
      (defun SS_SeleBlk (name) (ssget "X" (list (cons 0 "INSERT") (cons 2 name))))

      ;; ! SS_SeleLay
      ;; ! Function : Returns a selection set based on the specified layer name
      ;; ! Argument : [name] - name of layer, may take wild card
      ;; ! Returns  : Selection set of entities matching the layer name.
      ;; !            Returns nil if none is found.
      (defun SS_Selelay (name) (ssget "X" (list (cons 8 name))))

      ;; ! SS_SeleType
      ;; ! Function : Select all entities of specified entity type
      ;; ! Argument : [Type] - The entity type
      ;; ! Return   : Selection set meeting the selection criterion.
      ;; !            Returns nil if none is found.
      (defun SS_SeleType (cType) (ssget "X" (list (cons 0 ctype))))

      ;; ! SSgetXD
      ;; ! Function : Does an ssget and applies extended entity data check also.
      ;; ! Arguments: 'filtr'    - Selection Set filter criteria
      ;; !                         Do not give Application Name with 'filtr'
      ;; !                         as this is given separately in the last parameter
      ;; !            'XdChk'    - Xdata condition to check
      ;; !             XdChk is in the form '('pos' 'operator' 'value')
      ;; !                 where
      ;; !                    'pos'      is the position of Xdata (starts with 1)
      ;; !                    'operator' is the comparison operator, can be either
      ;; !                               = , < , <= , >= or /=
      ;; !                    'value'    is the value to be checked
      ;; !                               wildcards can be used for string fields.
      ;; !                               If the 'value' parameter is a string
      ;; !                               and if the 'operator' is =, a literal
      ;; !                               equality check is done else if it is a *
      ;; !                               a wildcard match (wcmatch) is performed.
      ;; !                    'RetFmt'   Return Format
      ;; !                               0 - Selection Set
      ;; !                               1 - List containing (ename XdataValue)
      ;; !
      ;; !            'AppName'  - Application Name to check
      ;; ! Return   : Selection set matching criteria
      ;;
      ;;  (SSgetXD (list (cons 0 "INSERT")) (quote (2 "=" "160")) 0 "AA_OPENING")
      ;;  (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "DOOREL")) 1 "AA_OPENING") doesn't like mixed upper/lower case
      ;;  (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "DOORPL")) 1 "AA_OPENING")

      (defun SSgetXD (filtr XdChk RetFmt AppName / ss ss1 ssl _type xd cnt ename entl itm _itm pos Operator value Lst)
        (setq filtr (StripDXFCode filtr (list -3))
              filtr (append filtr (list (list -3 (list AppName))))
              ss    (ssget "_X" filtr)
        )
        (if (= RetFmt 0)
          (setq ss1 (ssadd))
          (setq Lst '())
        )
        (if ss
          (progn
            (setq ssl      (sslength ss)
                  cnt      0
                  pos      (nth 0 XdChk)
                  Operator (nth 1 XdChk)
                  value    (nth 2 XdChk)
            )
            (repeat ssl
              (setq ename (ssname ss cnt)
                    xd    (ReadXD ename AppName)
                    cnt   (1+ cnt)
              )
              (if xd
                (progn
                  (setq itm (nth (1- pos) xd))
                  (if itm
                    (progn
                      (setq _type (type itm))
                      (cond
                        ((member _type (list 'REAL 'INT 'LIST))
                         (if ((eval (read Operator)) itm value)
                           (progn
                             (if (= RetFmt 0)
                               (ssadd ename ss1)
                               (setq Lst (cons (list ename itm) Lst))
                             )
                           )
                         )
                        )
                        ((= _type 'STR)
                         (setq _itm (strcase itm))
                         (cond
                           ((= Operator "=")
                            (if (= _itm value)
                              (progn (if (= RetFmt 0)
                               (ssadd ename ss1)
                               (setq Lst (cons (list ename _itm) Lst))
                                 )
                              )
                            )
                           )
                           ((= Operator "*")
                            (if (wcmatch _itm value)
                              (progn (if (= RetFmt 0)
                               (ssadd ename ss1)
                               (setq Lst (cons (list ename itm) Lst))
                                 )
                              )
                            )
                           )
                         )
                        )
                      )
                    )
                  )
                )
              )
            )
          )
        )
        (if (= RetFmt 0)
          (if (> (sslength ss1) 0)
            ss1
            nil
          )
          Lst
        )
      )

      ;;(SSgetW pBLC pURC (list (cons 0 "INSERT")) (quote (1 "=" "DOORPL")) 1 "AA_OPENING")
      (defun SSgetW (pBLC pURC filtr XdChk RetFmt AppName / ss ss1 ssl _type xd cnt ename entl itm _itm pos Operator value Lst)
        (setq filtr (StripDXFCode filtr (list -3))
              filtr (append filtr (list (list -3 (list AppName))))
              ss    (ssget "_W" pBLC pURC filtr)
        )
        (if (= RetFmt 0)
          (setq ss1 (ssadd))
          (setq Lst '())
        )
        (if ss
          (progn
            (setq ssl      (sslength ss)
                  cnt      0
                  pos      (nth 0 XdChk)
                  Operator (nth 1 XdChk)
                  value    (nth 2 XdChk)
            )
            (repeat ssl
              (setq ename (ssname ss cnt)
                    xd    (ReadXD ename AppName)
                    cnt   (1+ cnt)
              )
              (if xd
                (progn
                  (setq itm (nth (1- pos) xd))
                  (if itm
                    (progn
                      (setq _type (type itm))
                      (cond
                        ((member _type (list 'REAL 'INT 'LIST))
                         (if ((eval (read Operator)) itm value)
                           (progn
                             (if (= RetFmt 0)
                               (ssadd ename ss1)
                               (setq Lst (cons (list ename itm) Lst))
                             )
                           )
                         )
                        )
                        ((= _type 'STR)
                         (setq _itm (strcase itm))
                         (cond
                           ((= Operator "=")
                            (if (= _itm value)
                              (progn (if (= RetFmt 0)
                               (ssadd ename ss1)
                               (setq Lst (cons (list ename _itm) Lst))
                                 )
                              )
                            )
                           )
                           ((= Operator "*")
                            (if (wcmatch _itm value)
                              (progn (if (= RetFmt 0)
                               (ssadd ename ss1)
                               (setq Lst (cons (list ename itm) Lst))
                                 )
                              )
                            )
                           )
                         )
                        )
                      )
                    )
                  )
                )
              )
            )
          )
        )
        (if (= RetFmt 0)
          (if (> (sslength ss1) 0)
            ss1
            nil
          )
          Lst
        )
      )

      (defun AAXDataAdd (en apid dlst / el tmp1)
        (TraceIsOn ".XDataAdd")
        (if (null (tblsearch "APPID" apid))
          (regapp apid)
        )
        (setq el   (entget en)
          tmp1 (list -3 (cons apid dlst))
        )
        (if (< (xdsize tmp1) (xdroom en))
          (entmod (append el (list tmp1)))
        )
      )

      ;; ! XD_AddXdata
      ;; ! Function : Adds a pre-processed Xdata string into an object and does an (entmod)
      ;; ! Arguments: 'ename' - Object Name
      ;; !            'xd'    - Xdata to add
      (defun XD_AddXdata (ename xd / entl) (setq entl (append (entget ename) (list xd))) (entmod entl))

      ;; ! XD_GetRegApps
      ;; ! Function : Return a list of all registered Extended Entity Data Applications
      (defun XD_GetRegApps (/ RegAppList CurrentApp Rewind)
        (setq Rewind T)
        (while (setq CurrentApp (tblnext "APPID" Rewind))
          (setq Rewind     nil
            RegAppList (cons (cdr (assoc 2 CurrentApp)) RegAppList)
          )
        )
        RegAppList
      )

      ;; ! XD_GetVxXdata
      ;; ! Function : Get extended entity data from the vertices of a polyline
      ;; ! Argument : [ename]   - polyline entity name
      ;; !            [AppName] - Application Name
      ;; !            [fmt]     - Xdata format
      ;; !               0 - Power Tools Xdata format (read and written by ReadXD & XD_writeX)
      ;; !               1 - Generic Xdata format
      ;; ! Return   : List of Xdata of each vertex
      (defun XD_GetVxXdata (ename AppName fmt / en xd xdLst)
        (setq en    "VERTEX"
          xdLst '()
        )
        (while (= en "VERTEX")
          (setq ename (entnext ename)
                en    (LI_item 0 (entget ename))
                xd    (cond
                        ((= fmt 0) (ReadXD ename AppName))
                        ((= fmt 1) (LI_item -3 (entget ename (list AppName))))
                      )
                xdLst (cons xd xdLst)
          )
          (setq xdLst (cdr xdLst))
          (reverse xdLst)
        )
      )

      ;;(AAXDataGet (entlast) sAppNameAA)
      (defun AAXDataGet (en apid / el)
        (TraceIsOn ".XDataGet")
        (setq el (entget en (list apid)))
        (cdadr (assoc -3 el))
      )

      ;; ! Function : Reads extended entity data from an entity
      ;; (ReadXD (entlast) sAppNameAA)
      ;; ! Argument : [e]       - the entity name of the entity to be read.
      ;; !            [AppName] - The Application Name
      ;; ! Return   : xdata is returned in a nexted list, if exists; else NIL
      (defun ReadXD   (ename AppName / entl xd)
        (setq entl (entget ename (list AppName))
              xd   (cdr (car (cdr (assoc -3 entl))))
              xd   (mapcar 'cdr xd)
              xd   (GroupListe xd "{" "}")
        )
      )

      ;; ! XD_remxd
      ;; ! Function : removes extended entity data from selection
      ;; ! Arguments:
      ;; !            'ss'      - Selection set to process
      ;; !            'AppName' - Application Name to remove Xdata (cannot be * or
      ;; !                        wildcards)
      ;; !            'Verbose' - If T, a message is displayed while deleting
      ;; ! Action   : Checks for extended entity data and removes them
      ;; ! Returns  : The selection set and number of objects whose Xdata was removed,
      ;; !            as well as the number of objects ignored.
      (defun XD_remxd (ss AppName Verbose / xd ssl cnt ename tmp NumProc NumNotProc RemSS)
        (setq NumProc   0
             NumNotProc 0
             RemSS     (ssadd)
        )
        (if (= (type ss) 'ENAME)
          (progn (setq tmp (ssadd)) (ssadd ss tmp) (setq ss tmp))
        )
        (if ss
          (progn
            (setq cnt 0
                  ssl (sslength ss)
                  tmp (strcat " of " (itoa ssl))
            )
            (if Verbose
              (princ "\n")
            )
            (repeat ssl
              (setq ename (ssname ss cnt)
                xd    (assoc -3 (entget ename (list AppName)))
                cnt   (1+ cnt)
              )
              (if xd
                (progn (entmod (list (cons -1 ename) (list -3 (list AppName))))
                   (setq NumProc (1+ NumProc))
                   (ssadd ename RemSS)
                )
                (setq NumNotProc (1+ NumNotProc))
              )
              (if Verbose
                (princ (strcat "\rRemoving extended entity data belonging to application "
                       AppName
                       "..."
                       (itoa cnt)
                       tmp
                   )
                )
              )
            )
          )
        )
        (list
          (if (> (sslength RemSS) 0)
            RemSS
            nil
          )
          NumProc
          NumNotProc
        )
      )

      ;; ! XD_WriteX
      ;; ! Function : Writes extended entity into an entity
      ;; ! Argument : [e]       - entity name
      ;; !            [xdLst]   - list of data
      ;; !            [AppName] - Application Name (must be already registered)
      ;; ! Return   : complete entity list is returned if (XD_WriteX ...) is successful,
      ;; !            else nil is returned
      ;; ! Comment  : - Program can detect data types (STR, REAL, INT, LIST).
      ;; !            - Entitiy handles must be prefixed by a "#" sign
      ;; !               (XD_WriteX (entlast) (list "300XLCU" "#5DE") sAppNameAA)
      ;; !            - 1010 group stores points, which is a list of 3 reals
      ;; !            - If you want to store a nested list, it should have {}
      ;; !            - as the first element of the list
      (defun XD_WriteX (ename xdLst AppName / entl xd)
        (setq xd   (reverse (XD_WriteX$ xdLst))
              xd   (list -3 (cons AppName xd))
              entl (append (entget ename) (list xd))
        )
        (entmod entl)
        entl
      )

      (defun XD_WriteX$ (xdLst / xd xd1 itm)
        (setq xd '())
        (foreach itm xdLst
          (cond
            ((and (= (type itm) 'LIST) (= (car itm) "{}"))
             (setq xd1 (cons (cons 1002 "}") (XD_WriteX$ (cdr itm)))
               xd1 (append xd1 (list (cons 1002 "{")))
               xd  (append xd1 xd)
             )
            )
            ((and (= (type itm) 'LIST) (= (length itm) 3)) (setq xd (cons (cons 1010 itm) xd)))
            ((and (= (type itm) 'STR) (= (substr itm 1 1) "#"))
             (setq xd (cons (cons 1005 (substr itm 2)) xd))
            )
            ((= (type itm) 'STR)  (setq xd (cons (cons 1000 itm) xd)))
            ((= (type itm) 'INT)  (setq xd (cons (cons 1070 itm) xd)))
            ((= (type itm) 'REAL) (setq xd (cons (cons 1040 itm) xd)))
          )
        )
        xd
      )

      ;; ! LI_item is same as dxf
      ;; ! Function : Returns the first occurence of a DXF dotted pair from a list
      ;; ! Argument : 'n'     - The DXF code to check
      ;; !            'alist' - The List to check
      ;; ! Returns  : The value of the DXF dotted pair, if it exists else returns nil
      (defun LI_item (n alist) (cdr (assoc n alist)))

      ;; Code   - The DXF Code to check
      ;; alist  - The List to check
      ;; Returns  : A list of all DXF dotted pair values, if it exists else returns nil
      (defun ListDXF  (Code entl / Lst itm)
        (setq Lst '())
        (foreach itm entl
          (if (= (car itm) Code)
            (setq Lst (cons (cdr itm) Lst))
          )
        )
        (if Lst
          (reverse Lst)
          nil
        )
      )

      ;; ! XD_Add_List_Bracket
      ;; ! Function : Add a bracket to an extended entity list to make it compatible
      ;; !            with XD_WriteX
      ;; ! Argument : [xd]      - the extended entity data
      ;; ! Return   : xdata is returned with brackets added before each nexted list
      ;; !            This can be passed on directly to XD_WriteX
      (defun XD_Add_List_Bracket (xd / cnt len xdLst)
        (setq len   (length xd)
          cnt   0
          xdLst '()
        )
        (foreach itm xd
          (if (listp itm)
            (setq itm (cons "{}" itm)
              itm (XD_Add_List_Bracket itm)
            )
          )
          (setq xdLst (append xdLst (list itm)))
        )
        xdLst
      )

      (defun AAXDataAppids (/ tmp1 tmp2)
        (TraceIsOn ".XData-appids")
        (setq tmp1 (tblnext "APPID" 'T))
        (while tmp1
          (setq tmp2 (cons (cdr (assoc 2 tmp1)) tmp2)
                tmp1 (tblnext "APPID")
          )
        )
        tmp2
      )

      ;; ! Function : Find the attribute in a block given the position
      ;; ! Argument : 'ename' - entity name of the block
      ;; !            'pos'   - position of the attribute from start (from 0)
      ;; ! Returns  : The attribute entity name, if it exists otherwise nil is returned
      (defun FindAttrsNo (ename pos / entl en cnt More _tag aname More)
        (setq en    "ATTRIB"
              aname nil
              More  T
              cnt   0
        )
        (while More
          (setq ename (entnext ename)
                entl  (entget ename)
                en    (LI_item 0 entl)
                cnt   (1+ cnt)
          )
          (if (= en "ATTRIB")
            (progn
              (if (= pos cnt)
                (setq aname ename More  nil)
              )
            )
            (setq More nil)
          )
        )
        aname
      )

      ;; ! Function : Get the values of the attributes and their flags in a BLOCK INSERT entity
      ;; ! Argument : BlkName (entity ename)
      ;; ! Return   : A list containing the the attribute value, tag and flag.
      (defun BlGetAtts (ename / entl en Att Tag AttLst flag tmp)
        (setq AttLst '()
              entl   (entget ename)
              tmp    (LI_item 66 entl)
        )
        (if (and tmp (not (zerop (logand tmp 1))))
          (progn
             (setq ename (entnext ename)
                   entl  (entget ename)
                   en    "ATTRIB"
             )
             (while (/= en "SEQEND")
               (setq Att    (LI_item 1 entl)
                 flag   (LI_item 70 entl)
                 Tag    (strcase (LI_item 2 entl))
                 AttLst (append AttLst (list (list Att Tag flag)))
                 ename  (entnext ename)
                 entl   (entget ename)
                 en     (LI_item 0 entl)
               )
             )
          )
        )
        AttLst
      )

      ;; ! Function : Returns the value of the attribute of a given tag
      ;; ! Argument : 'ename' - entity name of the block
      ;; !            'tag'   - tag value to search for
      ;; ! Returns  : The attribute value , if it exists otherwise nil is returned
      (defun BlReadAttr (ename tag)
        (setq ename (FindAttrs ename tag))
        (if ename
          (LI_item 1 (entget ename))
          nil
        )
      )

      ;; ! Function : Returns the value of the attribute of a given tag
      ;; ! Argument : 'ename' - entity name of the block
      ;; !            'pos'   - Position of the attribute from start
      ;; ! Returns  : The attribute value , if it exists otherwise nil is returned
      (defun BlReadAttrNo (ename pos)
        (setq ename (FindAttrsNo ename pos))
        (if ename
          (LI_item 1 (entget ename))
          nil
        )
      )

      ;; ! Function : Write a value into a specified attribute in a block
      ;; ! Argument : 'ename' - entity name of the block
      ;; !            'tag'   - tag name of attribute to write
      ;; !            'Value' - Value to be set for the attribute
      ;; ! Returns  : The updated block entity name if succesful otherwise nil
      (defun BlWriteAttr (ename tag value / aname entl)
        (setq aname (FindAttrs ename tag))    ; Search for attribute
        (if aname
          (progn
            (setq entl (entget aname)
                  entl (subst (cons 1 value) (assoc 1 entl) entl)
            )
            (entmod entl)
            (entupd ename)
          )
        )
        ename
      )

      ;; ! Function : Write a value into a specified attribute at a given position in a block
      ;; ! Argument : 'ename' - entity name of the block
      ;; !            'pos'   - position of the attribute in the block
      ;; !            'Value' - Value to be updated
      ;; ! Returns  : The updated block entity name if succesful otherwise nil
      (defun BlWriteAttrNo (ename pos value / aname entl)
        (setq aname (FindAttrsNo ename pos)) ; Search for attribute
        (if aname
          (progn
            (setq entl (entget aname)
                  entl (subst (cons 1 value) (assoc 1 entl) entl)
            )
            (entmod entl)
            (entupd ename)
          )
        )
        ename
      )

      ;; ! Function : Find out if three points are collinear within a given tolerance
      ;; ! Argument : [p1]   - First point
      ;; !            [p2]   - Second point
      ;; !            [p3]   - Third point
      ;; !            [InBetFlag] - In Between flag, if T, then additionally check if
      ;; !                          'p1' is in between 'p2' and 'p3'.
      ;; !            [fuzz] - Fuzzy distance
      ;; ! Return   : T if the three points are colinear within the tolerance, else nil
      (defun Colineaire (p1 p2 p3 InBetFlag fuzz / RetVal OS)
        (setq OS     (getvar "OSMODE")
              RetVal nil
        )
        (setvar "OSMODE" 0)
        (command "._Ucs" "_3Point" p1 p2 (polar p1 (+ (angle p1 p2) hshpi90) 10.0))
        (setq p3 (trans p3 0 1))
        (command "._Ucs" "_Previous")
        (if (<= (abs (cadr p3)) fuzz)
          (progn
            (if InBetFlag
              (progn
                (setq p1 (list 0.0 0.0 0.0)
                      p2 (list (distance p1 p2) 0.0 0.0)
                )
                (if (IsInBetween (car p1) (car p2) (car p3))
                  (setq RetVal T)
                  (setq RetVal nil)
                )
              )
              (setq RetVal T)
            )
          )
          (setq RetVal nil)
        )
        (setvar "OSMODE" OS)
        RetVal
      )

      ;;-----------------------------------
      ;; ! Function : Checks if a give value is in between two values
      ;; ! Arguments: 'X'  - Value to be checked
      ;; !            'X1' - Lower bound of check
      ;; !            'X2' - Upper bound of check
      ;; ! Return   : T if the value 'X' lies between 'X1' and 'X2' else returns nil
      ;; ! Comments : The values to be checked can be either integer or real.
      (defun IsInBetween (X X1 X2)
        (TraceIsOn ".IsInBetween")
        (if (and (>= X (min X1 X2)) (<= X (max X1 X2)))
          T
          nil
        )
      )

      ;; ! Function : Does a ssget Window around a point (either "_C" "_W" or "_F" modes can be used).
      ;; ! Arguments: 'pt'   - Point around which ssWindow is to be applied.
      ;; !            'Wid'  - Width of Window Rectangle
      ;; !                     This argument can either be a list of type (X Y) or a real
      ;; !                     or integer single argument.
      ;; !            'Mode' - Mode of selection to use "_C" "_W" or "_F".
      ;; !            'filtr'- Filter criteria to use
      ;; ! Return   : Selection set matching criteria
      ;; ! Comments : No Zooming is done to ensure that the points are on-screen
      (defun SSWindow (pt Wid Mode filtr / ss p1 p2 p3 p4 WidX WidY OS)
        (setq OS (getvar "OSMODE"))
        (setvar "OSMODE" 0)
        (if (= (type Wid) 'LIST)
          (setq WidX (nth 0 Wid)
               WidY (nth 1 Wid)
          )
          (setq WidX Wid
                WidY Wid
          )
        )
        (setq WidX (* 0.5 WidX)
              WidY (* 0.5 WidY)
              p1   (list (- (car pt) WidX) (+ (cadr pt) WidY) 0.0)
              p2   (list (+ (car pt) WidX) (+ (cadr pt) WidY) 0.0)
              p3   (list (- (car pt) WidX) (- (cadr pt) WidY) 0.0)
              p4   (list (+ (car pt) WidX) (- (cadr pt) WidY) 0.0)
              ss   nil
        )
        (cond
          ((member Mode (list "_C" "_W"))
           (if filtr
             (setq ss (ssget Mode p3 p2 filtr))
             (setq ss (ssget Mode p3 p2))
           )
          )
          ((= Mode "_F")
           (if filtr
             (setq ss (ssget Mode (list p1 p2 p4 p3 p1) filtr))
             (setq ss (ssget Mode (list p1 p2 p4 p3 p1)))
           )
          )
        )
        (setvar "OSMODE" OS)
        ss
      )

      ;; ! Function : Rearranges the passed points 'p1' and 'p2' such that the returned
      ;; !            points are the lower left and upper right corners.
      ;; ! Argument : 'p1' and 'p2', two passed points
      ;; ! Returns  : A list of p1 and p2, where p1 is lower left point and p2 is upper right point.
      (defun CornersGet (p1 p2 / tmp xmin ymin xmax ymax)
        (setq xmin (car p1)
              ymin (cadr p1)
              xmax (car p2)
              ymax (cadr p2)
        )
        (if (> xmin xmax)
          (setq tmp  xmax
                xmax xmin
                xmin tmp
          )
        )
        (if (> ymin ymax)
          (setq tmp  ymax
                ymax ymin
                ymin tmp
          )
        )
        (list (list xmin ymin 0.0) (list xmax ymax 0.0))
      )

      ;; ! Function : Returns the Offset points on either side of a line
      ;; ! Arguments:
      ;; !            'pt1'      - Start point of line
      ;; !            'pt2'      - End point of line
      ;; !            'Dist'     - Distance on either side
      ;; !            'Num'      - Number of offset points on either side
      ;; !            'Collapse' - If T, first offset will collapse to half width
      ;; ! Return   : Returns the list of offset points d'un cote et autre du pt de depart.
      (defun GetOffsetPts (pt1 pt2 Dist Num Collapse / PtLst _Dist hDist cnt OS)
        (setq PtLst '()
             _Dist   0.0
              OS     (getvar "OSMODE")
        )
        (setvar "OSMODE" 0)
        (command "._Ucs" "_3Point" pt1 pt2 (polar pt1 (+ (angle pt1 pt2) hshpi90) 10.0))
        (setq cnt 1)
        (repeat Num
          (if (and Collapse (= cnt 1))
            (setq _Dist   (* 0.5 Dist)
                   hDist _Dist
            )
            (if Collapse
              (setq _Dist (+ hDist (* (1- cnt) Dist)))
              (setq _Dist (* cnt Dist))
            )
          )
          (setq PtLst (append PtLst (list (trans (list 0.0 _Dist 0.0) 1 0)))
                cnt   (1+ cnt)
          )
        )
        (setq cnt 1)
        (repeat Num
          (if (and Collapse (= cnt 1))
            (setq _Dist (* 0.5 Dist)
                  hDist _Dist
            )
            (if Collapse
              (setq _Dist (+ hDist (* (1- cnt) Dist)))
              (setq _Dist (* cnt Dist))
            )
          )
          (setq PtLst (append PtLst (list (trans (list 0.0 (- 0.0 _Dist) 0.0) 1 0)))
                cnt   (1+ cnt)
          )
        )
        (command "._Ucs" "_Previous")
        (setvar "OSMODE" OS)
        PtLst
      )

      ;  If ACADAPP is not loaded, then display a message indicating
      ;  such in an alert box, and return NIL to the caller.  This
      ;  function does not generate an error condition, so if that is
      ;  appropriate, then the caller must test the result and generate
      ;  the error condition itself.
      (defun ai_abort (app msg)
        (defun *error* (s)
          (if old_error
            (setq *error* old_error)
          )
          (princ)
        )
        (if msg
          (alert
             (strcat " Application error: " app " \n\n  " msg "  \n")
           )
        )
        (exit)
      )

      (defun ai_return (value) value)  ; Make act of returning value explicit

      ;;; Beep function conditional on user-preferred setting.
      (defun ai_beep ( / f)
        (write-line "\007" (setq f (open "CON" "w")))
        (setq f (close f))
      )

      ;;; (ai_alert <message> )
      ;;; Shell for (alert)
      (defun ai_alert (msg)
        (if ai_beepQuest (ai_beep))
        (alert (strcat " " msg "  "))
      )

      (defun ai_notrans ()
        (if (zerop (logand (getvar "cmdactive") (+ 1 2 8) ))
          T
          (progn
            (alert "This command may not be invoked transparently.")
            nil
          )
        )
      )

      ;;; Looks for a current selection set, and returns it if found,
      ;;; or throws user into interactive multiple object selection,
      ;;; and returns the resulting selection set if one was selected.
      ;;; Sets the value of ai_seltype to:
      ;;;    1 = resulting selection set was autoselected
      ;;;    2 = resulting selection set was prompted for.
      (defun ai_aselect ( / ss)
        (cond
          ((and (eq 1 (logand 1 (getvar "pickfirst")))
                (setq ss (ssget "_i"))
           )
           (setq ss (ai_ssget ss))  ;; only if ss exists.
           (setq ai_seltype 1)
           (ai_return ss)
          )
          ((setq ss (ssget))
           (if ss (setq ss (ai_ssget ss)))
           (setq ai_seltype 2)
           (ai_return ss)
          )
        )
      )

      ;;; Returns val with the any trailing zeros beyond the current setting of luprec removed.
      (defun ai_rtos (val / a b units old_dimzin)
        (setq units (getvar "lunits"))
        ;; No fiddling if units are Architectural or Fractional
        (if (or (= units 4) (= units 5))
          (rtos val)
          ;; Otherwise work off trailing zeros
          (progn
            (setq old_dimzin (getvar "dimzin"))
            ;; Turn off bit 8
            (setvar "dimzin" (logand old_dimzin (~ 8)))
            (setq a (rtos val))
            ;; Turn on bit 8
            (setvar "dimzin" (logior old_dimzin 8))
            (setq b (rtos val units 15))
            ;; Restore dimzin
            (setvar "dimzin" old_dimzin)
            ;; Fuzz factor used in equality check.
            (if (equal (distof a) (distof b) 0.000001)
              a
              b
            )
          )
        )
      )

      ;---Function inserts item before number specified in list. If number is invalid, the original list is returned.
      (defun Ins_Before (NUM ITEM LST / NLST)
        (defun SplitIt (NUM LST / HEAD TAIL CNT)
          (setq CNT 0
                NUM (1+ NUM)
          )
          (cond
            ((and (>= NUM 1) (<= NUM (length LST)))
             (while (>= (length LST) CNT)
               (cond
                 ( (< CNT NUM)
                   (setq HEAD (cons (nth CNT LST) HEAD))
                 )
                 ( (>= CNT NUM)
                   (setq TAIL (cons (nth CNT LST) TAIL))
                 )
               )
               (setq CNT (1+ CNT))
             )
             (list (reverse HEAD) (reverse (cdr TAIL)))
            )
            (T LST)
          )
        )
        (setq NUM (1- NUM))
        (cond
          ((and (>= NUM 0) (< NUM (1- (length LST))))
           (setq NLST (SplitIt NUM LST))
           (append (car NLST) (list ITEM) (cadr NLST))
          )
          (T LST)
        )
      )

      (defun Wallserror (msgHsh)
        (setq *error* OLDERROR)
        (command)                     ;Cancel any pending command and end any UNDO group
        (command)
        (command)
        (command "._UNDO" "_E")
        (dos_waitcursor)
        (if bLibrary
          (Bolt sWhatLib slacle)
        )
        (reset nil) (resset varlst)
        (if (member msgHsh '("console break" "Function cancelled" "Error: quit / exit abort"))
          (princ "")
          (princ (strcat "\nError " (itoa (getvar "ERRNO")) ": " msgHsh "\n"))     ;Otherwise, print an error indication
        )
        (princ)
      )

      (vl-load-com)
      ;;(vlr-types)                    ;first look for existing reactors types
      (vl-load-reactors)

      (defun VLxx_SnapOff ()
        (vla-put-ObjectSnapMode (VLxx_ActiveDocument) :vlax-false)
      )

      (defun VLxx_SnapOn ()
        (vla-put-ObjectSnapMode (VLxx_ActiveDocument) :vlax-true)
      )

      ;Purpose: Returns a list containing all tokens in a delimited string.
      ;Arguments: A delimited string and the delimiter character.
      ;Example: (vlxx-ParseString (getenv "ACAD") ";")
      ;Note: AutoLISP does not correctly interpret any character code outside the
      ;range of 1 to 255, so you cannot parse a null-delimited string.
      (defun vlxx-ParseString (str delim / lst pos token)
        (setq pos (vl-String-Search delim str))
        (while pos
          (setq lst (cons
                      (if (= (setq token (substr str 1 pos)) delim)
                        nil
                        token
                      )
                      lst
                    )
            str (substr str (+ (strlen delim) pos 1))
            pos (vl-String-Search delim str)
          )
        )
        (if (> (strlen str) 0)
          (setq lst (cons str lst))
        )
        (reverse lst)
      )

      (TraceIsOn ".9")

      ;Purpose: Returns the extents of an individual object
      ;Arguments; An object name
      ;Example: (ax:GetBoundingBox (car (entsel)))
      ;Notes: Do not use this routine with XLINES or RAYS
      (defun ax:GetBoundingBox (ent / ll ur)
        (vla-getboundingbox (vlax-ename->vla-object ent) 'll 'ur)
        (mapcar 'vlax-safearray->list (list ll ur))
      )

      ;Purpose: Returns the extents of a selection set
      ;Example: (ssextents (ssget))
      ;Notes: Be sure not to include objects like XLINES or RAYS in your selection set as their extents cannot be determined
      (defun ssextents (ss / c item lst)
        (setq c -1)
        (repeat (sslength ss)
          (foreach item (ax:GetBoundingBox (ssname ss (setq c (1+ c))))
            (setq lst (cons item lst))
          )
        )
        (VLxx_Extents lst)
      )

      ;Purpose: Returns a selection set with objects sorted by distance from the
      ;lower left corner of the original sets bounding box
      ;Example: (sssort (ssget))
      (defun sssort (ss key / extents lst c return index)
        (setq extents (ssextents ss)
               c      -1
        )
        (repeat (sslength ss)
          (setq c    (1+ c)
                lst  (cons
                       (list (ssname ss c)
                             (distance (car extents)
                                       (cdr (assoc key (entget (ssname ss c))))
                             )
                       )
                       lst
                     )
          )
        )
        (setq return (ssadd))
        (foreach index
          (vl-sort-i lst '(lambda (d1 d2) (< (cadr d1) (cadr d2))))
          (ssadd (car (nth index lst)) return)
        )
      )

      (defun VLxx_Layout-Exists-p (name)
        (cond
          ((member (strcase name) (mapcar 'strcase (AXLayoutsListAll)))
           T
          )
        )
      )

      (TraceIsOn ".10")

      ;;; Arguments [Typ]:  Obj = Object [VLA-OBJECT]
      ;;; Return [Typ]: > Dotted pair list '(("Tag1" . "Val1")...) [LIST]
      (defun AXGetAtts (Obj)
        (mapcar
         '(lambda (Att)
            (cons (vla-get-TagString Att)
              (vla-get-TextString Att)
            )
          )
          (vlax-invoke Obj "GetAttributes")
        )
      )

      ;Most methods for calculating viewport scale use the xdata attached to the viewport.
      ;This one takes advantage of ActiveX. Of course that means you can only use it in A2K:
      ;Returns the scale of a paperspace viewport.

      (defun getvpscale (vpname)  (vla-get-customscale (vlax-ename->vla-object vpname)) )

      ;LISP users can take advantage of group code 410 to select objects from specific layouts. Here's how:
      ;(ssget "X" '((0 . "LWPOLYLINE")(410 . "Layout1")))
      ;To select from more than one layout, include multiple names separated by a comma:
      ;(ssget "X" '((0 . "LWPOLYLINE")(410 . "Model,Layout1")))

      ;Notes: If you don't mind using VisualLISP, here's a real easy way to do the same thing:
      ;Returns the scale factor of an image
      (defun ImageScale (image) (vl-load-com) (vla-get-scalefactor (vlax-ename->vla-object image)) )

      (TraceIsOn ".13")

      ;-Define (one of the) Global Enumerated types VLISP forgot about
      (setq vlax-vbDecimal 14)

      ;-----
      ;-Define Global Reference to the ACAD application/session object
      (setq *acad-object* nil)      ; Initialize global variable

      (defun VLxx_AcadObject ()
        (cond
          (*acad-object*)       ; Return the cached object
          (T
           (setq *acad-object* (vlax-Get-Acad-Object))
          )
        )
      )

      ;-Define Global Reference to the ACTIVE DOCUMENT object
      (setq *active-document* nil)  ; Initialize global variable
      (defun VLxx_ActiveDocument ()
        (cond
          (*active-document*)   ; Return the cached object
          (T
           (setq *active-document* (vla-Get-ActiveDocument (VLxx_AcadObject)))
          )
        )
      )

      ;;;-Define Global Reference to the MODEL SPACE object
      (setq *model-space* nil)      ; Initialize global variable
      (defun VLxx_ModelSpace ()
        (cond
          (*model-space*)       ; Return the cached object
          (T
           (setq *model-space* (vla-Get-ModelSpace (VLxx_ActiveDocument)))
          )
        )
      )

      (TraceIsOn ".14")

      ;;;-Define Global Reference to the PAPER SPACE object
      (setq *paper-space* nil)        ; Initialize global variable
      (defun VLxx_PaperSpace ()
        (cond
          (*paper-space*)         ; Return the cached object
          (T
           (setq *paper-space* (vla-Get-PaperSpace (VLxx_ActiveDocument)))
          )
        )
      )

      ;;;-Return object to current "space"
      (defun VLxx_ActiveSpace ()
        (if (= 1 (vla-get-ActiveSpace (VLxx_ActiveDocument)))
          (VLxx_ModelSpace)
          (VLxx_PaperSpace)
        )
      )

      ;;;-Define Global Reference to the AcadPreferences object
      (setq *VLxx_acadprefs* nil)     ; Initialize
      (defun VLxx_AcadPrefs ()
        (cond
          (*VLxx_acadprefs*)
          (T
           (setq *VLxx_acadprefs*
              (vlax-Get-Property (VLxx_AcadObject) 'Preferences)
           )
          )
        )
      )

      ;---PLOT STAMPING with Reactors
      (defun ObjAdd (/ acadapp acaddoc mspace)
        (TraceIsOn ".ObjAdd")
        (if (= inMS 1)
          (vla-addText (VLxx_ModelSpace) "Plot Stamp" (vlax-3d-point cpnt) chgt)
          (vla-addText (VLxx_PaperSpace) "Plot Stamp" (vlax-3d-point cpnt) chgt)
        )
      )

      (defun ObjUpdate (txtobj / clog cpfx cdwg)
        (TraceIsOn ".ObjUpdate")
        (setvar "CLAYER" sPltLay)
        (setq   CLOG  (menucmd "M=$(getvar,loginname)")
                CPFX  (menucmd "M=$(getvar,dwgprefix)")
                CDWG  (menucmd "M=$(getvar,dwgname)")
                cdate (menucmd "M=$(edtime, $(getvar,date),DDD DD MON YY H:MMa/p)")
                cdate (strcat cpfx cdwg " plotted on " cdate " by " clog)
        )

        (cond
          ((or (not txtobj) (vlax-erased-p txtobj))
           (setq txtobj (ObjAdd))
          )
        )
        (vla-put-TextString txtobj cdate)
        txtobj
      )

      (defun CallOnObj  (calling-reactor command-list / txtobject)
        (TraceIsOn ".CallOnObj")
        (setq sCurLayr (getvar 'CLAYER))
        (if (=  bPlt T)
          (cond
            ((member "PLOT" command-list)
             (setq txtobject (vlax-ldata-get "PlotStamping" "TXTOBJ")
                   txtobject (ObjUpdate txtobject)
             )
             (vlax-ldata-put "PlotStamping" "TXTOBJ" txtobject)
            )
          )
        )
        (setvar "CLAYER" sCurLayr)                     ;Ca nique les UNDOs
      )

      (if *TxtObjReacteur* (vlr-remove *TxtObjReacteur*))
      (setq *TxtObjReacteur* (vlr-Editor-reactor nil '((:vlr-commandEnded . CallOnObj))))

      ;;;;---***
      ;;; EXAMPLE: (VLxx_GetPrefKey 'Files 'SupportPath)
      ;;;-Retrieve Preferences Settings: use strings for Tab/Key
      (defun VLxx_GetPrefKey (TabName KeyName)
        (vlax-get-property
          (vlax-get-property
            (VLxx_AcadPrefs)
            TabName
          )
          KeyName
        )
      )

      ;;;-Change Preferences Settings.
      ;;; EXAMPLE: (VLxx_SetPrefKey "OpenSave" "IncrementalSavePercent" 0)
      (defun VLxx_SetPrefKey (TabName KeyName NewVal)
        (vlax-put-property
          (vlax-get-property
            (VLxx_AcadPrefs)
            TabName
          )
          KeyName
          NewVal
        )
      )

      ;;; DESCRIPTION: Return the count [number] of a given collection object.
      ;;; EXAMPLE: (setq LayCount (VLxx_CollectionCount (VLxx_GetLayers)))
      (defun VLxx_CollectionCount (Collection)
        (vlax-get-property Collection 'Count)
      )

      ;;; DESCRIPTION: Return a list of collection member names.
      ;;; EXAMPLE: (VLxx_CollectionList (VLxx_GetLtypes))
      (defun VLxx_CollectionList (Collection / name out)
        (setq out '())
        (vlax-for each Collection
          (setq name (VLxx_Name each))
          (setq out (cons name out))
        )
        (reverse out)
      )

      ;;; DESCRIPTION: Returns the count of the documents collection
      ;;; EXAMPLE: (setq NumDocsOpen (VLxx_DocsCount))
      (defun VLxx_DocsCount ()  (VLxx_CollectionCount (VLxx_GetDocsCollection)))

      ;Purpose: Returns the number of open documents.                 ;;<<--**
      (defun AXDocCount ()
        (if (equal (getvar "SDI") 1)
          1
          (vla-get-count (vla-get-documents (vlax-get-acad-object)))
        )
      )

      ;;; DESCRIPTION: Returns a list of all opened document names
      ;;; EXAMPLE: (setq alldocs (VLxx_DocsList T))
      ;;; NOTES: Verbose returns full path+filename for each document in the list
      ;;; if set to T (true), otherwise only the filenames are returned.
      (defun VLxx_DocsList (verbose / docname out)
        (setq out '())
        (vlax-for each (VLxx_GetDocsCollection)
          (if verbose
            (setq docname
               (strcat
                 (vlax-get-property each 'Path)
                 "\\"
                 (VLxx_Name each)
               )
            )
            (setq docname (VLxx_Name each))
          )
          (setq out (cons docname out))
        )
        (reverse out)
      )

      ;;; DESCRIPTION: Various collection functions to return collection objects as OBJECTS.
      ;;; EXAMPLE: (setq collLayers (VLxx_GetLayers))
      (defun VLxx_GetLayers ()    (VLxx_DocCollection 'Layers))
      (defun VLxx_GetLtypes ()    (VLxx_DocCollection 'Linetypes))
      (defun VLxx_GetTextStyles ()    (VLxx_DocCollection 'TextStyles))
      (defun VLxx_GetDimStyles ()     (VLxx_DocCollection 'DimStyles))
      (defun VLxx_GetLayouts ()       (VLxx_DocCollection 'Layouts))
      (defun VLxx_GetDictionaries ()  (VLxx_DocCollection 'Dictionaries))
      (defun VLxx_GetBlocks ()    (VLxx_DocCollection 'Blocks))
      (defun VLxx_GetPlotConfigs ()   (VLxx_DocCollection 'PlotConfigurations))                   ;;<<--**
      (defun VLxx_GetViews ()     (VLxx_DocCollection 'Views))
      (defun VLxx_GetViewPorts () (VLxx_DocCollection 'Viewports))
      (defun VLxx_GetGroups ()    (VLxx_DocCollection 'Groups))
      (defun VLxx_GetRegApps ()   (VLxx_DocCollection 'RegisteredApplications))
      ;;(defun VLxx_GetLayersF () (VLxx_DocCollection 'vla-GetExtensionDictionary))

      ;;; DESCRIPTION: Return list of all collection member names
      ;;; EXAMPLE: (VLxx_ListCollectionMemberNames (VLxx_GetLayers))
      (defun VLxx_ListCollectionMemberNames (collection / itemname out)
        (setq out '())
        (vlax-for each collection
          (setq itemname (VLxx_Name each)
                out      (cons itemname out)
          )
        )
        (reverse out)
      )

      ;;; List Collection Member Names
      (defun VLxx_ListLtypes ()       (VLxx_ListCollectionMemberNames (VLxx_GetLtypes)))
      (defun VLxx_ListLayers ()       (VLxx_ListCollectionMemberNames (VLxx_GetLayers)))
      (defun VLxx_ListTextStyles ()       (VLxx_ListCollectionMemberNames (VLxx_GetTextStyles)))
      (defun VLxx_ListDimStyles ()        (VLxx_ListCollectionMemberNames (VLxx_GetDimStyles)))
      (defun VLxx_ListLayouts ()          (VLxx_ListCollectionMemberNames (VLxx_GetLayouts)))
      (defun VLxx_ListDictionaries ()     (VLxx_ListCollectionMemberNames (VLxx_GetDictionaries)))
      (defun VLxx_ListBlocks ()       (VLxx_ListCollectionMemberNames (VLxx_GetBlocks)))
      (defun VLxx_ListPlotConfigs ()      (VLxx_ListCollectionMemberNames (VLxx_GetPlotConfigs)))
      (defun VLxx_ListViews ()        (VLxx_ListCollectionMemberNames (VLxx_GetViews)))
      (defun VLxx_ListViewPorts ()        (VLxx_ListCollectionMemberNames (VLxx_GetViewports)))
      (defun VLxx_ListGroups ()       (VLxx_ListCollectionMemberNames (VLxx_GetGroups)))
      (defun AXListRegApps () (TraceIsOn ".AXListRegApps") (VLxx_ListCollectionMemberNames (VLxx_GetRegApps)))

      ;;; DESCRIPTION: Returns the count of the linetypes collection.
      ;;; EXAMPLE: (setq NumLtypes (VLxx_CountLtypes))
      (defun VLxx_CountLtypes ()  (VLxx_CollectionCount (VLxx_GetLtypes)))

      ;;; DESCRIPTION: Return a root collection of the AcadApplication object
      (defun VLxx_AcadCollection (Cname)  (vlax-Get-Property (VLxx_AcadObject) Cname))

      ;;; DESCRIPTION: Invokes the Delete method on a given object to erase it.
      ;;; EXAMPLE: (VLxx_DeleteObject arc-object1)
      (defun VLxx_DeleteObject (obj)
        (TraceIsOn "\n***DeleteObject")
        (cond
          ((and
             (not (vlax-erased-p obj))
             (vlax-read-enabled-p obj)
             (vlax-write-enabled-p obj)
           )
           (vlax-invoke-method obj 'Delete)
           (if (not (vlax-object-released-p obj))
             (vlax-release-object obj)
           )
          )
          (T (princ "\nNot able to erase object!"))
        )
      )

      ;;; DESCRIPTION: Returns ObjectName value for given object
      ;;; EXAMPLE: (= "AcDbArc" (VLxx_ObjectType myobject))
      (defun VLxx_ObjectType (obj)  (vlax-get-property obj 'ObjectName))

      ;;; DESCRIPTION: Dump all methods and properties for selected objects
      (defun C:DumpIt ( / ent)
        (while (setq ent (entsel))
          (vlax-Dump-Object
            (vlax-Ename->Vla-Object (car ent))
          )
        )
        (princ)
      )

      ;-Convert 3D point lists into 2D point lists (from Garden Path tutorial)
      (defun 3dPoint->2dPoint (3dpt)
        (list (float (car 3dpt)) (float (cadr 3dpt)))
      )

      ;-Convert a list of X,Y values from a single list into a list of paired lists
      ;-From (x y x y x y...) into ((x y) (x y) (x y) ...). This is necessary to convert
      ;the results of using (vla-Get-Coordinates) on a LwPolyline object into a list of vertex points.
      (defun xyList->ListOfPoints (coordList / ptlist)
        (while coordList
          (setq ptlist
                       (append ptlist
                              (list (list (car coordList) (cadr coordList)))
                       )
                coordList (cddr coordList)
          )
        )
        ptlist
      )

      (TraceIsOn ".15")

      ;-Boolean test if data type is VLA-OBJECT
      (defun Is_Vla_Object (obj) (equal (type obj) 'vla-object))

      ;-Boolean test if data type is STRing
      (defun Is_String (arg) (equal (type arg) 'str))

      ;-Boolean test if data type is REAL number (double, float, etc.)
      (defun Is_Real (arg) (equal (type arg) 'real))

      ;-Boolean test if data type is AutoCAD ENAME (object name)
      (defun Is_Ename (arg) (equal (type arg) 'ename))

      ;-Boolean test if data type is VARIANT
      (defun Is_Variant (arg) (equal (type arg) 'variant))

      ;;; MODULE: VLxx_SortPoints (points-list sortfield)
      ;;; DESCRIPTION: Sorts a list of point-lists on x, y or z coordinates
      ;;; ARGS: list of points (lists), sortfield(char "X", "Y" or "Z")
      ;;; EXAMPLE: (VLxx_SortPoints myPoints "Y") sorts on Y-coord values
      (defun VLxx_SortPoints (points-list xyz)
        (setq xyz (strcase xyz))
        (cond
          ((= xyz "Z")
           ;; 3-point lists required!
           (if (apply '=
                      (mapcar
                       '(lambda (lst) (length lst))
                        points-list
                      )
               )
             (vl-sort points-list
                      (function
                        (lambda (p1 p2) (< (caddr p1) (caddr p2)))
                      )
             )
             (princ "\nCannot sort on Z-coordinates with 2D points!")
           )
          )
          ((= xyz "X")
           (vl-sort points-list
                    (function
                      (lambda (p1 p2) (< (car p1) (car p2)))
                    )
           )
          )
          ((= xyz "Y")
           (vl-sort points-list
                    (function
                      (lambda (p1 p2) (< (cadr p1) (cadr p2)))
                    )
           )
          )
        )
      )

      ;-Convert VLA-OBJECT into ENAME data type.
      (defun VLxx_MakeEname (object)
        (if (Is_Vla_Object object)
          (vlax-Vla-Object->Ename object)
          object
        )
      )

      ;;; EXAMPLE: (setq myobj (VLxx_MakeObject (car (entsel))) )
      (defun VLxx_MakeObject (entname)
        (cond
          ((= (type entname) 'ENAME)
           (vlax-ename->vla-object entname)
          )
          ((= (type entname) 'VLA-OBJECT)
           entname
          )
        )
      )

      ;;;-Convert a LIST of INTeger values into a VARIANT SAFE-ARRAY.
      (defun Intlist->VarArray (aList)
        (vlax-SafeArray-Fill
          (vlax-Make-SafeArray
            vlax-vbInteger            ; (2) Integer
            (cons 0 (- (length aList) 1))
          )
          aList
        )
      )

      ;;;-Convert a LIST of VARIANT values into a VARIANT SAFE-ARRAY.
      (defun VarList->VarArray (aList)
        (vlax-SafeArray-fill
          (vlax-Make-SafeArray
            vlax-vbVariant            ;(12) Variant
            (cons 0 (- (length aList) 1))
          )
          aList
        )
      )

      ;;;-Apply a defined Linetype to a graphic object.
      (defun VLxx_ApplyLtype (ent strLtype / entlist)
        (setq obj (VLxx_MakeObject ent));; convert to vla-object first
        (cond
          ((member (strcase strLtype) '("CONTINUOUS" "BYBLOCK"))
           (DPR "\nApplying CONTINUOUS or BYBLOCK linetype...")
           (vla-Put-LineType obj strLtype); Watch this!  May cause Fatal Error!

           ;;; IF the above line causes a crash, use the following
           ;;; method instead and simply swap the commenting to enable it...
           ;
           ;(setq ent (VLxx_MakeEname ent) entlist (entget ent))
           ;(if (assoc 6 entlist)
           ;  (setq entlist (subst (cons 6 strLtype) (assoc 6 entlist) entlist))
           ;  (setq entlist (append entlist (list (cons 6 strLtype))))
           ;)
           ;(entmod entlist)
          )
          ((/= (strcase strLtype) "BYLAYER")
           (DPR "\nApplying named linetype...")
           (if (tblsearch "ltype" strLtype)
             (vla-Put-linetype obj strLtype); Watch this! May cause Fatal Error!

             ;;; IF the above line causes a crash, use the following
             ;;; method instead and simply swap the commenting to enable it...
             ;
             ;(progn
             ;  (setq ent (VLxx_MakeEname ent) entlist (entget ent))
             ;  (if (assoc 6 entlist)
             ;    (setq entlist (subst (cons 6 strLtype) (assoc 6 entlist) entlist))
             ;    (setq entlist (append entlist (list (cons 6 strLtype))))
             ;  )
             ;  (entmod entlist)
             ;)

             (princ (strcat "\nVLxx_ApplyLtype: Linetype " strLtype " not loaded..."))
           )
          )
          (T (princ "\nVLxx_ApplyLtype: Unknown parameter!") )
        )
      )

      ;;;-Apply Object Linetype Scale to an Object
      (defun VLxx_ApplyLtScale (ent dblLtScale / obj)
        (setq obj (VLxx_MakeObject ent))  ; convert into vla-object first
        (if (vlax-Write-Enabled-p obj)
          (vla-Put-LinetypeScale obj dblLtScale)
          (princ
            "\nVLxx_ApplyLtScale: Unable to modify object linetype scale."
          )
        )
        (vlax-Release-Object obj)
      )

      ;;;-Apply Linetype Generation to LwPolyline Object
      (defun VLxx_ApplyLtypeGen (object / obj)
        (setq object (VLxx_MakeObject object))
        ;; make sure not Ename first!
        (vla-Put-LinetypeGeneration object :vlax-True)
      )

      ;;;-Put Object Color=ByLayer, Linetype=ByLayer
      (defun VLxx_Put-ByLayer (obj)
        (if (vlax-write-enabled-p obj)
          (progn
            (vla-put-Color obj 255)
            ;;(vla-put-Linetype obj ...);; <-- I need to figure this out!!!
          )
        )
      )

      ;;; MODULE: VLxx_CopyProp (property source-obj target-obj)
      ;;; DESCRIPTION: Copy named property from one object to another.
      ;;; ARGS: property(string or quotedval), source(object), target(object)
      ;;; EXAMPLE: (VLxx_CopyProp "Layer" arc-object1 arc-object2)
      (defun VLxx_CopyProp (propName sourceObj targetObj)
        (cond
          ((and
             (not (vlax-erased-p sourceObj));; not erased?
             (not (vlax-erased-p targetObj));; not erased?
             (vlax-read-enabled-p sourceObj);; able to read from?
             (vlax-write-enabled-p targetObj);; open to change?
             (vlax-property-available-p sourceObj propName);; available?
             (vlax-property-available-p targetObj propName T);; modifiable?
           )
           (vla-put-Property targetObj
             (vla-get-Property sourceObj propName)
           )
           T
          )
        )
      )

      ;;; MODULE: VLxx_MapPropertyList (properties source-obj target-obj)
      ;;; DESCRIPTION: Copies a list of properties from one object to another
      ;;; ARGS: properties(list), source(object), target(object)
      ;;; EXAMPLE: (VLxx_MapPropertyList '("Layer" "Color") arc-object1 arc-object2)
      (defun VLxx_MapPropertyList (propList sourceObj targetObj)
        (foreach prop propList
          (VLxx_CopyProp prop sourceObj targetObj)
        )
      )

      ;;; DESCRIPTION: Begins an UNDO-MARK group
      ;;; EXAMPLE: (VLxx_UndoBegin)
      (defun VLxx_UndoBegin ()  (vlax-invoke-method (VLxx_ActiveDocument) 'StartUndoMark))

      ;;; DESCRIPTION: Closes an UNDO group
      ;;; EXAMPLE: (VLxx_UndoEnd)
      (defun VLxx_UndoEnd ()  (vlax-invoke-method (VLxx_ActiveDocument) 'EndUndoMark))

      (defun VLxx_ActiveLayout ()
        (vla-get-ActiveLayout (VLxx_ActiveDocument))
      )

      (defun VLxx_ActiveLayoutName ()
        (vla-get-Name (VLxx_ActiveLayout))
      )

      (defun VLxx_GetDocsCollection ()  (VLxx_AcadCollection "Documents"))

      (defun VLxx_ActivePlotConfig ()
        (vlax-get-Property (VLxx_ActiveLayout) 'ConfigName)
      )

      ;;; DESCRIPTION: Display methods and properties for each collection member
      ;;; EXAMPLE: (VLxx_DumpCollection (VLxx_GetLayers))
      (defun VLxx_DumpCollection (Collection)
        (VLxx_MapCollection Collection 'vlax-dump-object)
      )

      ;;; DESCRIPTION: Apply a function to all members of a given collection
      ;;; EXAMPLE: (VLxx_MapCollection all-arcs 'VLxx_DeleteObject)
      (defun VLxx_MapCollection (Collection qFunction)
          (vlax-map-collection  Collection  qFunction)
      )

      (defun VLxx_PlotConfigs (/ pc)
        (if (setq pc
                  (vlax-Invoke-Method
                    (vlax-get-Property (VLxx_ActiveDocument) 'PlotConfigurations)
                    'GetPlotDeviceNames
                  )
            )
          (vlax-SafeArray->List pc)
        )
      )

      (defun DeletePlotConfig (pcname /)
        (setq PlCfg
          (vl-catch-all-apply
            'vla-item
             (list (vla-get-PlotConfigurations (vla-get-activedocument (vlax-get-acad-object)))
                   pcname
             )
          )
        )
        (if (not (vl-catch-all-error-p plcfg))
          (vla-delete PlCfg)
        )
      )

      (setq pl (vla-get-PlotConfigurations (vla-get-activedocument (vlax-get-acad-object))))
      (vlax-for item pl (vlax-invoke-method item 'delete))

      ;;;-Get Document collection objects
      ;;; EXAMPLE: (setq all-ltypes (VLxx_DocCollection 'Linetypes))
      (defun VLxx_DocCollection (Cname)
        (vlax-Get-Property (VLxx_ActiveDocument) Cname)
      )

      ;;;-Get ObjectName property for given object
      (defun VLxx_Name (obj) (vlax-get-property obj 'Name))

      ;;;-Get LIST of collection member names
      (defun VLxx_ListCollectionMemberNames (collection / itemname out)
        (setq out '())
        (vlax-for each collection
          (setq itemname (VLxx_Name each)
                out      (cons itemname out)
          )
        )
        (reverse out)
      )

      ;;; EXAMPLE: Example (VLxx_AcadProp 'WindowState)
      (defun VLxx_AcadProp (PropName)  (vlax-get-property (VLxx_AcadObject) PropName))

      ;;;-Return LIST of Document Layer names
      (defun VLxx_ListLayers ( / itemname out)
        (VLxx_ListCollectionMemberNames (VLxx_GetLayers))
      )

      ;Create a new DimStyle
      ;Example: (VLxx_AddDimStyle "MyDimStyle" dvarlist T)
      (defun VLxx_AddDimStyle
        (name varlist current / all-dims new-dim var-set out)
        (cond
          ((not (tblsearch "dimstyle" name))
           (cond
             ((and
                (setq all-dims (vla-get-DimStyles (VLxx_ActiveDocument)))
                (setq new-dim (vla-Add all-dims name))
              )
              (foreach var-set varlist
                (setvar (car var-set) (cadr var-set))
              )
              (vlax-Invoke-Method new-dim 'CopyFrom (VLxx_ActiveDocument))
              (if current
                (vla-put-ActiveDimStyle (VLxx_ActiveDocument) new-dim)
              )
              (setq out new-dim)
             )
           )
          )
          (T
           (vla-put-ActiveDimStyle
             (VLxx_ActiveDocument)
             (setq out (AXGetDimStyles name))
           )
           (if current
             (vla-put-ActiveDimStyle (VLxx_ActiveDocument) out)
           )
          )
        )
        out
      )

      ;Create a New DimStyle from an Existing DimStyle
      ;EXAMPLE: (VLxx_CopyDimStyle "Standard" "MyDimStyle")
      (defun VLxx_CopyDimStyle
        (new-name copy-from-name / exst-dimstyle new-dimstyle)
        (cond
          ((and
             (not (tblsearch "dimstyle" new-name))
             (tblsearch "dimstyle" copy-from-name)
           )
           (setq exst-dimstyle (AXGetDimStyles copy-from-name))
           (setq new-dimstyle
             (vla-Add
               (vla-get-DimStyles (VLxx_ActiveDocument)) new-name
             )
           )
           (vlax-Invoke-Method new-dimstyle 'CopyFrom exst-dimstyle)
          )
        )
        new-dimstyle
      )

      (defun MxRelease (obj) (vlax-Release-Object obj))

      ;;; Returns a list containing the endpoints of the selected lwpoly segment
      ;;; The entity name of an lwpolyline and the point at which it was selected
      ;;; Example: (apply 'getseg (entsel))
      (defun VLxx_GetPolySegment (poly pt / pts i)
        (setq pts (VLxx_Massoc 10 (entget poly))
              i   (caddar (ssnamex (ssget pt)))
        )
        (list
          (nth (1- i) pts)
          (if (and
                (VLxx_IsClosed poly)
                (= i (length pts))
              )
            (car pts)
            (nth i pts)
          )
        )
      )

      ;;; EXAMPLE: (VLxx_AddLine (VLxx_ModelSpace) pt1 pt2 "DOORS" 4 "DASHED")
      ;;; NOTES: <intColor> and <strLtype> can each be 'nil'
      (defun VLxx_AddLine (objSpace p1 p2 lay col ltype / obj)
        (cond
          ((setq obj
               (vla-AddLine
                 ObjSpace
                     (vlax-3D-Point p1)
                     (vlax-3D-Point p2)
               )
           )
           (if lay (vla-put-Layer obj lay))
           (if col (vla-put-Color obj col))
           (if ltype (VLxx_ApplyLtype obj ltype))
           (vla-update obj)
           (vlax-release-object obj)
           (entlast)
          )
        )
      )

      ;;; EXAMPLE: (VLxx_AddArc pt1 0.5 0 90 "0" 3 "DASHED")
      ;;;    <StartAng> and <EndAng> are in DEGREE values, not Radians
      ;;;    <intColor> and <strLtype> can each be 'nil'
      (defun VLxx_AddArc (cpt rad ang-s ang-e lay col ltype / obj)
        (cond
          ((setq obj
             (vla-AddArc
               (VLxx_ActiveSpace)
               (vlax-3d-point cpt)
               rad
               (DTR ang-s)
               (DTR ang-e)
             )
           )
           (if lay (vla-put-Layer obj lay))
           (if col (vla-put-Color obj col))
           (if ltype (VLxx_ApplyLtype obj ltype))
           (vla-update obj)
           (vlax-release-object obj)
           (entlast)
          )
        )
      )

      ;;; EXAMPLE: (VLxx_AddCircle pt1 0.5 "0" 3 "DASHED")
      ;;; NOTES: <intColor> and <strLtype> can each be 'nil'
      (defun VLxx_AddCircle (cpt rad lay col ltype / obj)
        (cond
          ((setq obj
             (vla-AddCircle
               (VLxx_ActiveSpace)
               (vlax-3d-point cpt)
               rad
             )
           )
           (if lay (vla-put-Layer obj lay))
           (if col (vla-put-Color obj col))
           (if ltype (VLxx_ApplyLtype obj ltype))
           (vla-update obj)
           (vlax-release-object obj)
           (entlast)
          )
        )
      )

      ;;; DESCRIPTION: Create LwPolyline with given properties
      ;;; ARGS: space, points-list, layername, closed(T or nil), <color> is
      ;;; integer, <ltype> is string name, <width> is double/real number
      ;;; EXAMPLE: (VLxx_AddPline (VLxx_ModelSpace) ptlist "0" T 3 "DASHED" 0.125 nil)
      ;;; NOTES: <Bclosed> <intColor> <dblWidth> and <strLtype> can each be 'nil' which is ByLayer.
      (defun VLxx_AddPline (objSpace ptlist strLayer Bclosed intColor strLtype dblWidth bLTG / vrtcs plgen plist plpoints obj)
        (cond
          ((and ptlist (listp ptlist) (listp (car ptlist)))
           (setq plist     (apply 'append (mapcar '3dpoint->2dpoint ptlist))
                 plpoints  (List->VariantArray plist)
                 obj       (vla-addLightWeightPolyline objSpace plpoints)
           )
           (if Bclosed  (vla-Put-Closed obj :vlax-True));; make closed
           (if strLayer (vla-Put-Layer obj strLayer));; apply layer
           (if intColor (vla-Put-Color obj intColor));; apply color
           (if dblWidth (vla-Put-ConstantWidth obj dblWidth));; apply constant width
           (if strLtype (VLxx_ApplyLtype obj strLtype))
           (if bLTG       (VLxx_ApplyLtypeGen obj))
           (vla-Update obj);; force graphic update
           (vlax-release-object obj)
           (entlast)
          )
          (T (princ "\nVLxx_AddPline: Invalid parameter list...") )
        )
      )

      (defun VLxx_AddLineC (objSpace ptlist Bclosed strLayer intColor strLtype / pt1 ptz)
        (cond
          ((and ptlist (listp ptlist) (listp (car ptlist)))
           (setq pt1 (car ptlist)    ;; save first point
                 ptz (last ptlist)   ;; save last point
           )
           (while (and ptlist (>= (length ptlist) 2))
             (VLxx_AddLine
               objSpace
               (car ptlist)
               (cadr ptlist)
               strLayer
               intColor
               strLtype
             )
             (setq ptlist (cdr ptlist))
           )
           (if (= Bclosed T)
             (VLxx_AddLine objSpace pt1 ptz strLayer intColor strLtype)
           )
          )
          (T (princ "\nMakeLineC: Invalid parameter list..."))
        )
      )

      ;;; MODULE: VLxx_Roll->Ratio (Angle)
      ;;; DESCRIPTION: Converts ANGLE<degrees> into ratio for Ellipse roll angles
      ;;; ARGS: angle<degrees>
      ;;; EXAMPLE: (setq roll-ratio (VLxx_Roll->Ratio 45.0))
      (defun VLxx_Roll->Ratio (RollAngle)  (cos (DTR RollAngle)) )

      ;;; MODULE: VLxx_AddEllipse (objSpace ctr hmaj roll layer color ltype)
      ;;; DESCRIPTION: Create ELLIPSE object with given properties
      ;;; ARGS: objSpace centerpt hmajorpt rollangle layer color ltype
      ;;; EXAMPLE: (VLxx_AddEllipse (VLxx_ModelSpace) p1 p2 45 "PARTS" nil nil)
      ;;; NOTES: <space> is object, <centerpt> and <hmajorpt> are point lists
      ;;;  <roll> is degrees angle, <layer> is string name, <color> is integer,
      ;;;  <ltype> is string name.  <color> <ltype> may be 'nil' == ByLayer
      (defun VLxx_AddEllipse (objSpace ctr hmpt roll strLayer intColor strLtype / lst obj)
        (cond
          ((and ctr (listp ctr) hmpt (listp hmpt) roll)
           (setq hmpt (list
                        (- (car hmpt) (car ctr))
                        (- (cadr hmpt) (cadr ctr))
                      )
                 obj  (vla-addEllipse
                        objSpace
                        (vlax-3D-Point hmpt)
                        (VLxx_Roll->Ratio roll)
                      )
           )
           (cond
             ((vlax-Write-Enabled-p obj)
              (if strLayer
                (vla-Put-Layer obj strLayer)
              )
              (if intColor
                (vla-Put-Color obj intColor)
              )
              (if strLtype
                (VLxx_Apply-Ltype obj strLtype)
              )
              (vla-Update obj)
             )
             (T (princ "\nUnable to modify object properties..."))
           )
           (MxRelease obj)
           (entlast)
          )
          (T (princ "\nMakeEllipse: Invalid parameter list..."))
        )
      )

      (defun VLxx_AddEllipseArc1 (ctr hmpt roll StartAng
                                  EndAng strLayer intColor strLtype
                                  / obj rang
                                 )
        (cond
          ((and ctr (listp ctr) hmpt roll sa ea)
           (setq hmpt (list
                        (- (car hmpt) (car ctr))
                        (- (cadr hmpt) (cadr ctr))
                      )
                 obj  (vla-addEllipse
                        objSpace
                        (vlax-3D-Point ctr)
                        (vlax-3D-Point hmpt)
                        (VLxx_Roll->Ratio roll)
                      )
           )
           (cond
             ((vlax-Write-Enabled-p obj)
              (vla-Put-StartAngle obj (DTR StartAng))
              (vla-Put-EndAngle obj (DTR EndAng))
              (if strLayer
                (vla-Put-Layer obj strLayer)
              )
              (if intColor
                (vla-Put-Color obj intColor)
              )
              (if strLtype
                (VLxx_Apply-Ltype obj strLtype)
              )
              (vla-Update obj)
              (MxRelease obj)
              (entlast)
             )
             (T (princ "\nUnable to modify object properties..."))
           )
          )
          (T (princ "\nMakeArcEllipse: Invalid parameter list..."))
        )
      )

      (defun VLxx_AddEllipseArc2 (ctr      hmpt     hmin     StartAng
                                  EndAng   strLayer intColor strLtype
                                  /        obj      rang
                                 )
        (cond
          ((and ctr (listp ctr) hmpt (listp hmpt) hmin sa ea)
           (setq hmpt (list
                        (- (car hmpt) (car ctr))
                        (- (cadr hmpt) (cadr ctr))
                      )
                 obj  (vla-addEllipse
                        objSpace
                        (vlax-3D-Point ctr)
                        (vlax-3D-Point hmpt)
                        hmin
                      )
           )
           (cond
             ((vlax-Write-Enabled-p obj)
              (vla-Put-StartAngle obj (DTR StartAng))
              (vla-Put-EndAngle obj (DTR EndAng))
              (if strLayer
                (vla-Put-Layer obj strLayer)
              )
              (if intColor
                (vla-Put-Color obj intColor)
              )
              (if strLtype
                (VLxx_Apply-Ltype obj strLtype)
              )
              (vla-Update obj)
              (MxRelease obj)
              (entlast)
             )
             (T (princ "\nUnable to modify object properties..."))
           )
          )
          (T (princ "\nMakeArcEllipse2: Invalid Parameter list..."))
        )
      )

      ;;; DESCRIPTION: Creates POINT object with specified properties
      ;;; EXAMPLE: (VLxx_AddPoint p1 "DEFPOINTS")
      (defun VLxx_AddPoint (pt strLayer / obj)
        (cond
          ((setq obj (vla-addPoint
                       (VLxx_ActiveSpace)
                       (vlax-3D-Point pt)
                     )
           )
           (if strLayer (vla-Put-Layer obj strLayer))
           (vla-Update obj)
           (vlax-release-object obj)
           (entlast)
          )
        )
      )

      ;;; DESCRIPTION: Creates a circumscribed polygon
      ;;; ARGS: center, radius, sides, flag, width, layer, color, ltype, ltgen
      ;;; EXAMPLE: (VLxx_AddPolygon pt1 1.0 6 nil 0 "0" nil "DASHED" T)
      ;;; NOTES: flag is currently ignored, but will be used later
      (defun VLxx_AddPolygon (ctrpt dblRad intSides strType dblWid strLay intCol strLtype Bltgen / pa dg ptlist deg)
        (setq pa (polar ctrpt 0 dblRad)
              dg (/ 360.0 intSides);; get angles between faces
              deg dg
        )
        (repeat intSides
          (setq ptlist
            (if ptlist
              (append ptlist (list (polar ctrpt (DTR deg) dblRad)))
              (list (polar ctrpt (DTR deg) dblRad))
            )
          )
          (setq deg (+ dg deg))
        )
        (VLxx_AddPline objSpace ptlist strLay T intCol strLtype dblWid Bltgen)
      )

      (defun VLxx_AddRectangle (p1 p3 strLayer intColor strLtype dblWid bLTG / p2 p4 obj)
        (setq p2 (list (car p1) (cadr p3))
              p4 (list (car p3) (cadr p1))
        )
        (cond
          ((setq obj (VLxx_AddPline
                       objSpace (list p1 p2 p3 p4)
                       strLayer T intColor strLtype dblWid bLTG
                     )
           )
           obj
          )
        )
      )

      (defun VLxx_AddSolid (ptlist strLayer intColor / plist obj)
        (cond
          ((and ptlist (listp ptlist) (listp (car ptlist)))
           (if (= (length ptlist) 3)
               (setq plist (append ptlist (list (last ptlist))))
               (setq plist ptlist)
           )
           (DPR "\nMaking solid object...")
           (cond
             ((setq obj (vla-addSolid
                          (VLxx_ActiveSpace)
                          (vlax-3D-Point (car plist))
                          (vlax-3D-Point (cadr plist))
                          (vlax-3D-Point (caddr plist))
                          (vlax-3D-Point (cadddr plist))
                        )
              )
              (if strLayer (vla-Put-Layer obj strLayer))
              (if intColor (vla-Put-Color obj intColor))
              (vla-Update obj)
              (vlax-release-object obj)
              (entlast)
             )
             (T (princ "\nUnable to create object...") )
           )
          )
          (T (princ "\nVLxx_AddSolid: Invalid parameter list...") )
        )
      )

      ;;; DESCRIPTION: Creates TEXT object with specified properties
      ;;; ARGS: string, point, justification, style, hgt, wid, rot, lay, color
      ;;; EXAMPLE: (VLxx_AddText (VLxx_ModelSpace) "ABC" p1 "MC" "STANDARD" 0.25 1.0 0 "TEXT" nil)
      (defun VLxx_AddText (objSpace strTxt pt Just strStyle dblHgt dblWid dblRot strLay intCol / txtobj)
        (cond
          ((setq txtobj
             (vla-AddText
               ObjSpace
               strTxt
               (if (not (member (strcase Just) '("A" "F")))
                 (vlax-3d-Point pt)
                 (vlax-3d-Point (car pt))
               )
               dblHgt ;; ignored if Just="A" (aligned)
             )
           )
           (vla-put-StyleName txtobj strStyle)
           (vla-put-Layer txtobj strLay)
           (if intCol (vla-put-Color txtobj intCol))
           (setq Just (strcase Just));; force to upper case for comparisons...
           (cond
             ((= Just "L");; Left
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "C");; Center
              (vla-put-Alignment txtobj 1)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "R");; Right
              (vla-put-Alignment txtobj 2)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "A");; Aligned
              (vla-put-Alignment txtobj 3)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point (cadr pt)))
             )
             ((= Just "M");; Middle
              (vla-put-Alignment txtobj 4)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "F");; Fit
              (vla-put-Alignment txtobj 5)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
             )
             ((= Just "TL");; Top-Left
              (vla-put-Alignment txtobj 6)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "TC");; Top-Center
              (vla-put-Alignment txtobj 7)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "TR");; Top-Right
              (vla-put-Alignment txtobj 8)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "ML");; Middle-Left
              (vla-put-Alignment txtobj 9)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "MC");; Middle-Center
              (vla-put-Alignment txtobj 10)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "MR");; Middle-Right
              (vla-put-Alignment txtobj 11)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "BL");; Bottom-Left
              (vla-put-Alignment txtobj 12)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "BC");; Bottom-Center
              (vla-put-Alignment txtobj 13)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
             ((= Just "BR");; Bottom-Right
              (vla-put-Alignment txtobj 14)
              (vla-put-TextAlignmentPoint txtobj (vlax-3d-point pt))
              (vla-put-ScaleFactor txtobj dblWid)
              (vla-put-Rotation txtobj (DTR dblRot))
             )
           )
           (vla-Update txtobj)
           (vlax-Release-Object txtobj)
           (entlast)
          )
        )
      )

      ;;; EXAMPLE: (VLxx_Apply-Ltype circobj "DASHED")
      (defun VLxx_Apply-Ltype (obj strLtype / entlist)
        (cond
          ((VLxx_Ltype-Exists-p strLtype)
           (cond
             ((and
                (vlax-Read-Enabled-p obj);; object can be read from
                (vlax-Write-Enabled-p obj); object can be modified
              )
              (vla-Put-Linetype obj strLtype)
              T
             )
             (T (princ "\nVLxx_Apply-Ltype: Unable to modify object!"))
           )
          )
          (T (princ (strcat "\nVLxx_Apply-Ltype: Linetype [" strLtype "] not loaded.")))
        )
      )

      ;;; MODULE: VLxx_Apply-LtScale (object ltscale)
      ;;; ARGS: ename or object, scale (real)
      ;;; EXAMPLE: (VLxx_Apply-LtScale objLine 24.0)
      (defun VLxx_Apply-LtScale (obj dblLtScale)
        (cond
          ((and
             (vlax-Read-Enabled-p obj);; object can be read from
             (vlax-Write-Enabled-p obj); object can be modified
           )
           (vla-Put-LinetypeScale obj dblLtScale)
           T
          )
          (T (princ "\nVLxx_Apply-LtScale: Unable to modify object!"))
        )
      )

      ;Purpose: Returns a list containing cdrs for every occurence of the specified key in a list.
      ;Arguments: An integer and an object definition list
      ;Example: (massoc 10 (entget (car (entsel))))
      ;Notes: This is especially useful for retrieving all points associated with a lightweight polyline.
      (defun massoc   (key alist / x nlist)
        (foreach x alist
          (if (eq key (car x))
            (setq nlist (cons (cdr x) nlist))
          )
        )
        (reverse nlist)
      )

      (defun VLxx_Massoc (key alist)
        (apply 'append
          (mapcar '(lambda (x)
                     (if (eq (car x) key)
                       (list (cdr x))
                     )
                   )
              alist
          )
        )
      )

      ;Purpose: Returns T if the lwpoly is closed, nil otherwise
      ;Arguments: The object name of an lwpolyline
      ;Example: (isclosed (car (entsel)))
      (defun isclosed (poly)  (= 1 (logand 1 (cdr (assoc 70 (entget poly))))) )

      ;;; Returns a list containing the min and max points
      ;;; Arguments: A list with three or more points
      ;;; Example: (VLxx_Extents '((1 0 0) (2 2 0) (1 2 0)))
      (defun VLxx_Extents (plist /)
        (list
          (apply 'mapcar (cons 'min plist))
          (apply 'mapcar (cons 'max plist))
        )
      )

      ;Purpose: Removes the specified element from a list
      ;Arguments: A list and the element to remove from the list
      ;Example: (drop 10 '(6 8 10 12))
      (defun drop (lst item)
        (append (reverse (cdr (member item (reverse lst))))
            (cdr (member item lst))
        )
      )

      ;Purpose: Returns the "center" point of a rectangle (lwpolyline)
      ;Arguments: The object name of a rectangle
      ;Example: (rectcenter (car (entsel)))
      (defun RectCenter (rec)
        (VLxx_Mid (VLxx_Extents (VLxx_Massoc 10 (entget rec))))
      )

      ;Purpose: Creates a dictionary and returns its object name.
      ;Arguments: A string indicating the name of the dictionary to create.
      ;Example: (createdict "myDictionary")
      (defun createdict (dictName / dictionary xname newdict)
        (if (not (dictsearch (namedobjdict) dictName))
          (progn
            (setq dictionary '((0 . "DICTIONARY") (100 . "AcDbDictionary"))
                  xname      (entmakex dictionary)
                  newdict    (dictadd (namedobjdict) dictName xname)
            )
          )
        )
      )

      ;Purpose: Adds an xrecord to the specified dictionary and returns the ename of the xrecord.
      ;Example: (addrecord "myDictionary" "myRecord" (list (cons 300 "Sample data")))
      (defun addrecord (dictName key dataList / xname)
        (setq dataList (append (list '(0 . "XRECORD") '(100 . "AcDbXrecord"))
                               dataList
                       )
             xname    (entmakex dataList)
        )
        (dictadd (getdictname dictName) key xname)
      )

      ;Purpose: Modifies an existing xrecord.
      ;Arguments: A dictionary name, a key name specifying the record to change, and the new data list for that record.
      ;Example: (changerecord "myDictionary" "myRecord" (list (cons 300 "Sample data")))
      (defun changerecord (dictName key dataList)
        (if (getrecord dictName key)
          (progn
            (delrecord dictName key)
            (addrecord dictName key dataList)
          )
        )
      )

      ;Purpose: Deletes the specified dictionary.
      ;Arguments: The dictionary name to delete.
      ;Example: (deldict "myDictionary")
      (defun deldict (dictName) (dictremove (namedobjdict) dictName) )

      ;Purpose: Removes an xrecord from the specified dictionary.
      ;Arguments: The dictionary name and the key nname of the record to remove.
      ;Example: (delrecord "myDictionary" "myRecord")
      (defun delrecord (dictName key)  (dictremove (getdictname dictName) key) )

      ;Purpose: Returns the object definition list of the specified dictionary.
      ;Arguments: The dictionary name to retrieve.
      ;Example: (getdict "myDictionary")
      (defun getdict (dictName)  (dictsearch (namedobjdict) dictName) )

      ;Purpose: Returns the object name of the specified dictionary.
      ;Arguments: The dictionary name to retrieve.
      ;Example: (getdictname "myDictionary")
      (defun getdictname (dictName)
        (if (setq tmp (dictsearch (namedobjdict) dictName))
          (cdr (assoc -1 tmp))
        )
      )

      ;;;Purpose: Returns a list of keynames from the specified dictionary.
      ;;;Arguments: A dictionary name from which to retrieve keys.
      ;;;Example: (getkeys "myDictionary")
      (defun getkeys (dictName / tmp)
        (if (setq tmp (getdict dictname))
          (massoc 3 tmp)
        )
      )

      ;Purpose: Returns a list of keynames and their associated values from the specified dictionary.
      ;Arguments: A dictionary name from which to retrieve records.
      ;Example: (getkeyvalues "myDictionary")
      (defun getkeyvalues (dictName)
        (mapcar '(lambda (x) (cons x (cdar (getrecord dictName x))))
            (getkeys dictName)
        )
      )

      ;Purpose: Returns the original data list passed into addrecord.
      ;Arguments: A dictionary name and the key name for the record to retrieve.
      ;Example: (getrecord "myDictionary" "myRecord")
      (defun getrecord (dictName key / )
        (cdr (cddr (cddddr (dictsearch (getdictname dictName) key))))
        ;;(massoc 300 (dictsearch (getdictname dictName) key))
      )

      ;Purpose: Returns a list of all dictionaries defined in the current drawing
      (defun listdictionaries ()  (massoc 3 (entget (namedobjdict))) )

      ;Takes a layer name and returns True if the layer is locked, and nil if unlocked.
      (defun IsLayerLlocked (la / na e1)
        (setq na (tblobjname "layer" la)
              e1 (entget na)
        )
        (equal 4 (logand 4 (cdr (assoc 70 e1))))
      )

      ;---This will create a selectable unnamed group using the entities in the list LST, and give it the description DESC.
      ;   It takes a list of entity names or selection sets.
      (defun T:er_makgrp (LST DESC / EN)
        (command "-group" "_create" "*" DESC)
        (foreach EN LST (command EN))
        (command "")
      )

      ;---cross_prod returns the cross product (or normal) of two vectors.
      (defun cross_prod (p1 p2)
        (list
          (-
            (* (cadr p1) (caddr p2))
            (* (cadr p2) (caddr p1))
          )
          (* -1
             (-
                (* (car p1) (caddr p2))
                (* (car p2) (caddr p1))
             )
          )
          (-
            (* (car p1) (cadr p2))
            (* (car p2) (cadr p1))
          )
        )
      )

      ;---Sets the ucs such that the origin is at the upper left corner of the
      ;   mtext object and the x axis is aligned with the mtext rotation.
      (defun T:er_ucs_2_mtext (na / e1 p1 p2 p3 p4 )
        (TraceIsOn "T:er_ucs_2_mtext.")
        (setq e1 (entget na)
              p1 (dxf 10 e1)
              p2 (dxf 210 e1)
              p3 (dxf 11 e1)
              p1 (trans p1 na 1)
              p2 (trans p2 na 1 T)
              p3 (trans p3 na 1 T)
              p4 (cross_prod p2 p3)
              p2 (list (+ (car p1) (car p2))
                       (+ (cadr p1) (cadr p2))
                       (+ (caddr p1) (caddr p2))
                 )
              p3 (list (+ (car p1) (car p3))
                       (+ (cadr p1) (cadr p3))
                       (+ (caddr p1) (caddr p3))
                 )
              p4 (list (+ (car p1) (car p4))
                       (+ (cadr p1) (cadr p4))
                       (+ (caddr p1) (caddr p4))
                 )
        )
        (command "_.ucs" "_3p" p1 p3 p4)
      )

      (setq titre "About ArchAssist for AutoCAD.")

      ;---This function returns a list of four points describing the bounding box of the mtext (MTXT).
      (defun T:er_mtextbox (MTXT / na WDTH HGHT INS JUST ANG P1 P2 P3 P4)
        (if (and (listp MTXT) (= "MTEXT" (dxf 0 MTXT)))
          (progn
            (setq WDTH (dxf 42 MTXT)
                  HGHT (dxf 43 MTXT)
                    na (dxf -1 mtxt)
                   INS (trans (dxf 10 MTXT)
                              na ;changed by rk from 0
                              1
                       )
                  JUST (dxf 71 MTXT)
                  ANG  0.0 ;changed by rk from (cdr (assoc 50 MTXT)) because ucs_2_mtext
                           ;always aligns the rotation such that it is 0.
            )
            (cond
              ((= JUST 1)
                (setq P1 (polar INS (- ANG (* Pi 0.5)) HGHT) ; lower-left
                      P2 (polar P1 ANG WDTH)                 ; lower-right
                      P3 (polar INS ANG WDTH)                ; upper-right
                      p4 INS                                 ; upper-left
                )
              )
              ((= JUST 2)
                (setq P3 (polar INS ANG (/ WDTH 2))
                      P4 (polar INS (+ ANG Pi) (/ WDTH 2))
                      P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                      P2 (polar P1 ANG WDTH)
                )
              )
              ((= JUST 3)
                (setq P3 INS
                      P4 (polar INS (+ ANG Pi) WDTH)
                      P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                      P2 (polar P1 ANG WDTH)
                )
              )
              ((= JUST 4)
                (setq P4 (polar INS (+ ANG (* Pi 0.5)) (/ HGHT 2))
                      P3 (polar P4 ANG WDTH)
                      P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                      P2 (polar P1 ANG WDTH)
                )
              )
              ((= JUST 5)
                (setq P4 (polar INS (- ANG Pi) (/ WDTH 2))
                      P4 (polar P4 (+ ANG (* Pi 0.5)) (/ HGHT 2))
                      P3 (polar P4 ANG WDTH)
                      P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                      P2 (polar P1 ANG WDTH)
                )
              )
              ((= JUST 6)
                (setq P3 (polar INS (+ ANG (* Pi 0.5)) (/ HGHT 2))
                      P4 (polar P3 (+ ANG Pi) WDTH)
                      P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                      P2 (polar P1 ANG WDTH)
                )
              )
              ((= JUST 7)
                (setq P1 INS
                      P2 (polar P1 ANG WDTH)
                      P3 (polar P2 (+ ANG (* Pi 0.5)) HGHT)
                      P4 (polar P1 (+ ANG (* Pi 0.5)) HGHT)
                )
              )
              ((= JUST 8)
                (setq P1 (polar INS (+ ANG Pi) (/ WDTH 2))
                      P2 (polar P1 ANG WDTH)
                      P3 (polar P2 (+ ANG (* Pi 0.5)) HGHT)
                      P4 (polar P1 (+ ANG (* Pi 0.5)) HGHT)
                )
              )
              ((= JUST 9)
                (setq P2 INS
                      P1 (polar INS (+ ANG Pi) WDTH)
                      P3 (polar P2 (+ ANG (* Pi 0.5)) HGHT)
                      P4 (polar P1 (+ ANG (* Pi 0.5)) HGHT)
                )
              )
            )
          )
          (prompt "\nObject not Mtext!")
        )
        (list P1 P2 P3 P4)
      )

      ;---Type and msg are string arguments containing the control object type and the message prompt.
      ;   The mode boolean control was added to control the passed back argument, anything other
      ;   than nil returns the entity name only, nil returns the regular entsel list.
      ;   Legal objects: TEXT, POLYLINE, LINE, ARC, CIRCLE, POINT, SHAPE, SOLID and INSERT.
      (defun objsel (typ msg mode / ent)
        (while (null ent)        ;loop until a valid entity is selected
          (while (null ent)      ;loop after null input.
                                 ;select entity using prompt that was passed in
                                 ;store ename and point in variable ent.
            (setq ent (entsel (strcat "\n" msg ": ")))
          )                                               ;close second while
          (setq   typ1 (car  (strfind typ " or "))
                  typ2 (cadr (strfind typ " or "))
          )
          (if (= typ2 nil)(setq typ2 typ1) )
          (if (and
                (not (equal typ1 (dxf 0 (entget (car ent))) ))   ;check if selected object matches desired object type
                (not (equal typ2 (dxf 0 (entget (car ent))) ))   ;check if selected object matches desired object type
              )
            (progn    ;if not a match, nor a pline tell me to try again
              (WsFlashMess2
                (strcat "\nCet objet n'est pas valide! Slectionnez [" typ "]...")
                (strcat "\nNot a valid object! Select [" typ "]...")
                (strcat "\nEl objeto es no valido! Selleccione [" typ "]...")
                (strcat "\nNot a valid object! Select [" typ "]...")
              )
              (setq ent nil)  ;set ent to nil to force reselection
            )
          )
        )
        (if mode
          (car ent)  ;isolate ename and pass it back to original routine
          ent        ;pass back the entsel list
        )
      )

      ;---
      (defun pl_point_list ( na alt / dxf na2 a b c z p1 p2 p3 e1 lst lst2 code n flag )
        (if (equal (type na)
                   (type (list 1))
            )
            (setq code (cadr na)
                    na (car na)
            )
            (setq code 1);setq else
        )
        (setq e1 (entget na))
        (if (equal 1 (logand 1 (dxf 70 e1)))
            (setq flag 1)
            (setq flag nil)
        )
        (setq na2 na)
        (if (equal (dxf 0 e1) "POLYLINE")
          (progn
            (setq na (entnext na)
                  e1 (entget na)
                  p1 (dxf 10 e1)
                   b (dxf 42 e1)
            )
            (if (not (equal 16 (logand 16 (dxf 70 e1))))
                (setq lst (list p1))
                (setq lst nil)
            )
            (setq na (entnext na)
                  e1 (entget na)
            )
            (while (/= "SEQEND" (dxf 0 e1 ))
              (setq p2 (dxf 10 e1))
              (if (not (equal 16 (logand 16 (dxf 70 e1))))
                (progn
                  (if (and b
                           (not (equal b 0))
                           p1
                           (not (equal p1 p2))
                      )
                      (setq  p3 (pl_arc_info p1 p2 b)
                            lst (append lst
                                        (get_arc_points (car p3) p1 p2 (caddr p3) alt)
                                )
                      )
                      (setq lst (append lst (list p2)));setq else
                  )
                )
              )
              (setq  b (dxf 42 e1)
                    na (entnext na)
                    e1 (entget na)
                    p1 p2
              )
            )

            (if flag
              (progn
                (setq p2 (car lst))
                (if (and b
                         (not (equal b 0))
                         p1
                         (not (equal p1 p2))
                    )
                    (setq  p3 (pl_arc_info p1 p2 b)
                          lst (append lst
                                      (get_arc_points (car p3) p1 p2 (caddr p3) alt)
                              )
                    )
                    (setq lst (append lst (list p2)))
                )
                (setq flag nil)
              )
            )
          )
          (progn
            (setq   z (dxf 38 e1)
                  lst (member (assoc 10 e1) e1)
            )
            (if (not z) (setq z 0.0))
            (setq n 0)
            (repeat (length lst)
              (setq a (nth n lst))
              (if (equal 10 (car a))
                (setq    a (cons 10 (append (cdr a) (list z)))
                      lst2 (append lst2 (list a))
                )
                (progn
                  (if (equal 42 (car a))
                    (setq lst2 (append lst2 (list a)));setq then
                  )
                )
              )
              (setq n (+ n 1))
            )

            (setq  b (car lst2)
                 lst (list (cdr b))
            )

            (if flag
              (setq lst2 (append lst2 (list (car lst2)))
                    flag nil
              )
            )

            (setq n 1)
            (repeat (- (length lst2) 1)
              (setq a (nth n lst2))
              (if (and (equal 10 (car a))
                       (equal 42 (car b))
                       (not (equal 0.0 (cdr b)))
                       c
                       (not (equal (cdr a) (cdr c)))
                  )
                (progn
                  (setq  p3 (pl_arc_info (cdr c) (cdr a) (cdr b))
                        lst (append lst
                                    (get_arc_points (car p3) (cdr c) (cdr a) (caddr p3) alt)
                            )
                  )
                )
                (progn
                  (if (equal 10 (car a))
                    (setq lst (append lst (list (cdr a))))
                  )
                )
              )
              (setq c b
                    b a
              )
              (setq n (+ n 1))
            )
            (setq lst2 nil)
          )
        )
        (lsttrans lst na2 code)
      )

      ;---returns a list of points that lie along an arc described by the following arguments:
      ;   p1  - start point
      ;   p2  - end point
      ;   p3  - center point
      ;   ang - delta angle of the arc
      ;   alt - altitude (max error tolerance)
      (defun get_arc_points (p1 p2 p3 ang alt / sa ea lst b c d)
        (setq  b (distance p1 p2)       ;the radius
              sa (angle p1 p2)          ;the start angle
              ea (+ sa ang)             ;the end angle
        )
        (setq d (- ea sa))              ;setq full delta angle of the arc in radians
        (if (not alt)
            (setq c (/ (* 2.0 pi) 9.0)) ;then use default resolution
            (setq c (delta_ang b alt))  ;setq else use altitute specified.
        )
        (setq  c (* c (/ d (abs d))))   ;setq the delta angle increment of the loop

        (if (< (abs (/ d c)) 4.0)
          (setq c (/ d 4.0))
        )

        (repeat (+ (fix (+ (abs (/ d c))
                           0.000001
                        )
                   )
                   1
                )
        (if (not (equal (polar p1 sa b) (last lst)))
            (setq lst (append lst
                              (list (polar p1 sa b))
                      )
            )
        )
        (setq sa (+ sa c));setq
        )
        (if (not (equal p3 (last lst)))
          (setq lst (append lst (list p3)));setq
        )
        lst
      )

      ;---returns the delta angle of an arc with the specified altitude and radius.
      (defun delta_ang (r a / c ang)
        (setq c (* 2.0
                   (sqrt
                     (abs (- (* 2.0 r a)
                             (expt a 2.0)
                          )
                     )
                   )
                )
            ang (* 2.0 (atan (/ c 2.0)
                             (- r a)
                       )
                )
        )
        ang
      )

      ;---takes an entity name and returns a list of points on the entity.
      ;   args
      ;   na  - an entity name
      ;   alt - error tolerance for traversing arc segments
      (defun ep_list (na alt / e1 lst a b n c)
        (setq e1 (entget na))
        (cond
         ((or (equal (dxf 0 e1) "POLYLINE")
              (equal (dxf 0 e1) "LWPOLYLINE")
          )
          (setq lst (pl_point_list na alt))
         )
         ((equal (dxf 0 e1) "LINE")
          (setq lst (list (trans (dxf 10 e1) na 1)
                          (trans (dxf 11 e1) na 1)
                    )
          )
         )
         ((or (equal (dxf 0 e1) "ARC")
              (equal (dxf 0 e1) "CIRCLE")
          );or
          (progn
           (setq a (dxf 10 e1)             ;the center point
                 b (dxf 40 e1)             ;the radius
                 n (dxf 50 e1)             ;the start angle
                 c (dxf 51 e1)             ;the end angle
           )
           (if (not n)
               (setq n 0
                     c (* 2.0 pi)
               )
               (if (> n c)
                   (setq c (+ c (* 2.0 pi)));setq then
               )
           )
           (setq lst (append lst
                             (get_arc_points a
                                             (polar a n b)
                                             (polar a c b)
                                             (- c n)
                                             alt
                             )
                     )
           )
           (setq lst (lsttrans lst na 1))
          )
         )
         ((or (equal "MTEXT"  (dxf 0 e1))
              (equal "TEXT"   (dxf 0 e1))
              (equal "ATTDEF" (dxf 0 e1))
          )
          (setq lst (T:er_textbox e1 0.2)
                lst (append lst (list (car lst)))
          )
         )
         ((equal "ELLIPSE" (dxf 0 e1))
          (setq lst (get_ellipse_points na alt))
         )
         ((equal "IMAGE" (dxf 0 e1))
          (setq lst (image_clip_list na))
         )
        )
        lst
      )

      ;---MULTI-LINE ALERT BOX.
      (defun alertbox (title text / mklist path maxlen file fp id action)
        ;; Function to build the correct file name from a given path:
        (defun path (str)
          (if (wcmatch str "*:,*\\,*/")
             (strcat str "ALERTBOX")
             (strcat str "\\ALERTBOX")
          )
        )
        ;; Function to convert a string with CR ("\n") into a list:
        (defun mklist (str / i c text)
          (setq i 1)
          (while (/= c "")
             (setq c (substr str i 1))
             (if (member c '("\n" ""))
                (setq text (cons (substr str 1 (1- i)) text)
                      str (substr str (1+ i))
                      i 1
                )
                (setq i (1+ i))
             )
          )
          (reverse text)
        )
        ;; Try to find a temporary directory (path) to
        ;; to which the ALERTBOX.DCL file can be written,
        ;; and create the full file name:
        (cond
          ((setq file (getenv "TEMP"))
             (setq file (path file))
          )
          ((setq file (getenv "ACADPAGEDIR"))
             (setq file (path file))
          )
          ((setq file (getvar "TEMPPREFIX"))
             (setq file (path file))
          )
          ((setq file (getvar "DWGPREFIX"))
             (setq file (path file))
          )
          (1 (setq file "ALERTBOX"))
        )
        ;; Check the text input for its symbol type,
        ;; and convert it to a list if not.
        (setq text
          (cond
             ((= (type text) 'STR)
                (if (wcmatch text "*\n*")(mklist text)(list text))
             )
             ((not (atom text)) text)
             (1 '("Text input must be either a string" "or a list of strings."))
           )
        )
        ;; Check the title symbol.  If not a string, then make it one:
        (if (/= (type title) 'STR)(setq title "Alert"))
        ;; Determine the maximum string length:
        (setq maxlen 0)
        (foreach line text
          (if (= (type line) 'STR)
            (setq maxlen (max maxlen (strlen line)))
          )
        )
        ;; If able to create the ALERTBOX.DCL file, then write it,
        ;; and load it:
        (if (setq fp (open (strcat file ".DCL") "w"))
          (progn
            (write-line "alert : dialog {" fp)
            (write-line (strcat "  label = \"" title "\";") fp)
            (foreach line text
              (if (= (type line) 'STR)
                (write-line (strcat "  : text_part { label = \"" line "\"; width = " (itoa maxlen) "; }") fp)
              )
            )
            (write-line "  spacer;" fp)
            (write-line "  ok_cancel;" fp)
            (write-line "}" fp)
            (setq fp (close fp))
            (setq id (load_dialog file))
            (new_dialog "alert" id)
            (setq action 2)
            (while (> action 1)
               (action_tile "accept" "(done_dialog 1)")
               (action_tile "cancel" "(done_dialog 0)")
               (setq action (start_dialog))
            )
            (unload_dialog id)
            ;;action ;;### what is this ???
          )
          (prompt "\nError writing dialogue file.")
        )
      )

      (TraceIsOn ".16")

      ;---Used to round off a value up, down or to the nearest increment - where
      ;   the increment may be any real number.
      ;   value - The value to round (as a real or integer)
      ;   rndopt - 0 = No rounding.
      ;            1 = Standard rounding.
      ;            2 = Round up.
      ;            3 = Round down.
      ;   rndinc - The increment to round to.
      ;   The rounded number as a real.
      ;   (round 4.3 1  1.0) -->>  4.0
      ;   (round 4.5 1  1.0) -->>  5.0
      ;   (round 86  1 10.0) -->> 90.0
      ;   (round 84  1 10.0) -->> 80.0
      ;   (round 1.3 2  1.0) -->>  2.0

      (defun round (value rndopt rndinc / tmp)
        (cond
          ((= rndopt 1)                                         ;Standard rounding
            (if (< (setq tmp (rem value rndinc)) (* rndinc 0.5))
              (- value tmp)
              (+ (- value tmp) rndinc)
            )
          )
          ((= rndopt 2)                                         ;Round up
            (if (= (setq tmp (rem value rndinc)) 0.0)
              value
              (+ (- value tmp) rndinc)
            )
          )
          ((= rndopt 3)                                         ;Round down
            (if (= (setq tmp (rem value rndinc)) 0.0)
              value
              (- value tmp)
            )
          )
          (Hsht value)
        )
      )

      ;---Round to decimal places.
      (defun round2dec (val decpl / xs1 xs2 xs3)
        (setq xs1 (fix val)
              xs2 (* decpl (- val xs1))
              xs3 (/ (round xs2 1 1.0) decpl)
              val (+ xs1 xs3)
        )
      )

      ;---
      (defun CrossIt (pt col Hshsize / )
        (setq offset (* Hshsize (getvar "VIEWSIZE")))  ;0.005
        (setq x-min (- (car pt)  offset)
              x-max (+ (car pt)  offset)
              y-min (- (cadr pt) offset)
              y-max (+ (cadr pt) offset)
              ll    (list x-min y-min)
              ul    (list x-min y-max)
              ur    (list x-max y-max)
              lr    (list x-max y-min)
        )
        (grdraw ll ur col)                          ;draw first line of the X
        (grdraw ul lr col)                          ;draw second line of the X
        (grdraw lr pt col)                          ;draw first line of the X
        (grdraw ur pt col)                          ;draw second line of the X
      )

      (defun DotIt (cpt / blipmo ang inc pt)
        (setq cpt (trans cpt 1 2)                   ;trans center from UCS to DCS
              ang 0                                 ;initialize
              inc (* 0.08 dgrid)
              pt  (polar cpt 0 (* inc 0.5))
              ang (dtr 0)
              lstpts (append lstpts (list cpt))
        )
        (setq inc (/ inc 14)
              pt  (list (- (car cpt) (* 4.5 inc)) (- (cadr cpt) (* 3 inc)))  )
      )

      ;---CREATES RANDOM NUMBERS BETWEEN A LOW AND HIGH VALUE.
      (defun random (lvalue hvalue / value seed)
        (setq seed (/ (getvar "DATE") (getvar "TDINDWG") 100)  tdin (getvar "TDINDWG")
              seed (* (dos_random) seed (getvar "TDINDWG"))  cnt 16
        )
        (setq val1 (* (atoi (substr (rtos seed 2 18) cnt 2))
                      (atoi (substr (rtos tdin 2 18) cnt 2)) 0.01)
              value (+ val1 lvalue)
        )
        (while (> value hvalue)
           (setq value (/ value 1.957345)
                 value (+ value (* 0.5 lvalue)) )
        )
        (while (< value lvalue)
           (setq value (* value 1.957345)
                 value (- value (* 0.5 hvalue)) )
        )
        (setq newval value)
      )

      ;---Given chord and altitude, returns radius.
      (defun RADIUS (CHORD ALTI)(/ (+ (expt (/ CHORD 2.0) 2.0) (* ALTI ALTI)) (* 2.0 ALTI)) )

      ;---Given included angle in radians and radius, returns altitude.
      (defun ALTITD (RADIUS angle)(- RADIUS (* RADIUS (cos (/ angle 2.0)))) )

      ;---Given radius and included angle in radians, returns the arc length.
      (defun ARKLEN (RADIUS ANG)(while (> ANG HshPI360) (setq ANG (- ANG HshPI360)))(* RADIUS ANG) )

      ;---Given chord and radius, returns included angle in radians.
      (defun INCANG (CHORD RADIUS)
        (* 2.0
          (atan (/ CHORD 2.0 (sqrt (- (expt RADIUS 2) (expt (/ CHORD 2.0) 2)))))
        )
      )

      ;---Returns first n element in list. Counter (n) starts at 1.
      ;   (setq str '("abc" "def" "ghi" "jkl" "mno"))
      ;   (firstn 3 str) => ("abc" "def" "ghi")
      (defun firstn (n l)
        (cond
          ((zerop n) nil)
          (T
           (cons
             (car l)
             (firstn (1- n)(cdr l))
           )
          )
        )
      )

      ;;; Routine to replace NUMber element in a list with a new ITEM. Counting
      ;;; base is 0. If number is invalid for length of list, the original
      ;;; list is returned. (replace_n 2 '(a b c d e) bananas) returns (a b bananas d e)
      ;;; Used when multiple occurances of an element occur in a list and SUBST
      ;;; would replace all of them, and the list is indexed.
      (defun replace_n (num new lst / split_list head tail cnt)
        (setq  head '()
               tail '()
               cnt  0
               num  (1+ num)
        )
        (cond
          ((and (>= num 1) (<= num (length lst)))
           (while (>= (length lst) cnt)
             (cond
              ((< cnt num)
               (setq head (cons (nth cnt lst) head))
              )
              ((>= cnt num)
               (setq tail (cons (nth cnt lst) tail))
              )
             )
             (setq cnt (1+ cnt))
           )
           (setq split_list (list (reverse head) (reverse (cdr tail))))
           (append (reverse (cons new (cdr (reverse (car split_list)))))
                   (cadr split_list)
           )
          )
          (T lst)
        )
      )

      ;---Returns all remainder in list past nth position (inclusive).
      ;   (setq lst '(11 22 33 44 55 66))
      ;   (nthcdr 2 lst) => (33 44 55 66)
      (defun NTHCDR (N L)(if (zerop N) L (NTHCDR (1- N) (cdr L))) )

      ;---Returns what nth position in list, -1 if not in list.
      (defun what_nth (item alist / n)
        (setq n (member item alist))
        (if n (- (length alist)(length n)) -1)
      )

      ;---Get Properties.
      (defun get_prop ( mssg / what_is what_txt what_ltp what_sty what_lay what_col what_hgt edata sel_obj)
        (setq sel_obj nil)
        (while (= sel_obj nil)
          (setq sel_obj (entsel mssg))
        )
        (setq oname (car sel_obj)
              edata (entget oname)
              what_is  (dxf 0 edata)
              what_ltp (dxf 6 edata)
              what_lay (dxf 8 edata)
              what_col (dxf 62 edata) )
         (cond
          ((= what_is "LINE")
           (setq what_sta (dxf 10 edata)
                 what_end (dxf 11 edata)))
          ((= what_is "INSERT")
           (setq what_sta (dxf 10 edata)
                 what_end (dxf 11 edata)))
          ((= what_is "POINT")
           (setq what_sta (dxf 10 edata)))
          ((= what_is "POLYLINE")
           (setq what_mvtx (dxf 71 edata)
                 what_nvtx (dxf 72 edata)))
          ((= what_is "CIRCLE")
           (setq what_cen (dxf 10 edata)
                 what_rad (dxf 40 edata)))
          ((= what_is "TEXT")
           (setq what_jus (dxf 72 edata)
                 what_txt (dxf 1  edata)
                 what_hgt (dxf 40 edata)
                 what_rot (dxf 50 edata)))
            (if (= what_col nil)
                 (progn
                   (setq layinfo  (tblsearch "LAYER" what_lay)
                         what_col (abs (cdr (assoc 62 layinfo))))
                 )
            )
            (if (= what_ltp nil) (setq what_ltp "BYLAYER"))
         )
      )

      (TraceIsOn ".17")

      (defun get_block (mssg / what_is what_txt what_ltp what_sty what_lay what_col what_hgt edata sel_obj)
        (setq sel_obj nil)
        (while (= sel_obj nil)
          (setq sel_obj (nentsel mssg)
                sel_ent (entlast)
          )
        )
        (setq oname (car sel_obj)
              edata (entget oname)
              edat  (entget sel_ent)
              what_is  (dxf 0 edat)
              what_ins (dxf 10 edat)
              what_hdl (dxf 5 edat)
              what_lay (dxf 8 edat)
              what_col (dxf 62 edat)
        )
        (if (= what_col nil) (setq what_col "BYLAYER"))
        (if (= what_ltp nil) (setq what_ltp "BYLAYER"))
      )

      ;---RESET ERROR TILE.
      (defun rs_error ( / )(set_tile "error" "") (fill_image_color "wht_col" -15) )

      ;---(tile_msg "Fisheye%s" "%s" " Lens")
      (defun tile_msg (msg old new)(set_tile "error" (strrep msg old new)) )

      ;---With the xor function a list of tests are sent to the processor.
      ;   If any of the tests are true (T), the program returns T to the
      ;   calling function, however, if all tests return T false (nil) is returned.
      ;   (setq test (xor '((= 1 1)(= 1 2)))) would make test=T
      ;   (setq test (xor '((= 1 1)(= 2 2)))) would make test=nil
      (defun XOR (lst / cnt test out)
        (setq cnt 0)
        (foreach test lst
         (if (eval test) (setq cnt (1+ cnt)))
        )
        (if (or (= cnt (length lst))(= cnt 0))
            (setq out nil)(setq out T)
        )
        (setq out out)
      )

      ;---DOODLER TO KEEP USER AMUSED.
      ;  (setq c "|")...     ....(princ (chr 8))(princ (setq c (spin c)))
      (defun spin ( c )
        (cond
         ((= c  "|") "/")
         ((= c  "/") "-")
         ((= c  "-") "\\")
         ((= c "\\") "|")
        )
      )

      ;---Returns side b [delta X] of right angle triangles. GrandC est l'angle a 90deg, GrandA en radians et PetitC=hypothenuse.
      (defun BSide (GrandA PetitC / )
        (setq CosA   (cal (strcat "cos(" (angtos GrandA 1 8) ")" ))
          PetitB (* PetitC CosA)
        )
      )

      ;---Returns side b [delta X] of right angle triangles. GrandC est l'angle a 90deg, GrandA en radians et PetitC=hypothenuse.
      (defun SideB (GrandA PetitA / )
        (setq TanA   (cal (strcat "tang(" (angtos GrandA 1 8) ")" ))
              PetitB (/ PetitA TanA)
        )
      )

      ;---Returns side c [hypothenuse] of right angle triangles. GrandC est l'angle a 90deg, GrandA en radians.
      (defun SideC (GrandA PetitA /)
        (setq SinA   (cal (strcat "Sin(" (angtos GrandA 1 8) ")" ))
              PetitC (/ PetitA SinA)
        )
      )

      ;---This function simply returns the hypotinuse from 2 other sides.
      ;   (setq d1 (hyp 34.5 16.77)) would make d1=38.3599
      (DEFUN HYP (val1 val2)(SQRT (+ (EXPT val1 2)(EXPT val2 2))) )

      ;---Translate An Angle From One Coordinate System To Another
      ;   angle: angle to translate (in radians)
      ;   from:  an integer code similar to (trans)
      ;   toucs: an integer code similar to (trans)
      (defun trans_angl (angl fromucs toucs / pt1)
        (setq  pt1 (polar '(0 0 0) angl 1.0)      ;point in current UCS
               pt1 (trans pt1 1 fromucs)          ;translate current to FROM
               pt1 (trans pt1 fromucs toucs)      ;translate FROM to TO
        )
        (angle '(0 0 0) pt1)
      )

      ;---Calculates Beta angle, given two points is space.
      ;   (bangle '(10 10 10) '(50 50 50)) in radians.
      ;   pt1 origin, pt2 target.
      (defun bangle (pt1 pt2 / deltaz dist)
        (setq deltaz (- (caddr pt2)(caddr pt1))
             dist   (distance pt1 pt2)
        )
        (atan deltaz (sqrt (- (* dist dist)(* deltaz deltaz))))
      )

      ;---Calculates a 3D point given base pt, rotation from X axis
      ;   rotation from XY plane toward Z and offset distance.
      (defun 3dpolar (pt alpha beta dist / dx dz dy)
        (setq dx (* dist (cos alpha) (cos beta))
              dy (* dist (sin alpha) (cos beta))
              dz (* dist (sin beta))
        )
        (mapcar '+ (list dx dy dz) pt)
      )

      ;---Takes arg in radians, returns it with 6 decimal places.
      ;   (angtoc 3.14159265) ---> "<<180.000000"
      (defun angtoc (ang)
        (setq ang (rtos (atof (angtos ang 0 8)) 2 6) )
        (strcat "<<" ang)
      )

      (defun LetSwap (prmpt1 prmpt2 /)
        (setq ss1 nil
              ss2 nil
        )
        (princ "\nSelect objects to swap around...")
        (while (setq ss1 (entsel prmpt1))
          (if (setq ss2 (entsel prmpt2))
            (progn
              (setq t1  (entnext (car ss1))
                    t2  (entnext (car ss2))
                    bp1 (osnap (cadr ss1) "INS")
                    bp2 (osnap (cadr ss2) "INS")
              )
              (command "_.MOVE" ss1 "" bp1 bp2)
              (command "_.MOVE" ss2 "" bp2 bp1)
            )
          )
          (setq ss1 nil
                ss2 nil
          )
          (princ "\nSelect objects to swap around...")
        )
      )

      ;---Ask for 2 points and return the mid-point.
      (defun midp ()
        (setq pt1 (upoint 1 "" "Enter first point" nil nil)
              pt2 (upoint 1 "" "Enter second point" nil nil)
        )
        (polar pt1 (angle pt1 pt2) (/ (distance pt1 pt2) 2))
      )

      ;---Returns the average of X Y and Z of selected points. RGB
      (defun average ( / pt-a pt-b pt-c count size old_error MODE-V)
        (setvar "OSMODE" 33)
        (setq size (/ (getvar "VIEWSIZE") 12))
        (redraw)
        (setq pt-a  (upoint 1 "" "\nSpecify first point to average" nil nil)
              count 1
        )
        (setq pt-b (mapcar '(lambda (x) (/ x count)) pt-a)
              pt-c (trans pt-b 1 2)
        )
        (setvar "LASTPOINT" pt-b)
          (if (type grvecs)
            (grvecs (list 1 (trans '(-0.5 0 0) 0 2) (trans '(1 0 0) 0 2)
                            3 (trans '(0 -0.5 0) 0 2) (trans '(0 1 0) 0 2)
                            5 (trans '(0 0 -0.5) 0 2) (trans '(0 0 1) 0 2))
                      (list (list (eval size) 0.0 0.0 (nth 0 pt-c))
                            (list 0.0 (eval size) 0.0 (nth 1 pt-c))
                            (list 0.0 0.0 (eval size) (nth 2 pt-c))
                            (list 0.0 0.0 0.0 1.0))
            )
            (progn
              (grdraw (polar pt-b (* pi 0.25) size)
                      (polar pt-b (* pi 1.25) size)
                      -1 1
              )
              (grdraw (polar pt-b (* pi 0.75) size)
                      (polar pt-b (* pi 1.75) size)
                      -1 1
              )
            )
        )
        (prompt "\nOnly one point selected.")
        (while
          (setq pt-b (upoint 18 "" "Specify another point or <Exit>" nil nil) );setq
          (setvar "LASTPOINT" pt-b)
          (setq pt-a  (mapcar '+ pt-a pt-b)
                count (1+ count)
          )
          (redraw)
          (setq pt-b (mapcar '(lambda (x) (/ x count)) pt-a)
                pt-c (trans pt-b 1 2)
          )
          (if (type grvecs)
            (grvecs (list 1 (trans '(-0.5 0 0) 0 2) (trans '(1 0 0) 0 2)
                          3 (trans '(0 -0.5 0) 0 2) (trans '(0 1 0) 0 2)
                          5 (trans '(0 0 -0.5) 0 2) (trans '(0 0 1) 0 2)
                    )
                    (list (list (eval size) 0.0 0.0 (nth 0 pt-c))
                          (list 0.0 (eval size) 0.0 (nth 1 pt-c))
                          (list 0.0 0.0 (eval size) (nth 2 pt-c))
                          (list 0.0 0.0 0.0 1.0)
                    )
            )
            (progn
              (grdraw (polar pt-b (* pi 0.25) size)
                      (polar pt-b (* pi 1.25) size)
                      -1 1
              )
              (grdraw (polar pt-b (* pi 0.75) size)
                      (polar pt-b (* pi 1.75) size)
                      -1 1
              )
            )
          )
          (prompt (strcat "\n" (itoa count) " points selected.") );prompt
        )
        (redraw)
        (IF (> (BOOLE 1 (GETVAR "cmdactive") 3) 0)
          (command "_none")
        )
        (setvar "LASTPOINT" (mapcar '(lambda (x) (/ x count)) pt-a))
      )

      ;---Find mid of 2 given points.
      (defun mid_pt (p1 p2 / c1 c2)
        (mapcar '(lambda (c1 c2) (/ (+ c1 c2) 2.0)) p1 p2)
      )

      ;---
      (defun zaxis (pt1 pt2 / delta sum unitv)
        (setq delta (mapcar '- pt2 pt1)
              sqrpt (mapcar '* delta delta)
              sum   (apply '+ sqrpt)
              unitm (sqrt sum)
              unitv (mapcar '/ delta (list unitm unitm unitm))
        )
      )

      ;---
      (defun newecs (zaxis ename / ed bpt)
        (setq ed  (entget ename)
          bpt (trans (dxf 10 ed) ename zaxis)
          ed  (subst (cons 10 bpt) (assoc 10 ed) ed)
          ed  (subst (cons 210 zaxis) (assoc 210 ed) ed)
        )
        (entmod ed)
      )

      (TraceIsOn ".17AA")

      (setq bForUnit (getvar "MeasureInit")
            bOK      nil
      )

      (if (= bForUnit 0)
        (cond
          ((= langue "Franais")
           (setq bOK
             (dos_msgbox
               "Upon loading, ArchAssist has detected that the variable MeasureInit\nis set onto 0 indicating that you are working in inches [Imperial].\nThis is a metric version. Unless you authorise to change the variable\nto 1, ArchAssist will abort loading! Change?"
               "Message ArchAssist"
               4
               1
             )
           )
          )
          ((= langue "Italiano")
           (setq bOK
             (dos_msgbox
               "Upon loading, ArchAssist has detected that the variable MeasureInit\nis set onto 0 indicating that you are working in inches [Imperial].\nThis is a metric version. Unless you authorise to change the variable\nto 1, ArchAssist will abort loading! Change?"
               "ArchAssist  Message"
               4
               1
             )
           )
          )
          ((= langue "Espanol")
           (setq bOK
             (dos_msgbox
               "Upon loading, ArchAssist has detected that the variable MeasureInit\nis set onto 0 indicating that you are working in inches [Imperial].\nThis is a metric version. Unless you authorise to change the variable\nto 1, ArchAssist will abort loading! Change?"
               "Mensaje ArchAssist"
               4
               1
             )
           )
          )
          (T
           (setq bOK
             (dos_msgbox
               "Upon loading, ArchAssist has detected that the variable MeasureInit\nis set onto 0 indicating that you are working in inches [Imperial].\nThis is a metric version. Unless you authorise to change the variable\nto 1, ArchAssist will abort loading! Change?"
               "ArchAssist  Message"
               4
               1
             )
           )
          )
        )
      )
      (if (= bOK 6)
        (setvar "MeasureInit" 1)
      )
      (if (= bOK 3)
        (exit)
      )

      (TraceIsOn ".17BB")

      ;;(command "_.VIEWRES" "_Y" 1200)

      ;---viewpnts
      ;returns lower left and upper right coords of current view
      (defun viewpnts ( / na e1 x w h p1 p2 p3 p4 a b c d)
        (if (equal 0 (getvar "tilemode"))
          (progn
            ;;LCA - WARNING: Using 67 group code may retrieve more entities than anticipated.
            (setq na (ssget "_X" (list (cons 0 "VIEWPORT") (cons 67 1) (cons 69 (getvar "cvport"))) );ssget
                  na (ssname na 0)
                  e1 (entget na)
                   x (cdr (assoc 10 e1))
                   w (cdr (assoc 40 e1))
                   h (cdr (assoc 41 e1))
                  p1 (list (- (car x) (/ w 2.0))
                           (- (cadr x) (/ h 2.0))
                     )
                  p2 (list (+ (car x) (/ w 2.0))
                           (+ (cadr x) (/ h 2.0))
                     )
            )
            (if (not (equal 1 (getvar "cvport")))
                (setq p1 (trans p1 3 2)
                      p2 (trans p2 3 2)
                      na (ssget "_X" (list (cons 0 "VIEWPORT") (cons 67 1) (cons 69 1)) );ssget
                      na (ssname na 0)
                      e1 (entget na)
                       x (cdr (assoc 10 e1))
                       w (cdr (assoc 40 e1))
                       h (cdr (assoc 41 e1))
                      p3 (list (- (car x) (/ w 2.0))
                               (- (cadr x) (/ h 2.0))
                         )
                      p4 (list (+ (car x) (/ w 2.0))
                               (+ (cadr x) (/ h 2.0))
                         )
                      p3 (trans p3 3 2)
                      p4 (trans p4 3 2)
                      p1 (list (max (car p1) (car p3))
                               (max (cadr p1) (cadr p3))
                         )
                      p2 (list (min (car p2) (car p4))
                               (min (cadr p2) (cadr p4))
                         )
                      p1 (trans p1 2 1)
                      p2 (trans p2 2 1)
                )
                (setq p1 (trans p1 2 1)
                      p2 (trans p2 2 1)
                )
            )
          )
          (progn
            (setq b (getvar "viewsize")
                  c (car (getvar "screensize"))
                  d (cadr (getvar "screensize"))
                  a (* b (/ c d))
                  x (setq x (getvar "viewctr"))
                  x (trans x 1 2)
                 p1 (list (- (car x)  (/ a 2.0))
                          (- (cadr x) (/ b 2.0))
                          0.0
                    )
                 p2 (list (+ (car x)  (/ a 2.0))
                          (+ (cadr x) (/ b 2.0))
                          0.0
                    )
                 p1 (trans p1 2 1)
                 p2 (trans p2 2 1)
            )
          )
        )
        (list p1 p2)
      )

      ;returns the size of a single pixel in drawing units. Value depends on current zoom factor.
      ;pixunit/viewsize = one pixel/yscreensize
      ;pixunit=viewsize/yscreensize
      (defun pixel_unit ( / x y x1 y1)
        (setq  y (getvar "viewsize")
              x1 (car (getvar "screensize"))
              y1 (cadr (getvar "screensize"))
               x (* y (/ x1 y1))
        )
        (max (abs (/ y y1))
             (abs (/ x x1))
        )
      )

      ;---This function gets the selection set of the objects crossing the AutoCAD pickbox extents.
      ;   (ssgetc centerpt) ==> centerpt: the center of the pickbox or nil.

      (defun pixel ()
        (setq pix (/ (getvar "viewsize") (cadr (getvar "screensize"))))
      )

      (defun ssgetc (pt finer / c1 c2 delta)
        (setq pt    (trans pt 1 2)
          delta (* (getvar "pickbox") (* (pixel) finer))
        )
        (ssget "_C"
           (trans (mapcar '(lambda (x) (+ x delta)) pt) 2 1)
           (trans (mapcar '(lambda (x) (- x delta)) pt) 2 1)
        )
      )

      ;---Delete Wipeouts on a layer given in string format.
      ;   (WipeGo "Labels")
      (defun WipeGo (HshLyr / b)
        ;;LCA - WARNING: The PICKSTYLE sysvar is now saved in the registry.
        (setvar "pickstyle" 0)
        (setq b (ssget "_X" (list (cons 0 "WIPEOUT") (cons 8 HshLyr))))
        (if b
          (command "_.ERASE" b "")
        )
      )

      ;---Reset error tile, 0=invisible and 1=visible.
      (defun AArserror (sFName sValue /)
        (Odcl_Control_SetProperty
          ProjName2 sFName
          "OnError" "Visible" sValue
        )
      )

      ;---lsttrans
      ;   like standard autolisp trans but takes a LIST of points and returns a list of translated points.
      (defun lsttrans ( lst a b / lst2 c n)
        (setq n 0)
        (repeat (length lst)
          (setq c (nth n lst)
                c (trans c a b)
             lst2 (append lst2 (list c))
          )
          (setq n (+ n 1))
        )
        lst2
      )

      (TraceIsOn ".18")

      (defun LayerMake (nom /)
        (entmake (list
               (cons 0 "LAYER")
               (cons 100 "AcDbSymbolTableRecord")
               (cons 100 "AcDbLayerTableRecord")
               (cons 2 nom)
               (cons 70 0)
             )
        )
      )

      (defun PointMake (pt1 onlay)        ;used to be dopnt
        (entmake (list
                   (cons 0 "POINT")
                   (cons 100 "AcDbEntity")
                   (cons 100 "AcDbPoint")
                   (cons 10 pt1)
                   (cons 8 onlay)
                 )
        )
      )

      (defun CircleMake (pt1 rad /)
        (entmake (list
                   (cons 0 "CIRCLE")
                   (cons 100 "AcDbEntity")
                   (cons 100 "AcDbCircle")
                   (cons 10 pt1)
                   (cons 40 rad)
                 )
        )
        (if CMFlag
          (progn
            (setq elast (entlast))
            (ssadd elast vc2del)
          )
        )
      )

      ;---Entmake Lines using start, end points and layer name.
      (defun LineMake (pt1 pt2 lay /)
        (entmake (list
                   (cons 0 "LINE")
                   (cons 100 "AcDbEntity")
                   (cons 100 "AcDbLine")
                   (cons 10 pt1)
                   (cons 11 pt2)
                   (cons 8 lay)
                 )
        )
      )
      (defun PlineMake1 (shortlst / onlay )
        (entmake)
        (entmake (list (cons 0 "POLYLINE")
                       (cons 100 "AcDbEntity")
                       (cons 100 "AcDb2dPolyline")
                       (cons 10 (nth 0 shortlst))
                       (cons 70 0)
                       (cons 66 1)
                 )
        )
        (setq count 0)
        (while (< count (length shortlst))
          (setq pnt (nth count shortlst))
          (if (= "grls" Quefaire)
            (setq Hshp1 (list (+ (- (car pnt) bpch) conch)
                              (+ (* (- (cadr pnt) bpel) vsca) conel)
                        )
            )
            (setq Hshp1 pnt)
          )
          (entmake (list (cons 0 "Vertex")
                         (cons 100 "AcDbEntity")
                         (cons 100 "AcDbVertex")
                         (cons 100 "AcDb2dVertex")
                         (cons 10 pnt)     ;was changed from Hshp1 to pnt1
                         (cons 70 0)
                   )
          )
          (setq count (1+ count))
        )
        (entmake (list (cons 0 "Seqend")))
      )

      ;; ! Function : Given a list, draw a POLYLINE or LWPOLYLINE using the entmake function
      ;; ! Arguments:
      ;; !            'vlist'  - List of points from which to create polyline.
      ;; !            'flag'   - POLYLINE or LWPOLYLINE flag (DXF Code 70)
      ;; !            'Width'  - Width of the polyline.
      ;; ! Action   : Creates a POLYLINE or LWPOLYLINE from the list supplied with the specified
      ;; !            flag and Width.
      ;; !            If the flag has an '8' bit in it, a POLYLINE object is created else
      ;; !            if the '8' bit is absent, a POLYLINE or LWPOLYLINE object is created
      ;; !            depending on whether the PLINETYPE variable is 0 or 1 respectively.

      (defun PlineMake (vlist flag Width LWorPl / Elev VFlag pt entl LWPoly Lst len)
        (if (not (zerop (logand flag 8)))
          (setq Elev   0.0
                Width  0.0
                LWPoly nil
          )
          (progn
            (if (= LWorPl T)
              (setq LWPoly T)
              (setq LWPoly nil)
            )
            (setq Elev (caddr (nth 0 vlist)))
          )
        )
        (if LWPoly
          (progn (setq Lst '())
             (foreach pt vlist
               (setq Lst (cons (list 10 (car pt) (cadr pt)) Lst)
                 Lst (cons (cons 40 Width) Lst)
                 Lst (cons (cons 41 Width) Lst)
               )
             )
             (setq len (length vlist)
                   Lst (reverse Lst)
                   Lst (cons (cons 38 Elev) Lst)
                   Lst (cons (cons 70 flag) Lst)
                   Lst (cons (cons 90 len) Lst)
                   Lst (cons (cons 100 "AcDbPolyline") Lst)
                   Lst (cons (cons 100 "AcDbEntity") Lst)
                   Lst (cons (cons 0 "LWPOLYLINE") Lst)
             )
             (entmake Lst)
          )
          (progn
            (setq entl (list (cons 0 "POLYLINE")
                             (cons 66 1)
                             (list 10 0.0 0.0 Elev)
                             (cons 40 Width)
                             (cons 41 Width)
                             (cons 70 flag)
                       )
            )
            (entmake entl)
            (setq VFlag 0)
            (if (not (zerop (logand flag 8)))
              (setq VFlag (+ VFlag 32))
            )
            (if (not (zerop (logand flag 32)))
              (setq VFlag (+ VFlag 64))
            )
            (if (not (zerop (logand flag 64)))
              (setq VFlag (+ VFlag 128))
            )
            (foreach pt vlist
              (if (not (zerop (logand flag 8)))
                (setq Elev (caddr pt))
              )
              (setq entl (list (cons 0 "VERTEX") (list 10 (car pt) (cadr pt) (caddr pt)) (cons 70 VFlag)))
              (entmake entl)
            )
            (entmake (list (cons 0 "SEQEND")))
          )
        )
        (entlast)
      )

      (defun TextMake (Tip Tht Tang Tj1 Tj2 Tval sPolis)
        (if (= sPolis nil)
          (setq sPolis (getvar "TEXTSTYLE"))
        )
        (setq Tht (distof Tht))
        (entmake (list
                  (cons 0 "TEXT")
                  (cons 100 "AcDbEntity")
                  (cons 100 "AcDbText")
                  (cons 10 Tip)
                  (cons 11 Tip)
                  (cons 40 Tht)
                  (cons 1 Tval)
                  (cons 50 Tang)
                  (cons 7 sPolis)
                  ;;?         (cons 51 (cdr txtobang))
                  (cons 72 Tj1)
                  (cons 100 "AcDbText")
                  (cons 73 Tj2)
                )
        )
      )

      ;; 71 Attachment point:
      ;; 1 = Top left; 2 = Top center; 3 = Top right;
      ;; 4 = Middle left; 5 = Middle center; 6 = Middle right;
      ;; 7 = Bottom left; 8 = Bottom center; 9 = Bottom right
      (defun MTextMake (Tip Tht Wid Attach Tang Tj1 Tval sPolis)
        (if (= sPolis nil)
          (setq sPolis (getvar "TEXTSTYLE"))
        )
        (setq Tht (distof Tht))
        (entmake (list
                   (cons 0 "MTEXT")
                   (cons 100 "AcDbEntity")
                   (cons 100 "AcDbMText")
                   (cons 10 Tip)      ;Insertion point
                  ;(cons 11 Tip)      ;X-axis direction vector (in WCS)
                   (cons 40 Tht)      ;height
                   (cons 41 Wid)      ;Reference rectangle width
                   (cons 71 Attach)   ;Attachment point
                   (cons 72 Tj1)      ;Drawing direction: 1 = Left to right
                   (cons 1 Tval)      ;Text string.
                   (cons 7 sPolis)    ;style
                   (cons 50 Tang)
                   (cons 73 1)        ;Mtext line spacing style
                 )
        )
      )

      ;---(setq wlist '(10 20 15 45))
      ;   (T:ableMake p1 p3 3 6 wlist)   ===>horz. lines spaced by 3*6
      ;   Vertical lines will be drawn 3*element specified in wlist
      (defun T:ableMake (pt1 pt3 txtht spc wlst / pt2 pt4 pt5 pt6 v1)
        (if (= "Yes" (ukword 1  "Yes No" "Draw box [Yes/No] " "Yes"))
          (progn
            (setq
              pt2 (list (car pt1) (cadr pt3) (caddr pt1))
              pt4 (list (car pt3) (cadr pt1) (caddr pt1))
              pt5 pt2
              pt6 pt3
            )
            (command "_.pline" pt1 "_w" 0 0 pt2 pt3 pt4 "_c")
            (if (and
                  (not (equal pt2 (polar pt5 Hshpi270 (* txtht spc))  ))
                  (= "Yes" (ukword 1  "Yes No" "Draw horizontal lines [Yes/No] " "Yes"))
                )
              (repeat (fix (/ (distance pt1 pt2) (* txtht spc)))
                (command "_.line" (setq pt5 (polar pt5 Hshpi270 (* txtht spc)))
                  (setq pt6 (polar pt6 Hshpi270 (* txtht spc))) ""
                )
              )
            )
            (if (and wlst (= "Yes" (ukword 1  "Yes No" "Draw vertical lines [Yes/No] " "Yes")))
              (progn
                (setq
                  pt5 pt2
                  pt6 pt1
                )
                (foreach v1 wlst
                  (command "_.line" (setq pt5 (polar pt5 0.0 (* txtht v1)))
                    (setq pt6 (polar pt6 0.0 (* txtht v1)))   ""
                  )
                )
              )
            )
          )
        )
      )

      (defun 3DfMake (pt1 pt2 pt3 pt4 3flay)
        (entmake (list
                   (cons 0 "3DFACE")
                   (cons 100 "AcDbEntity")
                   (cons 100 "AcDbFace")
                   (cons 8 3flay)
                   (cons 10 pt1)
                   (cons 11 pt2)
                   (cons 12 pt3)
                   (cons 13 pt4)
                   (cons 67 0)
                   ;;LCA - COMMENT: True color support has been added
                   (cons 62 256)
                   (cons 70 0)
                 )
        )
      )

      ;---General linear 2d-pline, needs segments to support bulges cannot be used for splined plines (no tangent info) or width
      ;   segs: ((p1 p2 [bulge])(p2 p3 [bulge])...) elist may be nil or contain other properties, such ((8 . "0"))
      (defun PolylineMake (elist segs / opts flag seg)
        ;; check for pointlist instead of segmentlist
        (if (numberp (caar segs)) (setq segs (std-pts->segs segs)))
        ;; check again for last closing segment
        (setq flag
          (cond
            ((cdr (assoc 70 elist)))
            ((and (std-segs-closed-p segs)
                  (std-seg-straight-p (last segs))
             )
             1
            )
            (T 0)
          )
        )
        (setq opts (list
                     '(0 . "POLYLINE")
                     '(100 . "AcDbEntity")
                     '(100 . "AcDb2dPolyline")
                     '(66 . 1)
                      (cons 10 (list 0 0 (z-of (std-seg-p1 (car segs)))))
                      (cons 70 flag)
                   )
        )
        (std-%entmake-template elist opts nil)
        (setq opts '((0 . "VERTEX")
                     (100 . "AcDbEntity")
                     (100 . "AcDbVertex")
                     (100 . "AcDb2dVertex")
                     (70 . 0)
                    )
        )
        (foreach seg segs
          ;; how about layer info in vertex?
          ;; wrong layers might crash acad. to be fixed later.
          (std-%entmake-template (list (cons 10 (std-seg-p1 seg))
                                   (cons 42 (std-seg-bulge-num seg)))
                        opts nil)
        )
        ;; fixed
        (if (not (std-bitsetp 1 flag))    ; fixed by Serge Pashkov
          (std-%entmake-template (list (cons 10 (std-seg-p2 (last segs)))) opts nil)
        )
        ;; Todo: have to check layers!
        (std-%entmake-template '((0 . "SEQEND")) nil nil)
      )

      ;;; elist contains layer info: ((8 . "0")) is enough, () also faces: ((0 1 2)(-1 2 3)...) - zero based
      (defun PfaceMake (elist pts faces / opts flag face pt)
        ;; small internal helper to be redefined on every call
        (defun abs+1 (i)
          (if (minusp i) (1- i)(1+ i))
        )

        (setq flag (cond ((cdr (assoc 70 elist))) (0)))   ;flag or 0
        (setq opts (list
                    '(0 . "POLYLINE")
                    '(100 . "AcDbEntity")
                    '(100 . "AcDbPolyFaceMesh")
                    '(66 . 1)
                    '(10 0 0 0)
                     (cons 70 (+ 64 (logand 128 flag)))
                   )
        )
        (std-%entmake-template elist opts nil)

        (setq opts '(
                 (0 . "VERTEX")
                 (100 . "AcDbEntity")
                 (70 . 192))) ;points
        (foreach pt pts
          ;; no layer info in vertex
          (std-%entmake-template (list (cons 10 pt)) opts nil))

        ;; 100 groups missing for r14
        (setq opts '((0 . "VERTEX")
                 (100 . "AcDbEntity")
                 (70 . 128))) ;faceindices
        (foreach face faces
          (if (not (cadddr face)) (setq face (append face (list (car face)))))
          (std-%entmake-template (list
                                  '(10 0 0 0)
                                   (cons 71 (abs+1 (car face)))
                                   (cons 72 (abs+1 (cadr face)))
                                   (cons 73 (abs+1 (caddr face)))
                                   (cons 74 (abs+1 (cadddr face)))
                                 )
            opts nil
          )
        )
        ;; have to check layers!
        (std-%entmake-template '((0 . "SEQEND")) nil nil)
      )

      ;---Selective Regens
      (defun ObjRegen (s / index)
        (setq index 0)
        (if (not s)
          (command "_.REGEN")
          (repeat (sslength s)
            (entupd (ssname s index))
            (setq index (1+ index))
          )
        )
        (princ)
      )

      ;---(gc_pts (109.002 -39.7691 0.0)) returns "109.002 -39.769 0.000"
      (defun gc_pts (pt)
        (strcat (rtos (car pt))
                " "
                (rtos (cadr pt))
                (if (/= (caddr pt) nil)
                  (strcat " " (rtos (caddr pt)))
                  ""
                )
        )
      )

      (defun GRtos2 (pt) (rtos pt 2))

      ;---Checks for the validity of all members in a list to be a specific type.
      (defun GAllType (lst oftype)
        (setq lst (mapcar 'type lst))
        (and
          (= (car lst) oftype)
          (apply '= lst)
        )
      )

      ;---Converts a point into a string.
      ;   (GPnt2String (109.002 -39.7691 0.0)) returns ("109.002" "-39.769" "0.000")
      (defun GPnt2String (pt)
        (cond ((null pt) nil)
              ((= (type pt) 'list)       ;ok it's a list
                                         ;further checking
                  (if (and (= (length pt) 3)  ;3 things in the list
                          (GAllType pt 'real) ;Are they all real numbers?
                      )
               (mapcar 'GRtos2 pt)            ;Return point list as list
                                              ;of strings in 3decs
                   nil                        ;Otherwise return nil
               )
             )                                ;check for point list
             (T nil)
        )
      )

      ;---returns a point into a comma delimited string used by Geomcal.
      ;  (GPnt2Calc (109.002 -39.7691 0.0)) returns "109.002,-39.769,0.000"
      (defun GPnt2Calc (pt)
        (setq pt (GPnt2String pt))               ;each number x y z is a string in list pt
        (if pt
          (setq pt (apply 'append (mapcar '(lambda (x) (list x ",")) pt) )  ;apply
              pt (reverse (cdr (reverse pt)))    ;remove the last comma
              pt (apply 'strcat pt)              ;convert the point into a string separated by commas
          )
        )
      )

      ;---Returns a point into a bracketed list.
      ;   (GPnt2Bracket '(24.00 24.00 0)) Returns "[24.00,24.00,0]"
      (defun GPnt2Bracket (pt / ) (if (setq pt (GPnt2Calc pt)) (strcat "[" pt "]")) )

      ;---Indirectly set Symbol to Evaluated symbol's value.
      ;   Return evaluated list of values. In this manner if Nil is present
      ;   in the list, an appropriate error could be signaled.
      (defun GSetPnts (quoted_list / a)
        (setq a (mapcar '(lambda (x )
                           (set x (GPnt2Bracket (eval x)))
                         )
                         quoted_list
                )
        )
      )

      (defun do_convert ()
        (setq con11 (eval (read (strcat "ilp(" p1 ","  p2  ","  p3 "," p4 "," p5 ")" ))) )
      )

      (defun G:ilp (p1 p2 p3 p4 p5 / )
        (if (null (member (GSetPnts '(p1 p2 p3 p4 p5)) '(nil) ) )          ;null
          (progn
            (cal (strcat "ilp(" p1 ","  p2  ","  p3 "," p4 "," p5 ")" ))
          )
          nil
        )
      )

      (defun G:dpp (p1 p2 p3 p4 / )
        (if (null (member (GSetPnts '(p1 p2 p3 p4)) '(nil) ) )         ;null
          (progn
            (cal (strcat "dpp(" p1 ","  p2  ","  p3 "," p4 ")" ))
          )
          nil
        )
      )

      (defun G:ang (p1 p2 p3 / )
        (if (null (member (GSetPnts '(p1 p2 p3)) '(nil) ) )                ;null
          (progn
            (cal (strcat "ang(" p1 ","  p2  ","  p3 ")" ))
          )
          nil
        )
      )

      ;---OPEN, CLOSE FILE WITH ADVERTISING.
      ;   (do_ad "C:/DDARRAY.DCL" "DDARRAY routine." Hshfh14)
      (defun do_ad (dcl_dcl dcl_prg f_number / )
        (setq f_number (open dcl_dcl "w"))
        (write-line " TroCADero "  f_number)
        (write-line (strcat " Dialogue box for " dcl_prg)  f_number)
        (write-line " Thanks for using our products. "  f_number)
        (setq f_number (close f_number))
      )

      ;---CHECK FOR REAL NUMBER.
      ;   minusp : Returns T if -ve
      ;   zerop  : Returns T if equal to 0
      ;   numberp: Returns T if number

      ;---MORE USER-FRIENDLY PROMPTING.
      ;   (setq pt (upoint 1 ""  "Insertion point of slot" nil nil)
      ;         d  (udist  6 ""  "Diameter" d pt)
      ;         ol (udist  6 ""  "Overall length of slot" ol pt)
      ;         a  (uangle 1 ""  "Rotation angle" nil pt)
      ;  1 no null    2 no zero   4 no negative   8 no limits
      ; 16 use 3d points   32 use rubberband   64 ignore z)
      (defun udist (bit kwd msg def bpt / inp)
        (if def                                              ;test for a default
          (setq msg (strcat "\n" msg " <" (rtos def) ">: ")  ;string'em  with default
                bit (* 2 (fix (/ bit 2))) ;a default and no null bit code conflict so
          )
          (setq msg (strcat "\n" msg ": "))                  ;without default
        )
        (initget bit kwd)
        (setq inp
          (if bpt                  ;check for a base point
            (getdist msg bpt)      ;and use it in the GET commands
            (getdist msg)
          )
        )
        (if inp inp def)           ;compare the results, return appropriate value
      )

      (defun ukword (bit kwd msg def / inp)
        (if (and def (/= def ""))                     ;test for both nil and null string
          (setq msg (strcat "\n" msg "<" def ">: ")   ;string'em  with default
                bit (* 2 (fix (/ bit 2)))             ;a default and no null bit code conflict so
          )
          (if (= " " (substr msg (strlen msg) 1))
            (setq msg (strcat "\n" (substr msg 1 (1- (strlen msg))) ": "))
            (setq msg (strcat "\n" msg ": "))          ;without default
          )
        )
        (initget bit kwd)                             ;initialize the key words
        (setq inp (getkword msg))                     ;and use the GET command
        (if inp inp def)                          ;compare the results, return appropriate value
      )

      (defun ustr (bit msg def spflag / inp nval)
        (if (and def (/= def ""))                     ;test for both nil and null string
          (setq msg (strcat "\n" msg " <" def ">: ")  ;then include the default string
                inp (getstring msg spflag)            ;get input, ignore no null bit
                inp (if (= inp "") def inp)           ;if null input, return default
          )
          (progn
            (if (= " " (substr msg (strlen msg) 1))
             (setq msg (strcat "\n" (substr msg 1 (1- (strlen msg))) ": "))
              (setq msg (strcat "\n" msg ": "))         ;make string without the default
            )
            (if (= bit 1)                             ;if no null bit set to 1
              (while (= "" (setq inp (getstring msg spflag)))) ;then get input, no ""
              (setq inp (getstring msg spflag))     ;else get input, "" ok
            )
          )
        )
        inp
      )

      (defun uint (bit kwd msg def / inp)
        (if def                                                 ;test for a default
          (setq msg (strcat "\n" msg " <" (itoa def) ">: ")     ;string'em with default
               bit (* 2 (fix (/ bit 2)))  ;a default and no null bit code conflict so
          )                               ;this reduces bit by 1 if odd, to allow null
          (setq msg (strcat "\n" msg ": "))                     ;without default
        )
        (initget bit kwd)
        (setq inp (getint msg))       ;use the GETINT function
        (if inp inp def)              ;compare the results, return appropriate value
      )

      (defun ureal (bit kwd msg def / inp)
        (if def               ;test for a default
          (setq msg (strcat "\n" msg " <" (rtos def 2) ">: ") ;string'em with default
            bit (* 2 (fix (/ bit 2))) ;a default & no null bit code conflict so
          )                   ;this reduces bit by 1 if odd, to allow null
          (setq msg (strcat "\n" msg ": "))   ;without default
        )
        (initget bit kwd)
        (setq inp (getreal msg))      ;the GETREAL function
        (if inp
          inp
          def
        )
      )

      (defun upoint (bit kwd msg def bpt / inp)
        (if def               ;check for a default
          (setq pts (strcat (rtos (car def))
                    ","
                    (rtos (cadr def)) ;formats X,Y 2D pt as string
                    (if       ;formats 3D ,Z if supplied and FLATLAND off
                  (and (caddr def) (= 0 (getvar "FLATLAND")))
                   (strcat "," (rtos (caddr def)))
                   ""
                    )
                )             ;if&strcat
            msg (strcat "\n" msg " <" pts ">: ") ;string them with default
            bit (* 2 (fix (/ bit 2))) ;a default and no null bit code conflict so
          )                   ;this reduces bit by 1 if odd, to allow null
          (setq msg (strcat "\n" msg ": "))   ;or without
        )                 ;if a default was specified
        (initget bit kwd)
        (setq inp
                (if bpt         ;check for base point
                  (getpoint msg bpt)    ;and use it
                  (getpoint msg)        ;but not if nil
                )
        )
        (if inp
          inp
          def
        )
      )

      (defun upoint2 (bit kwd msg def bpt snp / inp)
        (if def               ;check for a default
          (setq pts (strcat (rtos (car def))
                    ","
                    (rtos (cadr def)) ;formats X,Y 2D pt as string
                    (if       ;formats 3D ,Z if supplied and FLATLAND off
                  (and (caddr def) (= 0 (getvar "FLATLAND")))
                   (strcat "," (rtos (caddr def)))
                   ""
                    )
                )             ;if&strcat
            msg (strcat "\n" msg " <" pts ">: ") ;string them with default
            bit (* 2 (fix (/ bit 2))) ;a default and no null bit code conflict so
          )                   ;this reduces bit by 1 if odd, to allow null
          (setq msg (strcat "\n" msg ": "))   ;or without
        )                 ;if a default was specified
        (initget bit kwd)
        (setq inp (if bpt         ;check for base point
                    (osnap (getpoint msg bpt) snp)    ;and use it
                    (osnap (getpoint msg) snp)    ;but not if nil
                  )
        )
        (if inp
          inp
          def
        )
      )

      (defun uangle (bit kwd msg def bpt / inp)
        (if def
          (setq msg (strcat "\n" msg " <" (angtos def) ">: ")
                bit (* 2 (fix (/ bit 2)))
          )
          (setq msg (strcat "\n" msg ": "))
        )
        (initget bit kwd)
        (setq inp
          (if bpt
            (getangle msg bpt)
            (getangle msg)
          )
        )
        (if inp inp def)
      )

      (defun uentsl (bit kwd msg def / inp ss)
        (if (and msg (/= msg ""))
          nil
          (setq msg "Select object")
        )
        (if (and def (/= def ""))
          (setq msg (strcat "\n" msg " <" def ">: ")  ;string them with default
            bit 0
          )                       ;this reduces bit by 1 if odd, to allow null
          (setq msg (strcat "\n" msg ": "))       ;or without
        )                     ;if a default was specified
        (initget bit kwd)
        (setq inp (getpoint msg))         ;but not if nil
        (cond ((= (type (setq kwd (if inp
                        inp
                        def
                      )
                )
              )
              'STR
           )
           kwd
          )
          ((not inp) nil)
          ((setq ss (ssget inp))
           (if ss
             (list (ssname ss 0) inp)
             nil
           )
          )
        )
      )

      (defun dxf (code elist)
        (cdr (assoc code elist))      ;finds the association pair, strips 1st element
      )                   ;defun

      (defun dxfExt (code ename) (cdr (assoc code (entget ename))))

      (defun initerr ()
        (setq Hsherror (append '((setvar "BLIPMODE" 0))))
      )

      (defun adderr (item)
        (if (not (member item Hsherror))  ;if it's not a duplicate item
          (setq Hsherror (cons item Hsherror)) ;add it to the list
        )
      )

      (if Hsherr
        nil
        (setq Hsherr *error*)
      )

      ;---Reset items.
      (defun reset (msg / x)
        (TraceIsOn ".reset")
        (if msg
          (prompt (strcat "Error: " msg "\n")) ;print out error message
        )
        (setq x 1)             ;sets the counter x
        (foreach func  Hsherror            ;loops through error items
          (eval func)              ;evaluates each one
          (grtext -2 (strcat (itoa x) " items reset."))
                           ;Shows user a running count of
          (setq x (1+ x))          ;the number of items reset
        )                  ;foreach
        (initerr)                ;resets default HshERROR list
        (dos_waitcursor)
        (princ)
      )

      ;---
      (defun XYZ_SDF (st_in st1 st2)
        (read
          (strcat "(" (substr st_in st1 st2) ")")
        )
      )

      ;---
      (defun ReadRecord (fh eor / ln ch)
        (setq ln "")
        (while (and (setq ch (read-char fh))
                    (/= ch eor)
               )
           (setq ln (strcat ln (chr ch)))
        )
        (if (> (strlen ln) 0) ln (if ch ln nil))
      )

      ;---
      (defun varstr (sysvrs)
        (mapcar
         '(lambda (varnam) (cons varnam (getvar varnam)))
           sysvrs)
      )

      ;---
      (defun resset (sysvrs)
        (TraceIsOn ".resset")
        (redraw)
        (command "_.UNDO" "_End")
        (foreach temp sysvrs
          (setvar (car temp) (cdr temp))
        )
        (princ)
      )

      ;---
      (defun generr ()
        (if (/= s "Function cancelled")(princ (strcat "\nError: " s)) )
        (if (/= s "Fonction Annule")(princ  "\n** Erreur Programme Interrompu **"))
        (setq s nil)
        (setq *error* olderr)
        (princ)
      )

      ;(defun *Error* (Err /)
      ;  (vl-bt)
      ;)

      ;---create a layer, given a name, color and ltype.
      (defun Hshset_lay   (nom Hshcol Hshltyp Hshlwt)
        (if Hshltyp nil (setq Hshltyp "CONTINUOUS") )
        (if Hshcol  nil (setq Hshcol "7") )
        (if Hshlwt  nil (setq Hshlwt "Default") )
        (if (tblsearch "LAYER" nom)
          (progn
            (command "._LAYER" "_T" nom "_On" nom "_S" nom "")
          )
          (progn
            (command "._LAYER" "_M" nom "_C" Hshcol "" "_LW" Hshlwt "" "_LT" Hshltyp "" "")
          )
        )
      )

      ;---SUB-ROUTINE TO GATHER INFO
      (defun Wallsgather_info ()
        (TraceIsOn ".Wallsgather_info")
        (WallsMod)
        (command "_.UNDO" "_End")
        (command "_.UNDO" "_BEgin")
        (graphscr)
        (initerr)              ;initialize error list
        (adderr                ;adds command function to reset layer name
          (list 'command "_.LAYER" "_S" (getvar 'CLAYER) "")
        )
        (FlashMess3
           " "
           " "
           " "
           " "
        )

        (setq varlst '( "angbase"   "angdir"    "aperture"
                        "attdia"    "aunits"    "auprec"
                        "blipmode"  "cecolor"       "clayer"    "celtype"   "celweight"
                        "cmdecho"   "dimdli"        "dimzin"    "elevation" "expert"
                        "filletrad" "gridmode"      "highlight"
                        "lunits"    "luprec"        "menuecho"  "maxsort"   "orthomode"
                        "osmode"    "pdmode"    "plinewid"
                        "pickbox"   "plinetype"     "snapang"   "splinesegs"
                        "snapbase"  "snapmode"  "regenmode"
                        "splframe"  "texteval"      "textsize"
                        "thickness" "tilemode"      "ucsicon"
                      )
        )
        (setq varlst (varstr varlst))
        (setvar "CECOLOR" "256")                   ;BYLAYER
        (setvar "AUPREC" 3)
        (setvar "CELTYPE" "BYLAYER")
        (setvar "CMDECHO" 0)
        (setvar "pickbox" 4)
        (setvar "BLIPMODE" 0)
        (setvar "UCSICON" 0)
        (setvar "CELWEIGHT" -1)
        (if (< (getvar "OSMODE") 16384) (toggle "osmode" 16384) )  ;If Osnap on, turn off
      )

      (defun c:AAMenu (/) (AAMenu nil))

      ;---------------------
      ;---Get'm All Objects.
      ;---------------------
      (defun getall (/ ssObj)
        (WsFlashMess2
          "\nSlectionner <Tout> ou"
          "\nPress ENTER to select All or"
          "\nPress ENTER to select All or"
          "\nPress ENTER to select All or"
        )
        (setq ssObj (ssget)
              ssObj (if ssObj
                      ssObj
                      (ssget "_X")
                    )
              n     (if ssObj
                      (sslength ssObj)
                      0
                    )
        )
        ssObj
      )

      ;------------------
      ;---Get one Object.
      ;------------------
      (defun SingPik (/ pPicked ssObj)
        (setq pPicked (cadr (grread nil 4 2)))
        (if (setq ssObj (ssget pPicked))
          (list (ssname ssObj 0) pPicked)
          pPicked
        )
      )

      (TraceIsOn ".19")

      ;---getx
      ; return <nth> dotted pair of the extended entity data from an entity association list <data>
      (defun getx (n data) (nth n (cdadr (assoc -3 data))) )

      ;---sslist
      ; convert selection-set <SS> into a list of entities <P>; result is the list <P>
      ; Example: (sslist <selection-set>)
      (defun sslist (ssObj / N P)
        (repeat (setq N (sslength ssObj)) ;seed N
          (setq N (1- N)          ;index number
            P (cons (ssname ssObj N) P)
          )
        )
      )

      ;-------------------
      ;---Show About Form.
      ;-------------------
      (defun c:Apropos_Ok_OnClicked () (Odcl_Form_Close ProjName2 "Apropos"))

      (MontreAA2)
      (MontreAA3)
      (MontreAA5)

      (defun _about (/) (Odcl_Form_Show ProjName2 "Apropos"))

      ;;;---
      ;(if (> (getvar "CDATE") (atof (strcat ElemUn ElemDe ElemTr)))
      ; (progn
      ;  (alert sMessA1)
      ;  (princ sMessA1)(princ)
      ;  (exit)
      ; )
      ;)
      ;
      ;(if (and (= sRevend "K")(> (getvar "CDATE") (atof (strcat ElemUna ElemDe ElemTr))))
      ; (progn                                ;pour revendeurs seulement >> "K"
      ;  (alert sMessA2)                          ;apres la date on arrete
      ;  (princ sMessA2)(princ)
      ;  (exit)
      ; )
      ;)
      ;
      ;-----------------
      ;---Main settings.
      ;-----------------
      ;(if (and (= (getvar "MeasureInit") 0)                  ;si dans le registre = 0
      ;    (or (= bImpUnit nil) (= bMetUnit nil))
      ;    )
      ;  (setq    bImpUnit T
      ;   bMetUnit nil
      ;  )
         (setq bImpUnit nil
               bMetUnit T
         )
      ;)

      (defun GetLayerINI ()                           ;initiate
        (TraceIsOn ".GetLayerINI")
        (setq sBrkWall1Lyr (dos_getini "Miscellany" "BrkWall1" Hshfile2)  ;0
              sBrkWall2Lyr (dos_getini "Miscellany" "BrkWall2" Hshfile2)  ;1
              sBlkWall1Lyr (dos_getini "Miscellany" "BlkWall1" Hshfile2)
              sConWall1Lyr (dos_getini "Miscellany" "ConWall1" Hshfile2)
              sParWall1Lyr (dos_getini "Miscellany" "ParWall1" Hshfile2)
              sTmbWall1Lyr (dos_getini "Miscellany" "TmbWall1" Hshfile2)
              sTmbWall2Lyr (dos_getini "Miscellany" "TmbWall2" Hshfile2)
              sSymbol25Lyr (dos_getini "Miscellany" "Symbol25" Hshfile2)
              sSymbol35Lyr (dos_getini "Miscellany" "Symbol35" Hshfile2)
              sText0Lyr    (dos_getini "Miscellany" "Text0Lyr" Hshfile2)
              sText1Lyr    (dos_getini "Miscellany" "Text1Lyr" Hshfile2)
              sText2Lyr    (dos_getini "Miscellany" "Text2Lyr" Hshfile2)
              sText3Lyr    (dos_getini "Miscellany" "Text3Lyr" Hshfile2)
              sText4Lyr    (dos_getini "Miscellany" "Text4Lyr" Hshfile2)
              sTAreaLyr    (dos_getini "Miscellany" "TAreaLyr" Hshfile2)
              sTBndryLyr   (dos_getini "Miscellany" "TBndryLyr" Hshfile2)
              sHatch1Lyr   (dos_getini "Miscellany" "Hatch1Lyr" Hshfile2)
              sJambLyr     (dos_getini "Miscellany" "JambLyr" Hshfile2)
              sThreshLyr   (dos_getini "Miscellany" "ThreshLyr" Hshfile2)
              sLeafLyr     (dos_getini "Miscellany" "LeafLyr" Hshfile2)
              sSwingLyr    (dos_getini "Miscellany" "SwingLyr" Hshfile2)
              sHeaderLyr   (dos_getini "Miscellany" "HeaderLyr" Hshfile2)
              sGlazeLyr    (dos_getini "Miscellany" "GlazeLyr" Hshfile2)
              sFrameLyr    (dos_getini "Miscellany" "FrameLyr" Hshfile2)
              sWHeaderLyr  (dos_getini "Miscellany" "WHeaderLyr" Hshfile2)
              sSashLyr     (dos_getini "Miscellany" "SashLyr" Hshfile2)
              sSillLyr     (dos_getini "Miscellany" "SillLyr" Hshfile2)
              sBarsLyr     (dos_getini "Miscellany" "BarsLyr" Hshfile2)
        )

        (if (= sBrkWall1Lyr nil)
          (setq sBrkWall1Lyr "BrickWork1|1,0.35")
        )
        (if (= sBrkWall2Lyr nil)
          (setq sBrkWall2Lyr "BrickWork2|7,0.25")
        )
        (if (= sBlkWall1Lyr nil)
          (setq sBlkWall1Lyr "BlockWork1|82,0.35")
        )
        (if (= sConWall1Lyr nil)
          (setq sConWall1Lyr "Concrete1|82,0.35")
        )
        (if (= sParWall1Lyr nil)
          (setq sParWall1Lyr "Partition1|60,0.35")
        )
        (if (= sTmbWall1Lyr nil)
          (setq sTmbWall1Lyr "Timber1|41,0.35")
        )
        (if (= sTmbWall2Lyr nil)
          (setq sTmbWall2Lyr "Timber2|200,0.25")
        )
        (if (= sSymbol35Lyr nil)
          (setq sSymbol35Lyr "Symbol3|230,0.35")
        )
        (if (= sSymbol25Lyr nil)
          (setq sSymbol25Lyr "Symbol2|123,0.25")
        )
        (if (= sText0Lyr nil)
          (setq sText0Lyr "Text1|6,0.18")
        )
        (if (= sText1Lyr nil)
          (setq sText1Lyr "Text2|7,0.25")
        )
        (if (= sText2Lyr nil)
          (setq sText2Lyr "Text3|2,0.35")
        )
        (if (= sText3Lyr nil)
          (setq sText3Lyr "Text5|1,0.5")
        )
        (if (= sText4Lyr nil)
          (setq sText4Lyr "Text7|4,0.7")
        )
        (if (= sTAreaLyr nil)
          (setq sTAreaLyr "TArea|124,0.35")
        )
        (if (= sTBndryLyr nil)
          (setq sTBndryLyr "TBoundary|124,0.35")
        )
        (if (= sHatch1Lyr nil)
          (setq sHatch1Lyr "Hatch1|152,0.18")
        )
        (if (= sJambLyr nil)
          (setq sJambLyr "Jamb|43,0.18")
        )
        (if (= sThreshLyr nil)
          (setq sThreshLyr "Sill|23,0.25")
        )
        (if (= sLeafLyr nil)
          (setq sLeafLyr "Leaf|42,0.18")
        )
        (if (= sSwingLyr nil)
          (setq sSwingLyr "DSwing|65,0.18")
        )
        (if (= sHeaderLyr nil)
          (setq sHeaderLyr "Header|23,0.35")
        )
        (if (= sGlazeLyr nil)
          (setq sGlazeLyr "Glass|151,0.18")
        )
        (if (= sFrameLyr nil)
          (setq sFrameLyr "Mullion|43,0.18")
        )
        (if (= sWHeaderLyr nil)
          (setq sWHeaderLyr "WHeader|23,0.35")
        )
        (if (= sSashLyr nil)
          (setq sSashLyr "Sash|42,0.18")
        )
        (if (= sSillLyr nil)
          (setq sSillLyr "Sill|23,0.25")
        )
        (if (= sBarsLyr nil)
          (setq sBarsLyr "Bars|32,0.25")
        )

      )

      (defun GetLayers ()                         ;initiate
        (TraceIsOn ".GetLayers")
        (setq sBrkWall1Lyr (dos_getini "Miscellany" "BrkWall1" Hshfile2)  ;0
              sBrkWall2Lyr (dos_getini "Miscellany" "BrkWall2" Hshfile2)  ;1
              sBlkWall1Lyr (dos_getini "Miscellany" "BlkWall1" Hshfile2)
              sConWall1Lyr (dos_getini "Miscellany" "ConWall1" Hshfile2)
              sParWall1Lyr (dos_getini "Miscellany" "ParWall1" Hshfile2)
              sTmbWall1Lyr (dos_getini "Miscellany" "TmbWall1" Hshfile2)
              sTmbWall2Lyr (dos_getini "Miscellany" "TmbWall2" Hshfile2)
              sSymbol25Lyr (dos_getini "Miscellany" "Symbol25" Hshfile2)
              sSymbol35Lyr (dos_getini "Miscellany" "Symbol35" Hshfile2)
              sText0Lyr    (dos_getini "Miscellany" "Text0Lyr" Hshfile2)
              sText1Lyr    (dos_getini "Miscellany" "Text1Lyr" Hshfile2)
              sText2Lyr    (dos_getini "Miscellany" "Text2Lyr" Hshfile2)
              sText3Lyr    (dos_getini "Miscellany" "Text3Lyr" Hshfile2)
              sText4Lyr    (dos_getini "Miscellany" "Text4Lyr" Hshfile2)
              sTAreaLyr    (dos_getini "Miscellany" "TAreaLyr" Hshfile2)
              sTBndryLyr   (dos_getini "Miscellany" "TBndryLyr" Hshfile2)
              sHatch1Lyr   (dos_getini "Miscellany" "Hatch1Lyr" Hshfile2)
              sJambLyr     (dos_getini "Miscellany" "JambLyr" Hshfile2)
              sThreshLyr   (dos_getini "Miscellany" "ThreshLyr" Hshfile2)
              sLeafLyr     (dos_getini "Miscellany" "LeafLyr" Hshfile2)
              sSwingLyr    (dos_getini "Miscellany" "SwingLyr" Hshfile2)
              sHeaderLyr   (dos_getini "Miscellany" "HeaderLyr" Hshfile2)
              sGlazeLyr    (dos_getini "Miscellany" "GlazeLyr" Hshfile2)
              sFrameLyr    (dos_getini "Miscellany" "FrameLyr" Hshfile2)
              sWHeaderLyr  (dos_getini "Miscellany" "WHeaderLyr" Hshfile2)
              sSashLyr     (dos_getini "Miscellany" "SashLyr" Hshfile2)
              sSillLyr     (dos_getini "Miscellany" "SillLyr" Hshfile2)
              sBarsLyr     (dos_getini "Miscellany" "BarsLyr" Hshfile2)
        )

        (if (= sBrkWall1Lyr nil)
          (setq sBrkWall1Lyr "BrickWork1|1,0.35")
        )
        (setq sBrkWall1Lwt (cadr (strfind sBrkWall1Lyr ","))
          sBrkWall1Col (cadr (strfind (car (strfind sBrkWall1Lyr ",")) "|"))
          sBrkWall1Lyr (car (strfind (car (strfind sBrkWall1Lyr ",")) "|"))
        )
        (if (= sBrkWall2Lyr nil)
          (setq sBrkWall2Lyr "BrickWork2|7,0.25")
        )
        (setq sBrkWall2Lwt (cadr (strfind sBrkWall2Lyr ","))
          sBrkWall2Col (cadr (strfind (car (strfind sBrkWall2Lyr ",")) "|"))
          sBrkWall2Lyr (car (strfind (car (strfind sBrkWall2Lyr ",")) "|"))
        )
        (if (= sBlkWall1Lyr nil)
          (setq sBlkWall1Lyr "BlockWork1|82,0.35")
        )
        (setq sBlkWall1Lwt (cadr (strfind sBlkWall1Lyr ","))
          sBlkWall1Col (cadr (strfind (car (strfind sBlkWall1Lyr ",")) "|"))
          sBlkWall1Lyr (car (strfind (car (strfind sBlkWall1Lyr ",")) "|"))
        )
        (if (= sConWall1Lyr nil)
          (setq sConWall1Lyr "Concrete1|82,0.35")
        )
        (setq sConWall1Lwt (cadr (strfind sConWall1Lyr ","))
          sConWall1Col (cadr (strfind (car (strfind sConWall1Lyr ",")) "|"))
          sConWall1Lyr (car (strfind (car (strfind sConWall1Lyr ",")) "|"))
        )
        (if (= sParWall1Lyr nil)
          (setq sParWall1Lyr "Partition1|60,0.35")
        )
        (setq sParWall1Lwt (cadr (strfind sParWall1Lyr ","))
          sParWall1Col (cadr (strfind (car (strfind sParWall1Lyr ",")) "|"))
          sParWall1Lyr (car (strfind (car (strfind sParWall1Lyr ",")) "|"))
        )
        (if (= sTmbWall1Lyr nil)
          (setq sTmbWall1Lyr "Timber1|41,0.35")
        )
        (setq sTmbWall1Lwt (cadr (strfind sTmbWall1Lyr ","))
          sTmbWall1Col (cadr (strfind (car (strfind sTmbWall1Lyr ",")) "|"))
          sTmbWall1Lyr (car (strfind (car (strfind sTmbWall1Lyr ",")) "|"))
        )
        (if (= sTmbWall2Lyr nil)
          (setq sTmbWall2Lyr "Timber2|200,0.25")
        )
        (setq sTmbWall2Lwt (cadr (strfind sTmbWall2Lyr ","))
          sTmbWall2Col (cadr (strfind (car (strfind sTmbWall2Lyr ",")) "|"))
          sTmbWall2Lyr (car (strfind (car (strfind sTmbWall2Lyr ",")) "|"))
        )
        (if (= sSymbol35Lyr nil)
          (setq sSymbol35Lyr "Symbol3|230,0.35")
        )
        (setq sSymbol35Lwt (cadr (strfind sSymbol35Lyr ","))
          sSymbol35Col (cadr (strfind (car (strfind sSymbol35Lyr ",")) "|"))
          sSymbol35Lyr (car (strfind (car (strfind sSymbol35Lyr ",")) "|"))
        )
        (if (= sSymbol25Lyr nil)
          (setq sSymbol25Lyr "Symbol2|123,0.25")
        )
        (setq sSymbol25Lwt (cadr (strfind sSymbol25Lyr ","))
          sSymbol25Col (cadr (strfind (car (strfind sSymbol25Lyr ",")) "|"))
          sSymbol25Lyr (car (strfind (car (strfind sSymbol25Lyr ",")) "|"))
        )
        (if (= sText0Lyr nil)
          (setq sText0Lyr "Text1|6,0.18")
        )
        (setq sText0Lwt (cadr (strfind sText0Lyr ","))
          sText0Col (cadr (strfind (car (strfind sText0Lyr ",")) "|"))
          sText0Lyr (car (strfind (car (strfind sText0Lyr ",")) "|"))
        )
        (if (= sText1Lyr nil)
          (setq sText1Lyr "Text2|7,0.25")
        )
        (setq sText1Lwt (cadr (strfind sText1Lyr ","))
          sText1Col (cadr (strfind (car (strfind sText1Lyr ",")) "|"))
          sText1Lyr (car (strfind (car (strfind sText1Lyr ",")) "|"))
        )
        (if (= sText2Lyr nil)
          (setq sText2Lyr "Text3|2,0.35")
        )
        (setq sText2Lwt (cadr (strfind sText2Lyr ","))
          sText2Col (cadr (strfind (car (strfind sText2Lyr ",")) "|"))
          sText2Lyr (car (strfind (car (strfind sText2Lyr ",")) "|"))
        )
        (if (= sText3Lyr nil)
          (setq sText3Lyr "Text5|1,0.5")
        )
        (setq sText3Lwt (cadr (strfind sText3Lyr ","))
          sText3Col (cadr (strfind (car (strfind sText3Lyr ",")) "|"))
          sText3Lyr (car (strfind (car (strfind sText3Lyr ",")) "|"))
        )
        (if (= sText4Lyr nil)
          (setq sText4Lyr "Text7|4,0.7")
        )
        (setq sText4Lwt (cadr (strfind sText4Lyr ","))
          sText4Col (cadr (strfind (car (strfind sText4Lyr ",")) "|"))
          sText4Lyr (car (strfind (car (strfind sText4Lyr ",")) "|"))
        )
        (if (= sTAreaLyr nil)
          (setq sTAreaLyr "TArea|124,0.35")
        )
        (setq sTAreaLwt (cadr (strfind sTAreaLyr ","))
          sTAreaCol (cadr (strfind (car (strfind sTAreaLyr ",")) "|"))
          sTAreaLyr (car (strfind (car (strfind sTAreaLyr ",")) "|"))
        )
        (if (= sTBndryLyr nil)
          (setq sTBndryLyr "TBoundary|124,0.35")
        )
        (setq sTBndryLwt (cadr (strfind sTBndryLyr ","))
          sTBndryCol (cadr (strfind (car (strfind sTBndryLyr ",")) "|"))
          sTBndryLyr (car (strfind (car (strfind sTBndryLyr ",")) "|"))
        )
        (if (= sHatch1Lyr nil)
          (setq sHatch1Lyr "Hatch1|152,0.18")
        )
        (setq sHatch1Lwt (cadr (strfind sHatch1Lyr ","))
          sHatch1Col (cadr (strfind (car (strfind sHatch1Lyr ",")) "|"))
          sHatch1Lyr (car (strfind (car (strfind sHatch1Lyr ",")) "|"))
        )
        (if (= sJambLyr nil)
          (setq sJambLyr "Jamb|43,0.18")
        )
        (setq sJambLwt (cadr (strfind sJambLyr ","))
          sJambCol (cadr (strfind (car (strfind sJambLyr ",")) "|"))
          sJambLyr (car (strfind (car (strfind sJambLyr ",")) "|"))
        )
        (if (= sThreshLyr nil)
          (setq sThreshLyr "Sill|23,0.25")
        )
        (setq sThreshLwt (cadr (strfind sThreshLyr ","))
          sThreshCol (cadr (strfind (car (strfind sThreshLyr ",")) "|"))
          sThreshLyr (car (strfind (car (strfind sThreshLyr ",")) "|"))
        )
        (if (= sLeafLyr nil)
          (setq sLeafLyr "Leaf|42,0.18")
        )
        (setq sLeafLwt (cadr (strfind sLeafLyr ","))
          sLeafCol (cadr (strfind (car (strfind sLeafLyr ",")) "|"))
          sLeafLyr (car (strfind (car (strfind sLeafLyr ",")) "|"))
        )
        (if (= sSwingLyr nil)
          (setq sSwingLyr "DSwing|65,0.18")
        )
        (setq sSwingLwt (cadr (strfind sSwingLyr ","))
          sSwingCol (cadr (strfind (car (strfind sSwingLyr ",")) "|"))
          sSwingLyr (car (strfind (car (strfind sSwingLyr ",")) "|"))
        )
        (if (= sHeaderLyr nil)
          (setq sHeaderLyr "Header|23,0.35")
        )
        (setq sHeaderLwt (cadr (strfind sHeaderLyr ","))
          sHeaderCol (cadr (strfind (car (strfind sHeaderLyr ",")) "|"))
          sHeaderLyr (car (strfind (car (strfind sHeaderLyr ",")) "|"))
        )
        (if (= sGlazeLyr nil)
          (setq sGlazeLyr "Glass|151,0.18")
        )
        (setq sGlazeLwt (cadr (strfind sGlazeLyr ","))
          sGlazeCol (cadr (strfind (car (strfind sGlazeLyr ",")) "|"))
          sGlazeLyr (car (strfind (car (strfind sGlazeLyr ",")) "|"))
        )
        (if (= sFrameLyr nil)
          (setq sFrameLyr "Mullion|43,0.18")
        )
        (setq sFrameLwt (cadr (strfind sFrameLyr ","))
          sFrameCol (cadr (strfind (car (strfind sFrameLyr ",")) "|"))
          sFrameLyr (car (strfind (car (strfind sFrameLyr ",")) "|"))
        )
        (if (= sWHeaderLyr nil)
          (setq sWHeaderLyr "WHeader|23,0.35")
        )
        (setq sWHeaderLwt (cadr (strfind sWHeaderLyr ","))
          sWHeaderCol (cadr (strfind (car (strfind sWHeaderLyr ",")) "|"))
          sWHeaderLyr (car (strfind (car (strfind sWHeaderLyr ",")) "|"))
        )
        (if (= sSashLyr nil)
          (setq sSashLyr "Sash|42,0.18")
        )
        (setq sSashLwt (cadr (strfind sSashLyr ","))
          sSashCol (cadr (strfind (car (strfind sSashLyr ",")) "|"))
          sSashLyr (car (strfind (car (strfind sSashLyr ",")) "|"))
        )
        (if (= sSillLyr nil)
          (setq sSillLyr "Sill|23,0.25")
        )
        (setq sSillLwt (cadr (strfind sSillLyr ","))
          sSillCol (cadr (strfind (car (strfind sSillLyr ",")) "|"))
          sSillLyr (car (strfind (car (strfind sSillLyr ",")) "|"))
        )
        (if (= sBarsLyr nil)
          (setq sBarsLyr "Bars|32,0.25")
        )
        (setq sBarsLwt (cadr (strfind sBarsLyr ","))
          sBarsCol (cadr (strfind (car (strfind sBarsLyr ",")) "|"))
          sBarsLyr (car (strfind (car (strfind sBarsLyr ",")) "|"))
        )
      )

      ;;;(if (= bMetUnit T)
      ;;;  (if (= sLeafLyr nil)   (setq sLeafLyr "LEAF-3|42,0.18") )
      ;;;  (if (= sLeafLyr nil)   (setq sLeafLyr "LEAF-3|42,0.18") )
      ;;;)
      ;;;(setq sLeafLwt (cadr (strfind sLeafLyr ","))
      ;;;      sLeafCol (cadr (strfind (car (strfind sLeafLyr ",")) "|"))
      ;;;      sLeafLyr (car  (strfind (car (strfind sLeafLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sSwingLyr nil)  (setq sSwingLyr "DSWING-3|65,0.18") )
      ;;;  (if (= sSwingLyr nil)  (setq sSwingLyr "DSWING-3|65,0.18") )
      ;;;)
      ;;;(setq sSwingLwt (cadr (strfind sSwingLyr ","))
      ;;;      sSwingCol (cadr (strfind (car (strfind sSwingLyr ",")) "|"))
      ;;;      sSwingLyr (car  (strfind (car (strfind sSwingLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sJambLyr nil) (setq sJambLyr "JAMB-3|43,018") )
      ;;;  (if (= sJambLyr nil) (setq sJambLyr "JAMB-3|43,018") )
      ;;;)
      ;;;(setq sJambLwt (cadr (strfind sJambLyr ","))
      ;;;      sJambCol (cadr (strfind (car (strfind sJambLyr ",")) "|"))
      ;;;      sJambLyr (car  (strfind (car (strfind sJambLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sHeaderLyr nil) (setq sHeaderLyr "HEADER|23,0.35") )
      ;;;  (if (= sHeaderLyr nil) (setq sHeaderLyr "HEADER|23,0.35") )
      ;;;)
      ;;;(setq sHeaderLwt (cadr (strfind sHeaderLyr ","))
      ;;;      sHeaderCol (cadr (strfind (car (strfind sHeaderLyr ",")) "|"))
      ;;;      sHeaderLyr (car  (strfind (car (strfind sHeaderLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sThreshLyr nil) (setq sThreshLyr "SILL-3|23,0.25") )
      ;;;  (if (= sThreshLyr nil) (setq sThreshLyr "SILL-3|23,0.25") )
      ;;;)
      ;;;(setq sThreshLwt (cadr (strfind sThreshLyr ","))
      ;;;      sThreshCol (cadr (strfind (car (strfind sThreshLyr ",")) "|"))
      ;;;      sThreshLyr (car  (strfind (car (strfind sThreshLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sGlazeLyr nil) (setq sGlazeLyr "GLASS-3|151,0.18") )
      ;;;  (if (= sGlazeLyr nil) (setq sGlazeLyr "GLASS-3|151,0.18") )
      ;;;)
      ;;;(setq sGlazeLwt (cadr (strfind sGlazeLyr ","))
      ;;;      sGlazeCol (cadr (strfind (car (strfind sGlazeLyr ",")) "|"))
      ;;;      sGlazeLyr (car  (strfind (car (strfind sGlazeLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sFrameLyr nil) (setq sFrameLyr "MULL-3|43,0.18") )
      ;;;  (if (= sFrameLyr nil) (setq sFrameLyr "MULL-3|43,0.18") )
      ;;;)
      ;;;(setq sFrameLwt (cadr (strfind sFrameLyr ","))
      ;;;      sFrameCol (cadr (strfind (car (strfind sFrameLyr ",")) "|"))
      ;;;      sFrameLyr (car  (strfind (car (strfind sFrameLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sWHeaderLyr nil) (setq sWHeaderLyr "HEADER|23,0.35") )
      ;;;  (if (= sWHeaderLyr nil) (setq sWHeaderLyr "HEADER|23,0.35") )
      ;;;)
      ;;;(setq sWHeaderLwt (cadr (strfind sWHeaderLyr ","))
      ;;;      sWHeaderCol (cadr (strfind (car (strfind sWHeaderLyr ",")) "|"))
      ;;;      sWHeaderLyr (car  (strfind (car (strfind sWHeaderLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sSashLyr nil) (setq sSashLyr "SASH-3|42,0.18") )
      ;;;  (if (= sSashLyr nil) (setq sSashLyr "SASH-3|42,0.18") )
      ;;;)
      ;;;(setq sSashLwt (cadr (strfind sSashLyr ","))
      ;;;      sSashCol (cadr (strfind (car (strfind sSashLyr ",")) "|"))
      ;;;      sSashLyr (car  (strfind (car (strfind sSashLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sSillLyr nil) (setq sSillLyr "SILL-3|23,0.25") )
      ;;;  (if (= sSillLyr nil) (setq sSillLyr "SILL-3|23,0.25") )
      ;;;)
      ;;;(setq sSillLwt (cadr (strfind sSillLyr ","))
      ;;;      sSillCol (cadr (strfind (car (strfind sSillLyr ",")) "|"))
      ;;;      sSillLyr (car  (strfind (car (strfind sSillLyr ",")) "|"))
      ;;;)
      ;;;(if (= bMetUnit T)
      ;;;  (if (= sBarsLyr nil) (setq sBarsLyr "BarsLayout|32,0.25") )
      ;;;  (if (= sBarsLyr nil) (setq sBarsLyr "BarsLayout|32,0.25") )
      ;;;)
      ;;;(setq sBarsLwt (cadr (strfind sBarsLyr ","))
      ;;;      sBarsCol (cadr (strfind (car (strfind sBarsLyr ",")) "|"))
      ;;;      sBarsLyr (car  (strfind (car (strfind sBarsLyr ",")) "|"))
      ;;;)

      (GetLayers)

      (TraceIsOn ".19a")

      ;-------------------
      ;---Layer Configure.
      ;-------------------
      (defun Descro ()
        (TraceIsOn ".Descro")
        (cond
          ((= iIndice 0)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing outside walls."
           )
          )
          ((= 1 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing inside lines for cavity walls."
           )
          )
          ((= 2 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing block walls such as Hebel..."
           )
          )
          ((= 3 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing concrete walls."
           )
          )
          ((= 4 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing partition walls."
           )
          )
          ((= 5 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing type 1 timber walls."
           )
          )
          ((= 6 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing type 2 timber walls."
           )
          )
          ((= 7 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a breakline."
           )
          )
          ((= 8 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing UP/DN arrow for stairs."
           )
          )
          ((= 9 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering 1.5mm text."
           )
          )
          ((= 10 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering 2.5mm text."
           )
          )
          ((= 11 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering 3.5mm text."
           )
          )
          ((= 12 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering 5.0mm text."
           )
          )
          ((= 13 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering 7.0mm text."
           )
          )
          ((= 14 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering text for areas."
           )
          )
          ((= 15 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when entering text for bearings and distances."
           )
          )
          ((= 16 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when hatching."
           )
          )
          ((= 17 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing jambs."
           )
          )
          ((= 18 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a threshold."
           )
          )
          ((= 19 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a door leaf."
           )
          )
          ((= 20 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a door swing."
           )
          )
          ((= 21 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a door header."
           )
          )
          ((= 22 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing glazing."
           )
          )
          ((= 23 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing mullions."
           )
          )
          ((= 24 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a window header."
           )
          )
          ((= 25 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a sash."
           )
          )
          ((= 26 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing a window sill."
           )
          )
          ((= 27 iIndice)
           (Odcl_Control_SetProperty
             AALayers_EditFile_DescLabel
             "Caption"
             "Specify layer parameters used by ArchAssist when drawing federation bars."
           )
          )
        )
      )

      (Defun InitSetUp (/)
        (TraceIsOn ".InitSetUp")
        (Odcl_Control_SetProperty AALayers_EditFile_Column1TextBox "Text" "")
        (Odcl_Control_SetProperty AALayers_EditFile_Column2TextBox "Text" "")
        ;;(Odcl_Control_SetProperty AALayers_EditFile_Column3TextBox "Text" "")
      )

      (defun c:UserL ()  (editfile "ArchAssist Layer Configuration File Editor" "abv"))      ;edit an ABV file
      (defun c:ULayer () (editfile "ArchAssist Layer Configuration File Editor" "abv"))      ;edit an ABV file

      (defun editfile (header filetype /)
        (setq olderror *error*  *error* Wallserror)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (princ (strcat "\nLayer Configuration File Editor." copyright) )
        (setq header_label header
              ftype        filetype
              cur_line     0
              #more        nil
        )

        (if (= (Odcl_LoadProject "AALayers" nil) -10)
          (progn
            (WsFlashMess1
              "Echec chargement bote de dialogue. Fichier AALayers.ods introuvable.\nSi la location a change, la mise  jour de MetricArchAssist2004.ini est indispensable."
              "Archivio AALayers.ods non trovato. Se il percorso modificasse da ultima installazione,\nl'aggiornamento di MetricArchAssist2004.ini  indispensabile."
              "Archivo AALayers.ods no ha sido encontrado. Si la ubicacion es modificada\ndesde la ultima instalacion, archivo MetricArchAssist2004.ini tendra que ser actualizado con su nueva ubicacion."
              "File AALayers.ods not found. If Path modified since last\ninstallation, update MetricArchAssist2004.ini file."
              1 5
            )
            (exit)
          )
        )

        (Odcl_LoadProject "AALayers" T)

        (defun c:EditFile_OnInitialize ()
          (InitSetUp)
          (GetFileName)
        )

        (defun c:EditFile_ListView_OnClicked (nRow nCol /)
          (setq iIndice  nRow
            rBotLeft (Odcl_Control_GetProperty
                   AALayers_EditFile_ListView
                   "BottomFromBottom"
                 )
          )
          (Setq rCol1Wid
             (Odcl_ListView_GetColWidth AALayers_EditFile_ListView 0)
          )
          (Setq rCol2Wid
             (Odcl_ListView_GetColWidth AALayers_EditFile_ListView 1)
          )
          (Setq rCol3Wid
             (Odcl_ListView_GetColWidth AALayers_EditFile_ListView 2)
          )
          (Setq sVal1 (Odcl_ListView_GetItemText
                        AALayers_EditFile_ListView
                        iIndice
                        0
                      )
          )
          (Setq sVal2 (Odcl_ListView_GetItemText
                        AALayers_EditFile_ListView
                        iIndice
                        1
                      )
          )
          (Setq sVal3 (Odcl_ListView_GetItemText
                        AALayers_EditFile_ListView
                        iIndice
                        2
                      )
          )
          (Odcl_Control_SetProperty
            AALayers_EditFile_Column1TextBox
            "Text"
            sVal1
          )
          (Odcl_Control_SetProperty
            AALayers_EditFile_Column2TextBox
            "Text"
            sVal2
          )
          ;;;(Odcl_Control_SetProperty
          ;;;  AALayers_EditFile_Column3TextBox
          ;;;  "Text"
          ;;;  sVal3
          ;;;)

          (Odcl_ComboBox_SelectString AALayers_EditFile_LWTComboBox (strcat (rtos (read sVal3) 2 2) " mm"))

          (Odcl_Control_SetProperty
            AALayers_EditFile_ColPictureBox
            "BackColor"
            (read sVal2)
          )
          (Descro)
        )

        ;;;(defun c:EditFile_ListView_OnSetFocus ()
        ;;; ;(Setq rValu (Odcl_ListView_GetSelectedItems AALayers_EditFile_ListView))
        ;;;  (Setq iCnt (car (Odcl_ListView_GetSelectedNths AALayers_EditFile_ListView)))
        ;;;)

        (defun c:EditFile_LWTComboBox_OnSelChanged (nSelection sSelText /)
          (setq sLwt1 (car (strfind sSelText " mm")))
          (princ)
        )

        ;; Usage for setting ACI: (acad_truecolordlg 152) Usage for True color: (acad_truecolordlg (cons 420 2594))
        ;; Usage for Color book or Pantone: (acad_truecolordlg (cons 430 "PANTONE(R) a & i-cotton$PANTONE 11-4800TC"))
        ;; Usage for getting:
        ;; ((62 . 53)) or ((62 . 142) (420 . 2797013)) or
        ;; ((62 . 31) (420 . 14599321) (430 . "PANTONE(R) a & i-cotton$PANTONE 14-1113 TC"))

        (defun c:EditFile_ColPictureBox_OnClicked ()
          (setq iVal2 (read sVal2))
          (setq temp_color (acad_colordlg iVal2 nil))         ;acad_colordlg accepts integers
          (if (numberp temp_color)                    ;Si c'est un nombre
            (setq iVal2 temp_color)                   ;La nouvelle valeur
          )
          (setq sVal2 (rtos iVal2))                   ;change to string
          (Odcl_Control_SetProperty
            AALayers_EditFile_ColPictureBox
            "BackColor"
            iVal2                             ;"BackColor" accepts integers
          )
          (Odcl_Control_SetProperty
            AALayers_EditFile_Column2TextBox
            "Text"
            sVal2                         ;"Text" accepts strings
          )
        )

        (defun c:EditFile_Column2TextBox_OnEditChanged (sText /)
          (Odcl_Control_SetProperty
            AALayers_EditFile_ColPictureBox
            "BackColor"
            (read sText)                      ;"BackColor" accepts integers
          )
        )

        (defun c:EditFile_ModifyTextButton_OnClicked ()
          (ModifyCurrent)
        )

        (defun c:EditFile_SaveTB_OnClicked ()
          (CreateFile)
          (ReadWallDat)
          (princ)
        )

        (defun c:EditFile_about_OnClicked ()
          (_about)
        )

        (Odcl_Form_Show AALayers_EditFile)

        (GetLayers)

        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun GetFileName (/ fl eof cnt tmpfile sortfile)
        (TraceIsOn ".GetFileName")
        (if (setq filename hshfile2)
          (progn
            (setq file_lines nil
              InfoLigne nil
              lInCol1 nil
              lInCol2 nil
              lInCol3 nil
              rCol1Wid nil
              rCol2Wid nil
              rCol3Wid nil
            )

            (GetLayerINI)

            (setq file_lines
               (list sBrkWall1Lyr    sBrkWall2Lyr    sBlkWall1Lyr
                 sConWall1Lyr    sParWall1Lyr    sTmbWall1Lyr
                 sTmbWall2Lyr    sSymbol25Lyr    sSymbol35Lyr
                 sText0Lyr       sText1Lyr       sText2Lyr       sText3Lyr
                 sText4Lyr       sTAreaLyr       sTBndryLyr
                 sHatch1Lyr      sJambLyr    sThreshLyr
                 sLeafLyr    sSwingLyr       sHeaderLyr
                 sGlazeLyr       sFrameLyr       sWHeaderLyr
                 sSashLyr    sSillLyr    sBarsLyr
                )
            )

            (setq TempStore file_lines)

            (foreach item file_lines
          (setq InfoLigne (car (strfind item "|"))
                lInCol1   (append lInCol1 (list InfoLigne))                   ;liste dans col 1
                InfoLigne (cadr (strfind item "|"))
                InfoLigne (strfind InfoLigne ",")
                lInCol2   (append lInCol2 (list (car InfoLigne)))                 ;liste dans col 2
                lInCol3   (append lInCol3 (list (cadr InfoLigne)))                ;liste dans col 3
          )
            )

            (setq column1width
                   (Odcl_ListView_CalcColWidth
                     AALayers_EditFile_ListView
                     "Layer name       "
                   )
              column2width 50
              ;;(Odcl_ListView_CalcColWidth
              ;;AALayers_EditFile_ListView
              ;;"Colour"
              ;;)
              column3width
                   65
                   ;;(Odcl_ListView_CalcColWidth
                   ;;AALayers_EditFile_ListView
                   ;;"Lineweight"
                   ;;)
            )

            (Odcl_ListView_Clear AALayers_EditFile_ListView)
            (Odcl_ListView_AddColumns
          AALayers_EditFile_ListView
          (list 0 "Layer name" 0 column1width)
          (list 1 "Colour" 0 column2width)
          (list 2 "Lineweight" 0 column3width)
            )

            (Odcl_ListView_FillList
          AALayers_EditFile_ListView
          (list lInCol1           ;add column 1 items
                lInCol2           ;add column 2 items
                lInCol3           ;add column 3 items
          )
            )
          )
        )
      )

      ;;;dismiss the dialogue box and create the file.
      (defun CreateFile (/ cnt fl)
        (TraceIsOn ".CreateFile")
        (dos_setini
          "Miscellany"
          "BrkWall1"
          (nth 0 file_lines)
          Hshfile2
        )                 ;0
        (dos_setini
          "Miscellany"
          "BrkWall2"
          (nth 1 file_lines)
          Hshfile2
        )                 ;1
        (dos_setini
          "Miscellany"
          "BlkWall1"
          (nth 2 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "ConWall1"
          (nth 3 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "ParWall1"
          (nth 4 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "TmbWall1"
          (nth 5 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "TmbWall2"
          (nth 6 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "Symbol25"
          (nth 7 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "Symbol35"
          (nth 8 file_lines)
          Hshfile2
        )

        (dos_setini
          "Miscellany"
          "Text0Lyr"
          (nth 9 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "Text1Lyr"
          (nth 10 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "Text2Lyr"
          (nth 11 file_lines)
          Hshfile2
        )                 ;11
        (dos_setini
          "Miscellany"
          "Text3Lyr"
          (nth 12 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "Text4Lyr"
          (nth 13 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "TAreaLyr"
          (nth 14 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "TBndryLyr"
          (nth 15 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "Hatch1Lyr"
          (nth 16 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "JambLyr"
          (nth 17 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "ThreshLyr"
          (nth 18 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "LeafLyr"
          (nth 19 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "SwingLyr"
          (nth 20 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "HeaderLyr"
          (nth 21 file_lines)
          Hshfile2
        )                 ;21
        (dos_setini
          "Miscellany"
          "GlazeLyr"
          (nth 22 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "FrameLyr"
          (nth 23 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "WHeaderLyr"
          (nth 24 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "SashLyr"
          (nth 25 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "SillLyr"
          (nth 26 file_lines)
          Hshfile2
        )
        (dos_setini
          "Miscellany"
          "BarsLyr"
          (nth 27 file_lines)
          Hshfile2
        )
        (Odcl_Form_Close AALayers_EditFile)
      )

      (defun ReadWallDat (/)
        (TraceIsOn ".ReadWallDat")
        (setq fiche1   nil
              lMursDat nil
              iPosit1  nil
              iPosit2  nil
              TmpBrk   nil
              TmpBrk3  nil
        )
        (if (= bMetUnit T)  ;Section metrique
          (setq fInfoMurs "MetricWallParts.Dat")
          (setq fInfoMurs "MetricWallParts.Dat")
        )
        (if (/= (setq fiche1 (open (strcat Chemin fInfoMurs) "r")) nil)
          (progn
            (while (setq ligne (read-line fiche1))
              (setq lMursDat (cons ligne lMursDat))
            )
            (close fiche1)

            (setq iCnt  0
                  iCnt2 0
            )

            ;;;Avant changement dans TempStore=("BrickWork1|1,0.35" "BrickWork2|7,0.25" "BlockWork1|82,0.35)"
            ;;;"Concrete1|82,0.35" "Partition1|60,0.35" "Timber1|41,0.35" "Timber2|...

            ;;;et apres changement dans file_lines=("LaBrique|1,0.35" "BrickWork2|7,0.25" "BlockWork1|82,0.35)"
            ;;;"BetonArme|82,0.35" "Partition1|60,0.35" "Timber1|41,0.35" "Timber2|...

            (while (< iCnt (length TempStore))                            ;Tant qu'on en trouve
              (setq sFormer1 (car (strfind (nth iCnt TempStore) "|"))                 ;Nom du calque
                    sFormer0 (cadr (strfind (nth iCnt TempStore) "|"))
                    sFormer0 (strfind sFormer0 ",")
                    sFormer2 (car sFormer0)                               ;Couleur
                    sFormer3 (cadr sFormer0)  ;Epaisseur Exist.
                    sNewer1  (car (strfind (nth iCnt file_lines) "|"))                ;Nom du calque
                    sNewer0  (cadr (strfind (nth iCnt file_lines) "|"))
                    sNewer0  (strfind sNewer0 ",")
                    sNewer2  (car sNewer0)                                ;Couleur
                    sNewer3  (cadr sNewer0)                               ;Epaisseur Proposee
              )

              (while (< iCnt2 (length lMursDat))                          ;Pour chaque article dans MetricWallParts.Dat
                (setq TmpBrk (DeStrAListe (nth iCnt2 lMursDat) ";"))
                (if (= (nth 1 TmpBrk) "DoubleW")
                  (progn
                    (setq TmpBrk2 (strfind (nth 10 TmpBrk) "|")
                      lSt1    (Replace_Nth 10 (cadr TmpBrk2) TmpBrk)
                      TmpBrk  (add_n 10 lSt1 "|")
                      TmpBrk  (add_n 10 TmpBrk (car TmpBrk2))
                      bDouble T
                    )
                  )
                  (setq bDouble nil
                    TmpBrk2 nil
                    lst1 nil
                  )
                )

                (setq iPosit1 (what_nth sFormer1 TmpBrk))
                (if (/= iPosit1 -1)                                   ;Si c'est trouve
                  (progn
                    (setq TmpBrk (Replace_Nth iPosit1 sNewer1 TmpBrk))                ;Remplacer nom du calque
                    (setq TmpBrk (Replace_Nth (+ iPosit1 1) sNewer2 TmpBrk))              ;Couleur et
                    (setq TmpBrk (Replace_Nth (+ iPosit1 2) sNewer3 TmpBrk))              ;Epaisseur Proposee
                  )
                )

                (setq TmpBrk3 (reverse TmpBrk)
                  iPosit2 (what_nth sFormer1 TmpBrk3)
                )

                (if (/= iPosit2 -1)                                   ;Si on trouve plus d'un
                  (progn
                    (setq TmpBrk3 (Replace_Nth iPosit2 sNewer1 TmpBrk3))
                    (setq TmpBrk3 (Replace_Nth (- iPosit2 1) sNewer2 TmpBrk3))
                    (setq TmpBrk3 (Replace_Nth (- iPosit2 2) sNewer3 TmpBrk3))
                    (setq TmpBrk (reverse TmpBrk3))
                  )
                )

                ;;; E.g. (strdcat '("One" "Two" "Three") ",") Returns -> "One,Two,Three"

                (setq TmpBrk (strdcat TmpBrk ";"))

                (if bDouble
                  (setq TmpBrk (strrep TmpBrk ";|;" "|"))
                  (progn
                    (if (= nil (cadr (strfind TmpBrk "|")))
                      (setq TmpBrk (strcat TmpBrk "|"))
                    )
                  )
                )

                (setq lMursDat (Replace_Nth iCnt2 TmpBrk lMursDat))
                (setq iCnt2 (1+ iCnt2))

                (cassure "PauseCafe4: lMursDat")
              )

              (setq iCnt  (1+ iCnt)
                    iCnt2 0
              )
            )

            (setq fiche1 (open (strcat Chemin fInfoMurs) "w"))
            (foreach article lMursDat                 ;Brick Veneer; Cavity Brick; Concrete 190...
              (wlf article fiche1)
              ;(princ article)
            )
            (close fiche1)
            (AlphaSort (strcat Chemin "MetricWallParts.Dat"))
          )
          (progn
            (dos_msgbox
               "Error:  MetricWallParts.Dat file not found."
               "ArchAssist Message"
               1
               5
            )
            (exit)
            (princ)
          )
        )
        (princ)
      )

      ;;;replace the current highlighted line in the file list with the edit box line.
      (defun ModifyCurrent ()
        (setq file_lines
           (replace_n
             iIndice
             (strcat (Odcl_Control_GetProperty
                   AALayers_EditFile_Column1TextBox
                   "Text"
                 )
                 "\|"
                 (Odcl_Control_GetProperty
                   AALayers_EditFile_Column2TextBox
                   "Text"
                 )
                 "\,"
                (car (strfind
                 (Odcl_Control_GetProperty
                   AALayers_EditFile_LWTComboBox
                   "Text"
                 )
                " mm"))
             )
             file_lines
           )
        )
        (UpdateFileBox)           ;update file list
      )

      ;;;update the file contents list box.
      (defun UpdateFileBox ()
        (TraceIsOn ".UpdateFileBox")
        (setq lInCol1 nil
              lInCol2 nil
              lInCol3 nil
        )

        (foreach item file_lines
          (setq InfoLigne (car (strfind item "|"))
            lInCol1   (append lInCol1 (list InfoLigne))                   ;liste dans col 1
            InfoLigne (cadr (strfind item "|"))
            InfoLigne (strfind InfoLigne ",")
            lInCol2   (append lInCol2 (list (car InfoLigne)))             ;liste dans col 2
            lInCol3   (append lInCol3 (list (cadr InfoLigne)))                ;liste dans col 3
          )
        )

        (Odcl_ListView_Clear AALayers_EditFile_ListView)
        (Odcl_ListView_AddColumns
          AALayers_EditFile_ListView
          (list 0 "Layer name" 0 column1width)
          (list 1 "Colour" 0 column2width)
          (list 2 "Lineweight" 0 column3width)
        )

        (Odcl_ListView_FillList
          AALayers_EditFile_ListView
          (list lInCol1           ;add column 1 items
            lInCol2           ;add column 2 items
            lInCol3           ;add column 3 items
          )
        )
      )

      (TraceIsOn ".19b")

      ;--------------
      ;---Room Sizes.
      ;--------------
      (defun c:PRS (/) (DimRoom))

      (defun DimRoom (/)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)

        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )

        (princ (strcat "\nPrint Room Sizes." copyright))
        (setq pCnr1     (upoint 1 "" "Specify first corner point" nil nil)
              pCnr2     (upoint 1 "" "Specify opposite corner point" nil pCnr1)
              sDispUn   (ukword 1
                        "m miLlimetres"
                        "Units shown in m or [miLlimetres] "
                        "m"
                    )
              lDesCoins (CornersGet pCnr1 pCnr2)
              pTxtIns   (mid_pt pCnr1 pCnr2)
              rLong     (- (caadr ldescoins) (caar ldescoins))
              rHaut     (- (cadadr ldescoins) (cadar ldescoins))
        )
        (if (= sDispUn "miLlimetres")
          (TextMake pTxtIns
                (rtos (* (getvar "DIMSCALE") 2.0))
                0
                1
                2
                (strcat (rtos rLong 2 0) " X " (rtos rHaut 2 0))
                nil
          )
          (TextMake pTxtIns
                (rtos (* (getvar "DIMSCALE") 2.0))
                0
                1
                2
                (strcat (rtos (/ rLong 1000) 2 1)
                    " X "
                    (rtos (/ rHaut 1000) 2 1)
                )
                nil
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (setq ocmdecho (getvar "cmdecho"))(setvar "cmdecho" 0) ;;### TM added to prevent echos
      (setq otexts (getvar "TEXTSTYLE"))
      (if (= (tblsearch "STYLE" "AAISO") nil)
        (command "_.style" "AAISO" "isocp.shx" "0" "1" "0" "_N" "_N")
      )
      (setvar "cmdecho" ocmdecho) ;;### TM added

      ;;(setvar "TEXTSTYLE" otexts)

      ;------------------------------------
      ;---DES MURS ET PUIS ENCORE DES MURS.
      ;------------------------------------
      ;A layer name (group code 1003).
      ;An entity handle (group code 1005).
      ;A 3D World space position (group code 1011).
      ;A 3D World space displacement (group code 1012).
      ;A 3D World space direction (group code 1013).
      (defun SetWallInfoTag (apid en / )
        (TraceIsOn ".SetWallInfoTag")
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 "AAWALLS")             ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1000 "Dummy")               ;Dummy
            (cons 1000 "Dummy")               ;Dummy

          )
        )
      )

      (defun SetJambInfoTag (apid en / )
        (TraceIsOn ".SetJambInfoTag")
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 "AAJAMBS")             ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1000 "Dummy")               ;Dummy
            (cons 1000 "Dummy")               ;Dummy
          )
        )
      )

      ;sort alphabetically
      (defun AlphaSort (filename /)
        (dos_exewait
          (strcat "SORT "
              (chr 34)
              filename
              (chr 34)
              " /O "
              (chr 34)
              Chemin
              "Walias.tmp"
              (chr 34)
          )
        )
        (dos_delete filename)
        (dos_rename (strcat Chemin "Walias.tmp") filename)
      )

      (TraceIsOn ".19aa")

      (defun AliasList (/)
        (TraceIsOn ".AliasList")
        (setq file1 nil)
        (setq file3 (open (strcat Chemin "Walias.odx") "w"))
        (if (= bMetUnit T)            ;Section metrique
          (setq fInfoMurs "MetricWallParts.Dat")
          (setq fInfoMurs "MetricWallParts.Dat")
        )
        (if (/= (setq file1 (open (strcat Chemin fInfoMurs) "r")) nil)
          (progn
            (setq lMursLayers nil)
            (while (setq ligne (read-line file1))
              (setq lMursLayers (cons ligne lMursLayers))
            )
            (close file1)
            (foreach item lMursLayers
              (setq LineInfo (DeStrAListe (cadr (strfind item "->")) ";"))

              (wlf (strcat "(Defun c:" (nth 0 LineInfo) " ()") file3)
              (wlf "(setq olderror *error*  *error* Wallserror)" file3)
              (wlf "(Wallsgather_info)" file3)

              (if (= bMetUnit T)      ;Section metrique.
                (progn
                  (setq Width1   (atof (nth 2 LineInfo))
                    Width2   (atof (nth 4 LineInfo))
                    Cavite   (atof (nth 3 LineInfo))
                    aOverall (rtos (+ Width1 Width2 Cavite) 2 0)
                    sWidth1  (rtos Width1 2 0)
                    sWidth2  (rtos Width2 2 0)
                    sCavite  (rtos Cavite 2 0)
                  )
                )
                (progn
                  (setq Width1   (/ (atof (nth 2 LineInfo)) 25.4)
                    Width2   (/ (atof (nth 4 LineInfo)) 25.4)
                    Cavite   (/ (atof (nth 3 LineInfo)) 25.4)
                    aOverall (rtos (+ Width1 Width2 Cavite) 3 2)
                    sWidth1  (rtos Width1 3 2)
                    sWidth2  (rtos Width2 3 2)
                    sCavite  (rtos Cavite 3 2)
                  )
                )
              )

              (wlf (strcat "(setq sTypes   " (etos (nth 1 LineInfo)))
                   file3
              )
              (wlf (strcat "      sWidth1  " (etos sWidth1)) file3)
              (wlf (strcat "      sCavite  " (etos sCavite)) file3)
              (wlf (strcat "      sWidth2  " (etos sWidth2)) file3)
              (wlf (strcat "      sOverall " (etos aOverAll)) file3)
              (wlf (strcat "      sLay1    " (etos (nth 5 LineInfo)))
                   file3
              )
              (wlf (strcat "      sCol1    " (etos (nth 6 LineInfo)))
                   file3
              )
              (wlf (strcat "      sLwt1    " (etos (nth 7 LineInfo)))
                   file3
              )
              (wlf (strcat "      sLay2    " (etos (nth 8 LineInfo)))
                   file3
              )
              (wlf (strcat "      sCol2    " (etos (nth 9 LineInfo)))
                   file3
              )
              (wlf (strcat "      sLwt2    "
                       (etos (car (strfind (nth 10 LineInfo) "|")))
                   )
                   file3
              )
              (wlf "      WLAYER   sLay1" file3)
              (wlf "      DLWIDTH1 (read sWidth1)" file3)
              (wlf "      DLWIDTH2 (read sWidth2)" file3)
              (wlf ")" file3)

              (if (= (nth 1 LineInfo) "DoubleW")
                (progn
                  (setq MoreInfo (DeStrAListe (cadr (strfind item "|")) ";"))
                  (wlf (strcat "(setq sLay3 " (etos (nth 0 MoreInfo))) file3)
                  (wlf (strcat "      sCol3 " (etos (nth 1 MoreInfo))) file3)
                  (wlf (strcat "      sLwt3 " (etos (nth 2 MoreInfo))) file3)
                  (wlf (strcat "      sLay4 " (etos (nth 3 MoreInfo))) file3)
                  (wlf (strcat "      sCol4 " (etos (nth 4 MoreInfo))) file3)
                  (wlf (strcat "      sLwt4 " (etos (nth 5 MoreInfo))) file3)
                  (wlf "      bTypes T" file3)
                  (wlf ")" file3)
                )
              )
              (wlf "(setvar \"TRACEWID\" (read sOverAll))" file3)
              (wlf "(if (= nil dlTrematzb)(setq dlTrematzb \"Top\") )"
                   file3
              )
              (wlf "(if (= nil dlTremaabk)(setq dlTremaabk \"ON\") )"
                   file3
              )
              (wlf "(if (= nil dlTremacap)(setq dlTremacap \"None\") )"
                   file3
              )
              (wlf "(WallLaunch)" file3)
              (wlf "(setq *error* OLDERROR)(reset nil)(resset varlst)"
                   file3
              )
              (wlf ")" file3)
            )
          )
          (progn (dos_msgbox
               "Error:  MetricWallParts.Dat file not found."
               "ArchAssist Message"
               1
               5
             )
             (exit)
             (princ)
          )
        )
        (close file3)
      )

      (AliasList)

      (TraceIsOn ".19bb")

      (setq Hshfile4 (findfile "Walias.odx"))
      (if (= Hshfile4 nil)
        (princ "Missing file is necessary to some routines.")
        (load Hshfile4)
      )

      (defun c:W () (SjgWalls) (princ))

      (defun ReadLayList (/)
        (TraceIsOn ".ReadLayList")
        (setq lMursLayers nil)
        (setq file1 nil)
        (if (= bMetUnit T)            ;Section metrique
          (setq fInfoMurs "MetricWallParts.Dat")
          (setq fInfoMurs "MetricWallParts.Dat")
        )
        (if (/= (setq file1 (open (strcat Chemin fInfoMurs) "r")) nil)
          (progn
            (while (setq ligne (read-line file1))
              (setq lMursLayers (cons ligne lMursLayers))
            )
            (close file1)
            (Odcl_ListBox_Clear Walls_InfoWall_WallListBox)
            (foreach item lMursLayers
              (Odcl_ListBox_AddString
                Walls_InfoWall_WallListBox
                (car (strfind item "->"))
              )
            )
          )
          (progn
            (dos_msgbox
               "Error:  MetricWallParts.Dat file not found."
               "ArchAssist Message"
               1
               5
            )
            (exit)
            (princ)
          )
        )
      )

      (TraceIsOn ".19cc")

      (defun XtractWallsList ()
        (TraceIsOn ".XtractWallsList")
        (setq SelWall (nth pickf lMursLayers))
        (XtractWallsInfo SelWall)
        (Odcl_Control_SetProperty
          Walls_InfoWall_NameTextBox
          "Text"
          sName
        )
        (Odcl_Control_SetProperty
          Walls_InfoWall_ShortTextBox
          "Text"
          sShortCut
        )
        (Odcl_Control_SetProperty
          Walls_InfoWall_Width1TextBox
          "Text"
          sWidth1
        )
        (Odcl_Control_SetProperty
          Walls_InfoWall_CavityTextBox
          "Text"
          sCavite
        )
        (Odcl_Control_SetProperty
          Walls_InfoWall_OverallTextBox
          "Text"
          sOverall
        )
        (Odcl_Control_SetProperty
          Walls_InfoWall_Width2TextBox
          "Text"
          sWidth2
        )

        (if (= bTypes T)
          (progn
            (Odcl_Control_SetProperty Walls_InfoWall_Frame1 "Visible" T)
            (Odcl_Control_SetProperty Walls_InfoWall_Label5 "Visible" T)
            (Odcl_Control_SetProperty Walls_InfoWall_Label7 "Visible" T)
            (Odcl_Control_SetProperty Walls_InfoWall_Label8 "Visible" T)
            (Odcl_Control_SetProperty
              Walls_InfoWall_Width2TextBox
              "Visible"
              T
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ComboBoxLay3
              "Visible"
              T
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ComboBoxLay4
              "Visible"
              T
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ColorBox3
              "Visible"
              T
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ColorBox4
              "Visible"
              T
            )
            (Odcl_Control_SetProperty Walls_InfoWall_LWBox3 "Visible" T)
            (Odcl_Control_SetProperty Walls_InfoWall_LWBox4 "Visible" T)
          )
          (progn
            (Odcl_Control_SetProperty Walls_InfoWall_Frame1 "Visible" nil)
            (Odcl_Control_SetProperty
              Walls_InfoWall_Label5
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_Label7
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_Label8
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_Width2TextBox
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ComboBoxLay3
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ComboBoxLay4
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ColorBox3
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_ColorBox4
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_LWBox3
              "Visible"
              nil
            )
            (Odcl_Control_SetProperty
              Walls_InfoWall_LWBox4
              "Visible"
              nil
            )
          )
        )
      )

      (TraceIsOn ".19dd")

      (defun XtractWallsInfo (string1 /)
        (TraceIsOn ".XtractWallsInfo")
        (setq LineInfo  (DeStrAListe (cadr (strfind SelWall "->")) ";")       ;A partir du fichier
              sName     (car (strfind SelWall "->"))
              sShortCut (nth 0 LineInfo)
              sTypes    (nth 1 LineInfo)
              sWidth1   (nth 2 LineInfo)
              sCavite   (nth 3 LineInfo)
              sWidth2   (nth 4 LineInfo)
              sLay1     (nth 5 LineInfo)
              sCol1     (nth 6 LineInfo)
              sLwt1     (nth 7 LineInfo)
              sLay2     (nth 8 LineInfo)
              sCol2     (nth 9 LineInfo)
              sLwt2     (car (strfind (nth 10 LineInfo) "|"))
        )

        (if (= bMetUnit T)            ;Section metrique.
          (progn
            (setq sOverall
              (rtos (+ (atof sWidth1) (atof sWidth2) (atof sCavite))
                    2
                    0
              )
            )
          )
          (progn
            (setq sWidth1  (/ (atof sWidth1) 25.4)
                  sWidth2  (/ (atof sWidth2) 25.4)
                  sCavite  (/ (atof sCavite) 25.4)
                  sOverall (rtos (+ sWidth1 sWidth2 sCavite) 3 2)
                  sWidth1  (rtos sWidth1 3 2)
                  sWidth2  (rtos sWidth2 3 2)
                  sCavite  (rtos sCavite 3 2)
            )
          )
        )                 ;Remplir boite de dialogue.

        (setq sTCol1 sCol1)           ;Temp storage of string
        (if (= (type (read sCol1)) 'INT)
          (setq iTCol1 sCol1
                bINT   T
          )
          (setq bINT nil)
        )

        (setq iTCol1 (AACol2ACI sCol1))   ;temp color number

        (if (= bINT T)
          (setq sCol1 iTCol1)
          (setq sCol1 sTCol1)
        )

        (if (= (type sCol1) 'STR)
          (progn
            (Odcl_ComboBox_SelectString Walls_InfoWall_ColorBox1 sCol1)
          )
          (progn
            (Odcl_ComboBox_AddColor Walls_InfoWall_ColorBox1 sCol1)
            (Odcl_ComboBox_SelectString
              Walls_InfoWall_ColorBox1
              (strcat "Color " (etos sCol1))
            )
          )
        )

        (setq sTCol2 sCol2)           ;Temp storage of string
        (if (= (type (read sCol2)) 'INT)
          (setq iTCol2 sCol2
                bINT   T
          )
          (setq bINT nil)
        )                 ;Is it an integer?

        (setq iTCol2 (AACol2ACI sCol2))   ;temp color number

        (if (= bINT T)
          (setq sCol2 iTCol2)
          (setq sCol2 sTCol2)
        )
        (if (= (type sCol2) 'STR)
          (progn
            (Odcl_ComboBox_SelectString Walls_InfoWall_ColorBox2 sCol2)
          )
          (progn
             (Odcl_ComboBox_AddColor Walls_InfoWall_ColorBox2 sCol2)
             (Odcl_ComboBox_SelectString
               Walls_InfoWall_ColorBox2
               (strcat "Color " (etos sCol2))
             )
          )
        )

        (Odcl_ComboBox_SelectString
          Walls_InfoWall_LWBox1
          (strcat sLwt1 " mm")
        )
        (Odcl_ComboBox_SelectString
          Walls_InfoWall_LWBox2
          (strcat sLwt2 " mm")
        )

        (if (= sTypes "DoubleW")
          (progn
            (setq bTypes T)
            (setq MoreInfo (DeStrAListe (cadr (strfind SelWall "|")) ";")
                  sLay3    (nth 0 MoreInfo)
                  sCol3    (nth 1 MoreInfo)
                  sLwt3    (nth 2 MoreInfo)
                  sLay4    (nth 3 MoreInfo)
                  sCol4    (nth 4 MoreInfo)
                  sLwt4    (nth 5 MoreInfo)
            )

            (setq sTCol3 sCol3)       ;Temp storage of string
            (if (= (type (read sCol3)) 'INT)
              (setq iTCol3 sCol3
                    bINT   T
              )
              (setq bINT nil)
            )                 ;Is it an integer?
            (setq iTCol3 (AACol2ACI sCol3))   ;temp color number
            (if (= bINT T)
              (setq sCol3 iTCol3)
              (setq sCol3 sTCol3)
            )
            (if (= (type sCol3) 'STR)
              (progn
                (Odcl_ComboBox_SelectString Walls_InfoWall_ColorBox3 sCol3)
              )
              (progn
                (Odcl_ComboBox_AddColor Walls_InfoWall_ColorBox3 sCol3)
                (Odcl_ComboBox_SelectString
                  Walls_InfoWall_ColorBox3
                  (strcat "Color " (etos sCol3))
                )
              )
            )

            (setq sTCol4 sCol4)       ;Temp storage of string
            (if (= (type (read sCol4)) 'INT)
              (setq iTCol4 sCol4
                    bINT   T
              )
              (setq bINT nil)
            )                 ;Is it an integer?
            (setq iTCol4 (AACol2ACI sCol4))   ;temp color number
            (if (= bINT T)
              (setq sCol4 iTCol4)
              (setq sCol4 sTCol4)
            )
            (if (= (type sCol4) 'STR)
              (progn
                (Odcl_ComboBox_SelectString Walls_InfoWall_ColorBox4 sCol4)
              )
              (progn
                (Odcl_ComboBox_AddColor Walls_InfoWall_ColorBox4 sCol4)
                (Odcl_ComboBox_SelectString
                  Walls_InfoWall_ColorBox4
                  (strcat "Color " (etos sCol4))
                )
              )
            )
            (Odcl_ComboBox_SelectString
              Walls_InfoWall_LWBox3
              (strcat sLwt3 " mm")
            )
            (Odcl_ComboBox_SelectString
              Walls_InfoWall_LWBox4
              (strcat sLwt4 " mm")
            )
          )
          (progn
            (setq bTypes nil)
            (setq sLay3 "nil"
                         ;        sCol3 ""
                         ;        sLwt3 ""
                  sLay4 "nil"
                         ;        sCol4 ""
                         ;        sLwt4 ""
            )
          )
        )
      )

      (TraceIsOn ".19ee")

      ;---this function retrieves all the blocks and adds their names to the LayerListBox.
      (defun AddLayersToListBox (/)
        (TraceIsOn ".AddLayersToListBox")
        (Odcl_ComboBox_Clear Walls_InfoWall_ComboBoxLay1)
        (Odcl_ComboBox_Clear Walls_InfoWall_ComboBoxLay2)
        (Odcl_ComboBox_Clear Walls_InfoWall_ComboBoxLay3)
        (Odcl_ComboBox_Clear Walls_InfoWall_ComboBoxLay4)

        (setq LayrInfo (ai_table "Layer" 4)
              lLay1    (Remove_Dup (cons sLay1 LayrInfo))
              lLay2    (Remove_Dup (cons sLay2 LayrInfo))
              lLay3    (Remove_Dup (cons sLay3 LayrInfo))
              lLay4    (Remove_Dup (cons sLay4 LayrInfo))
              lLay1    (acad_strlsort lLay1)
              lLay2    (acad_strlsort lLay2)
              lLay3    (acad_strlsort lLay3)
              lLay4    (acad_strlsort lLay4)
        )

        (foreach item lLay1
          (Odcl_ComboBox_AddString Walls_InfoWall_ComboBoxLay1 item)
        )
        (Odcl_ComboBox_SelectString
          Walls_InfoWall_ComboBoxLay1
          sLay1
        )

        (foreach item lLay2
          (Odcl_ComboBox_AddString Walls_InfoWall_ComboBoxLay2 item)
        )
        (Odcl_ComboBox_SelectString
          Walls_InfoWall_ComboBoxLay2
          sLay2
        )

        (foreach item lLay3
          (Odcl_ComboBox_AddString Walls_InfoWall_ComboBoxLay3 item)
        )
        (Odcl_ComboBox_SelectString
          Walls_InfoWall_ComboBoxLay3
          sLay3
        )

        (foreach item lLay4
          (Odcl_ComboBox_AddString Walls_InfoWall_ComboBoxLay4 item)
        )
        (Odcl_ComboBox_SelectString
          Walls_InfoWall_ComboBoxLay4
          sLay4
        )
      )

      (defun dl_cnv ( / ent spt dist)
        (while (setq ent (entsel "\nSelect object to convert to a wall: "))
          (if (/= nil ent)
            (progn
              (SetWallInfoTag sAppNameAA (car ent))
              (setq spt (upoint 1 "" "Specify point on side to offset" nil nil))
              (if (wcmatch (dxf 0 (entget (car ent))) "*POLYLINE")
                (setq bPline T)
                (setq bPline nil)
              )

              ;;(command "_.CHPROP" ent "" "_LA" sLAy1 "")
              (vla-put-layer (vlax-Ename->vla-Object (car ent)) sLAy1)

              (command "_.offset" (read sWidth1) ent spt "")              ;run the offset command
              (setq ew2 (entlast))
              (SetWallInfoTag sAppNameAA ew2)
              ;;(command "_.CHPROP" ew2 "" "_LA" sLAy2 "")
              (vla-put-layer (vlax-Ename->vla-Object ew2) sLAy2)

              (if (= sTypes "DoubleW")
                (progn
                  (command "_.offset"
                           (+ (read sWidth1) (read sCavite))
                           ent
                           spt
                           ""
                  )
                  (setq ew3 (entlast))
                  (SetWallInfoTag sAppNameAA ew3)
                  ;;(command "_.CHPROP" ew3 "" "_LA" sLAy4 "")
                  (vla-put-layer (vlax-Ename->vla-Object ew3) sLAy4)

                  (command "_.offset" (read sOverall) ent spt "")
                  (setq ew4 (entlast))
                  (SetWallInfoTag sAppNameAA ew4)
                  ;;(command "_.CHPROP" ew4 "" "_LA" sLAy3 "")
                  (vla-put-layer (vlax-Ename->vla-Object ew4) sLAy3)

                  (if bPline
                    (progn
                          (command "_.EXPLODE" ew3)
                          (command "_.EXPLODE" ew4)
                    )
                  )

                )
              )
              (if bPline
                (progn
                  (command "_.EXPLODE" ent)
                  (command "_.EXPLODE" ew2)
                )
              )
            )
          )
        )
        (princ)
      )

      (defun SjgWalls ( / )
        (setq olderror *error*  *error* Wallserror)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (princ (strcat "\nWalls." copyright))
        (setq lMursLayers nil
              MurDOUBLE   nil
        )

        (GetLayers)

        (if (= bMetUnit T)                               ;Section metrique
          (progn
            (if (= nil dlTrematzb)                         ;Si justif. est nulle, adopte
              (setq dlTrematzb "Top"  dlTremaaln 1   dlTremaosd -135.0)         ;formerly "Zero"
            )
            (if (= sOverall nil)                           ;Si nul
               (progn (setq sOverall "270"  bFirst T) (setvar "TRACEWID" 270))
               (setq bFirst nil)
            )
            (if (= bFirst nil)
              (progn
                (prompt (strcat "\nCurrent settings: " sOverall "mm wall"))
                (prompt (strcat " Break=" dlTremaabk))
                (prompt (strcat " Cappings=" dlTremacap))
                (prompt (strcat " Justification=" dlTrematzb))
                (setq CorN (ukword 1 "Current New" "Use Current wall or [New] " "Current"))
              )
            )
          )
          (progn
            (if (= nil dlTrematzb)
              (setq dlTrematzb "Top"  dlTremaaln 1   dlTremaosd -5)         ;formerly "Zero"
            )
            (if (= sOverall nil)
              (progn (setq sOverall "4"  bFirst T) (setvar "TRACEWID" 4) )      ;inches
              (setq bFirst nil)
            )
            (if (= bFirst nil)
              (progn
                (prompt (strcat "\nCurrent settings: " sOverall " wall"))
                (prompt (strcat " Break=" dlTremaabk))
                (prompt (strcat " Cappings=" dlTremacap))
                (prompt (strcat " Justification=" dlTrematzb))
                (setq CorN (ukword 1 "Current New" "Use Current wall or [New] " "Current"))
              )
            )
          )
        )

        (if (= nil dlTremaabk)
          (setq dlTremaabk "ON")
        )
        (if (= nil dlTremacap)
          (setq dlTremacap "None"  dlTremaecp 0)                    ;formerly "Auto"
        )


        (defun AACol2ACI (str /)
          (setq Hshstr (strcase str))
          (cond ((= Hshstr "RED") 1)
            ((= Hshstr "YELLOW") 2)
            ((= Hshstr "GREEN") 3)
            ((= Hshstr "CYAN") 4)
            ((= Hshstr "BLUE") 5)
            ((= Hshstr "MAGENTA") 6)
            ((= Hshstr "WHITE") 7)
            ((= Hshstr "GREY") 8)
            ((= Hshstr "LIGHT GREY") 9)
            ((= Hshstr "BYBLOCK") 0)
            ((= Hshstr "BY BLOCK") 0)
            ((= Hshstr "BYLAYER") 256)
            ((= Hshstr "BY LAYER") 256)
            ((and (< 0 (atoi Hshstr)) (> 256 (atoi Hshstr))) (atoi Hshstr))
            (nil)
          )
        )

        (defun c:InfoWall_OnInitialize ()
          (TraceIsOn ".InfoWall_OnInitialize")
          (Odcl_Control_SetProperty Walls_InfoWall_OnError "Visible" nil)

          (ReadLayList)
          (if (= SelWall nil)
            (progn (setq pickf   0
                         SelWall (nth 0 lMursLayers)
               )                                  ;If no wall selected go for 1st one
               (XtractWallsList)
            )
          )
          (AddLayersToListBox)
          (Odcl_ListBox_SetTopIndex Walls_InfoWall_WallListBox pickf)
          (Odcl_ListBox_SetCurSel Walls_InfoWall_WallListBox pickf)
          (princ)
        )

        (defun c:InfoWall_WallListBox_OnSelChanged (nSelection sSelText /)
          (TraceIsOn ".WallListBox_OnSelChanged")
          (Odcl_Control_SetProperty Walls_InfoWall_NameTextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ShortTextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_CavityTextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_Width1TextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay1 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox1 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox1 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay2 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox2 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox2 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_Width2TextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay3 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox3 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox3 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay4 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox4 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox4 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_Draw "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_CreateTextButton "Enabled" nil)
          ;;;(Odcl_MessageBox "To Do: code must be added to WallListBox_OnSelChanged" (etos sSelText))
          (if (or (= sSelText "Double Brick")(= sSelText "Single Brick"))
           (Odcl_Control_SetProperty Walls_InfoWall_DeleteTextButton "Enabled" nil)
           (Odcl_Control_SetProperty Walls_InfoWall_DeleteTextButton "Enabled" T)
          )
          (setq pickf nSelection)
          (XtractWallsList)
          (AddLayersToListBox)
          (princ)
        )

        (defun c:InfoWall_CreateTextButton_OnClicked ()
          ;;(Odcl_MessageBox "To Do: code must be added to CreateTextButton" (etos sLay1s))
          (if (= bMetUnit T)                            ;Section metrique
            (setq fInfoMurs "MetricWallParts.Dat")
            (setq fInfoMurs "MetricWallParts.Dat")
          )
          (setq file5 (open (strcat Chemin fInfoMurs) "a"))

          (Odcl_Control_SetProperty Walls_InfoWall_OnError "Visible" T)
          (Odcl_Control_ForceUpdateNow Walls_InfoWall_OnError)

          (Setq sNames (Odcl_Control_GetProperty Walls_InfoWall_NameTextBox "Text"))
          (Setq sShortCuts (Odcl_Control_GetProperty Walls_InfoWall_ShortTextBox "Text"))
          (Setq sCavites (Odcl_Control_GetProperty Walls_InfoWall_CavityTextBox "Text"))
          (Setq sWidth1s (Odcl_Control_GetProperty Walls_InfoWall_Width1TextBox "Text"))
          (Setq sLay1s (Odcl_ComboBox_GetTBText Walls_InfoWall_ComboBoxLay1))
          (Setq sLay2s (Odcl_ComboBox_GetTBText Walls_InfoWall_ComboBoxLay2))
          (Setq sSelTex1 (Odcl_ComboBox_GetTBText Walls_InfoWall_ColorBox1))

          (if (= (cadr (strfind sSelTex1 "Color ")) nil)
            (setq sCol1s (car (strfind sSelTex1 "Color ")))
            (setq sCol1s (cadr (strfind sSelTex1 "Color ")))
          )
          (Setq sSelTex2 (Odcl_ComboBox_GetTBText Walls_InfoWall_ColorBox2))
          (if (= (cadr (strfind sSelTex2 "Color ")) nil)
            (setq sCol2s (car (strfind sSelTex2 "Color ")))
            (setq sCol2s (cadr (strfind sSelTex2 "Color ")))
          )
          (setq sLwt1s (car (strfind (Odcl_ComboBox_GetTBText Walls_InfoWall_LWBox1) " mm")))
          (setq sLwt2s (car (strfind (Odcl_ComboBox_GetTBText Walls_InfoWall_LWBox2) " mm")))
          (if (= sTypes "DoubleW")
            (progn
              (Setq sWidth2s (Odcl_Control_GetProperty Walls_InfoWall_Width2TextBox "Text"))
              (Setq sLay3s (Odcl_ComboBox_GetTBText Walls_InfoWall_ComboBoxLay3))
              (Setq sLay4s (Odcl_ComboBox_GetTBText Walls_InfoWall_ComboBoxLay4))
              (Setq sSelTex3 (Odcl_ComboBox_GetTBText Walls_InfoWall_ColorBox3))
              (if (= (cadr (strfind sSelTex3 "Color ")) nil)
                (setq sCol3s (car (strfind sSelTex3 "Color ")))
                (setq sCol3s (cadr (strfind sSelTex3 "Color ")))
              )
              (Setq sSelTex4 (Odcl_ComboBox_GetTBText Walls_InfoWall_ColorBox4))
              (if (= (cadr (strfind sSelTex4 "Color ")) nil)
                (setq sCol4s (car (strfind sSelTex4 "Color ")))
                (setq sCol4s (cadr (strfind sSelTex4 "Color ")))
              )
              (setq sLwt3s (car (strfind (Odcl_ComboBox_GetTBText Walls_InfoWall_LWBox3) " mm")))
              (setq sLwt4s (car (strfind (Odcl_ComboBox_GetTBText Walls_InfoWall_LWBox4) " mm")))
            )
            (Setq sWidth2s "0")
          )

          (if (= bMetUnit T)                              ;Section metrique.
            (princ)
            (progn
              (setq sWidth1s  (* (atof sWidth1s) 25.4)
                sWidth2s  (* (atof sWidth2s) 25.4)
                sCavites  (* (atof sCavites) 25.4)
                sWidth1s  (rtos sWidth1s 2 2)
                sWidth2s  (rtos sWidth2s 2 2)
                sCavites  (rtos sCavites 2 2)
              )
            )
          )

          (setq Optio1 (strcat
                         (strcat sNames "->" sShortCuts ";" sTypes ";" sWidth1s ";" sCavites ";" sWidth2s ";")
                         (strcat sLay1s ";" sCol1s ";" sLwt1s ";" sLay2s ";" sCol2s ";" sLwt2s "|")
          )            )
          (if (= sTypes "SingleW")
            (wlf Optio1 file5)
            (progn
              (setq Optio2 (strcat sLay3s ";" sCol3s ";" sLwt3s ";" sLay4s ";" sCol4s ";" sLwt4s) )
              (wlf (strcat Optio1 Optio2) file5)
            )
          )
          (close file5)

          (AlphaSort (strcat Chemin "MetricWallParts.Dat"))

          (Odcl_Control_SetProperty Walls_InfoWall_NameTextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ShortTextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_CavityTextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_Width1TextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay1 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox1 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox1 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay2 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox2 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox2 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_Width2TextBox "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay3 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox3 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox3 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay4 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox4 "Enabled" nil)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox4 "Enabled" nil)
          (dos_waitcursor T)
          (Odcl_ListBox_Clear Walls_InfoWall_WallListBox)
          (setq lMursLayers nil)
          (ReadLayList)
          (if (= SelWall nil)
            (progn
              (setq pickf   0
                    SelWall (nth 0 lMursLayers)
              )
              (XtractWallsList)
            )
          )
          (AddLayersToListBox)
          (setq lMursLayers nil)

          (AliasList)
          (setq Hshfile4 (findfile "Walias.odx"))
          (if (= Hshfile4 nil)
            (princ "Missing file is necessary to some routines.")
            (load Hshfile4)
          )
          (Odcl_Control_SetProperty Walls_InfoWall_OnError "Visible" nil)
          (dos_waitcursor)
        )

        (defun c:InfoWall_DeleteTextButton_OnClicked ()
          (setq bOK nil)
          (cond
            ((= langue "Franais")
               (setq bOK (dos_msgbox "Really want to delete wall from the library?" "ArchAssist  Message" 4 1)) )
            ((= langue "Italiano")
               (setq bOK (dos_msgbox "Really want to delete wall from the library?" "ArchAssist  Message" 4 1)) )
            ((= langue "Espanol")
               (setq bOK (dos_msgbox "Really want to delete wall from the library?" "ArchAssist  Message" 4 1)) )
            (T (setq bOK (dos_msgbox "Really want to delete wall from the library?" "ArchAssist  Message" 4 1)) )
          )
          (if (= bOK 6)
            (Progn                         ;Yes
              (if (= bMetUnit T)                       ;Section metrique
                (setq fInfoMurs "MetricWallParts.Dat")
                (setq fInfoMurs "MetricWallParts.Dat")
              )
              (setq lAfterDel (remove_n pickf lmurslayers)
                    lAfterDel (reverse lAfterDel)
                    file2     (open (strcat Chemin fInfoMurs) "w")
              )
              (foreach item lAfterDel (wlf item file2))
              (close file2)
              (Odcl_ListBox_Clear Walls_InfoWall_WallListBox)
              (setq lMursLayers nil)
              (ReadLayList)
              (if (= SelWall nil)
                (progn
                  (setq pickf   0
                        SelWall (nth 0 lMursLayers)
                  )
                  (XtractWallsList)
                )
              )
              (AddLayersToListBox)
            )
          )
        )

        (defun c:InfoWall_Draw_OnClicked ()
          (TraceIsOn ".Draw_Clicked")
          ;;(Odcl_MessageBox "To Do: code must be added to c:InfoWall_Draw_Clicked" (etos sLay1))
          (setq WLAYER   sLay1
                DLWIDTH1 (read sWidth1)
                DLWIDTH2 (read sWidth2)
          )
          (setvar "TRACEWID" (read sOverall))
          (Odcl_Form_Close ProjName2 "InfoWall")
          (WallLaunch)
        )

        (defun c:InfoWall_WallListBox_OnDblClicked ()
          (Odcl_Control_SetProperty Walls_InfoWall_NameTextBox "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_ShortTextBox "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_Width1TextBox "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay1 "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox1 "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox1 "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay2 "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_ColorBox2 "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_LWBox2 "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_CreateTextButton "Enabled" T)
          (Odcl_Control_SetProperty Walls_InfoWall_Draw "Enabled" Nil)
          (Odcl_Control_SetProperty Walls_InfoWall_CavityTextBox "Enabled" nil)
          (if (= bTypes T)
            (progn (Odcl_Control_SetProperty Walls_InfoWall_Width2TextBox "Enabled" T)
                   (Odcl_Control_SetProperty Walls_InfoWall_CavityTextBox "Enabled" T)
               (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay3 "Enabled" T)
               (Odcl_Control_SetProperty Walls_InfoWall_ColorBox3 "Enabled" T)
               (Odcl_Control_SetProperty Walls_InfoWall_LWBox3 "Enabled" T)
               (Odcl_Control_SetProperty Walls_InfoWall_ComboBoxLay4 "Enabled" T)
               (Odcl_Control_SetProperty Walls_InfoWall_ColorBox4 "Enabled" T)
               (Odcl_Control_SetProperty Walls_InfoWall_LWBox4 "Enabled" T)
            )
          )
        )

        (defun c:InfoWall_OnClose (nUpperLeftX nUpperLeftY /)
          ;(Odcl_MessageBox "To Do: code must be added to c:InfoWall_OnClose" "To do")
          ;(setq sOverall nil)
          (princ)
        )

        (defun c:InfoWall_about_OnClicked () (_about) (princ))

        (if (or (= CorN "New") (= bFirst T))                   ;Appel de la boite de dialogue
          (progn
            ;;(MontreAA2)
            (Odcl_Form_Show ProjName2 "InfoWall")
          )
          (WallLaunch)
        )
      )

      (defun WallLaunch ()
        (TraceIsOn ".WallLaunch")
        (prompt (strcat "\nCurrent settings: " sOverall "mm wall"))
        (prompt (strcat " Break=" dlTremaabk))
        (prompt (strcat " Cappings=" dlTremacap))
        (prompt (strcat " Justification=" dlTrematzb))

        (if (= sTypes "SingleW")
          (progn (Hshset_lay sLay1 sCol1 "CONTINUOUS" sLwt1)
            (Hshset_lay sLay2 sCol2 "CONTINUOUS" sLwt2)
            (setq MurDOUBLE 0)
          )
          (progn (Hshset_lay sLay1 sCol1 "CONTINUOUS" sLwt1)
            (Hshset_lay sLay2 sCol2 "CONTINUOUS" sLwt2)
            (Hshset_lay sLay3 sCol3 "CONTINUOUS" sLwt3)
            (Hshset_lay sLay4 sCol4 "CONTINUOUS" sLwt4)
            (setq MurDOUBLE 1)
          )
        )

        (if (/= (tblsearch "layer" WLAYER) nil)
          (progn (command "_.layer" "_t" WLAYER "_on" WLAYER ""))
        )

        (defun dline (/ strtpt
                        nextpt  pt1
                        pt2 spts
                        wnames  elast
                        uctr    pr
                        prnum   temp
                        ans dir
                        ipt v
                        lst dist
                        cpt rad
                        orad    ftmp
                        spt ept
                        pt  en1
                        en2 npt
                        cpt1    flg
                        cont    flg2
                        flgn    ang
                        tmp undo_setting
                        brk_e1  brk_e2
                        bent1   bent2
                        nn  nnn
                        dl_osm  dl_oem
                        dl_oce  dl_opb
                        dl_obm  dl_ver
                        dl_err  dl_oer
                        dl_arc  fang
                        MAXSNP  ange
                        savpt1  savpt2
                        savpt3  savpt4
                        savpts
                     )
          (setq MAXSNP 10)
          (setq wnames nil)
          (setq wnamesa nil)
          (if (/= (tblsearch "LAYER" WLAYER) nil)
            (progn (command "_.layer" "_t" WLAYER "") (setvar "CLAYER" WLAYER))
            (progn (command "_.layer" "_m" WLAYER ""))
          )
          (command "_.UNDO" "_GROUP")
          (if (null dlTremaopb)
            (setq dlTremaopb (getvar "pickbox"))
          )
          (setq nextpt "Straight")
          (setq cont T
            bEMur T
            bIMur T
          )
          (while cont (dl_m1) (dl_m2))
          (princ)
        )

        (defun dl_m1 ()
          (TraceIsOn ".dl_m1")
          (setq temp T
            uctr nil
          )
          (if dl_arc
            (setq nextpt "Arc")
            (setq nextpt "Line")
          )
          (while temp
            (setq strtpt       ;;Undo not necessary
               (upoint
                 7
                 "Break BOundary CAppings cHain Convert Exit Justification Reference"
                 "Specify start point or [Break/BOundary/CAppings/cHain/Convert/Justification/Reference point/Exit]"
                 nil
                 nil
                )
            )     ;point1
            ;;    (toggle "osmode" 16384)                     ;toggle 16384
            (cond ((= strtpt "Justification") (dl_sao))
              ((= strtpt "Exit")
               (setq Temp   nil
                 cont   nil
                 nextpt "Quit"
               )
              )
              ((= strtpt "Break")
               (setq dlTremaabk (ukword 1
                            "ON OFF"
                            "Auto-break lines at start and end points [OFF/ON] "
                            dlTremaabk
                        )
               )
               (setq dlTremabrk (if (= dlTremaabk "OFF")
                      nil
                      T
                        )
               )
              )
              ((= strtpt "BOundary")  (BOffset1))
              ((= strtpt "cHain")     (ChainIt))
              ((= strtpt "Convert")   (dl_cnv))
              ((= strtpt "Reference") (dl_ofs))
              ((= strtpt "Snap")      (dl_sso))
              ((= strtpt "Undo")      (princ "\nAll segments already undone. ") (setq temp T))
              ((null strtpt)
               (if v:stpt
                 (setq strtpt v:stpt
                   temp   nil
                 )
                 (progn (princ "\nNo continuation point, please pick a point.")
                 )
               )
              )
              ((= strtpt "CAppings") (endcap))
              (T
               (setq v:stpt strtpt
                 temp   nil
               )
              )
            )
          )
        )

        (defun dl_m2 (/ temp)
          (TraceIsOn ".dl_m2")
          (setq spts  (list strtpt)
            uctr  0
            uctra 0
          )
          (cond ((/= strtpt "Exit")
                 (if dlTremasnp (dl_ved "brk_e1" strtpt) )
                )
          )
          (cond ((= dlTrematzb "Top")
             (setq dlTremaosd (* (read sOverall) -0.5) )
            )
            ((= dlTrematzb "Zero")
             (setq dlTremaosd 0.0 )
            )
            ((= dlTrematzb "Bottom")
             (setq dlTremaosd (* (read sOverall) 0.5) )
            )
          )

          (Cassure "1: temp dlTremaosd <<<<<<<<<<<<<<<<<")

          (while (and nextpt (/= nextpt "Close"))
            (if (/= nextpt "Quit")
          (if dl_arc
            (progn
              (initget "Break CAppings CEnter CLose Endpoint Line Undo") ;<<<CLose???!!!
              (setq
                nextpt (getpoint
                     strtpt
                     (strcat "\n[CAppings/CEnter/Endpoint/Line/Undo]/<second point>: ")
                   )
              )
            )
            (progn (setq nextpt
                  (upoint 8
                      "Arc CAppings CLose Undo"
                      "Specify next point or [Arc/CAppings/Undo]"
                      nil
                      strtpt
                         )
               )
            )
          )
            )
            (setq v:stpt (last spts))
            (cond ((= nextpt "Justification") (dl_sao))
              ((= nextpt "Width") (dl_snw))
              ((= nextpt "Undo")
               (cond ((= uctr 0) (setq nextpt nil))
                 ((> uctr 0)
                  (command "_.U")
                  (setq spts (dl_lsu spts 1))
                  (setq savpts (dl_lsu savpts 2))
                  (setq wnames (dl_lsu wnames 2))
                  (setq wnamesa (dl_lsu wnamesa 2))
                  (setq uctr (- uctr 2))
                  (setq uctra (- uctra 2))
                  (setq strtpt (last spts))
                 )
               )
               (if dlTremasnp
                 (if (= uctr 0)
               (dl_ved "brk_e1" strtpt)
                 )
               )
              )
              ((= nextpt "Break")
               (setq dlTremaabk (ukword 1
                            "ON OFF"
                            "Auto-break lines at start and end points [OFF/ON]"
                            dlTremaabk
                        )
               )
               (setq dlTremabrk (if (= dlTremaabk "OFF")
                      nil
                      T
                        )
               )
               (if dlTremasnp
                 (dl_ved "brk_e1" strtpt)
               )
               (if dl_arc
                 (setq nextpt "Arc")
                 (setq nextpt "Line")
               )
              )
              ((= nextpt "Snap") (dl_sso))
              ((= nextpt "Arc") (setq dl_arc T))
              ((= nextpt "Line") (setq dl_arc nil))
              ((= nextpt "CLose") (dl_cls))
              ((= (type nextpt) 'LIST) (dl_ds))
              ((= nextpt "CEnter") (dl_ceo))
              ((= nextpt "Endpoint") (dl_epo))
              ((= nextpt "CAppings") (endcap))
              (T
               (setq nextpt nil
                 cont nil
               )
               (if (> uctr 1)
                 (if (= (logand 4 dlTremaecp) 4)
               (progn (if (null brk_e1)
                    (LineMake savpt1 savpt2 sLay1)        ;premier capping Mur 1
                  )
                      (SetWallInfoTag sAppNameAA (entlast))
                  (Cassure "Apres SCap1")
                  (dl_ssp)
                  (if (null brk_e2)
                    (LineMake savpt3 savpt4 sLay1)        ;dernier capping Mur 1
                  )
                      (SetWallInfoTag sAppNameAA (entlast))
                  (Cassure "Apres ECap1")
               )
               (progn (if (= (logand 1 dlTremaecp) 1)
                    (LineMake savpt1 savpt2 sLay1)        ;premier capping
                  )
                      (SetWallInfoTag sAppNameAA (entlast))
                  (Cassure "Apres SCap2")
                  (if (= (logand 2 dlTremaecp) 2)
                    (progn (dl_ssp)
                       (LineMake savpt3 savpt4 sLay1)     ;dernier capping
                    )
                  )
                      (SetWallInfoTag sAppNameAA (entlast))
                  (Cassure "Apres ECap2")
               )
                 )
               )
               (if brk_e1
                 (setq brk_e1 nil)
               )
               (if brk_e2
                 (setq brk_e2 nil)
               )
               (if brk_e1a
                 (setq brk_e1a nil)
               )
               (if brk_e2a
                 (setq brk_e2a nil)
               )
               (command "_.UNDO" "_EN")
              )
            )
          )
        )

        (defun dl_cls ()                          ;CLose option
          (TraceIsOn ".dl_cls")
          (if (or (and (null dl_arc)
                  (< uctr 4)
               (if (> uctr 1)
                 (/= (dl_val 0 (entlast)) "ARC")
                 (not (> uctr 1))
               )
              )
              (and dl_arc (< uctr 2))
          )
            (progn (princ "\nCannot close until two or more segments drawn. ")
               (if dl_arc
                 (setq nextpt "Arc")
                 (setq nextpt "Line")
               )
            )
            (progn (command "_.UNDO" "_GROUP")
               (setq nextpt (nth 0 spts))             ;Point du depart du mur
               (if (null dl_arc)
                 (dl_mlf 3)
                 (progn (setq tmp  (last wnames)
                      ange (trans '(1 0 0) (dl_val -1 tmp) 1)
                      ange (angle '(0 0 0) ange)
                      dir  (if (= (dl_val 0 tmp) "LINE")
                         (angle (trans (dl_val 10 tmp) 0 1) (trans (dl_val 11 tmp) 0 1))
                         (progn (setq dir (+ (dl_val 50 tmp) ange)
                              dir (if (> dir (* 2 pi))
                                    (- dir (* 2 pi))
                                    dir
                                  )
                            )
                            (if (equal dir
                                   (setq dir (angle (trans (dl_val 10 tmp) (dl_val -1 tmp) 1)
                                            strtpt
                                         )
                                   )
                                   0.01
                                )
                              (- dir (/ pi 2))
                              (+ dir (/ pi 2))
                            )
                         )
                       )
                    )
                    (command "_.ARC" strtpt "_E" nextpt "_D" (* dir (/ 180 pi)))
                    (SetWallInfoTag sAppNameAA (entlast))
                    (dl_mlf 4)
                 )
               )
               (setq nextpt "CLose"
                 v:stpt nil
                 cont   nil
               )
            )
          )
        )

        (defun dl_ds ()
          (TraceIsOn ".dl_ds")
          (if (equal strtpt nextpt 0.0001)
            (progn (princ "\nCoincident point, please try again. ")
               (if dl_arc
                 (setq nextpt "Arc")
                 (setq nextpt "Line")
               )
            )
            (progn (command "_.UNDO" "_GROUP")
               (setq nextpt (list (car nextpt) (cadr nextpt) (caddr strtpt)))
               (if dl_arc
                 (progn (command "_.ARC" strtpt nextpt)
                    (prompt "\nEndpoint: ")
                    (command pause)
                    (setq nextpt (getvar "lastpoint")
                      v:stpt nextpt
                    )
                    (setq temp (entlast))
                    (entdel temp)
                    (if dlTremasnp
                  (dl_ved "brk_e2" nextpt)
                    )
                    (entdel temp)
                    (dl_mlf 2)
                 )
                 (progn (setq v:stpt nextpt)
                    (if dlTremasnp
                  (dl_ved "brk_e2" nextpt)
                    )
                    (if (and brk_e1 (eq brk_e1 brk_e2) (= (dl_val 0 brk_e1) "LINE"))
                  (progn (princ "\nSecond point cannot be on the same line segment. ")
                         (setq brk_e2 nil)
                  )
                  (dl_mlf 1)
                    )
                 )
               )
               (if brk_e2
                 (setq nextpt "Quit")
               )
            )
          )
        )

        (defun dl_ceo ()
          (command "_.UNDO" "_GROUP")
          (setq temp T)
          (while temp
            (setq cpt (upoint 1 "" "Specify center point" nil nil))
            (if (<= (distance cpt strtpt) (- (/ (getvar "tracewid") 2.0) dlTremaosd))
              (progn (setq messg (strcat "\nThe radius defined by the selected centre point is too small"
                         "\nfor the current wall width. Please select another point."
                     )
                 )
                 (dos_msgbox Messg "ArchAssist Message" 1 5)
          )
          (setq temp nil)
            )
          )
          (command "_.ARC" strtpt "_C" cpt)
          (setq nextpt (ukword 1 "Angle Length Endpoint" "[Angle/Length of chord] " "Endpoint")
          )
          (cond ((= nextpt "Angle")
             (prompt "\nIncluded angle: ")
             (command "_A" pause)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
                 (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
            ((= nextpt "Length")
             (prompt "\nChord length: ")
             (command "_L" pause)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
                 (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
            (T
             (prompt "\nEndpoint: ")
             (command pause)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
                 (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
          )
        )

        (defun dl_epo ()
          (command "_.UNDO" "_GROUP")
          (setq cpt (upoint 1 "" "Specify endpoint" nil nil))
          (command "_.ARC" strtpt "_E" cpt)
          (setq nextpt (ukword 1 "Angle Direction Radius Center" "[Angle/Direction/Radius] " "Center")
          )
          (cond
            ((= nextpt "Angle")
             (prompt "\nIncluded angle: ")
             (command "_A" pause)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
             (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
            ((= nextpt "Direction")
             (prompt "\nTangent direction: ")
             (command "_D" pause)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
             (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
            ((= nextpt "Radius")
             (setq temp T)
             (while temp
           (initget 1)
           (setq rad (getdist cpt "\nRadius: "))
           (if (or (<= rad (/ (getvar "tracewid") 2.0)) (< rad (/ (distance strtpt cpt) 2.0)))
             (progn (setq
                  messg (strcat "\nThe radius entered is less than half the wall width or is invalid"
                        "\nfor the selected endpoints. Please enter a greater radius."
                    )
                )
                (dos_msgbox Messg "ArchAssist Message" 1 5)
                (princ "Please enter a radius greater than ")
                (if (< (/ (getvar "tracewid") 2.0) (/ (distance strtpt cpt) 2.0))
                  (princ (rtos (/ (distance strtpt cpt) 2.0)))
                  (princ (rtos (/ (getvar "tracewid") 2.0)))
                )
                (princ ". ")
             )
             (setq temp nil)
           )
             )
             (command "_R" rad)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
             (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
            (T
             (prompt "\nCenter: ")
             (command pause)
             (setq nextpt (dl_vnp)
               v:stpt nextpt
             )
             (SetWallInfoTag sAppNameAA (entlast))
             (dl_mlf 2)
            )
          )
        )

        (defun dl_ssp (/ temp)
          (TraceIsOn ".dl_ssp")
          (setq temp (length savpts))
          (if (> temp 1)
            (progn (setq savpt3 (nth (- temp 2) savpts)
                 savpt4 (nth (- temp 1) savpts)
               )
            )
          )
        )

        (defun dl_sao (/ temp dragos)
          (TraceIsOn ".dl_sao")
          (setq temp dlTremaosd)
          (setq dlTrematzb (ukword 1
                       "Top Zero Bottom"
                       "Set dragline justification to [Top/Zero/Bottom] "
                       dlTrematzb
                   )
          )
          (prompt (strcat "\nCurrent settings: " sOverall "mm wall"))
          (prompt (strcat " Break=" dlTremaabk))
          (prompt (strcat " Cappings=" dlTremacap))
          (prompt (strcat " Justification=" dlTrematzb))

          (cond ((= dlTrematzb "Top")
             (setq dlTremaaln 1  dlTremaosd (* (read sOverall) -0.5) )
            )
            ((= dlTrematzb "Zero")
             (setq dlTremaaln 0  dlTremaosd 0.0 )
            )
            ((= dlTrematzb "Bottom")
             (setq dlTremaaln 2  dlTremaosd (* (read sOverall) 0.5) )
            )
            ((= (type dlTremaosd) 'REAL)
             (if dragos
               (setq dlTremaaln nil)
               (progn (setq dlTremaaln nil)
                  (if (> dlTremaosd (/ (getvar "tracewid") 2.0))
                    (progn (princ "\nValue entered is out of range. Reset to ")
                       (princ (/ (getvar "tracewid") 2.0))
                       (setq dlTremaosd (/ (getvar "tracewid") 2.0))
                    )
                  )
                  (if (< dlTremaosd (- (/ (getvar "tracewid") 2.0)))
                    (progn (princ "\nValue entered is out of range.  Reset to ")
                       (princ (- (/ (getvar "tracewid") 2.0)))
                       (setq dlTremaosd (- (/ (getvar "tracewid") 2.0)))
                    )
                  )
               )
             )
            )
            (T (setq dlTremaosd temp))
          )
        )

        (defun dl_snw ()
          (initget 6)
          (setvar "tracewid"
              (if (setq temp (getdist (strcat "\nNew wall width <" (rtos (getvar "tracewid")) ">: ")
                     )
              )
                temp
                (getvar "tracewid")
              )
          )
          (prompt (strcat "\nCurrent settings: " sOverall "mm wall"))
          (prompt (strcat " Break=" dlTremaabk))
          (prompt (strcat " Cappings=" dlTremacap))
          (prompt (strcat " Justification=" dlTrematzb))
          (if dlTremaaln
            (cond ((= dlTremaaln 1) (setq dlTremaosd (- (/ (getvar "tracewid") 2.0))))
              ((= dlTremaaln 2) (setq dlTremaosd (/ (getvar "tracewid") 2.0)))
              (T (princ))
            )
          )
        )

        (defun dl_ofs ()
          (toggle "osmode" 16385)     ;toggle 16384+1 for Endp
          (setq strtpt (upoint 1 "" "Specify reference point" nil nil))
          (toggle "osmode" 512)       ;toggle 16384+512 for nearest
          (setq nextpt (upoint 1 "" "Pick direction" nil strtpt))
          (toggle "osmode" 16384)     ;toggle 16384 for none
          (setq dist (udist 6 "" "Distance from reference point" (distance strtpt nextpt) strtpt)
          )
          (setq dist (if (or (= dist "") (null dist))
               (distance strtpt nextpt)
               (if (< dist 0)
                 (* (distance strtpt nextpt) (/ (abs dist) 100.0))
                 dist
               )
                 )
          )
          (setq strtpt (polar strtpt (angle strtpt nextpt) dist))
          (setq temp nil)
          (command "_.UNDO" "_GROUP")
        )

        (defun dl_sso ()
          (setq ans (ukword 1 "ON OFF Size" "\nSet Snap [ON/OFF/Size] " <ON>))
          (if (= ans "OFF")
            (progn (setq dlTremasnp nil) (setvar "pickbox" 0))
            (if (= ans "Size")
              (progn (setq dlTremasnp T
                   ans 0
                 )
                 (while (or (< ans 1) (> ans MAXSNP))
               (setq
                 ans (getint
                   (strcat "\nNew snap size (1 - " (itoa MAXSNP) ") <" (itoa dlTremaopb) ">: ")
                     )
               )
               (if (or (= ans "") (null ans))
                 (setq ans dlTremaopb)
               )
                 )
                 (setvar "pickbox" ans)
                 (setq dlTremaopb ans)
          )
          (progn (setq dlTremasnp T) (setvar "pickbox" dlTremaopb))
            )
          )
          (if dlTremasnp
            (if (= uctr 0)
          (dl_ved "brk_e1" strtpt)
            )
          )
          (if dl_arc
            (setq nextpt "Arc")
            (setq nextpt "Line")
          )
        )

        (defun dl_ved (vent pt)
          (TraceIsOn ".dl_ved")
          (if (set (read vent) (ssget pt))
            (progn
          (set (read vent) (ssname (eval (read vent)) 0))
          (if
            (and (or (= (dl_val 0 (eval (read vent))) "ARC") (= (dl_val 0 (eval (read vent))) "LINE"))
                 (equal (caddr (dl_val 210 (eval (read vent)))) (caddr (trans '(0 0 1) 1 0)) 0.001)
            )
             (princ)
             (progn (princ "\nObject found is not an arc/line, or not parallel to the UCS. ")
                (set (read vent) nil)
             )
          )
            )
          )
          (eval (read vent))
        )

        (defun dl_vnp (/ temp cpt ang rad)
          (TraceIsOn ".dl_vnp")
          (setq temp (entlast))
          (if (= (dl_val 0 temp) "LINE")
            (setq nextpt (if (equal strtpt (dl_val 10 temp) 0.001)
                   (dl_val 11 temp)
                   (dl_val 10 temp)
                 )
            )
            (progn (setq cpt (trans (dl_val 10 temp) (dl_val -1 temp) 1)
                 ang (dl_val 50 temp)
                 rad (dl_val 40 temp)
               )
               (setq ange (trans '(1 0 0) (dl_val -1 temp) 1)
                 ange (angle '(0 0 0) ange)
                 ang  (+ ang ange)
               )
               (if (> ang (* 2 pi))
                 (setq ang (- ang (* 2 pi)))
               )
               (setq nextpt (if (equal strtpt (polar cpt ang rad) 0.01)
                      (polar cpt (dl_val 51 temp) rad)
                      (polar cpt ang rad)
                    )
               )
            )
          )
        )

        (defun dl_mlf (flg / temp1 temp2 aNouv ang1 ang2 ent  cpt ang rad1 rad2 sent1 sent2 tmpt1 tmpt2
               tmpt3 tmpt4 templ1 templ2)
          (TraceIsOn ".dl_mlf")
          (if (null nextpt)
            (setq nextpt (dl_vnp))
          )
          (if (equal nextpt (nth 0 spts) 0.01)                ;Si presque meme que pt de depart
            (if dl_arc
          (setq flg 4)
          (setq flg 3)
            )
          )
          (setq temp1  0  ;;(+ (* (getvar "tracewid") 0.5) dlTremaosd)    ;int. mur 1 or picked pts
            temp2  (read sOverall)    ;;(- (getvar "tracewid") temp1) ;ext. mur 2
            aNouv (angle strtpt nextpt)
            ang1   (+ aNouv (/ pi 2))                 ;ang1=+90 Monte et ang2=-90 deg descend.
            ang2   (- aNouv (/ pi 2))
          )
          (Cassure "***********aNouv ang1 strtpt nextpt")

          (cond ((= dlTrematzb "Top")
             (setq templ1 (read sWidth1)                  ;ext. mur 1
               templ2 (+ (read sCavite) (read sWidth1))       ;int. mur 2
               aType  ang2
             )
            )
            ((= dlTrematzb "Bottom")
             (setq templ1 (read sWidth1)
               templ2 (+ (read sCavite) (read sWidth1))
               aType  ang1
             )
            )
            ((= dlTrematzb "Zero")
             (setq temp1  (* 0.5 (read sOverall)) ;(- temp1 DLWIDTH1) ;DLWIDTH1=murW1
               templ1 (- (* 0.5 (read sOverall))(read sWidth1))
               templ2 (- (* 0.5 (read sOverall))(read sWidth2))
               temp2  (* 0.5 (read sOverall))
               aType  ang2
             )
            )
          )

          (Cassure ".temp1 templ1 temp2 templ2 flg ang1 ang2 dlTremaosd <<<<<<<<<<<<<<<<<")
          (cond ((= flg 1)
             (if (= MurDOUBLE 1)              ;Si c'est un mur double ou avec cavite
               (progn (dl_dls nil aType temp1)        ;Le long des picked pts [Ext]
                  (dl_dls1 nil aType templ1)      ;Face interieure du 1er mur
              (Cassure "ang2 templ1")         ;dl_dls flgn ang temp

                  (if (= dlTrematzb "Zero")(setq aType ang1))
                  (dl_dls1 nil aType templ2)      ;Face interieure du 2eme mur
                  (dl_dls nil aType temp2)        ;Face exterieure du 2eme mur
               )
               (progn
                  (dl_dls nil aType temp1)        ;Pour mur simple le long des picked pts [Ext]
                  (if (= dlTrematzb "Zero")(setq aType ang1))
                  (dl_dls nil aType temp2)        ;
               )
             )

            )
            ((or (= flg 2) (= flg 4))
             (setq uctra 0)
             (setq restart "yes")
             (setq tmp (entlast)
               ent (entget tmp)
               cpt (trans (dl_val 10 tmp) (dl_val -1 tmp) 1)
               ang (dl_val 50 tmp)
             )
             (setq ange (trans '(1 0 0) (dl_val -1 tmp) 1)
               ange (angle '(0 0 0) ange)
               ang  (+ ang ange)
             )
             (if (> ang (* 2 pi))
               (setq ang (- ang (* 2 pi)))
             )
             (if (equal (angle cpt strtpt) ang 0.01)
               (progn (setq strt_a T
                    rad1   (+ (dl_val 40 tmp) temp2)
                    rad2   (- (dl_val 40 tmp) temp1)
                  )
                  (setq ent (subst (cons 40 rad2) (assoc 40 ent) ent))
                  (entmod ent)
                  (dl_atl)
                  (setq save_1 ent)
                  (setq sent1 (dl_val -1 tmp))
                  (if (= flg 4)
                    (if (> uctr 2)
                  (dl_das 0 rad2 50)
                    )
                    (dl_das nil rad2 50)
                  )
                  (if (= MurDOUBLE 1)             ;Si c'est un mur arrondi avec cavite
                    (progn (command "_.OFFSET"
                            (rtos (- (getvar "TRACEWID") DLWIDTH2) 2 0)
                            (list tmp '(0 0 0))
                            (polar cpt ang (+ 1 rad1 rad2))
                            ""
                       )
                       (setq tmp1a (entlast)
                         ent   (entget tmp1a)
                       )
                       (dl_atl1)
                       (command "_.OFFSET"
                            (rtos DLWIDTH1 2 0)
                            (list tmp '(0 0 0))
                            (polar cpt ang (+ 1 rad1 rad2))
                            ""
                       )
                       (setq tmp1a (entlast)
                         ent   (entget tmp1a)
                       )
                       (dl_atl1)
                    )
                  )
                  (command "_.OFFSET"
                       (getvar "tracewid")
                       (list tmp '(0 0 0))
                       (polar cpt ang (+ 1 rad1 rad2))
                       ""
                  )
                  (setq tmp (entlast)
                    ent (entget tmp)
                  )
                  (dl_atl)
                  (setq save_2 ent)
                  (setq sent2 tmp)
                  (if (= flg 4)
                    (if (> uctr 3)
                  (progn (dl_das 1 rad1 50)
                         (setq nextpt "CLose"
                           v:stpt nil
                           cont   nil
                         )
                  )
                    )
                    (dl_das nil rad1 50)
                  )
               )
               (progn (setq strt_a nil
                    rad1   (+ (dl_val 40 tmp) temp1)
                    rad2   (- (dl_val 40 tmp) temp2)
                  )
                  (setq ent (subst (cons 40 rad1) (assoc 40 ent) ent))
                  (entmod ent)
                  (dl_atl)
                  (setq save_1 ent)
                  (setq sent1 (dl_val -1 tmp))
                  (if (= flg 4)
                    (if (> uctr 2)
                  (dl_das 0 rad1 51)
                    )
                    (dl_das nil rad1 51)
                  )
                  (if (= MurDOUBLE 1)             ;Si c'est un mur arrondi avec cavite
                    (progn (command "_.OFFSET"
                            (rtos (- (getvar "TRACEWID") DLWIDTH2) 2 0)
                            (list tmp '(0 0 0))
                            cpt
                            ""
                       )
                       (setq tmp1a (entlast)
                         ent   (entget tmp1a)
                       )
                       (dl_atl1)
                       (command "_.OFFSET" (rtos DLWIDTH1 2 0) (list tmp '(0 0 0)) cpt "")
                       (setq tmp1a (entlast)
                         ent   (entget tmp1a)
                       )
                       (dl_atl1)
                    )
                  )
                  (command "_.OFFSET" (getvar "tracewid") (list tmp '(0 0 0)) cpt "")
                  (setq tmp (entlast)
                    ent (entget tmp)
                  )
                  (dl_atl)
                  (setq save_2 ent)
                  (setq sent2 tmp)
                  (if (= flg 4)
                    (if (> uctr 3)
                  (progn (dl_das 1 rad2 51)
                         (setq nextpt "CLose"
                           v:stpt nil
                           cont   nil
                         )
                  )
                    )
                    (dl_das nil rad2 51)
                  )
               )
             )
            )
            ((= flg 3)
             (setq nextpt (nth 0 spts)
               ang1   (+ (angle strtpt nextpt) (/ pi 2))
               ang2   (- (angle strtpt nextpt) (/ pi 2))
             )
             (if (= MurDOUBLE 1)                  ;Si c'est un mur avec cavity
               (progn (dl_dls1 0 ang1 templ1) (dl_dls1 1 ang2 templ2))
             )
             (dl_dls 0 ang1 temp1)
             (dl_dls 1 ang2 temp2)
             (setq nextpt "CLose"
               v:stpt nil
               cont   nil
             )
            )
            (T (princ "\nERROR: Value out of range.") (exit))
          )
          (setq strtpt nextpt
            spts   (append spts (list strtpt))
            savpts (append savpts (list savpt3))
            savpts (append savpts (list savpt4))
          )
          (command "_.UNDO" "_E")
        )

        (defun dl_dls (flgn ang temp / j k pt1 pt2 tmp1 ent1 p1 p2)
          (TraceIsOn ".dl_dls pour murs exterieurs")
          (mapcar '(lambda (j k) (set j (polar (eval k) ang temp))) '(pt1 pt2) '(strtpt nextpt))
          (cond
            ((= uctr 0)
             (setq p1 (if (dl_l01 brk_e1 "1" pt1 pt2 strtpt)
                ipt
                savpt1
              )
             )
             (Cassure "save 1: P1 pt1 pt2 savpt1")            ;Debut
             (setq pt2 (if (dl_l01 brk_e2 "3" pt2 pt1 nextpt)
                 ipt
                 savpt3
               )
             )
             (Cassure "save 3: pt1 pt2 savpt3")
             (setq pt1 p1)
            )
            ((= uctr 1)
             (setq p1 (if (dl_l01 brk_e1 "2" pt1 pt2 strtpt)
                ipt
                savpt2
              )
             )
             (Cassure "save 2: P1 pt1 pt2 savpt2")
             (setq pt2 (if (dl_l01 brk_e2 "4" pt2 pt1 nextpt)
                 ipt
                 savpt4
               )
             )
             (Cassure "save 4: pt1 pt2 savpt4")
             (setq pt1 p1)
             (if (and dlTremabrk brk_e1)
           (progn (command "_.BREAK" brk_e1 savpt1 savpt2))
             )
             (if (and dlTremabrk brk_e2)
           (progn (if (eq brk_e1 brk_e2)
                (progn (entdel (nth 0 wnames)) (dl_ved "brk_e2" nextpt) (entdel (nth 0 wnames)))
              )
              (command "_.BREAK" brk_e2 savpt3 savpt4)
           )
             )
            )
            ((= (rem uctr 2.0) 0)
             (setq fang nil)
             (setq p1 (dl_dl2 pt1))
             (setq pt2 (if (dl_l01 brk_e2 "3" pt2 pt1 strtpt)
                 ipt
                 savpt3
               )
             )
             (Cassure "save 3a")
             (setq pt1 p1)
             (if flgn
           (progn (setq tmp1 (nth flgn wnames)
                    ent1 (entget tmp1)
              )
              (if (= (dl_val 0 tmp1) "LINE")
                (setq pt2 (dl_mls nil 10))
                (setq pt2 (dl_mas T nil pt2 pt1 nil))
              )
           )
             )
            )
            (T
             (setq p1 (dl_dl2 pt1))
             (setq pt2 (if (dl_l01 brk_e2 "4" pt2 pt1 nextpt)
                 ipt
                 savpt4
               )
             )
             (Cassure "save 4a")
             (setq pt1 p1)
             (if flgn
           (progn (setq tmp1   (nth flgn wnames)
                    ent1   (entget tmp1)
                    brk_e1 nil
                    brk_e2 nil
              )
              (if (= (dl_val 0 tmp1) "LINE")
                (setq pt2 (dl_mls nil 10))
                (setq pt2 (dl_mas T nil pt2 pt1 nil))
              )
           )
             )
             (if (and dlTremabrk brk_e2)
           (progn (command "_.BREAK" brk_e2 savpt3 savpt4))
             )
            )
          )
          (if (= bEMur T)             ;Si mur exterieur
            (progn (LineMake pt1 pt2 sLay1)       ;Mur exterieur 1
               (SetWallInfoTag sAppNameAA (entlast))
               (setq bEMur nil)
               (Cassure "murE1")
            )
            (progn (if (= MurDOUBLE 1)
                 (LineMake pt1 pt2 sLay3)     ;Mur exterieur 2 si mur double
                 (LineMake pt1 pt2 sLay2)     ;Mur exterieur 2 si mur simple
               )
               (SetWallInfoTag sAppNameAA (entlast))
               (setq bEMur T)
               (Cassure "murE2")
            )
          )
          (setq wnames (if (null wnames)
                 (list (setq elast (entlast)))
                 (append wnames (list (setq elast (entlast))))
               )
            uctr   (1+ uctr)
          )
          wnames
        )

        (defun dl_dls1 (flgn ang temp / j k pt1 pt2 tmp1 ent1 p1 p2)
          (TraceIsOn ".dl_dls1 pour murs interieurs")
          (mapcar '(lambda (j k) (set j (polar (eval k) ang temp))) '(pt1 pt2) '(strtpt nextpt))
          (cond
            ((= uctra 0)
             (setq p1 (if (dl_l01 brk_e1a "5" pt1 pt2 strtpt)
                ipt
                savpt5
              )
             )
             (Cassure "save 5")
             (setq pt2 (if (dl_l01 brk_e2a "7" pt2 pt1 nextpt)
                 ipt
                 savpt7
               )
             )
             (Cassure "save 7")
             (setq pt1 p1)
            )
            ((= uctra 1)
             (setq p1 (if (dl_l01 brk_e1a "6" pt1 pt2 strtpt)
                   ipt
                   savpt6
              )
             )
             (Cassure "save 6")
             (setq pt2 (if (dl_l01 brk_e2a "8" pt2 pt1 nextpt)
                   ipt
                   savpt8
               )
             )
             (Cassure "save 8")
             (setq pt1 p1)
             (if (and dlTremabrk brk_e1)
               (progn (command "_.BREAK" brk_e1a savpt5 savpt6))
             )
             (if (and dlTremabrk brk_e2a)
               (progn
                 (if (eq brk_e1a brk_e2a)
                   (progn (entdel (nth 0 wnamesa)) (dl_ved "brk_e2a" nextpt) (entdel (nth 0 wnamesa)))
                 )
                 (command "_.BREAK" brk_e2a savpt7 savpt8)
               )
             )
            )
            ;;(dl_l01 bent1 node p1 p2 pt)
            ((= (rem uctra 2.0) 0)
             (setq fang nil)
             (setq p1 (dl_dl2a pt1))
             (setq pt2 (if (dl_l01 brk_e2a "7" pt2 pt1 strtpt)
                 ipt
                 savpt7
               )
             )
             (Cassure "save 7a")
             (setq pt1 p1)
             (if flgn
           (progn (setq tmp1 (nth flgn wnamesa)
                    ent1 (entget tmp1)
              )
              (if (= (dl_val 0 tmp1) "LINE")
                (setq pt2 (dl_mls nil 10))
                (setq pt2 (dl_mas T nil pt2 pt1 nil))
              )
           )
             )
            )
            (T
             (setq p1 (dl_dl2a pt1))
             (setq pt2 (if (dl_l01 brk_e2a "8" pt2 pt1 nextpt)
                 ipt
                 savpt8
               )
             )
             (Cassure "save 8a")
             (setq pt1 p1)
             (if flgn
           (progn (setq tmp1   (nth flgn wnamesa)
                    ent1   (entget tmp1)
                    brk_e1 nil
                    brk_e2 nil
              )
              (if (= (dl_val 0 tmp1) "LINE")
                (setq pt2 (dl_mls nil 10))
                (setq pt2 (dl_mas T nil pt2 pt1 nil))
              )
           )
             )
             (if (and dlTremabrk brk_e2a)
           (progn (command "_.BREAK" brk_e2a savpt7 savpt8))
             )
            )
          )
          (if (= bIMur T)
            (progn (LineMake pt1 pt2 sLay2)       ;Mur interieur 1
               (SetWallInfoTag sAppNameAA (entlast))
               (setq bIMur nil)
               (Cassure "murI1")
            )
            (progn (LineMake pt1 pt2 sLay4)       ;Mur interieur 2
               (SetWallInfoTag sAppNameAA (entlast))
               (setq bIMur T)
               (Cassure "murI2")
            )
          )
          (setq wnamesa (if (null wnamesa)
                  (list (setq elast (entlast)))
                  (append wnamesa (list (setq elast (entlast))))
                )
            uctra   (1+ uctra)
          )
          wnamesa
        )

        (defun dl_l01 (bent1 n p1 p2 pt / temp)
          (TraceIsOn ".dl_l01")
          (setq n (strcat "savpt" n))
          (setq spt nil)
          (if bent1
            (if (= (dl_val 0 bent1) "LINE")
          (progn (setq
               temp (inters (trans (dl_val 10 bent1) 0 1) (trans (dl_val 11 bent1) 0 1) p1 p2 nil) )
                 (if temp
               (set (read n) temp)
               (progn (set (read n) p1) (setq brk_e1 nil))
                 )
          )
          (progn (set (read n) (dl_ial bent1 p1 p2 pt))
                 (if spt
               (progn (setq ipt (eval (read n))) (set (read n) spt))
                 )
          )
            )
            (set (read n) p1)
          )
          (if spt
            T
            nil
          )
        )

        (defun dl_dl2 (npt)
          (TraceIsOn ".dl_dl2")
          (setq tmp1 (nth (- uctr 2) wnames)
            ent1 (entget tmp1)
          )
          (if (= (dl_val 0 tmp1) "LINE")
            (if (or (equal (angle strtpt nextpt)
                   (angle (trans (dl_val 10 tmp1) 0 1) (trans (dl_val 11 tmp1) 0 1))
                   0.001
                )
                (equal (angle strtpt nextpt)
                   (angle (trans (dl_val 11 tmp1) 0 1) (trans (dl_val 10 tmp1) 0 1))
                   0.001
                )
                (equal (+ (* 2 pi) (angle strtpt nextpt))
                   (angle (trans (dl_val 10 tmp1) 0 1) (trans (dl_val 11 tmp1) 0 1))
                   0.001
                )
            )
          (progn (setq brk_e2 nil)
                 (command "_.LINE" (trans (dl_val 11 tmp1) 0 1) pt1 "") pt1
          )
          (progn (dl_mls nil 11))
            )
            (dl_mas nil nil pt1 pt2 strtpt)
          )
        )

        (defun dl_dl2a (npt)
          (TraceIsOn ".dl_dl2a")
          (if (= restart "yes")
            (progn (setq uctra uctr) (setq restart nil))
          )
          (setq tmp1 (nth (- uctra 2) wnamesa)
            ent1 (entget tmp1)
          )
          (if (= (dl_val 0 tmp1) "LINE")
            (if (or (equal (angle strtpt nextpt)
                   (angle (trans (dl_val 10 tmp1) 0 1) (trans (dl_val 11 tmp1) 0 1))
                   0.001
                )
                (equal (angle strtpt nextpt)
                   (angle (trans (dl_val 11 tmp1) 0 1) (trans (dl_val 10 tmp1) 0 1))
                   0.001
                )
                (equal (+ (* 2 pi) (angle strtpt nextpt))
                   (angle (trans (dl_val 10 tmp1) 0 1) (trans (dl_val 11 tmp1) 0 1))
                   0.001
                )
            )
          (progn (setq brk_e2 nil) (command "_.LINE" (trans (dl_val 11 tmp1) 0 1) pt1 "") pt1)
          (progn (dl_mls nil 11))
            )
            (dl_mas nil nil pt1 pt2 strtpt)
          )
        )

        (defun dl_mls (flg2 nn / spt ept pt)
          (TraceIsOn ".dl_mls")
          (setq spt (trans (dl_val 10 tmp1) 0 1)
            ept (trans (dl_val 11 tmp1) 0 1)
          )
          (if flg2
            (progn (setq pt (dl_ial tmp
                        spt
                        ept
                        (if flgn
                      nextpt
                      strtpt
                        )
                    )
               )
            )
            (setq pt (inters spt ept pt1 pt2 nil))
          )
          (if pt
            (entmod (subst (cons nn (trans pt 1 0)) (assoc nn ent1) ent1))
            (setq pt pt2)
          )
          pt
        )

        (defun dl_ial (arc pt_1 pt_2 npt / d pi2 rad ang nang temp ipt)
          (TraceIsOn ".dl_ial")
          (setq cpt  (trans (dl_val 10 arc) (dl_val -1 arc) 1)
            pi2  (/ pi 2)
            ang  (angle pt_1 pt_2)
            nang (+ ang pi2)
            temp (inters pt_1 pt_2 cpt (polar cpt nang 1) nil)
            nang (angle cpt temp)
          )
          (setq d (distance cpt temp))
          (cond ((equal (setq rad (dl_val 40 arc)) d 0.01) (setq ipt temp))
            ((< rad d)
             (setq spt (polar cpt nang rad)
               ipt temp
             )
             (command "_.LINE" spt ipt "") ;                  ;Mur interieur
             ipt
            )
            (T
             (if (and dl_arc fang (> uctr 1))
               (setq npt (polar cpt fang rad))
             )
             (dl_g2p npt)
             (setq ipt (dl_bp arc pt_1 pt_2 ipt1 ipt2))
             (if fang
               (setq fang nil)
               (if dl_arc
                 (setq fang (angle cpt ipt))
               )
             )
             ipt
            )
          )
        )

        (defun dl_g2p (npt / temp l theta)
          (TraceIsOn ".dl_g2p")
          (if (equal d 0.0 0.01)
            (setq theta pi2
              nang  (+ ang pi2)
            )
            (setq l     (sqrt (abs (- (expt rad 2) (expt d 2))))
              theta (abs (atan (/ l d)))
            )
          )
          (setq ipt1 (polar cpt (- nang theta) rad))
          (setq ipt2 (polar cpt (+ nang theta) rad))
          (if (< (distance ipt2 npt) (distance ipt1 npt))
            (setq temp ipt1
              ipt1 ipt2
              ipt2 temp
            )
            (if (equal (distance ipt2 npt) (distance ipt1 npt) 0.01)
          (exit)
            )
          )
          ipt1
        )

        (defun dl_onl (sp ep pt / cpt sa ea ang)
          (TraceIsOn ".dl_onl")
          (if (inters sp
              ep
              pt
              (polar pt (+ (angle sp ep) (/ pi 2)) (/ (getvar "tracewid") 10))
              T
          )
            T
            nil
          )
        )

        (defun dl_ona (arc pt / cpt sa ea ang)
          (TraceIsOn ".dl_ona")
          (setq cpt (trans (dl_val 10 arc) (dl_val -1 arc) 1)
            sa  (dl_val 50 arc)
            ea  (dl_val 51 arc)
            ang (angle cpt pt)
          )
          (if (> sa ea)
            (if (or (and (> ang sa) (< ang (+ ea (* 2 pi)))) (and (> ang (- ea (* 2 pi))) (< ang ea)))
          T
          nil
            )
            (if (and (> ang sa) (< ang ea))
          T
          nil
            )
          )
        )

        (defun dl_bp (en1 p1 p2 pp1 pp2 / temp temp1 temp2)\
          (TraceIsOn ".dl_bp")
          (setq temp1 (dl_onl p1 p2 pp2)
            temp2 (dl_ona en1 pp2)
            temp  (if (or (= flg 1) (= flg 3))
                T
                nil
              )
          )
          (if (and temp1 temp2)
            (if (and (< uctr 2) (and brk_e1 brk_e2))
          pp1
          (if (and temp (not fang))
            pp1
            pp2
          )
            )
            pp1
          )
        )

        (defun dl_das (flgn orad nn / tmp1 ent1 pt ang)
          (TraceIsOn ".dl_das")
          (cond ((= uctr 0) (setq sent1 tmp) (dl_a01 brk_e1 "1" strtpt nil) (dl_a01 brk_e2 "3" nextpt T))
            ((= uctr 1)
             (setq sent1 tmp)
             (dl_a01 brk_e1 "2" strtpt nil)
             (dl_a01 brk_e2 "4" nextpt T)
             (dl_mae nil T)
             (dl_mae nil nil)
             (if (and dlTremabrk brk_e1)
               (progn (dl_mae T T) (dl_mae T nil) (command "_.BREAK" brk_e1 savpt1 savpt2))
             )
             (if (and dlTremabrk brk_e2)
               (progn (if (eq brk_e1 brk_e2)
                    (progn (entdel (nth 0 wnames))
                       (entdel (nth 1 wnames))
                       (dl_ved "brk_e2" nextpt)
                       (entdel (nth 0 wnames))
                       (entdel (nth 1 wnames))
                    )
                  )
                  (if (null brk_e1)
                    (progn (dl_mae T T) (dl_mae T nil))
                  )
                  (command "_.BREAK" brk_e2 savpt3 savpt4)
               )
             )
            )
            ((= (rem uctr 2.0) 0)
             (setq fang nil)
             (dl_da2)
             (if fang
               (setq ftmp fang
                 fang nil
               )
             )
             (setq save_1 ent)
             (setq sent1 (cdr (assoc -1 ent)))
             (setq pt2 (dl_a01 brk_e2 "3" nextpt T))
             (if ftmp
               (setq fang ftmp
                 ftmp nil
               )
             )
            )
            (T
             (dl_da2)
             (if fang
               (setq ftmp fang
                 fang nil
               )
             )
             (setq save_2 ent)
             (setq sent1 (cdr (assoc -1 ent)))
             (setq pt2 (dl_a01 brk_e2 "4" nextpt T))
             (if ftmp
               (setq fang fang
                 ftmp nil
               )
             )
             (if (and dlTremabrk brk_e2)
               (progn (dl_mae T T) (dl_mae T nil) (command "_.BREAK" brk_e2 savpt3 savpt4))
             )
            )
          )
          (setq uctr (1+ uctr))
        )

        (defun dl_a01 (bent1 n pt flg / pt1 pt2 ang1 ang2 anga angb)
          (TraceIsOn ".dl_a01")
          (setq n (strcat "savpt" n))
          (if bent1
            (if (= (dl_val 0 bent1) "LINE")
          (progn (set (read n)
                  (dl_ial tmp (trans (dl_val 10 bent1) 0 1) (trans (dl_val 11 bent1) 0 1) pt)
                 )
          )
          (progn (setq curcpt (trans (dl_val 10 sent1) (dl_val -1 sent1) 1)
                   prvcpt (trans (dl_val 10 bent1) (dl_val -1 bent1) 1)
                   pt1    (polar prvcpt (dl_val 50 bent1) (dl_val 40 bent1))
                   pt2    (polar curcpt (dl_val nn sent1) (dl_val 40 sent1))
                   ang1   (angle prvcpt pt1)
                 )
                 (if (not (equal ang1 (angle prvcpt strtpt) 0.01))
               (setq pt1  (polar prvcpt (dl_val 51 bent1) (dl_val 40 bent1))
                     ang1 (angle prvcpt pt1)
                     ang2 (angle curcpt pt2)
                     anga (- ang1 ang2)
                     angb (- ang2 ang1)
               )
                 )
                 (if (or (and (< anga 0.0872665) (> anga -0.0872665))
                     (and (< angb 0.0872665) (> angb -0.0872665))
                 )
               (progn (set (read n) pt)
                  (if (= bent1 brk_e1)
                    (setq brk_e1 nil)
                    (setq brk_e2 nil)
                  )
               )
               (set (read n) (dl_iaa sent1 bent1 pt flg))
                 )
          )
            )
            (progn (setq cpt (trans (dl_val 10 tmp) (dl_val -1 tmp) 1))
               (set (read n) (polar cpt (angle cpt pt) orad))
            )
          )
          (eval (read n))
        )

        (defun dl_da2 (/ pt)
          (TraceIsOn ".dl_da2")
          (setq tmp1 (nth (- uctr 2) wnames)
            ent1 (entget tmp1)
          )
          (if (= (dl_val 0 tmp1) "LINE")
            (setq pt (dl_mls T 11))
            (setq pt (dl_mas nil T nil nil strtpt))
          )
          (if pt
            (progn (setq ang (- (angle cpt pt) ange))
               (entmod (setq ent (subst (cons nn ang) (assoc nn ent) ent)))
            )
          )
          (if flgn
            (progn (setq tmp1 (nth flgn wnames)
                 ent1 (entget tmp1)
               )
               (if (= (dl_val 0 tmp1) "LINE")
                 (setq pt (dl_mls T 10))
                 (setq pt (dl_mas T T nil nil nextpt))
               )
               (if pt
                 (progn (setq ang (- (angle cpt pt) ange))
                    (setq nn (if (= nn 50)
                       51
                       50
                         )
                    )
                    (entmod (setq ent (subst (cons nn ang) (assoc nn ent) ent)))
                 )
               )
            )
          )
        )

        (defun dl_mae (eflg sflg / nn1 nn2)
          (TraceIsOn ".dl_mae")
          (if (= nn 50)
            (setq nn1 50
              nn2 51
            )
            (setq nn1 51
              nn2 50
            )
          )
          (if sflg
            (if eflg
          (setq save_1 (subst (cons nn2
                        (angle (trans cpt 1 (cdr (assoc -1 save_1)))
                           (trans savpt3 1 (cdr (assoc -1 save_1)))
                        )
                      )
                      (assoc nn2 save_1)
                      save_1
                   )
          )
          (setq save_1 (subst (cons nn1
                        (angle (trans cpt 1 (cdr (assoc -1 save_1)))
                           (trans savpt1 1 (cdr (assoc -1 save_1)))
                        )
                      )
                      (assoc nn1 save_1)
                      save_1
                   )
          )
            )
            (if eflg
          (setq save_2 (subst (cons nn2
                        (angle (trans cpt 1 (cdr (assoc -1 save_1)))
                           (trans savpt4 1 (cdr (assoc -1 save_2)))
                        )
                      )
                      (assoc nn2 save_2)
                      save_2
                   )
          )
          (setq save_2 (subst (cons nn1
                        (angle (trans cpt 1 (cdr (assoc -1 save_1)))
                           (trans savpt2 1 (cdr (assoc -1 save_2)))
                        )
                      )
                      (assoc nn1 save_2)
                      save_2
                   )
          )
            )
          )
          (if sflg
            (entmod save_1)
            (entmod save_2)
          )
        )

        (defun dl_mas (flg3 flg2 spt ept pt / nnn pt1 pt2 rad1 ange)
          (TraceIsOn ".dl_mas")
          (setq cpt1 (trans (dl_val 10 tmp1) (dl_val -1 tmp1) 1)
            rad1 (dl_val 40 tmp1)
            ang1 (dl_val 50 tmp1)
          )
          (if (null pt)
            (setq pt (nth 0 spts))
          )
          (setq ange (trans '(1 0 0) (dl_val -1 tmp1) 1)
            ange (angle '(0 0 0) ange)
            ang1 (+ ang1 ange)
          )
          (if (> ang1 (* 2 pi))
            (setq ang1 (- ang1 (* 2 pi)))
          )
          (if (equal (angle cpt1 pt) ang1 0.01)
            (setq nnn 50)
            (setq nnn 51)
          )
          (if flg2
            (progn (setq pt1 (dl_iaa tmp
                         tmp1
                         (if flg3
                       nextpt
                       strtpt
                         )
                         flg2
                     )
               )
               (if pt1
                 (progn (setq ang1 (- (angle cpt1 pt1) ange))
                    (setq ent1 (subst (cons nnn ang1) (assoc nnn ent1) ent1))
                    (entmod ent1)
                 )
               )
            )
            (progn (setq pt1 (dl_ial tmp1 spt ept pt))
               (setq ang1 (- (angle cpt1 pt1) ange))
               (setq ent1 (subst (cons nnn ang1) (assoc nnn ent1) ent1))
               (entmod ent1)
            )
          )
          pt1
        )

        (defun dl_iaa (en1 en2 npt flga / a b c s ang alpha alph ipt curcpt prvcpt temp temp1 temp2)
          (TraceIsOn ".dl_iaa")
          (setq curcpt (trans (dl_val 10 en1) (dl_val -1 en1) 1)
            prvcpt (trans (dl_val 10 en2) (dl_val -1 en2) 1)
            a  (dl_val 40 en2)
            b  (distance curcpt prvcpt)
            c  (dl_val 40 en1)
            s  (/ (+ a b c) 2.0)
            ang    (angle curcpt prvcpt)
          )
          (cond ((or (= (- s a) 0) (equal b (+ a c) 0.001) (equal b (abs (- a c)) 0.001))
             (setq ipt nil)
            )
            ((and (or (> b (+ a c))
                  (if (> c a)
                    (< (+ a b) c)
                    (< (+ c b) a)
                  )
              )
              (not (equal (+ a b) c (/ (+ a b c) 1000000)))
             )
             (if (= flg 4)
               (progn (setq ipt (polar curcpt (angle curcpt prvcpt) c))       ;Mur interieur
                  (command "_.LINE" (polar prvcpt (angle prvcpt ipt) a) ipt "")
                  (SetWallInfoTag sAppNameAA (entlast))

               )
               (progn (setq ipt (polar curcpt (angle curcpt prvcpt) c))       ;Mur interieur
                  (command "_.LINE" (polar prvcpt (angle prvcpt ipt) a) ipt "")
                  (SetWallInfoTag sAppNameAA (entlast))
               )
             )
            )
            (T
             (setq alpha (* 2.0
                    (atan (sqrt (abs (/ (* (- s b) (- s c)) (* s (- s a))))))
                     )
             )
             (setq tpt1 (polar curcpt (+ ang alpha) c)
               tpt2 (polar curcpt (- ang alpha) c)
               anga (angle curcpt npt)
               angb (angle prvcpt npt)
             )
             (if (and dl_arc fang (> uctr 1))
               (setq npt (polar prvcpt fang c))
             )
             (if (< (distance tpt1 npt) (distance tpt2 npt))
               (setq temp tpt1
                 tpt1 tpt2
                 tpt2 temp
               )
             )
             (setq temp (angle prvcpt curcpt))
             (setq ipt (dl_bap en1 en2 tpt2 tpt1 nil))
             (if fang
               (setq fang nil)
               (if dl_arc
                 (setq fang (angle cpt ipt))
               )
             )
            )
          )
          (setq cpt curcpt)
          (setq cpt1 prvcpt)
          ipt
        )

        (defun dl_bap (en1 en2 pp1 pp2 flg / temp1 temp2)
          (TraceIsOn ".dl_bap")
          (setq temp1 (dl_ona en1 pp2)
            temp2 (dl_ona en2 pp2)
          )
          (if temp2
            (if (and (< uctr 2) (and brk_e1 brk_e2))
          pp1
          (if temp1
            (if (< uctr 2)
              pp2
              (if (not fang)
                pp2
                pp1
              )
            )
            pp1
          )
            )
            pp1
          )
        )

        (defun dl_atl ()
          (setq wnames (if (null wnames)
                 (list (entlast))
                 (append wnames (list tmp))
               )
          )
          wnames
        )

        (defun dl_atl1 ()
          (TraceIsOn ".dl_atl1")
          (setq wnamesa (if (null wnamesa)
                  (list (entlast))
                  (append wnamesa (list tmp1a))
                )
          )
          wnamesa
        )

        (defun dl_val (v temp)
          (TraceIsOn ".dl_val")
          (cdr (assoc v (entget temp)))
        )

        (defun dl_lsu (lst v / m)
          (setq m    0
            temp '()
          )
          (repeat (- (length lst) v)
            (progn (setq temp (append temp (list (nth m lst)))) (setq m (1+ m)))
          )
          temp
        )

        (defun endcap ()
          (setq dlTremaecp (ukword 1
                             "Auto Both End None Start"
                             "Capping ends to show? [Auto/Both/End/None/Start] "
                             dlTremacap
                           )
                dlTremacap dlTremaecp
          )
          (prompt (strcat "\nCurrent settings: " sOverall "mm wall"))
          (prompt (strcat " Break=" dlTremaabk))
          (prompt (strcat " Cappings=" dlTremacap))
          (prompt (strcat " Justification=" dlTrematzb))
          (cond ((= dlTremaecp "None")  (setq dlTremaecp 0))
            ((= dlTremaecp "Start") (setq dlTremaecp 1))
            ((= dlTremaecp "End")   (setq dlTremaecp 2))
            ((= dlTremaecp "Both")  (setq dlTremaecp 3))
            ((= dlTremaecp "Auto")  (setq dlTremaecp 4))
          )
        )

        (if (null dlTremaecp)
          (setq dlTremaecp 0)         ;was 4
        )
        (if (null dlTremasnp)
          (setq dlTremasnp T)
        )
        (if (null dlTremabrk)
          (setq dlTremabrk T)
        )
        (if (null dlTremaosd)
          (setq dlTremaosd 0)
        )
        (dline)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (TraceIsOn ".19c")

      ;-------------------------------
      ;---Ter Boundary Offset routine.
      ;-------------------------------
      ; This program will return T if the given argument is a point.
      (defun ReturnPtp (pt / )
        (TraceIsOn ".ReturnPtp")
        (if (and (listp pt)
                 (>= (length pt) 2)
                 (<= (length pt) 3)
                 (= (type (car pt)) 'real)
                 (= (type (cadr pt)) 'real)
                 (member (type (caddr pt)) '(real nil))
            )
          T
        )
      )

      (defun WallBdary (/ BDRY bapt DBLAST BfSET)
        (TraceIsOn ".WallBdary")
        (if (setq BfSET (ssget "_i"))
          (princ "\nUsing current existing boundary set.")
        )
        (command "_.undo" "_g" "_.point" "0,0")
        (setq DBLAST (entdel (entlast)))
        (while (and (setq bapt (upoint 6 "exit Undo" "Select internal point or [Undo]" nil nil))
                (/= bapt "exit")
           )
          (cond ((eq bapt "Undo")
             (if (entnext DBLAST)
               (entdel (entlast))
               (princ "\nCommand has been completely undone.")
             )
            )
            ((setq BDRY (BPOLY bapt BfSET))
             ;;(command "_.CHPROP" BDRY "" "_LA" sLAy1 "")
             (vla-put-layer (vlax-Ename->vla-Object BDRY) sLAy1)
             (setq ew1 (entlast))
             (SetWallInfoTag sAppNameAA ew1)

             (command "_.offset" (read sWidth1) ew1 bapt "")              ;run the offset command
             (setq ew2 (entlast))
             (SetWallInfoTag sAppNameAA ew2)
             (vla-put-layer (vlax-Ename->vla-Object ew2) sLAy2)

             (if (= sTypes "DoubleW")
               (progn
                 (command "_.offset" (+ (read sWidth1) (read sCavite)) ew1 bapt "")
                 (setq ew3 (entlast))
                 (SetWallInfoTag sAppNameAA ew3)
                 (vla-put-layer (vlax-Ename->vla-Object ew3) sLAy4)

                 (command "_.offset" (read sOverall) ew1 bapt "")
                 (setq ew4 (entlast))
                 (SetWallInfoTag sAppNameAA ew4)
                 (vla-put-layer (vlax-Ename->vla-Object ew4) sLAy3)

                 (command "_.EXPLODE" ew3)
                 (command "_.EXPLODE" ew4)
               )
             )

             (command "_.EXPLODE" ew1)
             (command "_.EXPLODE" ew2)
            )
            (T (princ "\nValid boundary not found."))
          )
        )
        (princ)
      )

      (defun BOffset1 ( / bapt aire )
        (TraceIsOn ".BOffset1")
        (setq bapt nil  aire nil)
        (if (= odim nil)(setq odim 110))
        (toggle "osmode" 16384)               ;toggle 16384
        (WsFlashMess2
          (strcat "\nBoundary Offset." copyright)
          (strcat "\nBoundary Offset." copyright)
          (strcat "\nBoundary Offset." copyright)
          (strcat "\nBoundary Offset." copyright)
        )
        (WallBdary)
      )

      ;-----------------
      ;---Merged Cross.
      ;-----------------
      (defun c:MClean (/  Va90
                          $work  dists
                          edata  etype
                          FuzRound   get
                          SlopeGet
                          Tete   i
                          l0 ObjBlend
                          XTNet1 RidExtrem
                          perp   perps
                          pt0    pt1
                          pt2    pt3
                          pt4    pt5
                          pt6    slope
                          SortingC   ss
                          SsFGet Queue
                          wall1  wall2
                          walls  work
                       )

        (setq olderror *error*  *error* Wallserror)
        (Wallsgather_info)
        (setq clayer nil)

        (defun RidExtrem  (x)
          (cdr (reverse (cdr x)))
        )

        (defun XTNet1 (dist1 dist2)
          (cond ((cdr dist1)
             (XTNet2 (nth (cadar dist1)
                  wall1
                 )
                 (nth (cadr (last dist1)
                  )
                  wall1
                 )
                 (nth (cadar dist2)
                  wall2
                 )
                 (nth (cadr (last dist2)
                  )
                  wall2
                 )
             )
             (XTNet1 (RidExtrem dist1)
                 (RidExtrem dist2)
             )
            )
          )
        )

        (defun XTNet2 (a1 a2 b1 b2)
          (mapcar '(lambda (x l1 l2)
                 (setq pt1 (cadr l1)
                   pt2 (caddr l1)
                   pt3 (cadr l2)
                   pt4 (caddr l2)
                 )
                 (foreach l0 x
               (setq pt5 (cadr l0)
                     pt6 (caddr l0)
               )
               (command "._BREAK"
                    (car l0)
                    (inters pt5 pt6 pt1 pt2)
                    (inters pt5 pt6 pt3 pt4)
               )
                 )
               )
              (list (list a1 a2)
                (list b1 b2)
              )
              (list b1 a1)
              (list b2 a2)
          )
        )

        (defun GET (key alist)
          (if (atom key)
            (cdr (assoc key alist)
            )
            (mapcar '(lambda (x)
               (cdr (assoc x alist)
               )
                 )
                key
            )
          )
        )

        (defun FuzRound (x y)
          (< (abs (- x y)) 1.0e-6)
        )

        (defun SortingC (x)
          (cond ((null (cdr x)
             )
             x
            )
            (T
             (ObjBlend (SortingC  (Tete x
                        (1- (length x)
                        )
                  )
                )
                (SortingC (Queue x
                        (1- (length x)
                        )
                  )
                )
             )
            )
          )
        )

        (defun ObjBlend (a b)
          (cond ((null a)
             b
            )
            ((null b)
             a
            )
            ((< (caar a)
                (caar b)
             )
             (cons (car a)
               (ObjBlend  (cdr a)
                  b
               )
             )
            )
            (t
             (cons (car b)
               (ObjBlend  a
                  (cdr b)
               )
             )
            )
          )
        )

        (defun Tete (l n)
          (cond ((minusp n)
             nil
            )
            (t
             (cons (car l)
               (Tete (cdr l)
                     (- n 2)
               )
             )
            )
          )
        )

        (defun Queue (l n)
          (cond ((minusp n)
             l
            )
            (t
             (Queue (cdr l)
               (- n 2)
             )
            )
          )
        )

        (defun SlopeGet (pt1 pt2 / x)
          (if (FuzRound (setq x (abs (- (car pt1)
                         (car pt2)
                      )
                     )
                 )
                 0.0
          )
            nil
            (rtos (/ (abs (- (cadr pt1)
                     (cadr pt2)
                  )
                 )
                 x
              )
              2
              4
            )
          )
        )

        (defun ETYPE (edata match)
          (member (get 0 edata)
              (if (listp match)
                match
                (list match)
              )
          )
        )

        (defun SsFGet (ss func / i ename)
          (setq i -1)
          (while (setq ename (ssname ss
                         (setq i (1+ i)
                         )
                     )
             )
            (apply func nil)
          )
        )

        (defun PERP (pt0 pt1 pt2)
          (inters pt1
              pt2
              pt0
              (polar pt0
                 (+ (angle pt1 pt2)
                    Va90
                 )
                 1.0
              )
              nil
          )
        )

        ;Specify first corner point
        (setq Va90 (/ pi 2) )

        (while (progn
                 (setq pt0
                    (upoint
                  8
                  "Select"
                  "Select wall objects to clean by crossing-window or [Select] <Exit>"
                  nil
                  nil
                     )
                 )
           )
          (setq dists nil
            perps nil
            walls nil
          )
          (cond
            ((eq (type pt0) 'LIST)
             (initget 33)
             (setq pt1 (getcorner pt0 "\nSpecify opposite corner: ")
               ss  (ssget "_C" pt0 pt1)
             )
            )
            (T
             (while (progn (princ "\nSelect objects: ")
                   (command "._SELECT" pause)
                   (not (setq ss (ssget "_P")
                        )
                   )
                )
               (print "No object found...")
             )
                 (setq pt0
                    (upoint
                  1
                  ""
                  "Specify point on outside of wall"
                  nil
                  nil
                     )
                 )
            )
          )
          (command ".UNDO" "_Group")
          (SsFGet ss
              '(lambda ()
                 (setq edata (entget ename)
                 )
                 (if (etype edata "LINE")
               (setq edata (get '(-1 10 11) edata)
                     slope (SlopeGet (cadr edata)
                             (caddr edata)
                       )
                     walls (if (setq temp (assoc slope walls)
                       )
                         (subst (append temp
                                (list edata)
                            )
                            temp
                            walls
                         )
                         (cons (cons slope
                             (list edata)
                           )
                           walls
                         )
                       )
               )
                 )
               )
          )
          (cond ((< (length walls)
                2
             )
             (princ "\rError: Use J to join colinear walls.")
            )
            ((> (length walls)
                2
             )
             (princ "\rError: Only two walls may be cleaned.")
            )
            ((not (apply '=
                     (mapcar 'length walls)
              )
             )
             (princ "\rError: Walls with unequal number of lines.")
            )
            (T
             (setq perps (mapcar '(lambda (x)
                        (mapcar '(lambda (y)
                               (perp pt0
                                 (cadr y)
                                 (caddr y)
                               )
                             )
                            (cdr x)
                        )
                      )
                         walls
                     )
             )
             (setq dists (mapcar '(lambda (x)
                        (setq i 0)
                        (mapcar '(lambda (y)
                               (list (distance pt0 y)
                                 (setq i (1+ i)
                                 )
                               )
                             )
                            x
                        )
                      )
                         perps
                     )
             )
             (setq dists (mapcar 'SortingC dists)
             )
             (setq wall1 (car walls)
               wall2 (cadr walls)
             )
             (XTNet1 (car dists)
                 (cadr dists)
             )
            )
          )
          (command "_.UNDO" "_End")
        )
        (reset nil)
        (resset varlst)
      )

      ;-----------------
      ;---Corner Joint.
      ;-----------------
      (defun c:cClean (/ Va90
                         $work   ang1
                         ang2    dists
                         edata   eqpnt
                         etype   FuzRound
                         get     SlopeGet
                         Tete    ObjBlend
                         perp    perps
                         pt0     pt1
                         TNetL   slope
                         SortingC    ss
                         SsFGet  Queue
                         temp    walls
                         work
                      )

        (setq olderror *error*  *error* Wallserror)
        (Wallsgather_info)
        (setq clayer nil)

        (defun TNetL (dist1 dist2 / line1 line2 pt1 pt2 pt3 pt4 ipt pp1 pp2 x y group edata enm)
          (mapcar '(lambda (x y)
                 (setq x     (cadr x)
                   y     (cadr y)
                   line1 (nth x
                      (car walls)
                     )
                   line2 (nth y
                      (cadr walls)
                     )
                   pt1   (cadr line1)
                   pt2   (caddr line1)
                   pt3   (cadr line2)
                   pt4   (caddr line2)
                   pp1   (nth (1- x)
                      (car perps)
                     )
                   pp2   (nth (1- y)
                      (cadr perps)
                     )
                   ipt   (inters pt1 pt2 pt3 pt4 nil)
                 )
                 (mapcar '(lambda (enm pt1 pt2 pp)
                    (setq ang1  (angle pp ipt)
                      ang2  (angle pt1 pt2)
                      group (if (eqpnt (polar ipt ang1 1.0)
                               (polar ipt ang2 1.0)
                            )
                          11
                          10
                            )
                      edata (entget enm)
                    )
                    (entmod (subst (cons group ipt)
                           (assoc group edata)
                           edata
                        )
                    )
                  )
                     (list (car line1)
                       (car line2)
                     )
                     (list pt1 pt3)
                     (list pt2 pt4)
                     (list pp1 pp2)
                 )
               )
              dist1
              dist2
          )
        )

        (defun FuzRound (x y)
          (< (abs (- x y)
             )
             1.0e-6
          )
        )

        (defun EQPNT (p1 p2)
          (< (distance p1 p2)
             1.0e-6
          )
        )

        (defun GET (key alist)
          (if (atom key)
            (cdr (assoc key alist)
            )
            (mapcar '(lambda (x)
               (cdr (assoc x alist)
               )
                 )
                key
            )
          )
        )

        (defun SortingC (x)
          (cond ((null (cdr x)
             )
             x
            )
            (T
             (ObjBlend (SortingC  (Tete x
                        (1- (length x)
                        )
                  )
                )
                (SortingC (Queue x
                        (1- (length x)
                        )
                  )
                )
             )
            )
          )
        )

        (defun ObjBlend (a b)
          (cond ((null a)
             b
            )
            ((null b)
             a
            )
            ((< (caar a)
                (caar b)
             )
             (cons (car a)
               (ObjBlend  (cdr a)
                  b
               )
             )
            )
            (t
             (cons (car b)
               (ObjBlend  a
                  (cdr b)
               )
             )
            )
          )
        )

        (defun Tete (l n)
          (cond ((minusp n)
             nil
            )
            (t
             (cons (car l)
               (Tete (cdr l)
                     (- n 2)
               )
             )
            )
          )
        )

        (defun Queue (l n)
          (cond ((minusp n)
             l
            )
            (t
             (Queue (cdr l)
               (- n 2)
             )
            )
          )
        )

        (defun SlopeGet (pt1 pt2 / x)
          (if (FuzRound (setq x (abs (- (car pt1)
                         (car pt2)
                      )
                     )
                 )
                 0.0
          )
            nil
            (rtos (/ (abs (- (cadr pt1)
                     (cadr pt2)
                  )
                 )
                 x
              )
              2
              4
            )
          )
        )

        (defun ETYPE (edata match)
          (member (get 0 edata)
              (if (listp match)
                match
                (list match)
              )
          )
        )

        (defun SsFGet (ss func / i ename)
          (setq i -1)
          (while (setq ename (ssname ss
                         (setq i (1+ i)
                         )
                     )
             )
            (apply func nil)
          )
        )

        (defun PERP (pt0 pt1 pt2)
          (inters pt1
              pt2
              pt0
              (polar pt0
                 (+ (angle pt1 pt2)
                    Va90
                 )
                 1.0
              )
              nil
          )
        )

        (setq Va90 (/ pi 2))

        (while (progn
                 (setq pt0
                    (upoint
                  8
                  "Select"
                  "Specify first corner point as inside corner of walls to clean or [Select] <Exit>"
                  nil
                  nil
                     )
                 )
           )

          (setq dists nil
            perps nil
            walls nil
          )
          (cond
            ((eq (type pt0) 'LIST)
             (initget 33)
             (setq pt1 (getcorner pt0 "\nSpecify outside corner: ")
               ss  (ssget "_C" pt0 pt1)
             )
            )
            (T
             (while (progn (princ "\nSelect objects: ")
                   (command "_.SELECT" pause)
                   (not (setq ss (ssget "_P")
                        )
                   )
                )
               (print "No object found...")
             )
                 (setq pt0
                    (upoint
                  1
                  ""
                  "Specify point on inside of wall"
                  nil
                  nil
                     )
                 )
            )
          )
          (command "_.UNDO" "_Group")
          (SsFGet ss
              '(lambda ()
                 (setq edata (entget ename)
                 )
                 (if (etype edata "LINE")
               (setq edata (get '(-1 10 11) edata)
                     slope (SlopeGet (cadr edata)
                             (caddr edata)
                       )
                     walls (if (setq temp (assoc slope walls)
                       )
                         (subst (append temp
                                (list edata)
                            )
                            temp
                            walls
                         )
                         (cons (cons slope
                             (list edata)
                           )
                           walls
                         )
                       )
               )
                 )
               )
          )
          (cond ((< (length walls)
                2
             )
             (princ "\rError: Use J to join colinear walls.")
            )
            ((> (length walls)
                2
             )
             (princ "\rError: Only two walls may be cleaned.")
            )
            ((not (apply '=
                     (mapcar 'length walls)
              )
             )
             (princ "\rError: Walls with unequal number of lines.")
            )
            (T
             (setq perps (mapcar '(lambda (x)
                        (mapcar '(lambda (y)
                               (perp pt0
                                 (cadr y)
                                 (caddr y)
                               )
                             )
                            (cdr x)
                        )
                      )
                         walls
                     )
             )
             (setq dists (mapcar '(lambda (x)
                        (setq i 0)
                        (mapcar '(lambda (y)
                               (list (distance pt0 y)
                                 (setq i (1+ i)
                                 )
                               )
                             )
                            x
                        )
                      )
                         perps
                     )
             )
             (setq dists (mapcar 'SortingC dists)
             )
             (TNetL (car dists)
                (cadr dists)
             )
            )
          )
          (command "_.UNDO" "_End")
        )
        (reset nil)
        (resset varlst)
      )

      ;---------------
      ;---Merged Tee.
      ;---------------
      (defun c:TClean (/ Va90
                         $work   ang
                         dist1   dist2
                         dists   edata
                         eqpnt   etype
                         FuzRound    get
                         getside SlopeGet
                         Tete    i
                         ObjBlend    TNet1
                         TNet2   TNet3
                         RidExtrem   perp
                         perps   pt0
                         pt1     pt2
                         pt3     pt4
                         slope   SortingC
                         ss  SsFGet
                         temp    Queue
                         wall1   wall2
                         walls   work
                         x   y
              )
        (setq olderror *error*  *error* Wallserror)
        (Wallsgather_info)

        (setq clayer nil)

        (defun RidExtrem  (x)
          (cdr (reverse (cdr x)))
        )

        (defun TNet1  (dist1 dist2 / x y line1 line2 pt1 pt2 ip1)
          (cond ((cdr dist1)
             (setq x (cadar dist1)
               y (cadr (last dist1)
                 )
             )
             (TNet2 (nth  (cadar dist2)
                  wall2
                 )
                 (nth x wall1)
                 (nth y wall1)
                 (nth (1- x)
                  perps
                 )
                 (nth (1- y)
                  perps
                 )
             )
             (TNet1 (RidExtrem dist1)
                 (cdr dist2)
             )
            )
            ((car dist1)
             (setq line1 (nth (cadar dist1)
                      wall1
                     )
               line2 (nth (cadar dist2)
                      wall2
                     )
               pt1   (cadr line1)
               pt2   (caddr line1)
               ip1   (inters pt1
                         pt2
                         (cadr line2)
                         (caddr line2)
                         nil
                     )
             )
             (TNet3 line1
                 ip1
                 (nth (1- (cadar dist1)
                  )
                  perps
                 )
             )
            )
            (T nil)
          )
        )

        (defun TNet2  (line1 line2 line3 pp2 pp3 / pt1 pt2 ip2 ip3)
          (setq pt1 (cadr line1)
            pt2 (caddr line1)
            ip2 (inters pt1
                    pt2
                    (cadr line2)
                    (caddr line2)
                    nil
                )
            ip3 (inters pt1
                    pt2
                    (cadr line3)
                    (caddr line3)
                    nil
                )
          )
          (command "_.BREAK"
               (car line1)
               ip2
               ip3
          )
          (TNet3 line2 ip2 pp2)
          (TNet3 line3 ip3 pp3)
        )

        (defun TNet3  (line1 ip1 pp1 / edata group ang1 ang2)
          (setq pt1   (cadr line1)
            pt2   (caddr line1)
            ang1  (angle pp1 ip1)
            ang2  (angle pt1 pt2)
            group (if (eqpnt (polar ip1 ang1 1.0)
                     (polar ip1 ang2 1.0)
                  )
                11
                10
              )
            edata (entget (car line1)
              )
          )
          (entmod (subst (cons group ip1)
                 (assoc group edata)
                 edata
              )
          )
        )

        (defun GETSIDE (pt0 pp1 pp2 / temp)
          (setq temp (- (angle pt0 pp2)
                        (angle pt0 pp1)
                     )
          )
          (if
            ( (if (minusp temp)  ;; shall result in
                <
                >
              )
              (abs temp)
              pi
            )
            nil
            T
          )
        )

        (defun FuzRound (x y)
          (< (abs (- x y)) 1.0e-6)
        )

        (defun EQPNT (p1 p2)
          (< (distance p1 p2) 1.0e-6)
        )

        (defun GET (key alist)
          (if (atom key)
            (cdr (assoc key alist))
            (mapcar '(lambda (x)
                       (cdr (assoc x alist))
                     )
                     key
            )
          )
        )

        (defun SortingC (x)
          (cond ((null (cdr x)
             )
             x
            )
            (T
             (ObjBlend (SortingC  (Tete x
                        (1- (length x)
                        )
                  )
                )
                (SortingC (Queue x
                        (1- (length x)
                        )
                  )
                )
             )
            )
          )
        )

        (defun ObjBlend (a b)
          (cond ((null a)
             b
            )
            ((null b)
             a
            )
            ((< (caar a)
                (caar b)
             )
             (cons (car a)
               (ObjBlend  (cdr a)
                  b
               )
             )
            )
            (t
             (cons (car b)
               (ObjBlend  a
                  (cdr b)
               )
             )
            )
          )
        )

        (defun Tete (l n)
          (cond ((minusp n)
             nil
            )
            (T
             (cons (car l)
               (Tete (cdr l)
                     (- n 2)
               )
             )
            )
          )
        )

        (defun Queue (l n)
          (cond ((minusp n)
             l
            )
            (T
             (Queue (cdr l)
               (- n 2)
             )
            )
          )
        )

        (defun SlopeGet (pt1 pt2 / x)
          (if (FuzRound (setq x (abs (- (car pt1)
                         (car pt2)
                      )
                     )
                 )
                 0.0
          )
            nil
            (rtos (/ (abs (- (cadr pt1)
                     (cadr pt2)
                  )
                 )
                 x
              )
              2
              4
            )
          )
        )

        (defun ETYPE (edata match)
          (member (get 0 edata)
              (if (listp match)
                match
                (list match)
              )
          )
        )

        (defun SsFGet (ss func / i ename)
          (setq i -1)
          (while (setq ename (ssname ss
                         (setq i (1+ i)
                         )
                     )
             )
            (apply func nil)
          )
        )

        (defun PERP (pt0 pt1 pt2)
          (inters pt1
              pt2
              pt0
              (polar pt0
                 (+ (angle pt1 pt2)
                    Va90
                 )
                 1.0
              )
              nil
          )
        )

        (setq Va90 (/ pi 2))

        (while (progn
                 (setq pt0
                    (upoint
                  8
                  "Select"
                  "Specify first corner point left of leg wall or [Select] <Exit>"
                  nil
                  nil
                     )
                 )
           )
          (setq dists nil
            perps nil
            walls nil
          )
          (cond
            ((eq (type pt0) 'LIST)
             (initget 33)
             (setq pt1 (getcorner pt0 "\nSpecify opposite corner: ")
               ss  (ssget "_C" pt0 pt1)
             )
            )
            (T
             (while (progn (princ "\nSelect objects: ")
                   (command "_.SELECT" pause)
                   (not (setq ss (ssget "_P")
                        )
                   )
                )
               (print "No object found...")
             )
                 (setq pt0
                    (upoint
                  1
                  "Select"
                  "Specify a point left of leg wall"
                  nil
                  nil
                     )
                 )
            )
          )
          (command "_.UNDO" "_Group")
          (SsFGet ss
              '(lambda ()
                 (setq edata (entget ename)
                 )
                 (if (etype edata "LINE")
               (setq edata (get '(-1 10 11) edata)
                     slope (SlopeGet (cadr edata)
                             (caddr edata)
                       )
                     walls (if (setq temp (assoc slope walls)
                       )
                         (subst (append temp
                                (list edata)
                            )
                            temp
                            walls
                         )
                         (cons (cons slope
                             (list edata)
                           )
                           walls
                         )
                       )
               )
                 )
               )
          )
          (cond ((< (length walls)
                2
             )
             (princ "\rError: Use J to join colinear walls.")
            )
            ((> (length walls)
                2
             )
             (princ "\rError: Sorry, Only two walls may be cleaned.")
            )
            ((not (apply '=
                     (mapcar 'length walls)
              )
             )
             (princ "\rError: Walls with unequal number of lines.")
            )
            (T
             (setq perps (mapcar '(lambda (x)
                        (mapcar '(lambda (y)
                               (perp pt0
                                 (cadr y)
                                 (caddr y)
                               )
                             )
                            (cdr x)
                        )
                      )
                         walls
                     )
             )
             (setq dists (mapcar '(lambda (x)
                        (setq i 0)
                        (mapcar '(lambda (y)
                               (list (distance pt0 y)
                                 (setq i (1+ i)
                                 )
                               )
                             )
                            x
                        )
                      )
                         perps
                     )
             )
             (setq dists (mapcar 'SortingC dists)
             )
             (cond ((getside pt0
                     (caar perps)
                     (caadr perps)
                )
                (setq perps (car perps)
                  wall1 (car walls)
                  wall2 (cadr walls)
                  dist1 (car dists)
                  dist2 (cadr dists)
                )
               )
               (T
                (setq perps (cadr perps)
                  wall1 (cadr walls)
                  wall2 (car walls)
                  dist1 (cadr dists)
                  dist2 (car dists)
                )
               )
             )
             (setq line1 (cadr wall1)
               line2 (cadr wall2)
               pt1   (cadr line1)
               pt2   (caddr line1)
               pt3   (cadr line2)
               pt4   (caddr line2)
               ang   (angle pt1 pt2)
               pt0   (inters pt1 pt2 pt3 pt4 nil)
             )
             (cond ((inters pt3
                    pt4
                    pt0
                    (polar pt0 ang 1.0)
                )
                (TNet1 dist1 dist2)
               )
               (T
                (princ "\rError: Unable to clean-up specified intersection.")
               )
             )
            )
          )
          (command "_.UNDO" "_End")
        )
        (reset nil) (resset varlst)
      )

      (defun c:clean (/ answer)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\nWall Clean Up." copyright)
          (strcat "\nWall Clean Up." copyright)
          (strcat "\nWall Clean Up." copyright)
          (strcat "\nWall Clean Up." copyright)
        )
        (setq answer (ukword 1 "Corner Merged Tee" "Enter an option? [Corner/Merged cross/Tee] " "Corner"))
        (if (and (/= answer "Corner") (/= answer "Tee"))
          (progn (c:MClean))
          (progn
            (if (= answer "Corner")
              (progn (c:CClean))
            )
            (if (= answer "Tee")
              (progn (c:TClean))
            )
          )
        )
      )

      (TraceIsOn ".19d")

      ;------------------------
      ;---Visual Section Marks.
      ;------------------------
      (defun C:SM ()    (SectSymb))

      (defun C:FlushSM (/)
        (if (/= nil (getdictname "AA_SectNum")) (deldict "AA_SectNum") )
        (princ "\nStored info regarding Section Marks deleted!\n")
        (princ)
      )

      (defun Liste4Pline (plyline / v1 vertg)
        (setq v1 (entnext plyline)  Plylist nil)
        (if (/= v1 nil)
          (progn (setq vertg (entget v1))
            (while (/= (dxf 0 vertg) "SEQEND")
              (if
                (and
                 (/= (dxf 70 vertg) 16)            ;Skip control vertices for 2D spline curves
                 (/= (dxf 70 vertg) 48)            ;Skip control vertices for 3D spline curves
                )
                (progn
                  (setq Plylist (cons (dxf 10 vertg) Plylist))
                )
              )
              (setq v1 (entnext v1)  vertg (entget v1))
            )
          )
        )
        (reverse Plylist)
      )

      (defun Liste4LwPl (plyline / vert vertg)
        (setq vert (entget plyline)  Plylist nil  Plylist1 nil)
        (setq cntr 0)
        (setq Plylist ())
        (repeat (length vert)
           (if (= (car (nth cntr vert)) 10)
             (progn
              (setq Plylist1 (list
                             (dxf 38 vert)
                             (car (cdr (cdr (nth cntr vert))))
                             (car (cdr (nth cntr vert)))))
              (setq Plylist (append (list (reverse Plylist1)) Plylist))
             )
           )
         (setq cntr (1+ cntr))
        )
        (reverse Plylist)
      )

      (defun SectSymb (/  value      tst      Cont     OffOn    sAns pSecLoc1 rSymbRad
                          pSecLoc2 pSecLoc3 aAngl    pKink1   aRevAngl pKink2   aAngle  aRevAngle
                          rX1 rY1 rX2 rY2 rX3 rY3 rOScale SectStyle NImag sDWG sSect Imag1 rOfDist)
        (setq olderror *error*  *error* Wallserror)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )

        (defun SelCross (pCentral rDist aAttRot / )
          (TraceIsOn ".SelCross")
          (setq pLleft  (polar pCentral (dtr 200) rDist)
              pTRight (polar pCentral (dtr 20)  rDist)
              i (- 1)
              ssA   (ssget "_C" pLleft pTRight (list (cons 0 "INSERT") (cons 66 1)))
          )
          (if ssA
            (progn
              (repeat (sslength ssA)
                (setq eName (ssname ssA (setq i (1+ i))))
                (while (/= (dxf 0 (entget eName)) "SEQEND")
                  (setq eType (dxf 0 (entget eName)))
                  (if (= eType "ATTRIB")
                     (progn
                    (setq eListe  (entget eName)
                              lUpdate (subst (cons 50 aAttRot) (assoc 50 eListe) eListe)
                    )
                    (entmod lUpdate)
                    (entupd eName)
                     )
                  )
                  (setq eName (entnext eName))
                )
              )
            )
          )
        )

        (setvar "COORDS" 0)               ;COORDS sysvar is now saved in the registry
        (setvar "ATTDIA" 0)
        (setvar "ORTHOMODE" 0)
        (setvar "ATTREQ" 1)
        (WsFlashMess2
          (strcat "\nSection Marks." copyright)
          (strcat "\nSection Marks." copyright)
          (strcat "\nSection Marks." copyright)
          (strcat "\nSection Marks." copyright)
        )

        ;;LCA - WARNING: The UCSXDIR sysvar has changed.
        (setq pAlongX (getvar "UCSXDIR")  rOScale (getvar "DIMSCALE"))
        (setq sAns (ukword 1 "Draw Select" "Enter an option for the cutting plane? [Draw/Select] " "Select") )

        (defun c:SecSymbol_OnInitialize ()
          (TraceIsOn ".SecSymbol_OnInitialize")
          (if (= sAns "Draw")
            (Odcl_Control_SetProperty XpressForm_SecSymbol_KinkCheckBox "Enabled" T)
            (Odcl_Control_SetProperty XpressForm_SecSymbol_KinkCheckBox "Enabled" nil)
          )
          (if (= nil (getdictname "AA_SectNum"))
            (progn
              (createdict "AA_SectNum")
              (setq sSect     "65" ;chr 65 = "A", section number
                    sDWG      "-"  ;dwg number
              )
              (addrecord "AA_SectNum" "SectID" (list (cons 300 sSect)))       ;section number
              (addrecord "AA_SectNum" "CNType" (list (cons 300 "C")))         ;character or number
              (addrecord "AA_SectNum" "DwgID"  (list (cons 300 sDWG)))        ;dwg number
              (cond
                ((= sArStyle "Style1") (setq NImag "101") )
                ((= sArStyle "Style2") (setq NImag "102") )
                ((= sArStyle "Style3") (setq NImag "104") )
                ((= sArStyle "Style4") (setq NImag "105") )
              )
              (addrecord "AA_SectNum" "StSect" (list (cons 300 sArStyle)))        ;style
              (addrecord "AA_SectNum" "ImSect" (list (cons 300 NImag)))       ;image
              (addrecord "AA_SectNum" "SUnits" (list (cons 300 sUnitSet)))        ;inch or mm
              (addrecord "AA_SectNum" "OnCutPlane" (list (cons 300 bKinkOnL)))    ;on or away kinks
            )
          )
          (if (= (dxf 300 (getrecord "AA_SectNum" "CNType")) "C")
            (setq sSect (chr (read (dxf 300 (getrecord "AA_SectNum" "SectID")))) ) ;C'est une lettre
            (setq sSect (dxf 300 (getrecord "AA_SectNum" "SectID")) )
          )
          (setq sDWG      (dxf 300 (getrecord "AA_SectNum" "DwgID")) )
          (setq SectStyle (dxf 300 (getrecord "AA_SectNum" "StSect")) )
          (setq NImag     (read (dxf 300 (getrecord "AA_SectNum" "ImSect"))) )
          (setq unite     (dxf 300 (getrecord "AA_SectNum" "SUnits")) )
          (setq bKink     (dxf 300 (getrecord "AA_SectNum" "OnCutPlane")) )
          (cond
            ((= NImag 102)
               (setq SectStyle "Style2")
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 80)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  56)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
            )
            ((= NImag 104)
               (setq SectStyle "Style3")
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 20)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  56)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
            )
            ((= NImag 105)
               (setq SectStyle "Style4")
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 88)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  35)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
            )
            ((= NImag 101)
               (setq SectStyle "Style1")
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 45)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  40)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 1)
            )
          )
          (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SymbolPictureBox" "Picture" NImag)
          (if (= unite "in")
            (progn
              (Odcl_Control_SetProperty ProjName0 "SecSymbol" "inOptionButton" "Value" 1)
              (Odcl_Control_SetProperty ProjName0 "SecSymbol" "mmOptionButton" "Value" 0)
            )
            (progn
              (Odcl_Control_SetProperty ProjName0 "SecSymbol" "inOptionButton" "Value" 0)
              (Odcl_Control_SetProperty ProjName0 "SecSymbol" "mmOptionButton" "Value" 1)
            )
          )
          (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Text" sSect)
          (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Text" sDWG)
          (if (= bKink "T")
            (Odcl_Control_SetProperty ProjName0 "SecSymbol" "KinkCheckBox" "Value" 1)
            (Odcl_Control_SetProperty ProjName0 "SecSymbol" "KinkCheckBox" "Value" 0)
          )
        )

        (defun c:SecSymbol_SymbolPictureBox_Clicked ()
          (setq Imag1 (Odcl_Control_GetProperty
                     ProjName0
                     "SecSymbol"
                     "SymbolPictureBox"
                     "Picture"
                    )
          )
          (cond
            ((= Imag1 101)                            ;d'architect
               (setq NImag 102  SectStyle "Style2")                   ;Derriere
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 80)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  56)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
              )
            ((= Imag1 102)                            ;de derriere
               (setq NImag 104  SectStyle "Style3")                   ;Devant
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 20)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  56)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
              )
            ((= Imag1 104)                            ;de devant
               (setq NImag 105  SectStyle "Style4")                   ;Americain
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 88)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  35)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
              )
            ((= Imag1 105)                            ;d'americain
               (setq NImag 101  SectStyle "Style1")                   ;Architectural
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 45)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  40)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 1)
              )
          )
          (Odcl_Control_SetProperty
                     ProjName0
                     "SecSymbol"
                     "SymbolPictureBox"
                     "Picture"
                   NImag
          )
        )

        (defun c:SecSymbol_SymbolPictureBox_RightClick ()
          (setq Imag1 (Odcl_Control_GetProperty
                     ProjName0
                     "SecSymbol"
                     "SymbolPictureBox"
                     "Picture"
                    )
          )
          (cond
            ((= Imag1 104)                            ;d'architect    104
               (setq NImag 102  SectStyle "Style2")                   ;Derriere
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 80)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  56)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
              )
            ((= Imag1 105)                            ;de derriere    105
               (setq NImag 104  SectStyle "Style3")                   ;Devant
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 20)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  56)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
              )
            ((= Imag1 101)                            ;de devant  101
               (setq NImag 105  SectStyle "Style4")                   ;Americain
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 88)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  35)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 0)
              )
            ((= Imag1 102)                            ;d'americain    102
               (setq NImag 101  SectStyle "Style1")                   ;Architectural
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Left" 45)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "SectIDTextBox" "Top"  40)
               (Odcl_Control_SetProperty ProjName0 "SecSymbol" "DWGnumTextBox" "Visible" 1)
              )
          )
          (Odcl_Control_SetProperty
                     ProjName0
                     "SecSymbol"
                     "SymbolPictureBox"
                     "Picture"
                   NImag
          )
        )

        (defun c:SecSymbol_Ok_Clicked ()
          (setq bKinkOnL
            (Odcl_Control_GetProperty
              ProjName0
              "SecSymbol"
              "KinkCheckBox"
              "Value"
            )
          )
          (setq bInch
            (Odcl_Control_GetProperty
              ProjName0
              "SecSymbol"
              "inOptionButton"
              "Value"
            )
          )
          (setq bMM
            (Odcl_Control_GetProperty
              ProjName0
              "SecSymbol"
              "mmOptionButton"
              "Value"
            )
          )
          (setq sSect
            (Odcl_Control_GetProperty
              ProjName0
              "SecSymbol"
              "SectIDTextBox"
              "Text"
            )
          )
          (setq sDWG
            (Odcl_Control_GetProperty
              ProjName0
              "SecSymbol"
              "DWGnumTextBox"
              "Text"
            )
          )

          (Odcl_Form_Close ProjName0 "SecSymbol")

          (if (= sAns "Draw")
            (progn
              (setq                   ;;;DBLAST   (entdel (entlast)) !!!
                Cont      T
                OffOn     0
                pSecLoc1  (upoint 7 "" "Specify location for 1st section mark" nil nil)
                rX1       (car pSecLoc1)
                rY1       (cadr pSecLoc1)
                pSecLoc2  (upoint
                            7
                            ""
                            "Specify location for 2nd section mark"
                            nil
                            pSecLoc1
                          )
                rX2      (car pSecLoc2)
                rY2      (cadr pSecLoc2)
              )

              (if bMM
                (progn
                  (setq rThickLine 10  rSymbRad 7.0  bMM T  bInch nil  Type1 "SectMark"  Type1a "TurnArrowmm"  unite "mm"  rOfDist 2.828427
                        Type2 "Style3mm"  Type3 "Style2mm"  Type4 "Style4mm"  LaySt1 "ACAD_ISO08W100"  LaySt1a "0.35"  LaySt2a "0.70")
                  (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                  (Hshset_lay "CutMark" "171" nil LaySt2a)
                )
                (progn
                  (setq rThickLine 0.45  rSymbRad 0.45  bMM nil  bInch T  Type1 "SectMarkin"  Type1a "TurnArrowin"  unite "in"  rOfDist 0.1414
                        Type2 "Style3in"  Type3 "Style2in"  Type4 "Style4in"  LaySt1 "PHANTOM2"  LaySt1a "0.031"  LaySt2a "0.062")
                  (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                  (Hshset_lay "CutMark" "171" nil LaySt2a)
                )
              )
              (if (= SectStyle "Style4")(setq rSymbRad 0)) ;****Type2 et 3 sont inverses****
              (princ "\nSpecify orientation: left-click to change, right-click to accept.\n")
              (while Cont
                (setq got nil)
                (setq tst (grread T 9 2))       ;renvoi
                (setvar "HIGHLIGHT" 1)
                (setq code  (car tst)
                      value (cadr tst)
                )
                (if (or (= value 0) (= value 32) (= code 25) (= value 13))          ;13=CR  32=space  27=ESC
                  (progn
                (setvar "COORDS" 1)
                (setq Cont nil
                      tst  '(2 4 6)
                      got  T
                )
                  )
                )
                (if (= code 3)
                  (setq OffOn (- 1 OffOn))
                )
                (if (= (type value) 'LIST)
                  (setq got (ssget value))
                )
                (if
                  (and (= Cont T) (= code 5))
                   (progn
                 (redraw)
                 (setq xx (car value)
                       yy (cadr value)
                 )
                 (cond
                   ((= 1 OffOn)
                    (setq pKink1    (list xx rY1 0.0)
                      pKink2    (list xx rY2 0.0)
                      aAngl     (angle pSecLoc1 pKink1)
                      aRevAngl  (angle pKink1 pSecLoc1)
                      aAngle    (angle pKink1 pKink2)
                      aRevAngle (angle pKink2 pKink1)
                      pStrt1    (polar pSecLoc1 aAngl (* rSymbRad rOScale))
                      pStop1    (polar pSecLoc1 aAngl (* (+ rThickLine rSymbRad) rOScale) )
                      pStrt2    (polar pKink1 aRevAngl (* rThickLine rOScale))
                      pStop2    (polar pKink1 aAngle (* rThickLine rOScale))
                      pStrt3    (polar pKink2 aRevAngle (* rThickLine rOScale))
                      pStop3    (polar pKink2 aAngl (* rThickLine rOScale))
                      pStrt4    (polar pSecLoc2 aRevAngl (* (+ rThickLine rSymbRad) rOScale) )
                      pStop4    (polar pSecLoc2 aRevAngl (* rSymbRad rOScale))
                    )
                    (grdraw pStrt1 pStop1 -1)
                    (if (not (equal pKink1 pKink2))
                      (progn
                    (grdraw pStrt2 pKink1 -1)
                    (grdraw pKink1 pStop2 -1)
                    (grdraw pStrt3 pKink2 -1)
                    (grdraw pKink2 pStop3 -1)
                      )
                    )
                    (grdraw pStrt4 pStop4 -1)
                   )
                   ((= 0 OffOn)
                    (setq pKink1    (list rX1 yy 0.0)
                      pKink2    (list rX2 yy 0.0)
                      aAngl (angle pSecLoc1 pKink1)
                      aRevAngl  (angle pKink1 pSecLoc1)
                      aAngle    (angle pKink1 pKink2)
                      aRevAngle (angle pKink2 pKink1)
                      pStrt1    (polar pSecLoc1 aAngl (* rSymbRad rOScale))
                      pStop1    (polar pSecLoc1 aAngl (* (+ rThickLine rSymbRad) rOScale) )
                      pStrt2    (polar pKink1 aRevAngl (* rThickLine rOScale))
                      pStop2    (polar pKink1 aAngle (* rThickLine rOScale))
                      pStrt3    (polar pKink2 aRevAngle (* rThickLine rOScale))
                      pStop3    (polar pKink2 aAngl (* rThickLine rOScale))
                      pStrt4    (polar pSecLoc2 aRevAngl (* (+ rThickLine rSymbRad) rOScale) )
                      pStop4    (polar pSecLoc2 aRevAngl (* rSymbRad rOScale))
                    )
                    (grdraw pStrt1 pStop1 -1)
                    (if (not (equal pKink1 pKink2))
                      (progn
                    (grdraw pStrt2 pKink1 -1)
                    (grdraw pKink1 pStop2 -1)
                    (grdraw pStrt3 pKink2 -1)
                    (grdraw pKink2 pStop3 -1)
                      )
                    )
                    (grdraw pStrt4 pStop4 -1)
                   )
                 )
                 (setq got nil)
                   )
                )
              )

              (if (equal pKink1 pKink2)
                (princ "")          ;no kink
                (setq pSecLoc3 (upoint 7 "" "Specify location for cutting plane" nil nil)
                      xx       (car pSecLoc3)
                      yy       (cadr pSecLoc3)
                )
              )
              (redraw)
              (cond
                ((= 1 OffOn)
                 (setq pKink1    (list xx rY1 0.0)
                   pKink2    (list xx rY2 0.0)
                   aAngl     (angle pSecLoc1 pKink1)
                   aRevAngl  (angle pKink1 pSecLoc1)
                   aAngle    (angle pKink1 pKink2)
                   aRevAngle (angle pKink2 pKink1)
                   pStrt1    (polar pSecLoc1 aAngl (* rSymbRad rOScale))
                   pStop1    (polar pSecLoc1 aAngl (* (+ rThickLine rSymbRad) rOScale) )
                   pStrt2    (polar pKink1 aRevAngl (* rThickLine rOScale))
                   pStop2    (polar pKink1 aAngle (* rThickLine rOScale))
                   pStrt3    (polar pKink2 aRevAngle (* rThickLine rOScale))
                   pStop3    (polar pKink2 aAngl (* rThickLine rOScale))
                   pStrt4    (polar pSecLoc2 aRevAngl (* (+ rThickLine rSymbRad) rOScale) )
                   pStop4    (polar pSecLoc2 aRevAngl (* rSymbRad rOScale))
                   aCheck    (+ (rtd aAngl)(rtd aAngle) )
                 )
                 (if (= aCheck 270)
                 (setq a45deg (- aAngle (dtr 45))  a45Rev (+ a45deg (dtr 180)) )         ;far then nearest corner
                 (setq a45deg (+ aAngle (dtr 45))  a45Rev (+ a45deg (dtr 180)) )
                 )
                 (setq pJog1   (polar pKink1 a45Rev (* rOfDist rOScale))             ;near corner
                     pJog2   (polar pKink2 a45deg (* rOfDist rOScale))               ;far corner
                     pStrt2a (polar pJog1 aRevAngl (* rThickLine rOScale))
                     pStop2a (polar pJog1 aAngle (* rThickLine rOScale))
                     pStrt3a (polar pJog2 aRevAngle (* rThickLine rOScale))          ;far cnr
                     pStop3a (polar pJog2 aAngl (* rThickLine rOScale))
                 )
                 (grdraw pStrt1 pStop1 -1)
                 (if (not (equal pKink1 pKink2))
                   (progn
                 (grdraw pStrt2 pKink1 -1)
                 (grdraw pKink1 pStop2 -1)
                 (grdraw pStrt3 pKink2 -1)
                 (grdraw pKink2 pStop3 -1)
                   )
                 )
                 (grdraw pStrt4 pStop4 -1)
                )
                ((= 0 OffOn)
                 (setq pKink1    (list rX1 yy 0.0)
                   pKink2    (list rX2 yy 0.0)
                   aAngl (angle pSecLoc1 pKink1)
                   aRevAngl  (angle pKink1 pSecLoc1)
                   aAngle    (angle pKink1 pKink2)
                   aRevAngle (angle pKink2 pKink1)
                   pStrt1    (polar pSecLoc1 aAngl (* rSymbRad rOScale))
                   pStop1    (polar pSecLoc1 aAngl (* (+ rThickLine rSymbRad) rOScale) )
                   pStrt2    (polar pKink1 aRevAngl (* rThickLine rOScale))          ;near cnr
                   pStop2    (polar pKink1 aAngle (* rThickLine rOScale))
                   pStrt3    (polar pKink2 aRevAngle (* rThickLine rOScale))         ;far cnr
                   pStop3    (polar pKink2 aAngl (* rThickLine rOScale))
                   pStrt4    (polar pSecLoc2 aRevAngl (* (+ rThickLine rSymbRad) rOScale) )
                   pStop4    (polar pSecLoc2 aRevAngl (* rSymbRad rOScale) )
                   aCheck    (+ (rtd aAngl)(rtd aAngle) )
                 )
                 (if (= aCheck 270)
                 (setq a45deg (+ aAngle (dtr 45))  a45Rev (+ a45deg (dtr 180)) )         ;far then nearest corner
                 (setq a45deg (- aAngle (dtr 45))  a45Rev (+ a45deg (dtr 180)) )
                 )
                 (setq pJog1   (polar pKink1 a45Rev (* rOfDist rOScale))             ;near corner
                     pJog2   (polar pKink2 a45deg (* rOfDist rOScale))               ;far corner
                     pStrt2a (polar pJog1 aRevAngl (* rThickLine rOScale))
                     pStop2a (polar pJog1 aAngle (* rThickLine rOScale))
                     pStrt3a (polar pJog2 aRevAngle (* rThickLine rOScale))          ;far cnr
                     pStop3a (polar pJog2 aAngl (* rThickLine rOScale))
                 )
                 (grdraw pStrt1 pStop1 -1)
                 (if (not (equal pKink1 pKink2))
                   (progn
                 (grdraw pStrt2 pKink1 -1)
                 (grdraw pKink1 pStop2 -1)
                 (grdraw pStrt3 pKink2 -1)
                 (grdraw pKink2 pStop3 -1)
                   )
                 )
                 (grdraw pStrt4 pStop4 -1)
                )
              )

              (if (equal pKink1 pKink2)
                (setq bShowMiddle nil)
                (setq bShowMiddle T)
              )

              (if (< (getvar "OSMODE") 16384) (toggle "osmode" 16384) )             ;If Osnap on, turn off

              (setvar "ORTHOMODE" 1)
              (setq aSymbRot (rtd (angle (list 0 0 0) pAlongX)))
              ;;(command "_.INSERT" "SectMark" ^C) ;;### TM WRONG CODE !!!
              (command "_.INSERT" "SectMark")(command)(command)

              (cond
                ((= SectStyle "Style1")
                  (command "_.INSERT" Type1 pSecLoc1 rOScale rOScale (* -1 aSymbRot) sSect sDWG)
                      (command "_.INSERT" Type1 pSecLoc2 rOScale rOScale (* -1 aSymbRot) sSect sDWG)
                    (princ "\nSpecify rotation angle")
                    (command "_.INSERT" Type1a pSecLoc1 rOScale rOScale pause)
                    (setq eArro   (entlast)
                      lTArrow (entget eArro) )
                    (command "_.INSERT" Type1a pSecLoc2 rOScale rOScale (- (rtd (dxf 50 lTArrow)) aSymbRot) )

                    (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                    (command "_.PLINE" pStrt1 pKink1 pKink2 pStop4 "")

                    (Hshset_lay "CutMark" "171" nil LaySt2a)
                    (command "_.PLINE" pStrt1 pStop1 "")

                    (if bKinkOnL (setq pStrt2a pStrt2  pJog1 pKink1  pStop2a pStop2  pStrt3a pStrt3  pJog2 pKink2  pStop3a pStop3) )
                    (if bShowMiddle
                     (progn
                      (command "_.PLINE" pStrt2a pJog1 pStop2a "")
                      (command "_.PLINE" pStrt3a pJog2 pStop3a "")
                     )
                    )

                    (command "_.PLINE" pStrt4 pStop4 "")
                )
                ((= SectStyle "Style2")
                    (princ "\nSpecify rotation angle")
                  (command "_.INSERT" Type2 pSecLoc1 rOScale rOScale pause sSect)
                  (SelCross pSecLoc1 15 aSymbRot)

                    (setq eArro   (entlast)
                      lTArrow (entget eArro) )
                    (command "_.INSERT" Type2 pSecLoc2 rOScale rOScale (- (rtd (dxf 50 lTArrow)) aSymbRot) sSect)
                  (SelCross pSecLoc2 15 aSymbRot)

                    (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                    (command "_.PLINE" pStrt1 pKink1 pKink2 pStop4 "")

                    (Hshset_lay "CutMark" "171" nil LaySt2a)
                    (if bKinkOnL (setq pStrt2a pStrt2  pJog1 pKink1  pStop2a pStop2  pStrt3a pStrt3  pJog2 pKink2  pStop3a pStop3) )
                    (if bShowMiddle
                     (progn
                      (command "_.PLINE" pStrt2a pJog1 pStop2a "")
                      (command "_.PLINE" pStrt3a pJog2 pStop3a "")
                     )
                    )
                )
                ((= SectStyle "Style3")
                    (princ "\nSpecify rotation angle")
                  (command "_.INSERT" Type3 pSecLoc1 rOScale rOScale pause sSect)
                  (SelCross pSecLoc1 15 aSymbRot)

                    (setq eArro   (entlast)
                      lTArrow (entget eArro) )
                    (command "_.INSERT" Type3 pSecLoc2 rOScale rOScale (- (rtd (dxf 50 lTArrow)) aSymbRot) sSect)
                  (SelCross pSecLoc2 15 aSymbRot)

                    (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                    (command "_.PLINE" pStrt1 pKink1 pKink2 pStop4 "")

                    (Hshset_lay "CutMark" "171" nil LaySt2a)
                    (if bKinkOnL (setq pStrt2a pStrt2  pJog1 pKink1  pStop2a pStop2  pStrt3a pStrt3  pJog2 pKink2  pStop3a pStop3) )
                    (if bShowMiddle
                     (progn
                      (command "_.PLINE" pStrt2a pJog1 pStop2a "")
                      (command "_.PLINE" pStrt3a pJog2 pStop3a "")
                     )
                    )
                )
                ((= SectStyle "Style4")
                    (princ "\nSpecify rotation angle")
                  (command "_.INSERT" Type4 pSecLoc1 rOScale rOScale pause sSect)
                  (SelCross pSecLoc1 15 aSymbRot)

                    (setq eArro   (entlast)
                      lTArrow (entget eArro) )
                    (command "_.INSERT" Type4 pSecLoc2 rOScale rOScale (- (rtd (dxf 50 lTArrow)) aSymbRot) sSect)
                  (SelCross pSecLoc2 15 aSymbRot)

                    (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                    (command "_.PLINE" pStrt1 pKink1 pKink2 pStop4 "")

                    (Hshset_lay "CutMark" "171" nil LaySt2a)
                    (if bKinkOnL (setq pStrt2a pStrt2  pJog1 pKink1  pStop2a pStop2  pStrt3a pStrt3  pJog2 pKink2  pStop3a pStop3) )
                    (if bShowMiddle
                     (progn
                      (command "_.PLINE" pStrt2a pJog1 pStop2a "")
                      (command "_.PLINE" pStrt3a pJog2 pStop3a "")
                     )
                    )
                )
              )
            )

            (progn   ;option "Select" ou Endroit to select Pline
              (setq ent1 (objsel "POLYLINE or LWPOLYLINE" "Select an object or " T))
              (command "_.UCS" "_W")
              (if bMM
                (progn
                  (setq rThickLine 10  rSymbRad 7.0  bMM T  bInch nil  Type1 "SectMark"  Type1a "TurnArrowmm"  unite "mm"  rOfDist 2.828427
                        Type2 "Style3mm"  Type3 "Style2mm"  Type4 "Style4mm"  LaySt1 "ACAD_ISO08W100"  LaySt1a "0.35"  LaySt2a "0.70")
                  (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                  (Hshset_lay "CutMark" "171" nil LaySt2a)
                )
                (progn
                  (setq rThickLine 0.45  rSymbRad 0.45  bMM nil  bInch T  Type1 "SectMarkin"  Type1a "TurnArrowin"  unite "in"  rOfDist 0.1414
                        Type2 "Style3in"  Type3 "Style2in"  Type4 "Style4in"  LaySt1 "PHANTOM2"  LaySt1a "0.031"  LaySt2a "0.062")
                  (Hshset_lay "CutLine" "4" LaySt1 LaySt1a)
                  (Hshset_lay "CutMark" "171" nil LaySt2a)
                )
              )

              (if (= (dxf 0 (entget ent1)) "POLYLINE")
                (setq p_list (Liste4Pline ent1))
              )
              (if (= (dxf 0 (entget ent1)) "LWPOLYLINE")
                (setq p_list (Liste4LwPl ent1))
              )
              (setq pUn        (nth 0 p_list)
                    pdeux      (nth 1 p_list)
                    lReversed  (reverse p_list)
                    pDernier   (nth 0 lReversed)
                    pAvantD    (nth 1 lReversed)
                    aAngl      (angle pUn pDeux)
                    aRevAngl   (angle pDeux pUn)
                    aAngle     (angle pDernier pAvantD)
                    aRevAngle  (angle pAvantD pDernier)
                    pSecLoc1   (polar pUn aRevAngl (* rSymbRad rOScale) )
                    pSecLoc2   (polar pDernier aRevAngle (* rSymbRad rOScale) )
                    pStop1     (polar pUn aAngl (* (+ rThickLine rSymbRad) rOScale) )
                    pStop2     (polar pDernier aAngle (* (+ rThickLine rSymbRad) rOScale) )
              )

              (if (< (getvar "OSMODE") 16384) (toggle "osmode" 16384)) ;If Osnap on, turn off
              (setvar "ORTHOMODE" 1)
              (setq aSymbRot (rtd (angle (list 0 0 0) pAlongX)))

              ;;;(command "_.INSERT" "SectMark" ^C) ;;### TM WRONG CODE !!!
              (command "_.INSERT" "SectMark")(command)(command)

              ;;(command "_.CHPROP" ent1 "" "_LA" "CutLine" "")
              ;;(vlax-put-property obj "Layer" acByLayer)
              (vla-put-layer (vlax-Ename->vla-Object ent1) "CutLine")

              (cond
                ((= SectStyle "Style1")
                  (command "_.ZOOM" "_C" pSecLoc1 "3X")
                  (command "_.INSERT" Type1 pSecLoc1 rOScale rOScale  (* -1 aSymbRot) sSect sDWG)
                  (command "_.UNDO" "_M")
                      (command "_.INSERT" Type1a pSecLoc1 rOScale rOScale (+ 90 (rtd aAngl)))
                       (setq Hshans1 (ukword 1 "Yes No" "1st arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type1a pSecLoc1 rOScale rOScale (+ 270 (rtd aAngl)))
                      )
                       )
                  (command "_.ZOOM" "_P")
                  (command "_.ZOOM" "_C" pSecLoc2 "3X")
                      (command "_.INSERT" Type1 pSecLoc2 rOScale rOScale  (* -1 aSymbRot) sSect sDWG)
                  (command "_.UNDO" "_M")
                      (command "_.INSERT" Type1a pSecLoc2 rOScale rOScale (+ 90 (rtd aRevAngle)))
                       (setq Hshans1 (ukword 1 "Yes No" "2nd arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type1a pSecLoc2 rOScale rOScale (+ 270 (rtd aRevAngle)))
                      )
                       )
                  (command "_.ZOOM" "_P")
                      (Hshset_lay "CutMark" "171" nil LaySt2a)
                      (command "_.PLINE" pUn pStop1 "")
                      (command "_.PLINE" pDernier pStop2 "")
                )
                ((= SectStyle "Style2")
                  (command "_.ZOOM" "_C" pSecLoc1 "3X")
                  (command "_.UNDO" "_M")
                  (command "_.INSERT" Type2 pSecLoc1 rOScale rOScale (+ 90 (rtd aAngl)) sSect)
                  (SelCross pSecLoc1 15 aSymbRot)
                       (setq Hshans1 (ukword 1 "Yes No" "1st arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type2 pSecLoc1 rOScale rOScale (+ 270 (rtd aAngl)) sSect)
                               (SelCross pSecLoc1 15 aSymbRot)
                      )
                       )
                  (command "_.ZOOM" "_P")
                  (command "_.ZOOM" "_C" pSecLoc2 "3X")
                  (command "_.UNDO" "_M")
                      (command "_.INSERT" Type2 pSecLoc2 rOScale rOScale (+ 90 (rtd aRevAngle)) sSect)
                  (SelCross pSecLoc2 15 aSymbRot)
                       (setq Hshans1 (ukword 1 "Yes No" "2nd arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type2 pSecLoc2 rOScale rOScale (+ 270 (rtd aRevAngle)) sSect)
                               (SelCross pSecLoc2 15 aSymbRot)
                      )
                       )
                  (command "_.ZOOM" "_P")
                )
                ((= SectStyle "Style3")
                  (command "_.ZOOM" "_C" pSecLoc1 "3X")
                  (command "_.UNDO" "_M")
                  (command "_.INSERT" Type3 pSecLoc1 rOScale rOScale (+ 90 (rtd aAngl)) sSect)
                  (SelCross pSecLoc1 15 aSymbRot)
                       (setq Hshans1 (ukword 1 "Yes No" "1st arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type3 pSecLoc1 rOScale rOScale (+ 270 (rtd aAngl)) sSect)
                               (SelCross pSecLoc1 15 aSymbRot)
                      )
                       )
                  (command "_.ZOOM" "_P")
                  (command "_.ZOOM" "_C" pSecLoc2 "3X")
                  (command "_.UNDO" "_M")
                      (command "_.INSERT" Type3 pSecLoc2 rOScale rOScale (+ 90 (rtd aRevAngle)) sSect)
                  (SelCross pSecLoc2 15 aSymbRot)
                       (setq Hshans1 (ukword 1 "Yes No" "2nd arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type3 pSecLoc2 rOScale rOScale (+ 270 (rtd aRevAngle)) sSect)
                               (SelCross pSecLoc2 15 aSymbRot)
                      )
                       )
                  (command "_.ZOOM" "_P")
                )
                ((= SectStyle "Style4")
                  (command "_.ZOOM" "_C" pUn "3X")
                  (command "_.UNDO" "_M")
                  (command "_.INSERT" Type4 pUn rOScale rOScale (+ 90 (rtd aAngl)) sSect)
                  (SelCross pUn 17 aSymbRot)
                       (setq Hshans1 (ukword 1 "Yes No" "1st arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type4 pUn rOScale rOScale (+ 270 (rtd aAngl)) sSect)
                               (SelCross pUn 17 aSymbRot)
                      )
                       )
                  (command "_.ZOOM" "_P")
                  (command "_.ZOOM" "_C" pDernier "3X")
                  (command "_.UNDO" "_M")
                      (command "_.INSERT" Type4 pDernier rOScale rOScale (+ 90 (rtd aRevAngle)) sSect)
                  (SelCross pDernier 17 aSymbRot)
                       (setq Hshans1 (ukword 1 "Yes No" "2nd arrow on correct side? [Yes/No] " "Yes") )
                       (if (= Hshans1 "No")
                      (progn
                       (command "_.UNDO" "_B")
                               (command "_.INSERT" Type4 pDernier rOScale rOScale (+ 270 (rtd aRevAngle)) sSect)
                               (SelCross pDernier 17 aSymbRot)
                      )
                       )
                  (command "_.ZOOM" "_P")
                      (Hshset_lay "CutMark" "171" nil LaySt2a)
                      (command "_.PLINE" pUn pStop1 "")
                      (command "_.PLINE" pDernier pStop2 "")
                )
              )
            )
          )

          ;ecrit information sur dernier numero/lettre.
          (if (= 'INT (type (read sSect)))
            (progn
              (changerecord "AA_SectNum" "SectID" (list (cons 300 (etos (1+ (read sSect))))) )
              (changerecord "AA_SectNum" "CNType" (list (cons 300 "N")))
              (changerecord "AA_SectNum" "DwgID"  (list (cons 300 sDWG)))
            )
            (progn
              (changerecord "AA_SectNum" "SectID" (list (cons 300 (etos (1+ (ascii sSect))))) )
              (changerecord "AA_SectNum" "CNType" (list (cons 300 "C")))
              (changerecord "AA_SectNum" "DwgID"  (list (cons 300 sDWG)))
            )
          )
          (changerecord "AA_SectNum" "StSect" (list (cons 300 SectStyle)) )
          (changerecord "AA_SectNum" "ImSect" (list (cons 300 (etos NImag))) )
          (changerecord "AA_SectNum" "SUnits" (list (cons 300 unite)) )
          (changerecord "AA_SectNum" "OnCutPlane" (list (cons 300 (etos bKinkOnL))) )
        )

        (defun c:SecSymbol_Cancel_Clicked ()
          (Odcl_Form_Close ProjName0 "SecSymbol")
          (princ)
        )

        ;;;(MontreAA3)
        (Odcl_Form_Show ProjName0 "SecSymbol")

        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;----------------------------
      (defun getmid (ent / lpts lpt1 lpt2 x y)
        (setq lpts (getendpts ent))
        (setq lpt1 (car lpts)
              lpt2 (cadr lpts)
              x    (/ (+ (nth 0 lpt1) (nth 0 lpt2)) 2)
              y    (/ (+ (nth 1 lpt1) (nth 1 lpt2)) 2))
        (list x y)
      )

      (defun getmids (l1_pts l2_pts / l1_pt1 l1_pt2 l2_pt1 l2_pt2 pt1 pt2)
        (setq l1_pt1 (car l1_pts)
              l1_pt2 (cadr l1_pts)
              l2_pt1 (car l2_pts)
              l2_pt2 (cadr l2_pts)
              pt1    (mid_pt l1_pt1 l2_pt1)
              pt2    (mid_pt l1_pt2 l2_pt2)
        )
        (list pt1 pt2)
      )

      (defun getendpts (line_object / l lpt1 lpt2)
        (setq l    (entget line_object)
              lpt1 (dxf 10 l)
              lpt2 (dxf 11 l))
        (list lpt1 lpt2)
      )

      (defun getlength (ent / lpts)
        (setq lpts (getendpts ent))
        (distance (nth 0 lpts) (nth 1 lpts))
      )

      ;-----------------
      ;---LAYER MANAGER.
      ;-----------------
      ;Move selected objects to current layer.
      (defun c:cLayStates (/ cll ssno ssl alist)
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (setq cll (getvar "CLAYER"))
        (princ (strcat "\nSelect objects to be changed to layer " cll " (the current layer): ")
        )
        (setq ssno 0)
        (setq ssl (ssget))
        (if (and (/= ssl nil) (> (setq sslen (sslength ssl)) 0))
          (progn (while (< ssno sslen)
               (entmod
                 (subst (cons '8 cll) (assoc '8 (entget (ssname ssl ssno))) (entget (ssname ssl ssno)))
               )
               (setq ssno (1+ ssno))
             )
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;Toggle layers on/off.
      (defun c:LayStates ()
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\nToggle layers on/off.\n")
        (setq counter 1)
        (if (= bToggle T)
          (progn (command "_.layer" "_on" "*" "") (setq bToggle nil) (setq counter 2))
        )
        (if (and (< counter 2) (= bToggle nil) )
          (progn (command "_.layer" "_of" "*" "_N" "") (setq bToggle T))
        )
        (setq *error* OLDERROR)
        ;;(reset nil)
        (resset varlst)
      )

      ;Turn off a layer by selecting an object on that layer.
      (defun C:OLayStates (/ ent resp2)
        (setq olderror *error*
             *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (setq ent (entsel "Select an object on the layer to be turned off: "))
        (setq lay (cdr (assoc 8 (entget (car ent)))))
        (if (= lay (getvar "CLAYER"))
          (progn
             (setq resp2 (ukword 1 "Yes No" "Really want the CURRENT layer off? [Yes/No] " "No"))
             (if (= resp2 "Yes")
               (progn (command "_.layer" "_off" lay "_Y" "")
                  (princ (strcat "\nLayer " lay " has been turned off."))
               )
             )
          )
          (progn (command "_.layer" "_off" lay "")
             (princ (strcat "\nLayer " lay " has been turned off."))
          )
        )
        (setq *error* OLDERROR)
        ;(reset nil)
        (resset varlst)
      )

      ;Turn off a nested layer by selecting an object on that layer.
      (defun C:OOLayStates (/ ent resp2)
        (setq olderror *error*
             *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (setq ent (nentsel "Select an object on the nested layer to be turned off: "))
        (setq lay (cdr (assoc 8 (entget (car ent)))))
        (if (= lay (getvar "CLAYER"))
          (progn
             (setq resp2 (ukword 1 "Yes No" "Really want the CURRENT layer off? [Yes/No] " "No"))
             (if (= resp2 "Yes")
               (progn (command "_.layer" "_off" lay "_Y" "")
                  (princ (strcat "\nLayer " lay " has been turned off."))
               )
             )
          )
          (progn (command "_.layer" "_off" lay "")
             (princ (strcat "\nLayer " lay " has been turned off."))
          )
        )
        (setq *error* OLDERROR)
        ;(reset nil)
        (resset varlst)
      )

      ;Freeze a layer by selecting an object on that layer.
      (defun C:fLayStates (/ ent lay)
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (setq ent (entsel "Select an object on the layer to be frozen: "))
        (setq lay (cdr (assoc 8 (entget (car ent)))))
        (if (/= lay (getvar "CLAYER"))
          (progn (command "_.layer" "_f" lay "") (princ (strcat "\nLayer " lay " has been frozen.")))
          (princ (strcat "\nCannot freeze the current layer."))
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;Freeze a nested layer by selecting an object on that layer.
      (defun C:ffLayStates (/ ent lay)
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (setq ent (nentsel "Select an object on the nested layer to be frozen: "))
        (setq lay (cdr (assoc 8 (entget (car ent)))))
        (if (/= lay (getvar "CLAYER"))
          (progn (command "_.layer" "_f" lay "") (princ (strcat "\nLayer " lay " has been frozen.")))
          (princ (strcat "\nCannot freeze the current layer."))
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;Lock a layer by selecting an object on that layer.
      (defun C:LLayStates (/ e n cn)
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (while (not e) (setq e (car (entsel "\nSelect an object on the layer to be locked: "))))
        (if e
          (progn (setq e (entget e))
             (setq n (cdr (assoc 8 e)))
             (command "_.LAYER" "_lo" n "")
             (princ (strcat "\nLayer " n " has been locked."))
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;Unlock a layer by selecting an object on that layer.
      (defun C:ULayStates (/ e n cn)
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (while (not e) (setq e (car (entsel "\nSelect an object on the layer to be unlocked: "))))
        (if e
          (progn (setq e (entget e))
             (setq n (cdr (assoc 8 e)))
             (command "_.LAYER" "_U" n "")
             (princ (strcat "\nLayer " n " has been unlocked."))
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ; Pick isolate.
      (defun c:iLayStates (/ a2 b2 L1 L2)
        (setq olderror *error*
              *error*  Wallserror
        )
        ;;(Wallsgather_info)
        (princ "\n")
        (prompt "\nSelect an object on the layer to isolate: ")
        (setq a2 (entsel))
        (setq b2 (entget (car a2)))
        (setq L1 (assoc 8 b2))
        (setq L2 (cdr L1))
        (command "_.layer" "_s" L2 "")
        (command "_.layer" "_f" "*" "")
        (setq *error* OLDERROR)
        ;;(reset nil)
        (resset varlst)
      )

      ;Delete all objects on the layer of a picked object.
      (defun C:dLayStates (/ adoc ent obj lay sset i)
        (setq olderror *error*
             *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\n")
        (setq adoc (vla-get-activedocument (vlax-get-acad-object)))
        (while (setq ent (entsel "\nSelect object on layer to delete: "))
          (setq obj (vlax-ename->vla-object (car ent)))
          (setq lay (vla-get-layer obj))
          (cond
            ( (setq sset (ssget "_x" (list (cons 8 lay))))
              (princ (strcat "\nDeleting " (itoa (sslength sset)) " objects on layer " lay))
              (vla-StartUndoMark adoc)
              (setq i 0)
              (repeat (sslength sset)
                (vla-delete (vlax-ename->vla-object (ssname sset i)))
                (setq i (1+ i))
              )
              (vla-EndUndoMark adoc)
            )
          )
          (vlax-release-object obj)
        )
        (vlax-release-object adoc)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ; Thaw all layers in current drawing.
      (defun C:tLayStates (/ adoc layers aclay)
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (princ "\nThaw all layers.\n")
        (setq adoc   (vla-get-activedocument (vlax-get-acad-object))
              layers (vla-get-layers adoc)
              aclay  (vla-get-name (vla-get-activelayer adoc))
        )
        (vla-StartUndoMark adoc)
        (vlax-for each layers
          (if (/= (strcase (vla-get-name each)) (strcase aclay))
            (progn
              (vla-put-freeze each 0)
              (vla-put-layeron each 1)
            )
          )
        )
        (vla-Regen adoc T)
        (vla-EndUndoMark adoc)
        (vlax-release-object adoc)
        (vlax-release-object layers)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;--------------
      ;---BREAKLINE.
      ;--------------
      (defun C:BKL ( / ssObj1 ept ang2 dwgsc aDn rScal2)
        (setq olderror *error*
             *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (princ (strcat "\nDraw a Breakline." copyright))
        (SETQ spt    (upoint 1 "" "Specify first point for breakline" NIL NIL)
              ept    (upoint 1 "" "Specify second point for breakline" NIL spt)
              sOnOf  (ukword 1 "ON OFF" "Break when intersecting lines? [ON/OFF] " "ON")
              ang1   (ANGLE spt ept)
              dwgsc  (GETVAR "DIMSCALE")
              ang2   (ANGLE ept spt)
              aUp    (+ ang1 (/ PI 2.3))  ;2.5
              aDn    (+ ang2 (/ PI 2.3))
              rLong  (DISTANCE spt ept)
              rTest  rLong
              rScal2 (* dwgsc 3.0)        ;2.5
        )
        (IF (>= rTest (* dwgsc 200))
          (SETQ rScal2 (* rScal2 2))
        )
        (IF (<= rTest (* dwgsc 50))
          (SETQ rScal2 (/ rScal2 2))
        )
        (SETQ Echel  (* rScal2 0.5)
              pEntre (mid_pt spt ept)
              pUn    (POLAR spt ang1 (- (* rLong 0.5) Echel))
              pDeux  (POLAR pUn ang1 rScal2)
              pTrois (POLAR pUn aUp rScal2)
              pQuatre (POLAR pDeux aDn rScal2)
        )
        (Hshset_lay sSymbol25Lyr sSymbol25Col "CONTINUOUS" sSymbol25Lwt)
        (COMMAND "_.pline" spt "_w" "0" "" pUn pUn pTrois pQuatre pDeux pDeux ept "")
        (IF (> rTest (* dwgsc 200))
          (COMMAND "_.scale" "_l" "" spt 0.5 "_.copy" "_l" "" spt pEntre "_.pedit" "_l" "_j" "_p" "" "")
        )
        (if (= sOnOf "ON") (Slice2D nil))
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;--------------------
      ; Auto-Dimensioning.
      ;--------------------
      ;;; When passed a selection set, (ai_ssget) removes objects on locked
      ;;; layers from the returned selection set. Nil is returned if all objects
      ;;; in the selection set are locked.
      (defun ak_ssget (ss / start_size end_size a diff)
        (setq start_size (sslength ss))
        (setq a 0)
        (while (< a (sslength ss))
          (if (ak_entity_locked (ssname ss a) 1)
            (ssdel (ssname ss a) ss)
            (setq a (1+ a))           ; Only increment if non-deleted item.
          )
        )
        (setq end_size (sslength ss))
        (if (/= 1 (setq diff (- start_size end_size)))
          (princ (strcat "\n" (itoa diff) " were on a locked layer."))
        )
        (if (> (sslength ss) 0)
          ss
          nil
        )
      )

      ;;; Returns T if passed ename is on a locked layer.
      (defun ak_entity_locked (ename message)
        (if (= 4 (logand 4 (cdr (assoc 70 (tblsearch "layer" (cdr (assoc 8 (entget ename))))))))
          (progn
            (if (= 1 message)
              (princ "\n1 was on a locked layer. ")
            )
            T
          )
          nil
        )
      )

      (defun sort_x (DList Test)      ; / (Bbl CNT) modified*!
        (setq Bbl 1)
        (while Bbl
          (setq CNT 0
            Bbl nil
          )
          (while (< (setq CNT (1+ CNT)) (length DList))
            (if (Test (car (nth (1- CNT) DList)) (car (nth CNT DList)))  ;car added!
              (setq DList (swap Dlist (1- CNT) CNT)
                    Bbl   1
              )
            )
          )
        )
        DList
      )

      (defun sort_y (DList Test)      ; (/ Bbl CNT) modified*!
        (setq Bbl 1)
        (while Bbl
          (setq CNT 0
            Bbl nil
          )
          (while (< (setq CNT (1+ CNT)) (length DList))
            (if (Test (cadr (nth (1- CNT) DList)) (cadr (nth CNT DList)))         ;cadr added!
              (setq DList (swap Dlist (1- CNT) CNT)
                    Bbl   1
              )
            )
          )
        )
        DList
      )

      (defun C:AD (/ pt1 pt2 pt3 pt4 p1 p2 emx exo1 exo2 pto1 pto2 ang1 ang2 maxang minang pls plst)
        (setq olderror *error*
              *error*  Wallserror
              ss1      nil
              ptlst    nil
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (command "_.UCS" "_W")
        (princ (strcat "\nAutomatic Dimensioning by pick points." copyright))
        (setq pt1    (upoint 1 "" "Specify first point for cut line" nil nil)
              pt2    (upoint 1 "" "Specify next point" nil pt1)
              pt3    (mid_pt pt1 pt2)
              cnt    0
              ang1   (angle pt1 pt2)
              maxang (+ (rtd ang1) 120)
              minang (+ (rtd ang1) 60)
        )
        (prompt "\nSelect objects to add or Press shift to deselect...\n")

        (command "_.SELECT" (ssget "_F" (list pt1 pt2) (list (cons 0 "LINE")
                 (cons -4 "<NOT") (cons 8 sBrkWall2Lyr) (cons -4 "NOT>") ))
                 pause
        )

        (setq ss1 (ssget "_P" (list (cons 0 "LINE") )))

        (setq 1ss (ak_ssget ss1))
        (if ss1
          (progn (setq emx (sslength ss1))                        ;how many
            (if (> emx 1)
              (progn
                (setq sn   (ssname ss1 cnt)
                      edat (entget sn)
                      frp  (dxf 10 edat)                    ;start of previous
                      top  (dxf 11 edat)
                      cnt  (1+ cnt)
                      p1   (inters pt1 pt2 frp top nil)
                      plst (list p1)
                )
                (while (< cnt emx)
                   (setq sn (ssname ss1 cnt))
                   (if sn
                     (progn
                       (setq edat (entget sn)
                             frn  (dxf 10 edat)             ;start of next
                             ton  (dxf 11 edat)
                             p2   (inters pt1 pt2 frn ton nil)
                             pls  (list p2)
                             plst (append plst pls)
                       )
                     )
                   )
                   (setq cnt (1+ cnt))
                )
                (setq dst3  (udist 6 "" "Specify extension line origin" nil pt1)
                      pLast (getvar "lastpoint")
                      pt4   (upoint 1 "" "Specify dimension line location" nil pt1)
                      ang2  (angle pt1 pt4)
                )
                (if (and (< (rtd ang2) maxang) (> (rtd ang2) minang))    ;<---3
                  (setq realang 90)
                  (setq realang 270)
                )
                (if (or (= (rtd ang1) 90) (= (rtd ang1) 270))
                  (setq ptlst (sort_y plst >))              ;Le tri se fait ici
                  (setq ptlst (sort_x plst >))
                )
                (setq iter  2
                  count 1
                  exo1  (nth 0 ptlst)
                  exo2  (nth 1 ptlst)
                  pto1  (polar exo1 (+ ang1 (dtr realang)) dst3)    ;<---1
                  pto2  (polar exo2 (+ ang1 (dtr realang)) dst3)
                )
                (setvar "OSMODE" 0)
                (command "_.DIM" "_ALI" pto1 pto2 pt4 "" "_EX")         ;<---2
                (while (< iter emx)
                  (setq exo1 (nth count ptlst)
                    exo2 (nth (1+ count) ptlst)
                    pto1 (polar exo1 (+ ang1 (dtr realang)) dst3)
                    pto2 (polar exo2 (+ ang1 (dtr realang)) dst3)
                  )
                  (command "_.DIM" "_ALI" pto1 pto2 pt4 "" "_EX")
                  (setq iter  (1+ iter)
                    count (1+ count)
                  )
                )
              )
              (prompt "\nAt least two lines are required!")
            )
          )
          (prompt "\nNo lines selected!")
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun C:ADS (/ pt1 pt2 pt3 pt4 p1 CutAlong p2 yPto1 iMax pls plast exo1 exo2 pto1 pto2 p2Lst ang1 ang2
                     maxang minang)
        (setq olderror *error*
             *error*  Wallserror
             lFinal   nil
             ptlst    nil
             lCutPts  nil
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (command "_.UCS" "_W")
        (princ (strcat "\nAutomatic Dimensioning by selecting object." copyright))
        (setq CutAlong (car (entsel "\nSelect ARC, LINE, POLYLINE as defining plane...")))
        (while (or (null CutAlong)
                   (and (/= "LWPOLYLINE" (dxf 0 (entget CutAlong)))
                     (/= "LINE" (dxf 0 (entget CutAlong)))
                     (/= "ARC" (dxf 0 (entget CutAlong)))
                     (/= "POLYLINE" (dxf 0 (entget CutAlong)))
                   )
               )
          (prompt "\nNot a valid object! Select ARC, LINE, POLYLINE.")
        )
        (setq OBJLST (entget CutAlong))
        (cond
          ((= "POLYLINE" (dxf 0 OBJLST))
           (setq lCutPts (Liste4Pline CutAlong))
           (if (isclosed CutAlong)            ;add first point if closed
             (setq lCutPts (reverse (cons (car lCutPts) (reverse lCutPts))))
           )
           (setq lCutPts (RemDupPts lCutPts 0.1))
          )
          ((= "LWPOLYLINE" (dxf 0 OBJLST))
           (setq lCutPts (massoc 10 OBJLST))
           (if (isclosed CutAlong)            ;add first point if closed
             (setq lCutPts (reverse (cons (car lCutPts) (reverse lCutPts))))
           )
           (setq lCutPts (RemDupPts lCutPts 0.1))
          )
          ((= "LINE" (dxf 0 OBJLST)) (setq lCutPts (list (dxf 10 OBJLST) (dxf 11 OBJLST))))
          ((= "ARC" (dxf 0 OBJLST))
           (setq CENPT   (dxf 10 OBJLST)          ;center pt
                 RAD     (dxf 40 OBJLST)          ;radius
                 aStrt   (dxf 50 OBJLST)          ;start angle
                 aEnd    (dxf 51 OBJLST)
                 STPT    (polar CENPT aStrt RAD)      ;start pt
                 INANG   (DeflAng aStrt aEnd)         ;included angle
                 DIV     (fix (/ INANG (/ pi 180)))   ;no of div, approx one degree
                 INCRANG (/ INANG DIV)            ;increment angle
                 2ndANG  (+ aStrt INCRANG)        ;second angle
                 lCutPts (list STPT)
           )              ;setq
           (while (> DIV 1)
             (setq PT (polar CENPT 2ndANG RAD)) ;2nd pt
             (setq lCutPts (cons PT lCutPts))
             (setq 2ndANG (+ 2ndANG INCRANG))
             (setq DIV (1- DIV))
           )
          )
        )
        (setq ss1  nil
              ss1  (ssget "_F"
                      lCutPts
                      (list (cons 0 "LINE,LWPOLYLINE,POLYLINE,ARC,CIRCLE,ELLIPSE,SPLINE")
                        (cons -4 "<NOT")
                        (cons 8 sBrkWall2Lyr)
                        (cons -4 "NOT>")
                      )
                   )
        )
        (if (> (sslength ss1) 2)
          (progn
            (ssdel CutAlong ss1)
            (prompt "\nHold down Shift button to deselect objects...\n")
            (command "_.SELECT" ss1 pause)
            (setq ss1 (ssget "_P")
                  sAns (ukword 1
                           "Horizontal Vertical"
                           "Enter an option for? [Horizontal/Vertical] "
                           "Horizontal"
                       )
                  cnt  0
                  1ss  (ak_ssget ss1)
                  iMax (sslength ss1)    ;how many
            )
            (if (> iMax 1)
              (progn
                (setq sn   (ssname ss1 cnt)
                      edat (entget sn)
                      cnt  (1+ cnt)
                      p1   (getinters CutAlong sn acExtendNone)
                      plst p1
                )
                (while (< cnt iMax)
                  (setq sn (ssname ss1 cnt))
                  (if sn
                    (progn
                      (setq edat (entget sn)
                             p2   (getinters CutAlong sn acExtendNone)
                             pls  (list p2)
                             plst (append plst p2)
                      )
                    )
                  )
                  (setq cnt (1+ cnt))
                )
                (setq plst (Remove_Dup plst) )
                (cassure "ptlst plst")

                (if (= sAns "Horizontal")
                  (progn
                    (setq pExt1 (upoint 1 "" "Specify first extension line origin" nil (car p1))
                          pt4   (upoint 1 "" "Specify dimension line location" nil pExt1)
                          iter  0
                          rXp1  (car pExt1)
                          rYp1  (cadr pExt1)
                    )
                    (foreach pItem pLst
                      (setq ptt   (replace_n 1 rYp1 pItem)
                            ptLst (append ptLst (list ptt)))
                    )
                    (setq ptLst  (sort_x ptLst >)
                          p2Lst  (sort_x ptLst >)
                          p2Lst  (RemDupPts p2Lst 10)
                    )
                    (setvar "OSMODE" 0)
                    ;;(command "_.UCS" "_P")
                    (while (< iter (1- iMax))
                      (command "_.DIM" "_HOR" (nth iter p2Lst) (nth (1+ iter) p2Lst) pt4 "" "_EX")
                      (setq iter  (1+ iter) )
                    )
                  )
                )
                (if (= sAns "Vertical")
                  (progn
                    (setq pExt1 (upoint 1 "" "Specify first extension line origin" nil (car p1))
                          pt4   (upoint 1 "" "Specify dimension line location" nil pExt1)
                          iter  0
                          rXp1  (car pExt1)
                          rYp1  (cadr pExt1)
                    )
                    (foreach pItem pLst
                    (setq ptt   (replace_n 0 rXp1 pItem)
                          ptLst (append ptLst (list ptt)))
                    )
                    (setq ptLst  (sort_y ptLst >)
                      p2Lst  (sort_y ptLst >)
                      p2Lst  (RemDupPts p2Lst 10)
                    )
                    (setvar "OSMODE" 0)
                    (while (< iter (1- iMax))
                      (command "_.DIM" "_VER" (nth iter p2Lst) (nth (1+ iter) p2Lst) pt4 "" "_EX")
                      (setq iter  (1+ iter) )
                    )
                  )
                )
              )
              (prompt "\nAt least two intersected objects are required!")
            )
          )
          (prompt "\nAt least two intersected objects are required!")
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;-----------------------------
      ;---Elevations semi-automatic.
      ;-----------------------------
      (defun DigConv (midp str height color / p i fun Barrer Num0 Num1 Num2 Num3 Num4 Num5 Num6 Num7 Num8 Num9)

        (defun Barrer (p height color dir / ph n t- t+ b- b+ d++ d+-)
          (setq p       (list (car p) (cadr p))
                height  (* height 0.5)
                ph      (/ (getvar "viewsize") (cadr (getvar "screensize"))) ; pixel height
                n       (max 1 (fix (/ height 6 ph)))
                height  (- height (* ph (sqrt n)))
          )
          (if (= dir "UP")
            (setq t-  (mapcar '+ p (list 0.0 height))
                  b-  (mapcar '- p (list 0.0 height))
                  d++ (list ph ph)
                  d+- (list ph (- ph))
            )
            (setq t-  (mapcar '+ p (list height 0.0))
                  b-  (mapcar '- p (list height 0.0))
                  d+- (list (- ph) (- ph))
                  d++ (list ph (- ph))
            )
          )
          (setq t+ t-
                b+ b-
          )
          (grvecs (list color t- b-))
          (repeat n
            (grvecs (list color (setq t- (mapcar '- t- d++)) (setq b- (mapcar '- b- d+-))))
            (grvecs (list color (setq t+ (mapcar '+ t+ d+-)) (setq b+ (mapcar '+ b+ d++))))
          )
        )
        (defun Num0 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
          (Barrer (mapcar '+ midp (list (- 1/4h) (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list (- 1/4h) 1/4h)) 1/2h color "UP")
        )
        (defun Num1 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer (mapcar '+ midp (list 0.0 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/4h))) 1/2h color "UP")
        )
        (defun Num2 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
          (Barrer (mapcar '+ midp (list (- 1/4h) (- 1/4h))) 1/2h color "UP")
        )
        (defun Num3 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
        )
        (defun Num4 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list (- 1/4h) 1/4h)) 1/2h color "UP")
        )
        (defun Num5 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
          (Barrer (mapcar '+ midp (list (- 1/4h) 1/4h)) 1/2h color "UP")
        )
        (defun Num6 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
          (Barrer (mapcar '+ midp (list (- 1/4h) (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list (- 1/4h) 1/4h)) 1/2h color "UP")
        )
        (defun Num7 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
        )
        (defun Num8 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
          (Barrer (mapcar '+ midp (list (- 1/4h) (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list (- 1/4h) 1/4h)) 1/2h color "UP")
        )
        (defun Num9 (midp height color / h 1/4h 1/2h)
          (setq 1/2h (* height 0.5)
                1/4h (* height 0.25)
                midp (list (car midp) (cadr midp))
          )
          (Barrer midp 1/2h color nil)
          (Barrer (mapcar '+ midp (list 0.0 1/2h)) 1/2h color nil)
          (Barrer (mapcar '+ midp (list 1/4h 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 1/4h (- 1/4h))) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list (- 1/4h) 1/4h)) 1/2h color "UP")
          (Barrer (mapcar '+ midp (list 0.0 (- 1/2h))) 1/2h color nil)
        )

        (setq midp (list (car midp) (cadr midp))
              p    (polar midp pi (* height 0.4 (1- (strlen str))))
              i    1
        )
        (repeat (strlen str)
          (if (eval (setq fun (read (strcat "Num" (substr str i 1)))))
            (eval (list fun (quote p) height color))
          )
          (setq p (polar p 0.0 (* height 0.8))
                i (1+ i)
          )
        )
      )

      ;;(defun digicount (/ n p h i)
      ;;  (setq n 999
      ;;        p (getvar "viewctr")
      ;;        h (/ (getvar "viewsize") 10)
      ;;  )
      ;;  (while (<= n 2001)
      ;;    (setq n (1+ n))
      ;;    (DigConv p (rtos n 2 0) h 3)
      ;;    (princ)
      ;;    (DigConv p (rtos n 2 0) h 0)
      ;;  )
      ;;  (DigConv p (rtos n 2 0) h 3)
      ;;)

      ;; ! Function : Returns a point on a line at a specified distance from the first
      ;; !            point of the line.
      ;; ! Argument : 'pt1' - First point of the line (can be a 3d point too)
      ;; !            'pt2' - Last point of the line  (can be a 3d point too)
      ;; !            'd'   - Distance of point along line
      ;; ! Returns  : an interpolated 3d point between 'pt1' and 'pt2' at a distance
      ;; !            of 'd' from 'pt1' along 'pt2'
      (defun PutPtOnLineAtDist (pt1 pt2 d Flag / len Z Z1 Z2 dZ dH ang pt)
        (setq
          Z1  (caddr pt1)
          Z2  (caddr pt2)
          len (distance pt1 pt2)
          ang (angle pt1 pt2)
          Rang (angle pt2 pt1)    ;Reverse angle
          dZ  (/ (* d (- Z2 Z1)) len)
          dH  (sqrt (- (LeCarre d) (LeCarre dZ)))
        )
        (if (= Flag nil)
          (setq pt (polar pt1 ang dH)
                pt (list (car pt) (cadr pt) (+ Z1 dZ))
          )
          (setq pt (polar pt1 Rang dH)
                pt (list (car pt) (cadr pt) (+ Z1 dZ))
          )
        )
        pt
      )

      (defun LeCarre (nombre /)
        (* nombre nombre)
      )

      ;;;  Description:  This function tests and "fixes" very small numbers  ;
      ;;;                This is required when comparing reals to take care  ;
      ;;;                of rounding problems.  The granularity of the       ;
      ;;;                comparison is very fine, which should guarantee     ;
      ;;;                that this function will work for any garden path    ;
      ;;;                situation.  However, if this is applied to other    ;
      ;;;                applications, be sure to check for tolerances and   ;
      ;;;                numeric accuracy.                                   ;
      (defun gp:ZeroSmallNum (num)
        (setq num (rtos num 2 24))
        (distof
          (if (or (wcmatch num "*e-*")
                  (wcmatch num "*E-*")
              )
            "0.0"
            num
          )
        )
      )

      ;;;  Description:  This function converts any real or integer number   ;
      ;;;                into a string decimal.                              ;
      (defun gp:Rtos2 (AnyNumber) (rtos AnyNumber 2))

      ;;;  Description:  This function tests if points p1 and p2 are equal,  ;
      ;;;                given the accuracy defined in gp:ZeroSmallNum.      ;
      (defun gp:PointEqual (p1 p2 /)
        (setq p1 (mapcar 'gp:ZeroSmallNum p1) ; check for very small numbers in p1 and fix them
              p2 (mapcar 'gp:ZeroSmallNum p2) ; check for very small numbers in p2 and fix them
              p1 (mapcar 'gp:Rtos2 p1)
              p2 (mapcar 'gp:Rtos2 p2)
        )
        ;; Does every element the list p1 and p2 equal T
        (Vl-Every '(lambda (x) (equal x 'T)) (mapcar 'equal p1 p2))
      )

      ;;; Sets (operations on unordered lists)
      ;;; Our sets are unordered, most operations are not stable, but duplicates
      ;;; are ignored or removed.

      ;;; AA_INTERSECTION returns all elements that are in List1 and List2.
      ;;; Iterative, stable version by Serge Pashkov.
      ;;; Duplicates are handled incorrectly yet.
      ;;;   (AA_intersection '(0 1 2 3 4 5) '(2 0 6 7)) => (0 2)
      ;;; Problem: (AA_intersection '(0 0 0 1) '(0 0 1)) => (0 0 0 1)
      ;;;  but it should be (0 0 1) instead
      (defun AA_INTERSECTION (List1 List2 / new)      ;Ca marche, renvoi les elements communs
        (setq new nil)                                ;possible VL lsa compiler bug
        (foreach ele List1
          (if (member ele List2)
            (setq new (cons ele new))
          )
        )
        (reverse new)
      )

      ;;; highly recursive, may fail on large lists!

      ;;; AA_SET-DIFFERENCE returns all elements of List1 without elements
      ;;; also in List2, iterative, stable version by Serge Pashkov
      ;;;   (AA_set-difference '(0 1 2 3 4 5) '(2 0 6 7)) => (1 3 4 5)
      ;;; Big lists as second argument are faster:
      ;;;   (setq big (AA_int-list 1000) small (AA_int-list 10))
      ;;;   (AA_time '(AA_set-difference small big))
      ;;;   => (AA_SET-DIFFERENCE SMALL BIG)  : 0 ms
      ;;;   (AA_time '(AA_set-difference big small))
      ;;;   (AA_SET-DIFFERENCE BIG SMALL)     : 50 ms
      (defun AA_SET-DIFFERENCE (List1 List2 / new) ;Ca marche, renvoi ce qui est dans la 1ere mais pas 2eme liste
        (setq new nil)
        (foreach ele List1
          (if (not (member ele List2))
            (setq new (cons ele new))
          )
        )
        (reverse new)
      )

      ;;; highly recursive, may fail on large lists!

      ;;; AA_UNION returns a list of all elements in both lists
      ;;; Iterative version by Serge Pashkov
      ;;;   (AA_union '(0 1 2 3 4 5) '(2 0 6 7)) => like (7 6 0 1 2 3 4 5)
      ;;; Ignore duplicates in List2 are ignored, in List1 not.
      ;;; Simplified version without any ordering.
      ;;; See AA_ORDERED-UNION which keeps the orginal order intact.
      ;;; Big lists as first argument are faster:
      ;;;   (setq big (AA_int-list 1000) small (AA_int-list 10))
      ;;;   (AA_time '(AA_union small big))
      ;;;   => (AA_UNION SMALL BIG)   : 1222 ms
      ;;;   (AA_time '(AA_union big small))
      ;;;   (AA_UNION BIG SMALL)  : 0 ms
      (defun AA_UNION (List1 List2)
        (foreach ele List2
          (if (not (member ele List1))
            (setq List1 (cons ele List1))
          )
        )
        List1
      )

      ;;; We use reversing only for appending missing elements
      ;;; after the end of List1. Iterative version by Serge Pashkov
      (defun AA_ORDERED-UNION (List1 List2 / new)
        (setq new (reverse List1))
        (foreach ele List2
          (if (not (member ele new))
            (setq new (cons ele new))
          )
        )
        (reverse new)
      )

      ;;; highly recursive, may fail on large lists!

      ;;; Elements that appear in exactly one of lst1 and lst2
      (defun AA_SET-EXCLUSIVE-OR (List1 List2)
        (append (AA_set-difference List1 List2)
            (AA_set-difference List2 List1)
        )
      )

      ;;; AA_subsetp returns T if every element of List1 is a member of List2,
      ;;; ignoring order and duplicates, iterative version by Serge Pashkov
      ;;;   (AA_subsetp '(7 0) '(2 0 6 7)) -> T
      ;;;   (AA_subsetp '(1 7) '(2 0 6 7)) -> nil
      ;;; Note: in CL the default comparison is with EQL not with EQUAL!
      ;;;   (AA_subsetp '((1) (2)) '((1) (2))) => T
      ;;;   but in CL it would return nil
      (defun AA_SUBSETP (List1 List2)
        (while (and List1 (member (car List1) List2))
          (setq List1 (cdr List1))
        )
        (null List1)
      )

      ;;; highly recursive, may fail on large lists!

      ;;; have both sets the same elements? ignoring order and duplicates
      ;;;  (AA_set-equal-p '(a b c) '(c b a))   => T
      ;;;  (AA_set-equal-p '(a b c) '(c b a a)) => T
      (defun AA_SET-EQUAL-P (List1 List2)
        (and (AA_subsetp List1 List2)
             (AA_subsetp List2 List1)
        )
      )

      ;; ! Function : Return the x,y and z extents of a list of points
      ;; ! Argument : 'vlist' - List of points
      ;; ! Returns  : A list of p1 and p2, where p1 is lower left corner point (x,y,z)
      ;; !            and p2 is the upper right corner point (x,y,z).
      ;; ! Update   : March 6, 1998
      ;; ! Return the Extents of a list of points.
      (defun GetExtents (vlist / MinX MaxX MinY MaxY MinZ MaxZ tmp pt)
        (setq
          MinX 1E20
          MinY 1E20
          MinZ 1E20
          MaxX -1E20
          MaxY -1E20
          MaxZ -1E20
        )
        (foreach pt vlist
          (if (< (setq tmp (car pt)) MinX)
            (setq MinX tmp)
          )
          (if (< (setq tmp (cadr pt)) MinY)
            (setq MinY tmp)
          )
          (if (> (setq tmp (car pt)) MaxX)
            (setq MaxX tmp)
          )
          (if (> (setq tmp (cadr pt)) MaxY)
            (setq MaxY tmp)
          )
          (if (and (setq tmp (caddr pt)) (< tmp MinZ))
            (setq MinZ tmp)
          )
          (if (and (setq tmp (caddr pt)) (> tmp MaxZ))
            (setq MaxZ tmp)
          )
        )

        (if (= MinZ 1E20)
          (setq MinZ 0.0)
        )
        (if (= MaxZ -1E20)
          (setq MaxZ 0.0)
        )

        (list (list MinX MinY MinZ) (list MaxX MaxY MaxZ))
      )

      (Defun DelDoublon (lAFaire /)
        (setq iCnt  -1
              iCntx (length lAFaire)
        )
        (while (< iCnt iCntx)
          (setq iCnt (1+ iCnt))
          (if (> (Mem_Cnt (nth iCnt lAFaire) lAFaire) 1)
            (setq lAFaire (remove_n iCnt lAFaire)
              iCntx   (length lAFaire)
              iCnt    -1
            )
          )
        )
        lAFaire
      )

      (defun RemDupFromList1 (List1 List2 rFac /) ;For Pt list only
        (setq iCnt1 0
              iCnt2 0
              iCntx (length List1)
              iCnty (length List2)
        )
        (while (< iCnt1 iCntx)
          (while (< iCnt2 iCnty)
            (if (equal (nth iCnt1 List1) (nth iCnt2 List2) rFac)
              (setq List1 (remove_n iCnt1 List1)
                    iCntx (length List1)
              )
            )
            (setq iCnt2 (1+ iCnt2))
          )
          (setq iCnt1 (1+ iCnt1)
                iCnt2 0
          )
        )
        List1
      )

      ;; ! Function : Rearranges the passed points 'p1' and 'p2' such that the returned
      ;; !            points are the left and right corners.
      ;; ! Argument : 'p1' and 'p2', two passed points
      ;; ! Returns  : A list of p1 and p2, where p1 is left point and p2 is right point.
      (defun Left2R (p1 p2 / )
        (if (> (car p1) (car p2))
          (list p2 p1)
          (list p1 p2)
        )
      )

      (defun C:ADO (/ pN1 pN2 pN3 ang1
                      lBuildX lBase pRouge pStart pBleu
                      sAns2 rControl AllWalls aVersRed aVersBlue
                   )
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )

        (command "_.UCS" "_W")

        (princ (strcat "\nAuto Dimensions Outside." copyright))

        (setq oBase (objsel "LINE" "Select a line defining start of dimension" T))

        (setq obobj (vlax-ename->vla-object oBase))
        (setq lBase (VxGetExtrems obobj))
        (setq lBase (Left2R (car lBase) (cadr lBase)))

        (setq pN1   (car lBase)
              pN2   (cadr lBase)
              pN3   (mid_pt pN1 pN2)
              p     (getvar "viewctr")
              h     (/ (getvar "viewsize") 12)
              nVS   (getvar "viewsize")
              ang1      (angle pN1 pN2)
              aVersRed  (- ang1 hshpi90)
              aVersBlue (+ ang1 hshpi90)
              pRouge    (polar pN3 aVersRed (* 1.0 h)) ;turns right into red
              pBleu     (polar pN3 aVersBlue (* 1.0 h)) ;turns left into blue
        )

        (command "_.POINT" "0,0")
        (setq Marque (entlast))
        (entdel (entlast))

        ;;  (DigConv pRouge (rtos 5 2 0) h 20)
        ;;  (DigConv pBleu  (rtos 5 2 0) h 150)
        ;;  (if (eq rWHigh nil)
        ;;    (setq rWHigh 2400)
        ;;  )
        ;;  (setq sAns2
        ;;     (ukword
        ;;       1
        ;;       "Blue Red"
        ;;       "Specify on which side of plane is the building plan located. In [Blue/Red] sector? "
        ;;       "Blue"
        ;;     )
        ;;  )
        ;;  (if (= sAns2 "Blue")
        ;;    (command "_.UCS" "_3POINT" pN1 pN2 pBleu)
        ;;    (command "_.UCS" "_3POINT" pN2 pN1 pRouge)
        ;;  )
        ;;  (setq rWHigh (udist 6 "" "Specify wall height" rWHigh nil))
        ;;  (princ "\nSelect objects in plan view to use for generating the elevation...")

        (if (= (rtd ang1) 270)
          (command "_.UCS" "_3POINT" pN2 pN1 pRouge)
          (command "_.UCS" "_3POINT" pN1 pN2 pBleu)
        )

        (while
          (progn
            (princ "\nSelect objects in plan view to use for dimensioning...")
            (command "_.SELECT" pause)
            (not (setq ss1 (ssget "_P")))
          )
          (print "0 found")
        )

        ;;;(setq pCnr2
        ;;;  (GetCornerEx
        ;;;    (setq pCnr1 (getpoint "\nSpecify first corner point: "))
        ;;;    "\nSpecify opposite corner: "
        ;;;  )
        ;;;)

        ;; selected objects are in ss1

        (setq iCnt       0
              iCntx      (sslength ss1)
              lFirstPass nil
        )

        (while (< iCnt iCntx)  ;Toutes les extremites
          (if (or (= (dxf 0 (entget (ssname ss1 iCnt))) "LINE")
                  (= (dxf 0 (entget (ssname ss1 iCnt))) "ARC")
              )
            (progn
              (setq obobj (vlax-ename->vla-object (ssname ss1 iCnt)))
              (setq lDesPt (VxGetExtrems obobj))
              (setq lFirstPass (append lFirstPass lDesPt))  ;Mises dans la liste des points
            )
          )
          (setq iCnt (1+ iCnt))
        )

        (setq lFirstPass (Remove_Dup lFirstPass))                ;Enlever les doublons
        (setq pDepart (nth 0 (VLxx_SortPoints lFirstPass "X")))  ;Pt de depart
        ;;;(command "_.CIRCLE" pDepart 100)
        (setq pArrive (last (VLxx_SortPoints lFirstPass "X")))   ;Arrivee
        ;;;(command "_.CIRCLE" pArrive 200)
        (setq lBuildX (GetExtents lFirstPass))                   ;Extremites du batiment
        ;;;(command "_.LINE" (car lBuildX) (cadr lBuildX) "")

        (setq pOnLine     (PerpPt2Line       ;Pt de depart sur la ligne de vue
                            pDepart
                            (trans pN1 0 1)
                            (trans pN2 0 1)
                          )
              aToBuilding (angle pOnLine pDepart)
              pOnLine     (PutPtOnLineAtDist pOnLine pDepart 20 nil) ;Apres correction
              iIncDist    0
              lPLaunch    nil
              pLaunch     (car lBuildX)
        )

        (while (< (car pLaunch) (- (car pArrive) 200))
          (setq iIncDist (+ iIncDist 172)               ;tous les 0.172m
                pLaunch  (polar pOnLine 0 iIncDist)
                lPLaunch (append lPLaunch (list pLaunch))  ;Mise dans la liste des points
          )
        )

        (setq lPLaunch (Remove_Dup lPLaunch))      ;Enlever les doublons
        (setq ssGardes (ssadd)
              ssOpens  (ssadd)
        )
        (if (= (rtd aToBuilding) 90)
          (setq rLimite (abs (- (max (cadar (GetExtents lFirstPass))
                                     (cadadr (GetExtents lFirstPass))
                                )
                                (cadr (last lPLaunch))
                             )
                        )
          )
          (setq rLimite (abs (- (min (cadar (GetExtents lFirstPass))
                                     (cadadr (GetExtents lFirstPass))
                                )
                                (cadr (last lPLaunch))
                             )
                        )
          )
        )

        (foreach item lPLaunch   ;Pour chaque pt
          (setq iIncDist 0
                iNum 0
                bCarry T
          )

          (while (= bCarry T)  ;On continu?
            (dos_waitcursor T)
            (setq iIncDist (+ iIncDist 80))   ;Projete tous les 80mm
            (setq ssWalls (ssget "_F" (list item (polar item aToBuilding iIncDist))))
            (princ "\t\t\t\rProcessing data, please wait...  ")

            (if (/= ssWalls nil)  ;Si des objets sont trouves
              (progn
                (setq oWall (ssname sswalls iNum))

                (if (/= (member oWall (sslist ss1)) nil)          ;Et appartenant a ss1
                  (progn          ;++
                    (if (or (= (dxf 0 (entget oWall)) "LINE")(= (dxf 0 (entget oWall)) "ARC"))
                      (progn          ;//
                        (setq ssGardes (ssadd oWall ssGardes))
                        (setq bCarry nil)                 ;On arrete de lancer, pour attaquer un autre pt
                        ;;;(command "_.LINE" item (polar item aToBuilding iIncDist) "")
                      )
                    )

                    ;;(if (= (dxf 0 (entget oWall)) "LINE"))
                    (if (= (dxf 0 (entget oWall)) "INSERT")           ;Si c'est un bloc porte ou fenetre
                      (progn
                        (setq lXDs (ReadXD oWall sAppNameAA))
                        (if (and (/= lXDs nil)
                                 (or (= (car lXDs) "DOORPL")
                                     (= (car lXDs) "WINDPL")
                                 )
                            )
                          (progn
                            (setq ssOpens (ssadd oWall ssOpens))
                            (setq bCarry nil)
                            ;;;(command "_.LINE" item (polar item aToBuilding iIncDist) "")
                          )
                          (progn                      ;Tout autre bloc
                            (setq bCarry T)
                            (setq iIncDist (- rLimite 80))   ;;  iNum (1+ iNum)
                            (command "_.LINE"
                                 item
                                 (polar item aToBuilding iIncDist)
                                 ""
                            )
                          )
                        )
                      )
                    )
                  )
                  (progn
                    (setq ssWalls nil  ssWalls (ssget "_F" (list item (polar item aToBuilding rLimite))))
                    (setq ssGuard nil  ssGuard (ssadd))
                    (foreach wall (sslist ssWalls)
                      (if (/= (member wall (sslist ss1)) nil)  ;Select tous objets
                        (setq ssGuard (ssadd wall ssGuard))    ;Garde ceux appartenant a ss1
                      )
                    )
                    (setq   iCnt     0   ;Tri des retenus
                            iCntx    (sslength ssGuard)
                            aGarder  nil
                            rEtalon  rLimite
                    )
                    (while (< iCnt iCntx)
                      (if (= (dxf 0 (entget (ssname ssGuard iCnt))) "INSERT")   ;Si c'est une porte ou fenetre
                        (progn
                          (setq lXDs (ReadXD (ssname ssGuard iCnt) sAppNameAA))
                          (if (and (/= lXDs nil)
                                   (or (= (car lXDs) "DOORPL")
                                       (= (car lXDs) "WINDPL")
                                   )
                              )
                            (progn
                              (setq ssOpens (ssadd (ssname ssGuard iCnt) ssOpens))
                              (setq bCarry nil  bOvert T)
                              ;;;(command "_.LINE" item (polar item aToBuilding iIncDist) "")
                            )
                          )
                        )
                      )

                      (if (or (= (dxf 0 (entget (ssname ssGuard iCnt))) "LINE")(= (dxf 0 (entget oWall)) "ARC"))
                        (progn
                          (setq obobj (vlax-ename->vla-object (ssname ssGuard iCnt)))
                          (setq lDesPt (VxGetExtrems obobj))
                          (setq pEntre (mid_pt (car lDesPt) (cadr lDesPt)))
                          (setq pOnLine (PerpPt2Line pEntre (trans pN1 0 1) (trans pN2 0 1))
                                rCheck  (distance pOnLine pEntre)
                          )
                          (if (< rCheck rEtalon)
                            (setq rEtalon rCheck  aGarder (ssname ssGuard iCnt))
                          )
                          (setq bOvert nil)
                        )
                      )

                      (setq iCnt (1+ iCnt))
                    )
                    (if (= bOvert nil)
                      (setq ssGardes (ssadd aGarder ssGardes)  bCarry nil)  ;On arrete de lancer, pour attaquer un autre pt
                    )
                  )
                )
              )
              (progn                      ;if (= ssWalls nil) Objets ne sont pas trouves
                (setq bCarry T)
                (if (> iIncDist rLimite)  ;Cherchons jusqu'a la limite
                  (setq bCarry nil)       ;On arrete de lancer, pour attaquer un autre pt
                )
                ;;;(command "_.LINE" item (polar item aToBuilding iIncDist) "")
              )
            )
          )
        )
        (dos_waitcursor)

        ;;;-------
        (setq iCnt     0
              iCntx    (sslength ssGardes)
              lGardes  nil
              lGFinals nil
        )
        (while (< iCnt iCntx)                         ;Toutes les extremites
          (setq obobj (vlax-ename->vla-object (ssname ssGardes iCnt)))
          (setq lDesPt (VxGetExtrems obobj))
          (setq lGardes (append lGardes lDesPt))      ;Mises dans la liste des points a garder
          (setq iCnt (1+ iCnt))
        )
        ;;;(setq lGardes (add_n 1 lgardes (car lBuildX)))
        ;;;(setq lGardes (add_n 1 lgardes (cadr lBuildX)))
        (setq lGardes (Remove_Dup lGardes))           ;Enlever les doublons
        (foreach pItem lGardes                        ;Les coins + encadrures sur le batiment
          (setq lk (PerpPt2Line
                     pItem
                     (trans (car lBase) 0 1)
                     (trans (cadr lBase) 0 1)
                   )
          )
          (setq lGFinals (append lGFinals (list lk)))             ;Les coins + encadrures projetes sur la base
        )
        (setq lGFinals (Remove_Dup lGFinals))                 ;Enlever les doublons
        (setq lGFinals (DelDoublon lGFinals))

        (cassure "lGFinals 2")

        ;;;------
        (setq iCnt     0
              iCntx    (sslength ssOpens)
              lOtherpt nil
              lOFinals nil
        )
        (while (< iCnt iCntx)
          (setq p1p      (cdr (nth 6 (AAXDataGet (ssname ssOpens iCnt) sAppNameAA)))      ;Hinge
                p2p      (cdr (nth 7 (AAXDataGet (ssname ssOpens iCnt) sAppNameAA)))      ;meme cote
                p3p      (cdr (nth 8 (AAXDataGet (ssname ssOpens iCnt) sAppNameAA)))      ;cote oppose
                lOtherpt (append lOtherpt
                                  (list (trans p1p 0 1))
                                  (list (trans p2p 0 1))
                                  (list (trans p3p 0 1))
                         )
          )
          (setq iCnt (1+ iCnt))
        )
        (setq lOtherpt (Remove_Dup lOtherpt))         ;Enlever les doublons
        (foreach pItem lOtherpt               ;Les encadrures sur le batiment
          (setq lk (PerpPt2Line
                     pItem
                     (trans (car lBase) 0 1)
                     (trans (cadr lBase) 0 1)
                   )
          )
          (setq lOFinals (append lOFinals (list lk)))     ;Les encadrures projetes sur la base
        )
        (setq lOFinals (Remove_Dup lOFinals))         ;Enlever les doublons
        (setq lOFinals (DelDoublon lOFinals))

        (setq aEst (+ aToBuilding pi))

        (setq lGFinals (VLxx_SortPoints lGFinals "X")
              emx      (length lGFinals)
              iter     1
        )      ;Les coins + encadrures sur la base
        (setq pLoc (polar (trans (car lBase) 0 1)
                          aEst
                          (* 1 (getvar 'DIMDLI) (getvar 'DIMSCALE))
                   )
        )
        ;;;cotation des coins + encadrures
        (command "_.DIMLINEAR"
                 (nth 0 lGFinals)
                 (nth 1 lGFinals)
                 pLoc
        )
        (command "_.DIMCONTINUE")
        (while (< iter emx)
          (command (nth (1+ iter) lGFinals))
          (setq iter (1+ iter))
        )

        ;;;cotation des coins
        (setq lOFinal (RemDupFromList1 lGFinals lOFinals 0.01))
        (setq lOFinal (RemDupFromList1 lOFinal lOFinals 0.0001)
              lOFinal (VLxx_SortPoints lOFinal "X")
              emx     (length lOFinal)
              iter    1
        )     ;Les coins ***** sur la base
        (setq pLoc (polar (trans (car lBase) 0 1)
                          aEst
                          (* 2 (getvar 'DIMDLI) (getvar 'DIMSCALE))
                   )
        )  ;cotation extremites du batiment
        (command "_.DIMLINEAR" (nth 0 lOFinal) (nth 1 lOFinal) pLoc)
        (command "_.DIMCONTINUE")
        (while (< iter emx)                   ;cotation des coins
          (command (nth (1+ iter) lOFinal))
          (setq iter (1+ iter))
        )

        ;;;cotation extremites opposes du batiment
        (setq pLoc (polar (trans (car lBase) 0 1)
                          aEst
                          (* 3 (getvar 'DIMDLI) (getvar 'DIMSCALE))
                   )
        )
        (command "_.DIMLINEAR"
             (PerpPt2Line
               (car lBuildX)
               (trans (car lBase) 0 1)
               (trans (cadr lBase) 0 1)
             )
             (PerpPt2Line
               (cadr lBuildX)
               (trans (car lBase) 0 1)
               (trans (cadr lBase) 0 1)
             )
             pLoc
        )

        ;;;  (setq    sAns3
        ;;;    (ukword
        ;;;      1
        ;;;      "Yes No"
        ;;;      "Any doors or windows to appear in current elevation? [Yes/No] "
        ;;;      "Yes"
        ;;;    )
        ;;;  )
        ;;;  (if (= sAns3 "Yes")
        ;;;    (DrawElevs)
        ;;;  )
        ;;;  (setq AllWalls (SSAfterObj Marque))

        (command "_.UCS" "_W")

        ;;;  (if (= sAns2 "Blue")
        ;;;    (command "_.ROTATE"
        ;;;        AllWalls
        ;;;        ""
        ;;;        pN1
        ;;;        (* -1 (- (rtd aVersBlue) 90))
        ;;;    )
        ;;;    (command "_.ROTATE"
        ;;;        AllWalls
        ;;;        ""
        ;;;        pN1
        ;;;        (* -1 (- (rtd aVersRed) 90))
        ;;;    )
        ;;;  )
        ;;;  (prompt "Specify location for the elevation.")
        ;;;  (command "_.MOVE" AllWalls "" pN1 pause)

        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (TraceIsOn ".19e")

      ;--------------------
      ;---Text Placements.
      ;--------------------
      (defun C:T4 (/ Hshhgt)
        (setq olderror *error*  *error* Wallserror  Hshhgt nil  Hshdwgsc nil)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\n7mm Text." copyright)
          (strcat "\n7mm Text." copyright)
          (strcat "\n7mm Text." copyright)
          (strcat "\n7mm Text." copyright)
        )
        (prompt "\nSpecify start point and rotation angle of text: ")
        (setq bPTxt bTxt  bTxt nil)
        (Hshset_lay sText4Lyr sText4Col "CONTINUOUS" sText4Lwt)   ;texte 4
        (setq Hshdwgsc (getvar "DIMSCALE"))
        (setq Hshhgt (* Hshdwgsc 7.0))
        ;;LCA - WARNING: The DTEXT command requires an extra argument.
        (command "_.DTEXT" pause Hshhgt pause)
        (setq bTxt bPTxt)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun C:T3 (/ Hshhgt)
        (setq olderror *error*  *error* Wallserror  Hshhgt nil  Hshdwgsc nil)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\n5mm Text." copyright)
          (strcat "\n5mm Text." copyright)
          (strcat "\n5mm Text." copyright)
          (strcat "\n5mm Text." copyright)
        )
        (prompt "\nSpecify start point and rotation angle of text: ")
        (setq bPTxt bTxt  bTxt nil)
        (Hshset_lay sText3Lyr sText3Col "CONTINUOUS" sText3Lwt)
        (setq Hshdwgsc (getvar "DIMSCALE"))
        (setq Hshhgt (* Hshdwgsc 5.0))
        (command "_.DTEXT" pause Hshhgt pause)
        (setq bTxt bPTxt)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun C:T2 (/ Hshhgt)
        (setq olderror *error*  *error* Wallserror  Hshhgt nil  Hshdwgsc nil)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\n3.5mm Text." copyright)
          (strcat "\n3.5mm Text." copyright)
          (strcat "\n3.5mm Text." copyright)
          (strcat "\n3.5mm Text." copyright)
        )
        (prompt "\nSpecify start point and rotation angle of text: ")
        (setq bPTxt bTxt  bTxt nil)
        (Hshset_lay sText2Lyr sText2Col "CONTINUOUS" sText2Lwt)
        (setq Hshdwgsc (getvar "DIMSCALE"))
        (setq Hshhgt (* Hshdwgsc 3.5))
        (command "_.DTEXT" pause Hshhgt pause)
        (setq bTxt bPTxt)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun C:T1 (/ Hshhgt)
        (setq olderror *error*  *error* Wallserror  Hshhgt nil  Hshdwgsc nil)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\n2.5mm Text." copyright)
          (strcat "\n2.5mm Text." copyright)
          (strcat "\n2.5mm Text." copyright)
          (strcat "\n2.5mm Text." copyright)
        )
        (prompt "\nSpecify start point and rotation angle of text: ")
        (setq bPTxt bTxt  bTxt nil)
        (Hshset_lay sText1Lyr sText1Col "CONTINUOUS" sText1Lwt)
        (setq Hshdwgsc (getvar "DIMSCALE"))
        (setq Hshhgt (* Hshdwgsc 2.5))
        (command "_.DTEXT" pause Hshhgt pause)
        (setq bTxt bPTxt)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun C:T0 (/ Hshhgt)
        (setq olderror *error*  *error* Wallserror  Hshhgt nil  Hshdwgsc nil)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\n1.5mm Text." copyright)
          (strcat "\n1.5mm Text." copyright)
          (strcat "\n1.5mm Text." copyright)
          (strcat "\n1.5mm Text." copyright)
        )
        (prompt "\nSpecify start point and rotation angle of text: ")
        (setq bPTxt bTxt  bTxt nil)
        (Hshset_lay sText0Lyr sText0Col "CONTINUOUS" sText0Lwt)
        (setq Hshdwgsc (getvar "DIMSCALE"))
        (setq Hshhgt (* Hshdwgsc 1.5))
        (command "_.DTEXT" pause Hshhgt pause)
        (setq bTxt bPTxt)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;---------------------------
      ;---Non-Assisted Join Walls.
      ;---------------------------
      (defun NonAss (/ pt1
                       ssGardLin1 ssGardLin3
                       aGoto  aCross1
                       OFFDIST    rMurThk
                       xFactor    yFactor
                       pC1    pC1a
                       pC2    pC2a
                       pStLn1 pEnLn1
                       pEnLn2 iNum
                       iNum4G1    iNum4G3
                       sPorteOuiNon
                    )
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (command "_.UCS" "_W")
        (princ (strcat "\nHeal Walls." copyright))
        (setvar "osmode" 0)
        (setq doorpick (objsel "INSERT" "Select door or window to remove" nil))
        (if (/= nil (AAXDataGet (car doorpick) sAppNameAA))
          (setq sPorteOuiNon (cdr (nth 0 (AAXDataGet (car doorpick) sAppNameAA))))    ;Une porte ou fenetre? Vue en plan?
        )
        (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
          (progn
            (setq ODeNom  (car doorpick)
                  sDrHan2 (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))        ;Database handle
                  OFFDIST (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))))     ;Door width
                  pHinge  (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))        ;Hinge
                  pMCote  (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))        ;meme cote
                  pCoteO  (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))        ;cote oppose
                  rMurThk (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))       ;Wall thk
                  sTofIns (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))       ;Inseree a quelle heure?

                  ODeNom  (entget ODeNom)
                  lDoor   ODeNom
                  aA1     (angle pHinge pCoteO)
                  aA3     (angle pHinge pMCote)
                  aDiff1  (- aA1 aA3)
                  pYAxis  (polar pHinge (+ Pi aA1) rMurThk)
                  pPicked (cadr doorpick)
                  ODrNam  (dxf -1 lDoor)
                  pCnr1   (list 0 0 0)
                  ODrNam  (dxf -1 lDoor)
                  oBlNam  (dxf 2 lDoor)
                  xFactor (dxf 41 lDoor)
                  yFactor (dxf 42 lDoor)
            )
            (command "_.UCS" "_3POINT" pHinge pMCote pYAxis)

            (if (minusp yFactor)
              (if (minusp xFactor)
                (setq sOuvreTo "R")
                (setq sOuvreTo "L")
              )
              (if (minusp xFactor)
                (setq sOuvreTo "L")
                (setq sOuvreTo "R")
              )
            )

            (if (= sOuvreTo "L")
              (setq aCross1 135
                    aCross2 0
                    aCross3 HshPi270
              )
              (setq aCross1 135
                    aCross2 0
                    aCross3 HshPi270
              )
            )
            (setq pC1    (polar pCnr1 (dtr aCross1) 70)
                  pC1a   (polar pC1 aCross3 (+ rMurThk 99))
                  pC2    (polar pC1 aCross2 (+ Offdist 99))
                  pC2a   (polar pC2 aCross3 (+ rMurThk 99))
                  bCont   nil
                  bACont  nil
                  bPCont  nil
                  bLwCont nil
                  eObject1 nil
                  eObject2 nil
                  lObj1   nil
                  lObj2   nil
                  BonSens1 nil
                  BonSens2 nil
                  VxList1 nil
                  VxList2 nil
                  iCount 0
            )
            (entdel (car doorpick))
            (command "_.ERASE" (ssget "_WP" (list pC1 pC1a pC2a pC2)) "")
            ;;;(command "_.PLINE" pC1 pC1a pC2a pC2 "")

            ;;;(PointMake (trans pC1 1 0) "1")
            ;;;(PointMake (trans pC1a 1 0) "1a")
            ;;;(PointMake (trans pC2a 1 0) "2a")
            ;;;(PointMake (trans pC2 1 0) "2")

            (setq pBL  (trans pC1a 1 0)       ;0=World (WCS), 1=User (current UCS)
                  pTR  (trans pC2 1 0)
                  llur (CornersGet pBL pTR)
                  pBL  (car llur)
                  pTR  (cadr llur)
            )

            (setq ssGardArc1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "ARC")))
                  ssGardArc3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "ARC")))
                  ssGardLin1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "LINE")))
                  ssGardLin3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "LINE")))
                  ssGardPln1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "POLYLINE")))
                  ssGardPln3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "POLYLINE")))
                  ssGardLwP1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "LWPOLYLINE")))
                  ssGardLwP3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "LWPOLYLINE")))
            )

            (if (/= ssGardLin1 nil)           ;Pour les lignes
              (setq bCont    T
                    iNum4G1  (sslength ssGardLin1)
                    iNum     iNum4G1
                    ss1Garde ssGardLin1
              )
            )
            (if (/= ssGardArc1 nil)           ;Pour les arcs
              (setq bACont   T
                    iNum4G1  (sslength ssGardArc1)
                    iNumA    iNum4G1
                    ss1Guard ssGardArc1
              )
            )
            (if (/= ssGardPln1 nil)           ;Pour les polylignes
              (setq bPCont   T
                    iNum4G1  (sslength ssGardPln1)
                    iNumP    iNum4G1
                    ss1GardP ssGardPln1
              )
            )
            (if (/= ssGardLwp1 nil)           ;Pour les polylignes legeres
              (setq bLwCont   T
                    iNum4G1   (sslength ssGardLwp1)
                    iNumLw    iNum4G1
                    ss1GardLw ssGardLwp1
              )
            )
            (if (/= ssGardLin3 nil)
              (setq bCont    T
                    iNum4G3  (sslength ssGardLin3)
                    iNum     iNum4G3
                    ss2Garde ssGardLin3
              )
            )
            (if (/= ssGardArc3 nil)           ;Pour les arcs
              (setq bACont   T
                    iNum4G3  (sslength ssGardArc3)
                    iNumA    iNum4G3
                    ss2Guard ssGardArc3
              )
              (setq bACont nil)
            )
            (if (/= ssGardPln3 nil)           ;Pour les polylignes
              (setq bPCont   T
                    iNum4G3  (sslength ssGardPln3)
                    iNumP    iNum4G3
                    ss2GardP ssGardPln3
              )
            )
            (if (/= ssGardLwp3 nil)           ;Pour les polylignes legeres
              (setq bLwCont   T
                    iNum4G3   (sslength ssGardLwp3)
                    iNumLw    iNum4G3
                    ss2GardLw ssGardLwp3
              )
            )
            (if (equal iNum4G1 iNum4G3)
              (setq bCont1   T
                    bACont1  T
                    bPCont1  T
                    bLwCont1 T
                    bCont3   T
                    bACont3  T
                    bPCont3  T
                    bLwCont3 T
              )
              (progn (princ "\rError: Walls with unequal number of objects.")
                (setq bCont1   nil
                      bACont1  nil
                      bPCont1  nil
                      bLwCont1 nil
                      bCont3   nil
                      bACont3  nil
                      bPCont3  nil
                      bLwCont3 nil
                )
              )
            )
            (command "_.UCS" "_P")
            (if (and (= bCont T) (or (= bCont1 T)(= bCont3 T)) )
              (progn
                (While (< iCount iNum)
                  (setq eObject1 (ssname ss1Garde iCount)
                        eObject2 (ssname ss2Garde iCount)
                        lObj1    (entget eObject1)
                        lObj2    (entget eObject2)
                        pStLn1   (dxf 10 lObj1)          ;depart ligne 1
                        pStLn2   (dxf 10 lObj2)          ;depart ligne 2
                        pEnLn2   (dxf 11 lObj2)          ;fin ligne 2
                  )
                  (if (= (Colineaire pStLn2 pStLn1 pEnLn2 nil 5) T)
                    (progn (entdel eObject2)
                      (setvar "ORTHOMODE" 0)
                      (if (> (distance pStLn1 pStLn2) (distance pStLn1 pEnLn2))
                        (command "_.change" eObject1 "" (trans pStLn2 0 1))
                        (command "_.change" eObject1 "" (trans pEnLn2 0 1))
                      )
                    )
                    (princ "\rError: Non-colinear walls.")
                  )
                  (setq iCount (1+ iCount))
                )
              )
            )
            (if (and (= bACont T) (or (= bACont1 T)(= bACont3 T)) )
              (progn
                (While (< iCount iNumA)
                  (setq eObject1 (ssname ss1Guard iCount)
                        eObject2 (ssname ss2Guard iCount)
                        lObj1    (entget eObject1)
                        lObj2    (entget eObject2)
                        pCen1    (dxf 10 lObj1)
                        pCen2    (dxf 10 lObj2)
                        sLtyp    (dxf 6 lObj1)
                        sLyr (dxf 8 lObj1)
                        iClr (dxf 62 lObj1)
                  )
                  (if (equal (distance pCen1 pCen2) 0.0 0.001)
                    (progn
                      (setq lpt1 (polar pCen1 (dxf 50 lObj1) (dxf 40 lObj1))
                            lpt2 (polar pCen1 (dxf 51 lObj1) (dxf 40 lObj1))
                            lpt3 (polar pCen2 (dxf 50 lObj2) (dxf 40 lObj2))
                            lpt4 (polar pCen2 (dxf 51 lObj2) (dxf 40 lObj2))
                            ;;thick (dxf 39 lObj1)
                      )
                      (if (< (distance lpt1 lpt4) (distance lpt2 lpt3))
                        (command "_.arc" lpt3 lpt1 lpt2)
                        (command "_.arc" lpt1 lpt2 lpt4)
                      )
                      (entdel eObject1)(entdel eObject2)
                      (if (= iClr nil)
                        (setq iClr "BYLAYER")
                      )
                      (if (= sLtyp nil)
                        (setq sLtyp "BYLAYER")
                      )
                      ;;;(command "_.chprop" (entlast) "" "_LA" sLyr "_C" iClr "_LT" sLtyp "")
                      (vla-put-layer (vlax-Ename->vla-Object (entlast)) sLyr)
                      (vla-put-color (vlax-Ename->vla-Object (entlast)) iClr)
                      (vla-put-linetype (vlax-Ename->vla-Object (entlast)) sLtyp)
                    )
                    (progn
                      (if eObject1
                        (redraw eObject1)
                      )
                      (prompt "\nArcs centres must coincide. ")
                    )
                  )
                  (setq iCount (1+ iCount))
                )
              )
            )
            (if (and (= bPCont T) (or (= bPCont1 T)(= bPCont3 T)) )
              (progn
                (While (< iCount iNumP)
                  (setq eObject1 (ssname ss1GardP iCount)
                        eObject2 (ssname ss2GardP iCount)
                        lObj1    (entget eObject1)
                        lObj2    (entget eObject2)
                        sLyr     (dxf 8 lObj1)
                        VxList1  (Liste4Pline eObject1)
                        VxList2  (Liste4Pline eObject2)
                  )
                  (Hshset_lay sLyr nil nil nil)
                  (if (equal eObject1 eObject2)  ;si ce sont 2 differentes plines
                    (progn                       ;sinon
                      (setq VxList1 (remove_n (what_nth (first_Vtx eObject1) VxList1) VxList1))
                      (setq VxList1 (remove_n (what_nth (lastVtx eObject1) VxList1) VxList1))
                      ;;;(plinemake VxList1 0 0 nil)
                      (command "_.PLINE")(foreach pnt VxList1 (command pnt))(command "_C")
                      (entdel eObject1)
                    )
                    (progn
                      (if (= T (PtInRect (first_Vtx eObject1) pBL pTR))
                        (progn
                          (setq BonSens1 nil  VxList1 (remove_n (what_nth (first_Vtx eObject1) VxList1) VxList1))
                          (cassure "1er:BonSens1 (first_Vtx eObject1) VxList1")
                        )
                        (progn
                          (setq BonSens1 T  VxList1 (remove_n (what_nth (lastVtx eObject1) VxList1) VxList1))
                          (cassure "dernier:BonSens1 (lastVtx eObject1) VxList1")
                        )
                      )
                      (if (= T (PtInRect (first_Vtx eObject2) pBL pTR))
                        (progn
                          (setq BonSens2 T    VxList2 (remove_n (what_nth (first_Vtx eObject2) VxList2) VxList2))
                          (cassure "1er:BonSens2 (first_Vtx eObject2) VxList2")
                        )
                        (progn
                          (setq BonSens2 nil  VxList2 (remove_n (what_nth (lastVtx eObject2) VxList2) VxList2))
                          (cassure "dernier:BonSens2 (lastVtx eObject2) VxList2")
                        )
                      )
                      (if (= BonSens1 nil)
                        (setq VxList1 (reverse VxList1))
                      )
                      (if (= BonSens2 nil)
                        (setq VxList2 (reverse VxList2))
                      )
                      (plinemake (append VxList1 VxList2) 0 0 nil)
                      (entdel eObject1)(entdel eObject2)
                    )
                  )
                  (setq iCount (1+ iCount))
                )
              )
            )
            (if (and (= bLwCont T) (or (= bLwCont1 T)(= bLwCont3 T)) )
              (progn
                (While (< iCount iNumLw)
                  (setq eObject1 (ssname ss1GardLw iCount)
                        eObject2 (ssname ss2GardLw iCount)
                        lObj1    (entget eObject1)
                        lObj2    (entget eObject2)
                        sLyr     (dxf 8 lObj1)
                        pVSt1    (dxf 10 lObj1)
                        pVSt1    (list (car pVSt1)(cadr pVSt1) 0.0)
                        pVEn1    (dxf 10 (reverse lObj1))
                        pVEn1    (list (car pVEn1)(cadr pVEn1) 0.0)
                        pVSt2    (dxf 10 lObj2)
                        pVSt2    (list (car pVSt2)(cadr pVSt2) 0.0)
                        pVEn2    (dxf 10 (reverse lObj2))
                        pVEn2    (list (car pVEn2)(cadr pVEn2) 0.0)
                        VxList1  (Liste4LwPl eObject1)
                        VxList2  (Liste4LwPl eObject2)
                  )
                  (Hshset_lay sLyr nil nil nil)
                  (if (equal eObject1 eObject2)                  ;si ce sont 2 differentes plines
                    (progn                           ;sinon
                      (setq VxList1 (remove_n (what_nth pVSt1 VxList1) VxList1))
                      (setq VxList1 (remove_n (what_nth pVEn1 VxList1) VxList1))
                      (command "_.PLINE")(foreach pnt VxList1 (command pnt))(command "_C")
                      (entdel eObject1)
                    )
                    (progn
                      (if (= T (PtInRect pVSt1 pBL pTR))
                        (progn
                          (setq BonSens1 nil  VxList1 (remove_n (what_nth pVSt1 VxList1) VxList1))
                          (cassure "1er:BonSens1 pVSt1 VxList1")
                        )
                        (progn
                          (setq BonSens1 T    VxList1 (remove_n (what_nth pVEn1 VxList1) VxList1))
                          (cassure "dernier:BonSens1 pVEn1 VxList1")
                        )
                      )
                      (if (= T (PtInRect pVSt2 pBL pTR))
                        (progn
                          (setq BonSens2 T    VxList2 (remove_n (what_nth pVSt2 VxList2) VxList2))
                          (cassure "1er:BonSens2 pVSt2 VxList2")
                        )
                        (progn
                          (setq BonSens2 nil  VxList2 (remove_n (what_nth pVEn2 VxList2) VxList2))
                          (cassure "dernier:BonSens2 pVEn2 VxList2")
                        )
                      )
                      (if (= BonSens1 nil)
                        (setq VxList1 (reverse VxList1))
                      )
                      (if (= BonSens2 nil)
                        (setq VxList2 (reverse VxList2))
                      )
                      (plinemake (append VxList1 VxList2) 0 0 T)          ;T for LW
                      (entdel eObject1)(entdel eObject2)
                    )
                  )
                  (setq iCount (1+ iCount))
                )
              )
            )
            (if (= sPorteOuiNon "DOORPL")
              (progn
                (setq ssToKill (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "DOOREL")) 1 "AA_OPENING"))
                (foreach item ssToKill
                  (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan2)
                       (= (cdr (nth 4 (AAXDataGet (car item) sAppNameAA))) sTofIns))      ;le 4eme champ des ELEV
                    (entdel (car item))
                  )
                )
              )
            )
            (if (= sPorteOuiNon "WINDPL")
              (progn
                (setq
                  ssToKill2 (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "WINDEL")) 1 "AA_OPENING")
                )
                (foreach item ssToKill2
                  (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan2)
                           (= (cdr (nth 8 (AAXDataGet (car item) sAppNameAA))) sTofIns)
                      ) ;le 8eme champ des ELEV
                    (entdel (car item))
                  )
                )
                (setq
                  ssToKill2 (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "WINDSC")) 1 "AA_OPENING")
                )
                (foreach item ssToKill2
                  (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan2)
                           (= (cdr (nth 6 (AAXDataGet (car item) sAppNameAA))) sTofIns)
                      )      ;le 6eme champ des SECT
                    (entdel (car item))
                  )
                )
                (setq
                  ssToKill2 (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "WINDBR")) 1 "AA_OPENING")
                )
                (foreach item ssToKill2
                  (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan2)
                           (= (cdr (nth 2 (AAXDataGet (car item) sAppNameAA))) sTofIns)
                      )      ;le 2eme champ des Fed Bars
                    (entdel (car item))
                  )
                )
                (setq
                  ssToKill2 (SSgetXD (list (cons 0 "INSERT")) (quote (1 "=" "WALLSC")) 1 "AA_OPENING")
                )
                (foreach item ssToKill2
                  (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan2)
                           (= (cdr (nth 5 (AAXDataGet (car item) sAppNameAA))) sTofIns)
                      )      ;le 5eme champ des SECT mur
                    (entdel (car item))
                  )
                )
              )
            )
          )
          (princ "\nSelect an ArchAssist door/window in plan. ")
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;-----------------------
      ;---Assisted Join Walls.
      ;-----------------------
      (defun C:J ()  (WJoin))
      (defun C:EO () (NonAss))

      (defun LastVtx (e / old)
        (if (= (cdr (assoc 0 (entget e))) "POLYLINE")     ;check for polyline
          (progn                      ;if pline loop until SEQEND reached
            (while (/= (cdr (assoc 0 (entget e))) "SEQEND")
              (setq old e            ;keep previous VERTEX
                    e   (entnext e)  ;get next VERTEX
              )
            )
            (cdr (assoc 10 (entget old)))         ;get last VERTEX point
          )
          (progn
            (prompt "\nSelected object not a POLYLINE.") ;if not pline tell me
            nil
          )
        )
      )

      ; ll and ur are the diagonal defining points to 2d (x,y) rectangle
      ; pt is to be checked if inside that rectangle T is returned if inside; nil is returned if outside.
      (defun PtInRect (pt ll ur / llx urx lly ury ptx pty)
        (setq llx (car ll)
              urx (car ur)
              lly (cadr ll)
              ury (cadr ur)
              ptx (car pt)
              pty (cadr pt)
        )
        (cond
          ((and (> ptx llx) (< ptx urx) (> pty lly) (< pty ury)) T)
          (T nil)
        )
      )

      (defun WJoin (/)
        (setq  olderror    *error*
              *error*     Wallserror
        )
        (Wallsgather_info)
        (princ (strcat "\nRejoins Broken Segments." copyright))
        (setvar "osmode" 0)                           ;toggle off and none
        (setq pC1 (upoint 8 "" "Select wall objects to join" nil nil)
              pC2 (GetCorner pC1 "\nSpecify opposite corner:")
        )
        (WPatching pC1 pC2)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun WPatching (pC1 pC2 / ssGardLin1 ssGardLin2 ssGardLin3 ssGardLin4 ssGardArc1 ssGardArc2 ssGardArc3
                                  ssGardArc4 ssGardPln1 ssGardPln2 ssGardPln3 ssGardPln4 ssGardLwp1 ssGardLwp2 ssGardLwp3 ssGardLwp
                       )
        (setq LaListe (CornersGet pC1 pC2)
              pC1     (car LaListe)
              pC2     (cadr LaListe)
              pBL     pC1
              pTR     pC2
              ssBorar (ssget "_W" pC1 pC2)
              pC1a    (list (car pC1) (cadr pC2) 0.0)
              pC2a    (list (car pC2) (cadr pC1) 0.0)
              bCont   nil
              bACont  nil
              bPCont  nil
              bLwCont nil
              eObject1 nil
              eObject2 nil
              lObj1   nil
              lObj2   nil
              BonSens1 nil
              BonSens2 nil
              VxList1 nil
              VxList2 nil
              iCount  0
        )
        (command "_.erase" ssBorar "")
        (setq ssGardArc1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "ARC")))
              ssGardArc2 (ssget "_F" (list pC1 pC2a) (list (cons 0 "ARC")))
              ssGardArc3 (ssget "_F" (list pC2a pC2) (list (cons 0 "ARC")))
              ssGardArc4 (ssget "_F" (list pC1a pC2) (list (cons 0 "ARC")))
              ssGardLin1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "LINE")))
              ssGardLin2 (ssget "_F" (list pC1 pC2a) (list (cons 0 "LINE")))
              ssGardLin3 (ssget "_F" (list pC2a pC2) (list (cons 0 "LINE")))
              ssGardLin4 (ssget "_F" (list pC1a pC2) (list (cons 0 "LINE")))
              ssGardPln1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "POLYLINE")))
              ssGardPln2 (ssget "_F" (list pC1 pC2a) (list (cons 0 "POLYLINE")))
              ssGardPln3 (ssget "_F" (list pC2a pC2) (list (cons 0 "POLYLINE")))
              ssGardPln4 (ssget "_F" (list pC1a pC2) (list (cons 0 "POLYLINE")))
              ssGardLwP1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "LWPOLYLINE")))
              ssGardLwP2 (ssget "_F" (list pC1 pC2a) (list (cons 0 "LWPOLYLINE")))
              ssGardLwP3 (ssget "_F" (list pC2a pC2) (list (cons 0 "LWPOLYLINE")))
              ssGardLwP4 (ssget "_F" (list pC1a pC2) (list (cons 0 "LWPOLYLINE")))
        )

        (if (/= ssGardLin1 nil)           ;Pour les lignes
          (setq bCont    T
                iNum4G1  (sslength ssGardLin1)
                iNum     iNum4G1
                ss1Garde ssGardLin1
          )
        )
        (if (/= ssGardArc1 nil)           ;Pour les arcs
          (setq bACont   T
                iNum4G1  (sslength ssGardArc1)
                iNumA    iNum4G1
                ss1Guard ssGardArc1
          )
        )
        (if (/= ssGardPln1 nil)           ;Pour les polylignes
          (setq bPCont   T
                iNum4G1  (sslength ssGardPln1)
                iNumP    iNum4G1
                ss1GardP ssGardPln1
          )
        )
        (if (/= ssGardLwp1 nil)           ;Pour les polylignes legeres
          (setq bLwCont   T
                iNum4G1   (sslength ssGardLwp1)
                iNumLw    iNum4G1
                ss1GardLw ssGardLwp1
          )
        )

        (if (/= ssGardLin2 nil)
          (setq bCont    T
                iNum4G2  (sslength ssGardLin2)
                iNum     iNum4G2
                ss1Garde ssGardLin2
          )
        )
        (if (/= ssGardArc2 nil)           ;Pour les arcs
          (setq bACont   T
                iNum4G2  (sslength ssGardArc2)
                iNumA    iNum4G2
                ss1Guard ssGardArc2
          )
        )
        (if (/= ssGardPln2 nil)           ;Pour les polylignes
          (setq bPCont   T
                iNum4G2  (sslength ssGardPln2)
                iNumP    iNum4G2
                ss1GardP ssGardPln2
          )
        )
        (if (/= ssGardLwp2 nil)           ;Pour les polylignes legeres
          (setq bLwCont   T
                iNum4G2   (sslength ssGardLwp2)
                iNumLw    iNum4G2
                ss1GardLw ssGardLwp2
          )
        )

        (if (/= ssGardLin3 nil)
          (setq bCont    T
               iNum4G3  (sslength ssGardLin3)
               iNum     iNum4G3
               ss2Garde ssGardLin3
          )
        )
        (if (/= ssGardArc3 nil)           ;Pour les arcs
          (setq bACont   T
                iNum4G3  (sslength ssGardArc3)
                iNumA    iNum4G3
                ss2Guard ssGardArc3
          )
          (setq bACont nil)
        )
        (if (/= ssGardPln3 nil)           ;Pour les polylignes
          (setq bPCont   T
                iNum4G3  (sslength ssGardPln3)
                iNumP    iNum4G3
                ss2GardP ssGardPln3
          )
        )
        (if (/= ssGardLwp3 nil)           ;Pour les polylignes legeres
          (setq bLwCont   T
                iNum4G3   (sslength ssGardLwp3)
                iNumLw    iNum4G3
                ss2GardLw ssGardLwp3
          )
        )

        (if (/= ssGardLin4 nil)
          (setq bCont    T
                iNum4G4  (sslength ssGardLin4)
                iNum     iNum4G4
                ss2Garde ssGardLin4
          )
        )
        (if (/= ssGardArc4 nil)           ;Pour les arcs
          (setq bACont   T
                iNum4G4  (sslength ssGardArc4)
                iNumA    iNum4G4
                ss2Guard ssGardArc4
          )
        )
        (if (/= ssGardPln4 nil)           ;Pour les polylignes
          (setq bPCont   T
                iNum4G4  (sslength ssGardPln4)
                iNumP    iNum4G4
                ss2GardP ssGardPln4
          )
        )
        (if (/= ssGardLwp4 nil)           ;Pour les polylignes legeres
          (setq bLwCont   T
                iNum4G4   (sslength ssGardLwp4)
                iNumLw    iNum4G4
                ss2GardLw ssGardLwp4
          )
        )

        (if (equal iNum4G1 iNum4G3)
          (setq bCont1   T
                bACont1  T
                bPCont1  T
                bLwCont1 T
                bCont3   T
                bACont3  T
                bPCont3  T
                bLwCont3 T
          )
          (progn (princ "\rError: Walls with unequal number of objects.")
            (setq bCont1   nil
                  bACont1  nil
                  bPCont1  nil
                  bLwCont1 nil
                  bCont3   nil
                  bACont3  nil
                  bPCont3  nil
                  bLwCont3 nil
            )
          )
        )
        (if (equal iNum4G2 iNum4G4)
          (setq bCont2   T
                bACont2  T
                bPCont2  T
                bLwCont2 T
                bCont4   T
                bACont4  T
                bPCont4  T
                bLwCont4 T
          )
          (progn
            (princ "\rError: Walls with unequal number of objects.")
            (setq bCont2   nil
                  bACont2  nil
                  bPCont2  nil
                  bLwCont2 nil
                  bCont4   nil
                  bACont4  nil
                  bPCont4  nil
                  bLwCont4 nil
            )
          )
        )
        (if (and (= bCont T) (or (= bCont1 T)(= bCont2 T)(= bCont3 T)(= bCont4 T)) )
          (progn
            (While (< iCount iNum)
              (setq eObject1 (ssname ss1Garde iCount)
                    eObject2 (ssname ss2Garde iCount)
                    lObj1    (entget eObject1)
                    lObj2    (entget eObject2)
                    pStLn1   (dxf 10 lObj1)      ;depart ligne 1
                    pStLn2   (dxf 10 lObj2)      ;depart ligne 2
                    pEnLn2   (dxf 11 lObj2)      ;fin ligne 2
              )
              (if (= (Colineaire pStLn2 pStLn1 pEnLn2 nil 5) T)
                (progn (entdel eObject2)
                  (setvar "ORTHOMODE" 0)
                  (if (> (distance pStLn1 pStLn2) (distance pStLn1 pEnLn2))
                     (command "_.change" eObject1 "" (trans pStLn2 0 1))
                     (command "_.change" eObject1 "" (trans pEnLn2 0 1))
                  )
                )
                (princ "\rError: Non-colinear walls.")
              )
              (setq iCount (1+ iCount))
            )
          )
        )
        (if (and (= bACont T) (or (= bACont1 T)(= bACont2 T)(= bACont3 T)(= bACont4 T)) )
          (progn
            (While (< iCount iNumA)
              (setq eObject1 (ssname ss1Guard iCount)
                    eObject2 (ssname ss2Guard iCount)
                    lObj1    (entget eObject1)
                    lObj2    (entget eObject2)
                    pCen1    (dxf 10 lObj1)
                    pCen2    (dxf 10 lObj2)
                    sLtyp    (dxf 6 lObj1)
                    sLyr     (dxf 8 lObj1)
                    iClr     (dxf 62 lObj1)
              )
              (if (equal (distance pCen1 pCen2) 0.0 0.001)
                (progn
                  (setq lpt1 (polar pCen1 (dxf 50 lObj1) (dxf 40 lObj1))
                        lpt2 (polar pCen1 (dxf 51 lObj1) (dxf 40 lObj1))
                        lpt3 (polar pCen2 (dxf 50 lObj2) (dxf 40 lObj2))
                        lpt4 (polar pCen2 (dxf 51 lObj2) (dxf 40 lObj2))
                         ;;thick (dxf 39 lObj1)
                  )
                  (if (< (distance lpt1 lpt4) (distance lpt2 lpt3))
                    (command "_.arc" lpt3 lpt1 lpt2)
                    (command "_.arc" lpt1 lpt2 lpt4)
                  )
                  (entdel eObject1)(entdel eObject2)
                  (if (= iClr nil)
                    (setq iClr "BYLAYER")
                  )
                  (if (= sLtyp nil)
                    (setq sLtyp "BYLAYER")
                  )
                  (vla-put-layer (vlax-Ename->vla-Object (entlast)) sLyr)
                  (vla-put-color (vlax-Ename->vla-Object (entlast)) iClr)
                  (vla-put-linetype (vlax-Ename->vla-Object (entlast)) sLtyp)
                )
                (progn
                  (if eObject1
                    (redraw eObject1)
                  )
                  (prompt "\nArcs centres must coincide. ")
                )
              )
              (setq iCount (1+ iCount))
            )
          )
        )
        (if (and (= bPCont T) (or (= bPCont1 T)(= bPCont2 T)(= bPCont3 T)(= bPCont4 T)) )
          (progn
            (While (< iCount iNumP)
              (setq eObject1 (ssname ss1GardP iCount)
                    eObject2 (ssname ss2GardP iCount)
                    lObj1    (entget eObject1)
                    lObj2    (entget eObject2)
                    sLyr     (dxf 8 lObj1)
                    VxList1  (Liste4Pline eObject1)
                    VxList2  (Liste4Pline eObject2)
              )
              (Hshset_lay sLyr nil nil nil)
              (if (equal eObject1 eObject2)                  ;si ce sont 2 differentes plines
                (progn                           ;sinon
                  (setq VxList1 (remove_n (what_nth (first_Vtx eObject1) VxList1) VxList1))
                  (setq VxList1 (remove_n (what_nth (lastVtx eObject1) VxList1) VxList1))
                  ;;;(plinemake VxList1 0 0 nil)
                  (command "_.PLINE")(foreach pnt VxList1 (command pnt))(command "_C")
                  (entdel eObject1)
                )
                (progn
                  (if (= T (PtInRect (first_Vtx eObject1) pBL pTR))
                    (progn
                      (setq BonSens1 nil  VxList1 (remove_n (what_nth (first_Vtx eObject1) VxList1) VxList1))
                      (cassure "BonSens1 (first_Vtx eObject1) VxList1")
                    )
                    (progn
                      (setq BonSens1 T    VxList1 (remove_n (what_nth (lastVtx eObject1) VxList1) VxList1))
                      (cassure "BonSens1 (lastVtx eObject1) VxList1")
                    )
                  )
                  (if (= T (PtInRect (first_Vtx eObject2) pBL pTR))
                    (setq BonSens2 T    VxList2 (remove_n (what_nth (first_Vtx eObject2) VxList2) VxList2))
                    (setq BonSens2 nil  VxList2 (remove_n (what_nth (lastVtx eObject2) VxList2) VxList2))
                  )
                  (if (= BonSens1 nil)
                    (setq VxList1 (reverse VxList1))
                  )
                  (if (= BonSens2 nil)
                    (setq VxList2 (reverse VxList2))
                  )
                  (if (and (= BonSens1 nil)(= BonSens2 T))
                    (plinemake (append VxList2 VxList1) 0 0 nil)
                    (plinemake (append VxList1 VxList2) 0 0 nil)
                  )
                  (entdel eObject1)(entdel eObject2)
                )
              )
              (setq iCount (1+ iCount))
            )
          )
        )
        (if (and (= bLwCont T) (or (= bLwCont1 T)(= bLwCont2 T)(= bLwCont3 T)(= bLwCont4 T)) )
          (progn
            (While (< iCount iNumLw)
              (setq eObject1 (ssname ss1GardLw iCount)
                    eObject2 (ssname ss2GardLw iCount)
                    lObj1    (entget eObject1)
                    lObj2    (entget eObject2)
                    sLyr     (dxf 8 lObj1)
                    pVSt1    (dxf 10 lObj1)
                    pVSt1    (list (car pVSt1)(cadr pVSt1) 0.0)
                    pVEn1    (dxf 10 (reverse lObj1))
                    pVEn1    (list (car pVEn1)(cadr pVEn1) 0.0)
                    pVSt2    (dxf 10 lObj2)
                    pVSt2    (list (car pVSt2)(cadr pVSt2) 0.0)
                    pVEn2    (dxf 10 (reverse lObj2))
                    pVEn2    (list (car pVEn2)(cadr pVEn2) 0.0)
                    VxList1  (Liste4LwPl eObject1)
                    VxList2  (Liste4LwPl eObject2)
              )
              (Hshset_lay sLyr nil nil nil)
              (if (equal eObject1 eObject2)                  ;si ce sont 2 differentes plines
                (progn                           ;sinon
                  (setq VxList1 (remove_n (what_nth pVSt1 VxList1) VxList1))
                  (setq VxList1 (remove_n (what_nth pVEn1 VxList1) VxList1))
                  (command "_.PLINE")(foreach pnt VxList1 (command pnt))(command "_C")
                  (entdel eObject1)
                )
                (progn                            ;si ce sont 2 differentes plines
                  (if (= T (PtInRect pVSt1 pBL pTR))
                    (progn
                      (setq BonSens1 nil  VxList1 (remove_n (what_nth pVSt1 VxList1) VxList1))
                      (cassure "1er:BonSens1 pVSt1 VxList1")
                    )
                    (progn
                      (setq BonSens1 T    VxList1 (remove_n (what_nth pVEn1 VxList1) VxList1))
                      (cassure "dernier:BonSens1 pVEn1 VxList1")
                    )
                  )
                  (if (= T (PtInRect pVSt2 pBL pTR))
                    (progn
                      (setq BonSens2 T    VxList2 (remove_n (what_nth pVSt2 VxList2) VxList2))
                      (cassure "1er:BonSens2 pVSt2 VxList2")
                    )
                    (progn
                      (setq BonSens2 nil  VxList2 (remove_n (what_nth pVEn2 VxList2) VxList2))
                      (cassure "dernier:BonSens2 pVEn2 VxList2")
                    )
                  )
                  (if (= BonSens1 nil)
                    (setq VxList1 (reverse VxList1))
                  )
                  (if (= BonSens2 nil)
                    (setq VxList2 (reverse VxList2))
                  )
                  (if (and (= BonSens1 nil)(= BonSens2 T))
                    (plinemake (append VxList2 VxList1) 0 0 T)          ;T for LW
                    (plinemake (append VxList1 VxList2) 0 0 T)          ;T for LW
                  )
                  (entdel eObject1)(entdel eObject2)
                )
              )
              (setq iCount (1+ iCount))
            )
          )
        )
      )

      ;-------------------
      ;---Types de portes.
      ;-------------------
      (defun HeaderMk ()
        (TraceIsOn ".HeaderMk")
        (if (= bHeader T)
          (progn
            (entmake (list (cons 0 "LINE")
                           (cons 100 "AcDbEntity")
                           (cons 67 0)
                           (cons 8 sHeaderLyr)
                           (cons 100 "AcDbLine")
                           (list 10 0.0 0.0 0.0)
                           (list 11 OFFDIST 0.0 0.0)
                           (list 210 0.0 0.0 1.0)
                     )
            )
            (entmake (list (cons 0 "LINE")
                           (cons 100 "AcDbEntity")
                           (cons 67 0)
                           (cons 8 sHeaderLyr)
                           (cons 100 "AcDbLine")
                           (list 10 0.0 rMurThk 0.0)
                           (list 11 OFFDIST rMurThk 0.0)
                           (list 210 0.0 0.0 1.0)
                     )
            )
          )
        )
      )

      (defun ThresholdMk ()
        (TraceIsOn ".ThresholdMk")
        (if (= bMetUnit T)                            ;Section metrique
          (progn
            (setq rFullH      (+ 20.0 rMurThk)
                  rCorrected  (- rFullH 230.0)
            )
          )
        )
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (entmake (list (cons 0 "LWPOLYLINE")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sThreshLyr)
                       (cons 100 "AcDbPolyline")
                       (cons 90 4)
                       (cons 70 1)
                       (cons 43 0.0)
                       (cons 38 0.0)
                       (cons 39 0.0)
                       (list 10 0.0 rCorrected)
                       (cons 40 0.0)
                       (cons 41 0.0)
                       (cons 42 0.0)
                       (list 10 OFFDIST rCorrected)
                       (cons 40 0.0)
                       (cons 41 0.0)
                       (cons 42 0.0)
                       (list 10 OFFDIST rFullH)
                       (cons 40 0.0)
                       (cons 41 0.0)
                       (cons 42 0.0)
                       (list 10 0.0 rFullH)
                       (cons 40 0.0)
                       (cons 41 0.0)
                       (cons 42 0.0)
                       (list 210 0.0 0.0 1.0)
                 )
        )
        (setq BlockNameD2 (entmake '((0 . "ENDBLK"))))
      )

      (defun CavSlidDMk (/)           ;Cavity double
        (TraceIsOn ".CavSlidDMk")
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
              rEpaiss    30
        )
        ;;(setq rAdjust (* (- rMurThk 30.0) 0.5))
        (setq PERCDIST (* (/ sOpenAng_R PI) (/ OFFDIST 2)))
        (setq STARTYPT (- rMurThk (+ (/ rMurThk 2) (/ LEAFWIDT_R 2))))
        (entmake
          '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0))
        )
        (HeaderMk)
        ;;fleche gauche
        (entmake (list (cons 0 "POLYLINE")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDb2dPolyline")
                       (cons 66 1)
                       (list 10 0.0 0.0 0.0)
                       (list 210 0.0 0.0 1.0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDbVertex")
                       (cons 100 "AcDb2dVertex")
                       (list 10 32 -135 0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDbVertex")
                       (cons 100 "AcDb2dVertex")
                       (list 10 75 -110 0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDbVertex")
                       (cons 100 "AcDb2dVertex")
                       (list 10  -75 -110 0)
                 )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        ;;La porte gauche
        (entmake (list (cons 0 "POLYLINE")
                       (cons 8 sLeafLyr)
                       (cons 66 1)
                       (list 10 0.0 0.0 0.0)
                       (cons 70 1)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 8 sLeafLyr)
                       (list 10 (- 0.0 PERCDIST) STARTYPT 0.0)
                 )
        )
        (entmake
          (list (cons 0 "VERTEX")
                (cons 8 sLeafLyr)
                (list 10 (- 0.0 PERCDIST) (+ STARTYPT LEAFWIDT_R) 0.0)
          )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 8 sLeafLyr)
                       (list 10
                             (- (/ OFFDIST 2) PERCDIST)
                             (+ STARTYPT LEAFWIDT_R)
                             0.0
                       )
                 )
        )
        (entmake
          (list (cons 0 "VERTEX")
                (cons 8 sLeafLyr)
                (list 10 (- (/ OFFDIST 2) PERCDIST) STARTYPT 0.0)
          )
        )
        (entmake '((0 . "SEQEND")))

        ;;La porte droite
        (entmake (list (cons 0 "POLYLINE")
                       (cons 8 sLeafLyr)
                       (cons 66 1)
                       (list 10 0.0 0.0 0.0)
                       (cons 70 1)
                 )
        )
        (entmake
          (list (cons 0 "VERTEX")
                (cons 8 sLeafLyr)
                (list 10 (+ (/ OFFDIST 2) PERCDIST) STARTYPT 0.0)
          )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 8 sLeafLyr)
                       (list 10
                             (+ (/ OFFDIST 2) PERCDIST)
                             (+ STARTYPT LEAFWIDT_R)
                             0.0
                       )
                 )
        )
        (entmake
          (list (cons 0 "VERTEX")
                (cons 8 sLeafLyr)
                (list 10 (+ OFFDIST PERCDIST) (+ STARTYPT LEAFWIDT_R) 0.0)
          )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 8 sLeafLyr)
                       (list 10 (+ OFFDIST PERCDIST) STARTYPT 0.0)
                 )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        ;;fleche droite
        (entmake (list (cons 0 "POLYLINE")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDb2dPolyline")
                       (cons 66 1)
                       (list 10 0.0 0.0 0.0)
                       (list 210 0.0 0.0 1.0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDbVertex")
                       (cons 100 "AcDb2dVertex")
                       (list 10 (- OFFDIST 32) -135 0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDbVertex")
                       (cons 100 "AcDb2dVertex")
                       (list 10 (- OFFDIST 75) -110 0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDbVertex")
                       (cons 100 "AcDb2dVertex")
                       (list 10 (+ OFFDIST 75) -110 0)
                 )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun CavSlidSMk (/)                           ;Cavity single
        (TraceIsOn ".CavSlidSMk")
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
              rEpaiss    30
        )
        (setq rAdjust (* (- rMurThk 30.0) 0.5))
        (setq PERCDIST (* (/ sOpenAng_R PI) OFFDIST))
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        ;;fleche
        (entmake (list (cons 0 "POLYLINE")
                       (cons 100 "AcDbEntity")
                       (cons 67 0)
                       (cons 8 sLeafLyr)
                       (cons 100 "AcDb2dPolyline")
                       (cons 66 1)
                       (list 10 0.0 0.0 0.0)
                       (list 210 0.0 0.0 1.0)
                 )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 32 -135 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 75 -110 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10  -75 -110 0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        ;;La porte
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- 0.0 PERCDIST) rAdjust 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- 0.0 PERCDIST) (+ rAdjust 30.0) 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST PERCDIST) (+ rAdjust 30.0) 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST PERCDIST) rAdjust 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun SurfSlidDMk (/)          ;Surface double
        (TraceIsOn ".SurfSlidDMk")
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
          rEpaiss    30
        )
        ;;(setq rAdjust (* (- rMurThk 30.0) 0.5))
        (setq PERCDIST (* (/ sOpenAng_R PI) (/ OFFDIST 2)))
        (setq STARTYPT -60)
        (entmake
          '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0))
        )
        (HeaderMk)
        ;;fleche gauche
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 32 -135 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 75 -110 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10  -75 -110 0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        ;;La porte gauche
        (entmake (list (cons 0 "POLYLINE")
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 8 sLeafLyr)
               (list 10 (- 0.0 PERCDIST) STARTYPT 0.0)
             )
        )
        (entmake
          (list (cons 0 "VERTEX")
            (cons 8 sLeafLyr)
            (list 10 (- 0.0 PERCDIST) (+ STARTYPT LEAFWIDT_R) 0.0)
          )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 8 sLeafLyr)
               (list 10
                     (- (/ OFFDIST 2) PERCDIST)
                     (+ STARTYPT LEAFWIDT_R)
                     0.0
               )
             )
        )
        (entmake
          (list (cons 0 "VERTEX")
            (cons 8 sLeafLyr)
            (list 10 (- (/ OFFDIST 2) PERCDIST) STARTYPT 0.0)
          )
        )
        (entmake '((0 . "SEQEND")))

        ;;La porte droite
        (entmake (list (cons 0 "POLYLINE")
               (cons 8 sLeafLyr)
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
             )
        )
        (entmake
          (list (cons 0 "VERTEX")
            (cons 8 sLeafLyr)
            (list 10 (+ (/ OFFDIST 2) PERCDIST) STARTYPT 0.0)
          )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 8 sLeafLyr)
               (list 10
                     (+ (/ OFFDIST 2) PERCDIST)
                     (+ STARTYPT LEAFWIDT_R)
                     0.0
               )
             )
        )
        (entmake
          (list (cons 0 "VERTEX")
            (cons 8 sLeafLyr)
            (list 10 (+ OFFDIST PERCDIST) (+ STARTYPT LEAFWIDT_R) 0.0)
          )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 8 sLeafLyr)
               (list 10 (+ OFFDIST PERCDIST) STARTYPT 0.0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        ;;fleche droite
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST 32) -135 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST 75) -110 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (+ OFFDIST 75) -110 0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun SurfSlidSMk (/)                          ;Surface single
        (TraceIsOn ".SurfSlidSMk")
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
              rEpaiss    30
        )
        (setq PERCDIST (* (/ sOpenAng_R PI) OFFDIST))
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        ;;fleche
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 32 -135 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 75 -110 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10  -75 -110 0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        ;;La porte
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- 0.0 PERCDIST) -60.0 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- 0.0 PERCDIST) -30.0 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST PERCDIST) -30.0 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST PERCDIST) -60.0 0.0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (cons 70 0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun UnDblMk ()                           ;Unequal double
        (TraceIsOn ".UnDblMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (if (= sJambTyp "T")
          (TimberJamb)
          (SteelJamb)
        )
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
          rEpaiss    30
        )
        (setq OFFDISTR OFFDIST)
        (setq DOORANG1 (atan (/ LEAFWIDT_R (- (* (/ OFFDISTR 3) 2) (* 7 rUnit1)))))
        (setq DOORANG2 (atan (/ LEAFWIDT_R (- (/ OFFDISTR 3) (* 7 rUnit1)))))

        (if (> DOORANG1 sOpenAng_R)
          (setq DOORANG1 sOpenAng_R)
        )
        (if (> DOORANG2 sOpenAng_R)
          (setq DOORANG2 sOpenAng_R)
        )
        (setq pAvLeft (polar pArRight (- (* PI 2) sOpenAng_R) (- (* (/ OFFDISTR 3.0) 2.0) (* 7 rUnit1))))
        (setq pAvRight (polar pAvLeft (+ (- (* PI 2) sOpenAng_R) (/ PI 2)) LEAFWIDT_R))
        (setq JpArRight (polar pAvRight (+ PI (- (* PI 2) sOpenAng_R)) (- (* (/ OFFDISTR 3.0) 2.0) (* 7 rUnit1))))
        (entmake (list (cons 0 "ARC")
               (cons 8 sSwingLyr)
               (cons 10 pArRight)
               (cons 40 (- (* (/ OFFDISTR 3.0) 2.0) (* 7 rUnit1)))
               (cons 50 (+ (- (* PI 2) sOpenAng_R) DOORANG1))
               (cons 51 0.0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "POLYLINE") (cons 8 sLeafLyr) (cons 66 1) (list 10 0.0 0.0 0.0) (cons 70 1)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 pArRight)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 pAvLeft)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 pAvRight)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 JpArRight)))
        (entmake '((0 . "SEQEND")))
        (setq JJRP2 (polar JRP4 (- PI (- (* PI 2) sOpenAng_R)) (- (/ OFFDISTR 3) (* 7 rUnit1))))
        (setq JJRP3 (polar JJRP2 (- (- PI (- (* PI 2) sOpenAng_R)) (/ PI 2)) LEAFWIDT_R))
        (setq JJRP4 (polar JJRP3 (+ (- PI (- (* PI 2) sOpenAng_R)) PI) (- (/ OFFDISTR 3) (* 7 rUnit1))))
        (entmake (list (cons 0 "ARC")
               (cons 8 sSwingLyr)
               (cons 10 JRP4)
               (cons 40 (- (/ OFFDISTR 3.0) (* 7 rUnit1)))
               (cons 50 PI)
               (cons 51 (- (- PI (- (* PI 2) sOpenAng_R)) DOORANG2))
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "POLYLINE") (cons 8 sLeafLyr) (cons 66 1) (list 10 0.0 0.0 0.0) (cons 70 1))
        )
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 JRP4)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 JJRP2)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 JJRP3)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 JJRP4)))
        (entmake '((0 . "SEQEND")))
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun BifoldSMk (/)                        ;Bifold single
        (TraceIsOn ".BifoldSMk")
        (setq OFFDISTR OFFDIST)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
             rEpaiss    30
        )
        (setq sOpenAng_H (* sOpenAng_R 0.5))
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (if (= sJambTyp "T")
          (TimberJamb)
          (SteelJamb)
        )
        (setq PTA (polar pArRight (- (* PI 2) sOpenAng_H) (/ (- OFFDISTR (* 14 rUnit1)) 2)))
        (setq PTB (polar PTA (- (- (* PI 2) sOpenAng_H) (* PI 1.5)) LEAFWIDT_R))
        (setq PTC (polar PTB (+ (- (* PI 2) sOpenAng_H) PI) (/ (- OFFDISTR (* 14 rUnit1)) 2)))
        (entmake (list (cons 0 "POLYLINE") (cons 8 sLeafLyr) (cons 66 1) (list 10 0.0 0.0 0.0) (cons 70 1))
        )
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 pArRight)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTA)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTB)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTC)))
        (entmake '((0 . "SEQEND")))
        (setq PTD (polar PTB (+ (- PI (- (* PI 2) sOpenAng_H)) PI) (/ (- OFFDISTR (* 14 rUnit1)) 2)))
        (setq PTE (polar PTD (- (- PI (- (* PI 2) sOpenAng_H)) (* PI 1.5)) LEAFWIDT_R))
        (setq PTF (polar PTE (- PI (- (* PI 2) sOpenAng_H)) (/ (- OFFDISTR (* 14 rUnit1)) 2)))
        (entmake (list (cons 0 "POLYLINE") (cons 8 sLeafLyr) (cons 66 1) (list 10 0.0 0.0 0.0) (cons 70 1)) )
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTB)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTD)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTE)))
        (entmake (list (cons 0 "VERTEX") (cons 8 sLeafLyr) (cons 10 PTF)))
        (entmake '((0 . "SEQEND")))
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun BifoldDMk (/)                        ;Bifold double
        (TraceIsOn ".BifoldSMk")
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
              rEpaiss    30
        )
        (setq sOpenAng_H (* sOpenAng_R 0.5))
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (if (= sJambTyp "T")
          (TimberJamb)
          (SteelJamb)
        )

        (setq PTA (polar pArRight (- (* PI 2) sOpenAng_H) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (setq PTB (polar PTA (- (- (* PI 2) sOpenAng_H) (* PI 1.5)) LEAFWIDT_R))
        (setq PTC (polar PTB (+ (- (* PI 2) sOpenAng_H) PI) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        ;;2 panneaux gauche
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pArRight)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTA)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTB)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTC)
             )
        )
        (entmake '((0 . "SEQEND")))

        (setq PTD (polar PTB (+ (- PI (- (* PI 2) sOpenAng_H)) PI) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (setq PTE (polar PTD (- (- PI (- (* PI 2) sOpenAng_H)) (* PI 1.5)) LEAFWIDT_R))
        (setq PTF (polar PTE (- PI (- (* PI 2) sOpenAng_H)) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTB)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTD)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTE)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTF)
             )
        )
        (entmake '((0 . "SEQEND")))

        ;;2 panneaux droite
        (setq JJRP2 (polar JRP4 (+ (* PI 1) sOpenAng_H) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (setq JJRP3 (polar JJRP2 (+ (* PI 0.5) sOpenAng_H) LEAFWIDT_R))
        (setq JJRP4 (polar JJRP3 sOpenAng_H (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP4)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP2)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP3)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP4)
             )
        )
        (entmake '((0 . "SEQEND")))

        (setq PTG (polar JJRP3 (- PI sOpenAng_H) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (setq PTH (polar PTG (+ (* pi 0.5)(- PI sOpenAng_H)) LEAFWIDT_R))
        (setq PTI (polar PTH (+ (+ pi (- PI sOpenAng_H))) (/ (- (* OFFDIST 0.5) (* 7 rUnit1)) 2)))
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP3)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTG)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTH)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 PTI)
             )
        )
        (entmake '((0 . "SEQEND")))

        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun DoubleMk (/)
        (TraceIsOn ".DoubleMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (if (= sJambTyp "T")
          (TimberJamb)
          (SteelJamb)
        )
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
              rEpaiss    30
        )
        (setq DOORANG (atan (/ LEAFWIDT_R (- (* OFFDIST 0.5) (* 7 rUnit1)))))
        (if (> DOORANG sOpenAng_R)
          (setq DOORANG sOpenAng_R)
        )
        (setq pAvLeft (polar pArRight (- (* PI 2) sOpenAng_R) (- (* OFFDIST 0.5) (* 7 rUnit1))))
        (setq pAvRight (polar pAvLeft (+ (- (* PI 2) sOpenAng_R) (/ PI 2)) LEAFWIDT_R))
        (setq JpArRight (polar pAvRight (+ PI (- (* PI 2) sOpenAng_R)) (- (* OFFDIST 0.5) (* 7 rUnit1))))
        (entmake (list (cons 0 "ARC")
               (cons 100 "AcDbEntity")
               (cons 67 0)
          ;;;; (cons 410 "Model")
               (cons 8 sSwingLyr)
               (cons 100 "AcDbCircle")
               (cons 10 pArRight)
               (cons 40 (- (* OFFDIST 0.5) (* 7 rUnit1)))
               (list 210 0.0 0.0 1.0)
               (cons 100 "AcDbArc")
               (cons 50 (+ (- (* PI 2) sOpenAng_R) DOORANG))
               (cons 51 0.0)
             )
        )
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pArRight)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pAvLeft)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pAvRight)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JpArRight)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        (setq JJRP2 (polar JRP4 (- PI (- (* PI 2) sOpenAng_R)) (- (* OFFDIST 0.5) (* 7 rUnit1))))
        (setq JJRP3 (polar JJRP2 (- (- PI (- (* PI 2) sOpenAng_R)) (/ PI 2)) LEAFWIDT_R))
        (setq JJRP4 (polar JJRP3 (+ (- PI (- (* PI 2) sOpenAng_R)) PI) (- (* OFFDIST 0.5) (* 7 rUnit1))))
        (entmake (list (cons 0 "ARC")
               (cons 100 "AcDbEntity")
               (cons 67 0)
         ;;;   (cons 410 "Model")
               (cons 8 sSwingLyr)
               (cons 100 "AcDbCircle")
               (cons 10 JRP4)
               (cons 40 (- (* OFFDIST 0.5) (* 7 rUnit1)))
               (list 210 0.0 0.0 1.0)
               (cons 100 "AcDbArc")
               (cons 50 PI)
               (cons 51 (- (- PI (- (* PI 2) sOpenAng_R)) DOORANG))
             )
        )
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP4)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP2)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP3)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JJRP4)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun OpenMk   (/)                         ;Doorway
        (TraceIsOn ".OpenMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun SingleMk (/)
        (TraceIsOn ".SingleMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (if (= sJambTyp "T")
          (TimberJamb)
          (SteelJamb)
        )
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 30.0
          rEpaiss    30)
        (setq DOORANG (atan (/ LEAFWIDT_R (- OFFDIST (* 14 rUnit1)))))
        (if (> DOORANG sOpenAng_R)
          (setq DOORANG sOpenAng_R)
        )
        (setq pAvLeft (polar pArRight (- (* PI 2) sOpenAng_R) (- OFFDIST (* 14 rUnit1))))
        (setq pAvRight (polar pAvLeft (+ (- (* PI 2) sOpenAng_R) (/ PI 2)) LEAFWIDT_R))   ;Arc s'arrete a l'avant de la porte
        (setq JpArRight (polar pAvRight (- PI sOpenAng_R) (- OFFDIST (* 14 rUnit1))))
        (entmake (list (cons 0 "ARC")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sSwingLyr)
               (cons 100 "AcDbCircle")
               (cons 10 pArRight)                 ;Centre
               (cons 40 (- OFFDIST (* 14 rUnit1)))            ;Rayon
               (list 210 0.0 0.0 1.0)
               (cons 100 "AcDbArc")
               (cons 50 (+ (- (* PI 2) sOpenAng_R) DOORANG))      ;Start angle
               (cons 51 0.0)
             )
        )
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pArRight)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pAvLeft)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pAvRight)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sLeafLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JpArRight)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun FixSldFixMk (/)                              ;Porte FSF
        (TraceIsOn ".FixSldFixMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)                              ;Leaf width
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
        ;;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;Le montant gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;Le montant gauche milieu bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash milieu
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre milieu
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ Offdist 3.0) (* 0.5 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 11 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit milieu bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;Le montant gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre droit
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (+ (* 2 (/ Offdist 3.0)) (* 0.5 rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
          (if (= bSill T)
            (progn (SillMk)
            )
          )
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun FixSldSldFixMk (/)                           ;Porte FSSF
        (TraceIsOn ".FixSldFixMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)                              ;Leaf width
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
        ;;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;Le montant gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre gauche haut
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;Le montant gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 2.0) (* 1 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 2.0) (* 1 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre gauche bas
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ Offdist 4.0) (* 0.5 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 11 (- (/ Offdist 2.0) (* 1 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ Offdist 2.0) (* 1 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ Offdist 2.0) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ Offdist 2.0) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ Offdist 2.0) (* 1 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;;---2eme set
        ;;;Le montant gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 2 (/ Offdist 4.0)) (* 1 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 (/ Offdist 4.0)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 (/ Offdist 4.0)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 4.0)) (* 1 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 4.0)) (* 1 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ Offdist 4.0)) (* 1 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre gauche bas
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 11 (+ (* 2 (/ Offdist 4.0)) (* 1 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

        ;;Le montant gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;Le Sash droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        ;;La vitre droit haut
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (+ (* 3 (/ Offdist 4.0)) (* 0.5 rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
        ;;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )

          (if (= bSill T)
            (progn (SillMk)
            )
          )
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      ;-----------------------------
      ;---Door/Window Manipulations.
      ;-----------------------------
      (defun c:DW ()  (setq bTab 0) (Ouverture) (princ))
      (defun c:DRE () (DrawElevs2)  (princ))
      (defun c:DRF () (DoorModUn)   (princ))
      (defun c:WK ()  (DoorModDeux) (princ))
      (defun c:DRM () (ReSwing)  (princ))
      (defun c:MAA () (GetSwing) (princ))
      (defun c:DRS () (DrawSecs) (princ))
      (defun c:WD ()  (setq bTab 1) (Ouverture) (princ))

      (defun DoorModUn (/ sPorteOuiNon)
        (TraceIsOn ".DoorModUn")
        (setq olderror *error*
              *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setvar "ATTDIA" 0)
        (WsFlashMess2
          (strcat "\nDoor/Window Flip." copyright)
          (strcat "\nDoor/Window Flip." copyright)
          (strcat "\nDoor/Window Flip." copyright)
          (strcat "\nDoor/Window Flip." copyright)
        )
        (command "_.UCS" "_W")
        (GetOpeningXD)
        (setq sTr4b nil)
        (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
          (progn
            (setq ssAjout (ssget "_F"
                     (list pMcote pCoteO)
                     (list (cons 0 "HATCH"))
                  )
              ssDoor  (ssadd)
              ssDoor  (ssadd ODrNam ssDoor)
            )
            (if (and (= sPorteOuiNon "WINDPL") (/= ssAjout nil))
          (setq ssDoor (ssadd (ssname ssAjout 0) ssDoor))
            )
            (setq OP1 (polar pHinge aA3 (* 0.5 OFFDIST))
              OP2 (polar pCoteO aA3 (* 0.5 OFFDIST))
              MP1 (polar pHinge aA1 (* 0.5 rMurThk))
              MP2 (polar pMCote aA1 (* 0.5 rMurThk))
            )
            (grdraw mp1 mp2 20 1)
            (grdraw op1 op2 150 0)
            (setq flip
               (ukword
                 6
                 "Hinge Swing hInge+swing Exit"
                 "Flip around: red axis for hinge, blue for swing. Change [Hinge/Swing/hInge+swing] "
                 "Exit"
               )
            )
            (if (or (= flip "Swing")
                (= flip "Hinge")
                (= flip "hInge+swing")
            )
          (progn
            (if (= flip "Swing")
              (progn
                (command "_.mirror" ssdoor "" op1 op2 "_y")
              )
            )
            (if (= flip "Hinge")
              (progn
                (command "_.mirror" ssdoor "" mp1 mp2 "_y")
              )
            )
            (if (= flip "hInge+swing")
              (progn
                (command "_.mirror" ssdoor "" op1 op2 "_y")
                (command "_.mirror" ssdoor "" mp1 mp2 "_y")
              )
            )
          )
            )
          )
          (princ "\nSelect an ArchAssist door/window drawn in plan view. ")
        )
        (command "_.UCS" "_P")
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun DoorModDeux ( / sPorteOuiNon )
        (TraceIsOn ".DoorModDeux")
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (WsFlashMess2 (strcat "\nCut Openings around Windows and Doors." copyright)
              (strcat "\nCut Openings around Windows and Doors." copyright)
              (strcat "\nCut Openings around Windows and Doors." copyright)
              (strcat "\nCut Openings around Windows and Doors." copyright)
        )
        (GetOpeningXD)

        (setq sTr4b nil)
        (if (or (= sPorteOuiNon "DOORPL")(= sPorteOuiNon "WINDPL"))
          (progn (setq pP4 (polar pMCote aA1 rMurThk))
             (setq e (nentselp pCoteO))
             (if (/= e nil)
               (setq lname (dxf 8 (entget (car (nentselp pCoteO)))))
               (setq lname "0")
             )

             (setq sTofIns (substr (rtos (getvar "cdate") 2 8) 5 12)) ;Heure, pour ajouter aux ouvertures.

             (if (= sPorteOuiNon "DOORPL")
               (SetDoorInfoTag sAppNameAA ODrNam 2)           ;2=Du moment que ce n'est pas nil
               (SetWindInfoTag sAppNameAA ODrNam 2)
             )

             (LineMake pHinge pCoteO lname)
             (setq oCutEdg1 (entlast))
             (SetJambInfoTag sAppNameAA oCutEdg1)

             (setq pMidCutEdg1 (getmid oCutEdg1))
             (LineMake pMCote pP4 lname)
             (setq oCutEdg2 (entlast))
             (SetJambInfoTag sAppNameAA oCutEdg2)

             (setq ssTrim1 (ssadd))

             (ssadd oCutEdg1 ssTrim1)
             (ssadd oCutEdg2 ssTrim1)

             (setq oCutEdg1Pts (getendpts oCutEdg1)
               oCutEdg2Pts (getendpts oCutEdg2)
                   midpts      (getmids oCutEdg1Pts oCutEdg2Pts)
                   trimpt1     (polar pHinge aA3 (* 0.5 OFFDIST))
                   trimpt2     (polar pCoteO aA3 (* 0.5 OFFDIST))
                   pTr1        (polar trimpt1 (+ Pi aA1) 2.0)
                   pTr2        (polar trimpt2 aA1 2.0)
          )
          (command "_.trim" ssTrim1 "" "_F" pTr1 pTr2 "" "")
          )
          (princ "\nSelect an ArchAssist door/window in plan. ")
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;;;--------Version Assistee avec DRE
      (defun DrawElevs2 (/ sPorteOuiNon)
        (TraceIsOn ".DrawElevs2")
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1))
          (progn
            (FlashMess3
          "ArchAssist maintenant Actif."
          "ArchAssist now Active."
          "ArchAssist now Active."
          "ArchAssist now Active."
            )
          )
        )
        (WsFlashMess2
          (strcat "\nDraw Elevation." copyright)
          (strcat "\nDraw Elevation." copyright)
          (strcat "\nDraw Elevation." copyright)
          (strcat "\nDraw Elevation." copyright)
        )
        (GetOpeningXD)

        (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
          (progn
            (if (= bMetUnit T)        ;Section metrique
          (progn
            (setq rEpaiss 50)
          )
            )
            (setq ssdoor (ssadd ODrNam))
            (setq ent1   (objsel "LINE or LWPOLYLINE" "Select a reference line" nil)
              rY2    (cadr (dxf 10 (entget (car ent1)))) ;get y value
              rXDif1 (BSide aA1 rMurThk)  ;Delta X for wall thk
              rXDif2 (BSide aA3 OffDist)  ;Delta X for Opening
            )

            (if (minusp rXDif2)
          (setq xFactor -1)
          (setq xFactor 1)
            )

            (if (= sPorteOuiNon "DOORPL")
          (progn
            (GetDrINI)
            (ElevDrMk2)
            (entmake (list (cons 0 "INSERT")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 2 BlockName3D)
                   (cons 10 pNewD)
                   (cons 41 xFactor)
      ;;;          (cons 42 YFactor)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (DoorElevTagSet sAppNameAA (entlast))
          )
            )
            (if (= sPorteOuiNon "WINDPL")
          (progn
            (GetWnINI)

            (cond ((= lWn "FW")
               (setq lSeuils
                  (DeStrAListe (car (DeStrAListe lTFxdM1 "-")) "|")
               )
               (setq sSeuil1 (car lSeuils))
               (setq sSeuil2 (cadr lSeuils))
               (setq sSeuil3 (caddr lSeuils))
              )
              ((= lWn "SF")
               (setq lSeuils
                  (DeStrAListe (car (DeStrAListe lTSLdM1 "-")) "|")
               )
               (setq sSeuil1 (car lSeuils))
               (setq sSeuil2 (cadr lSeuils))
               (setq sSeuil3 (caddr lSeuils))
              )
              ((= lWn "SFS")
               (setq lSeuils
                  (DeStrAListe (car (DeStrAListe lTSLdM1 "-")) "|")
               )
               (setq sSeuil1 (car lSeuils))
               (setq sSeuil2 (cadr lSeuils))
               (setq sSeuil3 (caddr lSeuils))
              )
              ((= lWn "DH")
               (setq sSeuil4 (last (DeStrAListe lTDHgM1 "-")))
               (setq lSeuils
                  (DeStrAListe (car (DeStrAListe lTDHgM1 "-")) "|")
               )
               (setq sSeuil1 (car lSeuils))
               (setq sSeuil2 (cadr lSeuils))
               (setq sSeuil3 (caddr lSeuils))
              )
            )

            (ElevWnMk2)
            (entmake (list (cons 0 "INSERT")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 2 BlockName3W)
                   (cons 10 pNewW)
                   (cons 41 xFactor)
      ;;;          (cons 42 YFactor)
      ;;;          (cons 50 aA3)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (WindElevTagSet sAppNameAA (entlast))
            (setq sTr4b nil)
          )
            )
          )
          (princ "\nCannot generate an elevation from this object. ")
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;;;--------Version Assistee avec DRE pour portes
      (defun ElevDrMk2 ()
        (TraceIsOn ".ElevDrMk2")
        (setq rXCheck1 (car pHinge)       ;x value for p1
          rYCheck1 (cadr pHinge)      ;y   for p1
          rXCheck2 (car pMCote)       ;x value for p2
          rYCheck2 (cadr pMCote)      ;y value for p2
          rXCheck3 (car pCoteO)       ;x   for p3
          rYCheck3 (cadr pCoteO)      ;y value for p3
          rXJmb1   (abs (BSide aA3 35.0)) ;Delta X for jamb thk
          rXDif1   (abs rXDif1)       ;Delta X for wall thk
          rXDif2   (abs rXDif2)       ;Delta X for Opening
        )
        (if (> rYCheck3 rYCheck1)
          (progn              ;p1 et p2 de ce cote
                          ;(princ "de ce cote")
            (setq pNewD     (list (car pHinge) rY2 0.0)
              Charniere pHinge
            )

            (if (and (> rXCheck3 rXCheck1) (< rXCheck3 rXCheck2))
          (setq Gauche T
                Droit nil
          )               ;p1 et p3 = gauche
          (setq Gauche nil
                Droit T
          )               ;p2 et p4 = droit
            )
          )
          (progn              ;p1 et p2 de l'autre cote
                          ;(princ "de l'autre cote")
            (setq pNewD     (list (car pCoteO) rY2 0.0)
              Charniere pCoteO
            )

            (if (and (> rXCheck1 rXCheck3) (< rXCheck1 rXCheck2))
          (setq Gauche T
                Droit nil
          )               ;p1 et p3 = gauche
          (setq Gauche nil
                Droit T
          )               ;p2 et p4 = droit
            )
          )
        )

        (entmake
          '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0))
        )

        (cond
          ((= LWn "FSFd")
           (setq sTr4b  2100
             rXJmb1 (abs (BSide aA3 50))
           )

      ;;;Cadre exterieur brique
           (entmake (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 0 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 0 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
           )

      ;;;Cadre exterieur de la fenetre
           (entmake (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 0 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 0 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
           )

                          ;rect gauche exterieur
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ rXDif2 3) (* rXJmb1 0.5)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXJmb1 rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXJmb1 (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ rXDif2 3) (* rXJmb1 0.5)) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

                          ;rect gauche interieur
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ rXDif2 3) (* rXJmb1 0.5)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (- (/ rXDif2 3) (* rXJmb1 0.5))
                 (- sTr4b (* 2 rEpaiss))
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

                          ;Milieu tout en haut
           (entmake (list (cons 0 "LINE")
                  (cons 100 "AcDbEntity")
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbLine")
                  (list 10
                    (+ (/ rXDif2 3) (* rXJmb1 0.5))
                    (- sTr4b rEpaiss)
                    0
                  )
                  (list 11
                    (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                    (- sTr4b rEpaiss)
                    0
                  )
                  (list 210 0 0 1)
                )
           )
                          ;Milieu dessous tout en haut
           (entmake (list (cons 0 "LINE")
                  (cons 100 "AcDbEntity")
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbLine")
                  (list 10
                    (+ (/ rXDif2 3) (* rXJmb1 0.5))
                    (- sTr4b (* 2 rEpaiss))
                    0
                  )
                  (list 11
                    (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                    (- sTr4b (* 2 rEpaiss))
                    0
                  )
                  (list 210 0 0 1)
                )
           )

                          ;Milieu dessus tout en bas
           (entmake
             (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ rXDif2 3) (* rXJmb1 0.5)) (* 2 rEpaiss) 0)
               (list 11
                 (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                 (* 2 rEpaiss)
                 0
               )
               (list 210 0 0 1)
             )
           )
                          ;Milieu tout en bas
           (entmake
             (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ rXDif2 3) (* rXJmb1 0.5)) rEpaiss 0)
               (list 11 (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5)) rEpaiss 0)
               (list 210 0 0 1)
             )
           )

                          ;Fleche
           (entmake (list (cons 0 "POLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDb2dPolyline")
                  (cons 66 1)
                  (list 10 0.0 0.0 0.0)
                  (list 210 0.0 0.0 1.0)
                )
           )
           (entmake
             (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sFrameLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10
                 (- (* 2 (/ rXDif2 3)) (* rXJmb1 1.5))
                 (+ (* 0.5 sTr4b) 50)
                 0
               )
             )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10
                    (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                    (* 0.5 sTr4b)
                    0
                  )
                )
           )
           (entmake
             (list
           (cons 0 "VERTEX")
           (cons 100 "AcDbEntity")
           (cons 67 0)
           (cons 8 sFrameLyr)
           (cons 100 "AcDbVertex")
           (cons 100 "AcDb2dVertex")
           (list
             10
             (- (* 2 (/ rXDif2 3)) (* rXJmb1 1.5))
             (- (* 0.5 sTr4b) 50)
             0
           )
             )
           )
           (entmake (QUOTE ((0 . "SEQEND"))))

                          ;rect droite exterieur
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 rXJmb1) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (- (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                 (- sTr4b rEpaiss)
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

                          ;rect droite interieur
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10
                 (+ (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                 (* 2 rEpaiss)
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (+ (* 2 (/ rXDif2 3)) (* rXJmb1 0.5))
                 (- sTr4b (* 2 rEpaiss))
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )
          )
          ((= LWn "FSSFd")

           (setq sTr4b  2100
             rXJmb1 (abs (BSide aA3 50))
           )

      ;;;Cadre exterieur brique
           (entmake (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 0 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 0 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
           )

      ;;;Cadre exterieur de la fenetre
           (entmake (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 0 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 0)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 rXDif2 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 0 sTr4b)
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
           )

      ;;;rect gauche exterieur fixe
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ rXDif2 4) (* rXJmb1 0.5)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXJmb1 rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXJmb1 (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ rXDif2 4) (* rXJmb1 0.5)) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;rect gauche interieur fixe
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (/ rXDif2 4) (* rXJmb1 0.5)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (- (/ rXDif2 4) (* rXJmb1 0.5))
                 (- sTr4b (* 2 rEpaiss))
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;rect gauche exterieur coulissant
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ rXDif2 4) (* rXJmb1 0.5)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ rXDif2 2) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ rXDif2 2) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ rXDif2 4) (* rXJmb1 0.5)) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;rect gauche interieur coulissant
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ rXDif2 4) (* rXJmb1 0.5)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ rXDif2 2) (* 1 rXJmb1)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (/ rXDif2 2) (* 1 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (+ (/ rXDif2 4) (* rXJmb1 0.5))
                 (- sTr4b (* 2 rEpaiss))
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;Fleche gauche
           (entmake (list (cons 0 "POLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDb2dPolyline")
                  (cons 66 1)
                  (list 10 0.0 0.0 0.0)
                  (list 210 0.0 0.0 1.0)
                )
           )
           (entmake
             (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sFrameLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10
                 (+ (* 1 (/ rXDif2 4)) (* rXJmb1 1.5))
                 (+ (* 0.5 sTr4b) 50)
                 0
               )
             )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10
                    (+ (* 1 (/ rXDif2 4)) (* rXJmb1 0.5))
                    (* 0.5 sTr4b)
                    0
                  )
                )
           )
           (entmake
             (list
           (cons 0 "VERTEX")
           (cons 100 "AcDbEntity")
           (cons 67 0)
           (cons 8 sFrameLyr)
           (cons 100 "AcDbVertex")
           (cons 100 "AcDb2dVertex")
           (list
             10
             (+ (* 1 (/ rXDif2 4)) (* rXJmb1 1.5))
             (- (* 0.5 sTr4b) 50)
             0
           )
             )
           )
           (entmake (QUOTE ((0 . "SEQEND"))))

      ;;;---
      ;;;rect droit exterieur coulissant
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ rXDif2 2) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ rXDif2 2) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5)) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;rect droit interieur coulissant
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ rXDif2 4)) (* rXJmb1 1)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ rXDif2 4)) (* rXJmb1 1)) (- sTr4b (* 2 rEpaiss)))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5))
                 (- sTr4b (* 2 rEpaiss))
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;Fleche droite
           (entmake (list (cons 0 "POLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDb2dPolyline")
                  (cons 66 1)
                  (list 10 0.0 0.0 0.0)
                  (list 210 0.0 0.0 1.0)
                )
           )
           (entmake
             (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sFrameLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10
                 (- (* 3 (/ rXDif2 4)) (* rXJmb1 1.5))
                 (+ (* 0.5 sTr4b) 50)
                 0
               )
             )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10
                    (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5))
                    (* 0.5 sTr4b)
                    0
                  )
                )
           )
           (entmake
             (list
           (cons 0 "VERTEX")
           (cons 100 "AcDbEntity")
           (cons 67 0)
           (cons 8 sFrameLyr)
           (cons 100 "AcDbVertex")
           (cons 100 "AcDb2dVertex")
           (list
             10
             (- (* 3 (/ rXDif2 4)) (* rXJmb1 1.5))
             (- (* 0.5 sTr4b) 50)
             0
           )
             )
           )
           (entmake (QUOTE ((0 . "SEQEND"))))

      ;;;rect droite exterieur fixe
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 rXJmb1) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (- (* 3 (/ rXDif2 4)) (* rXJmb1 0.5))
                 (- sTr4b rEpaiss)
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )

      ;;;rect droite interieur fixe
           (entmake
             (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10
                 (+ (* 3 (/ rXDif2 4)) (* rXJmb1 0.5))
                 (* 2 rEpaiss)
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10
                 (+ (* 3 (/ rXDif2 4)) (* rXJmb1 0.5))
                 (- sTr4b (* 2 rEpaiss))
               )
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
           )


          )
          (T
           (if (= Gauche T)
                          ;Epaisseur du mur gauche
             (entmake (list (cons 0 "LWPOLYLINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDbPolyline")
                    (cons 90 3)
                    (cons 70 0)
                    (cons 43 0.0)
                    (cons 38 0.0)
                    (cons 39 0.0)
                    (list 10 0 0)
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 10 rXDif1 0) ;yes
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 10 rXDif1 2065) ;yes
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 210 0 0 1)
              )
             )
           )
                          ;Cadre exterieur brique
           (entmake (list (cons 0 "POLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDb2dPolyline")
                  (cons 66 1)
                  (cons 10 Charniere)
                  (list 210 0 0 1)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 0 0 0)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 0 2100 0)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 rXDif2 2100 0)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sBrkWall1Lyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 rXDif2 0 0)
                )
           )
           (entmake (quote ((0 . "SEQEND"))))
                          ;Cadre interieur bois
           (entmake (list (cons 0 "POLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sJambLyr)
                  (cons 100 "AcDb2dPolyline")
                  (cons 66 1)
                  (cons 10 Charniere)
                  (list 210 0 0 1)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sJambLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 rXJmb1 0 0)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sJambLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 rXJmb1 2065 0)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sJambLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 (- rXDif2 rXJmb1) 2065 0)
                )
           )
           (entmake (list (cons 0 "VERTEX")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sJambLyr)
                  (cons 100 "AcDbVertex")
                  (cons 100 "AcDb2dVertex")
                  (list 10 (- rXDif2 rXJmb1) 0 0)
                )
           )
           (entmake (quote ((0 . "SEQEND"))))
          )
        )

        (setq BlockName3D (entmake '((0 . "ENDBLK"))))
      )

      ;;;--------Version Assistee avec DRE pour fenetres
      (defun ElevWnMk2    ()
        (TraceIsOn ".ElevWnMk2")
        (setq rXCheck1 (car pHinge)       ;x value for p1
          rYCheck1 (cadr pHinge)      ;y   for p1
          rXCheck2 (car pMCote)       ;x value for p2
          rYCheck2 (cadr pMCote)      ;y value for p2
          rXCheck3 (car pCoteO)       ;x   for p3
          rYCheck3 (cadr pCoteO)      ;y value for p3
          rXJmb1   (abs (BSide aA3 rEpaiss)) ;Delta X for cadre thk
          rXDif1   (abs rXDif1)       ;Delta X for wall thk
          rXDif2   (abs rXDif2)       ;Delta X for Opening
          rHaut    (- 2100.0 sTr4b)
        )
        (if (> rYCheck3 rYCheck1)
          (progn              ;p1 et p2 de ce cote
                          ;(princ "Feinster de ce cote")
            (setq pNewW     (list (car pHinge) (+ rY2 rHaut) 0.0)
              Charniere pHinge
              bDispSill nil
            )
            (if (and (> rXCheck3 rXCheck1) (< rXCheck3 rXCheck2))
          (setq Gauche T
                Droit nil
          )               ;p1 et p3 = gauche
          (setq Gauche nil
                Droit T
          )               ;p2 et p4 = droit
            )
          )
          (progn              ;p1 et p2 de l'autre cote
                          ;(princ "de l'autre cote")
            (setq pNewW     (list (car pCoteO) (+ rY2 rHaut) 0.0)
              Charniere pCoteO
              bDispSill T
            )
            (if (and (> rXCheck1 rXCheck3) (< rXCheck1 rXCheck2))
          (setq Gauche T
                Droit nil
          )               ;p1 et p3 = gauche
          (setq Gauche nil
                Droit T
          )               ;p2 et p4 = droit
            )
          )
        )
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
                          ;  (if (= Droit T)          ;Epaisseur du mur droit
                          ;    (entmake (list (cons 0 "LWPOLYLINE")
                          ;          (cons 100 "AcDbEntity")
                          ;          (cons 67 0)
                          ;          (cons 410 "Model")
                          ;          (cons 8 "MurGBRICK-7")
                          ;          (cons 100 "AcDbPolyline")
                          ;          (cons 90 3)
                          ;          (cons 70 0)
                          ;          (cons 43 0.0)
                          ;          (cons 38 0.0)
                          ;          (cons 39 0.0)
                          ;          (list 10 rXDif2 0)
                          ;          (cons 40 0.0)
                          ;          (cons 41 0.0)
                          ;          (cons 42 0.0)
                          ;          (list 10 (+ rXDif1 rXDif2) 0)
                          ;          (cons 40 0.0)
                          ;          (cons 41 0.0)
                          ;          (cons 42 0.0)
                          ;          (list 10 (+ rXDif1 rXDif2) (- sTr4b rXJmb1))
                          ;          (cons 40 0.0)
                          ;          (cons 41 0.0)
                          ;          (cons 42 0.0)
                          ;          (list 210 0 0 1)
                          ;        )
                          ;   )
                          ;)
      ;;;Cadre exterieur brique
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXDif2 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXDif2 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;;;Cadre exterieur de la fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXDif2 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rXDif2 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;;;-------------------- For a fixed type ----------------------
        (if (= LWn "FW")
          (progn
            (if (<= OffDist (atof sSeuil1))
          (progn              ;1er rect interieur
              (if (or  (= bTransom nil)
            (and (= bTransom T)(< sTr4b (atof sSeuil4))))     ;if height is less than 1600 and transom asked - not possible
                (progn
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rXJmb1) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                )
            (progn                        ;Transom at 1600
            (entmake (list (cons 0 "LWPOLYLINE")          ;1er rect interieur
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rXJmb1) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
            (entmake (list (cons 0 "LWPOLYLINE")          ;Imposte
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
               )
              )
          )
            )
      ;;;-----------------------------------------------------------------------
            (if (and (> OffDist (atof sSeuil1)) (<= OffDist (atof sSeuil2)))
          (progn
              (if (or  (= bTransom nil)
              (and (= bTransom T)(< sTr4b (atof sSeuil4))))       ;if height is less than 1600 and transom asked - not possible
                (progn
            (entmake (list (cons 0 "LWPOLYLINE")      ;1er rect gauche
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (* 0.5 (- rXDif2 (* 3 rXJmb1)))) rEpaiss) ;705
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (- sTr4b (* 1 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b (* 1 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 0.5 (- rXDif2 (* 3 rXJmb1))) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 0.5 (- rXDif2 (* 3 rXJmb1))) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;1er rect droit
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 rXJmb1) rEpaiss)
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 0.5 (- rXDif2 (* 3 rXJmb1)))) rEpaiss) ;765
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (- sTr4b (* 1 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 rXJmb1) (- sTr4b (* 1 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;2eme rect droit
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (* 2 rEpaiss)) ;765
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
                )
            (progn                        ;Transom at 1600
            (entmake (list (cons 0 "LWPOLYLINE")          ;1er rect gauche
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (- sTr4b (* 1 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b (* 1 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 0.5 (- rXDif2 (* 3 rXJmb1))) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 0.5 (- rXDif2 (* 3 rXJmb1))) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;1er rect droit
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 rXJmb1) (+ (/ sTr4b 3) rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (+ (/ sTr4b 3) rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 0.5 (- rXDif2 (* 3 rXJmb1)))) (- sTr4b (* 1 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 rXJmb1) (- sTr4b (* 1 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;2eme rect droit
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (+ (/ sTr4b 3)(* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1)(* 0.5 (- rXDif2 (* 3 rXJmb1)))) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1)(* 0.5 (- rXDif2 (* 3 rXJmb1)))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
            (entmake (list (cons 0 "LWPOLYLINE")          ;Imposte
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
               )
              )
          )
            )
      ;;;-----------------------------------------------------------------------
            (if (> OffDist (atof sSeuil2))
          (progn
              (if (or  (= bTransom nil)
              (and (= bTransom T)(< sTr4b (atof sSeuil4))))       ;if height is less than 1600 and transom asked - not possible
                (progn
      ;1er rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (/ (- rXDif2 (* 4 rXJmb1)) 3)) rEpaiss) ;705
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (/ (- rXDif2 (* 4 rXJmb1)) 3)) (- sTr4b (* 1 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b (* 1 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ (- rXDif2 (* 4 rXJmb1)) 3) (* 2 rEpaiss)) ;705
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ (- rXDif2 (* 4 rXJmb1)) 3) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;rect du milieu
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) rEpaiss)
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) rEpaiss)
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;rect du milieu interieur
            (entmake
              (list
                (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (* 2 rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ rXJmb1 (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (* 2 rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ rXJmb1 (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;1er rect droit
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 rXJmb1) rEpaiss)
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) rEpaiss)
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;2eme int rect droit
            (entmake
              (list
                (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 4 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (* 2 rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 4 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
               )
      ;------------
               (progn
      ;cadre du bas ou imposte
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;1er rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (/ (- rXDif2 (* 4 rXJmb1)) 3)) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ rXJmb1 (/ (- rXDif2 (* 4 rXJmb1)) 3)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ (- rXDif2 (* 4 rXJmb1)) 3) (+ (/ sTr4b 3)(* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ (- rXDif2 (* 4 rXJmb1)) 3) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;rect du milieu
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (+ (/ sTr4b 3) rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (+ (/ sTr4b 3) rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 2 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;rect du milieu interieur
            (entmake
              (list
                (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ rXJmb1 (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ rXJmb1 (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 1 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;1er rect droit
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 rXJmb1) (+ (/ sTr4b 3) rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (+ (/ sTr4b 3) rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 3 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
      ;2eme int rect droit
            (entmake
              (list
                (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 4 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 4 rXJmb1) (* 2 (/ (- rXDif2 (* 4 rXJmb1)) 3))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
               )
              )
          )
            )
          )
        )
      ;;;------------------------ For a sliding, fixed type Wind and Door --------------------------
        (if (or (= LWn "SF")(= LWn "SFd"))
          (progn              ;fleche
            (if (or  (= bTransom nil)
            (and (= bTransom T)(< sTr4b (atof sSeuil4))))     ;if height is less than 1600 and transom asked - not possible
              (progn
            (entmake (list (cons 0 "POLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0.0 0.0 0.0)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (entmake
              (list (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                (list 10
                  (- (/ rXDif2 2) 75)
                  (+ (* 0.5 sTr4b) 50)
                  0
                )
              )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                         (- (/ rXDif2 2) 25)
                         (* 0.5 sTr4b)
                         0
                   )
                 )
            )
            (entmake
              (list
                (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                (list
              10
              (- (/ rXDif2 2) 75)
              (- (* 0.5 sTr4b) 50)
              0
                )
              )
            )
            (entmake (QUOTE ((0 . "SEQEND"))))
      ;1er rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;rect droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- (* 0.5 rXDif2) (* 0.5 rXJmb1)) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 (- (* 0.5 rXDif2) (* 0.5 rXJmb1))) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 (- (* 0.5 rXDif2) (* 0.5 rXJmb1))) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- (* 0.5 rXDif2) (* 0.5 rXJmb1)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (+ (* 0.5 rXDif2) (* 0.5 rXJmb1)) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 (- (* 0.5 rXDif2) (* 1.0 rXJmb1))) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 (- (* 0.5 rXDif2) (* 1.0 rXJmb1))) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ (* 0.5 rXDif2) (* 0.5 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
              )
            )
            (if (and (= bTransom T)(> sTr4b (atof sSeuil4)))      ;if height is more than 1600 and transom asked
          (progn
            (entmake (list (cons 0 "POLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0.0 0.0 0.0)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (entmake
              (list (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                (list 10
                  (- (/ rXDif2 2) 75)
                  (+ (* 2 (/ sTr4b 3)) 50)
                  0
                )
              )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                         (- (/ rXDif2 2) 25)
                         (* 2 (/ sTr4b 3))
                         0
                   )
                 )
            )
            (entmake
              (list
                (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                (list
              10
              (- (/ rXDif2 2) 75)
              (- (* 2 (/ sTr4b 3)) 50)
              0
                )
              )
            )
            (entmake (QUOTE ((0 . "SEQEND")))) ;cadre du bas ou imposte
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;1er rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* (- rXDif2 rXJmb1) 0.5) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;rect droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- (* 0.5 rXDif2) (* 0.5 rXJmb1)) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 (- (* 0.5 rXDif2) (* 0.5 rXJmb1))) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 (- (* 0.5 rXDif2) (* 0.5 rXJmb1))) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- (* 0.5 rXDif2) (* 0.5 rXJmb1)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
      ;rect interieur
            (entmake
              (list (cons 0 "LWPOLYLINE")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 410 "Model")
                (cons 8 sFrameLyr)
                (cons 100 "AcDbPolyline")
                (cons 90 4)
                (cons 70 1)
                (cons 43 0.0)
                (cons 38 0.0)
                (cons 39 0.0)
                (list 10 (+ (* 0.5 rXDif2) (* 0.5 rXJmb1)) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (* 2 (- (* 0.5 rXDif2) (* 1.0 rXJmb1))) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (* 2 (- (* 0.5 rXDif2) (* 1.0 rXJmb1))) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 10 (+ (* 0.5 rXDif2) (* 0.5 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                (cons 40 0.0)
                (cons 41 0.0)
                (cons 42 0.0)
                (list 210 0 0 1)
              )
            )
          )
            )
          )
        )
      ;;;----------------------- For a sliding, fixed type and sliding ------------------------
        (if (= LWn "SFS")
          (progn
            (if (or  (= bTransom nil)
            (and (= bTransom T)(< sTr4b (atof sSeuil4))))     ;if height is less than 1600 and transom asked - not possible
              (progn
            (entmake (list (cons 0 "POLYLINE")            ;fleche de gauche
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0.0 0.0 0.0)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (entmake
              (list (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                (list 10
                  (- (/ rXDif2 4) 50)
                  (+ (* 0.5 sTr4b) 50)
                  0
                )
              )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (/ rXDif2 4)
                      (* 0.5 sTr4b)
                      0
                   )
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (- (/ rXDif2 4) 50)
                      (- (* 0.5 sTr4b) 50)
                      0
                   )
                 )
            )
            (entmake (QUOTE ((0 . "SEQEND"))))            ;fleche de droite
            (entmake (list (cons 0 "POLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0.0 0.0 0.0)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                (list 10
                  (+ (* 3 (/ rXDif2 4)) 50)
                  (+ (* 0.5 sTr4b) 50)
                  0
                )
              )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (* 3 (/ rXDif2 4))
                      (* 0.5 sTr4b)
                      0
                   )
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (+ (* 3 (/ rXDif2 4)) 50)
                      (- (* 0.5 sTr4b) 50)
                      0
                   )
                 )
            )
            (entmake (QUOTE ((0 . "SEQEND"))))            ;1er rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect du milieu
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (/ rXDif2 4) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect interieur du milieu
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (+ (/ rXDif2 4) rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- (* 3 (/ rXDif2 4)) rXJmb1) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- (* 3 (/ rXDif2 4)) rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ (/ rXDif2 4) rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rXJmb1) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
          )
            )
      ;;;;;***
            (if (and (= bTransom T)(> sTr4b (atof sSeuil4)))      ;if height is more than 1600
          (progn
            (entmake (list (cons 0 "POLYLINE")            ;fleche de gauche
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0.0 0.0 0.0)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (entmake
              (list (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                (list 10
                  (- (/ rXDif2 4) 50)
                  (+ (* 2 (/ sTr4b 3)) 50)        ;;;;;*** 2/3
                  0
                )
              )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (/ rXDif2 4)
                      (* 2 (/ sTr4b 3))       ;;;;;*** 2/3
                         0
                   )
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (- (/ rXDif2 4) 50)
                      (- (* 2 (/ sTr4b 3)) 50)
                         0
                   )
                 )
            )
            (entmake (QUOTE ((0 . "SEQEND"))))
                                      ;fleche de droite
            (entmake (list (cons 0 "POLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0.0 0.0 0.0)
                   (list 210 0.0 0.0 1.0)
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (+ (* 3 (/ rXDif2 4)) 50)
                      (+ (* 2 (/ sTr4b 3)) 50)
                         0
                   )
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10
                      (* 3 (/ rXDif2 4))
                      (* 2 (/ sTr4b 3))
                         0
                   )
                 )
            )
            (entmake
              (list (cons 0 "VERTEX")
                (cons 100 "AcDbEntity")
                (cons 67 0)
                (cons 8 sFrameLyr)
                (cons 100 "AcDbVertex")
                (cons 100 "AcDb2dVertex")
                   (list 10
                      (+ (* 3 (/ rXDif2 4)) 50)
                      (- (* 2 (/ sTr4b 3)) 50)
                      0
                   )
              )
            )
            (entmake (QUOTE ((0 . "SEQEND"))))
                                      ;cadre du bas ou imposte
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (/ sTr4b 3))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;1er rect gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rXJmb1 (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rXJmb1 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;2eme rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect du milieu
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (/ rXDif2 4) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (/ rXDif2 4) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect interieur du milieu
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (+ (/ rXDif2 4) rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- (* 3 (/ rXDif2 4)) rXJmb1) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- (* 3 (/ rXDif2 4)) rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (+ (/ rXDif2 4) rXJmb1) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 rXJmb1) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (+ (/ sTr4b 3) rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 rXJmb1) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                                      ;rect interieur
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (+ (/ sTr4b 3) (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 (/ rXDif2 4)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
          )
            )
          )
        )

      ;;;-------------------- For a Double Hung ----------------------
      (if (= LWn "DH")
        (progn
          (if (<= OffDist (atof sSeuil1))
            (progn
          (if (or (= bTransom nil)
              (and (= bTransom T) (< sTr4b (atof sSeuil4)))
              )               ;if height is less than 1600 and transom asked - not possible
            (progn
                          ;1er rect interieur du bas
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- (/ rXDif2 2) 50) (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (/ rXDif2 2) (* 0.5 sTr4b) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ rXDif2 2) 50) (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))
      ;;;;;-----------
                          ;1er rect interieur du haut
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 (/ sTr4b 2)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (+ rXJmb1 (/ sTr4b 2)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- (/ rXDif2 2) 50) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (/ rXDif2 2) (+ (* 0.5 sTr4b) 50) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ rXDif2 2) 50) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

            )
            (progn            ;Transom at 1600

                          ;1er rect interieur du bas
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (+ (/ sTr4b 3) rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (+ (/ sTr4b 3) rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (+ (* 2 rXJmb1) (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (+ (* 2 rXJmb1) (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- (/ rXDif2 2) 50) (- (* (/ sTr4b 3) 2) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (/ rXDif2 2) (* (/ sTr4b 3) 2) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ rXDif2 2) 50) (- (* (/ sTr4b 3) 2) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))
      ;;;;;-----------
                          ;1er rect interieur du haut
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- (/ rXDif2 2) 50) (+ (* 2 (/ sTr4b 3)) 100) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (/ rXDif2 2) (+ (* 2 (/ sTr4b 3)) 50) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ rXDif2 2) 50) (+ (* 2 (/ sTr4b 3)) 100) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

              (entmake (list (cons 0 "LWPOLYLINE") ;Imposte
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rEpaiss (/ sTr4b 3))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rEpaiss rEpaiss)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rEpaiss) rEpaiss)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
            )
          )
            )
          )
      ;;;-----------------------------------------------------------------------
          (if (and (> OffDist (atof sSeuil1)) (<= OffDist (atof sSeuil2)))
            (progn
          (if (or (= bTransom nil)
              (and (= bTransom T) (< sTr4b (atof sSeuil4)))
              )               ;if height is less than 1600 and transom asked - not possible
            (progn
                          ;1er rect interieur du bas
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1)  (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1) (* 0.5 sTr4b) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1 rXJmb1) (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

      ;;;;;-----------
                          ;1er rect interieur du haut
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 (/ sTr4b 2)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (+ rXJmb1 (/ sTr4b 2)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1 rXJmb1) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1) (+ (* 0.5 sTr4b) 50) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

                          ;Panneau fixe a droite
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (+ (/ rXDif2 2) (* 0.5 rXJmb1)) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (+ (/ rXDif2 2) (* 0.5 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
            )

            (progn            ;Transom at 1600

                          ;1er rect interieur du bas
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (+ rXJmb1 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (+ rXJmb1 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 rXJmb1 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (+ rXJmb1 rXJmb1 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1)  (- (* 2 (/ sTr4b 3)) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1) (* 2 (/ sTr4b 3)) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1 rXJmb1) (- (* 2 (/ sTr4b 3)) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

      ;;;;;-----------
                          ;1er rect interieur du haut
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 2) (* 0.5 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 2) (* 1.5 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1 rXJmb1) (+ rXJmb1 rXJmb1 (* 2 (/ sTr4b 3))) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1 rXJmb1) (+ rXJmb1 (* 2 (/ sTr4b 3))) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 2) (* 3.5 50)) 2) rXJmb1) (+ rXJmb1 rXJmb1 (* 2 (/ sTr4b 3))) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

                          ;Panneau fixe a droite
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (+ (/ rXDif2 2) (* 0.5 rXJmb1)) (+ rXJmb1 (* 1 (/ sTr4b 3))))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (+ rXJmb1 (* 1 (/ sTr4b 3))))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (+ (/ rXDif2 2) (* 0.5 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )

              (entmake (list (cons 0 "LWPOLYLINE") ;Imposte
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rEpaiss (/ sTr4b 3))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rEpaiss rEpaiss)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rEpaiss) rEpaiss)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
            )
          )
            )
          )
      ;;;-----------------------------------------------------------------------
          (if (> OffDist (atof sSeuil2))
            (progn
          (if (or (= bTransom nil)
              (and (= bTransom T) (< sTr4b (atof sSeuil4)))
              )               ;if height is less than 1600 and transom asked - not possible
            (progn

                          ;1er rect interieur du bas
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (/ rXDif2 3) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (/ rXDif2 3)  (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1 rXJmb1)) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1 rXJmb1)) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1)  (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1) (* 0.5 sTr4b) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1 rXJmb1) (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

      ;;;;;-----------
                          ;1er rect interieur du haut
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (/ rXDif2 3) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (/ rXDif2 3) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 (/ sTr4b 2)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1 rXJmb1)) (+ rXJmb1 (/ sTr4b 2)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut gauche
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1) (+ (* 0.5 sTr4b) 50) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1 rXJmb1) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

                          ;Panneau fixe au milieu
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (+ (/ rXDif2 3) (* 1 rXJmb1)) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (* (/ rXDif2 3) 2) rXJmb1) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (* (/ rXDif2 3) 2) rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (+ (/ rXDif2 3) (* 1 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )

                          ;1er rect interieur du bas droit
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rXJmb1) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* 2 (/ rXDif2 3)) rXJmb1)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* 2 (/ rXDif2 3)) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas droit
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (+ (* 2 (/ rXDif2 3)) (* 1 rXJmb1)) (* 2 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (+ (* 2 (/ rXDif2 3)) (* 1 rXJmb1)) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (/ sTr4b 2))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas droite
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1))  (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1)) (* 0.5 sTr4b) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1 rXJmb1)) (- (* 0.5 sTr4b) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))


                          ;1er rect interieur du haut droit
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rXJmb1) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* 2 (/ rXDif2 3)) (/ sTr4b 2))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* 2 (/ rXDif2 3)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut droit
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (+ (/ sTr4b 2) rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (+ (* 2 (/ rXDif2 3)) (* 1 rXJmb1)) (+ (/ sTr4b 2) rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (+ (* 2 (/ rXDif2 3)) (* 1 rXJmb1)) (- sTr4b rXJmb1 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 (* 2 rXJmb1)) (- sTr4b rXJmb1 rXJmb1))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )

                          ;Fleche du haut droit
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1)) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1)) (+ (* 0.5 sTr4b) 50) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1 rXJmb1)) (+ (* 0.5 sTr4b) 100) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

            )
      ;------------
            (progn

                          ;1er rect interieur du bas gauche
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (+ rXJmb1 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 3) (* 0.5 rXJmb1)) (+ rXJmb1 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 3) (* 0.5 rXJmb1)) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas gauche
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 rXJmb1 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1.5 rXJmb1)) (+ rXJmb1 rXJmb1 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1.5 rXJmb1)) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas gauche
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3.5 50)) 2) rXJmb1)  (- (* 2 (/ sTr4b 3)) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 3) (* 3.5 50)) 2) rXJmb1 rXJmb1) (* 2 (/ sTr4b 3)) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3.5 50)) 2) rXJmb1 rXJmb1 rXJmb1) (- (* 2 (/ sTr4b 3)) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

      ;;;;;-----------
                          ;1er rect interieur du haut g
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 rXJmb1 (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 3) (* 0.5 rXJmb1)) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (/ rXDif2 3) (* 0.5 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rXJmb1 (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut g
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (* 2 rXJmb1) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1.5 rXJmb1)) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- (/ rXDif2 3) (* 1.5 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (* 2 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3.5 50)) 2) rXJmb1 rXJmb1 rXJmb1) (+ rXJmb1 rXJmb1 (* 2 (/ sTr4b 3))) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (+ (/ (- (/ rXDif2 3) (* 3.5 50)) 2) rXJmb1 rXJmb1) (+ rXJmb1 (* 2 (/ sTr4b 3))) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (+ (/ (- (/ rXDif2 3) (* 3.5 50)) 2) rXJmb1) (+ rXJmb1 rXJmb1 (* 2 (/ sTr4b 3))) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

                          ;Panneau fixe au milieu
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (+ (/ rXDif2 3) (* 0.5 rXJmb1)) (+ rXJmb1 (* 1 (/ sTr4b 3))))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (* 2 (/ rXDif2 3)) rXJmb1) (+ rXJmb1 (* 1 (/ sTr4b 3))))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- (* 2 (/ rXDif2 3)) rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (+ (/ rXDif2 3) (* 0.5 rXJmb1)) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )

                          ;1er rect interieur du bas droite
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rXJmb1) (+ rXJmb1 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* (/ rXDif2 3) 2) (+ rXJmb1 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* (/ rXDif2 3) 2) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du bas droite
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (- rXDif2 rXJmb1 rXJmb1) (+ rXJmb1 rXJmb1 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (+ (* (/ rXDif2 3) 2) (* 1 rXJmb1)) (+ rXJmb1 rXJmb1 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (+ (* (/ rXDif2 3) 2) (* 1 rXJmb1)) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 rXJmb1 rXJmb1) (* 2 (/ sTr4b 3)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du bas droit
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1))  (- (* 2 (/ sTr4b 3)) 50) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1)) (* 2 (/ sTr4b 3)) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1 rXJmb1)) (- (* 2 (/ sTr4b 3)) 50) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))

      ;;;;;-----------?
                          ;1er rect interieur du haut droite
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rXJmb1) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* (/ rXDif2 3) 2) (* 2 (/ sTr4b 3)))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (* (/ rXDif2 3) 2) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rXJmb1) (- sTr4b rXJmb1))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )
                          ;2eme rect interieur du haut droite
              (entmake
                (list (cons 0 "LWPOLYLINE")
                  (cons 100 "AcDbEntity")
                  (cons 67 0)
                  (cons 410 "Model")
                  (cons 8 sFrameLyr)
                  (cons 100 "AcDbPolyline")
                  (cons 90 4)
                  (cons 70 1)
                  (cons 43 0.0)
                  (cons 38 0.0)
                  (cons 39 0.0)
                  (list 10 (- rXDif2 rXJmb1 rXJmb1) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10   (+ (* (/ rXDif2 3) 2) (* 1 rXJmb1)) (+ rXJmb1 (* 2 (/ sTr4b 3))))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10   (+ (* (/ rXDif2 3) 2) (* 1 rXJmb1)) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 10 (- rXDif2 rXJmb1 rXJmb1) (- sTr4b (* 2 rXJmb1)))
                  (cons 40 0.0)
                  (cons 41 0.0)
                  (cons 42 0.0)
                  (list 210 0 0 1)
                )
              )
                          ;Fleche du haut droit
              (entmake (list
                     (cons 0 "POLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDb2dPolyline")
                     (cons 66 1)
                     (list 10 0 0 0)
                     (list 210 0 0 1)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1)) (+ rXJmb1 rXJmb1 (* 2 (/ sTr4b 3))) 0)
                )
              )
              (entmake (list
                     (cons 0 "VERTEX")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbVertex")
                     (cons 100 "AcDb2dVertex")
                     (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1)) (+ rXJmb1 (* 2 (/ sTr4b 3))) 0)
                   )
              )
              (entmake
                (list
              (cons 0 "VERTEX")
              (cons 100 "AcDbEntity")
              (cons 67 0)
              (cons 8 sFrameLyr)
              (cons 100 "AcDbVertex")
              (cons 100 "AcDb2dVertex")
              (list 10 (- rXDif2 (+ (/ (- (/ rXDif2 3) (* 3 50)) 2) rXJmb1 rXJmb1 rXJmb1)) (+ rXJmb1 rXJmb1 (* 2 (/ sTr4b 3))) 0)
                )
              )
              (entmake (quote ((0 . "SEQEND"))))


                          ;cadre du bas ou imposte
              (entmake (list (cons 0 "LWPOLYLINE")
                     (cons 100 "AcDbEntity")
                     (cons 67 0)
                     (cons 410 "Model")
                     (cons 8 sFrameLyr)
                     (cons 100 "AcDbPolyline")
                     (cons 90 4)
                     (cons 70 1)
                     (cons 43 0.0)
                     (cons 38 0.0)
                     (cons 39 0.0)
                     (list 10 (- rXDif2 rEpaiss) (/ sTr4b 3))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rEpaiss (/ sTr4b 3))
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 rEpaiss rEpaiss)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 10 (- rXDif2 rEpaiss) rEpaiss)
                     (cons 40 0.0)
                     (cons 41 0.0)
                     (cons 42 0.0)
                     (list 210 0 0 1)
                   )
              )

            )
          )
            )
          )
        )
      )

        (if (and (= bThreSil T) (= bDispSill T))
          (progn (entmake (list (cons 0 "LWPOLYLINE")
                    (cons 100 "AcDbEntity")
                    (cons 410 "Model")
                    (cons 8 sSillLyr)
                    (cons 100 "AcDbPolyline")
                    (cons 90 4)
                    (cons 70 1)
                    (cons 43 0.0)
                    (cons 38 0.0)
                    (cons 39 0.0)
                    (list 10 0 -110)
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 10 rXDif2 -110)
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 10 rXDif2 0)
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 10 0 0)
                    (cons 40 0.0)
                    (cons 41 0.0)
                    (cons 42 0.0)
                    (list 210 0 0 1)
                  )
             )
             (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 410 "Model")
                    (cons 8 sSillLyr)
                    (cons 100 "AcDbLine")
                    (list 10 0 -35 0)
                    (list 11 rXDif2 -35 0)
                    (list 210 0 0 1)
                  )
             )
             (setq rSpace2 (/ OffDist (fix (/ OffDist 100)) 1.0))
             (command "_.UCS" "_M" pNewW)
             (setvar "snapbase" (list 0 0))
             (Hshset_lay sSillLyr "23" nil "0.25")
             (command "_.HATCH"
                  "_U"
                  "90"
                  rSpace2
                  "_N"
                  ""
                  "_N"
                  (list 0 0 0)
                  (list 0 -110 0)
                  (list rXDif2 -110 0)
                  (list rXDif2 0 0)
                  "_C"
                  ""
             )
             (command "_.UCS" "_P")
          )
        )
        (setq BlockName3W (entmake '((0 . "ENDBLK"))))
      )

      (TraceIsOn ".19f")

      ;;------------Version non-Assistee pour automation des facades
      (defun DrawElevs (/)
        (TraceIsOn ".DrawElevs")
        (setq doorpick T)
        (while (/= nil doorpick)
          (princ "\nSelect a door or window: ")
          (setq doorpick (ssget "+.:E:S" '((0 . "INSERT"))))
          ;;(princ doorpick)

          (if (and (/= nil doorpick)(/= nil (AAXDataGet (ssname doorpick 0) sAppNameAA)))
            (setq sPorteOuiNon (cdr (nth 0 (AAXDataGet (ssname doorpick 0) sAppNameAA)))) ;Une porte? Vue en plan?
            (setq sPorteOuiNon nil)
          )

          (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
            (progn (setq ODeNom   (ssname doorpick 0)
                 sDrHan2  (read (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA))))     ;Database handle
                 LWn      (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))        ;Type de fenetre
                 sTr4b    (read (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA))))     ;Window ONLY height***********
                 OFFDIST  (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))))     ;Door width
                 pHinge   (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))        ;Hinge
                 pMCote   (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))        ;meme cote
                 pCoteO   (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))        ;cote oppose
                 rMurThk  (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))       ;Wall thk
                 bHead    (read (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA))))    ;Header/HeaderW present?
                 bThreSil (read (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA))))    ;Threshold/Sill present?
                 bTransom (read (cdr (nth 13 (AAXDataGet ODeNom sAppNameAA))))    ;Transom present?
                 sSeuil4  (cdr (nth 14 (AAXDataGet ODeNom sAppNameAA)))       ;Transom after what height?
                 sTofIns  (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))       ;Inseree a quelle heure?
                 ODeNom   (entget ODeNom)
                 lDoor    ODeNom
                 aA1      (round2dec (angle pHinge pCoteO) 100000)
                 aA3      (round2dec (angle pHinge pMCote) 100000)
                 aDiff1   (- aA1 aA3)
                 pCnr1    (list 0 0 0)
                 ODrNam   (dxf -1 lDoor)
                 xFactor  (dxf 41 lDoor)
                 yFactor  (dxf 42 lDoor)
                 aRot     (dxf 50 lDoor)
               )
               (if (minusp yFactor)
                 (if (minusp xFactor)
               (setq sOuvreTo "R")
               (setq sOuvreTo "L")
                 )
                 (if (minusp xFactor)
               (setq sOuvreTo "L")
               (setq sOuvreTo "R")
                 )
               )
               (if (minusp aDiff1)
                 (setq aDiff1 (+ HshPi360 aDiff1))
               )
               (if (= (round2dec HshPi90 100000000) (round2dec aDiff1 100000000))
                 (Setq sOuvreA "Right")
                 (Setq sOuvreA "Left")
               )
               (setq pP4 (polar pMCote aA1 rMurThk))
            )
            (princ "\nCannot generate an elevation from this object. ")
          )

          (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
            (progn (setq ssdoor (ssadd ODrNam))
               (setq ent1    (objsel "LINE or LWPOLYLINE" "Select a reference line" nil)
                 pDepart (dxf 10 (entget (car ent1)))
                 pArriv  (dxf 11 (entget (car ent1)))
                 rY2     (cadr pDepart)                       ;get y value of reference line
                 rEpaiss 40
                 aAlph1  (- aA1 ang1)                         ;diff. d'angle sur epaiss. mur
               )
               (if (or (IsInBetween aAlph1 1.5616 1.58) (IsInBetween aAlph1 -0.0092 0.0092));if 1/2deg. from parallel
                 (setq bParall T)
                 (setq bParall nil)
               )
               (if (and (minusp aAlph1) (= bParall nil))                  ;si angle est negatif et lignes non paralleles
                 (progn (while (> aAlph1 0)                       ;add multiple of 90 deg
                  (setq aAlph1 (+ aAlph1 hshpi90))
                    )
                 )
                 (progn                                   ;si angle est positif
               (if (> aAlph1 hshpi90)
                 (while (> aAlph1 hshpi90)                        ;remove multiple of 90 deg
                   (setq aAlph1 (- aAlph1 hshpi90))
                 )
               )
                 )
               )
               (if (= bParall T)
                 (setq aAlph1 aA1
                   aAlph2 aA3
                 )
                 (setq aAlph2 (- hshpi90 aAlph1))
               )
                          (princ bParall)(princ " % ")(princ aDiff1)(princ " %% ")
               (setq rXDif1   (round2dec (BSide aAlph1 rMurThk) 100)          ;Delta X for wall thk
                 rXDif2   (round2dec (BSide aAlph2 OffDist) 100)  ;Delta X for Opening ****wrong if almost parallel***
                 pAlongX  (getvar "UCSXDIR")
                 aBlokRot (rtd (angle (list 0 0 0) pAlongX))
               )

               (if (= sPorteOuiNon "DOORPL")
                 (progn (GetDrINI)
                    (ElevDrMk)
                    (entmake (list (cons 0 "INSERT")
                           (cons 100 "AcDbEntity")
                           (cons 67 0)
                           (cons 2 BlockName3D)
                           (cons 10 (trans pNewD 1 0))
                           (cons 41 NxFactor)
                           (list 210 0.0 0.0 1.0)
                         )
                    )
                    (command "_.ROTATE" (entlast) "" pNewD aBlokRot)
                    (DoorElevTagSet sAppNameAA (entlast))
                 )
               )
               (if (= sPorteOuiNon "WINDPL")
                 (progn (GetWnINI)
                    (ElevWnMk)
                    (entmake (list (cons 0 "INSERT")
                           (cons 100 "AcDbEntity")
                           (cons 67 0)
                           ;;      (cons 100 "AcDbInsertBlock")
                           (cons 2 BlockName3W)
                           (cons 10 (trans pNewW 1 0))
                           (cons 41 NxFactor)
                           (list 210 0.0 0.0 1.0)
                         )
                    )
                    (command "_.ROTATE" (entlast) "" pNewW aBlokRot)
                    (WindElevTagSet sAppNameAA (entlast))
                 )
               )
            )

          )
        )
        ;;  (princ "Sortie")
      )

      ;;-----------Version non-Assistee pour automation des facades
      (defun ElevDrMk ()
        (TraceIsOn ".ElevDrMk")
        (setq pHing    (trans pHinge 0 1)
          pMCote   (trans pMCote 0 1)
          pCoteO   (trans pCoteO 0 1)
          pP4      (trans pP4 0 1)
          pMid13   (mid_pt pHing pMCote)
          pMid24   (mid_pt pCoteO pP4)
          rXCheck1 (car pHing)                ;x value for p1
          rYCheck1 (cadr pHing)               ;y   for p1
          rXCheck2 (car pMCote)               ;x value for p2
          rYCheck2 (cadr pMCote)              ;y value for p2
          rXCheck3 (car pCoteO)               ;x   for p3
          rYCheck3 (cadr pCoteO)              ;y value for p3
          rXJmb1   (round2dec (BSide aAlph2 35.0) 100)    ;Delta X for jamb thk
          rYCheck4 (cadr pMid13)              ;y for pt between p1 and p3
          rYCheck5 (cadr pMid24)              ;y for pt between p2 and p4
        )

        (if (> rXCheck2 rXCheck1)
          (progn
                                      ;(princ "Charniere a gauche ")
            (setq bNorm T)
          )
          (progn
                                      ;(princ "Charniere a droite ")
            (setq bNorm nil)
          )
        )

        (if (> rYCheck5 rYCheck4)
          (progn                      ;p1 et p2 de ce cote
                                      ;(princ "de ce cote ")
            (setq Charniere pHing  bAutreCote nil)
          )
          (progn                      ;p1 et p2 de l'autre cote
                                      ;(princ "de l'autre cote ")
            (setq Charniere pCoteO  bAutreCote T)
          )
        )

        (setq pNewD    (PerpPt2Line Charniere (trans pDepart 0 1) (trans pArriv 0 1)) ;0=World (WCS), 1=User (current UCS)
          pOnLin   (PerpPt2Line pHing (trans pN1 0 1) (trans pN2 0 1))        ;0=World (WCS), 1=User (current UCS)
          rHContrl (distance pHing pOnLin)
          lKeep    (list (cons rHContrl (list pHing)))
          pOnLin   (PerpPt2Line pCoteO (trans pN1 0 1) (trans pN2 0 1))       ;0=World (WCS), 1=User (current UCS)
          rOContrl (distance pCoteO pOnLin)
          lKeep    (append (list (cons rOContrl (list pCoteO))) lKeep)
          pOnLin   (PerpPt2Line pMCote (trans pN1 0 1) (trans pN2 0 1))       ;0=World (WCS), 1=User (current UCS)
          rMContrl (distance pMCote pOnLin)
          lKeep    (append (list (cons rMContrl (list pMCote))) lKeep)
          pOnLin   (PerpPt2Line pP4 (trans pN1 0 1) (trans pN2 0 1))      ;0=World (WCS), 1=User (current UCS)
          rPContrl (distance pP4 pOnLin)
          lKeep    (append (list (cons rPContrl (list pP4))) lKeep)
          iKeep    (car (vl-sort-i lkeep (function (lambda (e1 e2) (> (car e1) (car e2))))))  ;Index du plus grand au plus petit
          pKept    (cadr (nth iKeep lKeep))                   ;Point a garder
          rDif5    (- (car pKept)(car pNewD))
        )

         (princ aBlokRot)(princ "|")(princ rDif5)(princ "||")(princ bAutreCote)

         (if (>= aBlokRot 90)                 ;angle > 90
          (progn  ;(princ "o2o")
          (if (= bAutreCote T)                ;si de l'autre cote
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor -1  rDif5 (* -1 rDif5))
                (setq NxFactor 1)
               )
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor 1)
                (setq NxFactor -1  rDif5 (* -1 rDif5))
               )
              )
          )
          (progn  ;(princ "o1o")              ;angle < 90
          (if (= bAutreCote T)                ;si de l'autre cote
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor 1)
                (setq NxFactor -1  rDif5 (* -1 rDif5))
               )
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor 1)
                (setq NxFactor -1  rDif5 (* -1 rDif5))
               )
              )
          )
         )

        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (if (and (= bParall nil)(< (abs rXDif1)(abs rXDif2))) ;Epaisseur du mur
          (entmake (list (cons 0 "LWPOLYLINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
                 (cons 8 sBrkWall1Lyr)
                 (cons 100 "AcDbPolyline")
                 (cons 90 3)
                 (cons 70 0)
                 (cons 43 0.0)
                 (cons 38 0.0)
                 (cons 39 0.0)
                 (list 10 rDif5 0)            ;yes
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 rDif5 2065)         ;yes
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 210 0 0 1)
               )
          )
        )

      ;Cadre exterieur brique
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (cons 10 Charniere)
               (list 210 0 0 1)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 0 0 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 0 2100 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 rXDif2 2100 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 rXDif2 0 0)
             )
        )
        (entmake (quote ((0 . "SEQEND"))))

      ;Cadre interieur bois
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sJambLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (cons 10 Charniere)
               (list 210 0 0 1)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 rXJmb1 0 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 rXJmb1 2065 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- rXDif2 rXJmb1) 2065 0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- rXDif2 rXJmb1) 0 0)
             )
        )
        (entmake (quote ((0 . "SEQEND"))))
        (setq BlockName3D (entmake '((0 . "ENDBLK"))))
      )

      ;;-------------Version non-Assistee pour automation des facades
      (defun ElevWnMk ()
        (TraceIsOn ".ElevWnMk")
        (setq pHing    (trans pHinge 0 1)
          pMCote   (trans pMCote 0 1)
          pCoteO   (trans pCoteO 0 1)
          pP4      (trans pP4 0 1)
          pMid13   (mid_pt pHing pMCote)
          pMid24   (mid_pt pCoteO pP4)
          rXCheck1 (car pHing)                    ;x value for p1
          rYCheck1 (cadr pHing)                   ;y   for p1
          rXCheck2 (car pMCote)                   ;x value for p2
          rYCheck2 (cadr pMCote)                  ;y value for p2
          rXCheck3 (car pCoteO)                   ;x   for p3
          rYCheck3 (cadr pCoteO)                  ;y value for p3
          rXJmb1   (round2dec (BSide aAlph2 rEpaiss) 100)     ;Delta X for cadre thk
          rHaut    (- 2100.0 sTr4b)               ;Hauteur jusqu'a l'appui
          rYCheck4 (cadr pMid13)                  ;y for pt between p1 and p3
          rYCheck5 (cadr pMid24)                  ;y for pt between p2 and p4
        )

        (if (> rXCheck2 rXCheck1)
          (progn                          ;
            (princ "Charniere a gauche ")
            (setq bNorm T)
          )
          (progn                          ;
            (princ "Charniere a droite ")
            (setq bNorm nil)
          )
        )

        (if (> rYCheck5 rYCheck4)
          (progn                          ;p1 et p2 de ce cote
            (princ "Feinster de ce cote ")
            (setq Charniere pHing  bDispSill nil)
          )
          (progn                          ;p1 et p2 de l'autre cote
            (princ "Feinster de l'autre cote ")
            (setq Charniere pHing  bDispSill T)
          )
        )

        (cond
         ((and (= bDispSill T)(= bNorm T))                ;si fenetre est en dela, prendre pCoteO
          (setq rCadre (* -1 rXDif1))
         )
         ((and (= bDispSill T)(= bNorm nil))
          (setq rCadre rXDif1)
         )
         (T (setq rCadre 0))
        )

        (setq pNewWt   (PerpPt2Line Charniere (trans pDepart 0 1) (trans pArriv 0 1)) ;0=World (WCS), 1=User (current UCS)
          pNewW    (polar pNewWt hshpi90 rHaut)
          pOnLin   (PerpPt2Line pHing (trans pN1 0 1) (trans pN2 0 1))        ;0=World (WCS), 1=User (current UCS)
          rHContrl (distance pHing pOnLin)
          lKeep    (list (cons rHContrl (list pHing)))
          pOnLin   (PerpPt2Line pCoteO (trans pN1 0 1) (trans pN2 0 1))       ;0=World (WCS), 1=User (current UCS)
          rOContrl (distance pCoteO pOnLin)
          lKeep    (append (list (cons rOContrl (list pCoteO))) lKeep)
          pOnLin   (PerpPt2Line pMCote (trans pN1 0 1) (trans pN2 0 1))       ;0=World (WCS), 1=User (current UCS)
          rMContrl (distance pMCote pOnLin)
          lKeep    (append (list (cons rMContrl (list pMCote))) lKeep)
          pOnLin   (PerpPt2Line pP4 (trans pN1 0 1) (trans pN2 0 1))      ;0=World (WCS), 1=User (current UCS)
          rPContrl (distance pP4 pOnLin)
          lKeep    (append (list (cons rPContrl (list pP4))) lKeep)
          iKeep    (car (vl-sort-i lkeep (function (lambda (e1 e2) (> (car e1) (car e2))))))  ;Index du plus grand au plus petit
          pKept    (cadr (nth iKeep lKeep))                   ;Point a garder
          rDif5    (- (car pKept)(car pNewWt))
        )

        (princ aBlokRot)(princ "|")(princ rDif5)

        (if (>= aBlokRot 90)                 ;angle > 90
          (progn  (princ "o2o")
          (if (= bAutreCote T)                ;si de l'autre cote
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor -1  rDif5 (* -1 rDif5))
                (setq NxFactor 1)
               )
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor 1)
                (setq NxFactor -1  rDif5 (* -1 rDif5))
               )
              )
          )
          (progn  (princ "o1o")               ;angle < 90
          (if (= bAutreCote T)                ;si de l'autre cote
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor 1)
                (setq NxFactor -1  rDif5 (* -1 rDif5))
               )
           (if (= bNorm T)                ;et charniere a gauche
                (setq NxFactor 1)
                (setq NxFactor -1  rDif5 (* -1 rDif5))
               )
              )
          )
        )

        ;;;(entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        ;;;(setq BlockName3W (entmake '((0 . "ENDBLK"))))
      )

      ;-----------------------------
      (defun DrawSecs (/ sPorteOuiNon)
        (TraceIsOn ".DrawSecs")
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (WsFlashMess2 (strcat "\nDraw Section." copyright)
              (strcat "\nDraw Section." copyright)
              (strcat "\nDraw Section." copyright)
              (strcat "\nDraw Section." copyright)
        )
        (GetOpeningXD)

        (if (= sPorteOuiNon "WINDPL")
          (progn
             (setq ssdoor  (ssadd ODrNam))
             (setq ent1    (objsel "LINE or LWPOLYLINE" "Select a reference line" nil)
               rY2     (cadr (dxf 10 (entget (car ent1))))    ;get y value
               rEpaiss 40
               rXDif1  (BSide aA1 rMurThk)            ;Delta X for wall thk
               rXDif2  (BSide aA3 OffDist)            ;Delta X for Opening
             )

            (if (minusp rXDif2)(setq xFactor -1)(setq xFactor 1))

            (if (= sPorteOuiNon "DOORPL")
             (progn
             (GetDrINI)
             (ElevDrMk)
             (entmake (list (cons 0 "INSERT")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    ;;         (cons 100 "AcDbInsertBlock")
                    (cons 2 BlockName5DS)
                    (cons 10 pNewD)
                    (cons 41 xFactor)
      ;             (cons 42 YFactor)
                    (list 210 0.0 0.0 1.0)
                  )
             )
      ;      (DoorElevTagSet sAppNameAA (entlast))
                 )
                )
                (if (= sPorteOuiNon "WINDPL")
             (progn
             (GetWnINI)
             (SectionWnMk)
             (entmake (list (cons 0 "INSERT")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    ;;         (cons 100 "AcDbInsertBlock")
                    (cons 2 BlockName5WS)
                    (cons 10 pNewW)
                    (cons 41 xFactor)
      ;             (cons 42 YFactor)
      ;             (cons 50 aA3)
                    (list 210 0.0 0.0 1.0)
                  )
             )
             (WindSecTagSet sAppNameAA (entlast))
             (entmake (list (cons 0 "INSERT")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    ;;         (cons 100 "AcDbInsertBlock")
                    (cons 2 BlockName4)
                    (cons 10 pNewW)
                    (cons 41 xFactor)
      ;             (cons 42 YFactor)
      ;             (cons 50 aA3)
                    (list 210 0.0 0.0 1.0)
                  )
             )
             (WallSecTagSet sAppNameAA (entlast))
                 (setq sTr4b nil)
                 )
                )
          )
         (princ "\nCannot generate a section from this object. ")
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;----------------------
      (defun SectionWnMk ()
        (TraceIsOn ".SectionWnMk")
        (setq rXCheck1 (car pHinge)       ;x value for p1
          rYCheck1 (cadr pHinge)      ;y   for p1
          rXCheck2 (car pMCote)       ;x value for p2
          rYCheck2 (cadr pMCote)      ;y value for p2
          rXCheck3 (car pCoteO)       ;x   for p3
          rYCheck3 (cadr pCoteO)      ;y value for p3
          rXJmb1   (abs (BSide aA3 rEpaiss)) ;Delta X for cadre thk
          rXDif1   (abs rXDif1)
          rXDif2   (abs rXDif2)
          rHaut    (- 2100.0 sTr4b)
        )
        (if (> rYCheck3 rYCheck1)
          (progn              ;p1 et p2 de ce cote
                          ;(princ "Feinster de ce cote")
            (setq pNewW     (list (car pHinge) (+ rY2 rHaut) 0.0)
              Charniere pHinge
              bDispSill nil
              xFactor   -1
            )
            (if (and (> rXCheck3 rXCheck1) (< rXCheck3 rXCheck2))
          (setq Gauche T
                Droit nil
          )               ;p1 et p3 = gauche
          (setq Gauche nil
                Droit T
          )               ;p2 et p4 = droit
            )
          )
          (progn              ;p1 et p2 de l'autre cote
                          ;(princ "de l'autre cote")
            (setq pNewW     (list (car pCoteO) (+ rY2 rHaut) 0.0)
              Charniere pCoteO
              bDispSill T
              xFactor   1
            )
            (if (and (> rXCheck1 rXCheck3) (< rXCheck1 rXCheck2))
          (setq Gauche T
                Droit nil
          )               ;p1 et p3 = gauche
          (setq Gauche nil
                Droit T
          )               ;p2 et p4 = droit
            )
          )
        )
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0))) ;Mur haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sBrkWall1Lyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 160 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- 160 rMurThk) sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- 160 rMurThk) (+ 300 sTr4b))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 (+ 300 sTr4b))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        (if (and (= bThreSil T) (= bDispSill T))
          (progn (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDbLine")
                    (list 10 (- 160 rMurThk) sTr4b 0)
                    (list 11 (- 160 rMurThk) -29.615 0)
                    (list 210 0 0 1)
                  )
             )
                          ;Mur bas
             (entmake (list (cons 0 "POLYLINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDb2dPolyline")
                    (cons 66 1)
                    (list 10 0 0 0)
                    (list 210 0 0 1)
                  )
             )
             (entmake (list (cons 0 "VERTEX")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDbVertex")
                    (cons 100 "AcDb2dVertex")
                    (list 10 160 0 0)
                  )
             )
             (entmake (list (cons 0 "VERTEX")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDbVertex")
                    (cons 100 "AcDb2dVertex")
                    (list 10 160 (- sTr4b 2100) 0)
                  )
             )
             (entmake (list (cons 0 "VERTEX")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDbVertex")
                    (cons 100 "AcDb2dVertex")
                    (list 10 (- 160 rMurThk) (- sTr4b 2100) 0)
                  )
             )
             (entmake (list (cons 0 "VERTEX")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 8 sBrkWall1Lyr)
                    (cons 100 "AcDbVertex")
                    (cons 100 "AcDb2dVertex")
                    (list 10 (- 160 rMurThk) -110 0)
                  )
             )
             (entmake (quote ((0 . "SEQEND"))))
          )
          (progn              ;S'il n'y a pas d'allege
            (entmake (list (cons 0 "LINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sBrkWall1Lyr)
                   (cons 100 "AcDbLine")
                   (list 10 (- 160 rMurThk) sTr4b 0)
                   (list 11 (- 160 rMurThk) 0 0)
                   (list 210 0 0 1)
                 )
            )
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sBrkWall1Lyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 160 0)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- 160 rMurThk) 0)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (- 160 rMurThk) (- sTr4b 2100))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 160 (- sTr4b 2100))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
          )
        )
        (setq BlockName4 (entmake '((0 . "ENDBLK"))))
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0))) ;Frame
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
                          ;Cadre du bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
                          ;Cadre du haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 (- sTr4b rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 160 sTr4b)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        (if (and (= bThreSil T) (= bDispSill T))
          (progn              ;Sill
            (entmake (list (cons 0 "POLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSillLyr)
                   (cons 6 "Continuous")
                   (cons 100 "AcDb2dPolyline")
                   (cons 66 1)
                   (list 10 0 0 0)
                   (cons 70 1)
                   (list 210 0 0 1)
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSillLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10 -130 -35 0)
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSillLyr)
                   (cons 6 "Continuous")
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10 -130 -110 0)
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSillLyr)
                   (cons 6 "Continuous")
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10 0 -110 0)
                 )
            )
            (entmake (list (cons 0 "VERTEX")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSillLyr)
                   (cons 100 "AcDbVertex")
                   (cons 100 "AcDb2dVertex")
                   (list 10 0 0 0)
                 )
            )
            (entmake (quote ((0 . "SEQEND"))))
          )
        )
        (if (= LWn "FW")          ;For a fixed type
          (progn              ;Montant du bas
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 100 (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 100 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 60 rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 60 (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Montant du haut
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 100 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 100 (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 60 (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 60 (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Sash
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSashLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 60 (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 60 (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 100 (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 100 (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Glass
            (entmake (list (cons 0 "LINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sGlazeLyr)
                   (cons 100 "AcDbLine")
                   (list 10 (* 2 rEpaiss) (- sTr4b (* 2 rEpaiss)) 0)
                   (list 11 (* 2 rEpaiss) (* 2 rEpaiss) 0)
                   (list 210 0 0 1)
                 )
            )
          )
          (progn              ;Montant du bas gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rEpaiss (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Montant du haut gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 rEpaiss (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Sash gauche
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSashLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rEpaiss) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 rEpaiss (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Glass gauche
            (entmake (list (cons 0 "LINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sGlazeLyr)
                   (cons 100 "AcDbLine")
                   (list 10 (* 1.5 rEpaiss) (- sTr4b (* 2 rEpaiss)) 0)
                   (list 11 (* 1.5 rEpaiss) (* 2 rEpaiss) 0)
                   (list 210 0 0 1)
                 )
            )
                          ;Montant du bas droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 rEpaiss) rEpaiss)
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 rEpaiss) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Montant du haut droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sFrameLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 2 rEpaiss) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 rEpaiss) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 rEpaiss) (- sTr4b rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Sash droit
            (entmake (list (cons 0 "LWPOLYLINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sSashLyr)
                   (cons 100 "AcDbPolyline")
                   (cons 90 4)
                   (cons 70 1)
                   (cons 43 0.0)
                   (cons 38 0.0)
                   (cons 39 0.0)
                   (list 10 (* 3 rEpaiss) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 3 rEpaiss) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 10 (* 2 rEpaiss) (- sTr4b (* 2 rEpaiss)))
                   (cons 40 0.0)
                   (cons 41 0.0)
                   (cons 42 0.0)
                   (list 210 0 0 1)
                 )
            )
                          ;Glass droit
            (entmake (list (cons 0 "LINE")
                   (cons 100 "AcDbEntity")
                   (cons 67 0)
                   (cons 410 "Model")
                   (cons 8 sGlazeLyr)
                   (cons 100 "AcDbLine")
                   (list 10 (* 2.5 rEpaiss) (- sTr4b (* 2 rEpaiss)) 0)
                   (list 11 (* 2.5 rEpaiss) (* 2 rEpaiss) 0)
                   (list 210 0 0 1)
                 )
            )
          )
        )
        (setq BlockName5WS (entmake '((0 . "ENDBLK"))))
      )

      ;-----------------------------
      (defun TimberJamb ()
        (TraceIsOn ".TimberJamb")
        (setq rUnit1   5.0
              pArRight (list (* 7 rUnit1) 0.0 0.0)
        )
        (setq JLP1 (list (* 10 rUnit1) rJambWidth 0.0))
        (setq JLP2 (list (* 10 rUnit1) (atoi LEAFWIDT) 0.0))
        (setq JLP3 (list (* 7 rUnit1) (atoi LEAFWIDT) 0.0))
        (setq JLP5 (list 0.0 0.0 0.0))
        (setq JLP6 (list 0.0 rJambWidth 0.0))
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JLP1)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JLP2)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JLP3)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 pArRight)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JLP5)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JLP6)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))

        (setq JRP1 (list (- OFFDIST (* 10 rUnit1)) rJambWidth 0.0))
        (setq JRP2 (list (- OFFDIST (* 10 rUnit1)) (atoi LEAFWIDT) 0.0))
        (setq JRP3 (list (- OFFDIST (* 7 rUnit1)) (atoi LEAFWIDT) 0.0))
        (setq JRP4 (list (- OFFDIST (* 7 rUnit1)) 0.0 0.0))
        (setq JRP5 (list OFFDIST 0.0 0.0))
        (setq JRP6 (list OFFDIST rJambWidth 0.0))
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 1)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP1)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP2)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP3)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP4)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP5)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (cons 10 JRP6)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
      )

      (defun SteelJamb ()
        (TraceIsOn ".SteelJamb")
        (setq rUnit1   5.0
          pArRight (list (* 7 rUnit1) (* -2 rUnit1) 0.0)
        )

        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 rUnit1 0.0 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 0.0 0.0 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 0.0 (* -2 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (* 7 rUnit1) (* -2 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (* 7 rUnit1) (* 4 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (* 10 rUnit1) (* 4 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (* 10 rUnit1) (- rJambWidth (* 4 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (* 7 rUnit1) (- rJambWidth (* 4 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (* 7 rUnit1) (+ rJambWidth (* 2 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 0.0 (+ rJambWidth (* 2 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 0.0 (+ rJambWidth 0.0) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 rUnit1 (+ rJambWidth 0.0) 0.0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
        (setq JRP4 (list (- OFFDIST (* 7 rUnit1)) (* -2 rUnit1) 0.0))
        (entmake (list (cons 0 "POLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDb2dPolyline")
               (cons 66 1)
               (list 10 0.0 0.0 0.0)
               (cons 70 0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST rUnit1) 0.0 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 OFFDIST 0.0 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 OFFDIST (* -2 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST (* 7 rUnit1)) (* -2 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST (* 7 rUnit1)) (* 4 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST (* 10 rUnit1)) (* 4 rUnit1) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST (* 10 rUnit1)) (- rJambWidth (* 4 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST (* 7 rUnit1)) (- rJambWidth (* 4 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST (* 7 rUnit1)) (+ rJambWidth (* 2 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 OFFDIST (+ rJambWidth (* 2 rUnit1)) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 OFFDIST (+ rJambWidth 0.0) 0.0)
             )
        )
        (entmake (list (cons 0 "VERTEX")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sJambLyr)
               (cons 100 "AcDbVertex")
               (cons 100 "AcDb2dVertex")
               (list 10 (- OFFDIST rUnit1) (+ rJambWidth 0.0) 0.0)
             )
        )
        (entmake (QUOTE ((0 . "SEQEND"))))
      )

      ;-----------------------------
      (defun GetOpeningXD (/ aDiff1)
        (TraceIsOn ".GetOpeningXD")
        (princ "\nSelect a door or window: ")
        (setq doorpick (ssget "+.:S" '((0 . "INSERT"))))

        (if (/= nil doorpick)
         (progn
        (if (/= nil (AAXDataGet (ssname doorpick 0) sAppNameAA))
          (setq sPorteOuiNon
             (cdr
               (nth 0 (AAXDataGet (ssname doorpick 0) sAppNameAA))
             )
          )                   ;Une porte? Vue en plan?
        )

        (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
          (progn
            (setq ODeNom   (ssname doorpick 0)
              rJambWidth (read (cdr (nth 1 (AAXDataGet ODeNom sAppNameAA))))
              sDrHan2  (read (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA))))                ;Database handle
              LWn      (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))                   ;Type de fenetre
              jambprefix (read (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA))))              ;Door ONLY jamb***********
              sTr4b    (read (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA))))                ;Window ONLY height***********
              OFFDIST  (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))))                ;Door width
              pHinge   (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))                   ;Hinge
              pMCote   (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))                   ;meme cote
              pCoteO   (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))                   ;cote oppose
              PercOpen  (cdr (nth 9 (AAXDataGet ODeNom sAppNameAA)))
              rMurThk  (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))                  ;Wall thk
              bHead    (read (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA))))               ;Header/HeaderW present?
              bThreSil (read (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA))))               ;Threshold/Sill present?
              sTrSi    (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA)))
              bTransom (read (cdr (nth 13 (AAXDataGet ODeNom sAppNameAA))))               ;Transom present?
              sSeuil4  (cdr (nth 14 (AAXDataGet ODeNom sAppNameAA)))                  ;Transom after what height?
              sTofIns  (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))                  ;Inseree a quelle heure?
              ODeNom   (entget ODeNom)
              lDoor    ODeNom
              aA1      (round2dec (angle pHinge pCoteO) 100000)
              aA3      (round2dec (angle pHinge pMCote) 100000)
              aDiff1   (- aA1 aA3)
              ;;   pPicked  (cadr doorpick)
              pCnr1    (list 0 0 0)
              ODrNam   (dxf -1 lDoor)
              xFactor  (dxf 41 lDoor)
              yFactor  (dxf 42 lDoor)
              aRot     (dxf 50 lDoor)

              sDrHand  (etos sDrHan2)
              sWnHand  (etos sDrHan2)

              str2     LWn
              str2W    LWn

              str3a    (rtos OFFDIST)
              sTrW3a   (rtos OFFDIST)

              p1       pHinge
              p2       pMCote
              p3       pCoteO

              bHeader  bHead
              bHeaderW bHead

              bThreshold bThreSil
            )

            (if (= sPorteOuiNon "DOORPL")
            (setq sJx      (etos jambprefix))             ;Si c'est une porte
            (setq sSb      (etos sTr4b)                   ;Sinon hauteur de la fenetre
              bSill    (read (car (strfind strsi "/")))
              bSillHat (read (cadr (strfind strsi "/")))
            )
            )

            (if (minusp yFactor)
          (if (minusp xFactor)
            (setq sOuvreTo "R")
            (setq sOuvreTo "L")
          )
          (if (minusp xFactor)
            (setq sOuvreTo "L")
            (setq sOuvreTo "R")
          )
            )
            (if (minusp aDiff1)
          (setq aDiff1 (+ HshPi360 aDiff1))
            )
            (if (= (round2dec HshPi90 100000000)
               (round2dec aDiff1 100000000)
            )
          (Setq sOuvreA "Right")
          (Setq sOuvreA "Left")
            )
            (setq pP4 (polar pMCote aA1 rMurThk))
          )
        )

          )
        )

      )

      ;-----------------
      ;---Door Main Prg.
      ;-----------------
      (defun GetDrINI ()             ;initiate
        (TraceIsOn ".GetDrINI")
        (if (= bMetUnit T)           ;Section metrique
          (setq file1 (strcat Chemin "MetricOpenWidth.ini"))
          (setq file1 (strcat Chemin "ImperialOpenWidth.ini"))
        )
        (setq
          sMulti     (dos_getini "Door Settings" "MultiPrompt" file1)
          sJambTyp   (dos_getini "Door Settings" "JambType" file1)
          sOpenAng   (dos_getini "Door Settings" "OpenAng" file1)
          lSngM1     (dos_getini "Single Doors-Manuf1" "Width" file1)         ;Width List for Single... AABuild108
          lSngM2     (dos_getini "Single Doors-Manuf2" "Width" file1)         ;Width List for Single... AABuild108
          lSngM3     (dos_getini "Single Doors-Manuf3" "Width" file1)         ;Width List for Single... AABuild108

          lSldM1     (dos_getini "Sliding Doors-Manuf1" "Width" file1)            ;Width List for Sliding S-F
          lSldM2     (dos_getini "Sliding Doors-Manuf2" "Width" file1)            ;Width List for Sliding
          lSldM3     (dos_getini "Sliding Doors-Manuf3" "Width" file1)            ;Width List for Sliding

          lFSFM1     (dos_getini "Sliding Doors FSF-Manuf1" "Width" file1)        ;Width List for Sliding F-S-F
          lFSSFM1    (dos_getini "Sliding Doors FSSF-Manuf1" "Width" file1)       ;Width List for Sliding F-S-S-F

          lWid2      (dos_getini "Double Doors" "Width" file1)                ;Width List for Double Doors
          lWid3      (dos_getini "Folding Doors" "Width" file1)               ;Width List for Folding Doors
        )

        (setq sManuf1 (car  (strfind lSngM1 "-"))                     ;"Single Doors-Manuf1"
          lWid1   (cadr (strfind lSngM1 "-"))
          lDoorWidth1 nil  lDoorWidth1 (DeStrAListe lWid1 "|")

          lDoorWidth1s nil lDoorWidth1s (DeStrAListe (cadr (strfind lSldM1 "-")) "|") ;"Sliding Doors-Manuf1"

          sManuf2 (car (strfind lSngM2 "-"))                      ;"Single Doors-Manuf2"
          sManuf3 (car (strfind lSngM3 "-"))                      ;"Single Doors-Manuf3"

          lDFSFWidth1 nil lDFSFWidth1 (DeStrAListe (cadr (strfind lFSFM1 "-")) "|")   ;"Sliding Doors FSF-Manuf1"
          lDFSSFWidth1 nil lDFSSFWidth1 (DeStrAListe (cadr (strfind lFSSFM1 "-")) "|")    ;"Sliding Doors FSSF-Manuf1"

          lDoorWidth2 nil  lDoorWidth2 (DeStrAListe lWid2 "|")                ;"Double Doors"
          lDoorWidth3 nil  lDoorWidth3 (DeStrAListe lWid3 "|")                ;"Folding Doors"
        )

        (if (= sJambTyp nil) (setq sJambTyp "S") )
        (if (/= sMulti "T")  (setq bMulti nil)(setq bMulti T))

        (if (= sOpenAng nil) (setq sOpenAng "90") )
        (if (or (< (atoi sOpenAng) 0)
                (> (atoi sOpenAng) 180))
            (setq sOpenAng "90")
        )

        (setq sOpenAng_c sOpenAng)

        (if (= bMetUnit T)
         (if (= leafwidt nil) (setq leafwidt "30") )
         (if (= leafwidt nil) (setq leafwidt "30") )
        )
        (setq leafwidt_c leafwidt)

        (if (= sJambTyp "T")
          (setq jambprefix "T"  ajambtype "Timber Jamb")
          (setq jambprefix "S"  ajambtype "Steel Jamb")
        )
        (princ)
      )

      (defun GetWnINI ()                                  ;initiate
        (TraceIsOn ".GetWnINI")

        (if (= bMetUnit T)                                    ;Section metrique
          (setq file1 (strcat Chemin "MetricOpenWidth.ini"))
          (setq file1 (strcat Chemin "ImperialOpenWidth.ini"))
        )
        (setq lWAwgM1 (dos_getini "Awning Windows-Manuf1" "Width" file1)            ;Width List for Awning... AA1.3b100
            lWAwgM2 (dos_getini "Awning Windows-Manuf2" "Width" file1)            ;Width List for Awning... AA1.3b100
            lWAwgM3 (dos_getini "Awning Windows-Manuf3" "Width" file1)            ;Width List for Awning... AA1.3b100
            lHAwgM1 (dos_getini "Awning Windows-Manuf1" "Height" file1)           ;Height List for Awning... AA1.3b100
            lHAwgM2 (dos_getini "Awning Windows-Manuf2" "Height" file1)           ;Height List for Awning... AA1.3b100
            lHAwgM3 (dos_getini "Awning Windows-Manuf3" "Height" file1)           ;Height List for Awning... AA1.3b100
            lTAwgM1 (dos_getini "Awning Windows-Manuf1" "Thresh" file1)           ;Thresh List for Awning... AA1.3b100
            lTAwgM2 (dos_getini "Awning Windows-Manuf2" "Thresh" file1)           ;Thresh List for Awning... AA1.3b100
            lTAwgM3 (dos_getini "Awning Windows-Manuf3" "Thresh" file1)           ;Thresh List for Awning... AA1.3b100

            lWCstM1 (dos_getini "Casement Windows-Manuf1" "Width" file1)          ;Width List for Casement... AA1.3b100
            lWCstM2 (dos_getini "Casement Windows-Manuf2" "Width" file1)          ;Width List for Casement... AA1.3b100
            lWCstM3 (dos_getini "Casement Windows-Manuf3" "Width" file1)          ;Width List for Casement... AA1.3b100
            lHCstM1 (dos_getini "Casement Windows-Manuf1" "Height" file1)         ;Height List for Casement... AA1.3b100
            lHCstM2 (dos_getini "Casement Windows-Manuf2" "Height" file1)         ;Height List for Casement... AA1.3b100
            lHCstM3 (dos_getini "Casement Windows-Manuf3" "Height" file1)         ;Height List for Casement... AA1.3b100
            lTCstM1 (dos_getini "Casement Windows-Manuf1" "Thresh" file1)         ;Thresh List for Casement... AA1.3b100
            lTCstM2 (dos_getini "Casement Windows-Manuf2" "Thresh" file1)         ;Thresh List for Casement... AA1.3b100
            lTCstM3 (dos_getini "Casement Windows-Manuf3" "Thresh" file1)         ;Thresh List for Casement... AA1.3b100

            lWDHgM1 (dos_getini "Double Hung Windows-Manuf1" "Width" file1)       ;Width List for Double Hung... AA1.3b100
            lWDHgM2 (dos_getini "Double Hung Windows-Manuf2" "Width" file1)       ;Width List for Double Hung... AA1.3b100
            lWDHgM3 (dos_getini "Double Hung Windows-Manuf3" "Width" file1)       ;Width List for Double Hung... AA1.3b100
            lHDHgM1 (dos_getini "Double Hung Windows-Manuf1" "Height" file1)      ;Height List for Double Hung... AA1.3b100
            lHDHgM2 (dos_getini "Double Hung Windows-Manuf2" "Height" file1)      ;Height List for Double Hung... AA1.3b100
            lHDHgM3 (dos_getini "Double Hung Windows-Manuf3" "Height" file1)      ;Height List for Double Hung... AA1.3b100
            lTDHgM1 (dos_getini "Double Hung Windows-Manuf1" "Thresh" file1)      ;Thresh List for Double Hung... AA1.3b100
            lTDHgM2 (dos_getini "Double Hung Windows-Manuf2" "Thresh" file1)      ;Thresh List for Double Hung... AA1.3b100
            lTDHgM3 (dos_getini "Double Hung Windows-Manuf3" "Thresh" file1)      ;Thresh List for Double Hung... AA1.3b100

            lWFxdM1 (dos_getini "Fixed Windows-Manuf1" "Width" file1)         ;Width List for Fixed... AA1.3b100
            lWFxdM2 (dos_getini "Fixed Windows-Manuf2" "Width" file1)         ;Width List for Fixed... AA1.3b100
            lWFxdM3 (dos_getini "Fixed Windows-Manuf3" "Width" file1)         ;Width List for Fixed... AA1.3b100
            lHFxdM1 (dos_getini "Fixed Windows-Manuf1" "Height" file1)            ;Height List for Fixed... AA1.3b100
            lHFxdM2 (dos_getini "Fixed Windows-Manuf2" "Height" file1)            ;Height List for Fixed... AA1.3b100
            lHFxdM3 (dos_getini "Fixed Windows-Manuf3" "Height" file1)            ;Height List for Fixed... AA1.3b100
            lTFxdM1 (dos_getini "Fixed Windows-Manuf1" "Thresh" file1)            ;Thresh List for Fixed... AA1.3b100
            lTFxdM2 (dos_getini "Fixed Windows-Manuf2" "Thresh" file1)            ;Thresh List for Fixed... AA1.3b100
            lTFxdM3 (dos_getini "Fixed Windows-Manuf3" "Thresh" file1)            ;Thresh List for Fixed... AA1.3b100

            lWSLdM1 (dos_getini "Sliding Windows-Manuf1" "Width" file1)           ;Width List for Sliding... AA1.3b100
            lWSLdM2 (dos_getini "Sliding Windows-Manuf2" "Width" file1)           ;Width List for Sliding... AA1.3b100
            lWSLdM3 (dos_getini "Sliding Windows-Manuf3" "Width" file1)           ;Width List for Sliding... AA1.3b100
            lHSLdM1 (dos_getini "Sliding Windows-Manuf1" "Height" file1)          ;Height List for Sliding... AA1.3b100
            lHSLdM2 (dos_getini "Sliding Windows-Manuf2" "Height" file1)          ;Height List for Sliding... AA1.3b100
            lHSLdM3 (dos_getini "Sliding Windows-Manuf3" "Height" file1)          ;Height List for Sliding... AA1.3b100
            lTSLdM1 (dos_getini "Sliding Windows-Manuf1" "Thresh" file1)          ;Thresh List for Sliding... AA1.3b100
            lTSLdM2 (dos_getini "Sliding Windows-Manuf2" "Thresh" file1)          ;Thresh List for Sliding... AA1.3b100
            lTSLdM3 (dos_getini "Sliding Windows-Manuf3" "Thresh" file1)          ;Thresh List for Sliding... AA1.3b100
        )

        (setq sManuf1  (car  (strfind lWFxdM1 "-"))
          lFWin1W  (cadr (strfind lWFxdM1 "-"))                       ;Fixed Windows
          lFxdWWidth nil  lFxdWWidth (DeStrAListe lFWin1W "|")
          lFWin1H  (cadr (strfind lHFxdM1 "-"))
          lFxdWHeight nil  lFxdWHeight (DeStrAListe lFWin1H "|")

          lSWin1W  (cadr (strfind lWSLdM1 "-"))                       ;Sliding Windows
          lSLdWWidth nil  lSLdWWidth (DeStrAListe lSWin1W "|")
          lSWin1H  (cadr (strfind lHSLdM1 "-"))
          lSLdWHeight nil  lSLdWHeight (DeStrAListe lSWin1H "|")
          lSLdWWidt3 nil

          lDWin1W  (cadr (strfind lWDHgM1 "-"))                       ;Double Hung Windows
          lDHgWWidth nil  lDHgWWidth (DeStrAListe lDWin1W "|")
          lDWin1H  (cadr (strfind lHDHgM1 "-"))
          lDHgWHeight nil  lDHgWHeight (DeStrAListe lDWin1H "|")
          lDHgWWidt3 nil

          ;;;   sManuf2 (car (strfind lWFxdM2 "-"))
          ;;;   sManuf3 (car (strfind lWFxdM3 "-"))
        )

        (if (= sOpenAng nil)    (setq sOpenAng "90") )
        (TraceIsOn "GetWnINI.")
        (princ)
      )

      (TraceIsOn ".19g")

      ;;-----------------------------------
      ;A layer name (group code 1003).
      ;An entity handle (group code 1005).
      ;A 3D World space position (group code 1011).
      ;A 3D World space displacement (group code 1012).
      ;A 3D World space direction (group code 1013).
      (defun SetDoorInfoTag (apid en bDrapo / )
        (TraceIsOn ".SetDoorInfoTag")
        (setq jambprefix sJx)
        (setq t0 "DOORPL"
          t1 (rtos rJambWidth 2 0)            ;
          t2 sDrHand                  ;Handle number
          t3 str2                     ;Door type
          t4 jambprefix                   ;Type of jamb
          t5 str3a                    ;Opening width
          t6 p1                       ;Hinge
          t7 p2                       ;Meme cote que la charniere
          t8 p3                       ;Mur oppose
          t9 PercOpen                 ;OpenAng
          t10 rMurThk
          t11 (etos bHeader)              ;Header?
          t12 (etos bThreshold)               ;Threshold?
        )
        (if (= bDrapo nil)
          (setq t15 (substr (rtos (getvar "cdate") 2 8) 5 12));Heure, pour ajouter aux ouvertures.
          (setq t15 sTofIns)
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1000 t1)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                    ;Handle number
            (cons 1000 t3)                    ;Door type
            (cons 1000 t4)                    ;Type of jamb
            (cons 1000 t5)                    ;Opening width
            (cons 1011 t6)                    ;3D World space position
            (cons 1011 t7)                    ;3D World space position
            (cons 1011 t8)                    ;3D World space position
            (cons 1000 t9)                    ;OpenAng                    /integre 1070
            (cons 1040 t10)                   ;epaisseur reelle du mur
            (cons 1000 t11)                   ;is there a Header?
            (cons 1000 t12)                   ;is there a Threshold?
            (cons 1000 "Dummy")               ;Dummy
            (cons 1000 "Dummy")               ;Dummy
            (cons 1000 t15)                   ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      (defun SetWindInfoTag (apid en bDrapo / )
        (TraceIsOn ".SetWindInfoTag")
        (setq sTr4b sSb)
        (setq t0 "WINDPL"
          t1 (rtos rJambWidth 2 0)            ;
          t2 sWnHand                  ;Handle number
          t3 str2W                    ;Wind type
          t4 sTr4b                    ;Opening Height
          t5 sTrW3a                   ;Opening width
          t6 p1                       ;Hinge
          t7 p2                       ;Meme cote que la charniere
          t8 p3                       ;Mur oppose
          t9 PercOpen                 ;OpenAng - Number of panes or points??
          t10 rMurThk
          t11 (etos bHeaderW)             ;is there a Header?
          t12 (strcat (etos bSill) "/" (etos bSillHat))   ;is there a Sill? and Hatched or Outline?
          t13 (etos bTransom)             ;is there a Transom?
          t14 sSeuil4                 ;Transom after what height?
        )
        (if (= bDrapo nil)
          (setq t15 (substr (rtos (getvar "cdate") 2 8) 5 12));Heure, pour ajouter aux ouvertures.
          (setq t15 sTofIns)
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1000 t1)                ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                ;Handle number
            (cons 1000 t3)                ;Wind type
            (cons 1000 t4)                ;Opening Height
            (cons 1000 t5)                ;Opening width
            (cons 1011 t6)                ;3D World space position
            (cons 1011 t7)                ;3D World space position
            (cons 1011 t8)                ;3D World space position
            (cons 1000 t9)                ;OpenAng                /integre 1070
            (cons 1040 t10)               ;epaisseur reelle du mur
            (cons 1000 t11)               ;Header?
            (cons 1000 t12)               ;Sill? Hatched or Outline?
            (cons 1000 t13)               ;Transom?
            (cons 1000 t14)               ;Add transom if higher
            (cons 1000 t15)               ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      (defun DoorElevTagSet (apid en / )
        (TraceIsOn ".DoorElevTagSet")
        (setq t0 "DOOREL"
          t2 (etos sDrHan2)               ;Handle number
          t3 LWn
          t6 pNewD                    ;Insert Pt
          t15 sTofIns
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                    ;Handle number
            (cons 1000 t3)                    ;Door/Win type
            (cons 1011 t6)                    ;3D World space position
            (cons 1000 t15)                   ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      (defun WindElevTagSet (apid en / )
        (TraceIsOn ".WindElevTagSet")
        (setq t0  "WINDEL"
          t2  (etos sDrHan2)              ;Handle number
          t3  LWn
          t4  (etos sTr4b)
          t5  (etos OffDist)
          t6  pNewW                   ;
          t13 (etos bTransom)             ;is there a Transom?
          t14 sSeuil4                 ;Transom after what height?
          t15 sTofIns
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                    ;Handle number
            (cons 1000 t3)                    ;Door/Win type
            (cons 1000 t4)
            (cons 1000 t5)
            (cons 1011 t6)                    ;3D World space position
            (cons 1000 t13)
            (cons 1000 t14)                   ;Add transom if higher
            (cons 1000 t15)                   ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      ;;-----------------------------------
      (defun WindSecTagSet (apid en / )
        (TraceIsOn ".WindSecTagSet")
        (setq t0 "WINDSC"
          t2 (etos sDrHan2)               ;Handle number
          t3 LWn
          t4 (etos sTr4b)
          t5 (etos OffDist)
          t6 pNewW                    ;
          t15 sTofIns
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                    ;Handle number
            (cons 1000 t3)                    ;Door/Win type
            (cons 1000 t4)
            (cons 1000 t5)
            (cons 1011 t6)                    ;3D World space position
            (cons 1000 t15)                   ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      ;;-----------------------------------
      (defun WindBarsTagSet (apid en bDrapo / )
        (TraceIsOn ".WindBarsTagSet")
        (setq t0 "WINDBR"
          t2  (etos sDrHan2)              ;Handle number
          t15 sTofIns
        )
        (if (= bDrapo nil)
          (setq t15 (substr (rtos (getvar "cdate") 2 8) 5 12));Heure, pour ajouter aux ouvertures.
          (setq t15 sTofIns)
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                    ;Handle number
            (cons 1000 t15)                   ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      ;;-----------------------------------
      (defun WallSecTagSet (apid en /)
        (TraceIsOn ".WallSecTagSet")
        (setq t0 "WALLSC"
          t2 (etos sDrHan2)               ;Handle number
          t4 (etos sTr4b)
          t5 (etos OffDist)
          t6 pNewW                    ;
          t15 sTofIns
        )
        (AAXDataAdd
          en
          apid
          (list
            (cons 1000 t0)                    ;An ASCII string up to 255 bytes long (group code 1000)
            (cons 1005 t2)                    ;Handle number
            (cons 1000 t4)
            (cons 1000 t5)
            (cons 1011 t6)                    ;3D World space position
            (cons 1000 t15)                   ;Heure, pour ajouter aux ouvertures.
          )
        )
      )

      ;;-----------------------------------
      (defun AddLayersForDoors (/)
        (TraceIsOn ".AddLayersForDoors")
        (Odcl_ComboBox_Clear Walls_Openings_HeaderComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_ThresholdComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_JambComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_LeafComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_SwingComboBox)

        (setq LayrInfo (ai_table "Layer" 4)
          lLay1    (Remove_Dup (cons sHeaderLyr LayrInfo))
          lLay2    (Remove_Dup (cons sJambLyr LayrInfo))
          lLay3    (Remove_Dup (cons sLeafLyr LayrInfo))
          lLay4    (Remove_Dup (cons sSwingLyr LayrInfo))
          lLay5    (Remove_Dup (cons sThreshLyr LayrInfo))
          lLay1    (acad_strlsort lLay1)
          lLay2    (acad_strlsort lLay2)
          lLay3    (acad_strlsort lLay3)
          lLay4    (acad_strlsort lLay4)
          lLay5    (acad_strlsort lLay5)
        )

        (foreach item lLay1 (Odcl_ComboBox_AddString Walls_Openings_HeaderComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_HeaderComboBox sHeaderLyr)             ;sLay1 23
        (foreach item lLay2 (Odcl_ComboBox_AddString Walls_Openings_JambComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_JambComboBox sJambLyr)                 ;43
        (foreach item lLay3 (Odcl_ComboBox_AddString Walls_Openings_LeafComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_LeafComboBox sLeafLyr)                 ;42
        (foreach item lLay4 (Odcl_ComboBox_AddString Walls_Openings_SwingComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_SwingComboBox sSwingLyr)               ;65
        (foreach item lLay5 (Odcl_ComboBox_AddString Walls_Openings_ThresholdComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_ThresholdComboBox sThreshLyr)          ;23
      )

      (defun AddLayersForWinds (/)
        (TraceIsOn ".AddLayersForWinds")
        (Odcl_ComboBox_Clear Walls_Openings_WnHeaderComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_WnSillComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_WnGlassComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_WnFrameComboBox)
        (Odcl_ComboBox_Clear Walls_Openings_WnSashComboBox)

        (setq LayrInfo (ai_table "Layer" 4)
          lLays1   (Remove_Dup (cons sWHeaderLyr LayrInfo))
          lLays2   (Remove_Dup (cons sGlazeLyr LayrInfo))
          lLays3   (Remove_Dup (cons sFrameLyr LayrInfo))
          lLays4   (Remove_Dup (cons sSashLyr LayrInfo))
          lLays5   (Remove_Dup (cons sSillLyr LayrInfo))
          lLays1   (acad_strlsort lLays1)
          lLays2   (acad_strlsort lLays2)
          lLays3   (acad_strlsort lLays3)
          lLays4   (acad_strlsort lLays4)
          lLays5   (acad_strlsort lLays5)
        )

        (foreach item lLays1 (Odcl_ComboBox_AddString Walls_Openings_WnHeaderComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_WnHeaderComboBox sWHeaderLyr)          ;41
        (foreach item lLays2 (Odcl_ComboBox_AddString Walls_Openings_WnGlassComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_WnGlassComboBox sGlazeLyr)             ;151
        (foreach item lLays3 (Odcl_ComboBox_AddString Walls_Openings_WnFrameComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_WnFrameComboBox sFrameLyr)             ;43 mullions
        (foreach item lLays4 (Odcl_ComboBox_AddString Walls_Openings_WnSashComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_WnSashComboBox sSashLyr)               ;42
        (foreach item lLays5 (Odcl_ComboBox_AddString Walls_Openings_WnSillComboBox item))
        (Odcl_ComboBox_SelectString Walls_Openings_WnSillComboBox sSillLyr)               ;23
      )

      ;;-----------------------------------
      ; Returns the ACI-ColorN of the actual graphics background
      ; Return [Typ]: ACI-Colornumber of the active space background [INT]
      (defun AXGetDispBackgCol ( / DspObj)
        (setq Cm:AcDObj (cond (Cm:AcDObj) ((vlax-get-acad-object)))
              DspObj (vla-get-display (vla-get-preferences Cm:AcDObj))
        )
        (AXOleToAciCol
          (if (= (caar (vports)) 1)
            (vla-get-GraphicsWinLayoutBackgrndColor DspObj)
            (vla-get-GraphicsWinModelBackgrndColor DspObj)
          )
        )
      )

      ; Converts a OLE- to a ACI-Colornumber
      ;   OleCol = OLE-Colornumber [INT], Returns [Typ]: ACI-Colornumber [INT]
      (defun AXOleToAciCol (OleCol)
        (vl-position
          (boole
           1
           (vlax-variant-value (vlax-variant-change-type OleCol vlax-vbLong))
           16777215
          )
         '(0        255      65535    65280    16776960
           16711680 16711935 16777215 8421504  12632256
           255      8421631  166      5460902  128
           4210816  76       2500172  38       1250086
           16639    8429567  10662    5466278  8320
           4214912  4940     2502732  2598     1251366
           33023    8437759  21414    5471398  16512
           4219008  9804     2505036  4902     1252646
           49151    8445951  31910    5476774  24704
           4223104  14668    2507596  7462     1253670
           65535    8454143  42662    5482150  32896
           4227200  19532    2509900  9766     1254950
           65471    8454111  42620    5482129  32864
           4227184  19513    2509891  9757     1254945
           65408    8454079  42579    5482108  32832
           4227168  19494    2509881  9747     1254941
           65344    8454047  42537    5482088  32800
           4227152  19475    2509872  9738     1254936
           65280    8454016  42496    5482067  32768
           4227136  19456    2509862  9728     1254931
           4259584  10485632 2729472  6858323  2129920
           5275712  1264640  3165222  665088   1582611
           8453888  12582784 5481984  8169043  4227072
           6324288  2509824  3755046  1254912  1910291
           12582656 14679936 8168960  9545299  6324224
           7372864  3755008  4410406  1910272  2172435
           16776960 16777088 10921472 10921555 8421376
           8421440  5000192  5000230  2500096  2500115
           16760576 16768896 10910720 10916179 8413184
           8417344  4995328  4997926  2497792  2498835
           16744448 16760704 10900224 10910803 8404992
           8413248  4990464  4995366  2495232  2497811
           16728064 16752512 10889472 10905683 8396800
           8409152  4985600  4993062  2492928  2496531
           16711680 16744576 10878976 10900307 8388608
           8405056  4980736  4990502  2490368  2495251
           16711744 16744607 10879017 10900328 8388640
           8405072  4980755  4990512  2490378  2495256
           16711808 16744639 10879059 10900348 8388672
           8405088  4980774  4990521  2490387  2495261
           16711871 16744671 10879100 10900369 8388704
           8405104  4980793  4990531  2490397  2495265
           16711935 16744703 10879142 10900390 8388736
           8405120  4980812  4990540  2490406  2495270
           12517631 14647551 8126630  9524134  6291584
           7356544  3735628  4400716  1900582  2167590
           8388863  12550399 5439654  8147878  4194432
           6307968  2490444  3745356  1245222  1905446
           4194559  10453247 2687142  6837158  2097280
           5259392  1245260  3155532  655398   1577766
           5526612  7763574  10000536 12303291 14540253
           16777215
          )
        )
      )

      (defun ShowDoor (/ bDrawJamb)
        (TraceIsOn ".ShowDoor")
        (Setq bTimb (Odcl_Control_GetProperty
              Walls_Openings_TimberOption
              "Value"
                )
        )
        (Setq bJbThk (Odcl_Control_GetProperty
               Walls_Openings_JambThkCheckBox
               "Value"
                 )
        )
        (Setq bCenter (Odcl_Control_GetProperty
                Walls_Openings_CenteredCheckBox
                "Value"
              )
        )

        (Odcl_PictureBox_Clear Walls_Openings_DoorPictureBox)
        (cond
          ((= sdr_pass "Single hung")
           (setq iLg       100
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 8 16 3 94 iLeaf)             ;Porte
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_Openings_DoorPictureBox                ;Swing
             (list (list 8 110 (- iLg 6) 25 110 11 10 iSwing))
           )
          )

          ((= sdr_pass "Double door")
           (setq iLg       140
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 8 44 3 (fix (/ (- iLg 7) 2)) iLeaf)      ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_Openings_DoorPictureBox                ;Swing gauche
             (list (list 8 110 (fix (/ (- iLg 7) 2)) 25 110 11 10 iSwing)
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 139 44 3 (fix (/ (- iLg 7) 2)) iLeaf)    ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_Openings_DoorPictureBox                ;Swing droit
             (list
           (list 139 110 (fix (/ (- iLg 7) 2)) 139 10 8 110 iSwing)
             )
           )
          )

          ((= sdr_pass "Double door unequal")
           (setq iLg       140
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 8 20 3 90 iLeaf)             ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_Openings_DoorPictureBox                ;Swing gauche
             (list (list 8 110 90 25 110 11 10 iSwing))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 139 67 3 43 iLeaf)               ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_Openings_DoorPictureBox                ;Swing droit
             (list (list 139 110 43 139 10 8 110 iSwing))
           )
          )

          ((= sdr_pass "Bifold double")
           (setq iLg       140
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;gauche
             (list (list 5 110 8 113 iLeaf)
               (list 5 110 30 85 iLeaf)
               (list 33 88 30 85 iLeaf)
               (list 33 88 8 113 iLeaf)
               (list 33 88 36 85 iLeaf)
               (list 36 85 60 110 iLeaf)
               (list 60 110 57 113 iLeaf)
               (list 57 113 33 88 iLeaf)
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;droite
             (list (list 145 110 142 113 iLeaf)
               (list 145 110 120 85 iLeaf)
               (list 142 113 117 88 iLeaf)
               (list 120 85 117 88 iLeaf)
               (list 117 88 114 85 iLeaf)
               (list 114 85 90 110 iLeaf)
               (list 90 110 93 113 iLeaf)
               (list 93 113 117 88 iLeaf)
             )
           )
          )

          ((= sdr_pass "Bifold single")
           (setq iLg       100
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox
             (list (list 5 110 8 113 iLeaf)
               (list 5 110 38 77 iLeaf)
               (list 38 77 41 80 iLeaf)
               (list 41 80 8 113 iLeaf)
               (list 41 80 44 77 iLeaf)
               (list 41 80 74 113 iLeaf)
               (list 74 113 77 110 iLeaf)
               (list 44 77 77 110 iLeaf)
             )
           )
          )

          ((= sdr_pass "Surface sliding double")
           (setq iLg 140
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list -30 104 70 3 iLeaf)              ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_Openings_DoorPictureBox
             (list (list -2 99 13 99 iLeaf)
               (list 13 99 8 97 iLeaf)
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 110 104 70 3 iLeaf)              ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_Openings_DoorPictureBox
             (list (list 152 99 137 99 iLeaf)
               (list 137 99 142 97 iLeaf)
             )
           )
          )

          ((= sdr_pass "Surface sliding single")
           (setq iLg 100
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 55 104 95 3 iLeaf)               ;Porte
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_Openings_DoorPictureBox
             (list (list 112 99 97 99 iLeaf)
               (list 97 99 102 97 iLeaf)
             )
           )
          )

          ((= sdr_pass "Cavity sliding double")
           (setq iLg 140
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list -30 122 70 3 iLeaf)              ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_Openings_DoorPictureBox
             (list (list -2 99 13 99 iLeaf)
               (list 13 99 8 97 iLeaf)
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 110 122 70 3 iLeaf)              ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_Openings_DoorPictureBox
             (list (list 152 99 137 99 iLeaf)
               (list 137 99 142 97 iLeaf)
             )
           )
          )

          ((= sdr_pass "Cavity sliding single")
           (setq iLg 100
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 50 122 95 3 iLeaf)               ;Porte
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_Openings_DoorPictureBox
             (list (list 112 99 97 99 iLeaf)
               (list 97 99 102 97 iLeaf)
             )
           )
          )

          ((= sdr_pass "Doorway")                 ;AABuild108
           (setq iLg 100
             bDrawJamb nil
             sPortW "None"
           )
          )

          ((= sdr_pass "Sliding [Sliding-Fixed]")             ;AABuild108
           (setq iLg 140
             iHt 16
             bDrawJamb nil
             sPortW sTr3a
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list
           (list 5 121 iLg iHt iMull)             ;cadre ext.
           (list 5 121 4 iHt iMull)               ;cadre gauche
           (list 141 121 4 iHt iMull)             ;cadre droit
             )
           )

           (Setq bThreshold
              (Odcl_Control_GetProperty
                Walls_Openings_ThresholdCheckBox
                "Value"
              )
           )
           (if (= bThreshold T)
             (progn
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 5 108 iLg 13 iSill)          ;Sill
             )
           )
           (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_Openings_DoorPictureBox
               (list (list 6 109 (- iLg 2) 11 iSill 1)        ;hatch
               )
             )
           )
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 9 129 4 4 iMull)             ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 13 129 60 4 iSash)               ;sash gauche
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing gauche
             (list (list 13 131 73 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 73 129 4 4 iMull)                ;montant gauche 2
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 73 125 4 4 iMull)                ;montant droit 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 77 125 60 4 iSash)               ;sash droit
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing droit
             (list (list 77 127 137 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 137 125 4 4 iMull)               ;montant droit 2
             )
           )
          )

          ((= sdr_pass "Sliding [F-S-F]")
           (setq iLg 140
             iHt 16
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list
           (list 5 121 iLg iHt iMull)             ;cadre ext.
           (list 5 121 4 iHt iMull)               ;cadre gauche
           (list 141 121 4 iHt iMull)             ;cadre droit
             )
           )

           (Setq bThreshold
              (Odcl_Control_GetProperty
                Walls_Openings_ThresholdCheckBox
                "Value"
              )
           )
           (if (= bThreshold T)
             (progn
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 5 108 iLg 13 iSill)          ;Sill
             )
           )
           (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_Openings_DoorPictureBox
               (list (list 6 109 (- iLg 2) 11 iSill 1)        ;hatch
               )
             )
           )
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 9 125 4 4 iMull)             ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 13 125 37 4 iSash)               ;sash gauche
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing gauche
             (list (list 13 127 50 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 50 125 4 4 iMull)                ;montant droit
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 50 129 4 4 iMull)                ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 54 129 43 4 iSash)               ;sash milieu
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing milieu
             (list (list 54 131 96 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 96 129 4 4 iMull)                ;montant droit
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 96 125 4 4 iMull)                ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 100 125 37 4 iSash)              ;sash droit
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing droit
             (list (list 100 127 137 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 137 125 4 4 iMull)               ;montant droit
             )
           )
          )

          ((= sdr_pass "Sliding [F-S-S-F]")
           (setq iLg 140
             iHt 16
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list
           (list 5 121 iLg iHt iMull)             ;cadre ext.
           (list 5 121 4 iHt iMull)               ;cadre gauche
           (list 141 121 4 iHt iMull)             ;cadre droit
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 9 125 4 4 iMull)             ;montant gauche fixe
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 13 125 25 4 iSash)               ;sash gauche
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing gauche
             (list (list 13 127 38 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 38 125 4 4 iMull)                ;montant droit fixe
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 38 129 4 4 iMull)                ;montant gauche coulissant
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 42 129 29 4 iSash)               ;sash milieu
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing milieu
             (list (list 42 131 71 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 71 129 4 4 iMull)                ;montant droit coulissant
             )
           )
      ;;;---
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 75 129 4 4 iMull)                ;montant gauche coulissant
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 79 129 29 4 iSash)               ;sash milieu
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing milieu
             (list (list 79 131 108 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 108 129 4 4 iMull)               ;montant droit coulissant
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 108 125 4 4 iMull)               ;montant droit fixe
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 112 125 25 4 iSash)              ;sash droit
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_Openings_DoorPictureBox                ;glazing droit
             (list (list 112 127 137 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 137 125 4 4 iMull)               ;montant droit fixe
             )
           )

           (Setq bThreshold
              (Odcl_Control_GetProperty
                Walls_Openings_ThresholdCheckBox
                "Value"
              )
           )
           (if (= bThreshold T)
             (progn
           (Odcl_PictureBox_DrawRect
             Walls_Openings_DoorPictureBox
             (list (list 5 108 iLg 13 iSill)          ;Sill
             )
           )
           (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_Openings_DoorPictureBox
               (list (list 6 109 (- iLg 2) 11 iSill 1)        ;hatch
               )
             )
           )
             )
           )
          )

        )

        (Odcl_PictureBox_DrawText
          Walls_Openings_DoorPictureBox
          (list (list 5 160 34 (strcat "Opening: " sTr3a) "BL" 0)
            (list 90 160 34 (strcat "Door: " sPortW) "BL" 0)  ;AABuild108
          )
        )

        (Setq bHeader (Odcl_Control_GetProperty
                Walls_Openings_HeaderCheckBox
                "Value"
              )
        )
        (if (= bHeader T)
          (if (= iLg 140)
            (Odcl_PictureBox_DrawLine
          Walls_Openings_DoorPictureBox               ;header
          (list (list 5 110 25 110 iHeadr)            ;Top
                (list 35 110 55 110 iHeadr)
                (list 65 110 85 110 iHeadr)
                (list 95 110 115 110 iHeadr)
                (list 125 110 145 110 iHeadr)
                (list 5 137 25 137 iHeadr)            ;Bottom
                (list 35 137 55 137 iHeadr)
                (list 65 137 85 137 iHeadr)
                (list 95 137 115 137 iHeadr)
                (list 125 137 145 137 iHeadr)
          )
            )
            (Odcl_PictureBox_DrawLine                 ;100 iLg
          Walls_Openings_DoorPictureBox               ;header
          (list (list 5 110 24 110 iHeadr)            ;Top
                (list 32 110 51 110 iHeadr)
                (list 59 110 78 110 iHeadr)
                (list 86 110 105 110 iHeadr)
                (list 5 137 24 137 iHeadr)            ;Bottom
                (list 32 137 51 137 iHeadr)
                (list 59 137 78 137 iHeadr)
                (list 86 137 105 137 iHeadr)
          )
            )
          )
        )

        (Setq bThreshold
           (Odcl_Control_GetProperty
             Walls_Openings_ThresholdCheckBox
             "Value"
           )
        )
        (if (and (= bThreshold T)
             (/= sdr_pass "Sliding [Sliding-Fixed]")
             (/= sdr_pass "Sliding [F-S-F]")
             (/= sdr_pass "Sliding [F-S-S-F]")
            )                             ;AABuild108
          (progn
            (if (= bCenter nil)
          (progn                          ;not centered
            ;(Odcl_PictureBox_DrawRect
              ;Walls_Openings_DoorPictureBox
              ;(list (list 5 117 iLg 23 iThres)           ;Sill
              ;)
            ;)
               (Odcl_PictureBox_DrawLine
                 Walls_Openings_DoorPictureBox            ;Sill
                 (list (list 5 117 (+ 5 iLg) 117 iSill)
               (list 5 117 5 140 iSill)
               (list (+ 5 iLg) 117 (+ 5 iLg) 140 iSill)
               (list 5 140 (+ 5 iLg) 140 iSill)
             )
               )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
              Walls_Openings_DoorPictureBox
              (list (list 5 111 iLg 23 iThres)            ;Sill
              )
            )
            ;;       (Odcl_PictureBox_DrawHatchRect
            ;;         Walls_Openings_DoorPictureBox
            ;;         (list (list 6 112 (- iLg 2) 21 iThres 1)   ;hatch
            ;;         )
            ;;       )
          )
            )
          )
        )

        (if (= bCenter nil)
          (progn                          ;not centered
            (Odcl_PictureBox_DrawRect
          Walls_Openings_DoorPictureBox
          (list (list -10 110 15 11 24)               ;Mur haut gauche
                (list -10 126 15 11 24)               ;Mur bas gauche
          )
            )
            (Odcl_PictureBox_DrawHatchRect
          Walls_Openings_DoorPictureBox
          (list (list -10 111 13 9 252 3)             ;Top hatch
                (list -10 127 13 9 252 3)             ;Bot. hatch
          )
            )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
          Walls_Openings_DoorPictureBox
          (list (list -10 104 15 11 24)               ;Mur haut gauche
                (list -10 120 15 11 24)               ;Mur bas gauche
          )
            )
            (Odcl_PictureBox_DrawHatchRect
          Walls_Openings_DoorPictureBox
          (list (list -10 105 13 9 252 3)             ;Top hatch
                (list -10 121 13 9 252 3)             ;Bot. hatch
          )
            )
          )
        )

        (if (= iLg 100)
          (progn
            (if (= bCenter nil)
          (progn                          ;not centered
            (Odcl_PictureBox_DrawRect
              Walls_Openings_DoorPictureBox
              (list (list 105 110 95 11 24)           ;Mur haut droit
                (list 105 126 95 11 24)           ;Mur bas droit
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_Openings_DoorPictureBox
              (list (list 106 111 93 9 252 3)             ;Top hatch
                (list 106 127 93 9 252 3)             ;Bot. hatch
              )
            )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
              Walls_Openings_DoorPictureBox
              (list (list 105 104 95 11 24)           ;Mur haut
                (list 105 120 95 11 24)           ;Mur bas
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_Openings_DoorPictureBox
              (list (list 106 105 93 9 252 3)             ;Top hatch
                (list 106 121 93 9 252 3)             ;Bot. hatch
              )
            )
          )
            )
          )
        )

        (if (= iLg 140)
          (progn
            (if (= bCenter nil)
          (progn                          ;not centered
            (Odcl_PictureBox_DrawRect
              Walls_Openings_DoorPictureBox
              (list (list 145 110 70 11 24)           ;Mur haut droit
                (list 145 126 70 11 24)           ;Mur bas droit
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_Openings_DoorPictureBox
              (list (list 146 111 68 9 252 3)             ;Top hatch
                (list 146 127 68 9 252 3)             ;Bot. hatch
              )
            )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
              Walls_Openings_DoorPictureBox
              (list (list 145 104 70 11 24)           ;Mur haut
                (list 145 120 70 11 24)           ;Mur bas
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_Openings_DoorPictureBox
              (list (list 146 105 68 9 252 3)             ;Top hatch
                (list 146 121 68 9 252 3)             ;Bot. hatch
              )
            )
          )
            )
          )
        )

        (if (= bJbThk nil)
          (setq bThkOn 0)
          (setq bThkOn 1)
        )

        (if (= bDrawJamb T)
          (progn
            (if (= bTimb T)
          (progn
            (if (= bThkOn 0)
              (setq iDepth 126)
              (setq iDepth 137)
            )
            (Odcl_PictureBox_DrawLine             ;timber jamb
              Walls_Openings_DoorPictureBox           ;gauche
              (list (list 5 110 8 110 iJamb)
                (list 8 110 8 113 iJamb)
                (list 8 113 10 113 iJamb)
                (list 10 113 10 iDepth iJamb)
                (list 10 iDepth 5 iDepth iJamb)
                (list 5 iDepth 5 110 iJamb)
                (list (+ iLg 5) 110 (+ iLg 2) 110 iJamb)  ;droit
                (list (+ iLg 2) 110 (+ iLg 2) 113 iJamb)
                (list (+ iLg 2) 113 iLg 113 iJamb)
                (list iLg 113 iLg iDepth iJamb)
                (list iLg iDepth (+ iLg 5) iDepth iJamb)
                (list (+ iLg 5) iDepth (+ iLg 5) 110 iJamb)
              )
            )
          )
          (progn
            (if (= bThkOn 0)
              (setq iDepth 125)
              (setq iDepth 136)
            )
            (Odcl_PictureBox_DrawLine             ;steel jamb
              Walls_Openings_DoorPictureBox           ;gauche
              (list
                (list 5 110 8 110 iJamb)
                (list 8 110 8 113 iJamb)
                (list 8 113 10 113 iJamb)
                (list 10 113 10 iDepth iJamb)
                (list 10 iDepth 8 iDepth iJamb)
                (list 8 iDepth 8 (+ iDepth 3) iJamb)
                (list 8 (+ iDepth 3) 5 (+ iDepth 3) iJamb)
                (list (+ iLg 5) 110 (+ iLg 2) 110 iJamb)      ;droit
                (list (+ iLg 2) 110 (+ iLg 2) 113 iJamb)
                (list (+ iLg 2) 113 iLg 113 iJamb)
                (list iLg 113 iLg iDepth iJamb)
                (list iLg iDepth (+ iLg 2) iDepth iJamb)
                (list (+ iLg 2) iDepth (+ iLg 2) (+ iDepth 3) iJamb)
                (list (+ iLg 2) (+ iDepth 3) (+ iLg 5) (+ iDepth 3) iJamb)
              )
            )
          )
            )
          )
        )
        (Odcl_PictureBox_StoreImage Walls_Openings_DoorPictureBox)
      )

      ;;;-------------
      (defun c:Openings_DrWidthComboBox_OnSelChanged (nSelection sSelText /)
        (Setq sTr3a sSelText)
        (ShowDoor)
      )
      (defun c:Openings_DrWidthComboBox_OnEditChanged (sText /)
        (Setq sTr3a sText  sWDTest sText)
        (ShowDoor)
      )
      (defun c:Openings_HeaderComboBox_OnSelChanged (nSelection sSelText /)
        (Setq layinfo (tblsearch "LAYER" sSelText)
              iHeadr  (abs (cdr (assoc 62 layinfo)))        ;get color
        )
        (ShowDoor)
      )
      (defun c:Openings_ThresholdComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iThres  (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowDoor)
      )

      (defun c:Openings_JambComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iJamb   (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowDoor)
      )
      (defun c:Openings_LeafComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iLeaf   (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowDoor)
      )
      (defun c:Openings_SwingComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iSwing  (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowDoor)
      )

      ;;-----------------------------------
      (defun c:Openings_WnWidthComboBox_OnSelChanged (nSelection sSelText /)
          (TraceIsOn ".Openings_WnWidthComboBox_OnSelChanged")
          (Setq sTrW3a sSelText)
          (ShowWind)
      )
      (defun c:Openings_WnWidthComboBox_OnEditChanged (sText /)
          (TraceIsOn ".Openings_WnWidthComboBox_OnSelChanged")
          (Setq sTrW3a sText  sWTest sText)
          (ShowWind)
      )

      (defun c:Openings_WnHeightComboBox_OnSelChanged (nSelection sSelText /)
          (Setq sTr4b sSelText)
          (ShowWind)
      )
      (defun c:Openings_WnHeightComboBox_OnEditChanged (sText /)
          (Setq sTr4b sText  sHTest sText)
          (ShowWind)
      )

      (defun c:Openings_WnHeaderComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iHead   (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowWind)
      )
      (defun c:Openings_WnSillComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iSill   (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowWind)
      )

      (defun c:Openings_WnGlassComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iGlass  (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowWind)
      )
      (defun c:Openings_WnFrameComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iMull   (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowWind)
      )
      (defun c:Openings_WnSashComboBox_OnSelChanged (nSelection sSelText /)
          (Setq layinfo (tblsearch "LAYER" sSelText)
                iSash   (abs (cdr (assoc 62 layinfo)))        ;get color
          )
          (ShowWind)
      )

      (defun ShowWind (/)
        (TraceIsOn ".ShowWind")
        (Setq bJbThk nil  bCenter nil)
        (Setq bSillOut (Odcl_Control_GetProperty Walls_Openings_OutlineOption "Value"))
        (Setq bSillHat (Odcl_Control_GetProperty Walls_Openings_HatchSillOption "Value"))

        ;;;(Setq sTrW3a (Odcl_Control_GetProperty Walls_Openings_WnWidthComboBox "Text"))
        ;;;(Setq sTr4b  (Odcl_Control_GetProperty Walls_Openings_WnHeightComboBox "Text"))

        (setq   iLg 180                         ;1800mm at 1/10eme
            iHt 16
        )

        (Odcl_PictureBox_Clear Walls_Openings_WindowPictureBox)

        (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
          (list (list 5 140 34 (strcat "Width:  " sTrW3a) "BL" 0)      ;AABuild108
            (list 5 155 34 (strcat "Height: " sTr4b) "BL" 0)       ;AABuild108
          )
        )

        (Odcl_PictureBox_DrawRect
           Walls_Openings_WindowPictureBox
           (list
             (list   5 89 iLg iHt iMull)          ;cadre ext.
             (list   5 89   4 iHt iMull)          ;cadre gauche
             (list 181 89   4 iHt iMull)          ;cadre droit
           )
        )

        (Setq bHeaderW (Odcl_Control_GetProperty Walls_Openings_WnHeaderCheckBox "Value"))
        (if (= bHeaderW T)
          (Odcl_PictureBox_DrawLine                   ;(StartX StartY EndX EndY LineColor)
            Walls_Openings_WindowPictureBox               ;header
            (list
              (list   5  78  23  78 iHead)            ;18 et 9
              (list  32  78  50  78 iHead)
              (list  59  78  77  78 iHead)
              (list  86  78 104  78 iHead)
              (list 113  78 131  78 iHead)
              (list 140  78 158  78 iHead)
              (list 167  78 185  78 iHead)
              (list   5  105  23  105 iHead)
              (list  32  105  50  105 iHead)
              (list  59  105  77  105 iHead)
              (list  86  105 104  105 iHead)
              (list 113  105 131  105 iHead)
              (list 140  105 158  105 iHead)
              (list 167  105 185  105 iHead)
            )
          )
        )

        (Setq bTransom (Odcl_Control_GetProperty Walls_Openings_TransomCheckBox "Value"))
        (if (= bTransom T)
           (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
           (list (list 140 30 34 "+ Transom" "BL" 0))
           )
        )

        (Setq bSill (Odcl_Control_GetProperty Walls_Openings_WnSillCheckBox "Value"))
        (if (= bSill T)
          (progn
             (Odcl_PictureBox_DrawRect
               Walls_Openings_WindowPictureBox
               (list (list 5 76 iLg 13 iSill)             ;Sill
               )
             )
             (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_Openings_WindowPictureBox
               (list (list 6 77 (- iLg 2) 11 iSill 1)         ;hatch
               )
             )
             )
          )
        )

        (cond
          ((= sWn_pass "Fixed Panels")
           (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
           (list (list 5 30 34 "Fixed type" "BL" 0))
           )
           (if (<= (atoi sTrW3a) (atof sSeuil1))
             (progn (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 95 4 4 iMull)             ;gauche
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 95 164 4 iSash)          ;sash
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing
              (list (list 13 97 177 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 95 4 4 iMull)           ;droit
              )
                )
             )
           )
           (if (and (> (atoi sTrW3a)(atof sSeuil1))(<= (atoi sTrW3a)(atof sSeuil2)))
            (progn
                   (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
                    (list (list 5 30 34 "Fixed type" "BL" 0))
                   )
               (Odcl_PictureBox_DrawRect              ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 95 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 95 76 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing gauche
              (list (list 13 97 89 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 89 95 4 4 iMull)            ;montant gauche 2
              )
                )
              (Odcl_PictureBox_DrawRect
                Walls_Openings_WindowPictureBox
                (list (list 93 89 4 iHt iMull)        ;cadre du milieu
                )
               )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 97 95 4 4 iMull)            ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 101 95 76 4 iSash)          ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing droit
              (list (list 101 97 177 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 95 4 4 iMull)           ;montant droit 2
              )
                )
            )
           )
           (if (> (atoi sTrW3a) (atof sSeuil2))
             (progn
                   (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
                    (list (list 5 30 34 "Fixed type" "BL" 0))
                   )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
                  Walls_Openings_WindowPictureBox
              (list (list 9 95 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 95 47 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing gauche
              (list (list 13 97 60 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 60 95 4 4 iMull)            ;montant gauche 2
              )
                )
              (Odcl_PictureBox_DrawRect
                Walls_Openings_WindowPictureBox
                (list (list 64 89 4 iHt iMull)        ;cadre
                )
               )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 68 95 4 4 iMull)            ;montant milieu 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 72 95 46 4 iSash)           ;sash milieu
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing milieu
              (list (list 72 97 118 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 118 95 4 4 iMull)           ;montant milieu 2
              )
                )
              (Odcl_PictureBox_DrawRect
                Walls_Openings_WindowPictureBox
                (list (list 122 89 4 iHt iMull)       ;cadre
                )
               )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 126 95 4 4 iMull)           ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 130 95 47 4 iSash)          ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing droit
              (list (list 130 97 177 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 95 4 4 iMull)           ;montant droit 2
              )
                )
            )
           )
          )
          ((= sWn_pass "Sliding [Sliding-Fixed]")
                   (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
                    (list (list 5 30 34 "Sliding-Fixed type" "BL" 0))
                   )
               (Odcl_PictureBox_DrawRect              ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 97 80 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing gauche
              (list (list 13 99 93 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 93 97 4 4 iMull)            ;montant gauche 2
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 93 93 4 4 iMull)            ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 97 93 80 4 iSash)           ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing droit
              (list (list 97 95 177 95 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;montant droit 2
              )
                )
          )
          ((= sWn_pass "Sliding [S-F-S]")
                   (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
                    (list (list 5 30 34 "Sliding-Fixed-Sliding type" "BL" 0))
                   )
               (Odcl_PictureBox_DrawRect              ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 97 37 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing gauche
              (list (list 13 99 50 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 50 97 4 4 iMull)            ;montant gauche 2
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 50 93 4 4 iMull)            ;montant milieu 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 54 93 82 4 iSash)           ;sash milieu
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing milieu
              (list (list 54 95 136 95 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 136 93 4 4 iMull)           ;montant milieu 2
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 136 97 4 4 iMull)           ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 140 97 37 4 iSash)          ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing droit
              (list (list 140 99 177 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 97 4 4 iMull)           ;montant droit 2
              )
                )
          )
          ((= sWn_pass "Sliding [F-S-F]")     ;1/3 chaque pour FSF a creer*******
               (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 97 39 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing gauche
              (list (list 13 99 53 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 53 97 4 4 iMull)            ;montant gauche 2
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 53 93 4 4 iMull)            ;montant milieu 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 57 93 38 4 iSash)           ;sash milieu
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing milieu
              (list (list 57 95 95 95 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 93 93 4 4 iMull)            ;montant milieu 2
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 93 97 4 4 iMull)            ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 97 97 39 4 iSash)           ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_Openings_WindowPictureBox         ;glazing droit
              (list (list 97 99 136 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 136 97 4 4 iMull)           ;montant droit 2
              )
                )
          )

          ((= sWn_pass "Double Hung")
           (Odcl_PictureBox_DrawText Walls_Openings_WindowPictureBox
           (list (list 5 30 34 "Double Hung type" "BL" 0))
           )
           (if (<= (atoi sTrW3a) (atof sSeuil1))
             (progn (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;gauche, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 93 4 4 iMull)             ;gauche, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 97 164 4 iSash)          ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 93 164 4 iSash)          ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, bas
              (list (list 13 99 177 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, haut
              (list (list 13 95 177 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;droit, haut
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 97 4 4 iMull)           ;droit, bas
              )
                )
             )
           )
           (if (and (> (atoi sTrW3a)(atof sSeuil1))(<= (atoi sTrW3a)(atof sSeuil2)))
             (progn (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;gauche, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 93 4 4 iMull)             ;gauche, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 97 80 4 iSash)           ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 93 80 4 iSash)           ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, bas
              (list (list 13 99 93 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, haut
              (list (list 13 95 93 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 93 97 4 4 iMull)            ;milieu, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 93 93 4 4 iMull)            ;milieu, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 97 93 80 4 iSash)           ;sash, haut fixe
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, haut fixe
              (list (list 97 95 177 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;droit, haut
              )
                )
             )
           )
           (if (> (atoi sTrW3a) (atof sSeuil2))
             (progn
                    (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;gauche, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 9 93 4 4 iMull)             ;gauche, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 97 52 4 iSash)           ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 13 93 52 4 iSash)           ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, bas
              (list (list 13 99 65 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, haut
              (list (list 13 95 65 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 65 97 4 4 iMull)            ;1, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 65 93 4 4 iMull)            ;1, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 69 93 52 4 iSash)           ;sash, haut fixe
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, haut fixe
              (list (list 69 95 121 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 121 97 4 4 iMull)           ;2, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_Openings_WindowPictureBox
              (list (list 121 93 4 4 iMull)           ;2, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 125 97 52 4 iSash)          ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 125 93 52 4 iSash)          ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, bas
              (list (list 125 99 177 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_Openings_WindowPictureBox         ;glazing, haut
              (list (list 125 95 177 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;droit, haut
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_Openings_WindowPictureBox
              (list (list 177 97 4 4 iMull)           ;droit, bas
              )
                )
             )
           )
          )
        )

                (Odcl_PictureBox_DrawRect
               Walls_Openings_WindowPictureBox
               (list (list -10 78 15 11 24)           ;Mur haut gauche
                 (list -10 94 15 11 24)           ;Mur bas gauche
                 (list 185 78 10 11 24)           ;Mur haut droit
                 (list 185 94 10 11 24)           ;Mur bas droit
               )
             )
             (Odcl_PictureBox_DrawHatchRect
               Walls_Openings_WindowPictureBox
               (list (list -10 79 13 9 252 3)             ;Top hatch
                 (list -10 95 13 9 252 3)             ;Bot. hatch
                 (list 186 79  9 9 252 3)
                 (list 186 95  9 9 252 3)
               )
             )
         (Odcl_PictureBox_StoreImage Walls_Openings_WindowPictureBox)
      )

      ;;-----------------------------------
      (defun Ouverture (/ ssdoor A0a A1a A2 A3)
        (TraceIsOn ".Ouverture")
        (setq sOpenAng1 nil  leafwidt1 nil  bBypass T)
        (setq ssdoor nil  ssWind nil)
        (setq bPorte nil  bFeinster nil  bThreSil nil  rMoveInsPt 0.0)
        (GetLayers) (GetDrINI) (GetWnINI)

      ;For Doors
        (if (= iHeadr nil) (Setq iHeadr (atoi sHeaderCol)))
        (if (= iThres nil) (Setq iThres (atoi sThreshCol)))
        (if (= iJamb nil)  (Setq iJamb  (atoi sJambCol)))
        (if (= iLeaf nil)  (Setq iLeaf  (atoi sLeafCol)))
        (if (= iSwing nil) (Setq iSwing (atoi sSwingCol)))
      ;For Windows
        (if (= iHead nil)  (Setq iHead  (atoi sWHeaderCol)))
        (if (= iSill nil)  (Setq iSill  (atoi sSillCol)))
        (if (= iMull nil)  (Setq iMull  (atoi sFrameCol)))
        (if (= iSash nil)  (Setq iSash  (atoi sSashCol)))
        (if (= iGlass nil) (Setq iGlass (atoi sGlazeCol)))

        (defun Onglet1 (/)
        (TraceIsOn ".Onglet1")
         (if (= bPorte nil)
          (AddLayersForDoors)
         )

          (Odcl_ComboBox_Clear Walls_Openings_ManufComboBox)
          (Odcl_ComboBox_AddList Walls_Openings_ManufComboBox (list sManuf1 sManuf2 sManuf3))
          (Odcl_ComboBox_SelectString Walls_Openings_ManufComboBox sManuf1)

          (if (= sdr_pass nil)(setq sdr_pass "Single hung"))
          (if (= sWn_pass nil)(setq sWn_pass "Fixed Panels"))

          (setq lDoorWidth1  (Remove_Dup lDoorWidth1)             ;"Single Doors-Manuf1"
            lDoorWidth1s (Remove_Dup lDoorWidth1s)            ;"Sliding Doors-Manuf1"

            lDFSFWidth1  (Remove_Dup lDFSFWidth1)             ;"Sliding Doors FSF-Manuf1"
            lDFSSFWidth1 (Remove_Dup lDFSSFWidth1)            ;"Sliding Doors FSSF-Manuf1"

            lDoorWidth2  (Remove_Dup lDoorWidth2)             ;"Double Doors"
            lDoorWidth3  (Remove_Dup lDoorWidth3)             ;"Folding Doors"
          )

          (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)

          (defun c:Openings_ManufComboBox_OnSelChanged (nSelection sSelText /)
            (cond
          ((= nSelection 0)
           (setq lWid1  (cadr (strfind lSngM1 "-"))
                 lWid1s (cadr (strfind lSldM1 "-")))
          )
          ((= nSelection 1)
           (setq lWid1  (cadr (strfind lSngM2 "-"))
                 lWid1s (cadr (strfind lSldM2 "-")))
          )
          ((= nSelection 2)
           (setq lWid1  (cadr (strfind lSngM3 "-"))
                 lWid1s (cadr (strfind lSldM3 "-")))
          )
            )
            (setq lDoorWidth1 nil  lDoorWidth1 (DeStrAListe lWid1 "|")
              lDoorWidth1s nil lDoorWidth1s (DeStrAListe lWid1s "|")
              lDoorWidth2 nil  lDoorWidth2 (DeStrAListe lWid2 "|")
              lDoorWidth3 nil  lDoorWidth3 (DeStrAListe lWid3 "|")
            )
          (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
          (cond ((= sdr_pass "Single hung")
             (setq str2  "S")
             ;...
            )
          )

          )

          (cond ((= sdr_pass "Single hung")
             (setq str2  "S")
             (foreach item lDoorWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth1) nil))
               (setq sTr3a (nth 4 lDoorWidth1))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
            )
            ((= sdr_pass "Double door")
             (setq str2 "D")
             (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Double door unequal")
             (setq str2 "UD")
             (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Bifold double")
             (setq str2 "BD")
             (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Bifold single")
             (setq str2 "BS")
             (foreach item lDoorWidth3 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Surface sliding double")
             (setq str2 "SSD")
             (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Surface sliding single")
             (setq str2 "SSS")
             (foreach item lDoorWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Cavity sliding double")
             (setq str2 "CSD")
             (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Cavity sliding single")
             (setq str2 "CSS")
             (foreach item lDoorWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))
            )
            ((= sdr_pass "Doorway")
             (setq str2 "W")
             (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item)) ;AABuild108
            )
            ((= sdr_pass "Sliding [Sliding-Fixed]")
             (setq str2 "SFd")
             (foreach item lDoorWidth1s (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))    ;AABuild108
            )
            ((= sdr_pass "Sliding [F-S-F]")
             (setq str2 "FSFd")
             (foreach item lDFSFWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item)) ;AABuild108
            )
            ((= sdr_pass "Sliding [F-S-S-F]")
             (setq str2 "FSSFd")
             (foreach item lDFSSFWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))    ;AABuild108
            )
          )
          (Odcl_ListBox_SelectString Walls_Openings_DoorListBox sdr_pass)

          (Odcl_Control_SetProperty Walls_Openings_SteelOption "Value" nil)
          (Odcl_Control_SetProperty Walls_Openings_TimberOption "Value" nil)
          (if bSorT
           (progn
            (Odcl_Control_SetProperty Walls_Openings_TimberOption "Value" bTimb)
            (Odcl_Control_SetProperty Walls_Openings_SteelOption "Value" bSteel)
           )
           (if (= sJambTyp "S")
          (Odcl_Control_SetProperty Walls_Openings_SteelOption "Value" T)
          (Odcl_Control_SetProperty Walls_Openings_TimberOption "Value" T)
           )
          )

          (if (= PercOpen nil)
            (Setq PercOpen sOpenAng)
          )
          (Odcl_Control_SetProperty Walls_Openings_OpenAngTextBox "Text" PercOpen)
          (Odcl_Control_SetProperty Walls_Openings_OpenAngSliderBar "Value" (atoi PercOpen))

          (if (= (atoi PercOpen) 90)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 3)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 1)
          )

          (if (/= nil sWDTest)
           (progn
            (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox sWDTest)
            (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sWDTest)
           )
      ;      (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
          )

          (defun c:Openings_DoorListBox_OnSelChanged (nSelection sSelText /)
            (cond ((= sSelText "Single hung")
               (setq sdr_pass "Single hung"
                 str2     "S"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" T)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth1) nil))
               (setq sTr3a (nth 4 lDoorWidth1))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Double door")
               (setq sdr_pass "Double door"
                 str2     "D"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" T)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth2) nil))
               (setq sTr3a (nth 5 lDoorWidth2))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Double door unequal")
               (setq sdr_pass "Double door unequal"
                 str2     "UD"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" T)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth2) nil))
               (setq sTr3a (nth 5 lDoorWidth2))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Bifold double")
               (setq sdr_pass "Bifold double"
                 str2     "BD"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" T)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth2) nil))
               (setq sTr3a (nth 5 lDoorWidth2))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Bifold single")
               (setq sdr_pass "Bifold single"
                 str2     "BS"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" T)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" T)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth3 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth3) nil))
               (setq sTr3a (nth 1 lDoorWidth3))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Surface sliding double")
               (setq sdr_pass "Surface sliding double"
                 str2     "SSD"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth2) nil))
               (setq sTr3a (nth 5 lDoorWidth2))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Surface sliding single")
               (setq sdr_pass "Surface sliding single"
                 str2     "SSS"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth1) nil))
               (setq sTr3a (nth 4 lDoorWidth1))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Cavity sliding double")
               (setq sdr_pass "Cavity sliding double"
                 str2     "CSD"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth2) nil))
               (setq sTr3a (nth 5 lDoorWidth2))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Cavity sliding single")
               (setq sdr_pass "Cavity sliding single"
                 str2     "CSS"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth1) nil))
               (setq sTr3a (nth 4 lDoorWidth1))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Doorway")                 ;AABuild108
               (setq sdr_pass "Doorway"
                 str2     "W"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth2 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth2) nil))
               (setq sTr3a (nth 0 lDoorWidth2))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Sliding [Sliding-Fixed]")         ;AABuild108
               (setq sdr_pass "Sliding [Sliding-Fixed]"
                 str2     "SFd"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDoorWidth1s (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDoorWidth1s) nil))
               (setq sTr3a (nth 0 lDoorWidth1s))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Sliding [F-S-F]")             ;AABuild108
               (setq sdr_pass "Sliding [F-S-F]"
                 str2     "FSFd"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDFSFWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDFSFWidth1) nil))
               (setq sTr3a (nth 0 lDFSFWidth1))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

              ((= sSelText "Sliding [F-S-S-F]")               ;AABuild108
               (setq sdr_pass "Sliding [F-S-S-F]"
                 str2     "FSSFd"
               )
               (Odcl_Control_SetProperty Walls_Openings_SteelOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_TimberOption "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
               (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
               (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
               (Odcl_ComboBox_Clear Walls_Openings_DrWidthComboBox)
               (foreach item lDFSSFWidth1 (Odcl_ComboBox_AddString Walls_Openings_DrWidthComboBox item))

                   (if (or (= sTr3a nil)(= (member sTr3a lDFSSFWidth1) nil))
               (setq sTr3a (nth 0 lDFSSFWidth1))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_DrWidthComboBox sTr3a)
               (ShowDoor)
              )

            )
          )

          (defun c:Openings_HeaderCheckBox_OnClicked (nValue /)
            (if (= nValue 1)
          (progn (Odcl_Control_SetProperty Walls_Openings_HeaderComboBox "Enabled" T)
                 (Odcl_Control_SetProperty Walls_Openings_ThresholdCheckBox "Value" nil)
                 (Odcl_Control_SetProperty Walls_Openings_ThresholdComboBox "Enabled" nil)
          )
          (progn (Odcl_Control_SetProperty Walls_Openings_HeaderComboBox "Enabled" nil))
            )
            (ShowDoor)
          )

          (defun c:Openings_ThresholdCheckBox_OnClicked (nValue /)
            (if (= nValue 1)
          (progn (Odcl_Control_SetProperty Walls_Openings_ThresholdComboBox "Enabled" T)
                 (Odcl_Control_SetProperty Walls_Openings_HeaderCheckBox "Value" nil)
                 (Odcl_Control_SetProperty Walls_Openings_HeaderComboBox "Enabled" nil)
          )
          (progn (Odcl_Control_SetProperty Walls_Openings_ThresholdComboBox "Enabled" nil))
            )
            (ShowDoor)
          )

          (defun c:Openings_SteelOption_OnClicked (nValue /) (ShowDoor))

          (defun c:Openings_TimberOption_OnClicked (nValue /) (ShowDoor))

          (defun c:Openings_JambThkCheckBox_OnClicked (nValue /)
            (Setq bThkOn nValue)
            (if (= bThkOn 1)
          (progn (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Value" nil)
          )
          (progn (Odcl_Control_SetProperty Walls_Openings_CenteredCheckBox "Enabled" T))
            )
            (ShowDoor)
          )

          (defun c:Openings_CenteredCheckBox_OnClicked (nValue /)
            (Setq bCenter nValue)
            (if (= bCenter 1)
          (progn (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Value" nil)
                 (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" nil)
          )
          (progn (Odcl_Control_SetProperty Walls_Openings_JambThkCheckBox "Enabled" T))
            )
            (ShowDoor)
          )

          (defun c:Openings_OpenAngSliderBar_OnReleasedCapture (nValue /)
            (Odcl_Control_SetProperty Walls_Openings_OpenAngTextBox "Text" nValue)
            (if (= nValue 90)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 3)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 1)
            )
          )
          (defun c:Openings_OpenAngTextBox_OnReturnPressed ()
            (Setq iValue (Odcl_Control_GetProperty Walls_Openings_OpenAngTextBox "Text"))
            (Odcl_Control_SetProperty Walls_Openings_OpenAngSliderBar "Value" (atoi iValue))
            (if (= (atoi iValue) 90)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 3)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 1)
            )
          )
          (defun c:Openings_OpenAngTextBox_OnKillFocus ()
            (Setq iValue (Odcl_Control_GetProperty Walls_Openings_OpenAngTextBox "Text"))
            (Odcl_Control_SetProperty Walls_Openings_OpenAngSliderBar "Value" (atoi iValue))
            (if (= (atoi iValue) 90)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 3)
              (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 1)
            )
          )
          (defun c:Openings_AnglePictureBox_OnClicked ()
           (Odcl_Control_SetProperty Walls_Openings_OpenAngTextBox "Text" "0")
           (Odcl_Control_SetProperty Walls_Openings_OpenAngSliderBar "Value" 0)
           (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 1)
          )
          (defun c:Openings_AnglePictureBox_OnDblClicked ()
           (Odcl_Control_SetProperty Walls_Openings_OpenAngTextBox "Text" "90")
           (Odcl_Control_SetProperty Walls_Openings_OpenAngSliderBar "Value" 90)
           (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 3)
          )
          (defun c:Openings_AnglePictureBox_OnRightClick ()
           (Odcl_Control_SetProperty Walls_Openings_OpenAngTextBox "Text" "180")
           (Odcl_Control_SetProperty Walls_Openings_OpenAngSliderBar "Value" 180)
           (Odcl_Control_SetProperty Walls_Openings_AnglePictureBox "BackColor" 1)
          )
        (ShowDoor)
        )
      ;;-----------------------------------
        (defun Onglet2 (/)
        (TraceIsOn ".Onglet2")
         (if (= bFeinster nil)
          (AddLayersForWinds)
         )

         (if (= T (Odcl_Control_GetProperty Walls_Openings_WnSillCheckBox "Value"))
          (progn
           (Odcl_Control_SetProperty Walls_Openings_OutlineOption "Enabled" T)
           (Odcl_Control_SetProperty Walls_Openings_HatchSillOption"Enabled" T)
          )
          (progn
           (Odcl_Control_SetProperty Walls_Openings_OutlineOption "Enabled" nil)
           (Odcl_Control_SetProperty Walls_Openings_HatchSillOption"Enabled" nil)
          )
         )

          (if (= sdr_pass nil)(setq sdr_pass "Single hung"))
          (if (= sWn_pass nil)(setq sWn_pass "Fixed Panels"))

          (setq lFxdWWidth  (Remove_Dup lFxdWWidth))
          (setq lFxdWHeight (Remove_Dup lFxdWHeight))

          (setq lDHgWWidth  (Remove_Dup lDHgWWidth))                  ;Double Hung 2 panneaux
          (setq lDHgWHeight (Remove_Dup lDHgWHeight))

      ;    (setq iCnnt (1- (length lDHgWWidth)))
      ;    (while (> (atoi (nth iCnnt lDHgWWidth)) (atoi sSeuil2))
      ;     (setq lDHgWWidt3 (append (list (nth iCnnt lDHgWWidth)) lDHgWWidt3))   ;Double Hung 3 panneaux
      ;     (setq iCnnt (1- iCnnt))
      ;    )

          (Odcl_ComboBox_Clear Walls_Openings_WnWidthComboBox)
          (Odcl_ComboBox_Clear Walls_Openings_WnHeightComboBox)

          (cond ((= sWn_pass "Fixed Panels")
             (setq sTr2W "FW")
             (foreach item lFxdWWidth  (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))
             (foreach item lFxdWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

               (setq sSeuil4  (last (DeStrAListe lTFxdM1 "-")))
               (setq lSeuils  (DeStrAListe (car (DeStrAListe lTFxdM1 "-")) "|"))
               (setq sSeuil1  (car lSeuils))
               (setq sSeuil2  (cadr lSeuils))
               (setq sSeuil3  (caddr lSeuils))

                   (if (or (= sTrW3a nil)(= (member sTrW3a lFxdWWidth) nil))
               (setq sTrW3a (nth 3 lFxdWWidth))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sTrW3a)

                   (if (or (= sTr4b nil)(= (member sTr4b lFxdWHeight) nil))
               (setq sTr4b (nth 3 lFxdWHeight))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sTr4b)

             (ShowWind)
            )
            ((= sWn_pass "Sliding [Sliding-Fixed]")
             (setq sTr2W "SF")
             (foreach item lSLdWWidth  (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))
             (foreach item lSLdWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

             (ShowWind)
            )
            ((= sWn_pass "Sliding [S-F-S]")
             (setq sTr2W "SFS")
             (foreach item lSLdWWidt3  (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))
             (foreach item lSLdWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

             (ShowWind)
            )

            ((= sWn_pass "Double Hung")
             (setq sTr2W "DH")
             (foreach item lDHgWWidth  (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))
             (foreach item lDHgWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

             (ShowWind)
            )
          )

          (Odcl_ListBox_SelectString Walls_Openings_WindListBox sWn_pass)
          (if (/= nil sWTest)
           (progn
            (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox sWTest)
            (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sWTest)
           )
      ;      (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sTrW3a)
          )
          (if (/= nil sHTest)
           (progn
            (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox sHTest)
            (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sHTest)
           )
      ;      (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sTr4b)
          )

          (defun c:Openings_WindListBox_OnSelChanged (nSelection sSelText /)
          (TraceIsOn ".Openings_WindListBox_OnSelChanged")
            (cond ((= sSelText "Fixed Panels")
               (setq sWn_pass     "Fixed Panels"
                 sTr2W    "FW"
               )              ;Single
               (Odcl_ComboBox_Clear Walls_Openings_WnWidthComboBox)
               (Odcl_ListBox_SelectString Walls_Openings_WindListBox sWn_pass)
               (foreach item lFxdWWidth (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))

               (setq sSeuil4  (last (DeStrAListe lTFxdM1 "-")))
               (setq lSeuils  (DeStrAListe (car (DeStrAListe lTFxdM1 "-")) "|"))
               (setq sSeuil1  (car lSeuils))
               (setq sSeuil2  (cadr lSeuils))
               (setq sSeuil3  (caddr lSeuils))

                   (Odcl_ComboBox_Clear Walls_Openings_WnHeightComboBox)
               (foreach item lFxdWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

                   (if (or (= sTrW3a nil)(= (member sTrW3a lFxdWWidth) nil))
               (setq sTrW3a (nth 3 lFxdWWidth))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sTrW3a)

                   (if (or (= sTr4b nil)(= (member sTr4b lFxdWHeight) nil))
               (setq sTr4b (nth 3 lFxdWHeight))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sTr4b)

               (ShowWind)
              )
              ((= sSelText "Sliding [Sliding-Fixed]")
               (setq sWn_pass     "Sliding [Sliding-Fixed]"
                 sTr2W    "SF"
               )
               (Odcl_ComboBox_Clear Walls_Openings_WnWidthComboBox)
               (Odcl_ListBox_SelectString Walls_Openings_WindListBox sWn_pass)

               (setq sSeuil4  (last (DeStrAListe lTSLdM1 "-")))
               (setq lSeuils  (DeStrAListe (car (DeStrAListe lTSLdM1 "-")) "|"))
               (setq sSeuil1  (car lSeuils))
               (setq sSeuil2  (cadr lSeuils))
               (setq sSeuil3  (caddr lSeuils))

                   (setq lSLdWWidth  (Remove_Dup lSLdWWidth))                 ;Sliding Windows 2 panneaux
                   (setq lSLdWHeight (Remove_Dup lSLdWHeight))

               (foreach item lSLdWWidth (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))

                   (Odcl_ComboBox_Clear Walls_Openings_WnHeightComboBox)
               (foreach item lSLdWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

                   (if (or (= sTrW3a nil)(= (member sTrW3a lSLdWWidth) nil))
               (setq sTrW3a (nth 0 lSLdWWidth))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sTrW3a)

                   (if (or (= sTr4b nil)(= (member sTr4b lSLdWHeight) nil))
               (setq sTr4b (nth 3 lSLdWHeight))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sTr4b)

               (ShowWind)
              )
              ((= sSelText "Sliding [S-F-S]")
               (setq sWn_pass "Sliding [S-F-S]"
               sTr2W    "SFS"
               )
               (Odcl_ComboBox_Clear Walls_Openings_WnWidthComboBox)
               (Odcl_ListBox_SelectString Walls_Openings_WindListBox sWn_pass)

               (setq sSeuil4  (last (DeStrAListe lTSLdM1 "-")))
               (setq lSeuils  (DeStrAListe (car (DeStrAListe lTSLdM1 "-")) "|"))
               (setq sSeuil1  (car lSeuils))
               (setq sSeuil2  (cadr lSeuils))
               (setq sSeuil3  (caddr lSeuils))

                   (setq lSLdWWidt3 nil)
                   (setq lSLdWWidth  (Remove_Dup lSLdWWidth))                 ;Sliding Windows 2 panneaux
                   (setq iCnnt (1- (length lSLdWWidth)))
                   (while (> (atoi (nth iCnnt lSLdWWidth)) (atoi sSeuil2))
                     (setq lSLdWWidt3 (append (list (nth iCnnt lSLdWWidth)) lSLdWWidt3))  ;Sliding Windows 3 panneaux
                     (setq iCnnt (1- iCnnt))
                   )
                   (setq lSLdWHeight (Remove_Dup lSLdWHeight))

               (foreach item lSLdWWidt3 (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))

                   (Odcl_ComboBox_Clear Walls_Openings_WnHeightComboBox)
               (foreach item lSLdWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

                   (if (or (= sTrW3a nil)(= (member sTrW3a lSLdWWidt3) nil))
               (setq sTrW3a (nth 0 lSLdWWidt3))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sTrW3a)

                   (if (or (= sTr4b nil)(= (member sTr4b lSLdWHeight) nil))
               (setq sTr4b (nth 3 lSLdWHeight))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sTr4b)

               (ShowWind)
              )
              ((= sSelText "Double Hung")
               (setq sWn_pass "Double Hung"
               sTr2W    "DH"
               )
               (Odcl_ComboBox_Clear Walls_Openings_WnWidthComboBox)
               (Odcl_ListBox_SelectString Walls_Openings_WindListBox sWn_pass)
               (foreach item lDHgWWidth (Odcl_ComboBox_AddString Walls_Openings_WnWidthComboBox item))

                   (Odcl_ComboBox_Clear Walls_Openings_WnHeightComboBox)
               (foreach item lDHgWHeight (Odcl_ComboBox_AddString Walls_Openings_WnHeightComboBox item))

                   (if (or (= sTrW3a nil)(= (member sTrW3a lDHgWWidth) nil))
               (setq sTrW3a (nth 3 lDHgWWidth))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnWidthComboBox sTrW3a)

                   (if (or (= sTr4b nil)(= (member sTr4b lDHgWHeight) nil))
               (setq sTr4b (nth 3 lDHgWHeight))
               )
               (Odcl_ComboBox_SelectString Walls_Openings_WnHeightComboBox sTr4b)

               (setq sSeuil4  (last (DeStrAListe lTDHgM1 "-")))
               (setq lSeuils  (DeStrAListe (car (DeStrAListe lTDHgM1 "-")) "|"))
               (setq sSeuil1  (car lSeuils))
               (setq sSeuil2  (cadr lSeuils))
               (setq sSeuil3  (caddr lSeuils))

               (ShowWind)
              )

            )
          )

          (defun c:Openings_OutlineOption_OnClicked (nValue /)
           (ShowWind)
          )

          (defun c:Openings_HatchSillOption_OnClicked (nValue /)
           (ShowWind)
          )

          (defun c:Openings_WnHeaderCheckBox_OnClicked (nValue /)
            (if (= nValue 1)
          (progn (Odcl_Control_SetProperty Walls_Openings_WnHeaderComboBox "Enabled" T)
                 (Odcl_Control_SetProperty Walls_Openings_WnSillCheckBox "Value" nil)
                 (Odcl_Control_SetProperty Walls_Openings_WnSillComboBox "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_OutlineOption "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_HatchSillOption "Enabled" nil)
          )
          (progn (Odcl_Control_SetProperty Walls_Openings_WnHeaderComboBox "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_OutlineOption "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_HatchSillOption "Enabled" nil)
              )
            )
              (ShowWind)
          )

          (defun c:Openings_WnSillCheckBox_OnClicked (nValue /)
            (if (= nValue 1)
          (progn (Odcl_Control_SetProperty Walls_Openings_WnSillComboBox "Enabled" T)
                 (Odcl_Control_SetProperty Walls_Openings_WnHeaderCheckBox "Value" nil)
                 (Odcl_Control_SetProperty Walls_Openings_WnHeaderComboBox "Enabled" nil)

                 (Odcl_Control_SetProperty Walls_Openings_OutlineOption "Enabled" T)
                 (Odcl_Control_SetProperty Walls_Openings_HatchSillOption "Enabled" T)
          )
          (progn (Odcl_Control_SetProperty Walls_Openings_WnSillComboBox "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_OutlineOption "Enabled" nil)
                 (Odcl_Control_SetProperty Walls_Openings_HatchSillOption "Enabled" nil)
              )
            )
              (ShowWind)
          )

          (defun c:Openings_TransomCheckBox_OnClicked (nValue /)
            (if (= nValue 1)
          (setq bTransom T)
          (setq bTransom nil)
            )
              (ShowWind)
          )
         (ShowWind)
        )

      ;;-----------------------------------
        (defun c:Openings_OnInitialize ()
         (TraceIsOn ".Openings_OnInitialize")
         (setq bBypass T)
          (if (= bTab 0)
            (progn
               (setq bOnglet1 T  bOnglet2 nil)
               (Onglet1)
               (Odcl_Tab_SetCurSel Walls_Openings_TabControl1 0)
            )
            (progn
               (setq bOnglet1 nil  bOnglet2 T)
               (Onglet2)
               (Odcl_Tab_SetCurSel Walls_Openings_TabControl1 1)
            )
          )

          (defun c:Openings_TabControl1_OnChanged (nSelIndex /)
            (if (= bBypass nil)
             (TraceIsOn ".Openings_TabControl1_OnChanged")
             (if (= nSelIndex 0)                  ;Tab set for doors
           (progn (setq bOnglet1 T  bOnglet2 nil)(Onglet1)(Odcl_PictureBox_Refresh Walls_Openings_DoorPictureBox))
           (progn (setq bOnglet1 nil  bOnglet2 T)(Onglet2)(Odcl_PictureBox_Refresh Walls_Openings_WindowPictureBox))
             )
            )
          )
        )

      ;;-----------------------------------
        (defun c:Openings_OK_OnClicked ()
          (setq olderror *error*
            *error*  Wallserror
          )
          (Wallsgather_info)

          (setq CLAYER (getvar "CLAYER"))

          (if (= bOnglet1 T)                              ;Tab set for doors
            (progn
          (TraceIsOn ".bOnglet1")

          (Setq bSteel (Odcl_Control_GetProperty Walls_Openings_SteelOption "Value"))
          (if (= bSteel T)
            (setq jambprefix "S"
              sJx    "S"  ;;jambprefix
              ajambtype  "Steel Jamb"
              sJambTyp   "S"
            )
          )
          (Setq bTimb (Odcl_Control_GetProperty Walls_Openings_TimberOption "Value"))
          (if (= bTimb T)
            (setq jambprefix "T"
              sJx    "T"  ;;jambprefix
              ajambtype  "Timber Jamb"
              sJambTyp   "T"
            )
          )
          (Setq bSorT T)

          (Setq bJbThk (Odcl_Control_GetProperty Walls_Openings_JambThkCheckBox "Value"))
          (Setq PercOpen (Odcl_Control_GetProperty Walls_Openings_OpenAngTextBox "Text"))
          (Setq sTr3a (Odcl_ComboBox_GetTBText Walls_Openings_DrWidthComboBox))   ;get displayed width

          (Setq sJambLyr (Odcl_ComboBox_GetTBText Walls_Openings_JambComboBox))
          (Setq sLeafLyr (Odcl_ComboBox_GetTBText Walls_Openings_LeafComboBox))
          (Setq sSwingLyr (Odcl_ComboBox_GetTBText Walls_Openings_SwingComboBox))

      ;   (Setq sGlazeLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnGlassComboBox))
      ;   (Setq sFrameLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnFrameComboBox))
      ;   (Setq sSashLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnSashComboBox))

          (Setq bCenter (Odcl_Control_GetProperty Walls_Openings_CenteredCheckBox "Value"))

          (Setq bHeader (Odcl_Control_GetProperty Walls_Openings_HeaderCheckBox "Value"))
          (if (= bHeader T)
            (progn (Setq bThreshold nil
                     sHeaderLyr (Odcl_ComboBox_GetTBText Walls_Openings_HeaderComboBox)
               )
               (Hshset_lay sHeaderLyr sHeaderCol "ACAD_ISO02W100" sHeaderLwt)
            )
          )

          (Setq bThreshold (Odcl_Control_GetProperty Walls_Openings_ThresholdCheckBox "Value"))
          (if (= bThreshold T)
            (progn (Setq bHeader    nil
                     sThreshLyr (Odcl_ComboBox_GetTBText Walls_Openings_ThresholdComboBox)
               )
               (Hshset_lay sThreshLyr sThreshCol nil sThreshLwt)
            )
          )

          (if (or (= Str2 "SFd")(= Str2 "FSFd")(= Str2 "FSSFd"))  ;AABuild108
           (progn
            (Hshset_lay sGlazeLyr sGlazeCol "CONTINUOUS" sGlazeLwt)
            (Hshset_lay sFrameLyr sFrameCol "CONTINUOUS" sFrameLwt)
            (Hshset_lay sSashLyr sSashCol "CONTINUOUS" sSashLwt)
           )
           (progn
            (Hshset_lay sJambLyr sJambCol "CONTINUOUS" sJambLwt)
            (Hshset_lay sLeafLyr sLeafCol "CONTINUOUS" sLeafLwt)
            (Hshset_lay sSwingLyr sSwingCol "CONTINUOUS" sSwingLwt)
           )
          )

          (Odcl_PictureBox_StoreImage Walls_Openings_WindowPictureBox)
          (setq bDisplay T)
          (Odcl_Form_Close Walls_Openings_Openings)

          (setq sCut "Yes")                           ;"Automatic wall cutting? [Yes/No]
          (setq az1 nil  az2 nil)
          (setvar "osmode" 513)                           ;near & endpoint
          (while (null az1)
            (setvar "APERTURE" 3)
            (setq az1 (uentsl 7
                      "Centered Reference"
                      "Specify interior corner point on wall or [Centered/Reference point]"
                      nil
                  )
            )
          )
          (cond
            ((= "Reference" az1)
                   (setvar "osmode" 1)
               (setq strtpt (upoint 1 "" "Specify reference point" nil nil))
               (setvar "osmode" 512)                      ;nearest
               (setq nextpt (upoint 1 "" "Pick direction along the wall" nil strtpt))
               (setvar "osmode" 0)                        ;none
               (setq dist (udist 6
                         ""
                         "Specify distance from reference point"
                         (distance strtpt nextpt)
                         strtpt
                      )
               )
               (setq direc  (angle strtpt nextpt)
                     p1     (polar strtpt direc dist)
                     bFlag2 T
               )
            )
            ((= "Centered" az1)
                   (setvar "osmode" 1)
               (setq strtpt  (upoint 7 "" "Specify first point" nil nil))
               (setq nextpt  (upoint 7 "" "Specify second point" nil nil))
               (setvar "osmode" 0)                        ;none
               (setq dist  (distance strtpt nextpt)
                     HalfD (* 0.5 dist)
                     direc (angle strtpt nextpt)
                     bFlag2 nil
               )
            )
            (T (setq p1 (cadr az1)  bFlag2 T))                    ;hinge_pt=p1
          )

           (if (= bFlag2 nil) (setq p1 (polar strtpt direc HalfD)) )

           (if (= bMulti T)
            (progn
              (setq lBoxx (BoxItIn p1 (* 5 (atoi sTr3a)) (* 5 (atoi sTr3a)) 0))
          (command "_.'ZOOM" "_W" (nth 1 lBoxx) (nth 3 lBoxx))
            )
           )
          (setvar "osmode" 128)       ;toggle set to perp
          (while (null az2)
           (setq p3  (upoint 1 "" "Select opposite wall" nil p1)
                 az2 (ssgetc p3 0.5))
          )
          (setq rMurThk (distance p1 p3))

          (cond ((and (= bJbThk T) (= bCenter nil)) (setq rJambWidth rMurThk))
                ((and (= bJbThk nil) (= bCenter T))
                 (if (< (distance p1 p3) 130)
               (setq rJambWidth rMurThk
                     rMoveInsPt 0.0
               )
               (setq rJambWidth 130
                     rMoveInsPt (* 0.5 (- rMurThk 130))
               )
                 )
                )
                ((and (= bJbThk nil) (= bCenter nil))
                 (if (< rMurThk 130)
               (setq rJambWidth rMurThk
                     rMoveInsPt 0.0
               )
               (setq rJambWidth 160
                     rMoveInsPt 0.0
               )
                 )
                )
          )

          (setq sOpenAng_c PercOpen
                sOpenAng   PercOpen
                leafwidt_c "30"
                leafwidt   "30"
                OFFDIST    (atoi sTr3a)
                LDR    str2
          )

              (if (= "Centered" az1)
                (progn
               (setq aExter (angle p1 p3)
                     aDifer (rtd (- aExter direc)))
                   (if (minusp aDifer) (setq aDifer (+ aDifer 360)))
               (while (> aDifer 360)                          ;remove multiple of 360 deg
                      (setq aDifer (- aDifer 360.0))
               )
               (setq aDifer (dtr aDifer))
               (if (= 1.5708 (round2dec aDifer 100000))
                       (setq p1 (polar p1 (+ pi direc) (* 0.5 OFFDIST))   p3 (polar p3 (+ pi direc) (* 0.5 OFFDIST)))
                       (setq p1 (polar p1 pi (* 0.5 OFFDIST))  p3 (polar p3 pi (* 0.5 OFFDIST)))
               )
            )
          )

          (setq A1      (angle P1 P3)                 ;ang A1 in rad
                revA1   (angle P3 P1)                 ;reverse ang A1 in rad
                A0      (+ revA1 hshpi90)                 ;ang A0 in rad
                revA0   (+ A1 hshpi90)                    ;reverse ang A0 in rad
                pTCrnr1 (polar p1 revA1 25)               ;Bas ouvrant a droite
                pExtr1a (polar pTCrnr1 revA1 250)
                pExtr1b (polar pTCrnr1 A0 250)
                pExtr1c (polar pTCrnr1 revA0 250)             ;Bas ouvrant a gauche
                pTCrnr3 (polar p3 A1 25)                  ;Haut ouvrant a droite
                pExtr3a (polar pTCrnr3 A1 250)
                pExtr3b (polar pTCrnr3 A0 250)
                pExtr3c (polar pTCrnr3 revA0 250)             ;Haut ouvrant a gauche

                pTCnr1 p1
                pTCnr2 (polar p1 A0 OFFDIST)
                pTCnr3 (polar pTCnr2 A1 rMurThk)
                pTCnr4 (polar pTCnr3 revA0 OFFDIST)

                pTCnr5 p3
                pTCnr6 (polar p3 A0 OFFDIST)
                pTCnr7 (polar pTCnr6 A1 (* -1 rMurThk))
                pTCnr8 (polar pTCnr7 revA0 OFFDIST)

                pTCnr9 p3
                pTCnr10 (polar p3 revA0 OFFDIST)
                pTCnr11 (polar pTCnr10 A1 (* -1 rMurThk))
                pTCnr12 (polar pTCnr11 A0 OFFDIST)

                pTCnr13 p1
                pTCnr14 (polar p1 revA0 OFFDIST)
                pTCnr15 (polar pTCnr14 A1 rMurThk)
                pTCnr16 (polar pTCnr15 A0 OFFDIST)

                OLine3  (ssname (ssget p1) 0)
                LayerIs (vla-get-layer (vlax-ename->vla-object OLine3))               ;Nom du calque pour l'objet
      ;         ent1    (entget OLine3)
      ;         ObjIs   (dxf 0 ent1)
      ;         LayerIs (dxf 8 ent1)
                lname   LayerIs
                sRHing  nil
          )

          (grdraw pTCrnr1 pExtr1a -1 2)
          (grdraw pTCrnr1 pExtr1b -1 2)

          (grdraw pTCnr1 pTCnr2 -1 2)
          (grdraw pTCnr2 pTCnr3 -1 2)
          (grdraw pTCnr3 pTCnr4 -1 2)
          (grdraw pTCnr4 pTCnr1 -1 2)

          (setq iCas 1  pPt1 p1  pPt3 p3  aUsde A1  aUsed A0  XFactor 1  YFactor 1)

          (while (/= sRHing "Place")
            (SETQ sRHing (ukword 6 "Place Rotate" "Place door or [Rotate] hinge: " "Place"))
            (cond
              ((and (= iCas 1)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr3 pExtr3a -1 2)
               (grdraw pTCrnr3 pExtr3b -1 2)

               (grdraw pTCnr5 pTCnr6 -1 2)
               (grdraw pTCnr6 pTCnr7 -1 2)
               (grdraw pTCnr7 pTCnr8 -1 2)
               (grdraw pTCnr8 pTCnr5 -1 2)
               (setq iCas 2  pPt1 p3  pPt3 p1  aUsde revA1  aUsed A0  XFactor 1  YFactor -1)
              )
              ((and (= iCas 2)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr3 pExtr3a -1 2)
               (grdraw pTCrnr3 pExtr3c -1 2)

               (grdraw pTCnr9 pTCnr10 -1 2)
               (grdraw pTCnr10 pTCnr11 -1 2)
               (grdraw pTCnr11 pTCnr12 -1 2)
               (grdraw pTCnr12 pTCnr9 -1 2)
               (setq iCas 3  pPt1 p3  pPt3 p1  aUsde revA1  aUsed revA0  XFactor 1  YFactor 1)
              )
              ((and (= iCas 3)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr1 pExtr1a -1 2)
               (grdraw pTCrnr1 pExtr1c -1 2)

               (grdraw pTCnr13 pTCnr14 -1 2)
               (grdraw pTCnr14 pTCnr15 -1 2)
               (grdraw pTCnr15 pTCnr16 -1 2)
               (grdraw pTCnr16 pTCnr13 -1 2)
               (setq iCas 4  pPt1 p1  pPt3 p3  aUsde A1  aUsed revA0  XFactor 1  YFactor -1)
              )
              ((and (= iCas 4)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr1 pExtr1a -1 2)
               (grdraw pTCrnr1 pExtr1b -1 2)

               (grdraw pTCnr1 pTCnr2 -1 2)
               (grdraw pTCnr2 pTCnr3 -1 2)
               (grdraw pTCnr3 pTCnr4 -1 2)
               (grdraw pTCnr4 pTCnr1 -1 2)
               (setq iCas 1  pPt1 p1  pPt3 p3  aUsde A1  aUsed A0  XFactor 1  YFactor 1)
              )
            )
          )

          (setq p1 pPt1  p3 pPt3  A1 aUsde  A0 aUsed  p2 (polar p1 A0 OFFDIST))

          (setvar "osmode" 0)                 ;toggle off and none

          (if (= sCut "Yes")
            (progn (LineMake p1 p3 LayerIs)
               (setq oCutEdg1 (entlast))
               (SetJambInfoTag sAppNameAA oCutEdg1)

               (setq pP3 (Polar P2 A1 rMurThk))

               (LineMake p2 pP3 LayerIs)
               (setq oCutEdg2 (entlast))
               (SetJambInfoTag sAppNameAA oCutEdg2)

               (setq ssTrim1 (ssadd))
               (ssadd oCutEdg1 ssTrim1)
               (ssadd oCutEdg2 ssTrim1)
               (setq trimpt1 (polar p1 A0 (* 0.5 OFFDIST))
                     trimpt2 (polar p3 A0 (* 0.5 OFFDIST))
                     pTr1    (polar trimpt1 (+ Pi A1) 2.0)
                     pTr2    (polar trimpt2 A1 2.0)
               )
               (command "_.trim" ssTrim1 "" "_F" pTr1 pTr2 "" "")
            )
          )

          (Hshset_lay "DOOR" nil "CONTINUOUS" nil)
          (setq pDoorIns (polar p1 A1 rMoveInsPt))
          (cond ((= LDR "S")     (SingleMk))
                ((= LDR "D")     (DoubleMk))
                ((= LDR "UD")    (UnDblMk))
                ((= LDR "BD")    (BifoldDMk))         ;AA1.3b100
                ((= LDR "BS")    (BifoldSMk))
                ((= LDR "W")     (OpenMk))            ;AABuild108
                ((= LDR "SFd")   (SldFixMkD))         ;AABuild108
                ((= LDR "FSFd")  (FixSldFixMk))           ;AA1.3b100
                ((= LDR "FSSFd") (FixSldSldFixMk))        ;AA1.3b100
                ((= LDR "SSS")   (SurfSlidSMk))
                ((= LDR "SSD")   (SurfSlidDMk))           ;AA1.3b100
                ((= LDR "CSS")   (CavSlidSMk))
                ((= LDR "CSD")   (CavSlidDMk))            ;AA1.3b100
          )

          (entmake (list (cons 0 "INSERT")
                     (cons 2 BlockNameD)
                     (cons 10 pDoorIns)
                     (cons 41 XFactor)
                     (cons 42 YFactor)
                     (cons 50 A0)
               )
          )

          (if (or (= LDR "SFd")(= LDR "FSF9"))            ;AABuild108
           (progn
            (setq str2W str2  strW3a str3a  bTransom nil  sSb "2100" sSeuil4 "2100" sWnHand (dxf 5 (entget (entlast))))
                (SetWindInfoTag sAppNameAA (entlast) nil)
               )
           (progn
            (setq sDrHand (dxf 5 (entget (entlast))))
            (SetDoorInfoTag sAppNameAA (entlast) nil)
           )
          )

              (if (= bMulti T)(command "_.'ZOOM" "_P") )

          (if (< rMurThk 230)
            (setq bThreshold nil)
          )
          (if (= bThreshold T)
            (progn (ThresholdMk)
               (entmake (list (cons 0 "INSERT")
                      (cons 2 BlockNameD2)
                      (cons 10 p1)
                      (cons 41 XFactor)
                      (cons 42 YFactor)
                      (cons 50 A0)
                    )
               )
      ;;                 (setq rSpace2 (/ OffDist (fix (/ OffDist 100)) 1.0))
      ;;       (setvar "snapbase" (list (car pDoorIns) (cadr pDoorIns)))
      ;;       (Hshset_lay sThreshLyr "23" nil "0.25")
      ;;       (command ".-BHATCH" "_P" "_U" (rtd A1) rSpace2 "_N" "_S" (entlast) "" "")
            )
          )
          (setq bTab 0)
            )
          )
      ;;-----------------------------------
          (if (= bOnglet2 T)                              ;Tab set for windows
            (progn                                    ;Fenetres
          (TraceIsOn ".bOnglet2")

          (Setq sTrW3a (Odcl_ComboBox_GetTBText Walls_Openings_WnWidthComboBox))  ;get displayed width
          (Setq sTr4b  (Odcl_ComboBox_GetTBText Walls_Openings_WnHeightComboBox)) ;get displayed height

          (Setq bJbThk nil  sSb sTr4b)
          (Setq bCenter nil)

              (Setq bSillOut (Odcl_Control_GetProperty Walls_Openings_OutlineOption "Value"))
              (Setq bSillHat (Odcl_Control_GetProperty Walls_Openings_HatchSillOption "Value"))

          (Setq sGlazeLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnGlassComboBox))
          (Setq sFrameLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnFrameComboBox))
          (Setq sSashLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnSashComboBox))
          (Hshset_lay sGlazeLyr sGlazeCol "CONTINUOUS" sGlazeLwt)
          (Hshset_lay sFrameLyr sFrameCol "CONTINUOUS" sFrameLwt)
          (Hshset_lay sSashLyr sSashCol "CONTINUOUS" sSashLwt)

          (Setq bHeaderW (Odcl_Control_GetProperty Walls_Openings_WnHeaderCheckBox "Value"))
          (if (= bHeaderW T)
            (progn (Setq bSill      nil
                     sWHeaderLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnHeaderComboBox)
               )
               (Hshset_lay sWHeaderLyr sWHeaderCol "ACAD_ISO02W100" sWHeaderLwt)
            )
          )

          (Setq bSill (Odcl_Control_GetProperty Walls_Openings_WnSillCheckBox "Value"))
          (if (= bSill T)
            (progn (Setq bHeaderW   nil
                     sSillLyr (Odcl_ComboBox_GetTBText Walls_Openings_WnSillComboBox)
               )
               (Hshset_lay sSillLyr sSillCol nil sSillLwt)
            )
          )

      ;if height less than 1600 and transom asked, deny.
          (Setq bTransom (Odcl_Control_GetProperty Walls_Openings_TransomCheckBox "Value"))
              (if (and (= bTransom T)(< (atoi sTr4b) (atof sSeuil4)))
           (setq bTransom nil)
              )

          (setq bDisplay T  PercOpen "90")
          (Odcl_Form_Close Walls_Openings_Openings)

          (setq sCut "Yes")                           ;"Automatic wall cutting? [Yes/No]
          (setq az1 nil  az2 nil)
          (setvar "osmode" 513)                           ;near & endpoint
          (while (null az1)
            (setvar "APERTURE" 3)
            (setq az1 (uentsl 7
                      "Centered Reference"
                      "Specify interior corner point on wall or [Centered/Reference point]"
                      nil
                  )
            )
          )
          (cond
            ((= "Reference" az1)
                   (setvar "osmode" 1)
               (setq strtpt (upoint 1 "" "Specify reference point" nil nil))
               (setvar "osmode" 512)                      ;nearest
               (setq nextpt (upoint 1 "" "Pick direction along the wall" nil strtpt))
               (setvar "osmode" 0)                        ;none
               (setq dist (udist 6
                         ""
                         "Specify distance from reference point"
                         (distance strtpt nextpt)
                         strtpt
                      )
               )
               (setq direc  (angle strtpt nextpt)
                     p1     (polar strtpt direc dist)
                     bFlag2 T
               )
            )
            ((= "Centered" az1)
                   (setvar "osmode" 1)
               (setq strtpt  (upoint 7 "" "Specify first point" nil nil))
               (setq nextpt  (upoint 7 "" "Specify second point" nil nil))
               (setvar "osmode" 0)                        ;none
               (setq dist  (distance strtpt nextpt)
                     HalfD (* 0.5 dist)
                     direc (angle strtpt nextpt)
                     bFlag2 nil
               )
            )
            (T (setq p1 (cadr az1)  bFlag2 T))                    ;hinge_pt=p1
          )

           (if (= bFlag2 nil) (setq p1 (polar strtpt direc HalfD)) )

           (if (= bMulti T)
            (progn
              (setq lBoxx (BoxItIn p1 (* 4 (atoi sTrW3a)) (* 4 (atoi sTrW3a)) 0))
          (command "_.'ZOOM" "_W" (nth 1 lBoxx) (nth 3 lBoxx))
            )
           )

      ;check if object is a curved wall
              (if (= "ARC" (dxf 0 (entget (car (nentselp p1)))))
           (setq bCurvWall T)
           (setq bCurvWall nil)
              )

          (setvar "osmode" 128)                           ;toggle set to perp
          (while (null az2)
           (setq p3  (upoint 1 "" "Select opposite wall" nil p1)
                 az2 (ssgetc p3 0.5)
           )
          )

          (setq rMurThk (distance p1 p3))
          (cond ((and (= bJbThk T) (= bCenter nil)) (setq rJambWidth rMurThk))
                ((and (= bJbThk nil) (= bCenter T))
                 (if (< (distance p1 p3) 130)
               (setq rJambWidth rMurThk
                     rMoveInsPt 0.0
               )
               (setq rJambWidth 130
                     rMoveInsPt (* 0.5 (- rMurThk 130))
               )
                 )
                )
                ((and (= bJbThk nil) (= bCenter nil))
                 (if (< rMurThk 130)
               (setq rJambWidth rMurThk
                     rMoveInsPt 0.0
               )
               (setq rJambWidth 160
                     rMoveInsPt 0.0
               )
                 )
                )
          )
          (setq sOpenAng_c PercOpen
                sOpenAng   PercOpen
                leafwidt_c "30"
                leafwidt   "30"
                LWn    sTr2W
                OFFDIST    (atoi sTrW3a)
          )

              (if (= "Centered" az1)
                (progn
               (setq aExter (angle p1 p3)
                     aDifer (rtd (- aExter direc)))
                   (if (minusp aDifer) (setq aDifer (+ aDifer 360)))
               (while (> aDifer 360)                          ;remove multiple of 360 deg
                      (setq aDifer (- aDifer 360.0))
               )
               (setq aDifer (dtr aDifer))
               (if (= 1.5708 (round2dec aDifer 100000))
                       (setq p1 (polar p1 (+ pi direc) (* 0.5 OFFDIST))   p3 (polar p3 (+ pi direc) (* 0.5 OFFDIST)))
                       (setq p1 (polar p1 pi (* 0.5 OFFDIST))  p3 (polar p3 pi (* 0.5 OFFDIST)))
               )
            )
          )

          (setvar "osmode" 512)                       ;nearest

          (if bCurvWall
           (setq rArcRad (dxf 40 (entget (car (nentselp p1))))
                 pCentre (dxf 10 (entget (car (nentselp p1))))
                 aInclus (incang OFFDIST rArcRad)             ;in Radians
                 aDepart (angle pCentre p1)               ;Radians
                 aArrive (+ aDepart aInclus)
                 p2      (polar pCentre aArrive rArcRad)
           )
              )

          (setq A1      (angle P1 P3)                 ;ang A1 in rad
                revA1   (angle P3 P1)                 ;reverse ang A1 in rad
                A0      (+ revA1 hshpi90)                 ;ang A0 in rad
                revA0   (+ A1 hshpi90)                    ;reverse ang A0 in rad
                pTCrnr1 p1
                pTCrnr2 (polar p1 A0 OFFDIST)
                pTCrnr3 (polar pTCrnr2 A1 160)
                pTCrnr4 (polar pTCrnr3 revA0 OFFDIST)

                pTCrnr5 p3
                pTCrnr6 (polar p3 A0 OFFDIST)
                pTCrnr7 (polar pTCrnr6 A1 -160)
                pTCrnr8 (polar pTCrnr7 revA0 OFFDIST)

                pTCrnr9 p3
                pTCrnr10 (polar p3 revA0 OFFDIST)
                pTCrnr11 (polar pTCrnr10 A1 -160)
                pTCrnr12 (polar pTCrnr11 A0 OFFDIST)

                pTCrnr13 p1
                pTCrnr14 (polar p1 revA0 OFFDIST)
                pTCrnr15 (polar pTCrnr14 A1 160)
                pTCrnr16 (polar pTCrnr15 A0 OFFDIST)

                OLine3  (ssname (ssget p1) 0)
                LayerIs (vla-get-layer (vlax-ename->vla-object OLine3))               ;Nom du calque pour l'objet
      ;         ent1    (entget OLine3)
      ;         ObjIs   (dxf 0 ent1)
      ;         LayerIs (dxf 8 ent1)
                lname   LayerIs
                sRHing  nil
          )

          (grdraw pTCrnr1 pTCrnr2 202 0)
          (grdraw pTCrnr2 pTCrnr3 -1 2)
          (grdraw pTCrnr3 pTCrnr4 -1 2)
          (grdraw pTCrnr4 pTCrnr1 202 0)

          (setq iCas 1  pPt1 p1  pPt3 p3  aUsde A1  aUsed A0  XFactor 1  YFactor 1)

          (while (/= sRHing "Place")
            (SETQ sRHing (ukword 6 "Place Rotate" "Place window or [Rotate] corner: " "Place"))
            (cond
              ((and (= iCas 1)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr5 pTCrnr6 202 0)
               (grdraw pTCrnr6 pTCrnr7 -1 2)
               (grdraw pTCrnr7 pTCrnr8 -1 2)
               (grdraw pTCrnr8 pTCrnr5 202 0)
               (setq iCas 2  pPt1 p3  pPt3 p1  aUsde revA1  aUsed A0  XFactor 1  YFactor -1)
              )
              ((and (= iCas 2)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr9 pTCrnr10 202 0)
               (grdraw pTCrnr10 pTCrnr11 -1 2)
               (grdraw pTCrnr11 pTCrnr12 -1 2)
               (grdraw pTCrnr12 pTCrnr9 202 0)
               (setq iCas 3  pPt1 p3  pPt3 p1  aUsde revA1  aUsed revA0  XFactor 1  YFactor 1)
              )
              ((and (= iCas 3)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr13 pTCrnr14 202 0)
               (grdraw pTCrnr14 pTCrnr15 -1 2)
               (grdraw pTCrnr15 pTCrnr16 -1 2)
               (grdraw pTCrnr16 pTCrnr13 202 0)
               (setq iCas 4  pPt1 p1  pPt3 p3  aUsde A1  aUsed revA0  XFactor 1  YFactor -1)
              )
              ((and (= iCas 4)(= sRHing "Rotate"))
                   (redraw)
               (grdraw pTCrnr1 pTCrnr2 202 0)
               (grdraw pTCrnr2 pTCrnr3 -1 2)
               (grdraw pTCrnr3 pTCrnr4 -1 2)
               (grdraw pTCrnr4 pTCrnr1 202 0)
               (setq iCas 1  pPt1 p1  pPt3 p3  aUsde A1  aUsed A0  XFactor 1  YFactor 1)
              )
            )
          )

          (setq p1 pPt1  p3 pPt3  A1 aUsde  A0 aUsed  p2 (polar p1 A0 OFFDIST))

          (setvar "osmode" 0)     ;toggle off and none

          (if (= sCut "Yes")
            (progn (LineMake p1 p3 LayerIs)
               (setq oCutEdg1 (entlast))
               (SetJambInfoTag sAppNameAA oCutEdg1)

               (setq pP3 (Polar P2 A1 rMurThk))

               (LineMake p2 pP3 LayerIs)
               (setq oCutEdg2 (entlast))
               (SetJambInfoTag sAppNameAA oCutEdg2)

               (setq ssTrim1 (ssadd))
               (ssadd oCutEdg1 ssTrim1)
               (ssadd oCutEdg2 ssTrim1)
               (setq trimpt1 (polar p1 A0 (* 0.5 OFFDIST))
                     trimpt2 (polar p3 A0 (* 0.5 OFFDIST))
                     pTr1    (polar trimpt1 (+ Pi A1) 2.0)
                     pTr2    (polar trimpt2 A1 2.0)
               )
               (command "_.trim" ssTrim1 "" "_F" pTr1 pTr2 "" "")
            )
          )
          (Hshset_lay "WIND" nil "CONTINUOUS" nil)
          (setq pWindIns (polar p1 A1 rMoveInsPt))
          (cond ((= LWn "FW")  (FixedMk))             ;Fixed window
                ((= LWn "SF")  (SldFixMk))            ;
                ((= LWn "SFS") (SldFixSldMk))
                ((= LWn "DH")  (DblHungMk))           ;
          )

          (if bCurvWall
           (setq A0 (angle p1 p2))
              )

          (entmake (list (cons 0 "INSERT")
                     (cons 2 BlockNameW)
                     (cons 10 pWindIns)
                     (cons 41 XFactor)
                     (cons 42 YFactor)
                     (cons 50 A0)
               )
          )
          (setq sWnHand (dxf 5 (entget (entlast))))
              (SetWindInfoTag sAppNameAA (entlast) nil)
              (if (= bMulti T)(command "_.'ZOOM" "_P") )
          (if (< rMurThk 230)
            (setq bSill nil)
          )
          (setq bTab 1)
            )
          )
          (setq bDisplay T)
          (setq *error* OLDERROR)
          (reset nil) (resset varlst)
        )

        (defun c:Openings_about_OnClicked () (_about) (princ))

        (defun c:Openings_OnClose (nUpperLeftX nUpperLeftY /) (princ))

        (if (= (Odcl_LoadProject ProjName2 nil) -10)
          (progn
            (WsFlashMess1
          "Echec chargement bote de dialogue. Fichier Walls.ods introuvable.\nSi la location a change, la mise  jour de MetricArchAssist2004.ini est indispensable."
          "Archivio Walls.ods non trovato. Se il percorso modificasse da ultima installazione,\nl'aggiornamento di MetricArchAssist2004.ini  indispensabile."
          "Archivo Walls.ods no ha sido encontrado. Si la ubicacion es modificada\ndesde la ultima instalacion, archivo MetricArchAssist2004.ini tendra que ser actualizado con su nueva ubicacion."
          "File Walls.ods not found. If Path modified since last\ninstallation, update MetricArchAssist2004.ini file."
          1 5)
            (exit)
          )
         (setq bDisplay T)
        )

        (if (= bMulti T)
         (while bDisplay
          (setq bDisplay nil)
          (princ (strcat "\nOpenings." copyright))
          (Odcl_Form_Show Walls_Openings_Openings)
         )
          (Odcl_Form_Show Walls_Openings_Openings)
        )
      )

      (TraceIsOn ".19h")

      ;----------------------------
      ;---ZAPS DUPLICATED objects.
      ;----------------------------
      (defun DelNulObj (/ ss sslen cnt astring)
        (if (setq ss (ssget "_X" (list (cons 0 "TEXT"))))
          (progn (setq sslen (sslength ss)
               cnt   0
             )
             (prompt "\nText processed for null string(s).")
             (repeat sslen
               (setq sslen   (1- sslen)
                 astring (cdr (assoc 1 (entget (ssname ss sslen))))
               )
               (if (or (= "" astring) (allSpaces astring))
                 (progn (entdel (ssname ss sslen)) (setq cnt (1+ cnt)))
               )
             )
             (prompt (strcat "\tDeleted: " (itoa cnt)))
          )
        )
        (princ)
      )

      (defun allSpaces (astring / cnt space)
        (setq cnt 1
          space T
        )
        (repeat (strlen astring)
          (if (/= " " (substr astring cnt 1))
            (setq space nil)
          )
          (setq cnt (1+ cnt))
        )
        space
      )

      (defun ZAPerror (msg#)
        (setq *error* OLDERROR)
        (command)             ;Cancel any pending command and end any UNDO group
        (command)
        (command)
        (command "._UNDO" "_E")
        (if (= bWasRotated T)
          (command "_.ROTATE" (ssget "_X") "" "0,0" "-0.000001")
        )
        (dos_waitcursor)
        (reset nil)
        (resset varlst)
        (if (member msg# '("console break" "Function cancelled" "Error: quit / exit abort"))
          (princ " ")
          (princ (strcat "\nError " (itoa (getvar "ERRNO")) ": " msg# "\n"))
                          ;Otherwise, print an error indication
        )
        (princ)
      )

      (defun C:ZAP (/      Borrar
                ss     en
                en_n   el_n
                ei     en_i
                el_i   elist
                etyp   lay
                llist  ans
                n      n1
                i      d
                done   HandlesOff
                xdata  fuzz
                p10    p11
                elr    ans1
               )
        (setq OLDERROR *error*
          *error*  ZAPerror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nEliminate Duplicate Objects." copyright))
        (setq i  0
          n1 0
          d  0
        )
        (defun Borrar ()
          (TraceIsOn ".Borrar")
          (setq ss (ssdel ei ss)
            d  (1+ d)
            n  (1- n)
            i  (1- i)
          )
          (entdel ei)
        )
        (defun HandlesOff (el) (TraceIsOn ".HandlesOff") (subst (cons 5 "0") (assoc 5 el) el))
        (defun Get_ObjectType (e)
          (TraceIsOn ".Get_ObjectType")
          (if xdata
            (entget e '("*"))
            (entget e)
          )
        )
        (princ "Enter type of duplicate objects to eliminate\n")
        (setq ans
           (ukword
             1
             "3DFace Arc Circle Insert Line Mline MText PLine POint Ray SOlid SPline Text Xline *"
             "[3DFace/Arc/Circle/Insert/Line/MLine/MText/PLine/POint/Ray/SOlid/SPline/Text/Xline/*]: "
             "*"
           )
        )
        (setq ans1 (ukword 1
                   "NUll Wipeout NOne Both"
                   "Specify objects to delete [NUll string/Wipeout/Both] or "
                   "NOne"
               )
        )
        (if (= ans1 "Null")
          (delnulobj)
        )
        (if (= ans1 "Wipeout")
          (progn (setvar "pickstyle" 0)
             (setq b (ssget "_X" (list (cons 0 "WIPEOUT"))))
             (if b
               (command "_.ERASE" b "")
               (princ "\nNo WIPEOUT objects found.")
             )
          )
        )
        (if (= ans1 "Both")
          (progn (delnulobj)
             (setvar "pickstyle" 0)
             (setq b (ssget "_X" (list (cons 0 "WIPEOUT"))))
             (if b
               (command "_.ERASE" b "")
               (princ "\nNo WIPEOUT objects found.")
             )
          )
        )
        (if (= ans "PLine")
          (setq ans "*Polyline")
        )
        (if (or (not ans) (= ans "*"))
          (setq ss (ssget "_X"))      ;Do All
          (setq etyp (strcase ans)        ;Do selectively
            ss   (ssget "_X" (list (cons 0 etyp)))
          )
        )
        (if ss
          (progn (if etyp
               (progn (prompt (strcat "\n" (itoa (sslength ss)) " " etyp " objects found.\n")
                  )
                  (setq elist (list etyp))
               )
               (progn (prompt (strcat "\n" (itoa (sslength ss)) " objects found.\n"))
                  (prompt "Sorting objects by type...\r")
                  (while (< i (sslength ss))
                    (setq etyp (dxf 0 (entget (ssname ss i)))
                      i    (1+ i)
                    )
                    (if (not (member etyp elist))
                  (setq elist (cons etyp elist))
                    )
                  )
               )
             )
                          ;         (initget "Yes No")
                          ;         (setq xdata (= "Yes" (getkword "\nInclude 'XDATA' comparisons?  Yes/<No>: ")))
             (if (= rFFactor nil)
               (setq rFFactor 0)
             )
             (setq rFFactor (ureal 4 "" "Fuzz factor for comparisons" rFFactor)) ;4=no negative
             (setq fuzz rFFactor)
             (if (/= rFFactor 0)
               (progn (command "_.ROTATE" (ssget "_X") "" "0,0" "0.000001") (Setq bWasRotated T))
               (Setq bWasRotated nil)
             )
             (foreach etyp (reverse elist)
               (prompt (strcat "\rSorting " etyp " objects by layer...\n"))
               (if (> (length elist) 1)
                 (setq ss (ssget "_X" (list (cons 0 etyp))))
               )
               (setq i 0
                 llist nil
               )
               (while (< i (sslength ss))
                 (setq lay (dxf 8 (entget (ssname ss i)))
                   i   (1+ i)
                 )
                 (if (not (member lay llist))
               (setq llist (cons lay llist))
                 )
               )
               (foreach lay (reverse llist)
                 (setq ss (ssget "_X" (list (cons 0 etyp) (cons 8 lay)))
                   n  (sslength ss)
                   i  0
                 )
                 (while (> n 0)
               (setq n  (1- n)
                     n1 (1+ n1)
               )
               (prompt (strcat "\rProcessing " etyp " on layer: " lay))
               (setq tot (strcat "Objects checked: " (itoa n1)))
               (command "_.modemacro" TOT)
               (setq i    (1- n)
                     en   (ssname ss n)
                     el   (HandlesOff (cdr (Get_ObjectType en)))
                     el_n el
               )
               (while (> i -1)
                 ;;  (show_ad)
                 (setq ei   (ssname ss i)
                   el_i (HandlesOff (cdr (Get_ObjectType ei)))
                 )
                 (cond ((= etyp (dxf 0 el_i) "LINE") ;Si c'est une ligne
                    (setq p10 (cons 10 (dxf 11 el_n))
                      p11 (cons 11 (dxf 10 el_n))
                      elr (subst p10 (assoc 10 el_n) el_n)
                      elr (subst p11 (assoc 11 el_n) elr)
                    )
                    (if (or (equal el_n el_i fuzz) (equal elr el_i fuzz))
                      (Borrar)
                      (setq i (1- i))
                    )
                   )
                   ((not (equal el_n el_i fuzz)) (setq i (1- i)))
                   ((and (assoc 66 el_n) (assoc 66 el_i))
                    (setq done nil
                      en_n en
                      en_i ei
                    )
                    (while (not done)
                      (setq en_n (entnext en_n)
                        el_n (HandlesOff (cdr (Get_ObjectType en_n)))
                        en_i (entnext en_i)
                        el_i (HandlesOff (cdr (Get_ObjectType en_i)))
                      )
                      (cond ((equal el_n el_i fuzz))
                        ((and (= (cdar el_n) "SEQEND") (= (cdar el_i) "SEQEND"))
                         (setq done 1)
                         (Borrar)
                        )
                        ((not (equal el_n el_i fuzz))
                         (setq done 1
                           i    (1- i)
                         )
                        )
                      )
                    )
                    (setq el_n el)
                   )
                   (T (Borrar))
                 )
               )
                 )
               )
             )
             (if (/= rFFactor 0)
               (progn (command "_.ROTATE" (ssget "_X") "" "0,0" "-0.000001") (Setq bWasRotated nil))
             )
             (setq tot (strcat (itoa d) " Duplicate deleted!"))
             (command "_.modemacro" TOT)
             (if (> d 0)
               (progn (redraw))
             )
          )
          (if (or (not ans) (= ans "*"))
            (prompt "\nNo objects found.")
            (prompt (strcat "\nNo " etyp " objects found."))
          )
        )
        (princ "\n")
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (TraceIsOn ".19i")

      ;------------------
      ; Arrow for stairs.
      ;------------------
      (defun C:AS () (AAArrow))

      (defun LaPointe ()
        (TraceIsOn ".LaPointe")
        (entmake
          '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0))
        )
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 "0")
               (cons 100 "AcDbPolyline")
               (cons 90 3)
               (cons 70 0)
               (cons 43 0.2)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 -1.13 1.95)
               (cons 40 0.2)
               (cons 41 0.2)
               (cons 42 0.0)
               (list 10 0 0)
               (cons 40 0.2)
               (cons 41 0.2)
               (cons 42 0.0)
               (list 10 -1.13 -1.95)
               (cons 40 0.2)
               (cons 41 0.2)
               (cons 42 0.0)
               (list 210 0.0 0.0 1.0)
             )
        )
        (setq sBlockName6f (entmake '((0 . "ENDBLK"))))
        (entmake (list (cons 0 "INSERT")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sSymbol35Lyr)
               (cons 2 sBlockName6f)
               (cons 10 pMid1)
               (cons 41 rScl)     ;width
               (cons 42 rScl)
               (cons 50 aRot1)
               (list 210 0.0 0.0 1.0)
             )
        )
      )

      (defun AAArrow (/)
        (setq olderror *error*
          *error*  Wallserror
          pNum1    nil
          pNum2    nil
          pMid     nil
          pMid1    nil
          rScl     nil
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (command "_.UCS" "_W")
        (setvar "ORTHOMODE" 0)
        (princ (strcat "\nArrow for Stairs." copyright))
        (prompt "\nArrow placed mid-distance between next 2 points:"
        )
        (setvar "osmode" 1)           ;toggle 16384+1 for Endp
        (setq pNum1 (upoint 8 "" "\t\t\tSpecify 1st point" nil nil)
          cnt   0
        )
        (if (/= pNum1 nil)
          (progn
            (Hshset_lay
          sSymbol35Lyr
          sSymbol35Col
          "CONTINUOUS"
          sSymbol35Lwt
            )
            (setq pNum2 (upoint 8 "" "\t\t\tSpecify 2nd point" nil nil)
              pMid  (mid_pt pNum1 pNum2)
              pMid1 pMid
              rScl  (fix (getvar "DIMSCALE"))
            )
            (CarryOn)
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (Defun CarryOn ()
        (TraceIsOn ".CarryOn")
        (while (and (/= pNum1 nil) (/= pNum2 nil))
          (if (= cnt 1)
            (progn
          (setq aRot1 (angle pMid pMid1))
          (LaPointe)
          (command "_.PLINE" pMid1 pMid)
            )
          )
          (prompt
            "\nVertex placed mid-distance between next 2 points:"
          )
          (setvar "osmode" 1)
          (setq pNum1 (upoint 8           "Undo"
                  "\t\t\tSpecify 1st point or <exit>"
                  nil         pMid
                     )
            cnt   (1+ cnt)
          )
          (if (/= pNum1 nil)
            (progn (setq pNum2 (upoint 8
                       ""
                       "\t\t\tSpecify 2nd point or <exit>"
                       nil
                       nil
                      )
                 pMid  (mid_pt pNum1 pNum2)
               )
               (setvar "osmode" 0)    ;toggle off and none
               (command pMid)
            )
            (progn
          (command "")
            )
          )
        )
      )

      ;-----------------------
      ; Sets SNAPANG to angle.
      ;-----------------------
      (defun C:RXH () (AAXHair))

      ;; Sets SNAPANG to angle of selected LINE, LWPOLYLINE, POLYLINE, XLINE
      ; or LEADER segment. Works for subentities (nested in BLOCK or XREF) as well.
      (defun AAXHair ( / a)
        (setq olderror *error*  *error* Wallserror  ss1 nil)
        (Wallsgather_info)
         (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
          (progn
           (FlashMess3
             "ArchAssist maintenant Actif."
             "ArchAssist now Active."
             "ArchAssist now Active."
             "ArchAssist now Active."
           )
          )
         )
         (WsFlashMess2
          (strcat "\nRotate Crosshair Angle." copyright)
          (strcat "\nRotate Crosshair Angle." copyright)
          (strcat "\nRotate Crosshair Angle." copyright)
          (strcat "\nRotate Crosshair Angle." copyright)
         )
        (if (setq a (XhairSw))
          (setvar "snapang" a)
        )
        (setq *error* OLDERROR)
        (reset nil)
        ;;(resset varlst)
      )

      (defun XhairSw (/ e et ed p1 p2 aptr l_ang)
        (if (or (while (progn (initget "Angle Reset")
                  (setq ne (nentsel "\nSelect a linear object or [Angle/Reset to 0]: "))
                  (cond ((not ne) (princ "0 found"))
                        ((= ne "Reset") (setq l_ang 0) nil)
                        ((= ne "Angle") (not (setq l_ang (uangle 1 "" "Specify angle" nil nil))))
                        ((listp ne)
                         (setq e  (car ne)
                           ed (entget e)
                           et (cdr (assoc 0 ed))
                         )
                         (if (not (wcmatch et "*LINE,VERTEX,LEADER,HATCH"))
                       (princ "\nSelected object must be a line/pline/xline/block/dimension/hatch/leader/xref.")
                         )
                        )
                  )
               )
            )
            (/= ne "")
            )
          (or l_ang
          (progn (setq p1    (osnap (cadr ne) "end")
                   p2    (osnap (cadr ne) "mid")
                   l_ang (angle p1 p2)
                 )
          )
          )
        )
        l_ang
      )

      ;-----------------------
      ;---ERASE in Real-time.
      ;-----------------------
      (defun C:EM () (AAEraser))

      (defun AAEraser (/ value tst Cont OffOn a e)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (princ (strcat "\nErase Realtime." copyright))
        (setvar "COORDS" 0)           ;COORDS sysvar is now saved in the registry
        (princ
          "\nLeft-Click=Toggle On/Off, numeric keypad 2 and 8 to alter pickbox.\nU=Undo. Place crosshairs on object(s) to erase or <Exit>: "
        )
        (command "_.undo" "_g" "_.point" "0,0")
        (setq oLastDB (entdel (entlast)))
        (setq Cont T
          OffOn 0
        )
        (while Cont
          (setq got nil)
          (while (not got)
            (setq tst (grread T 9 2))                     ;renvoi
            (setvar "HIGHLIGHT" 1)
            (setq code  (car tst)
              value (cadr tst)
            )

            (if (or (= value 0) (= value 13) (= value 27) (= value 32))   ;13=CR or "\r"   32=space or " "   27=ESC or "\e"
          (progn (setvar "COORDS" 1)
                 (if e
               (redraw e)
                 )
                 (setq Cont nil
                   tst  '(2 4 6)
                   got  T
                 )
          )
            )
            (cond ((or (= value 85) (= value 117))
               (if (entnext oLastDB)
                 (progn (entdel (entlast)) (princ "\tCommand has been completely undone."))
                 (Command "_.U")
               )
              )
            )
            (if (= code 2)
          (cond ((= value 50)
                 (setq pikbox (- (getvar "PICKBOX") 1))
                 (if (<= pikbox 1)
               (setq pikbox 1)
                 )
                 (setvar "PICKBOX" pikbox)
                )                             ;50=2 on keypad
                ((= value 56)
                 (setq pikbox (+ (getvar "PICKBOX") 1))
                 (if (>= pikbox 26)
               (setq pikbox 26)
                 )
                 (setvar "PICKBOX" pikbox)
                )                             ;56=8 on keypad
          )
            )
            (if (= code 3)
          (setq OffOn (- 1 OffOn))
            )
            (if (= (type value) 'LIST)
          (setq got (ssget value))
            )
          )
          (if (and (= Cont T) (= code 5) (= 0 OffOn))
            (progn (setq a (ssname got 0))
               (if e
                 (redraw e)
               )
               (redraw a 3)
               (setq e a
                 b (entget a)
               )
               (command "_.ERASE" e "")
               (setq got nil)
            )
          )
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;-----------------------
      ;---TRIM in Real-time.
      ;-----------------------
      (defun C:TM () (AATrim))

      (defun AATrim (/ value tst Cont OffOn a e)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (princ (strcat "\nTrim Realtime." copyright))
        (setvar "COORDS" 0)           ;COORDS sysvar is now saved in the registry
        (princ "\nSelect cutting edges ...")
        (setq ss (getall))
        (princ "\nLeft-Click=Toggle On/Off, numeric keypad 2 and 8 to alter pickbox.\nPlace crosshairs on object(s) to trim or <Exit>: ")
        (command "_.undo" "_g" "_.point" "0,0")
        (setq oLastDB (entdel (entlast)))
        (setq Cont T
          OffOn 0
        )
        (while Cont
          (setq got nil)
          (while (not got)
            (setq tst (grread T 9 2))             ;renvoi
            (setvar "HIGHLIGHT" 1)
            (setq code  (car tst)
              value (cadr tst)
            )

            (if (or (= value 0) (= value 13) (= value 27) (= value 32))   ;13=CR or "\r"   32=space or " "   27=ESC or "\e"
          (progn (setvar "COORDS" 1)
                 (if e
               (redraw e)
                 )
                 (setq Cont nil
                   tst  '(2 4 6)
                   got  T
                 )
          )
            )
            (cond ((or (= value 85) (= value 117))
               (if (entnext oLastDB)
                 (progn (entdel (entlast)) (princ "\tCommand has been completely undone."))
                 (Command "_.U")
               )
              )
            )
            (if (= code 2)
          (cond ((= value 50)
                 (setq pikbox (- (getvar "PICKBOX") 1))
                 (if (<= pikbox 1)
               (setq pikbox 1)
                 )
                 (setvar "PICKBOX" pikbox)
                )                     ;50=2 on keypad
                ((= value 56)
                 (setq pikbox (+ (getvar "PICKBOX") 1))
                 (if (>= pikbox 26)
               (setq pikbox 26)
                 )
                 (setvar "PICKBOX" pikbox)
                )                     ;56=8 on keypad
          )
            )
            (if (= code 3)
          (setq OffOn (- 1 OffOn))
            )
            (if (= (type value) 'LIST)
          (setq got (ssget value))
            )
          )
          (if (and (= Cont T) (= code 5) (= 0 OffOn))
            (progn (setq a (ssname got 0))
               (if e
                 (redraw e)
               )
               (redraw a 3)
               (setq e a)
               (command "_.TRIM" ss "" value "")
               (setq got nil)
            )
          )
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;-----------------------
      ;---Extend in Real-time.
      ;-----------------------
      (defun C:XM () (AAExtend))

      (defun AAExtend (/ value tst Cont OffOn a e)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (princ (strcat "\nTrim Realtime." copyright))
        (setvar "COORDS" 0)           ;COORDS sysvar is now saved in the registry
        (princ "\nSelect boundary edges ...")
        (setq ss (getall))
        (princ "\nLeft-Click=Toggle On/Off, numeric keypad 2 and 8 to alter pickbox.\nPlace crosshairs on object(s) to extend or <Exit>: ")
        (command "_.undo" "_g" "_.point" "0,0")
        (setq oLastDB (entdel (entlast)))
        (setq Cont T
          OffOn 0
        )
        (while Cont
          (setq got nil)
          (while (not got)
            (setq tst (grread T 9 2))             ;renvoi
            (setvar "HIGHLIGHT" 1)
            (setq code  (car tst)
              value (cadr tst)
            )

            (if (or (= value 0) (= value 13) (= value 27) (= value 32))   ;13=CR or "\r"   32=space or " "   27=ESC or "\e"
          (progn (setvar "COORDS" 1)
                 (if e
               (redraw e)
                 )
                 (setq Cont nil
                   tst  '(2 4 6)
                   got  T
                 )
          )
            )

            (cond ((or (= value 85) (= value 117))
               (if (entnext oLastDB)
                 (progn (entdel (entlast)) (princ "\tCommand has been completely undone."))
                 (Command "_.U")
               )
              )
            )

            (if (= code 2)
          (cond ((= value 50)
                 (setq pikbox (- (getvar "PICKBOX") 1))
                 (if (<= pikbox 1)
               (setq pikbox 1)
                 )
                 (setvar "PICKBOX" pikbox)
                )                     ;50="2" on keypad
                ((= value 56)
                 (setq pikbox (+ (getvar "PICKBOX") 1))
                 (if (>= pikbox 26)
               (setq pikbox 26)
                 )
                 (setvar "PICKBOX" pikbox)
                )                     ;56=8 on keypad
          )
            )
            (if (= code 3)
          (setq OffOn (- 1 OffOn))
            )
            (if (= (type value) 'LIST)
          (setq got (ssget value))
            )
          )
          (if (and (= Cont T) (= code 5) (= 0 OffOn))
            (progn (setq a (ssname got 0))
               (if e
                 (redraw e)
               )
               (redraw a 3)
               (setq e a)
               (command "_.EXTEND" ss "" value "")
               (setq got nil)
            )
          )
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;---------------------------
      ;---Hatch Base Point Editor.
      ;---------------------------
      (defun C:EH () (AAHedit))

      (defun AAHedit (/ e2 ent2 item_is hPT1 hXCRD hYCRD hSP1)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (princ (strcat "\nHatch base point editor." copyright))

        (princ "Select associative hatch object...")
        (setq HPick (ssget "+.:E:S" '((0 . "HATCH"))))

        (if (/= HPick nil)
          (progn
            (setq e2      (ssname HPick 0)
              ent2    (entget e2)
              sHMotif (dxf 2 ent2)
            )

            (setq hPT1  (upoint 65 "" "Specify hatch new base point" nil nil)
              hXCRD (car hPT1)
              hYCRD (cadr hPT1)
              hSP1  (list hXCRD hYCRD)
            )
            (setvar "snapbase" hSP1)
            (command "-hatchedit" e2 "_P" "" "" "")
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun C:RH () (AAHRep))

      (defun AAHRep (/ e2 ent2 item_is hPT1 hXCRD hYCRD hSP1)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (princ (strcat "\nReplace Hatch with Solid" copyright))

        (princ "Select associative hatch object...")
        (setq HPick (ssget "+.:E:S" '((0 . "HATCH"))))

        (if (/= HPick nil)
          (progn
            (setq e2      (ssname HPick 0)
              ent2    (entget e2)
              sHMotif (dxf 2 ent2)
            )

            (setq AcaDoc (vla-get-ActiveDocument (vlax-get-acad-object)))
            (vla-StartUndoMark AcaDoc)
            (if (setq
              CurSet (ssget "X" (list (cons 0 "HATCH") (cons 2 sHMotif)))
            )
          (while (setq CurEnt (ssname CurSet 0))
            (setq CurObj (vlax-ename->vla-object CurEnt))
            (vla-SetPattern CurObj acHatchPatternTypePreDefined "SOLID")
            (ssdel CurEnt CurSet)
          )
            )
            (vla-EndUndoMark AcaDoc)

          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;;;------------
      ;;;---ReSwing.
      ;;;------------
      ;;;(setq *Acad-Object* nil
      ;;;      *active-document*    nil
      ;;;      *model-space* nil
      ;;;)
      ;;;(VLxx_AcadObject)
      ;;;(VLxx_activedocument)
      ;;;
      ;;;(defun ReacSouris (sAppNameAA)
      ;;;  (TraceIsOn ".ReacSouris")
      ;;;  (setq MouseReac (vlr-mouse-reactor                   ;Constructs an editor reactor object that notifies of mouse event.
      ;;;          sAppNameAA
      ;;;           '((:vlr-beginDoubleClick . Openings_DblClick)   ;The user has double-clicked.
      ;;;             ;;(:vlr-beginRightClick . Openings_RightClick)    ;The user has right-clicked.
      ;;;            )
      ;;;         )
      ;;;  )
      ;;;  (vlr-set-notification MouseReac 'active-document-only)
      ;;;  (princ)
      ;;;)
      ;;;
      ;;;(defun Openings_DblClick (obj lst / ssObj typ)
      ;;;  (setq  ssObj nil
      ;;;         typ nil
      ;;;  )
      ;;;  (if lst
      ;;;    (progn (setq ssObj (ssget "i"))
      ;;;      (if (and ssObj (= 'PICKSET (type ssObj)) (= (sslength ssObj) 1))
      ;;;        (progn (if (/= nil (AAXDataGet (ssname ssObj 0) sAppNameAA))
      ;;;             (setq sPorteOuiNon (cdr (nth 0 (AAXDataGet (ssname ssObj 0) sAppNameAA))));Une porte? Vue en plan?
      ;;;           )
      ;;;           (if (= sPorteOuiNon "DOORPL")
      ;;;             (progn
      ;;;           (vla-sendcommand *active-document* "(ReSwing) ")
      ;;;                        ;;(menucmd (strcat "P" "" "=+ArchAssist.pop515"))
      ;;;
      ;;;             )
      ;;;           )
      ;;;        )
      ;;;      )
      ;;;    )
      ;;;  )
      ;;;  (princ)
      ;;;)
      ;;;
      ;;;(defun Openings_RightClick (obj lst / ssObj sPorteOuiNon )
      ;;;  (setq    ssObj nil  sPorteOuiNon nil)
      ;;;  (if lst
      ;;;    (progn (setq ssObj (ssget "i"))
      ;;;      (if (and ssObj (= 'PICKSET (type ssObj)) (= (sslength ssObj) 1))
      ;;;        (progn (if (/= nil (AAXDataGet (ssname ssObj 0) sAppNameAA))
      ;;;             (setq sPorteOuiNon (cdr (nth 0 (AAXDataGet (ssname ssObj 0) sAppNameAA))));Une porte? Vue en plan?
      ;;;           )
      ;;;           (if (= sPorteOuiNon "DOORPL")
      ;;;             (progn
      ;;;           (vla-sendcommand *active-document* "(ReSwing) ")
      ;;;                        ;;(menucmd (strcat "P" "" "=+ArchAssist.pop515"))
      ;;;
      ;;;             )
      ;;;           )
      ;;;           (if (= sPorteOuiNon "WINDPL")
      ;;;             (progn
      ;;;           (vla-sendcommand *active-document* "_.hatchedit ")
      ;;;             )
      ;;;           )
      ;;;        )
      ;;;      )
      ;;;    )
      ;;;  )
      ;;;  (princ)
      ;;;)
      ;;;
      ;;;(defun RemReacSouris ()
      ;;;  (if (vlr-reactors :vlr-mouse-reactor)
      ;;;    (progn (vlr-remove-all :vlr-mouse-reactor) (setq MouseReac nil))
      ;;;  )
      ;;;  (princ)
      ;;;)
      ;;;
      ;;;(defun CargaReacSouris ()
      ;;;  (if ReacSouris
      ;;;    (ReacSouris nil)
      ;;;  )
      ;;;  (princ)
      ;;;)
      ;;;
      ;;;(RemReacSouris)
      ;;;(CargaReacSouris)

      ;;;----
      (defun ReSwing (/ eObListe eObjName)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (WsFlashMess2 (strcat "\nDoor Opening Modify." copyright)
              (strcat "\nDoor Opening Modify." copyright)
              (strcat "\nDoor Opening Modify." copyright)
              (strcat "\nDoor Opening Modify." copyright)
        )
        (GetDrINI)
        (setq eObjName (objsel "INSERT" "Select door to modify" nil))
        (if (/= nil (AAXDataGet (car eObjName) sAppNameAA))
          (setq sPorteOuiNon (cdr (nth 0 (AAXDataGet (car eObjName) sAppNameAA))))        ;Une porte? Vue en plan?
        )

        (if (= sPorteOuiNon "DOORPL")
          (progn
            (setq ODeNom     (car eObjName)
              rJambWidth (read (cdr (nth 1 (AAXDataGet ODeNom sAppNameAA))))      ;Jamb Width
              sDrHan2    (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))             ;Database handle
              LDR        (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))             ;Door type
              jambprefix (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA)))             ;Jamb type
              OFFDIST    (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))))      ;Door width
              p1         (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))             ;Hinge
              pMCote     (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))             ;meme cote
              pCoteO     (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))             ;cote oppose
              PercOpen   (cdr (nth 9 (AAXDataGet ODeNom sAppNameAA)))             ;opening angle
              rMurThk    (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))            ;Wall thk
              bHeader    (read (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA))))         ;Header present
              bThreshold (read (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA))))         ;Threshold/Sill present?
              sTofIns    (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))            ;Inseree a quelle heure?
              aA3        (angle p1 pMCote)
              ODeNom     (entget ODeNom)
              lDoor      ODeNom
              ODrNam     (dxf -1 lDoor)
              pCnr1      (list 0 0 0)
              oBlNam     (dxf 2 lDoor)
              pHinge     (dxf 10 lDoor)
              xFactor    (dxf 41 lDoor)
              yFactor    (dxf 42 lDoor)
              aRot       (dxf 50 lDoor)
              sTr2       LDR
              sTr3a      (rtos Offdist)
              sDrHand    sDrHan2
              p2         pMCote
              p3         pCoteO
              bEfface    nil
              sJambTyp   jambprefix
              sJx        jambprefix
            )

            (if (= (dxf 0 ODeNom) "INSERT")
          (progn
            (defun c:ReSwinger_OnInitialize ()
              (Odcl_Control_SetProperty XpressForm_ReSwinger_SwingScrollBar "Value" (atoi PercOpen))
              (Odcl_Control_SetProperty XpressForm_ReSwinger_LabelSldBar "Caption" PercOpen)
                  (if (= (atoi PercOpen) 90)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 3)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 1)
                  )
            )

            (defun c:ReSwinger_AnglePictureBox_OnClicked ()
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_LabelSldBar "Caption" "0")
              (Odcl_Control_SetProperty XpressForm_ReSwinger_SwingScrollBar "Value" 0)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 1)
              (setq sOpenAng 0)
              (DoorAction)
                )
            (defun c:ReSwinger_AnglePictureBox_OnDblClicked ()
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_LabelSldBar "Caption" "90")
              (Odcl_Control_SetProperty XpressForm_ReSwinger_SwingScrollBar "Value" 90)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 3)
              (setq sOpenAng 90)
              (DoorAction)
                )
            (defun c:ReSwinger_AnglePictureBox_OnRightClick ()
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_LabelSldBar "Caption" "180")
              (Odcl_Control_SetProperty XpressForm_ReSwinger_SwingScrollBar "Value" 180)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 1)
              (setq sOpenAng 180)
              (DoorAction)
                )
            (defun c:ReSwinger_SwingScrollBar_OnScroll (nValue /)
              (setq sOpenAng nValue)                          ;string in degrees
                  (if (= sOpenAng 90)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 3)
                  (Odcl_Control_SetProperty XpressForm_ReSwinger_AnglePictureBox "BackColor" 1)
                  )
              (DoorAction)
            )
          )
            )
            ;;;(MontreAA3)
            (Odcl_Form_Show XpressForm_ReSwinger)
          )
          (princ "\nCannot modify this object. ")
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun DoorAction ()
        (Odcl_Control_SetProperty
          XpressForm_ReSwinger_LabelSldBar
          "Caption"
          (angtos (dtr sOpenAng) 1 1)
        )
        (if (= bEfface nil)
          (entdel (car eObjName))
          (entdel (entlast))
        )
        (setq bEfface  T
          sOpenAng (etos sOpenAng)
          PercOpen sOpenAng
        )
        (cond ((= LDR "S")     (SingleMk))
          ((= LDR "D")     (DoubleMk))
          ((= LDR "UD")    (UnDblMk))
          ((= LDR "BD")    (BifoldDMk))
          ((= LDR "BS")    (BifoldSMk))
      ;;;;((= LDR "W")     (OpenMk))            ;AABuild108
      ;;;;((= LDR "SFd")   (SldFixMkD))         ;AABuild108
      ;;;;((= LDR "FSFd")  (FixSldFixMk))       ;AA1.3b100
      ;;;;((= LDR "FSSFd") (FixSldSldFixMk))        ;AA1.3b100
          ((= LDR "SSS")   (SurfSlidSMk))
          ((= LDR "SSD")   (SurfSlidDMk))       ;AA1.3b100
          ((= LDR "CSD")   (CavSlidDMk))        ;AA1.3b100
          ((= LDR "CSS")   (CavSlidSMk))
        )
        (Hshset_lay (dxf 8 lDoor) nil "CONTINUOUS" nil)
        (entmake (list (cons 0 "INSERT")
               (cons 2 BlockNameD)
               (cons 10 pHinge)
               (cons 41 xFactor)
               (cons 42 YFactor)
               (cons 50 aRot)
             )
        )
        (SetDoorInfoTag sAppNameAA (entlast) nil)
      )
      ;------------------------------
      ;---Ter Boundary Area routine.
      ;------------------------------
      (defun C:AA () (AABarea))

      ; This program will return T if the given argument is a point.
      (defun AA_pointp (pt /)
        (if (and (listp pt)
             (>= (length pt) 2)
             (<= (length pt) 3)
             (= (type (car pt)) 'real)
             (= (type (cadr pt)) 'real)
             (member (type (caddr pt)) '(real nil))
            )
          T
        )
      )

      (defun AABdary (/ BDRY P BPSET)
        (if (setq BfSET (ssget "i"))
          (princ "\nUsing current existing boundary set.")
        )

        (while
          (and (setq bapt
              (upoint 6 "exit" "Select internal point or <exit>" nil nil)
           )
           (/= bapt "exit")
          )
           (Hshset_lay "BAreas" "7" "ACAD_ISO02W100" "0.40")
           (cond
             ((setq BDRY (BPOLY bapt BfSET))
          (command "_.AREA" "_O" BDRY)
          (setq aire (getvar "AREA"))
          (setq mm2 "mm\\u+00b2")
          (setq m2 "m\\u+00b2")
          (setq km2 "km\\u+00b2")
          (cond
            ((< aire 50000)
             (setq aire (strcat (rtos aire 2 0) " " mm2))
            )
            ((>= aire 100000000000)
             (setq
               aire (strcat (rtos (/ aire 1000000000000.0) 2 1) " " km2)
             )
            )
            (T
             (setq aire (strcat (rtos (/ aire 1000000.0) 2 1) " " m2))
            )
          )
          (Hshset_lay sTAreaLyr sTAreaCol "CONTINUOUS" sTAreaLwt)
          (TextMake bapt
                (rtos (* (getvar "DIMSCALE") 2))
                0
                1
                2
                aire
                nil
          )               ;MC
          (dos_pause 2)
          (command "_.ERASE" BDRY "")
                          ;      (command "-Bhatch" "_P" "_S" bapt "")
             )
             (T (princ "\nValid boundary not found."))
           )
        )
        (princ)
      )

      (defun AABArea (/)
        (setq olderror *error*
          *error*  Wallserror
          bapt     nil
          aire     nil
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setvar "OSMODE" 0)
        (WsFlashMess2
          (strcat "\nPrint Areas." copyright)
          (strcat "\nPrint Areas." copyright)
          (strcat "\nPrint Areas." copyright)
          (strcat "\nPrint Areas." copyright)
        )
        (AABdary)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;--------------------------------
      ;---Federation Boundary routine.
      ;--------------------------------
      (defun C:FB () (AAFedB))
      (defun AAFBdary (/ BDRY P BPSET)
        (if (setq BfSET (ssget "i"))
          (princ "\nUsing current existing boundary set.")
        )
        (setvar "PLINETYPE" 0)
        (while (and (setq bapt (upoint 6 "exit" "Select point inside pane or <exit>" nil nil))
                (/= bapt "exit")
           )

          (GetWnINI)
          (Hshset_lay sBarsLyr sBarsCol "CONTINUOUS" sBarsLwt)
          (cond ((setq BDRY (BPOLY bapt BfSET))
             (setq lFB (Liste4Pline BDRY))
             (setq lOpPt (make_newpt "-" (nth 0 lfb) (nth 2 lfb)))            ;liste des pts opposes
             (setq pHingg (car (CornersGet (nth 0 lfb) (nth 2 lfb))))
             (setq rLarg (car lOpPt))
             (setq rHaut (cadr lOpPt))
             (setq rFifthL (/ rLarg 5.0))
             (setq rFifthH (/ rHaut 5.0))
             (command "_.ERASE" BDRY "")
             (setq ssFinalW nil
               ssFinalW (SSWindow pHingg 10 "_C" (list (cons 0 "INSERT") (list -3 (list "AA_OPENING"))))
             )

             (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
             ;;Bottom Horizontal
             (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 100 "AcDbLine")
                    (list 10 0 rFifthH 0)
                    (list 11 rLarg rFifthH 0)
                    (list 210 0 0 1)
                  )
             )
             ;;Top Horizontal
             (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 100 "AcDbLine")
                    (list 10 rLarg (* rFifthH 4) 0)
                    (list 11 0 (* rFifthH 4) 0)
                    (list 210 0 0 1)
                  )
             )
             ;;Left Vertical
             (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 100 "AcDbLine")
                    (list 10 rFifthL rHaut 0)
                    (list 11 rFifthL 0 0)
                    (list 210 0 0 1)
                  )
             )
             (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 410 "Model")
                    (cons 100 "AcDbLine")
                    (list 10 (* rFifthL 4) 0 0)
                    (list 11 (* rFifthL 4) rHaut 0)
                    (list 210 0 0 1)
                  )
             )
             (setq BlockName3b (entmake '((0 . "ENDBLK"))))
             (entmake (list (cons 0 "INSERT")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 2 BlockName3b)
                    (cons 10 pHingg)
                    (list 210 0.0 0.0 1.0)
                  )
             )
             (if ssFinalW
               (progn (setq doorpick (ssname ssFinalW 0))
                  (if (/= nil (AAXDataGet doorpick sAppNameAA))
                    (setq sPorteOuiNon (cdr (nth 0 (AAXDataGet doorpick sAppNameAA))))        ;Une fenetre? Vue en elevation?
                  )
                  (if (= sPorteOuiNon "WINDEL")
                    (progn (setq ODeNom  doorpick
                         sDrHan2 (read (cdr (nth 1 (AAXDataGet doorpick sAppNameAA))))    ;Database handle
                         sTofIns (cdr (nth 8 (AAXDataGet doorpick sAppNameAA)))       ;
                       )
                    )
                  )
                   (WindBarsTagSet sAppNameAA (entlast) 2)
               )
             )
            )
            (T (princ "\nValid boundary not found."))
          )
        )
        (princ)
      )

      (defun AAFedB (/)
        (setq olderror *error*
          *error*  Wallserror
          bapt     nil
          aire     nil
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setvar "OSMODE" 0)
        (WsFlashMess2 (strcat "\nAdd Federation Bars to Windows." copyright)
              (strcat "\nAdd Federation Bars to Windows." copyright)
              (strcat "\nAdd Federation Bars to Windows." copyright)
              (strcat "\nAdd Federation Bars to Windows." copyright)
        )
        (AAFBdary)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (TraceIsOn ".19j")

      ;-----------
      ;---Resizer.
      ;-----------
      (defun C:RS () (AAResize))

      (defun AAResize (/ bXbox bYbox bZbox rCurrX rCurrY rCurrZ eObListe eObjName bDispBox rScalVal)
        (setq OLDERROR  *error*
          *error*   Wallserror
          sBlocNom$ "."
        )
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (Wallsgather_info)
        (WsFlashMess2 (strcat "\nBlock Resizer." copyright)
              (strcat "\nBlock Resizer." copyright)
              (strcat "\nBlock Resizer." copyright)
              (strcat "\nBlock Resizer." copyright)
        )
      ;;;(MontreAA3)
        (setq eObjName (objsel "INSERT" "Select one block object to resize or" T))
        (setq eObListe (entget eObjName)
          eObType  (dxf 0 eObListe)
        )
        (if (= eObType "INSERT")
          (progn (setq sBlocNom$ (dxf 2 eObListe)
               rCurrX    (dxf 41 eObListe)
               rCurrY    (dxf 42 eObListe)
               rCurrZ    (dxf 43 eObListe)
               bDispBox  T
             )
             (defun c:ReSizer_OnInitialize ()
      ;;           (TraceIsOn ".c:ReSizer_OnInitialize1")
               (Odcl_Control_SetProperty XpressForm_ReSizer_ObjLabel "Caption" sBlocNom$)
      ;;           (TraceIsOn ".c:ReSizer_OnInitialize2")
               (Odcl_Control_SetProperty XpressForm_ReSizer_LabelSldBar "Caption" "1.0")
               (TraceIsOn ".c:ReSizer_OnInitialize3")
             )

               (TraceIsOn ".AAResize1")
             (defun c:ReSizer_ScrollBar_OnScroll (nValue /)
               (TraceIsOn ".c:ReSizer_ScrollBar_OnScroll1")
               (setq rScalVal (* nValue 0.1))
               (TraceIsOn ".c:ReSizer_ScrollBar_OnScroll2")
               (Odcl_Control_SetProperty XpressForm_ReSizer_LabelSldBar "Caption" (rtos rScalVal 2 1))
               (TraceIsOn ".c:ReSizer_ScrollBar_OnScroll3")
               (Setq bXbox (Odcl_Control_GetProperty XpressForm_ReSizer_CheckBoxX "Value") )
               (Setq bYbox (Odcl_Control_GetProperty XpressForm_ReSizer_CheckBoxY "Value") )
               (Setq bZbox (Odcl_Control_GetProperty XpressForm_ReSizer_CheckBoxZ "Value") )

               (if bXbox
                 (setq eObListe (subst (cons 41 (* rScalVal rCurrX)) (assoc 41 eObListe) eObListe)
                 )
               )
               (if bYbox
                 (setq eObListe (subst (cons 42 (* rScalVal rCurrY)) (assoc 42 eObListe) eObListe)
                 )
               )
               (if bZbox
                 (setq eObListe (subst (cons 43 (* rScalVal rCurrZ)) (assoc 43 eObListe) eObListe)
                 )
               )
               (entmod eObListe)
               (princ)
             )
          )
        )

        (TraceIsOn ".AAResize2")
        (Odcl_Form_Show XpressForm_ReSizer)
        ;;;(setq *error* OLDERROR)
        ;;;(reset nil)(resset varlst)
      )

      ;----------------------
      ;---Types de fenetres.
      ;----------------------
      (defun SillMk ()
        (TraceIsOn ".SillMk>")
        (if (= bMetUnit T)                            ;Section metrique
         (progn
          (setq rFullH    (+ 20.0 rMurThk)
            rCorrected    (- rFullH 230.0) )
         )
        )

        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 8 sSillLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0.0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 OFFDIST 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 OFFDIST rFullH)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0.0 rFullH)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0.0 0.0 1.0)
             )
        )
         (setq rSpace2 (/ OffDist (fix (/ OffDist 100)) 1.0)
           pBLC    (polar pWindIns A1 160)
           pTLC    (polar pWindIns A1 rFullH)
           pTRC    (polar pTLC A0 OffDist)
           pBRC    (polar pBLC A0 OffDist)
         )
         (if (= bSillHat T)
          (progn
           (setvar "snapbase" (list (car pWindIns) (cadr pWindIns)))
           (Hshset_lay sSillLyr "23" nil "0.25")
           (command "_.HATCH" "_U" (rtd A1) rSpace2 "_N" "" "_N" pBLC pTLC pTRC pBRC "_C" "")
          )
         )
        (TraceIsOn ".SillMk>>")
      )

      (defun WnHeaderMk   ()
        (TraceIsOn ".WnHeaderMk>")
        (if (= bHeaderW T)
          (progn (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 8 sWHeaderLyr)
                    (cons 100 "AcDbLine")
                    (list 10 0.0 0.0 0.0)
                    (list 11 OFFDIST 0.0 0.0)
                    (list 210 0.0 0.0 1.0)
                  )
             )
             (entmake (list (cons 0 "LINE")
                    (cons 100 "AcDbEntity")
                    (cons 67 0)
                    (cons 8 sWHeaderLyr)
                    (cons 100 "AcDbLine")
                    (list 10 0.0 rMurThk 0.0)
                    (list 11 OFFDIST rMurThk 0.0)
                    (list 210 0.0 0.0 1.0)
                  )
             )
          )
        )
        (TraceIsOn ".WnHeaderMk>>")
      )

      (defun FixedMk (/)
        (TraceIsOn ".FixedMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (WnHeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
      ;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
       (if (<= OffDist (atof sSeuil1))
        (progn
      ;Le Sash
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) 80 0)
               (list 11 (- Offdist rEpaiss rEpaiss) 80 0)
               (list 210 0 0 1)
             )
        )
        )
       )
       (if (and (> OffDist (atof sSeuil1))(<= OffDist (atof sSeuil2)))
        (progn
      ;Le 2eme montant [gauche]
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 0.5 (- Offdist (* 3 rEpaiss))) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist (* 3 rEpaiss))) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le 3eme montant [droite]
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 2 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 3 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 3 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 2 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre du milieu
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 2 rEpaiss)) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 2 rEpaiss)) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist (* 3 rEpaiss))) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist (* 3 rEpaiss))) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 3 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 3 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) 80 0)
               (list 11 (* 0.5 (- Offdist (* 3 rEpaiss))) 80 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (* 0.5 (- Offdist (* 3 rEpaiss))) (* 3 rEpaiss)) 80 0)
               (list 11 (- Offdist (* 2 rEpaiss)) 80 0)
               (list 210 0 0 1)
             )
        )
        )
       )
       (if (> OffDist (atof sSeuil2))
        (progn
      ;Le 2eme montant [gauche]
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (/ (- Offdist (* 4 rEpaiss)) 3) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ (- Offdist (* 4 rEpaiss)) 3) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le 3eme montant [droite]
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 2 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 3 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 3 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 2 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le 4eme montant [gauche]
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 2 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 2 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le 5eme montant [droite]
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 3 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 4 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 4 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 3 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre du milieu 1
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 2 rEpaiss)) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 2 rEpaiss)) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre du milieu 2
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 3 rEpaiss)) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 2 rEpaiss)) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 2 rEpaiss)) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 3 rEpaiss)) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ (- Offdist (* 4 rEpaiss)) 3) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ (- Offdist (* 4 rEpaiss)) 3) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash du milieu
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 3 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) rEpaiss) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) rEpaiss) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 3 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 4 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 60)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 4 rEpaiss)) 100)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) 80 0)
               (list 11 (/ (- Offdist (* 4 rEpaiss)) 3) 80 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre mediane
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ (- Offdist (* 4 rEpaiss)) 3) (* 3 rEpaiss)) 80 0)
               (list 11 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) rEpaiss) 80 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (* 2 (/ (- Offdist (* 4 rEpaiss)) 3)) (* 4 rEpaiss)) 80 0)
               (list 11 (- Offdist (* 2 rEpaiss)) 80 0)
               (list 210 0 0 1)
             )
        )
        )
       )
       (if (= bSill T)
         (progn (SillMk))
       )
       (setq BlockNameW (entmake '((0 . "ENDBLK"))))
      )

      (defun SldFixMk (/)                             ;1 Slide and 1 fixed panel for Windows
        (TraceIsOn ".SlideFixMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (WnHeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
      ;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 1
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 2
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;L'avant dernier montant
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 rEpaiss) (* 0.5 (- Offdist (* 2 rEpaiss)))) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 rEpaiss) (* 0.5 (- Offdist (* 2 rEpaiss)))) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 1.5 rEpaiss) 0)
               (list 11 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 100 0)
               (list 11 (- Offdist (* 2 rEpaiss)) 100 0)
               (list 210 0 0 1)
             )
        )
          (if (= bSill T)
            (progn (SillMk)
            )
          )
        (setq BlockNameW (entmake '((0 . "ENDBLK"))))
      )

      (TraceIsOn ".19k")

      (defun SldFixMkD (/)                                ;1 Slide and 1 fixed panel for Doors
        (TraceIsOn ".SlideFixMkD")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (HeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
      ;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 1
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 2
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;L'avant dernier montant
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 rEpaiss) (* 0.5 (- Offdist (* 2 rEpaiss)))) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 rEpaiss) (* 0.5 (- Offdist (* 2 rEpaiss)))) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 1.5 rEpaiss) 0)
               (list 11 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 0.5 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (* 0.5 (- Offdist (* 2 rEpaiss))) (* 1.5 rEpaiss)) 100 0)
               (list 11 (- Offdist (* 2 rEpaiss)) 100 0)
               (list 210 0 0 1)
             )
        )
          (if (= bSill T)
            (progn (SillMk)
            )
          )
        (setq BlockNameD (entmake '((0 . "ENDBLK"))))
      )

      (defun SldFixSldMk (/)                              ;1 Slide, 1 fixed panel, 1 Slide for Windows
        (TraceIsOn ".SldFixSldMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (WnHeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)                              ;Leaf width
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
      ;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 1
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 2
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (/ Offdist 4) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4) rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4) rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ Offdist 4) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 3
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (/ Offdist 4) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4) rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4) rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ Offdist 4) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche 4
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* (/ Offdist 4) 3) rEpaiss)  120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* (/ Offdist 4) 3) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* (/ Offdist 4) 3) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* (/ Offdist 4) 3) rEpaiss)  80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;L'avant dernier montant
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- (* (/ Offdist 4) 3) rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* (/ Offdist 4) 3) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* (/ Offdist 4) 3) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* (/ Offdist 4) 3) rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ Offdist 4) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (/ Offdist 4) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash du milieu
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ Offdist 4) rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* (/ Offdist 4) 3) rEpaiss) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- (* (/ Offdist 4) 3) rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ Offdist 4) rEpaiss) 120)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* (/ Offdist 4) 3) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 40)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (* 2 rEpaiss)) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* (/ Offdist 4) 3) 80)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 1.5 rEpaiss) 0)
               (list 11 (/ Offdist 4) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre du milieu
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ Offdist 4) rEpaiss) 100 0)
               (list 11 (- (* (/ Offdist 4) 3) rEpaiss) 100 0)
               (list 210 0 0 1)
             )
        )
      ;La vitre droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* (/ Offdist 4) 3) (* 1.5 rEpaiss) 0)
               (list 11 (- Offdist (* 2 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
          (if (= bSill T)
            (progn (SillMk)
            )
          )
        (setq BlockNameW (entmake '((0 . "ENDBLK"))))
      )

      (defun DblHungMk (/)
        (TraceIsOn ".DblHungMk")
        (entmake '((0 . "BLOCK") (2 . "*U") (70 . 1) (10 0.0 0.0 0.0)))
        (WnHeaderMk)
        (setq sOpenAng_R (angtof sOpenAng 0))
        (setq LEAFWIDT_R 40.0
          rEpaiss    40)
        (setq WindANG (atan (/ LEAFWIDT_R (- OFFDIST 70.0))))
        (if (> WindANG sOpenAng_R)
          (setq WindANG sOpenAng_R)
        )
      ;La fenetre
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 0 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 0 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le cadre droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 Offdist 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 0)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 Offdist 160)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
       (if (<= OffDist (atof sSeuil1))
        (progn
      ;Le montant gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre basse
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 1.5 rEpaiss) 0)
               (list 11 (- Offdist rEpaiss rEpaiss) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre haute
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (- Offdist rEpaiss rEpaiss) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        )
       )
       (if (and (> OffDist (atof sSeuil1))(<= OffDist (atof sSeuil2)))
        (progn
      ;Le montant gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre basse
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 1.5 rEpaiss) 0)
               (list 11 (* 0.5 (- Offdist rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre haute
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (* 0.5 (- Offdist rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le montant milieu bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant milieu haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 0.5 (- Offdist rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash haut droit fixe
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre haute droite fixe
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (* 0.5 (- Offdist rEpaiss)) rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (- Offdist rEpaiss rEpaiss) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        )
       )
       (if (> OffDist (atof sSeuil2))
        (progn
      ;Le montant gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 rEpaiss (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 rEpaiss (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash bas gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre basse gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 1.5 rEpaiss) 0)
               (list 11 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash haut gauche
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (* 2 rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (* 2 rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre haute gauche
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (* 2 rEpaiss) (* 2.5 rEpaiss) 0)
               (list 11 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant gauche haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash haut fixe
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre haute fixe
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss)) (* 2.5 rEpaiss) 0)
               (list 11 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit bas
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant droit haut
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 3 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le Sash bas droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) rEpaiss)
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre basse droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 1.5 rEpaiss) 0)
               (list 11 (- Offdist rEpaiss rEpaiss) (* 1.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )

      ;Le Sash haut droit
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sSashLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;La vitre haut droite
        (entmake (list (cons 0 "LINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sGlazeLyr)
               (cons 100 "AcDbLine")
               (list 10 (- Offdist (+ (/ (- Offdist (* 6 rEpaiss)) 3) (* 2 rEpaiss))) (* 2.5 rEpaiss) 0)
               (list 11 (- Offdist rEpaiss rEpaiss) (* 2.5 rEpaiss) 0)
               (list 210 0 0 1)
             )
        )
      ;Le montant bas completement a droite
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 1 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
      ;Le montant haut completement a droite
        (entmake (list (cons 0 "LWPOLYLINE")
               (cons 100 "AcDbEntity")
               (cons 67 0)
               (cons 410 "Model")
               (cons 8 sFrameLyr)
               (cons 100 "AcDbPolyline")
               (cons 90 4)
               (cons 70 1)
               (cons 43 0.0)
               (cons 38 0.0)
               (cons 39 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 3 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 10 (- Offdist rEpaiss rEpaiss) (* 2 rEpaiss))
               (cons 40 0.0)
               (cons 41 0.0)
               (cons 42 0.0)
               (list 210 0 0 1)
             )
        )
        )
       )
       (if (= bSill T)
         (progn (SillMk))
       )
       (setq BlockNameW (entmake '((0 . "ENDBLK"))))
      )

      (TraceIsOn ".19l")

      ;---------------------------
      ;---Unequal Multiple Offset.
      ;---------------------------
      (defun c:umo ()   (umoff))
      (DEFUN umoff (/ rDist spt ent lDistGet)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (IF (= nil sDistGet)
          (SETQ sDistGet "110/50/110")
        )
        (princ (strcat "\nUnequal Multiple Offset." copyright))
        (SETQ sDistGet (ustr 6 "Specify offset distances" sDistGet nil))
        (setq ent (entsel "\nSelect object to offset or <exit>:"))
                          ;get the object
        (IF (/= nil ent)          ;test to ensure ss contains data
          (progn (SETQ spt      (upoint 1 "" "Specify point on side to offset" nil nil)
                          ;get the side
               lDistGet (str_2_list sDistGet "/")
               rDist    0
             ) ;_ _end of SETQ
             (FOREACH item lDistGet
               (SETQ rDist (+ rDist (READ item))
               ) ;_ _end of SETQ
               (COMMAND "_.OFFSET" rDist ent spt "")
             ) ;_ _end of FOREACH
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun c:umc () (umcopy))
      (DEFUN umcopy (/) ;;(rDist spt ent lDistGet)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (IF (= nil sDistGet)
          (SETQ sDistGet "110/50/110")
        )
        (princ (strcat "\nUnequal Multiple ." copyright))
        (SETQ sDistGet (ustr 6 "Specify offset distances" sDistGet nil))
        (setq ent (entsel "\nSelect object to copy or <exit>:"))
                          ;get the object
        (IF (/= nil ent)          ;test to ensure ss contains data
          (progn
            (SETQ pSurObj  (cadr ent)
              spt      (upoint 1 "" "Specify point on side to offset" nil pSurObj)
                          ;get the side
              lDistGet (str_2_list sDistGet "/")
              rDist    0
            ) ;_ _end of SETQ
            (FOREACH item lDistGet
          (SETQ rDist (+ rDist (READ item))
          ) ;_ _end of SETQ
          (COMMAND "_.COPY" ent "" pSurObj rDist)
            ) ;_ _end of FOREACH
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;------------------
      ;---Text Swapping.
      ;------------------
      (defun c:SW ()    (SwapLoc))
      (defun SwapLoc ()
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setvar "OSMODE" 0)
        (WsFlashMess2
          (strcat "\nSwap Objects Location." copyright)
          (strcat "\nSwap Objects Location." copyright)
          (strcat "\nSwap Objects Location." copyright)
          (strcat "\nSwap Objects Location." copyright)
        )
        (LetSwap "\nSelect 1st object:" "\nSelect 2nd object:")
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;----------------------
      ;---Bearing & Distance.
      ;----------------------
      (defun c:BD ()    (LaParcelle))
      (defun LaParcelle (/         spnt      nxt   ang1      iter
                 sp_dcl_id f2h2      bbearing  ddist     nxt_b
                 NFmt      st_pt     en_pt     Site?     ne_list
                 nst_pt    nen_pt    nxt_d     STYLENAME STYLELIST
                 STYLEENT  STYLECUR  rHgT  angtxt    count
                 nxt1      ShowTxt
                )
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setvar "OSMODE" 0)
        (WsFlashMess2
          (strcat "\nAdd Bearing and Distance." copyright)
          (strcat "\nAdd Bearing and Distance." copyright)
          (strcat "\nAdd Bearing and Distance." copyright)
          (strcat "\nAdd Bearing and Distance." copyright)
        )
        (if (= (getvar "ANGDIR") 0)
          (setq sAngDir "counterclockwise")
          (setq sAngDir "clockwise")
        )
        (setq sAngBase (rtos (rtd (getvar "ANGBASE"))))
        (princ (strcat "Current positive angle in UCS:  ANGDIR="
               sAngDir
               "  ANGBASE="
               sAngBase
           )
        )

        (cond ((= langue "Franais")
           (setq Site? (ukword 1
                       "Slectionnez"
                       "Slectionnez proprits ou [Echanger] "
                       "Slectionnez"
                   )
           )
          )
          ((= langue "Italiano")
           (setq Site? (ukword 1
                       "Select"
                       "Select boundaries "
                       "Select"
                   )
           )
          )
          ((= langue "Espanol")
           (setq Site? (ukword 1
                       "Select"
                       "Select boundaries "
                       "Select"
                   )
           )
          )
          (T
           (setq Site? (ukword 1
                       "Select"
                       "Select boundaries "
                       "Select"
                   )
           )
          )
        )
        (if (or (= Site? "Select") (= Site? "Slectionnez"))
          (bdm)
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ; if line angle greater than 90 but less than 270, the text angle is opposite to line direction for std settings.
      ; (setvar "ANGDIR" 0)               ;0=CCW  1=CW

      (defun Ecriture ()
        (if (= (getvar "ANGDIR") 0)       ;0=CCW
          (progn
            (if (and (> ang1 (dtr 90)) (< ang1 (dtr 270)))
          (setq angtxt (+ ang1 (dtr 180)))
          (setq angtxt ang1)
            )
            (if (and (> ang1 (dtr 90)) (< ang1 (dtr 270)))
                          ;if angle greater than 90 but less than 270
          (progn (command
               "_.TEXT"
               "_M"
               (polar (mid_pt spnt nxt) (- ang1 (dtr 90)) (* 0.9 rHgT))
               rHgT
               (rtd angtxt)
               (strrep (angtos ang1 1) "d" "%%d")
                 )
                 (command
               "_.TEXT"
               "_M"
               (polar (mid_pt spnt nxt) (+ ang1 (dtr 90)) (* 0.9 rHgT))
               rHgT
               (rtd angtxt)
               (strcat (rtos ddist 2 3) "m")
                 )
          )
          (progn              ;if angle greater than 270 but less than 90
            (command
              "_.TEXT"
              "_M"
              (polar (mid_pt spnt nxt) (+ ang1 (dtr 90)) (* 0.9 rHgT))
              rHgT
              (rtd angtxt)
              (strrep (angtos ang1 1) "d" "%%d")
            )
            (command
              "_.TEXT"
              "_M"
              (polar (mid_pt spnt nxt) (- ang1 (dtr 90)) (* 0.9 rHgT))
              rHgT
              (rtd angtxt)
              (strcat (rtos ddist 2 3) "m")
            )
          )
            )
          )
        )
        (if (= (getvar "ANGDIR") 1)       ;0=CCW  1=CW
          (progn
            (if
          (and (> (atof (angtos (angle spnt nxt) 0)) 180)
               (< (atof (angtos (angle spnt nxt) 0)) 360)
          )
           (setq angtxt (angtos (+ (angle spnt nxt) (dtr 180)) 0))
           (setq angtxt (angtos (angle spnt nxt) 0))
            )
            (if (and (> ang1 (dtr 90)) (< ang1 (dtr 270)))
                          ;if angle greater than 180 but less than 360
          (progn (command
               "_.TEXT"
               "_M"
               (polar (mid_pt spnt nxt) (- ang1 (dtr 90)) (* 0.9 rHgT))
               rHgT
               (atof angtxt)
               (strrep (angtos ang1 1) "d" "%%d")
                 )
                 (command
               "_.TEXT"
               "_M"
               (polar (mid_pt spnt nxt) (+ ang1 (dtr 90)) (* 0.9 rHgT))
               rHgT
               (atof angtxt)
               (strcat (rtos ddist 2 3) "m")
                 )
          )
          (progn              ;if angle greater than 180 but less than 360
            (command
              "_.TEXT"
              "_M"
              (polar (mid_pt spnt nxt) (+ ang1 (dtr 90)) (* 0.9 rHgT))
              rHgT
              (atof angtxt)
              (strrep (angtos ang1 1) "d" "%%d")
            )
            (command
              "_.TEXT"
              "_M"
              (polar (mid_pt spnt nxt) (- ang1 (dtr 90)) (* 0.9 rHgT))
              rHgT
              (atof angtxt)
              (strcat (rtos ddist 2 3) "m")
            )
          )
            )
          )
        )
      )

      (defun BDM (/ spnt nxt ang1 ddist angtxt count)
        (TraceIsOn ".BDM")

        (if (< (getvar 'TEXTSIZE) 10)
          (setq rHgT (* (getvar 'DIMSCALE) (getvar 'TEXTSIZE)))
          (setq rHgT (getvar 'TEXTSIZE))
        )

        (setq S1    (getall)
              count 0
        )
        (Hshset_lay sTBndryLyr sTBndryCol "CONTINUOUS" sTBndryLwt)
        (setq LG (sslength S1))
        (while (/= count LG)
          (setq A    (entget (ssname S1 count))
            enam (dxf 0 A)
          )
          (if (= enam "LINE")
            (progn (setq spnt  (dxf 10 A)
                 nxt   (dxf 11 A)
                 ang1  (angle spnt nxt)
                 ddist (distance spnt nxt)
                 ddist (/ ddist 1000)
                 ;;;; mm and m conversion
               )
               (ecriture)
            )
          )
          (if (= enam "POLYLINE")
            (progn (setq VxList (Liste4Pline (cdar A))
                 iter   (length VxList)
                 cnt    0
               )
               (while (/= cnt (1- iter))
                 (setq spnt  (nth cnt VxList)
                   nxt   (nth (1+ cnt) VxList)
                   ang1  (angle spnt nxt)
                   ddist (distance spnt nxt)
                   ddist (/ ddist 1000)
      ;;;; mm and m conversion
                 )
                 (ecriture)
                 (setq cnt (1+ cnt))
               )
            )
          )
          (if (= enam "LWPOLYLINE")
            (progn (setq VxList (Liste4LwPl (cdar A))
                 iter   (length VxList)
                 cnt    0
               )
               (while (/= cnt (1- iter))
                 (setq spnt  (nth cnt VxList)
                   nxt   (nth (1+ cnt) VxList)
                   ang1  (angle spnt nxt)
                   ddist (distance spnt nxt)
                   ddist (/ ddist 1000)
      ;;;; mm and m conversion
                 )
                 (ecriture)
                 (setq cnt (1+ cnt))
               )
            )
          )
          (setq count (1+ count))
        )
        (princ)
      )

      ;------------------------
      ;---Adding/DEl... XData.
      ;------------------------
      (defun WsDataGet (en apid / el)
        (TraceIsOn ".WsDataGet")
        (setq el (entget en (list apid)))
        (cdadr (assoc -3 el))
      )

      (defun WsData-appids (/ tmp1 tmp2)
        (TraceIsOn ".WsData-appids")
        (setq tmp1 (tblnext "APPID" 'T))
        (while tmp1
          (setq tmp2 (cons (cdr (assoc 2 tmp1)) tmp2)
            tmp1 (tblnext "APPID")
          )
        )
        tmp2
      )

      ;;;    1.  An ASCII string up to 255 bytes long (group code 1000).
      ;;;    2.  A layer name (group code 1003).
      ;;;    3.  An entity handle (group code 1005).
      ;;;    4.  3 real numbers (group code 1010).
      ;;;    5.  A 3D World space position (group code 1011).
      ;;;    6.  A 3D World space displacement (group code 1012).
      ;;;    7.  A 3D World space direction (group code 1013).
      ;;;    8.  A real number (group code 1040).
      ;;;    9.  A distance (group code 1041).
      ;;;   10.  A scale factor (group code 1042).
      ;;;   11.  A 16-bit integer (group code 1070).
      ;;;   12.  A 32-bit signed long integer (group code 1071).
      ;;;   Numbers 5, 6, 7, 9 and 10 are "transformable" data types, and
      ;;;   are either moved, scaled, rotated or mirrored along with the parent
      ;;;   entity, or possibly some combination of these, depending on the
      ;;;   group code and the nature of the operation on the parent entity.
      ;;;   Binary data chunks (group code 1004) are not supported.
      (defun xdataerr (s)
        (if (/= s "Function cancelled")
          (princ (acet-str-format "\nError: %1" s))
        )
        (setq *error* olderr)
        (princ)
      )

      (defun WsGetInput (/ cont esel)
        (TraceIsOn ".WsGetInput")
        ;; Forces selection of an entity and sets ename to the name of the
        ;; selected entity.
        (if (= which 1)           ;If XDATA() is happening...
          (progn (setq ename (cdar Entite))   ;Get entity info...  entite=(entget (entlast)) ou (entget (car (entsel)))
             (setq elist (entget ename (list "*")))
                          ;...including xdata for all registered applications.
             (setq templ elist)
             ;; Force the entry of a registered application name (group code 1001).
             (setq rname WsRegap)
          )
        )
      )

      (defun WsGetData (/)
        (TraceIsOn ".WsGetData")
        (setq xflag 0)
        ;; Check whether the selected entity has some extended data already.
        (if (assoc -3 elist)
          (progn (setq size_old (xdsize (assoc -3 elist)))

          )
        )
        (setq xd_list (list '(1002 . "}")))   ; Initialize list of xdata for this app.
        (setq hand (getvar "handles"))
        ;; Add sub-list to xdata list.
        (cond ((eq xd_type "3Real")
           (if (/= (setq input (getpoint "\nSpecify 3 real numbers: ")) nil)
             (setq xd_list (cons (cons 1010 input) xd_list))
           )
          )
          ((eq xd_type "DIRection")
           (if (/= (setq input (getpoint "\nSpecify 3D World space direction: ")) nil)
             (setq xd_list (cons (cons 1013 input) xd_list))
           )
          )
          ((eq xd_type "DISPlacement")
           (if (/= (setq input (getpoint "\nSpecify 3D World space displacement: ")) nil)
             (setq xd_list (cons (cons 1012 input) xd_list))
           )
          )
          ((eq xd_type "DISTance")
           (if (/= (setq input (getdist "\nSpecify distance: ")) nil)
             (setq xd_list (cons (cons 1041 input) xd_list))
           )
          )
          ((eq xd_type "Handle")
           (if (or (= (setq hand (getstring "\nEnter database handle: ")) "0") (handent hand))
             (setq xd_list (cons (cons 1005 hand) xd_list))
             (if (/= hand "")
               (princ "\nInvalid handle - handle must exist or have a 0 value.")
             )
           )
          )
          ;; Values entered greater than 32767 cause AutoLISP to issue an
          ;; error message stating "Value must be between -32767 and 32767. "
          ;; Values less than 0 are trapped out by the (initget).  Though the
          ;; message may be confusing, the values are always correct.  This is
          ;; an AutoLISP limitation.
          ((eq xd_type "Int")
           (if (/= input nil)
             (setq xd_list (cons (cons 1070 input) xd_list))
           )
          )
          ((eq xd_type "LAyer")
           (setq input (getstring "\nEnter layer name: "))
           (if (tblsearch "layer" input)
             (setq xd_list (cons (cons 1003 input) xd_list))
             (if (/= input "")
               (princ "\nInvalid layer name - layer must exist.")
             )
           )
          )
          ((eq xd_type "LOng")
           (if (/= (setq input (getint "\nEnter 32-bit signed long integer: ")) nil)
             (setq xd_list (cons (cons 1071 input) xd_list))
           )
          )
          ((eq xd_type "Position")
           (if (/= (setq input (getpoint "\nSpecify 3D World space position: ")) nil)
             (setq xd_list (cons (cons 1011 input) xd_list))
           )
          )
          ((eq xd_type "Real")
           (if (/= input nil)
             (setq xd_list (cons (cons 1040 input) xd_list))
           )
          )
          ((eq xd_type "SCale")
           (if (/= (setq input (getreal "\nEnter scale factor: ")) nil)
             (setq xd_list (cons (cons 1042 input) xd_list))
           )
          )
          ((eq xd_type "String") (setq xd_list (cons (cons 1000 input) xd_list)))
          (T)
        )
        ;; Was any xdata entered besides a registered application name ??
        (setq xflag (length xd_list))
        ;; Append opening brace to front of xdata list.
        (setq xd_list (cons '(1002 . "{") xd_list))
        ;; Append application name to front of xdata list.
        (setq xd_list (cons rname xd_list))
        ;; Append -3 group code to front of list containing xdata list.
        (setq xd_list (list -3 xd_list))
        ;; Find the total size of the new xdata.
        (setq size_new (xdsize xd_list))
      )

      ; "\nEnter an option [3Real/DIR/DISP/DIST/Hand/Int/LAyer/LOng/Pos/Real/SCale/STr/eXit] <eXit>: "
      (defun WsDataAdd (Entite    WsRegap
                xd_type   input
                /     all
                elist     ename
                old       olderr
                new       size_new
                xd_list   xd_list1
                xd_list2  xd_list3
                xd_ent    regflag
                hand      xflag
                size_old  which
               )
        (setq olderr  *error*         ; Use special error handling function.
          *error* xdataerr
        )
        (TraceIsOn ".WsDataAdd")
        (setq which 1)            ; Flag for (WsGetInput)
        (setq regflag 0)          ; Regapp flag.
        (WsGetInput)              ; Prompt for user input
        (if (regapp rname)            ; Register the application name.
          (princ "WsDANew.\n")
          (princ "WsDAExist.\n")      ;    (princ (strcat "\n" rname " new application.\n"))
                          ;    (princ (strcat "\nApplication " rname " was registered.\n"))
        )
        ;; Prompt for user values for xdata and build xdata list XD_LIST.
        (WsGetData)
        ;; The extended data list is now added to the entity data.  This is a
        ;; little more involved if the entity already has extended data.  A check
        ;; of available Xdata space must be made too.
        (if (< size_new (xdroom ename))               ; If there is room for more...
          (progn (if (assoc -3 elist)                 ; and contains xdata already...
               (progn (setq xd_list (cdr xd_list))        ; New xdata.
                  (setq xd_ent (cdr (assoc -3 elist)))    ; Old xdata.
                  ;; Find old xdata with same regapp
                  (if (setq old (cddr (assoc rname xd_ent)))
                    (progn (setq regflag 1)
                       (setq new (cdr (reverse (cddr (assoc rname xd_list)))))
                       (setq all (append new old))
                          ; Join old and new xdata with same application name.
                       (setq xd_list1 (cons (cons 1002 "{") all))     ; Add open curly
                       (setq xd_list2 (cons rname xd_list1))      ; Add regapp
                       ;; Substitute back into existing xdata list.
                       (setq xd_list3 (subst xd_list2 (assoc rname xd_ent) (assoc -3 elist)))
                    )
                    (progn                    ; This is a new regapp...
                  (setq xd_list (append xd_ent xd_list))  ; Joins xdata.
                  (setq xd_list3 (cons -3 xd_list))
                    )
                  )
                  (setq elist (subst xd_list3 (assoc -3 elist) elist))    ; Joins entity
               )
               (setq elist (cons xd_list elist))          ; No xdata yet.
             )
          )
          (princ "\nInsufficient XData space available on object - no Xdata appended.")
        )
        ;; Finally update the entity in the database to contain the new xdata.
        (if (entmod elist)
          (if (and (= 1 regflag) (<= xflag 1))            ; old application name
            (princ "\n.")
            (princ "\n ")
          )
        )
        (setq *error* olderr)                     ; Reset the error function.
        (prin1)
      )

      ;--------------------
      ;---Text along Pline.
      ;--------------------
      (defun C:S2It ()  (WaveIt))
      (defun square (x) (* x x))

      (defun band (a b) (boole 1 a b))

      (defun ObjFlag (ent) (dxf 70 ent))

      (defun next-3 (e) (entget (entnext (cdr (assoc -1 e)))))

      (defun ObjTypGet (ent) (dxf 0 ent))

      (defun TextLengthIs (blurb rFacteur /)
        (TraceIsOn ".TextLengthIs")
        (setq txtstyl (getvar "TEXTSTYLE")
          txtst   (tblsearch "STYLE" txtstyl)
        )
        (setq txtlength (textbox (list (cons 0 "TEXT")
                       (cons 1 blurb)
                       (cons 40 txtht)
                       (cons 7 txtstyl)
                       (cons 51 (dxf 50 txtst))
                     )
                )
        )
        (setq tl1 (- (caadr txtlength) (caar txtlength))
          tl1 (+ tl1 (* rFacteur txtht))
          tl1 (round tl1 2 1.0)
        )
      )

      (defun WsTicker (ent / dpat closed x0 y0 da0 rd dx0 dy0 p0)
        (TraceIsOn ".WsTicker")
        (if bPkGen
          (setq dpat 0.0000001)
          (setq dpat (* 0.5 tikdst))                      ;0.001=1mm
        )
        (setq closed (= 1 (band 1 (ObjFlag ent)))
          e0     (next-3 ent)
          r      (assoc 42 e0)                        ;bulge
          p0     (dxf 10 e0)
          x0     (car p0)
          y0     (cadr p0)
          z0     (caddr p0)                       ;vertex elevation
          closed (if closed
               e0
               nil
                 )
          iCompt 1
        )
        (while (= "VERTEX" (ObjTypGet (setq e0 (next-3 e0))))         ;CHANGEMENT DE NOM ***
          (if (zerop (band 16 (ObjFlag e0)))
            (WsTickTo (dxf 10 e0))                        ;epoint!
          )
        )
        (if closed
          (WsTickTo (dxf 10 closed))
        )
      )

      (defun WsTickTo (p / pa pb)
        (TraceIsOn ".WsTickTo")
        (setq x1    (car p)
          y1    (cadr p)
          z1    (caddr p)
          dx    (- x1 x0)                         ;Spacing?
          dy    (- y1 y0)
          dz    (- z1 z0)
          dline (sqrt (+ (square dx) (square dy)))
          dd    (if (zerop dline)
              1
              dline
                )                             ;Dist from vertex to vertex
          r     (cdr r)
          r     (if (zerop r)
              nil
              r
                )
        )
        (if r
          (setq alfa  (atan r)
            r (/ (* r dd 0.25) (square (sin alfa)))
            beta  (- (+ (atan dy dx) (* 0.5 pi)) (+ alfa alfa))
            xc    (+ x0 (* r (cos beta)))
            yc    (+ y0 (* r (sin beta)))
            a1    (- beta pi)
            a2    (+ a1 (* 4 alfa))
            da    (/ tikdst r)
            da0   (/ dpat r)
            r0    (if (= "Center" tikhow)
                (+ r tl)
                r
              )
            rd    (- r tl)
            dline (* r (- a2 a1))
          )
          (setq dx  (/ dx dd)
            dy  (/ dy dd)
            dz  (/ dz dd)
            dx0 (* dx dpat)
            dy0 (* dy dpat)
            dz0 (* dz dpat)
            dxd (- (* dy tl))                     ;Delta X
            dyd (* dx tl)                         ;Delta Y
            dzd (* dz tl)                         ;Delta Z
            dx  (* dx tikdst)
            dy  (* dy tikdst)
            dz  (* dz tikdst)
          )
        )
        (while (< dpat dline)
          (if r                               ;Fit
            (setq a1  (+ a1 da0)
              da0 da
              x   (cos a1)
              y   (sin a1)
              x0  (+ xc (* r x))
              y0  (+ yc (* r y))
              pa  (list x0 y0 z0)                     ;<--
              pb  (list (+ xc (* rd x)) (+ yc (* rd y)) z0)       ;<--
              aAB (+ (dtr 270) (angle pa pb))
            )
            (setq x0  (+ x0 dx0)                      ;Spline ou straight
              y0  (+ y0 dy0)
              z0  (+ z0 dz0)
              dx0 dx
              dy0 dy
              dz0 dz
              pb  (list (+ x0 dxd) (+ y0 dyd) z0)             ;<--
              pa  (list x0 y0 z0)                     ;<--
              aAB (+ (dtr 270) (angle pa pb))
            )
          )
          (TextMake pa (etos txtht) aAB 1 2 (substr complet iCompt 1) nil)    ;MC
          (setq dline  (- dline dpat)
            iCompt (1+ iCompt)
            dpat   tikdst
          )
          (Cassure "iCompt tl")
        )
        (setq dpat (- dpat dline)
          x0   x1
          y0   y1
          z0   z1
          r    (assoc 42 e0)
        )
        (Cassure "tickto>")
      )

      (defun WaveIt (/ index result bPkGen)
        (TraceIsOn ".WaveIt")
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (WsFlashMess2 (strcat "\nStick to It." copyright)
                (strcat "\nStick to It." copyright)
                (strcat "\nStick to It." copyright)
                (strcat "\nStick to It." copyright)
        )
        (cond ((= langue "Franais")
           (setq result (objsel "POLYLINE or LWPOLYLINE" "Slectionnez une polyligne" T)
           )
          )
          ((= langue "Italiano")
           (setq result (objsel "POLYLINE or LWPOLYLINE" "Select polyline" T))
          )
          ((= langue "Espanol")
           (setq result (objsel "POLYLINE or LWPOLYLINE" "Seleccione polilnea" T)
           )
          )
          (T
           (setq result (objsel "POLYLINE or LWPOLYLINE" "Select polyline" T))
          )
        )
      ;;;(MontreAA3)
        (command "_.AREA" "_O" result)
        (setq rPLong (getvar "PERIMETER")                     ;longueur de pline
          n      (if ss
               (sslength ss)
               0
                 )
          i      (1- n)
          p      nil
          ent    (entget result)
        )

        (defun c:Stick2It_OnInitialize ()
          (setq sWave (cdr (cadr (WsDataGet result sAppName4PlTxt))))
          (if (= sWave nil)
            (setq sWave "")
          )
          (Odcl_Control_SetProperty XpressForm_Stick2It_TextBoxS2It "Text" sWave)
          (Odcl_Control_ZOrder XpressForm_Stick2It_TextBoxS2It 1)
        )

        (defun c:Stick2It_TextBoxS2It_OnEditChanged (sText /)
          (cond ((or (= sText " ")                            ;si vide ou nul
                 (= sText "")
             )
             (Odcl_Control_SetProperty XpressForm_Stick2It_TextBoxS2It "Text" "")
             (Odcl_Control_SetProperty ProjName0 "Stick2It" "RWarn1" "Visible" 1) ;rendre visible
             (setq bPoursui nil)                          ;flag to stop
            )
            (T
             (setq reguser sText
               bPoursui T
             )
             (Odcl_Control_SetProperty ProjName0 "Stick2It" "RWarn1" "Visible" 0) ;rendre invisible
            )
          )
          (princ)
        )

        (defun c:Stick2It_TextBoxS2It_OnKillFocus ()
          (setq sText (Odcl_Control_GetProperty XpressForm_Stick2It_TextBoxS2It "Text")
          )
          (if (or (= sText " ")                           ;si vide ou nul
              (= sText "")
          )
            (setq sText ""
              bPoursui nil
            )
            (Odcl_Control_SetProperty XpressForm_Stick2It_TextBoxS2It "Text" sText)
          )
          (princ)
        )

        (defun c:Stick2It_OnClose (nUpperLeftX nUpperLeftY /) (Odcl_Form_Close ProjName0 "Stick2It"))

        (defun c:Stick2It_Ok_OnClicked ()
          (if bPoursui
            (progn
          (Setq sWave (Odcl_Control_GetProperty XpressForm_Stick2It_TextBoxS2It "Text"))

              (if (< (getvar 'TEXTSIZE) 10)
               (setq txtht (* (getvar 'DIMSCALE) (getvar 'TEXTSIZE)))
               (setq txtht (getvar 'TEXTSIZE))
              )

          (setq sTxtHt  (rtos txtht)
                iStrLen (strlen sWave)                        ;nombre de characteres
                rLongTx (TextLengthIs sWave 0.8)                  ;text length in mm
                rLongCh (/ rLongTx iStrLen)                   ;longueur d'un char.
                iMaxCh  (/ rPLong rLongCh)                    ;max. number of charac.
          )
          (Odcl_Form_Close ProjName0 "Stick2It")
          (if (< iStrLen iMaxCh)
            (if (zerop (band 80 (ObjFlag ent)))                   ;Not a polygon/polyface mesh
              (progn (setq iPad    (fix (* 0.5 (- iMaxCh iStrLen)))       ;1/2 padding
                   Devant  (PadIt sWave (+ iStrLen iPad) nil)         ;rajout a l'avant
                   complet (PadIt Devant (+ iPad iStrLen iPad) T)     ;a l'arriere
                   tikdst  rLongCh                    ;dist. entre characteres
                   tiklen  1.0
                   bWave   T
                   bPkGen  nil
                 )
                 (while (or (= 13 (band 13 (ObjFlag ent)))            ;Closed curved-fit 3D poly
                        (= 5 (band 5 (ObjFlag ent)))          ;Closed splined-fit 3D poly.
                    )
                   (setq ent (next-3 ent))                    ;Find next vtx.
                 )
                 (setq i 0)
                 (command "_.UNDO" "_M")
                 (setq tl tiklen)
                 (WsTicker ent)
                 (WsDataAdd ent sAppName4PlTxt "String" sWave)        ;Attache la phrase a l'objet.
              )
            )
            (dos_msgbox
              (Strcat
                "Selected polyline is too short!\nReduce text height or shorten phrase by min. "
                (etos (fix (abs (- iMaxCh iStrLen))))
                " character(s)."
              )
              "ArchAssist Message"
              1
              3
            )
          )
            )
          )
        )
        (Odcl_Form_Show ProjName0 "Stick2It")
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;-----------------------
      ;---Veil/reveal Objects.
      ;-----------------------
      ;;;   Changes:
      ;;;   - Original (rather useless) reveal command renamed to UBHANDLE
      ;;;   - Instead a new reveal command allowing users to pick entities to make
      ;;;     visible. This is possible because all Veiled entities are temporarily
      ;;;     displayed as highlighted objects for the duration of the command.
      ;;;   - Processing of selection-sets incorporated into the "AFaireVoir" subroutine.
      ;;;   - Original "Veil60" and "show60" subroutines merged into a single
      ;;;     "Soixante" subroutine. For efficiency "entmod" and "entupd" actions have
      ;;;     been incorporated into the "Soixante" subroutine.
      ;;;   - Original 3 separate error routines changed into the "GrabInfo" and
      ;;;     "Remettre" subroutines, both are valid for all 4 commands.
      ;;;     The new subroutines handle saving and restoring the CMDECHO
      ;;;     system variable, UNDO handling and error trapping. In case of a
      ;;;     console break settings are restored properly.
      (defun C:VV  ()  (Veil))
      (defun C:RVS ()  (UnVeil))
      (defun C:RVA ()  (UnVeilAll))

      (defun GrabInfo ()
        (setq olderror *error*
          *error*  Veilerror
          echo     (getvar "CMDECHO")
        )

        (defun Veilerror (msg#)
          (setq *error* OLDERROR)
          (command)               ;Cancel any pending command and end any UNDO group
          (command)
          (command)
          (command "._UNDO" "_E")
          ;; Restore invisibility in case reveal command has been used.
          (if (= cmn "reveal")
            (AFaireVoir 1 (ssget "P"))
          )
          (Remettre)
          ;;  (reset nil)
          ;;  (resset varlst)
          (if (member msg#
              '("console break"
                "Function cancelled"
                "Error: quit / exit abort"
               )
          )
            (princ " Error: Function cancelled by operator.\n")
            (princ
          (strcat "\nError " (itoa (getvar "ERRNO")) ": " msg# "\n")
            )                 ;Otherwise, print an error indication
          )
          (princ)
        )
        (command "_.UNDO" "_GROUP")
        (setvar "CMDECHO" 0)          ; Turn off cmdecho sysvar
      )

      (defun Remettre ()
        (setq *error* olderror)       ; restore error function
        (command "_.UNDO" "_END")
        (setvar "CMDECHO" echo)       ; restore cmdecho sysvar
        (princ)
      )

      (defun Soixante (vis elist)
        (if (not (null (assoc '60 elist)))
          (entmod (subst (cons 60 vis) (cons 60 (- 1 vis)) elist))
          (entmod (append elist (list (cons 60 vis))))
        )
        (entupd ename)
      )

      (defun AFaireVoir (vis ss / idx ename elist)
        (setq idx 0)
        (while (< idx (sslength ss))
          (setq ename (ssname ss idx)
            elist (entget ename)
          )
          (Soixante vis elist)
          ;; Veil vertices of polyline, if necessary
          (if (eq (cdr (nth 1 elist)) "POLYLINE")
            (progn
          (setq ename (entnext ename))
          (setq elist (entget ename))
          (while (eq (cdr (nth 1 elist)) "VERTEX")
            (Soixante vis elist)
            (setq ename (entnext ename))
            (setq elist (entget ename))
          )               ; while
            )                 ; progn
          )                   ; if polyline
          (if (and (eq (cdr (nth 1 elist)) "INSERT")
               (assoc '66 elist)
          )
            (progn
          (setq ename (entnext ename))
          (setq elist (entget ename))
          (while (eq (cdr (nth 1 elist)) "ATTRIB")
            (Soixante vis elist)
            (setq ename (entnext ename))
            (setq elist (entget ename))
          )               ; while
            )                 ; progn
          )                   ;if insert
          (setq idx (1+ idx))
        )                 ; while
      )

      (defun Veil (/ cmn olderr echo ss)
        (setq cmn "Veil")
        (GrabInfo)
        (princ (strcat "\nVeil Objects." copyright))
        (setq ss (ssget))
        (if (not (null ss))
          (AFaireVoir 1 ss)
        )
        (Remettre)
        (princ)               ; Quiet exit.
      )

      (defun UnVeil (/ cmn olderr echo ss idx ename cnt)
        (setq cmn "reveal")
        (GrabInfo)
        (princ (strcat "\nReveal Veiled Objects." copyright))
        ;; Select all Veiled entities     Select a highlighted one
        (setq ss (ssget "_X" '((60 . 1))))
        (if (not (null ss))
          (progn
            (princ
          (strcat (itoa (sslength ss))
              " veiled objects were found. Select object to reveal..."
          )
            )
            ;; Temporarily reveal each entity in the set
            (AFaireVoir 0 ss)
            (command "REDRAW")
            ;; Highlight each entity in the set to show these were the "Veiled" ones.
            (setq idx 0)
            (while (< idx (sslength ss))
          (progn
            (setq ename (ssname ss idx))
            (redraw ename 3)
            (setq idx (1+ idx))
          )
            )
            ;; Pick entities to remove from the set, these removed entities will
            ;; be revealed when rest the set is going to be "re-blanked".
            (setq cnt 0)
            (while (setq ename (car (entsel)))
          (if (ssdel ename ss)
            (progn
              (redraw ename 4)
              (setq cnt (1+ cnt))
            )
            (princ (strcat "This is not a veiled object."))
          )
            )
            ;; Re-blank temporarily Unveiled entities (not selected for Unveiling).
            (AFaireVoir 1 ss)
            (princ
          (strcat (itoa cnt)
              " objects have been selected to reveal.\n"
              (itoa (sslength ss))
              " objects are still left invisible."
          )
            )
          )
          (princ "\nNone found.\n")       ;
        )
        (Remettre)
      )

      (defun UnVeilAll (/ cmn olderr echo ss)
        (setq cmn "UnveilAll")
        (GrabInfo)
        (princ (strcat "\nReveal All Veiled Objects." copyright))
        ;; Select all Veiled entities
        (setq ss (ssget "_X" '((60 . 1))))
        (if (not (null ss))
          (progn
            (princ (sslength ss))
            (princ " objects made visible.\n") ;
            (AFaireVoir 0 ss)
          )
          (princ "\nNone found.\n")       ;
        )
        (Remettre)
      )

      (defun astero (/ cmn olderr echo ename hand)
        (setq cmn "UBHANDLE")
        (GrabInfo)
        (setq hand (getstring "\nEnter handle of entity to be Unveiled: "))
        ;; reveal the entity if handle is not an empty string
        (if (> (strlen hand) 0)
          (progn
            (setq ename (handent hand))
            (if (/= nil ename)
          (AFaireVoir 0 (ssadd ename))
          (alert "Invalid handle.")
            )
          )
        )
        (Remettre)
      )

      ;;-----------------
      ;;---Block Library.
      ;;-----------------
      ;;;(MontreAA5)
      (setq slacle "SortEnts")
      (defun Bolt (sPort sCle)
        (dos_encrypt sPort sCle)
      )

      (setq *acad* (vlax-get-acad-object))
      (setq *sDoc  (vla-get-activedocument *acad*))

      (defun CkHasAtt (BlockObj / check1)
        (if (= (vla-get-ObjectName BlockObj) "AcDbBlockTableRecord")
          (vlax-for item BlockObj
            (if (= (vla-get-ObjectName item) "AcDbAttributeDefinition")
          (setq check1 T)
            )
          )
        )
        check1
      )

      ;;-----------------------------
      ;;Get a block from another dwg.
      ;;-----------------------------
      (defun BlkFromDwg (sDwgName sBlkName / blocs dbxDoc)
        (TraceIsOn ".BlkFromDwg")
        (setq blocs
           (vla-get-blocks        ;Get all blocks
             (vla-get-ActiveDocument *acad*)
                          ;from current dwg's Block Table
           )              ;to be added to variable blocs
        )

        ;;**** Check if source file is open
        (vlax-for oDoc (vla-get-documents *acad*)
          (if (= (vla-get-fullname oDoc) sDwgName)
            (setq Doctest t)
          )
        )

        (cond
          ((= (wcmatch (getvar "ACADVER") "*16*") T)      ;Ensure that ObjectDBX is available
           (setq dbxDoc (vla-GetInterfaceObject
                  *acad*
                  "ObjectDBX.AxDbDocument.16"
                )
           )
          )
          ((= (wcmatch (getvar "ACADVER") "*15*") T)

           (if (not (vl-registry-read
              "HKEY_CLASSES_ROOT\\ObjectDBX.AxDbDocument\\CLSID"
                )
           )
             (startapp "regsvr32.exe"
               (strcat "/s \"" (findfile "axdb15.dll") "\""))
           )

           (setq dbxDoc (vla-GetInterfaceObject
                  *acad*
                  "ObjectDBX.AxDbDocument"
                )
           )
          )
        )

        (vla-open dbxDoc sDwgName)
        (setq FK (list dbxDoc sDwgName))
        (setq *blocks (vla-get-Blocks (car FK)))

        (vla-CopyObjects
          dbxDoc
          (vlax-safearray-fill
            (vlax-make-safearray
          vlax-vbObject
          '(0 . 0)
            )
            (list
          (vla-item
            (vla-get-blocks dbxDoc)
            sBlkName          ;"ElevTree02"
          )
            )
          )
          blocs               ;blocks
        )
        (vlax-release-object dbxDoc)
        (vla-item blocs sBlkName)
      )
      ;; (BlkFromDwg "C:\\Source\\\ArchAssist2004\\Machina.dwg" "Bagnole")
      ;; (BlkToDwg "c:\\CadMenTrees.dwg" "tete")
      ;; (PurgeRemote (strcat Chemin "AATrucks.dwg") "testblock")
      ;; (setq Chemin "C:\\Program Files\\ArchAssist\\")

      (defun SerT ()
        (Bolt (strcat Chemin "AACars.dwg") slacle)
        (Bolt (strcat Chemin "AASanitary.dwg") slacle)
        (Bolt (strcat Chemin "AAFurniture.dwg") slacle)
        (Bolt (strcat Chemin "AATrees.dwg") slacle)
        (Bolt (strcat Chemin "AAPeople.dwg") slacle)
        (Bolt (strcat Chemin "AANorth.dwg") slacle)
        (Bolt (strcat Chemin "AATrucks.dwg") slacle)
      )

      ;;-----------------------------
      ;;Purge a block in another dwg.
      ;;-----------------------------
      (defun AXBlkPurge (sAADoc sAANam)
        (TraceIsOn ".AXBlkPurge")
        (if (vl-catch-all-error-p
          (vl-catch-all-apply
            'vla-delete
            (list (vl-catch-all-apply
                'vla-item
                (list (vla-get-blocks sAADoc) sAANam)
              )
            )
          )
            )
          nil                 ; name cannot be purged or doesn't exist
          T                   ; name purged
        )
      )

      (defun PurgeRemote (source bname / Doctest bname)
        (TraceIsOn ".PurgeRemote")
        (Bolt sWhatLib slacle)
        ;;**** Check if source file is open
        (vlax-for oDoc (vla-get-documents *acad*)
          (if (= (vla-get-fullname oDoc) source)
            (setq Doctest t)
          )
        )
        (if (and (not Doctest) (findfile source))
          (progn

            (cond
          ((= (wcmatch (getvar "ACADVER") "*16*") T)
                          ;Ensure that ObjectDBX is available
           (setq oDBX (vla-GetInterfaceObject
                    *acad*
                    "ObjectDBX.AxDbDocument.16"
                  )
           )
          )
          ((= (wcmatch (getvar "ACADVER") "*15*") T)

           (if (not (vl-registry-read
                  "HKEY_CLASSES_ROOT\\ObjectDBX.AxDbDocument\\CLSID"
                )
               )
             (startapp "regsvr32.exe"
                   (strcat "/s \"" (findfile "axdb15.dll") "\"")
             )
           )
           ;;if

           (setq oDBX (vla-GetInterfaceObject
                    *acad*
                    "ObjectDBX.AxDbDocument"
                  )
           )
          )
            )

            (vla-open oDBX source)        ;"opens" source for our use

            (if (AXBlkPurge oDBX bname)   ;calls Jimmy B.'s function
          (progn
            (Odcl_Control_SetProperty
              AALib_Library_OnError2
              "Caption"
              (strcat sSelText " deleted from library.")
            )
            (cond
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 0)
               (Odcl_ListView_DeleteItem AALib_Library_CarListView iNdx)
              )
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 1)
               (Odcl_ListView_DeleteItem AALib_Library_FurnListView iNdx)
              )
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 2)
               (Odcl_ListView_DeleteItem AALib_Library_NorthListView iNdx)
              )
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 3)
               (Odcl_ListView_DeleteItem
                 AALib_Library_PeopleListView
                 iNdx
               )
              )
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 4)
               (Odcl_ListView_DeleteItem AALib_Library_SanitListView iNdx)
              )
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 5)
               (Odcl_ListView_DeleteItem AALib_Library_TreesListView iNdx)
              )
              ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 6)
               (Odcl_ListView_DeleteItem AALib_Library_TruckListView iNdx)
              )
            )
            (vla-saveas oDBX source)
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Unable to delete block from library."
          )
            )
            (vlax-release-object oDBX)    ;we're done with it
          )                   ;progn
          (progn
            (if doctest
          (princ
            "\nFile already in use by AutoCAD, please close it and try again..."
          )
          (princ "\nFile not found, please try again...")
            )
          )
        )                 ;if
        (Bolt sWhatLib slacle)
        (princ)               ;exit quietly
      )                   ;defun

      ;;---------------------------
      ;;Add a block to another dwg.
      ;;---------------------------
      (defun BlkToDwg (sDwgName sBlkName / blocs dbxDoc)
        (TraceIsOn ".BlkToDwg")
        (setq blocs
           (vla-get-blocks
             (vla-get-ActiveDocument *acad*)
           )
        )

        ;;**** Check if source file is open
        (vlax-for oDoc (vla-get-documents *acad*)
          (if (= (vla-get-fullname oDoc) sDwgName)
            (setq Doctest t)
          )
        )
        (if (> (atoi (getvar "AcadVer")) 15)  ;Ensure that ObjectDBX is available
          (setq dbxDoc (vla-GetInterfaceObject
                   *acad*
                   "ObjectDBX.AxDbDocument.16"
                 )
          )
          (progn
            (if (not (vl-registry-read
                   "HKEY_CLASSES_ROOT\\ObjectDBX.AxDbDocument\\CLSID"
                 )
                )
              (startapp "regsvr32.exe"
                    (strcat "/s \"" (findfile "axdb15.dll") "\"")
              )
            )   ;;if
            (setq dbxDoc (vla-GetInterfaceObject
                     *acad*
                     "ObjectDBX.AxDbDocument"
                   )
            )
          )
        )

        (vla-open dbxDoc sDwgName)
        (vla-CopyObjects
          (vla-get-activedocument *acad*)
          (vlax-safearray-fill
            (vlax-make-safearray
          vlax-vbObject
          '(0 . 0)
            )
            (list
          (vla-item
            blocs
            sBlkName
          )
            )
          )
          (vla-get-blocks dbxDoc)
        )
        (vlax-invoke dbxDoc 'SaveAs sDwgName)
        (vlax-release-object dbxDoc)
        (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          (strcat sNomBlk " added to library.")
        )
      )

      ;;----------------------------------------
      ;;Attach attrib to a block in another dwg.
      ;;----------------------------------------
      (defun AttAttDBX (sDwgName sBlkName / blocs dbxDoc)
        (TraceIsOn ".AttAttDBX")
        (Bolt sWhatLib slacle)
        ;;**** Check if source file is open
        (vlax-for oDoc (vla-get-documents *acad*)
          (if (= (vla-get-fullname oDoc) sDwgName)
            (setq Doctest t)
          )
        )

            (if (> (atoi (getvar "AcadVer")) 15)  ;Ensure that ObjectDBX is available
          (setq dbxDoc (vla-GetInterfaceObject
                   *acad*
                   "ObjectDBX.AxDbDocument.16"
                 )
          )
          (progn
            (if (not (vl-registry-read
                   "HKEY_CLASSES_ROOT\\ObjectDBX.AxDbDocument\\CLSID"
                 )
                )
              (startapp "regsvr32.exe"
                    (strcat "/s \"" (findfile "axdb15.dll") "\"")
              )
            )   ;;if
            (setq dbxDoc (vla-GetInterfaceObject
                     *acad*
                     "ObjectDBX.AxDbDocument"
                   )
            )
          )   ;;progn
            )      ;;if
        (vla-open dbxDoc sDwgName)            ;Open the drawing "c:\\AATrees.dwg"
        (vla-addAttribute
          (vla-item (vla-get-blocks dbxDoc)
                sBlkName
          )
          2.5
          acAttributeModeInvisible
          "Enter a description:"
          (vlax-3d-point (list 1.0 1.0 0.0))
          "AALEGEND"
          sLegDesc
        )
        (vla-saveas dbxDoc sDwgName)
        (vlax-release-object dbxDoc)
        (Bolt sWhatLib slacle)
        (princ)
      )

      (defun c:AALib () (BlkLib1))

      (defun BlkLib1 (/ bLibrary)
        (setvar "ATTDIA" 1)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
          bLibrary T
        )
        (princ (strcat "\nSymbols Library." copyright))

        (vl-registry-write
          (strcat "HKEY_CURRENT_USER\\"
              (vlax-product-key)
              "\\RestoreAALib"
          )
          "Value"
          "Montre"
        )

        (defun c:Library_OnInitialize ()

          (setq bNor nil
            bMeubl nil
            bCamion nil
            bVoitur nil
            bPeopl nil
            bSanit nil
            bTree nil
          )

          (Odcl_Form_SetDialogMinMaxSizes
            AALib_Library 105 620 250 700)

          (if (= bVoitur nil)
            (progn
          (setq sWhatLib (strcat Chemin "AACars.dwg"))
          (Bolt sWhatLib slacle)
          (Odcl_BlockView_preLoadDwg
            AALib_Library_BlockViewCar
            sWhatLib
          )
          (Setq
            lVoitur (Odcl_BlockView_GetBlockList
                  AALib_Library_BlockViewCar
                )
          )
          (Odcl_ListView_Clear AALib_Library_CarListView)
          (foreach item lVoitur
            (Odcl_ListView_AddItem AALib_Library_CarListView item)
          )
          (setq bVoitur T)
          (Bolt sWhatLib slacle)
            )
          )

          (Odcl_Control_SetProperty
            AALib_Library_DescrGraphicButton
            "Enabled"
            nil
          )
          (Odcl_Control_SetProperty
            AALib_Library_DescrTextBox
            "Text"
            ""
          )

          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (if (= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 5)
            (Odcl_Control_SetProperty
          AALib_Library_SizeOptionList
          "Visible"
          T
            )
            (Odcl_Control_SetProperty
          AALib_Library_SizeOptionList
          "Visible"
          nil
            )
          )
          (princ)
        )

                          ;  (defun c:Library_BlockView2_OnRightClick ()
                          ;    (Odcl_MessageBox "To Do: code must be added to BlockView2" "To do")
                          ;    (Odcl_Control_SetProperty AALib_Library_BlockView2 "RenderMode" 2) ;Wireframe
                          ;  )

        (defun c:Library_TabControl1_OnChanged (nSelIndex /)
          (cond
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 0)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AACars.dwg"))
             (if (= bVoitur nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewCar
               sWhatLib
             )
             (Setq
               lVoitur (Odcl_BlockView_GetBlockList
                     AALib_Library_BlockViewCar
                   )
             )
             (Odcl_ListView_Clear AALib_Library_CarListView)
             (foreach item lVoitur
               (Odcl_ListView_AddItem AALib_Library_CarListView item)
             )
             (setq bVoitur T)
             (Bolt sWhatLib slacle)
           )
             )
            )

            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 1)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AAFurniture.dwg"))
             (if (= bMeubl nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewFurn
               sWhatLib
             )
             (Setq
               lMeubl (Odcl_BlockView_GetBlockList
                    AALib_Library_BlockViewFurn
                  )
             )
             (Odcl_ListView_Clear AALib_Library_FurnListView)
             (foreach item lMeubl
               (Odcl_ListView_AddItem AALib_Library_FurnListView item)
             )
             (setq bMeubl T)
             (Bolt sWhatLib slacle)
           )
             )
            )

            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 2)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AANorth.dwg"))
             (if (= bNor nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewNorth
               sWhatLib
             )
             (Setq
               lNord (Odcl_BlockView_GetBlockList
                   AALib_Library_BlockViewNorth
                 )
             )
             (Odcl_ListView_Clear AALib_Library_NorthListView)
             (foreach item lNord
               (Odcl_ListView_AddItem AALib_Library_NorthListView item)
             )
             (setq bNor T)
             (Bolt sWhatLib slacle)
           )
             )
            )

            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 3)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AAPeople.dwg"))
             (if (= bPeopl nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewPeople
               sWhatLib
             )
             (Setq
               lGens (Odcl_BlockView_GetBlockList
                   AALib_Library_BlockViewPeople
                 )
             )
             (Odcl_ListView_Clear AALib_Library_PeopleListView)
             (foreach item lGens
               (Odcl_ListView_AddItem AALib_Library_PeopleListView item)
             )
             (setq bPeopl T)
             (Bolt sWhatLib slacle)
           )
             )
            )

            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 4)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AASanitary.dwg"))
             (if (= bSanit nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewSanit
               sWhatLib
             )
             (Setq
               lSanitair (Odcl_BlockView_GetBlockList
                   AALib_Library_BlockViewSanit
                     )
             )
             (Odcl_ListView_Clear AALib_Library_SanitListView)
             (foreach item lSanitair
               (Odcl_ListView_AddItem AALib_Library_SanitListView item)
             )
             (setq bSanit T)
             (Bolt sWhatLib slacle)
           )
             )
            )

            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 5)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AATrees.dwg"))
             (if (= bTree nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewTree
               sWhatLib
             )
             (Setq
               lArbres (Odcl_BlockView_GetBlockList
                     AALib_Library_BlockViewTree
                   )
             )
             (Odcl_ListView_Clear AALib_Library_TreesListView)
             (foreach item lArbres
               (Odcl_ListView_AddItem AALib_Library_TreesListView item)
             )
             (setq bTree T)
             (Bolt sWhatLib slacle)
           )
             )
            )

            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 6)
             (Odcl_Control_SetProperty
           AALib_Library_OnError2
           "Caption"
           ""
             )
             (setq sWhatLib (strcat Chemin "AATrucks.dwg"))
             (if (= bCamion nil)
           (progn
             (Bolt sWhatLib slacle)
             (Odcl_BlockView_preLoadDwg
               AALib_Library_BlockViewTruck
               sWhatLib
             )
             (Setq
               lCamion (Odcl_BlockView_GetBlockList
                     AALib_Library_BlockViewTruck
                   )
             )
             (Odcl_ListView_Clear AALib_Library_TruckListView)
             (foreach item lCamion
               (Odcl_ListView_AddItem AALib_Library_TruckListView item)
             )
             (setq bCamion T)
             (Bolt sWhatLib slacle)
           )
             )
            )
          )

          (if (= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 5)
            (Odcl_Control_SetProperty
          AALib_Library_SizeOptionList
          "Visible"
          T
            )
            (Odcl_Control_SetProperty
          AALib_Library_SizeOptionList
          "Visible"
          nil
            )
          )
        )

        (defun c:Library_DescrGraphicButton_OnClicked ()
          (AttAttDBX sWhatLib sSelText)
          (AXBlkPurge *sDoc sSelText)
          (cond
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 0)
             (Bolt sWhatLib slacle)
             (Odcl_ListView_Clear AALib_Library_CarListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewCar
           sWhatLib
             )
             (Setq
           lVoitur
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewCar)
             )
             (foreach item lVoitur
           (Odcl_ListView_AddItem AALib_Library_CarListView item)
             )
             (Bolt sWhatLib slacle)
            )
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 1)
             (Bolt sWhatLib slacle)
             (Odcl_ListView_Clear AALib_Library_FurnListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewFurn
           sWhatLib
             )
             (Setq
           lMeubl
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewFurn)
             )
             (foreach item lMeubl
           (Odcl_ListView_AddItem AALib_Library_FurnListView item)
             )
             (Bolt sWhatLib slacle)
            )
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 2)
             (Bolt sWhatLib slacle)
             (BlkToDwg sWhatLib sSelText)
             (Odcl_ListView_Clear AALib_Library_NorthListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewNorth
           sWhatLib
             )
             (Setq
           lNord
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewNorth)
             )
             (foreach item lNord
           (Odcl_ListView_AddItem AALib_Library_NorthListView item)
             )
             (Bolt sWhatLib slacle)
            )
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 3)
             (Bolt sWhatLib slacle)
             (BlkToDwg sWhatLib sSelText)
             (Odcl_ListView_Clear AALib_Library_PeopleListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewPeople
           sWhatLib
             )
             (Setq
           lGens
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewPeople)
             )
             (foreach item lGens
           (Odcl_ListView_AddItem AALib_Library_PeopleListView item)
             )
             (Bolt sWhatLib slacle)
            )
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 4)
             (Bolt sWhatLib slacle)
             (BlkToDwg sWhatLib sSelText)
             (Odcl_ListView_Clear AALib_Library_SanitListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewSanit
           sWhatLib
             )
             (Setq
           lSanitair
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewSanit)
             )
             (foreach item lSanitair
           (Odcl_ListView_AddItem AALib_Library_SanitListView item)
             )
             (Bolt sWhatLib slacle)
            )
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 5)
             (Bolt sWhatLib slacle)
             (BlkToDwg sWhatLib sSelText)
             (Odcl_ListView_Clear AALib_Library_TreesListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewTree
           sWhatLib
             )
             (Setq
           lArbres
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewTree)
             )
             (foreach item lArbres
           (Odcl_ListView_AddItem AALib_Library_TreesListView item)
             )
             (Bolt sWhatLib slacle)
            )
            ((= (Odcl_Tab_GetCurSel AALib_Library_TabControl1) 6)
             (Bolt sWhatLib slacle)
             (BlkToDwg sWhatLib sSelText)
             (Odcl_ListView_Clear AALib_Library_TruckListView)
             (Odcl_BlockView_preLoadDwg
           AALib_Library_BlockViewTruck
           sWhatLib
             )
             (Setq
           lCamion
            (Odcl_BlockView_GetBlockList AALib_Library_BlockViewTruck)
             )
             (foreach item lCamion
           (Odcl_ListView_AddItem AALib_Library_TruckListView item)
             )
             (Bolt sWhatLib slacle)
            )
          )
        )

        (defun c:Library_DescrTextBox_OnEditChanged (sText /)
          (if (= " " sText)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_DescrTextBox
            "Text"
            "Invalid string"
          )
          (Odcl_Control_SetProperty
            AALib_Library_DescrGraphicButton
            "Enabled"
            nil
          )
            )
            (progn
          (if (= "" sText)
            (Odcl_Control_SetProperty
              AALib_Library_DescrGraphicButton
              "Enabled"
              nil
            )
            (progn
              (setq sLegDesc sText)
              (Odcl_Control_SetProperty
                AALib_Library_DescrGraphicButton
                "Enabled"
                T
              )
            )
          )
            )
          )
        )

        (defun c:Library_DelGraphicButton_OnClicked ()

          (setq bOK nil)
          (cond
            ((= langue "Franais")
             (setq bOK (dos_msgbox
                 "Really want to delete block from the library?"
                 "ArchAssist  Message"
                 4
                 1
               )
             )
            )
            ((= langue "Italiano")
             (setq bOK (dos_msgbox
                 "Really want to delete block from the library?"
                 "ArchAssist  Message"
                 4
                 1
               )
             )
            )
            ((= langue "Espanol")
             (setq bOK (dos_msgbox
                 "Really want to delete block from the library?"
                 "ArchAssist  Message"
                 4
                 1
               )
             )
            )
            (T
             (setq bOK (dos_msgbox
                 "Really want to delete block from the library?"
                 "ArchAssist  Message"
                 4
                 1
               )
             )
            )
          )
          (if (= bOK 6)
            (PurgeRemote sWhatLib sSelText)
          )                   ;Yes
                          ;    (if (= bOK 3) (exit) )             ;No
        )

        (if (null iSize)
          (setq iSize 0)
        )

        (if (= nil (Odcl_Form_IsActive AALib_Library))
          (Odcl_Form_Show AALib_Library)
        )

        ;;Cars section
        (defun c:Library_CarListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewCar)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_CarListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewCar
            sSelText
            0
            0.9
          )

      ;;    (setq sDescr2 (AXGetAttribute dbxDoc sSelText "AALEGEND"))

          (Bolt sWhatLib slacle)
          (princ)
        )

        (defun InsCars (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )

          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_CarListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsCars) ")
        )

                          ;  (defun c:Library_BlockViewCar_OnDragnDropToAutoCAD (3dPoint /)
                          ;    (Odcl_MessageBox
                          ;      "To Do: code must be added to BlockViewCar"
                          ;      "To do"
                          ;    )
                          ;    (Odcl_Control_SetProperty
                          ;      AALib_Library_OnError2
                          ;      "Caption"
                          ;      ""
                          ;    )
                          ;    (Setq p1 3dPoint)
                          ;    (setq pAlongX  (getvar "UCSXDIR")
                          ;     aBlokRot (rtd (angle (list 0 0 0) pAlongX))
                          ;    )
                          ;    (Odcl_SetCmdBarFocus)
                          ;    (vla-sendcommand *active-document* "(InsCars) ")
                          ;  )

        (defun c:Library_CarListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (TraceIsOn ".CarListView_OnDragnDropFromOther")
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lVoitur))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_CarListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewCar
                sWhatLib
              )
              (Setq
                lVoitur
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewCar)
              )
              (foreach item lVoitur
                (Odcl_ListView_AddItem AALib_Library_CarListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

        ;;Furniture section
        (defun c:Library_FurnListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewFurn)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_FurnListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewFurn
            sSelText
            0
            0.9
          )
          (Bolt sWhatLib slacle)
          (princ)
        )

        (defun InsFurn (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )

          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_FurnListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsFurn) ")
        )

        (defun c:Library_BlockViewFurn_OnDragnDropToAutoCAD (3dPoint /)
          (Odcl_MessageBox
            "To Do: code must be added to BlockViewFurn"
            "To do"
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsFurn) ")
        )

        (defun c:Library_FurnListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lMeubl))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_FurnListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewFurn
                sWhatLib
              )
              (Setq
                lMeubl
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewFurn)
              )
              (foreach item lMeubl
                (Odcl_ListView_AddItem AALib_Library_FurnListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

        ;;North section
        (defun c:Library_NorthListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewNorth)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_NorthListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewNorth
            sSelText
            0
            0.9
          )
          (Bolt sWhatLib slacle)
          (princ)
        )

        (defun InsNorth (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )
          (command "_.SCALE"
               (entlast)
               ""
               (trans p1 0 1)
               (* 0.6 (getvar "DIMSCALE"))
          )
          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_NorthListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsNorth) ")
        )

        (defun c:Library_BlockViewNorth_OnDragnDropToAutoCAD (3dPoint /)
          (Odcl_MessageBox
            "To Do: code must be added to BlockViewNorth"
            "To do"
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsNorth) ")
        )

        (defun c:Library_NorthListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lNord))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_NorthListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewNorth
                sWhatLib
              )
              (Setq
                lNord
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewNorth)
              )
              (foreach item lNord
                (Odcl_ListView_AddItem AALib_Library_NorthListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

        ;;Trucks section
        (defun c:Library_TruckListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewTruck)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_TruckListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewTruck
            sSelText
            0
            0.9
          )
          (Bolt sWhatLib slacle)
          (princ)
        )

        (defun InsTrucks (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )

          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_TruckListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsTrucks) ")
        )

        (defun c:Library_BlockViewTruck_OnDragnDropToAutoCAD (3dPoint /)
          (Odcl_MessageBox
            "To Do: code must be added to BlockViewTruck"
            "To do"
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsTrucks) ")
        )

        (defun c:Library_TruckListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lCamion))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_TruckListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewTruck
                sWhatLib
              )
              (Setq
                lCamion
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewTruck)
              )
              (foreach item lCamion
                (Odcl_ListView_AddItem AALib_Library_TruckListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

        ;;People section
        (defun c:Library_PeopleListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewPeople)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_PeopleListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewPeople
            sSelText
            0
            0.9
          )
          (Bolt sWhatLib slacle)
          (princ)
        )

        (defun InsPeople (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )

          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_PeopleListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsPeople) ")
        )

        (defun c:Library_BlockViewPeople_OnDragnDropToAutoCAD (3dPoint /)
          (Odcl_MessageBox
            "To Do: code must be added to BlockViewPeople"
            "To do"
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsPeople) ")
        )

        (defun c:Library_PeopleListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lGens))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_PeopleListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewPeople
                sWhatLib
              )
              (Setq
                lGens
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewPeople)
              )
              (foreach item lGens
                (Odcl_ListView_AddItem AALib_Library_PeopleListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

        ;;Sanitary section
        (defun c:Library_SanitListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewSanit)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_SanitListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewSanit
            sSelText
            0
            0.9
          )
          (Bolt sWhatLib slacle)
          (princ)
        )

        (defun InsSanit (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )

          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_SanitListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsSanit) ")
        )

        (defun c:Library_BlockViewSanit_OnDragnDropToAutoCAD (3dPoint /)
          (Odcl_MessageBox
            "To Do: code must be added to BlockViewSanit"
            "To do"
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsSanit) ")
        )

        (defun c:Library_SanitListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lSanitair))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_SanitListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewSanit
                sWhatLib
              )
              (Setq
                lSanitair
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewSanit)
              )
              (foreach item lSanitair
                (Odcl_ListView_AddItem AALib_Library_SanitListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

        ;;Tree section
        (defun c:Library_TreesListView_OnClicked (nRow nCol /)
          (setq iNdx nRow)
          (Odcl_BlockView_Clear AALib_Library_BlockViewTree)
          (Setq sSelText (Odcl_ListView_GetItemText
                   AALib_Library_TreesListView
                   nRow
                 )
          )
          (Bolt sWhatLib slacle)
          (BlkFromDwg sWhatLib sSelText)

          (Odcl_BlockView_DisplayBlock
            AALib_Library_BlockViewTree
            sSelText
            0
            0.9
          )
          (Bolt sWhatLib slacle)
          (princ)
        )

        ;;(defun c:Library_BlockViewTree_OnRightClick ()              ***** This Works!!!
        ;;     (Odcl_MessageBox "To Do: code must be added to BlockViewTree" "To do")
        ;;)

        (defun InsTrees (/)
          (Setq
            bInsert (Odcl_Control_GetProperty AALib_Library_OptionN "Value")
          )                   ;Is it Normal or Protected?
          (Setq bMInsert
             (Odcl_Control_GetProperty AALib_Library_OptionP "Value")
          )

          (if (= bInsert T)
            (command "_.INSERT" sSelText p1 1 1 0)
          )
          (if (= bMInsert T)
            (command "_.MINSERT" sSelText p1 1 1 0 2 1 0 0)
          )

          (cond
            ((= iSize 0)
             (command "_.SCALE"
              (entlast)
              ""
              (trans p1 0 1)
              (random 0.89 5.18)
             )
            )                 ;5000
            ((= iSize 1)
             (command "_.SCALE"
              (entlast)
              ""
              (trans p1 0 1)
              (random 3.89 10.15)
             )
            )                 ;4000 - 10000
            ((= iSize 2)
             (command "_.SCALE"
              (entlast)
              ""
              (trans p1 0 1)
              (random 8.75 15.21)
             )
            )                 ;9000 - 15000
          )
          (command "_.ROTATE" (entlast) "" (trans p1 0 1) aBlokRot)
          (princ)
        )

        (defun c:Library_TreesListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsTrees) ")
        )

        (defun c:Library_BlockViewTree_OnDragnDropToAutoCAD (3dPoint /)
          (Odcl_MessageBox
            "To Do: code must be added to BlockViewTree"
            "To do"
          )
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (Setq p1 3dPoint)
          (setq pAlongX  (getvar "UCSXDIR")
            aBlokRot (rtd (angle (list 0 0 0) pAlongX))
          )
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(InsTrees) ")
        )

        (defun c:Library_SizeOptionList_OnSelChanged (nSelection sSelText /)
          (setq iSize nSelection)
          (princ)
        )

        (defun c:Library_TreesListView_OnDragnDropFromOther (Point /)
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            "Try again."
          )
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "INSERT"))))

          (if (/= ssDragnDrop nil)
            (progn
          (Odcl_Control_SetProperty
            AALib_Library_OnError2
            "Caption"
            ""
          )
          (setq oBlockIs (ssname ssDragnDrop 0)
                sNomBlk  (dxf 2 (entget oBlockIs))
          )
          (if (/= nil (member sNomBlk lArbres))
            ;;Si retourne different de nil nom du bloc est deja liste
            (WsFlashMess1
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              (strcat
                sNomBlk
                " is a name already listed in library.\nThe block will not be added unless you rename it."
              )
              1
              4
            )
            (progn
              (Bolt sWhatLib slacle)
              (BlkToDwg sWhatLib sNomBlk)
              (Odcl_ListView_Clear AALib_Library_TreesListView)
              (Odcl_BlockView_preLoadDwg
                AALib_Library_BlockViewTree
                sWhatLib
              )
              (Setq
                lArbres
                 (Odcl_BlockView_GetBlockList AALib_Library_BlockViewTree)
              )
              (foreach item lArbres
                (Odcl_ListView_AddItem AALib_Library_TreesListView item)
              )
              (Bolt sWhatLib slacle)
            )
          )
            )
            (Odcl_Control_SetProperty
          AALib_Library_OnError2
          "Caption"
          "Selected object not a block."
            )
          )
          (princ)
        )

      ;;;---
        ;; (vl-registry-write reg-key [val-name val-data])
        ;; reg-key: A string specifying a Windows registry key.
        ;; val-name: A string containing the value of a registry entry.
        ;; val-data: A string containing registry data.
        ;;(vl-registry-write "HKEY_CURRENT_USER\\Test" "" "test data") returns "test data" if T

        ;; Returns data stored in the Windows registry for the specified key/value pair
        ;; (vl-registry-read reg-key [val-name])

        ;; Returns the AutoCAD Window registry path (vlax-product-key)
        ;; The AutoCAD registry path can be used to register an application for demand loading.
        ;; Return Values: A string containing the AutoCAD registry path.

        (defun c:Library_OnClose ()
          ;;(Odcl_MessageBox "To Do: code must be added to Library" "To do")
          (vl-registry-write
            (strcat "HKEY_CURRENT_USER\\"
                (vlax-product-key)
                "\\RestoreAALib"
            )
            "Value"
            "Cache"
          )                   ;Creates a key in the Windows registry
        )
        (princ)
      )

      (defun c:CES () (ExpProtected))
      ;;; Modified to support alignment point of Attributes
      ;;; Modified to support Inserts with Attributes
      (defun ExpProtected
              (/        Objet    ObListe  column_count      row_count
               column_spacing    row_spacing   m    n
               x0       y0       z0   ax0  ay0      az0
               x1       y1       ax1  ay1  ObDat0   ObDat1
               inspt    xm_er    bx0  by0  bz0      bx1
               by1      alignment
              )

        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nChange Protected to Explodable Symbol." copyright))

        (setq ObjName
           (objsel "INSERT" "Select one protected block to change" T)
        )                 ;nil stores the picked pt
        (setq ObListe (entget ObjName)
          ObType  (dxf 0 ObListe)
        )
        (if (= ObType "INSERT")
          (progn
            (setq column_count (dxf 70 ObListe)
              row_count    (dxf 71 ObListe)
            )

            (if (or (>= column_count 1) (>= row_count 1))
          (progn

            ;; set variables
            (setq x0 (car (dxf 10 ObListe)))
            (setq y0 (cadr (dxf 10 ObListe)))
            (setq z0 (caddr (dxf 10 ObListe)))
            (if (null z0)
              (setq z0 0.0)
            )
            (setq column_spacing (dxf 44 ObListe))
            (setq row_spacing (dxf 45 ObListe))

            ;; construct preliminary list for INSERT entity creation
            (setq ObDat0 ObListe)
            (setq ObDat1 (subst (cons 70 0) (assoc 70 ObDat0) ObDat0))
            (setq ObDat0 ObDat1)
            (setq ObDat1 (subst (cons 71 0) (assoc 71 ObDat0) ObDat0))
            (setq ObDat0 ObDat1)
            (setq ObDat1 (subst (cons 44 0.0) (assoc 44 ObDat0) ObDat0))
            (setq ObDat0 ObDat1)
            (setq ObDat1 (subst (cons 45 0.0) (assoc 45 ObDat0) ObDat0))
            (setq ObListe ObDat1)

            ;; remember last entity in database
            (setq Objet (entlast))

            (setq m 0)
            (while (< m column_count)
              (setq n 0)
              (while (< n row_count)
                (setq ObDat0 ObListe)

                ;; calculate insertion point
                (setq x1 (+ x0 (* m column_spacing)))
                (setq y1 (+ y0 (* n row_spacing)))
                (setq inspt (list x1 y1 z0))

                ;; construct final list for INSERT entity construction
                (setq ObDat1
                   (subst (cons 10 inspt) (assoc 10 ObDat0) ObDat0)
                )

                ;; create new INSERT entity
                (if (null (entmake ObDat1))
              (progn
                (prompt "\nError in ENTMAKE 1")
                (print ObDat1)
                (exit)
              )
                )

                ;; duplicate attribute entities if they exist
                (if (= (DXF 66 ObListe) 1)
              (progn
                (setq ObDat0 ObListe)
                (while (/= (DXF 0 ObDat0) "SEQEND")
                  ;; get next entity
                  (setq ObDat0 (entget (entnext (DXF -1 ObDat0))))
                  (if (= (DXF 0 ObDat0) "ATTRIB")
                    ;; modify insertion point for attributes
                    (progn
                  ;; set variables
                  (setq ax0 (car (DXF 10 ObDat0)))
                  (setq ay0 (cadr (DXF 10 ObDat0)))
                  (setq az0 (caddr (DXF 10 ObDat0)))
                  (if (null az0)
                    (setq az0 0.0)
                  )

                  ;; calculate insertion point for attribute
                  (setq ax1 (+ ax0 (* m column_spacing)))
                  (setq ay1 (+ ay0 (* n row_spacing)))
                  (setq inspt (list ax1 ay1 az0))

                  ;; modify list for
                  ;; ATTRIB entity construction
                  (setq ObDat1
                         (subst
                       (cons 10 inspt)
                       (assoc 10 ObDat0)
                       ObDat0
                         )
                  )

                  (if
                    (or
                      (/= (DXF 72 ObDat0) 0)
                      (/= (DXF 74 ObDat0) 0)
                    )
                     (progn
                       ;; set variables
                       (setq bx0 (car (DXF 11 ObDat0)))
                       (setq by0 (cadr (DXF 11 ObDat0)))
                       (setq bz0 (caddr (DXF 11 ObDat0)))
                       (if (null bz0)
                         (setq bz0 0.0)
                       )

                       ;; calculate alignment point for attribute
                       (setq bx1 (+ bx0 (* m column_spacing)))
                       (setq by1 (+ by0 (* n row_spacing)))
                       (setq alignment (list bx1 by1 bz0))

                       ;; modify list for
                       ;; ATTRIB entity construction
                       (setq ObDat1
                          (subst
                            (cons 11 alignment)
                            (assoc 11 ObDat1)
                            ObDat1
                          )
                       )
                     )
                  )
                    )
                    ;; else no modication required for SEQEND
                    (setq ObDat1 ObDat0)
                  )
                  ;; create new Sub entity
                  (if (null (entmake ObDat1))
                    (progn
                  (prompt "\nError in ENTMAKE 2")
                  (print ObDat1)
                  (exit)
                    )
                  )
                )
              )
                )
                (setq n (1+ n))
              )
              (setq m (1+ m))
            )

            (entdel (DXF -1 ObListe)) ;delete MInsert object [original]

            ;; redraw newly created INSERT entities
            (while (setq Objet (entnext Objet))
              (redraw Objet)
            )
          )
          (prompt "\nObject not minserted.")
            )
          )
        )
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;------------------
      ;---Labels Library.
      ;------------------
      ;;; Returns a list of all text style names and her font files
      ;;; Return [Typ]:  Dotted pair list '(("StyleName" . "FontFile")...)
      (defun AXListTextStyles (/ StyLst)
        (setq Cm:AcDObj (cond (Cm:AcDObj)
                  ((vlax-get-acad-object))
                )
          Cm:AcDb   (cond (Cm:AcDb)
                  ((vla-get-ActiveDocument Cm:AcDObj))
                )
        )
        (vlax-for Sty (vla-get-TextStyles Cm:AcDb)
          (setq
            StyLst (cons (cons (vla-get-Name Sty) (vla-get-FontFile Sty))
                 StyLst
               )
          )
          (vlax-release-object Sty)
        )
        (setq lStyFon (reverse StyLst))
        (princ)
      )

      (defun c:LView ()
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
              *error*  Wallserror
        )
        (princ (strcat "\nLabelling Library." copyright))

        (vl-registry-write
          (strcat "HKEY_CURRENT_USER\\"
              (vlax-product-key)
              "\\RestoreLView"
          )
          "Value"
          "Montre"
        )

        (defun LabelList (Ficheros /)
          (TraceIsOn ".LabelList")
          (setq file11 nil
            llabls nil
            lResid1 nil
            lResid2 nil
            lResid3 nil
            lResidL1 nil
            lResidS1 nil
            lResidL2 nil
            lResidS2 nil
            lResidL3 nil
            lResidS3 nil
          )
          (if (/= (setq file11 (open Ficheros "r")) nil)
            (progn (while
                 (setq ligne (read-line file11))
              (setq lLabls (cons ligne lLabls))
               )
               (close file11)
               (setq sLingua (strParse (nospace (last llabls)) "|"))
               (setq iNombre (length sLingua))
               (foreach item lLabls
                 (setq InfoLigne (strParse item "|")
                   lResid1   (append lResid1
                             (list (ai_strtrim (nth 0 InfoLigne)))
                         )    ;liste dans col 1 anglais
                   lResid2   (append lResid2
                             (list (ai_strtrim (nth 1 InfoLigne)))
                         )    ;liste dans col 2 francais
                   lResid3   (append lResid3
                             (list (ai_strtrim (nth 2 InfoLigne)))
                         )    ;liste dans col 3 italien
                 )
               )
               (setq lResid1 (remove_n 0 (reverse lResid1)) ;upper
                 lResid1 (remove_n 0 lResid1)
                 sHeadr1 (car lResid1) ;titre col anglais
                 lResid1 (Remove_Dup (remove_n 0 lResid1))
                 lResid2 (remove_n 0 (reverse lResid2))
                 lResid2 (remove_n 0 lResid2)
                 sHeadr2 (car lResid2) ;titre col francais
                 lResid2 (Remove_Dup (remove_n 0 lResid2))
                 lResid3 (remove_n 0 (reverse lResid3))
                 lResid3 (remove_n 0 lResid3)
                 sHeadr3 (car lResid3) ;titre col italien
                 lResid3 (Remove_Dup (remove_n 0 lResid3))
               )
               (foreach item lResid1  ;dans 1ere colonne dat.
                 (setq sTextIs (strcase item T))
                 (setq lResidL1 (append lResidL1 (list sTextIs))
                          ;for lower case
                   flet     (strcase (substr sTextIs 1 1))
                   rlet     (strcase (substr sTextIs 2 (strlen sTextIs)) T)
                   lResidS1 (append lResidS1 (list (strcat flet rlet)))
                          ;for sentence case
                 )
               )
               ;; (cassure "sTextIs")
               (foreach item lResid2  ;dans 2eme colonne dat.
                 (setq sTextIs (strcase item T))
                 (setq lResidL2 (append lResidL2 (list sTextIs))
                          ;for lower case
                   flet     (strcase (substr sTextIs 1 1))
                   rlet     (strcase (substr sTextIs 2 (strlen sTextIs)) T)
                   lResidS2 (append lResidS2 (list (strcat flet rlet)))
                          ;for sentence case
                 )
               )
               (foreach item lResid3  ;dans 3eme colonne
                 (setq sTextIs (strcase item T))
                 (setq lResidL3 (append lResidL3 (list sTextIs))
                          ;for lower case
                   flet     (strcase (substr sTextIs 1 1))
                   rlet     (strcase (substr sTextIs 2 (strlen sTextIs)) T)
                   lResidS3 (append lResidS3 (list (strcat flet rlet)))
                          ;for sentence case
                 )
               )
            )
          )
        )

        (LabelList (strcat Chemin "LabResid.Dat"))
        (setq lResColumU1 lResid1
          lResColumL1 lResidL1
          lResColumS1
           lResidS1           ;Anglais pour col 1 Residential
          lResColumU2 lResid2
          lResColumL2 lResidL2
          lResColumS2
           lResidS2           ;Francais "
          lResColumU3 lResid3
          lResColumL3 lResidL3
          lResColumS3
           lResidS3           ;Italien  "
          sEngRH sHeadr1
          sFraRH sHeadr2
          sItaRH sHeadr3
        )
        (if (not sHeadr)
          (setq sHeadr sEngRH)
        )

        (LabelList (strcat Chemin "LabComm.Dat"))
        (setq lComColumU1 lResid1
          lComColumL1 lResidL1
          lComColumS1
           lResidS1           ;Anglais pour col 2 Commercial
          lComColumU2 lResid2
          lComColumL2 lResidL2
          lComColumS2
           lResidS2           ;Francais "
          lComColumU3 lResid3
          lComColumL3 lResidL3
          lComColumS3
           lResidS3           ;Italien  "
          sEngCH sHeadr1
          sFraCH sHeadr2
          sItaCH sHeadr3
        )
        (if (not sHeadc)
          (setq sHeadc sEngCH)
        )

        (LabelList (strcat Chemin "LabIndus.Dat"))
        (setq lIndColumU1 lResid1
          lIndColumL1 lResidL1
          lIndColumS1
           lResidS1           ;Anglais pour col 3 Industrial
          lIndColumU2 lResid2
          lIndColumL2 lResidL2
          lIndColumS2
           lResidS2           ;Francais "
          lIndColumU3 lResid3
          lIndColumL3 lResidL3
          lIndColumS3
           lResidS3           ;Italien  "
          sEngIH sHeadr1
          sFraIH sHeadr2
          sItaIH sHeadr3
        )
        (if (not sHeadi)
          (setq sHeadi sEngIH)
        )

        (if (not nLingua)
          (setq nLingua 0)
        )
        (if (not rTHeight)
          (setq rTHeight 2.5)
        )
        (if (not rSTHeight)
          (setq rSTHeight 3.5)
        )
        (if (not rTTHeight)
          (setq rTTHeight 7.0)
        )
        (if (not OptionUpper)
          (setq OptionUpper T)
        )
        (if (not OptionLeft)
          (setq OptionLeft T)
        )
        (if (not OptionT)
          (setq OptionT T)
        )

        (defun TitlesList (/)
          (TraceIsOn ".TitlesList")
          (setq file1 nil
            lDWGTitres
             nil
          )
          (if (/= (setq file1 (open (strcat Chemin "DwgTitles.Dat") "r"))
              nil
          )
            (progn (while (setq ligne (read-line file1))
                 (setq lDWGTitres (cons ligne lDWGTitres))
               )
               (close file1)
               (setq lDWGTitres (reverse (Remove_Dup lDWGTitres)))
               (foreach item lDWGTitres
                 (Odcl_ListView_AddItem Walls_Label_TitleListView item)
               )
            )
            (progn (dos_msgbox
                 "Error:  DwgTitles.Dat file not found."
                 "ArchAssist Message"
                 1
                 5
               )
               (exit)
               (princ)
            )
          )
        )

        (defun c:Label_OnInitialize
           (/ column1width column2width column3width columnwidth1)
          (TraceIsOn ".c:Label_OnInitialize")
          (Odcl_ComboBox_AddList Walls_Label_LinguaComboBox sLingua)
          (Odcl_ComboBox_SetCurSel Walls_Label_LinguaComboBox nLingua)
          (Odcl_Control_SetProperty
            Walls_Label_HeightTextBox
            "Text"
            (rtos rTHeight 2 1)
          )
          (Odcl_Control_SetProperty
            Walls_Label_HgtTextBox1
            "Text"
            (rtos rTTHeight 2 1)
          )
          (Odcl_Control_SetProperty
            Walls_Label_HgtTextBox2
            "Text"
            (rtos rSTHeight 2 1)
          )
          (Exchanger)
          (AXListTextStyles)
          (foreach item lStyFon
            (Odcl_ComboBox_AddString
          Walls_Label_StyleComboBoxLab
          (car item)
            )
            (Odcl_ComboBox_AddString
          Walls_Label_StyleComboBoxTit
          (car item)
            )
            (Odcl_ComboBox_AddString
          Walls_Label_StyleComboBoxSca
          (car item)
            )
          )
          (Odcl_Tab_SetCurSel Walls_Label_TabControl 2)
        )

        (defun c:Label_TabControl_OnChanged (nSelIndex /)
          ;;    (Odcl_MessageBox "To Do: code must be added to TabControl" (etos nSelIndex))
          (if (= nSelIndex 0)         ;for RoomLabels
            (progn (setq lDWGTitres nil) (Exchanger))
          )

          (if (= nSelIndex 1)
            (progn                ;for DwgTitles
          (defun c:Label_ScaleCheckBox_OnClicked (nValue /)
            (Odcl_Control_SetProperty
              Walls_Label_ScalComboBox
              "Enabled"
              nValue
            )             ;1 or 0
          )
          (Odcl_ListView_Clear Walls_Label_TitleListView)
          (setq columnwidth1
                 (Odcl_ListView_CalcColWidth
               Walls_Label_TitleListView
               "   CONSTRUCTION DETAILS   "
                 )
          )
          (Odcl_ListView_AddColumns
            Walls_Label_TitleListView
            (list 0 sHeadr 0 columnwidth1)
          )
          (TitlesList)
          (defun c:Label_DelGraphicButton_OnClicked ()
            (Odcl_ListView_DeleteItem Walls_Label_TitleListView iIndx)
            (Setq iTitNum (Odcl_ListView_CountItems Walls_Label_TitleListView)
              iCnt    0
              file2   nil
              file2   (open (strcat Chemin "DwgTitles.Dat") "w")
            )
            (while (< iCnt iTitNum)
              (Setq sTitl (Odcl_ListView_GetItemText
                    Walls_Label_TitleListView
                    iCnt
                  )
                iCnt  (1+ iCnt)
              )
              (wlf sTitl file2)
            )
            (close file2)
          )

          (defun c:Label_SaveGraphicButton_OnClicked ()
            (Setq iTitNum (Odcl_ListView_CountItems Walls_Label_TitleListView)
              iCnt    0
              file2   nil
              file2   (open (strcat Chemin "DwgTitles.Dat") "w")
            )
            (while (< iCnt iTitNum)
              (Setq sTitl (Odcl_ListView_GetItemText
                    Walls_Label_TitleListView
                    iCnt
                  )
                iCnt  (1+ iCnt)
              )
              (wlf sTitl file2)
            )
            (close file2)
          )
            )
          )
          (if (= nSelIndex 2)
            (progn
          (defun c:Label_StyleComboBoxLab_OnSelChanged
                 (nSelection sSelText /)
            ;; (Odcl_MessageBox "To Do: code must be added to StyleComboBoxLab" (etos nSelection))
            (Odcl_Control_SetProperty
              Walls_Label_FontTextBoxLab
              "Text"
              (cdr (nth nSelection lStyFon))
            )
          )
          (defun c:Label_StyleComboBoxTit_OnSelChanged
                 (nSelection sSelText /)
            (Odcl_Control_SetProperty
              Walls_Label_FontTextBoxTit
              "Text"
              (cdr (nth nSelection lStyFon))
            )
          )
          (defun c:Label_StyleComboBoxSca_OnSelChanged
                 (nSelection sSelText /)
            (Odcl_Control_SetProperty
              Walls_Label_FontTextBoxSca
              "Text"
              (cdr (nth nSelection lStyFon))
            )
          )
          (defun c:Label_SpinButton1_OnChanged (nValue nDelta /)
            ;;    (Odcl_MessageBox "To Do: code must be added to SpinButton" (strcat (etos nValue) "-" (etos nDelta)))
            (setq rTTHeight (* 0.1 nValue))
            (Odcl_Control_SetProperty
              Walls_Label_HgtTextBox1
              "Text"
              (rtos rTTHeight 2 1)
            )
          )
          (defun c:Label_ScaleSpinButton_OnChanged (nValue nDelta /)
            ;;    (Odcl_MessageBox "To Do: code must be added to SpinButton" (strcat (etos nValue) "-" (etos nDelta)))
            (setq rSTHeight (* 0.1 nValue))
            (Odcl_Control_SetProperty
              Walls_Label_HgtTextBox2
              "Text"
              (rtos rSTHeight 2 1)
            )
          )
            )
          )
          (princ)
        )

        (defun Motto ()
          (if bForT
            (TextMake pIns
              (rtos (* (getvar "DIMSCALE") rTHeight))
              0
              iTJust
              0
              sMotto
              sLStyle
            )
            (MTextMake pIns
               (rtos (* (getvar "DIMSCALE") rTHeight))
               (* (getvar "DIMSCALE") 110.0)
               iAttPt
               0
               1
               sMotto
               sLStyle
            )
          )
          (princ)
        )
        (defun MottoT ()
          (if bForT
            (TextMake pIns
              (rtos (* (getvar "DIMSCALE") rTTHeight))
              0
              iTJust
              0
              sMotto
              sTStyle
            )
            (MTextMake pIns
               (rtos (* (getvar "DIMSCALE") rTTHeight))
               (* (getvar "DIMSCALE") 110.0)
               iAttPt
               0
               1
               sMotto
               sTStyle
            )
          )
          (princ)
        )
        (defun MottoS ()
          (setq pIns2 (polar pIns Hshpi270 (* (getvar "DIMSCALE") rTDist)))
          (if bForT
            (TextMake pIns2
              (rtos (* (getvar "DIMSCALE") rSTHeight))
              0
              iTJust
              0
              sMotto2
              sSStyle
            )
            (MTextMake pIns2
               (rtos (* (getvar "DIMSCALE") rSTHeight))
               (* (getvar "DIMSCALE") 110.0)
               iAttPt
               0
               1
               sMotto2
               sSStyle
            )
          )
          (princ)
        )

        (defun c:Label_TitreListView_OnColumnClick (nColumn /)
          ;;    (Odcl_MessageBox "To Do: code must be added to TitreListView" (etos nColumn))
          (cond
            ((= nColumn 0)
             (if (= bAscend0 T)
           (setq bAscend0 nil)
           (setq bAscend0 T)
             )
             (Odcl_ListView_SortTextItems
           Walls_Label_TitreListView
           nColumn
           bAscend0
             )
            )
            ((= nColumn 1)
             (if (= bAscend1 T)
           (setq bAscend1 nil)
           (setq bAscend1 T)
             )
             (Odcl_ListView_SortTextItems
           Walls_Label_TitreListView
           nColumn
           bAscend1
             )
            )
            ((= nColumn 2)
             (if (= bAscend2 T)
           (setq bAscend2 nil)
           (setq bAscend2 T)
             )
             (Odcl_ListView_SortTextItems
           Walls_Label_TitreListView
           nColumn
           bAscend2
             )
            )
          )
          (Odcl_Control_ForceUpdateNow Walls_Label_TitreListView)
        )

      ;;; ***  Coding for RoomLabels
        (defun c:Label_TitreListView_OnClicked (nRow nCol /)
          ;;    (Odcl_MessageBox "To Do: code must be added to TitreListView" (strcat (etos nRow) "-" (etos nCol)))
          (Odcl_Control_ForceUpdateNow Walls_Label_TitreListView)
          (Setq bTagSx
             (Odcl_Control_GetProperty Walls_Label_TagCheckBox "Value")
          )                   ;Tag for suffix
          (Setq sMotto (Odcl_ListView_GetItemText
                 Walls_Label_TitreListView
                 nRow
                 nCol
               )
          )
          (if bTagSx
            (Setq sMotto (strcat sMotto
                     (Odcl_Control_GetProperty
                       Walls_Label_TagTextBox
                       "Text"
                     )
                 )
            )
          )
        )

        (defun c:Label_TitreListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Setq
            bJust (Odcl_Control_GetProperty Walls_Label_OptionLeft "Value")
          )
          (if bJust
            (setq iTJust 0
              iAttPt 7
            )                 ;Left justified
            (setq iTJust 1
              iAttPt 8
            )                 ;center
          )

          (Setq
            sLStyle (Odcl_ComboBox_GetTBText Walls_Label_StyleComboBoxLab)
          )                   ;style?
          (if (or (= sLStyle "") (= sLStyle nil))
            (setq sLStyle (getvar "TEXTSTYLE"))
          )
          (Setq bForT (Odcl_Control_GetProperty Walls_Label_OptionT "Value"))
                          ;text ou mtext
          (Setq rTHeight (atof (Odcl_Control_GetProperty
                     Walls_Label_HeightTextBox
                     "Text"
                   )
                 )
          )
          (Setq pIns 3dPoint)
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(Motto) ")
          (princ)
        )

      ;;; ***  Coding for DwgTitles
        (defun c:Label_TitleListView_OnClicked (nRow nCol /)
          ;;    (Odcl_MessageBox "To Do: code must be added to TitreListView" (strcat (etos nRow) "-" (etos nCol)))
          (Setq iIndx nRow)
          (Odcl_Control_ForceUpdateNow Walls_Label_TitleListView)
          (Setq sMotto (Odcl_ListView_GetItemText
                 Walls_Label_TitleListView
                 nRow
                 nCol
               )
          )
          (Setq bScals (Odcl_Control_GetProperty
                 Walls_Label_ScaleCheckBox
                 "Value"
               )
          )                   ;any scale?
          (if bScals
            (Setq
          sMotto2 (strcat
                "SCALE "
                (Odcl_ComboBox_GetTBText Walls_Label_ScalComboBox)
              )
            )
            (Setq sMotto2 nil)
          )
        )

        (defun c:Label_TitleListView_OnDragnDropToAutoCAD (3dPoint /)
          (TraceIsOn ".OnDragnDropToAutoCAD")
          (Setq
            sTStyle (Odcl_ComboBox_GetTBText Walls_Label_StyleComboBoxTit)
          )                   ;style for title?
          (if (or (= sTStyle "") (= sTStyle nil))
            (setq sTStyle (getvar "TEXTSTYLE"))
          )

          (Setq
            sSStyle (Odcl_ComboBox_GetTBText Walls_Label_StyleComboBoxSca)
          )                   ;style for scale?
          (if (or (= sSStyle "") (= sSStyle nil))
            (setq sSStyle (getvar "TEXTSTYLE"))
          )

          (Setq bForT (Odcl_Control_GetProperty Walls_Label_OptionT "Value"))
          (Setq
            bJust (Odcl_Control_GetProperty Walls_Label_OptionLeft "Value")
          )
          (if bJust
            (setq iTJust 0
              iAttPt 7
            )                 ;Left justified
            (setq iTJust 1
              iAttPt 8
            )                 ;center
          )
          (Setq rTTHeight (atof (Odcl_Control_GetProperty
                      Walls_Label_HgtTextBox1
                      "Text"
                    )
                  )
          )
          (Setq rSTHeight (atof (Odcl_Control_GetProperty
                      Walls_Label_HgtTextBox2
                      "Text"
                    )
                  )
          )
          (setq rTDist (* (+ rTTHeight (* 2 rSTHeight)) 0.5))
          (Setq pIns 3dPoint)
          (Odcl_SetCmdBarFocus)
          (vla-sendcommand *active-document* "(MottoT) ")
          (if bScals
            (vla-sendcommand *active-document* "(MottoS) ")
          )
          (princ)
        )

        (defun c:Label_TitleListView_OnDragnDropFromOther (Point /)
          (setq ssDragnDrop (ssget "_P" (list (cons 0 "TEXT,MTEXT"))))
          (setq oTexte (ssname ssDragnDrop 0))
          (Odcl_ListView_AddItem
            Walls_Label_TitleListView
            (dxf 1 (entget oTexte))
          )
        )

        (defun c:Label_OptionUpper_OnClicked (nValue /)
          (setq OptionUpper T)
          (setq OptionLower nil)
          (setq OptionSentence nil)
          (Exchanger)
        )

        (defun c:Label_OptionLower_OnClicked (nValue /)
          (TraceIsOn ".Label_OptionLower_OnClicked")
          (setq OptionUpper nil)
          (setq OptionLower T)
          (setq OptionSentence nil)
          (Exchanger)
        )

        (defun c:Label_OptionSentence_OnClicked (nValue /)
          (setq OptionUpper nil)
          (setq OptionLower nil)
          (setq OptionSentence T)
          (Exchanger)
        )

        (defun c:Label_LinguaComboBox_OnSelChanged (nSelection sSelText /)
          ;;   (Odcl_MessageBox "To Do: code must be added to LinguaComboBox" (etos nSelection))
          (setq nLingua nSelection
            sLangue sSelText
          )
          (Exchanger)
        )

        (defun c:Label_TextSpinButton_OnChanged (nValue nDelta /)
          ;;    (Odcl_MessageBox "To Do: code must be added to TextSpinButton" (strcat (etos nValue) "-" (etos nDelta)))
          (setq rTHeight (* 0.1 nValue))
          (Odcl_Control_SetProperty
            Walls_Label_HeightTextBox
            "Text"
            (rtos rTHeight 2 1)
          )
        )
        (defun c:Label_TagCheckBox_OnClicked (nValue /)
          ;;     (Odcl_MessageBox "To Do: code must be added to TagCheckBox" (etos nValue))       1 or 0
          (Odcl_Control_SetProperty
            Walls_Label_TagTextBox
            "Enabled"
            nValue
          )
        )

        (defun Exchanger (/)
          (TraceIsOn ".Exchanger")
          (cond
            ((= sLangue "English")
             (Odcl_Control_SetProperty
           Walls_Label_FlagPictureBox
           "Picture"
           16
             )
            )
            ((= sLangue "Franais")
             (Odcl_Control_SetProperty
           Walls_Label_FlagPictureBox
           "Picture"
           18
             )
            )
            ((= sLangue "Italiano")
             (Odcl_Control_SetProperty
           Walls_Label_FlagPictureBox
           "Picture"
           17
             )
            )
            (T
             (Odcl_Control_SetProperty
           Walls_Label_FlagPictureBox
           "Picture"
           0
             )
            )
          )
          (setq lShow1 nil
            lShow2 nil
            lShow3 nil
          )
          (cond
            ((and (= nLingua 0) (= OptionUpper T))
             (setq lShow1 lResColumU1
               lShow2 lComColumU1
               lShow3 lIndColumU1
               sHeadr sEngRH
               sHeadc sEngCH
               sHeadi sEngIH
             )
            )
            ((and (= nLingua 0) (= OptionLower T))
             (setq lShow1 lResColumL1
               lShow2 lComColumL1
               lShow3 lIndColumL1
               sHeadr sEngRH
               sHeadc sEngCH
               sHeadi sEngIH
             )
            )
            ((and (= nLingua 0) (= OptionSentence T))
             (setq lShow1 lResColumS1
               lShow2 lComColumS1
               lShow3 lIndColumS1
               sHeadr sEngRH
               sHeadc sEngCH
               sHeadi sEngIH
             )
            )
            ((and (= nLingua 1) (= OptionUpper T))
             (setq lShow1 lResColumU2
               lShow2 lComColumU2
               lShow3 lIndColumU2
               sHeadr sFraRH
               sHeadc sFraCH
               sHeadi sFraIH
             )
            )
            ((and (= nLingua 1) (= OptionLower T))
             (setq lShow1 lResColumL2
               lShow2 lComColumL2
               lShow3 lIndColumL2
               sHeadr sFraRH
               sHeadc sFraCH
               sHeadi sFraIH
             )
            )
            ((and (= nLingua 1) (= OptionSentence T))
             (setq lShow1 lResColumS2
               lShow2 lComColumS2
               lShow3 lIndColumS2
               sHeadr sFraRH
               sHeadc sFraCH
               sHeadi sFraIH
             )
            )
            ((and (= nLingua 2) (= OptionUpper T))
             (setq lShow1 lResColumU3
               lShow2 lComColumU3
               lShow3 lIndColumU3
               sHeadr sItaRH
               sHeadc sItaCH
               sHeadi sItaIH
             )
            )
            ((and (= nLingua 2) (= OptionLower T))
             (setq lShow1 lResColumL3
               lShow2 lComColumL3
               lShow3 lIndColumL3
               sHeadr sItaRH
               sHeadc sItaCH
               sHeadi sItaIH
             )
            )
            ((and (= nLingua 2) (= OptionSentence T))
             (setq lShow1 lResColumS3
               lShow2 lComColumS3
               lShow3 lIndColumS3
               sHeadr sItaRH
               sHeadc sItaCH
               sHeadi sItaIH
             )
            )
          )
      ;;; ***  Coding for RoomLabels
                          ;      (Odcl_ListView_Clear Walls_Label_TitreListView)
          (setq column1width (Odcl_ListView_CalcColWidth
                   Walls_Label_TitreListView
                   (strcat "  " sHeadr "   ")
                     )
            column2width (Odcl_ListView_CalcColWidth
                   Walls_Label_TitreListView
                   (strcat "  " sHeadc "   ")
                     )
            column3width (Odcl_ListView_CalcColWidth
                   Walls_Label_TitreListView
                   (strcat "  " sHeadi "   ")
                     )
          )
          (Odcl_ListView_AddColumns
            Walls_Label_TitreListView
            (list 0 sHeadr 0 column1width)
            (list 1 sHeadc 0 column2width)
            (list 2 sHeadi 0 column3width)
          )
          (Odcl_ListView_FillList
            Walls_Label_TitreListView
            (list lShow1          ;add column 1 items
              lShow2          ;add column 2 items
              lShow3          ;add column 3 items
            )
          )
          (Odcl_Control_ForceUpdateNow Walls_Label_TitreListView)
        )

        (defun c:Label_OnClose ()
          ;;(Odcl_MessageBox "To Do: code must be added to Label" "To do")
          (vl-registry-write
            (strcat "HKEY_CURRENT_USER\\"
                (vlax-product-key)
                "\\RestoreLView"
            )
            "Value"
            "Cache"
          )                       ;Creates a key in the Windows registry
        )

        (Odcl_Form_Show ProjName2 "Label")
      )

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      (if (= nil bOpt)                ;Si option est nulle
        (progn
          (setq sValu3 (vl-registry-read      ;lire le registre
                 (strcat "HKEY_CURRENT_USER\\"
                     (vlax-product-key)
                     "\\RestoreLView"
                 )
                 "Value"
               )
          )

          (if (= sValu3 "Montre")         ;if the dockable form is to be shown...
            (c:LView)                 ;show it.
          )
        )
      )
      (if (= nil bOpt)
        (progn
          (setq sValu2 (vl-registry-read
                 (strcat "HKEY_CURRENT_USER\\"
                     (vlax-product-key)
                     "\\RestoreAALib"
                 )
                 "Value"
               )
          )

          (if (= sValu2 "Montre")         ;if the dockable form is to be shown...
            (c:AALib)                 ;show it.
          )
        )
      )

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      ;------------------
      ;---Nested Objects.
      ;------------------
      (defun C:EN ()    (nErase))
      (defun C:CN ()    (nCopy))
      (defun C:AN ()    (nAjout))

      ;---Returns a Variant array of subtype Object filled with the contents of a selection set.
      ;   Example: (SSToArray mySS)
      ;   Use this whenever you need to pass a selection set as an array to an ActiveX function.
      ;   If you need a different subtype, simply change the reference to vlax-vbObject.
      (defun SSToArray (ss / c r)
        (TraceIsOn ".SSToArray")
        (setq c -1)
        (repeat (sslength ss) (setq r (cons (ssname ss (setq c (1+ c))) r)))
        (setq r (reverse r))
        (vlax-safearray-fill
          (vlax-make-safearray vlax-vbObject (cons 0 (1- (length r))))
          (mapcar 'vlax-ename->vla-object r)
        )
      )

      ;---Returns a list containing the entity names of any block definitions that reference
      ;   the specified block. (AllBlockElements "DoorDouble")
      (defun AllBlockElements (blkName / doc)
        (TraceIsOn ".AllBlockElements")
        (setq doc (vla-get-ActiveDocument (vlax-get-acad-object)))
        (apply 'append
           (mapcar '(lambda (x)
                  (if (= :vlax-false
                     (vla-get-IsLayout
                       (vla-ObjectIdToObject doc (vla-get-OwnerId (vlax-ename->vla-object x)))
                     )
                  )
                    (list x)
                  )
                )
               (Ref2BlocksLst blkName)
           )
        )
      )

      ;---
      (defun Vis-SSet (ss code / na e1 n)
        (TraceIsOn ".Vis-SSet1")
        (if ss
          (progn (setq n 0)
             (repeat (sslength ss)
               (setq na (ssname ss n)
                 e1 (entget na)
               )              ;setq
               (TraceIsOn ".Vis-SSet2")
               (if (not (assoc 60 e1))
                 (setq e1 (append e1 (list (cons 60 code)))) ;setq then
                 (setq e1 (subst (cons 60 code) (assoc 60 e1) e1)) ;setq else
               )              ;if
               (entmod e1)
               (entupd na)
               (setq n (+ n 1))       ;setq
             )                ;repeat
          )                   ;progn
        )                 ;if
      )                   ;defun Vis-SSet

      ;sets current ucs to be parallel to the extrusion vector specified by p1. This function is used by extrim and exchprop
      (defun AKs_ucs_2_ent (p1 /)
        (TraceIsOn ".AKs_ucs_2_ent")
        (setq p1 (strcat "*" (rtos (car p1) 2 8) "," (rtos (cadr p1) 2 8) "," (rtos (caddr p1) 2 8))
                          ;strcat
        )                 ;setq
        (command "_.ucs" "_za" "*0.0,0.0,0.0" p1)
      )                   ;defun AKs_ucs_2_ent

      ;Clears the previous selection set in AutoCAD by executing the select command and then performing an undo 1
      ; NOTE: This function does not verify that undo is enabled and will fail if undo is disabled.
      (defun clearPrevSSet ()
        (TraceIsOn ".clearPrevSSet")
        (if (entnext)
          (progn (command "_.select" (entlast) (entnext))
             (while (not (equal (getvar "cmdnames") "")) (command ""))
             (command "_.undo" "1")   ;clear any selection sets
          )                   ;progn then
        )                 ;if
      )                   ;defun clearPrevSSet

      ;Takes a selection set of entities and a block name.
      ;If the block name (bna) is nil then a new anonymous block
      ;will be created. Otherwise the block will be re-defined.
      ;Also note that the bna argument can be passed in a list that has
      ;a two elements (bna flag), where a non-nil 'flag' will tell NobodysBusiness
      ;to delete the entities found in ss after they are placed in the block table.
      (defun NobodysBusiness (ss bna / AFaireByLayer flag n na e1)
        (TraceIsOn ".NobodysBusiness")
        (defun AFaireByLayer (e1 /)
          (if (not (assoc 62 e1))     ;add color and linetype info if bylayer
            (setq e1 (append e1 (list '(62 . 256)))) ;setq then force bylayer
          )                   ;if
          (if (not (assoc 6 e1))
            (setq e1 (append e1 (list '(6 . "BYLAYER")))) ;setq then force bylayer
          )                   ;if
          e1
        )                 ;defun AFaireByLayer
        (if (equal (type bna) 'LIST)
          (setq flag (cadr bna)
            bna  (car bna)
          )                   ;setq then
        )                 ;if
        (if (not bna)
          (setq bna "*T999")
        )                 ;if
        (entmake nil)
        (setq bna (entmake (list '(0 . "BLOCK") (cons 2 bna) '(10 0.0 0.0 0.0) '(70 . 1) (cons 3 bna))
                          ;list
              )               ;entmake
        )                 ;setq
        (setq n 0)                ;setq
        (repeat (sslength ss)
          (setq na (ssname ss n)
            e1 (entget na '("*"))
            e1 (RemGroup -1 e1)
            e1 (RemGroup 5 e1)
            e1 (AFaireByLayer e1)
          )                   ;setq
          (if flag
            (entdel na)           ;remove the entity
          )                   ;if
          (entmake e1)
          (if (equal '(66 . 1) (assoc 66 e1))
            (progn (setq na (entnext na)
                 e1 (entget na)
                 e1 (RemGroup -1 e1)
                 e1 (RemGroup 5 e1)
                 e1 (AFaireByLayer e1)
               )              ;setq
               (while (and na (not (wcmatch (cdr (assoc 0 e1)) "*END*"))) ;and
                 (entmake e1)
                 (setq na (entnext na)
                   e1 (entget na)
                   e1 (RemGroup -1 e1)
                   e1 (RemGroup 5 e1)
                   e1 (AFaireByLayer e1)
                 )            ;setq
               )              ;while
               (entmake e1)
            )                 ;progn then
          )                   ;if
          (setq n (+ n 1))            ;setq
        )                 ;repeat
        (setq bna (entmake (list (cons 0 "ENDBLK")))) ;setq
        bna
      )                   ;defun NobodysBusiness

      (defun RemGroup (a e1 / n e2 b)
        (TraceIsOn ".RemGroup")
        (setq n 0)
        (repeat (length e1)
          (setq b (nth n e1))
          (if (not (equal a (car b)))
            (setq e2 (append e2 (list b)))    ;setq
          )                   ;if
          (setq n (+ n 1))            ;setq
        )                 ;repeat
        e2
      )                   ;defun RemGroup

      ;;Makes a selection set from all entities after na SSetRenewed ss
      (defun SSetRenewed (na /)       ;**ss e1
        (TraceIsOn ".SSetRenewed")
        (if na
          (setq na (entnext na))
          (setq na (entnext))
        )                 ;if
        (setq ss (ssadd))
        (while na
          (setq e1 (entget na))
          (if (and (not (equal "VERTEX" (cdr (assoc 0 e1)))) ;not equal
               (not (equal "SEQEND" (cdr (assoc 0 e1)))) ;not equal
          )               ;and
            (setq ss (ssadd na ss))
          )                   ;if
          (Cassure "ss")
          (setq na (entnext na))
        )                 ;while
        ss
      )                   ;defun SSetRenewed

      (defun b_purge (bna /)
        (TraceIsOn ".b_purge")
        (if (and bna (tblobjname "block" bna)) ;and
          (progn (command "_.purge" "_block" bna)
             (while (wcmatch (getvar "cmdnames") "*PURGE*") (command "_y")) ;while
          )                   ;progn
        )                 ;if
      )                   ;defun b_purge

      (defun find_pline_header (na na2 / na3 na4)
        (TraceIsOn ".find_pline_header")
        (setq na3 (tblobjname "block" (cdr (assoc 2 (entget na))))) ;setq
        (while (and na3 (not (equal na2 na3))) ;and
          (if (equal "POLYLINE" (cdr (assoc 0 (entget na3))))
            (setq na4 na3)            ;a polyline header that could be the one we need.
          )                   ;if
          (setq na3 (entnext na3))
        )                 ;while
        na4
      )                   ;defun find_pline_header

      (defun extract_clone (lst / flag flag2 bna bna2 bna3 lst3 j n na ss e1)
        (TraceIsOn ".extract_clone")
        (setq na   (car lst)
          e1   (entget na)
          lst3 (last lst)         ;the ent family tree
          j    (- (length lst3) 1)
        )                 ;setq
        (if (setq flag2 (car lst3))
          (progn (AKs_ucs_2_ent (cdr (assoc 210 (entget (car lst3))))) ;5***
          )                   ;progn
        )                 ;if
        (if (equal "VERTEX" (cdr (assoc 0 e1)))
          (setq na (find_pline_header (car lst3) na) ;6***
            e1 (entget na)
          )                   ;setq
        )                 ;if
        (if (> (length lst3) 0)
          (progn (if (not (assoc 60 e1))  ;set the invisibility flag
               (setq e1 (append e1 (list (cons 60 1)))) ;setq then
               (setq e1 (subst (cons 60 1) (assoc 60 e1) e1)) ;setq else
             )                ;if
          )                   ;progn
        )                 ;if
        (setq e1 (CodeRemoval e1))        ;setq
        (if (equal "IMAGE" (cdr (assoc 0 e1)))
          (image_make na e1)          ;Make the nested ent the way it was first drawn
          (entmake e1)            ;except this time it's invisible!
        )                 ;if
        (if (equal '(66 . 1) (assoc 66 e1))   ;just in case its some variety of polyline,
          (progn              ;then better finish the entmake job.
            (setq na (entnext na)
              e1 (entget na)
            )                 ;setq
            (if (not (equal "ATTRIB" (cdr (assoc 0 e1))))
          (setq e1 (CodeRemoval e1))  ;setq  ;rk added if code around this
            )                 ;if
            (while (and na (not (wcmatch (cdr (assoc 0 e1)) "*END*"))) ;and
          (entmake e1)
          (setq na (entnext na)
                e1 (entget na)
          )               ;setq
          (if (not (equal "ATTRIB" (cdr (assoc 0 e1))))
            (setq e1 (CodeRemoval e1))    ;setq  ;rk added if code around this
          )               ;if
            )                 ;while
            (entmake e1)
          )                   ;progn then
        )                 ;if
        (setq na (entlast)
          ss nil
          ss (ssadd)
          ss (ssadd na ss)
        )                 ;setq
        (setq bna  (NobodysBusiness ss nil)   ;define an anonymous block
          bna2 bna            ;save the name of the original for comparison later
          bna3 ""             ;a string for adding names of purge-ables
        )                 ;setq
        (setvar "cmdecho" 0)
        (setvar "highlight" 0)
        (if (> (length lst3) 0)
          (command "_.erase" na "")       ;and then remove the ent.
        )                 ;if
        (setq n 0)
        (repeat (length lst3)
          (setq e1 (entget (nth n lst3))  ;get the insert.
            e1 (subst (cons 2 bna) (assoc 2 e1) e1) ;put in the name of the anonymous block
            e1 (CodeRemoval e1)       ;remove the entname and the handle
            e1 (subst (cons 66 0) (assoc 66 e1) e1)
          )                   ;setq
          (entmake e1)            ;make the insert
          (setq na (entlast))         ;setq
          (shift_xref_insbase na (nth n lst3))
          (command "_.explode" na)        ;explode it
          (Cassure "Tout saute")
          (while (wcmatch (getvar "cmdnames") "*EXPLODE*") ;si c'est un bloc nested
            (command "")
          )                   ;while
          (if (not (entget na))
            (progn (setq ss (SSetRenewed na)) ;setq
               (setq bna3 (strcat bna3 ",`" bna)) ;purge it out later
               (if (not (equal n j))
                 (progn (Vis-SSet ss 1)   ;Make the ents stealth.
                    ;;            (command "CHPROP" (ssname ss 0) "Co" "C" "")
                    (setq bna (NobodysBusiness ss nil) ;Define the anonymous block
                    )         ;setq                               ;such that it's non-nested.
                 )            ;progn
               )              ;if
            )                 ;progn then the explode operation was a success.
            (progn (setq ss nil
                 ss (ssadd)
                 ss (ssadd na ss)
               )              ;setq
               (if (not flag)
                 (setq flag bna)      ;then save the first non-exploded anonymous block name
               )              ;if
               (if (not (equal n j))
                 (setq bna (NobodysBusiness ss nil)) ;Define a nested anonymous block cuz we
               )              ;if        ;couldn't explode the insert of the last one
            )                 ;progn else
          )                   ;if
          (if (not (equal n j))       ;if this isn't the last one,
            (command "_.erase" ss "")     ;then remove the ents used to create block.
            (progn                ;Else turn the stuff back on cuz this is
          (if (not flag)          ;the last time through.
            (Vis-SSet ss 0)       ;then
            (progn (setq na  (tblobjname "block" flag)
                     e1  (entget na)
                     lst (list e1)
               )          ;setq
               (while (setq na (entnext na))
                 (princ na)
                 (setq e1  (entget na)
                   e1  (subst (cons 60 0) (assoc 60 e1) e1)
                   lst (append lst (list e1))
                 )            ;setq
               )          ;while
               (while lst
                 (entmake (car lst))
                 (setq lst (cdr lst)) ;setq
               )          ;while
               (entmake '((0 . "ENDBLK")))
               (Cassure "before move")
               (command "_.move" ss "" "0,0" "0,0") ;Done for highlighting reasons.
            )             ;progn else need to redefine the first block
          )               ;if
            )                 ;progn else make the ents visible again
          )                   ;if
          (setq n (+ n 1))            ;setq
        )                 ;repeat
        (if flag2
          (command "_.ucs" "_p")
        )                 ;if
        (if (not (equal bna3 ""))
          (progn (setq bna3 (substr bna3 2)) (b_purge bna3)) ;progn
        )
        ss
      )

      (defun shift_xref_insbase (ss na / bna p1 e1 e2)

        (if (and ss
             na
             (setq e1 (entget na))    ;setq
             (setq bna (cdr (assoc 2 e1))) ;setq
             (setq e2 (entget (tblobjname "block" bna)))

             (equal 4 (logand 4 (cdr (assoc 70 e2))))
             (setq p1 (cdr (assoc 10 e2)))
             (not (equal p1 '(0.0 0.0 0.0)))
            )                 ;and
          (command "_.move" ss "" "0,0,0" (scale_vector p1 -1.0))
        )
      )

      ;;;Dictionary notes:
      ;;;na  IMAGE
      ;;;na2  340 -> IMAGEDEF (missing in xref)
      ;;;na3          330 -> DICTIONARY1
      ;;;na4                  330 -> DICTIONARY2 (3  AND 350)
      ;;;na5                          "ACAD_IMAGE_DICT" 350 -> DICTIONARY1
      ;;;na6                          "ACAD_IMAGE_VARS" 350 -> "RASTERVARIABLES"
      ;;;na7                                                      DICTIONARY2
      ;;;na8                  350 -> IMAGEDEF
      ;;;na9          330 -> IMAGEDEF_REACTOR
      ;;;na10                 330 -> IMAGE
      ;;;na11 360 -> IMAGEDEF_REACTOR
      ;;;
      ;;; group {102 330 102}
      ;;; leader 340
      ;;; mline  340
      ;;; hatch  330
      ;;;
      ;;;mline has a mlstyle of group 2

      (defun image_make (na e1 / na2 e2 na3 e3 na4 e4 p1 p2 ina ina2)
        (setq p1   (image_bounds na)
          p2   (trans
                 (cadr p1)
                 0
                 1
               )
          p1   (trans
                 (car p1)
                 0
                 1
               )
          na2  (cdr (assoc 340 e1))
          e2   (entget na2)       ;the imagedef
          ina  (cdr (assoc 1 e2))     ;the image name w/full path
          na3  (cdr (assoc 330 e2))   ;owner dict
          e3   (entget na3)
          ina2 (cdr (assoc 3 (member (cons 350 na2) (reverse e3))))
                          ;image name w/out path
          na4  (namedobjdict)     ;current drawings' dictionary
          e4   (dictsearch na4 "ACAD_IMAGE_DICT") ;image dict
        )                 ;setq
        (if (member (cons 3 ina2) e4)
          (setq ina ina2)         ;then use the short name instead.
        )                 ;if

        (setq na (entlast))
        (command "_.-image" "_a" ina p1 p2 p2) ;command

        (if (equal na (entlast))
          (progn
            (alert (strcat "Problem attaching image: "
                   ina
                   "\nThe file may not be on AutoCAD's search path"
               )              ;strcat
            )                 ;alert
            (exit)
          )                   ;progn
          (progn
            (setq e2 (entget (entlast))
              e3 (append (list (assoc -1 e2)) e1)
              e3 (subst (assoc 5 e2) (assoc 5 e1) e3)
              e3 (subst (assoc 340 e2) (assoc 340 e1) e3)
              e3 (subst (assoc 360 e2) (assoc 360 e1) e3)
            )                 ;setq
            (entmod e3)
            (entupd (cdr (assoc -1 e3)))
            (Vis-SSet (ssadd (entlast) (ssadd)) 1)
          )
        )
      )

      (defun GetObjNested (msg /)     ;**flt errno_sel lst ss ss2 na e1 n a lst2
        (TraceIsOn ".GetObjNested")

        (defun errno_sel (msg / a)
          (TraceIsOn ". errno_sel")
          (setvar "errno" 0)
          (while (and (not (setq a (nentsel msg))) (equal 7 (getvar "errno")))
            (setvar "errno" 0)
          )                   ;while
          a
        )

        (setvar "cmdecho" 0)
        (clearPrevSSet)           ;2***
        (setvar "highlight" 1)
        (command "_.select")
        (while (setq lst (errno_sel msg)) ;3***
          (setq na (car lst)
            e1 (entget na)
          )                   ;setq
          (if (and (not (equal (length lst) 4)) (not (equal "ATTRIB" (cdr (assoc 0 e1))))) ;and
            (progn (if (equal "VERTEX" (cdr (assoc 0 e1)))
                 (command (cadr lst)) ;then get the polyline header by directly selecting
                 (setq ss nil
                   ss (ssadd)
                   ss (ssadd (car lst) ss)
                 )            ;setq
               )              ;if
            )                 ;progn then it's non-nested.
            (progn (command "")
               (setq ss2 (ssget "_p"))
               (if (and (equal "ATTRIB" (cdr (assoc 0 e1))) (not (equal (length lst) 4))) ;and
                 (progn (setq lst (append lst
                          (list nil)
                          (list nil)
                      )   ;append
                    )         ;setq
                 )            ;progn then do special stuff for the attrib
               )              ;if
               (setq ss (extract_clone lst)) ;4***
               (if (and ss (> (sslength ss) 0)) ;and
                 (setq lst2 (append lst2 (list ss)))
               )              ;if
               (setvar "cmdecho" 0)
               (clearPrevSSet)
               (setvar "highlight" 1)
               (command "_.select")
               (if (and ss2 (> (sslength ss2) 0)) ;and
                 (command ss2)
               )              ;if
            )                 ;progn else it's nested
          )                   ;if
          (if (and ss (> (sslength ss) 0))    ;and
            (command ss)
          )                   ;if
        )                 ;while
        (while (wcmatch (getvar "cmdnames") "*SELECT*") (command "")) ;while
        (setq ss (ssget "_p"))
        (setvar "cmdecho" 0)
        (setvar "highlight" 0)
        (clearPrevSSet)
        (command "_.select")
        (setq n 0)
        (repeat (length lst2)
          (setq a (nth n lst2))
          (command a)
          (setq n (+ n 1))            ;setq
        )                 ;repeat
        (command "")
        (setq lst2 nil)
        (setq ss2 (ssget "_p"))       ;nested ents
        (list ss ss2)
      )                   ;defun GetObjNested

      (defun ManipObj (msg do flt /)      ;**flag la ss ss2 n
        (TraceIsOn ".ManipObj")
        (if (setq la   (getvar "clayer")
              flag (IsLayerLlocked la)
            )                 ;setq
          (command "_.layer" "_unlock" la "")
        )                 ;if
        (setq ss    (GetObjNested msg)    ;<----1***
          ss2   (cadr ss)         ;nested only
          ss    (car ss)          ;all.
          jerri ss
        )                 ;setq
        (if
          (and
            ss
            (> (sslength ss) 0)
            (or (not flt)
            (progn (setq n (sslength ss))
               (command "_.select" ss "")
               (if (and (setq ss (ssget "_p" flt)) (not (equal n (sslength ss)))) ;and
                 (princ (strcat "\n" (itoa (- n (sslength ss))) " objects were invalid for " do))
                          ;print strcat
               )          ;if
               ss
            )             ;progn
            )                 ;or
          )                   ;and
           (progn (setvar "cmdecho" 0)
              (setvar "highlight" 1)
              (command (strcat "_." do) ss)
              (setvar "cmdecho" 1)
              (if (wcmatch (getvar "cmdnames") (strcat "*" (strcase do) "*"))
                (command "")
              )               ;if
              (while (wcmatch (getvar "cmdnames") (strcat "*" (strcase do) "*")) (command pause))
                          ;while
           )                  ;progn
        )                 ;if
        (setvar "cmdecho" 0)
        (if (and ss2 (> (sslength ss2) 0))    ;and
          (progn (if (IsLayerLlocked (getvar "clayer")) ;just in case of transparent layer changes
               (command "_.layer" "_unlock" la "")
             )                ;if
             (command "_.erase" ss2 "")
             (if (not ss)
               (princ (strcat "\nNo valid objects selected for " do ".\n"))
             )                ;if
          )                   ;progn
        )                 ;if
        (if (entlast)
          (progn (command "_.erase" (entlast) "") ;good for copy seulement not for move ou erase
             (command "_.oops")
          )                   ;progn clear "oops"
        )                 ;if
        (if flag
          (command "_.layer" "_lock" la "")
        )                 ; (restore_old_error)
        (princ)
      )                   ;defun ManipObj

      (defun CodeRemoval (e1 / na2 na3 e2 e3)
        (TraceIsOn ".CodeRemoval")
        (setq e1 (RemGroup -1 e1)
          e1 (RemGroup 5 e1)
          e1 (RemGroup 102 e1)
          e1 (RemGroup 67 e1)
          e1 (remx_codes e1)
        )                 ;setq
        (if (not (assoc 62 e1))
          (setq e1 (append e1 (list '(62 . 256)))) ;setq then force bylayer
        )                 ;if
        (if (not (assoc 6 e1))
          (setq e1 (append e1 (list '(6 . "BYLAYER")))) ;setq then force bylayer
        )                 ;if
        (cond ((equal "ATTRIB" (cdr (assoc 0 e1)))
           (setq e1 (RemGroup 3 e1)
                 e1 (subst '(0 . "ATTDEF") '(0 . "ATTRIB") e1)
                 e1 (append e1 (list (cons 3 (cdr (assoc 1 e1)))))
                 e1 (RemGroup 100 e1)
           )              ;setq
          )
          ((equal "HATCH" (cdr (assoc 0 e1)))
           (setq e1 (subst '(71 . 0) (assoc 71 e1) e1)
                 e1 (subst '(97 . 0) (assoc 97 e1) e1)
                 e1 (RemGroup 330 e1)
           )              ;setq
          )
          ((equal "MLINE" (cdr (assoc 0 e1)))
           (setq e2  (dictsearch (namedobjdict) "ACAD_MLINESTYLE")
                 na2 (cdr (assoc -1 e2))
                 na3 (cdr (assoc 350 (member (assoc 3 e2) e2)))
                 e1  (subst (cons 340 na3) (assoc 340 e1) e1)
           )
          )
          ((equal "MTEXT" (cdr (assoc 0 e1))) (setq e1 (RemGroup 50 e1)))
        )                 ;cond
        e1
      )                   ;defun CodeRemoval

      (defun remx_codes (e1 / lst a b c d n na2 e2)
        (TraceIsOn ".remx_codes")
        (setq d   (cdr (assoc 0 e1))
          lst (list '(6 . "LTYPE") '(7 . "STYLE") '(8 . "LAYER")) ;list
        )                 ;setq
        (if (or (equal "DIMENSION" d) (equal "TOLERANCE" d) (equal "LEADER" d)) ;or
          (setq lst (append lst (list '(3 . "DIMSTYLE")))) ;setq then deal with dimstyles
        )                 ;if
        (setq n 0)                ;setq
        (repeat (length lst)
          (setq a (nth n lst)
            b (cdr a)
            a (car a)
            c (cdr (assoc a e1))
          )                   ;setq
          (if (and c
               (setq na2 (tblobjname b c))
               (setq e2 (entget na2))
               (equal 16 (logand 16 (cdr (assoc 70 e2))))
          )               ;and
            (progn                ;yank it out for now.
          (setq e1 (RemGroup a e1))   ;(if (equal 32 (logand 32 (cdr (assoc 70 e2))))
                          ;    (progn
                          ;     ;bind it?
                          ;    );progn
                          ;    (setq e1 (rem_code a e1));yank it
                          ;);if
            )                 ;progn then xref dependant symbol table entry.
          )                   ;if
          (setq n (+ n 1))            ;setq
        )                 ;repeat
        e1
      )                   ;defun remx_code

      ;---Deletes the specified subentity from a block definition and returns the remaining number of
      ;   items in that block definition. The entity name of the subentity to delete.
      ;   Example: (DelFromBlock (car (nentsel)))
      ;   As shown, you can use the NENTSEL function to obtain the name of an entity within a block.
      ;   Existing block references will not show a change until you regen the drawing.
      (defun DelFromBlock (ent / doc blk)
        (TraceIsOn ".DelFromBlock")
        (setq doc (vla-get-ActiveDocument (vlax-get-acad-object))
          ent (vlax-ename->vla-object ent)
          blk (vla-ObjectIdToObject doc (vla-get-OwnerID ent))
        )
        (vla-Delete ent)
        (vla-get-Count blk)
      )

      (defun nCopy (/)            ;Name to be changed
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nCopy Nested Objects." copyright))
        (ManipObj "\nSelect nested objects to copy:" "Copy" nil)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun nErase (/)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nErase Nested Object." copyright))
        (princ "\nSelect one nested object to erase:")
        (DelFromBlock (car (nentsel)))
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;---------------
      ;---Slice in 2D.
      ;---------------
      (defun c:SL2 () (Slice2D T))

      ;;**added trans WCS to UCS 7/20/02
      ;;**added to pass two enames and extend cond
      ;;    - acExtendNone:        Does not extend either object.
      ;;    - acExtendThisEntity:  Extends the Fst object.
      ;;    - acExtendOtherEntity: Extends the Nxt object.
      ;;    - acExtendBoth:        Extends both objects.
      ;; (setq PTS (getinters CutEdg Obj2Slice acExtendNone))
      (defun GetInters (Fst Nxt Mde / IntPnt PNTLST NEWLST TmpLst TmpVal)
        (TraceIsOn ".GetInters")
        (setq Fst    (vlax-ename->vla-object Fst) ;**added
          Nxt    (vlax-ename->vla-object Nxt) ;**added
          IntPnt (vla-IntersectWith Fst Nxt Mde)
          TmpVal (vlax-variant-value IntPnt)
        )
        (if (safearray-value TmpVal)
          (progn (setq TmpLst (vlax-safearray->list TmpVal))
             (repeat (/ (length TmpLst) 3)
               (setq PNTLST (cons (list (car TmpLst) (cadr TmpLst) (caddr TmpLst))
                      PNTLST
                    )
                 TmpLst (cdddr TmpLst)
               )
             )
             (reverse PNTLST)
             (foreach x PNTLST        ;**added
               (setq NEWLST (cons (trans x 0 1) NEWLST))
             )
          )
          nil
        )
      )

      ;;;returns a selection set of elements after ename ent if nil, ent was entlast
      (defun SSAfterObj (ent / ssAdded)
        (TraceIsOn ".SSAfterObj")
        (cond ((entnext ent)
           (setq ssAdded (ssadd))
           (while (setq ent (entnext ent)) (ssadd ent ssAdded))
          )
        )
      )

      ;;;returns the deflection angle (in radians) of two angles
      (defun DeflAng (aPrimo aSecondo /)
        (TraceIsOn ".DeflAng")
        (cond ((> aPrimo (+ aSecondo pi))
           (setq aSecondo (+ aSecondo (* 2 pi)))
          )
          ((> aSecondo (+ aPrimo pi))
           (setq aPrimo (+ aPrimo (* 2 pi)))
          )
        )
        (- aSecondo aPrimo)
      )

      ;;;remove duplicate adjacent points from point list such as list generated from a pline.
      ;;;arguments: OLST - pointlist, FUZ - fuzz distance returns new list with duplicates removed.
      (defun RemDupPts (OLST FUZ / NLST P1 P2)
        (TraceIsOn ".RemDupPts")
        (while (> (length OLST) 1)
          (setq P1 (car OLST)
            P2 (cadr OLST)
          )
          (if (> (distance P1 P2) FUZ)
            (setq NLST (cons P1 NLST))
          )                   ;if
          (setq OLST (cdr OLST))
        )                 ;while
        (setq NLST (cons (last OLST) NLST))
        (reverse NLST)
      )

      (defun CercleBrise (ENAME / ELST aStrt aEnd)
        (TraceIsOn ".CercleBrise")
        (if (= "ARC" (dxf 0 (entget ENAME)))
          (progn (setq ELST (entget ENAME))   ;circle is arc
             (setq aStrt (dxf 50 ELST))
             (setq aEnd (dxf 51 ELST))
             (setq ELST (subst (cons 50 aEnd) (assoc 50 ELST) ELST))
             (setq ELST (subst (cons 51 aStrt) (assoc 51 ELST) ELST))
             (entmake ELST)
          )
        )
      )

      (defun EllipseBrisee (ENAME / ELST STPRAM ENPRAM)
        (TraceIsOn ".EllipseBrisee")
        (if (= "ELLIPSE" (dxf 0 (entget ENAME)))
          (progn (setq ELST (entget ENAME))   ;ellipse broken
             (setq STPRAM (dxf 41 ELST))
             (setq ENPRAM (dxf 42 ELST))
             (setq ELST (subst (cons 41 ENPRAM) (assoc 41 ELST) ELST))
             (setq ELST (subst (cons 42 STPRAM) (assoc 42 ELST) ELST))
             (entmake ELST)
          )                   ;progn
        )                 ;if
      )

      ;;break along selected line, lwpolyline, circle, arc or spline does not break closed splines, 3D plines, blocks, mlines, dims, leaders
      ;;breaks lines, acrs, circles, ellipses, polylines, lwplines, open splines
      ;;note: circles will break if picked object crosses more than once same with closed splines
      (defun Slice2D (Flagg  /      OSM      PTLST  TPTLST PTS
              MARK   CNT    PT     RAD    CENPT  STPT   aStrt  2ndANG
              INANG  INCRANG       DIV    PER    INC    SSPTS  PTNAME
              P0     P1     P2     P3     P4     P5
                 )
        (setq olderror  *error*
          *error*   Wallserror
          ssToSlice nil
          CutEdg    nil
          Obj2Slice nil
        )
        (Wallsgather_info)

        (setq MARK (entlast))
        (if Flagg
          (progn
            (setq CutEdg
               (car
                 (entsel
               "\nSelect ARC, CIRCLE, LINE, POLYLINE, and SPLINE as slicing object..."
                 )
               )
            )
            (while (or (null CutEdg)
               (and (/= "LWPOLYLINE" (dxf 0 (entget CutEdg)))
                    (/= "LINE" (dxf 0 (entget CutEdg)))
                    (/= "ARC" (dxf 0 (entget CutEdg)))
                    (/= "CIRCLE" (dxf 0 (entget CutEdg)))
                    (/= "SPLINE" (dxf 0 (entget CutEdg)))
                    (/= "POLYLINE" (dxf 0 (entget CutEdg)))
               )          ;and
               )              ;or
          (setq CutEdg
                 (car
               (entsel
                 "\nWrong object type or missed pick - try again: "
               )
                 )
          )
            )                 ;while
          )
          (setq CutEdg (entlast))
        )
        (setq OBJLST (entget CutEdg))
        (cond
          ((= "POLYLINE" (dxf 0 OBJLST))
           (setq PTLST (Liste4Pline CutEdg))
           (Cassure "c'est une pline ")
           (if (isclosed CutEdg)      ;add first point if closed
             (setq PTLST (reverse (cons (car PTLST) (reverse PTLST))))
           )                  ;if
           (setq PTLST (RemDupPts PTLST 0.1))
          )
          ((= "LWPOLYLINE" (dxf 0 OBJLST))
           (Cassure "c'est une lwpline ")
           (setq PTLST (massoc 10 OBJLST))
           (if (isclosed CutEdg)      ;add first point if closed
             (setq PTLST (reverse (cons (car PTLST) (reverse PTLST))))
           )                  ;if
           (setq PTLST (RemDupPts PTLST 0.1))
          )                   ;cond lwpline
          ((= "LINE" (dxf 0 OBJLST))
           (setq PTLST (list (cdr (assoc 10 OBJLST)) (cdr (assoc 11 OBJLST))))
          )                   ;cond line
          ((= "ARC" (dxf 0 OBJLST))
           (setq CENPT   (dxf 10 OBJLST)  ;center pt
             RAD     (dxf 40 OBJLST)  ;radius
             aStrt   (dxf 50 OBJLST)  ;start angle
             aEnd    (dxf 51 OBJLST)
             STPT    (polar CENPT aStrt RAD) ;start pt
             INANG   (DeflAng aStrt aEnd) ;included angle
             DIV     (fix (/ INANG (/ pi 180)))
                          ;no of div, approx one degree
             INCRANG (/ INANG DIV)    ;increment angle
             2ndANG  (+ aStrt INCRANG)    ;second angle
             PTLST   (list STPT)
           )                  ;setq
           (while (> DIV 1)
             (setq PT (polar CENPT 2ndANG RAD)) ;2nd pt
             (setq PTLST (cons PT PTLST))
             (setq 2ndANG (+ 2ndANG INCRANG))
             (setq DIV (1- DIV))
           )                  ;while
          )                   ;cond arc
          ((= "CIRCLE" (dxf 0 OBJLST))
           (setq CENPT   (dxf 10 OBJLST)  ;center pt
             RAD     (dxf 40 OBJLST)  ;radius
             STPT    (polar CENPT 0.0 RAD) ;start pt
             aStrt   0.0          ;start angle
             DIV     360          ;number of divisions
             INCRANG (/ pi 180)       ;increment angle
             2ndANG  (+ aStrt INCRANG)    ;second angle
             PTLST   (list STPT)
           )                  ;setq
           (while (> DIV 1)
             (setq PT (polar CENPT 2ndANG RAD)) ;2nd pt
             (setq PTLST (cons PT PTLST))
             (setq 2ndANG (+ 2ndANG INCRANG))
             (setq DIV (1- DIV))
           )                  ;while
          )                   ;cond circle
          ((= "SPLINE" (dxf 0 OBJLST))
           (command "._area" "_o" CutEdg)
           (setq PER (getvar "perimeter"))
           (setq INC (fix (/ PER 2)))
           (command "._divide" CutEdg INC)
           (setq SSPTS (SSAfterObj MARK))
           (while (> (sslength SSPTS) 0)
             (setq PTNAME (ssname SSPTS 0))
             (setq PT (dxf 10 (entget PTNAME)))
             (setq PTLST (cons PT PTLST))
             (ssdel PTNAME SSPTS)
           )                  ;while
           (command "._erase" (SSAfterObj MARK) "")
           (setq P4    (cadr PTLST)
             P3    (car PTLST)
             P5    (polar P3 (angle P4 P3) (distance P4 P3))
             PTLST (cons P5 PTLST)    ;add point at end
             P2    (cadr (reverse PTLST))
             P1    (car (reverse PTLST))
             P0    (polar P1 (angle P2 P1) (distance P1 P2))
             PTLST (cons P0 (reverse PTLST))
             PTLST (reverse PTLST)
           )                  ;setq
          )                   ;cond spline
        )
        (foreach x PTLST          ;trans for rotated UCS
          (setq TPTLST (cons (trans x 0 1) TPTLST))
        )                 ;WCS to UCS
        (setq TPTLST (reverse TPTLST))
        (if (setq
          ssToSlice
           (ssget
             "_F"
             TPTLST
             (list
               (cons
                 0
                 "LINE,LWPOLYLINE,POLYLINE,ARC,CIRCLE,ELLIPSE,SPLINE"
               )
             )
           )
            )
          (progn (ssdel CutEdg ssToSlice) ;remove CutEdg from fence selection
             (setq CNT (sslength ssToSlice))
          )
        )                 ;if
        (command "_undo" "_end" "_undo" "_begin")
        (if (> (sslength ssToSlice) 0)
          (while (> (sslength ssToSlice) 0)
            (setq Obj2Slice (ssname ssToSlice 0))
            (setq PTS (getinters CutEdg Obj2Slice acextendnone))
            (entdel CutEdg)           ;temp remove CutEdg
            (if (= "CIRCLE" (dxf 0 (entget Obj2Slice)))
          (progn (Cassure "CIRCLE: ")
                 (command "_.break" Obj2Slice (car PTS) (cadr PTS))
                 (CercleBrise Obj2Slice)
          )
            )                 ;if
            (if (and (= "ELLIPSE" (dxf 0 (entget Obj2Slice)))
                 (= 0.0 (dxf 41 (entget Obj2Slice))) ;closed
            )
          (progn (Cassure "ELLIPSE: ")
                 (command "_.break" Obj2Slice (car PTS) (cadr PTS))
                 (EllipseBrisee Obj2Slice)
          )
            )                 ;if
            (command "_.break" Obj2Slice (car PTS) (car PTS))
            (Cassure "ELSE: ")
            (if (cdr PTS)         ;if there's more than one intersection point
          (progn (setq PTS (cdr PTS))
                 (while (> (length PTS) 0)
               (command "_.break" (car PTS) (car PTS) (car PTS))
               (setq PTS (cdr PTS))
                 )            ;while
          )               ;progn
            )                 ;if
            (entdel CutEdg)           ;restore CutEdg
            (ssdel Obj2Slice ssToSlice)
          )                   ;while
          (princ "\nNo valid objects found crossing slicing object. ")
        )                 ;if
        (command "_undo" "_end")
        (if (> CNT 0)
          (princ (strcat "\nObjects found " (itoa CNT)))
        )                 ;if
        (if (SSAfterObj MARK)
          (princ (strcat "\nObjects created "
                 (itoa (sslength (SSAfterObj MARK)))
             )
          )
          (princ "\nNone created. ")
        )                 ;if
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;----------------------------
      ;---Reactor Object for Areas.
      ;----------------------------
      (defun c:OA (/) (ObjArea))

      (defun ObjArea (/ elist enam OType object AreaReactor pt hText ar aim aim2 m2 szText)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nObject Area." copyright))
        (setq enam (car (entsel)))
        (setq elist (entget enam))
        (setq OType (cdr (assoc 0 elist)))
        (if (or (= OType "CIRCLE")(= OType "POLYLINE")(= OType "LWPOLYLINE")(= OType "SPLINE")(= OType "ELLIPSE"))
          (progn
            ;; First check if there is a reference to the TEXT which contains the AREA information
            (setq hText (ChercheXD enam "OBJAREA" 1000))
            (if (or (= hText nil) (and (/= hText nil) (= (handent hText) nil)))
          (progn
            (setq pt (getpoint "\nSpecify start point of text: "))

      ;     (if (= (getvar "tilemode") 1)
      ;       (setq ht (* (getvar "Dimscale") 2.0))
      ;     )
      ;     (if (= (getvar "tilemode") 0)
      ;       (setq ht (setvar "textsize" (* (* (getvar "ltscale") 4.0) (/ 1 (getvar "ltscale")))))
      ;     )

            (command "_.AREA" "_O" enam)
            (setq ar (getvar "AREA"))
            (setq aim (/ ar 1000000.0))
            (setq aim2 (rtos aim 2 2))
            (setq m2 "m\\u+00b2")
            (setq szText (strcat aim2 m2))
            (Hshset_lay sTAreaLyr sTAreaCol "CONTINUOUS" sTAreaLwt)
            (TextMake pt (rtos (* (getvar "DIMSCALE") 2.0)) 0 1 2 szText nil)     ;MC
            ;;(command "_.TEXT" pt ht 0 szText)
            (setq hText (cdr (assoc 5 (entget (entlast)))))
            ;; Stores the handle of the text to future use
            (MettreXD enam "OBJAREA" hText 1000)
            ;; Creates the reactor
            (setq object (vlax-ename->vla-object enam))

      ;  (if (and (/= (strcase (getvar "CMDNAMES")) "UNDO")
      ;      (/= (strcase (getvar "CMDNAMES")) "U")
      ;      (/= (strcase (getvar "CMDNAMES")) "REDO")
      ;      )
            (setq AreaReactor (vlr-object-reactor
                        (list object)
                        "ObjectArea"
                        '((:vlr-modified . AreaMiseAJour))
                      )
            )
      ;  )

          )
            )
          )
         (princ "\nSelect only CIRCLE, ELLIPSE, POLYLINE, and SPLINE objects...")
        )
        (setq *error* OLDERROR)
        (reset nil)(resset varlst)
      )

      (defun AreaMiseAJour (notifier-object reactor-object parameter-list /)
        ;; Select the entity name
        (setq enam (vlax-vla-object->ename notifier-object))
        ;; Get the handle of the text
        (setq hText (ChercheXD enam "OBJAREA" 1000))
        ;; Updates the text
        (if (and (/= hText nil) (/= (handent hText) nil))
          (progn ;; Gets the Entity name of the text
             (setq txtEnam (handent hText))
             ;; Gets the new area (MUST USE ACTIVEX HERE, NEVER "COMMAND")
             (if (vlax-property-available-p notifier-object 'area)
               (progn (setq ar (vla-get-area notifier-object))
                  (setq aim (/ ar 1000000.0))
                  (setq aim2 (rtos aim 2 2))
                  (setq m2 "m\\u+00b2")
                  (setq szText (strcat aim2 m2))
                  (setq elist (entget txtEnam))
                  (if (= (cdr (assoc 0 elist)) "TEXT")
                    (progn (setq oldtxt (cdr (assoc 1 elist)))
                       (setq pair1 (cons 1 szText))
                       (setq pair2 (cons 1 oldtxt))
                       (setq elist (subst pair1 pair2 elist))
                       (entmod elist)
                    )
                  )
               )
               (progn (alert "It was not possible to get the area"))
             )
          )
        )
        (princ)
      )

      (defun ChercheXD (ename app group / Pos sd_list elist app_list app_sub_list xd_code xd_data Value)
        (setq Value nil)
        (setq elist (entget ename (list app)))
        (setq xd_list (assoc -3 elist))
        (setq xd_list (cdr xd_list))
        (setq app_list (car xd_list))
        (setq app_list (cdr app_list))    ; Strip app name
        (while app_list
          (setq app_sub_list (car app_list))  ; Get sub list
          (setq xd_code (car app_sub_list))   ; Get group code
          (setq xd_data (cdr app_sub_list))   ; Get data
          (cond ((= group xd_code) (setq Value xd_data)))
          (setq app_list (cdr app_list))
        )
        Value
      )

      (defun MettreXD (ename rname Value group / elist xd_list)
        (regapp rname)
        (setq elist (entget ename))
        (setq xd_list (list '(1002 . "}")))
        (setq xd_list (cons (cons group Value) xd_list))
        (setq xd_list (cons '(1002 . "{") xd_list))
        (setq xd_list (cons rname xd_list))
        (setq xd_list (list -3 xd_list))
        (setq elist (cons xd_list elist))
        (entmod elist)
      )

      ;-----------------------
      ;---Reposition Openings.
      ;-----------------------
      ;---Translate An Angle From One Coordinate System To Another
      ;;;angl:  angle to translate (in radians)
      ;;;from:  an integer code similar to (trans)
      ;;;toucs: an integer code similar to (trans)
      ;;;0 World (WCS) / 1 User (current UCS) / 2 If used with code 0 or 1, this indicates the Display
      ;;;Coordinate System (DCS) of the current viewport. When used with code 3, it indicates the DCS
      ;;;of the current model space viewport. / 3 Paper space DCS (used only with code 2)

      (defun AngTrans (angl fromucs toucs / pt1)
        (setq pt1 (polar '(0 0 0) angl 100.0) ;point in current UCS
          pt1 (trans pt1 1 fromucs)   ;translate current to FROM
          pt1 (trans pt1 fromucs toucs)   ;translate FROM to TO
        )
        (angle '(0 0 0) pt1)
      )

      ;---Find mid of 2 given points.
      (defun LocPt (pDeb pFin rRatio /)
        (polar pDeb
           (angle pDeb pFin)
           (/ (distance pDeb pFin) rRatio)
        )
      )

      (defun C:RP () (ReCentre))

      (defun ReCentre ()
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nReposition Openings." copyright))
        (command "_.UCS" "_W")
        (setvar "osmode" 0)
        (setq doorpick (objsel "INSERT"
                   "Select a door or window to reposition"
                   nil
               )
        )
        (if (/= nil (AAXDataGet (car doorpick) sAppNameAA))
          (setq sPorteOuiNon
             (cdr (nth 0 (AAXDataGet (car doorpick) sAppNameAA)))  )          ;Une porte ou fenetre? Vue en plan?
        )
        (if (or (= sPorteOuiNon "DOORPL") (= sPorteOuiNon "WINDPL"))
          (progn
            (setq ODeNom  (car doorpick)
              sDrHan2 (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))            ;Database handle
              OFFDIST (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))))         ;Door width
              pHinge  (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))            ;Hinge
              pMCote  (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))            ;meme cote
              pCoteO  (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))            ;cote oppose
              ;;      rMurThk (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))       ;Wall thk
              sTofIns (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))           ;Inseree a quelle heure?
              ODeNom  (entget ODeNom)
              lDoor   ODeNom
              aA1     (angle pHinge pCoteO)
              aA3     (angle pHinge pMCote)
              aDiff1  (- aA1 aA3)
              pYAxis  (polar pHinge (+ Pi aA1) rMurThk)
              pPicked (cadr doorpick)
              ODrNam  (dxf -1 lDoor)
              pCnr1   (list 0 0 0)
              ODrNam  (dxf -1 lDoor)
              oBlNam  (dxf 2 lDoor)
              xFactor (dxf 41 lDoor)
              yFactor (dxf 42 lDoor)
            )

            (setq pStart (upoint 7 "" "Specify first point" nil nil)
              pEnd   (upoint 7 "" "Specify second point" nil nil)
            )
            (if (and (= (Colineaire pStart pHinge pMCote nil 5) T)
                 (= (Colineaire pEnd pHinge pMCote nil 5) nil)
            )
          (setq pEnd (PerpPt2Line pEnd pMCote pHinge))
            )
            (if (and (= (Colineaire pStart pHinge pMCote nil 5) nil)
                 (= (Colineaire pEnd pHinge pMCote nil 5) T)
            )
          (setq pStart (PerpPt2Line pStart pMCote pHinge))
            )
            (if (and (= (Colineaire pStart pHinge pMCote nil 5) nil)
                 (= (Colineaire pEnd pHinge pMCote nil 5) nil)
            )
          (progn
            (setq pStart (PerpPt2Line pStart pMCote pHinge))
            (setq pEnd (PerpPt2Line pEnd pMCote pHinge))
          )
            )

            (SETQ sMorT
               (ukword
                 1
                 "Mid Third"
                 "Relocate opening [Mid/Third] of way between picked points "
                 "Mid"
               )
            )
            (if (= sMorT "Mid")
          (setq rMorT 2)
          (setq rMorT 3)
            )
            (setq rLong (distance pStart pEnd)
              aAnng (angle pStart pEnd)
              pTo   (LocPt pStart pEnd rMorT)
              pFrm  (LocPt pHinge pMCote rMorT)
            )

            (command "_.UCS" "_3POINT" pHinge pMCote pYAxis)

            (if (minusp yFactor)
          (if (minusp xFactor)
            (setq sOuvreTo "R")
            (setq sOuvreTo "L")
          )
          (if (minusp xFactor)
            (setq sOuvreTo "L")
            (setq sOuvreTo "R")
          )
            )

            (if (= sOuvreTo "L")
          (setq aCross1 135
                aCross2 0
                aCross3 HshPi270
          )
          (setq aCross1 135
                aCross2 0
                aCross3 HshPi270
          )
            )
            (setq pC1  (polar pCnr1 (dtr aCross1) 70)
              pC1a (polar pC1 aCross3 (+ rMurThk 99))
              pC2  (polar pC1 aCross2 (+ Offdist 99))
              pC2a (polar pC2 aCross3 (+ rMurThk 99))
            )

            ;;      (command "_.PLINE" pC1 pC1a pC2a pC2 "_c")

            (command "_.STRETCH"
                 (ssget "_CP" (list pC1 pC1a pC2a pC2))
                 ""
                 (trans pFrm 0 1)
                 (trans pTo 0 1)
            )
            (command "_.UCS" "_P")
            (command "_.UCS" "_P")
          )
          (princ "\nSelect only ArchAssist doors or windows in plan. "
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;-----------------
      ;---Stair Creator.
      ;-----------------
      (defun C:PPS () (PlaceStairs))

      (defun PlaceStairs ()
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nPlace Stairs." copyright))

      ;;;Avec 2 lisses
        (defun EscalB ()
          (setq sCage "EscalB")
          (entmake
            '((0 . "BLOCK") (2 . "EscalB") (70 . 2) (10 0.0 0.0 0.0))
          )
                          ;Bottom ou barreau du depart
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
      ;;         (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 0 (- FlightWidth RailWidth) 0)
                 (list 11 0 RailWidth 0)
                 (list 210 0 0 1)
               )
          )

          (setq iCnter 1)
          (repeat StepNb
            (entmake
          (list
            (cons 0 "LINE")
            (cons 100 "AcDbEntity")
            (cons 67 0)
            (cons 410 "Model")
      ;;    (cons 8 "Stairs")
            (cons 100 "AcDbLine")
            (list 10 (* iCnter rTreadWid) RailWidth 0)
            (list 11 (* iCnter rTreadWid) (- FlightWidth RailWidth) 0)
            (list 210 0 0 1)
          )
            )
            (setq iCnter (1+ iCnter))
          )

                          ;Rail to Left
          (entmake (list
                 (cons 0 "LWPOLYLINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbPolyline")
                 (cons 90 4)
                 (cons 70 1)
                 (cons 43 0.0)
                 (cons 38 0.0)
                 (cons 39 0.0)
                 (list 10 (+ RailWidth FlightLength) FlightWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10
                   (+ RailWidth FlightLength)
                   (- FlightWidth RailWidth)
                 )
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) (- FlightWidth RailWidth))
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) FlightWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 210 0 0 1)
               )
          )

                          ;Rail to Right
          (entmake (list
                 (cons 0 "LWPOLYLINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbPolyline")
                 (cons 90 4)
                 (cons 70 1)
                 (cons 43 0.0)
                 (cons 38 0.0)
                 (cons 39 0.0)
                 (list 10 (+ RailWidth FlightLength) 0)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (+ RailWidth FlightLength) RailWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) RailWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) 0)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 210 0 0 1)
               )
          )
          (entmake '((0 . "ENDBLK")))
        )

      ;;;Avec lisse a gauche
        (defun EscalG ()
          (setq sCage "EscalG")
          (entmake
            '((0 . "BLOCK") (2 . "EscalG") (70 . 2) (10 0.0 0.0 0.0))
          )
                          ;Bottom ou barreau du depart
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 0 (- FlightWidth RailWidth) 0)
                 (list 11 0 0 0)
                 (list 210 0 0 1)
               )
          )

          (setq iCnter 1)
          (repeat StepNb
            (entmake
          (list
            (cons 0 "LINE")
            (cons 100 "AcDbEntity")
            (cons 67 0)
            (cons 410 "Model")
          ;;    (cons 8 "Stairs")
            (cons 100 "AcDbLine")
            (list 10 (* iCnter rTreadWid) 0 0)
            (list 11 (* iCnter rTreadWid) (- FlightWidth RailWidth) 0)
            (list 210 0 0 1)
          )
            )
            (setq iCnter (1+ iCnter))
          )

                          ;Rail to Left
          (entmake (list
                 (cons 0 "LWPOLYLINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbPolyline")
                 (cons 90 4)
                 (cons 70 1)
                 (cons 43 0.0)
                 (cons 38 0.0)
                 (cons 39 0.0)
                 (list 10 (+ RailWidth FlightLength) FlightWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10
                   (+ RailWidth FlightLength)
                   (- FlightWidth RailWidth)
                 )
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) (- FlightWidth RailWidth))
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) FlightWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 210 0 0 1)
               )
          )

                          ;Montant droit
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 FlightLength 0 0)
                 (list 11 0 0 0)
                 (list 210 0 0 1)
               )
          )

          (entmake '((0 . "ENDBLK")))
        )

      ;;;Avec lisse a droite
        (defun EscalD ()
          (setq sCage "EscalD")
          (entmake
            '((0 . "BLOCK") (2 . "EscalD") (70 . 2) (10 0.0 0.0 0.0))
          )
                          ;Bottom ou barreau du depart
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 0 FlightWidth 0)
                 (list 11 0 RailWidth 0)
                 (list 210 0 0 1)
               )
          )

          (setq iCnter 1)
          (repeat StepNb
            (entmake
          (list
            (cons 0 "LINE")
            (cons 100 "AcDbEntity")
            (cons 67 0)
            (cons 410 "Model")
          ;;    (cons 8 "Stairs")
            (cons 100 "AcDbLine")
            (list 10 (* iCnter rTreadWid) RailWidth 0)
            (list 11 (* iCnter rTreadWid) FlightWidth 0)
            (list 210 0 0 1)
          )
            )
            (setq iCnter (1+ iCnter))
          )

                          ;Rail to Right
          (entmake (list
                 (cons 0 "LWPOLYLINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbPolyline")
                 (cons 90 4)
                 (cons 70 1)
                 (cons 43 0.0)
                 (cons 38 0.0)
                 (cons 39 0.0)
                 (list 10 (+ RailWidth FlightLength) 0)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (+ RailWidth FlightLength) RailWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) RailWidth)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 10 (- RailWidth) 0)
                 (cons 40 0.0)
                 (cons 41 0.0)
                 (cons 42 0.0)
                 (list 210 0 0 1)
               )
          )

                          ;Montant gauche
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 FlightLength FlightWidth 0)
                 (list 11 0 FlightWidth 0)
                 (list 210 0 0 1)
               )
          )

          (entmake '((0 . "ENDBLK")))
        )

      ;;;Sans lisse
        (defun Escal ()
          (setq sCage "Escal")
          (entmake
            '((0 . "BLOCK") (2 . "Escal") (70 . 2) (10 0.0 0.0 0.0))
          )
                          ;Bottom ou barreau du depart
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 0 FlightWidth 0)
                 (list 11 0 0 0)
                 (list 210 0 0 1)
               )
          )

          (setq iCnter 1)
          (repeat StepNb
            (entmake
          (list
            (cons 0 "LINE")
            (cons 100 "AcDbEntity")
            (cons 67 0)
            (cons 410 "Model")
          ;;    (cons 8 "Stairs")
            (cons 100 "AcDbLine")
            (list 10 (* iCnter rTreadWid) FlightWidth 0)
            (list 11 (* iCnter rTreadWid) 0 0)
            (list 210 0 0 1)
          )
            )
            (setq iCnter (1+ iCnter))
          )

                          ;Montant gauche
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 FlightLength FlightWidth 0)
                 (list 11 0 FlightWidth 0)
                 (list 210 0 0 1)
               )
          )
                          ;Montant droit
          (entmake (list
                 (cons 0 "LINE")
                 (cons 100 "AcDbEntity")
                 (cons 67 0)
                 (cons 410 "Model")
               ;;   (cons 8 "Stairs")
                 (cons 100 "AcDbLine")
                 (list 10 FlightLength 0 0)
                 (list 11 0 0 0)
                 (list 210 0 0 1)
               )
          )

          (entmake '((0 . "ENDBLK")))
        )

        (defun c:EscalAA_Ok_OnClicked ()
          (Setq FlightLength
             (read
               (Odcl_Control_GetProperty
                 Walls_EscalAA_LengthTextBox
                 "Text"
               )
             )
          )
          (Setq FlightWidth
             (read
               (Odcl_Control_GetProperty
                 Walls_EscalAA_WidthTextBox
                 "Text"
               )
             )
          )
          (Setq StepNb (read
                 (Odcl_Control_GetProperty
                   Walls_EscalAA_StepsTextBox
                   "Text"
                 )
               )
          )
          (Setq RailWidth (read (Odcl_Control_GetProperty
                      Walls_EscalAA_RailWTextBox
                      "Text"
                    )
                  )
          )
          (Setq bLeftR (Odcl_Control_GetProperty
                 Walls_EscalAA_LeftCheckBox
                 "Value"
               )
          )
          (Setq bRightR (Odcl_Control_GetProperty
                  Walls_EscalAA_RightCheckBox
                  "Value"
                )
          )

          (Odcl_Form_close Walls_EscalAA)

          (setq rTreadWid (/ FlightLength StepNb))
          (cond
            ((and (= bLeftR T) (= bRightR T))
             (escalB)
            )
            ((and (= bLeftR T) (= bRightR nil))
             (escalG)
            )
            ((and (= bLeftR nil) (= bRightR T))
             (escalD)
            )
            (T (escal))
          )

          (setq XVal 1  YVal 1  aStaiRot (uangle 1 "" "Specify angle for stairs\n" nil nil))

          (command "_.INSERT"
               sCage
               "_R"
               (rtd aStaiRot)
          )

          (Odcl_Form_Show Walls_StairFlippa)


          (while (wcmatch (getvar "cmdnames") "*INSERT*")

            (defun c:StairFlippa_XMoins1TB_OnClicked ()
              (setq XVal -1)
              (vla-sendcommand *active-document* "_X -1 ")
            )
            (defun c:StairFlippa_X1TB_OnClicked ()
              (setq XVal 1)
              (vla-sendcommand *active-document* "_X 1 ")
            )
            (defun c:StairFlippa_Ymoins1TB_OnClicked ()
              (setq YVal -1)
              (vla-sendcommand *active-document* "_Y -1 ")
            )
            (defun c:StairFlippa_Y1TB_OnClicked ()
              (setq YVal 1)
              (vla-sendcommand *active-document* "_Y 1 ")
            )
            (defun c:StairFlippa_Ok_OnClicked ()
      ;        (vla-sendcommand *active-document* "_X 1 ")
      ;        (vla-sendcommand *active-document* "_Y 1 ")
              (Odcl_Form_close Walls_StairFlippa)
              (princ "\nSpecify insertion point: ")
            )

            (command pause)

          )                   ;while

          (if (Odcl_Form_IsActive ProjName2 "StairFlippa") (Odcl_Form_Close ProjName2 StairFlippa))
          (command "_.EXPLODE" (entlast))
          (command "_.PURGE" "_B" sCage "_N")

        )

        (defun c:EscalAA_OnClose (nUpperLeftX nUpperLeftY /)
          (Odcl_Form_close Walls_EscalAA)
        )

        (Odcl_Form_Show Walls_EscalAA)

        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;------------------
      ;---Block Locator.
      ;------------------
      (defun C:PB () (PlaceBlocs))

      (defun BlocksToListBox (/ BlockInfo)
        (TraceIsOn "BlocksToListBox.")
        (Odcl_ListBox_Clear Walls_BlkLocator_BlockNameList)
        (setq BlockInfo (ai_table "block" 62))
        (if (/= nil (member "AADetails" BlockInfo))
          (setq BlockInfo
             (remove_n (positionIt "AADetails" blockinfo) blockinfo)
          )
        )
        (foreach blok BlockInfo
          (Odcl_ListBox_AddString Walls_BlkLocator_BlockNameList blok)
        )
      )

      (defun LayersToListBox (/ LayInfo)
        (TraceIsOn "LayersToListBox.")
        (Odcl_ListBox_Clear Walls_BlkLocator_LayerComboBox)
        (setq LayInfo (acad_strlsort (ai_table "layer" 4)))
        (foreach calq LayInfo
          (Odcl_ComboBox_AddString
            Walls_BlkLocator_LayerComboBox
            calq
          )
        )
      )

      ;; Find all intersections between objects in the selection set ssObj1.
      (defun TrueIntersGetter (ssObj1 / iSSCombien ;length of ssObj1
                   PTS        ;returning list
                   aObj1      ;Object 1
                   aObj2      ;Object 2
                   iCnt1      ;Loop counter
                   iCnt2      ;Loop counter
                   iPts       ;intersects
      )
        (TraceIsOn "TrueIntersGetter.")
        (setq iCnt1 0             ;index for outer loop
          iSSCombien
           (sslength ssObj1)
        )                 ;Outer loop, first through second to last
        (while (< iCnt1 (1- iSSCombien))  ;Get object 1, convert to VLA object type
          (setq aObj1 (ssname ssObj1 iCnt1)
            aObj1 (vlax-ename->vla-object aObj1)
            iCnt2 (1+ iCnt1)
          )                   ;index for inner loop
                          ;Inner loop, go through remaining objects
          (while (< iCnt2 iSSCombien)     ;Get object 2, convert to VLA object
            (setq aObj2 (ssname ssObj1 iCnt2)
              aObj2 (vlax-ename->vla-object aObj2)
                          ;Find intersections of Objects
              iPts  (vla-intersectwith aObj1 aObj2 0) ;variant result
              iPts  (vlax-variant-value iPts) ;Variant array has values?
            )
            (if (> (vlax-safearray-get-u-bound iPts 1) 0)
          (progn              ;array holds values, convert it
            (setq iPts            ;to a list.
               (vlax-safearray->list iPts)
            )             ;Loop through list constructing points
            (while (> (length iPts) 0)
              (setq Pts  (cons (list (car iPts) (cadr iPts) (caddr iPts)) Pts)
                iPts (cdddr iPts)
              )
            )
          )
            )
            (setq iCnt2 (1+ iCnt2))
          )                   ;inner loop end
          (setq iCnt1 (1+ iCnt1))
        )                 ;outer loop end
        Pts
      )                   ;return list of points found

      (defun PlaceBlocs ()
        (setq olderror *error*
          *error*  Wallserror
          rValX    1
          rValY    1
        )
        (Wallsgather_info)
        (TraceIsOn "PlaceBlocs.")
        (if (null sLeLayr)
          (Setq sLeLayr "0")
        )
        (if (= aRotBl nil)
          (Setq aRotBl 0
            bGlobal nil
          )
        )

        (defun c:BlkLocator_OnInitialize ()
          (if (= bGlobal nil)
            (progn (Odcl_ListBox_SetCurSel Walls_BlkLocator_AreaOptionList 0)
               (Odcl_Control_SetProperty
                 Walls_BlkLocator_SelAreaTextButton
                 "Enabled"
                 nil
               )
            )
            (progn (Odcl_ListBox_SetCurSel Walls_BlkLocator_AreaOptionList 1)
               (Odcl_Control_SetProperty
                 Walls_BlkLocator_SelAreaTextButton
                 "Enabled"
                 T
               )
            )
          )
          (LayersToListBox)
          (BlocksToListBox)
          (Odcl_ComboBox_SelectString
            Walls_BlkLocator_LayerComboBox
            sLeLayr
          )
          (if (= aRotBl 0)
            (progn
          (Odcl_Control_SetProperty
            Walls_BlkLocator_Option0
            "Value"
            T
          )
          (Odcl_Control_SetProperty
            Walls_BlkLocator_Option90
            "Value"
            nil
          )
          (Odcl_Control_SetProperty
            Walls_BlkLocator_Option180
            "Value"
            nil
          )
          (Odcl_Control_SetProperty
            Walls_BlkLocator_Option270
            "Value"
            nil
          )
            )
          )

          (Odcl_Control_SetProperty
            Walls_BlkLocator_LineCheckBox
            "Value"
            T
          )

        )

        (defun c:BlkLocator_AreaOptionList_OnSelChanged
           (nSelection sSelText /)
          ;;    (Odcl_MessageBox "To Do: code must be added to AreaOptionList" (etos nSelection))
          (if (= nSelection 0)
            (progn (setq bGlobal T)
               (Odcl_Control_SetProperty
                 Walls_BlkLocator_SelAreaTextButton
                 "Enabled"
                 nil
               )
            )
            (progn (setq bGlobal nil)
               (Odcl_Control_SetProperty
                 Walls_BlkLocator_SelAreaTextButton
                 "Enabled"
                 T
               )
            )
          )
        )

        (defun c:BlkLocator_SelAreaTextButton_OnClicked ()
          (setq bDispForm nil)
          (Setq bWin (Odcl_ListBox_GetCurSel Walls_BlkLocator_AreaOptionList))
          (Setq sLeLayr (Odcl_Control_GetProperty
                  Walls_BlkLocator_LayerComboBox
                  "Text"
                )
          )
          (Odcl_Form_Close Walls_BlkLocator)
          (setq pC1 (upoint 1 "" "Specify first corner" nil nil)
            pC2 (GetCorner pC1 "\nSpecify opposite corner:")
          )
          (if (= bWin 0)
            (setq bGlobal nil)
            (setq bGlobal T)
          )
          (setq bDispForm T)
        )

        (defun c:BlkLocator_BlockNameList_OnSelChanged
           (nSelection sSelText /)
          ;;    (Odcl_MessageBox "To Do: code must be added to BlockNameList" "To do")
          (setq sNomDuBloc sSelText)
          (Odcl_Control_SetProperty
            Walls_BlkLocator_BlockView
            "BlockName"
            sNomDuBloc
          )
        )

        (defun c:BlkLocator_Option0_OnClicked (nValue /) ;1 or 0
          ;;     (Odcl_MessageBox "To Do: code must be added to Option0" (etos nValue))
          (if (= 1 nValue)
            (setq aRotBl 0)
          )
        )

        (defun c:BlkLocator_Option90_OnClicked (nValue /)
          (if (= 1 nValue)
            (setq aRotBl 90)
          )
        )

        (defun c:BlkLocator_Option180_OnClicked (nValue /)
          (if (= 1 nValue)
            (setq aRotBl 180)
          )
        )

        (defun c:BlkLocator_Option270_OnClicked (nValue /)
          (if (= 1 nValue)
            (setq aRotBl 270)
          )
        )

        (defun c:BlkLocator_CancelTextButton_OnClicked ()
          (setq bDispForm nil)
          (Setq sLeLayr (Odcl_Control_GetProperty
                  Walls_BlkLocator_LayerComboBox
                  "Text"
                )
          )
          (Odcl_Form_Close Walls_BlkLocator)
        )

        (defun c:BlkLocator_Ok_OnClicked ()
          (setq bDispForm nil)        ;same for cancel
          (Setq sLeLayr (Odcl_Control_GetProperty
                  Walls_BlkLocator_LayerComboBox
                  "Text"
                )
          )
          (Setq bWin (Odcl_ListBox_GetCurSel Walls_BlkLocator_AreaOptionList))

          (Setq bArc (Odcl_Control_GetProperty
               Walls_BlkLocator_ArcCheckBox
               "Value"
                 )
          )
          (Setq bCircle (Odcl_Control_GetProperty
                  Walls_BlkLocator_CircleCheckBox
                  "Value"
                )
          )                   ;nil or T
          (Setq bLine (Odcl_Control_GetProperty
                Walls_BlkLocator_LineCheckBox
                "Value"
              )
          )                   ;nil or T
          (Setq bPline (Odcl_Control_GetProperty
                 Walls_BlkLocator_PlineCheckBox
                 "Value"
               )
          )                   ;nil or T
          (Setq bSpline (Odcl_Control_GetProperty
                  Walls_BlkLocator_SplineCheckBox
                  "Value"
                )
          )
          (Setq bZero
             (Odcl_Control_GetProperty Walls_BlkLocator_Option0 "Value")
          )                   ;1 or 0

          (Odcl_Form_Close Walls_BlkLocator)
          (if (= sLeLayr "")
            (Setq sLeLayr "0")
          )

          (setq ssObj2 (GETssObj)
            PTS    (TrueIntersGetter ssObj2)
          )
          (foreach PT PTS         ;Loop through list of points
            (command "_.INSERT"
                 sNomDuBloc
                 (trans Pt 0 1)
                 rValX
                 rValY
                 aRotBl
            )
          )
        )

        (if (= (Odcl_LoadProject ProjName2 nil) -10)
          (progn
            (WsFlashMess1
          "Echec chargement bote de dialogue. Fichier Walls.ods introuvable.\nSi la location a change, la mise  jour de MetricArchAssist2004.ini est indispensable."
          "Archivio Walls.ods non trovato. Se il percorso modificasse da ultima installazione,\nl'aggiornamento di MetricArchAssist2004.ini  indispensabile."
          "Archivo Walls.ods no ha sido encontrado. Si la ubicacion es modificada\ndesde la ultima instalacion, archivo MetricArchAssist2004.ini tendra que ser actualizado con su nueva ubicacion."
          "File Walls.ods not found. If Path modified since last\ninstallation, update MetricArchAssist2004.ini file."
          1 5)
            (exit)
          )
          (setq bDispForm T)
        )

        (while bDispForm
          (setq bDispForm nil)
          (Odcl_Form_Show Walls_BlkLocator)
        )

        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun GETssObj ()
        (TraceIsOn "GETssObj.")
        (setq lObjeto nil)
        (if bArc
          (setq sArc    "ARC"
            lObjeto (append lObjeto (list sArc))
          )
        )
        (if bCircle
          (setq sCircle "CIRCLE"
            lObjeto (append lObjeto (list sCircle))
          )
        )
        (if bLine
          (setq sLine   "LINE"
            lObjeto (append lObjeto (list sLine))
          )
        )
        (if bPline
          (setq sPline  "POLYLINE,LWPOLYLINE"
            lObjeto (append lObjeto (list sPline))
          )
        )
        (if bSpline
          (setq sSpline "SPLINE"
            lObjeto (append lObjeto (list sSpline))
          )
        )

        (setq sFinalSS (strdcat lObjeto ","))

        (if (= bWin 0)
          (progn
            (ssget "_X" (list (cons 8 sLeLayr) (cons 0 sFinalSS)))
          )
          (progn
            (ssget "_W"
               pC1
               pC2
               (list (cons 8 sLeLayr) (cons 0 sFinalSS))
            )
          )
        )
      )

      ;-------------------------------
      ;---AA Select a Wall by Fence.
      ;-------------------------------
      (defun C:SWF (/)
        (setq olderror   *error*
          *error*    Wallserror
          i      0
          ss2    nil
          iPts       nil
          lDesPuntos nil
          Pts    nil
        )
        (Wallsgather_info)

        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (TraceIsOn "SWF.")
        (princ (strcat "\nSelect a Wall by Fence." copyright))

        (princ "\nFence objects to clone...")
        (setq pDe (upoint 1 "" "Specify first point" nil nil)
          pA  (upoint 1 "" "Specify next point" nil pDe)
          ss2 (ssget
                "_F"
                (list pDe pA)
                (list
              (cons 0
                    "LINE,LWPOLYLINE,POLYLINE,ARC,CIRCLE,ELLIPSE,SPLINE"
              )
                )
              )
        )

        (LineMake pDe pA "0")

        (setq aObj1 (ssname (ssget "_L") 0)
          aObj1 (vlax-ename->vla-object aObj1)
        )

        (if (/= ss2 nil)
          (progn
            (setq istop (sslength ss2))
            (while (< i istop)

          ;;  (redraw (ssname ss2 i) 3)

          (setq aObj2 (ssname ss2 i)
                aObj2 (vlax-ename->vla-object aObj2)              ;Find intersections of Objects
                iPts  (vla-intersectwith aObj1 aObj2 0)               ;variant result
                iPts  (vlax-variant-value iPts)                   ;Variant array has values?
          )

          (if (> (vlax-safearray-get-u-bound iPts 1) 0)
            (progn                                ;array holds values, convert it
              (setq iPts                              ;to a list.
                 (vlax-safearray->list iPts)
              )                                   ;Loop through list constructing points
              (while (> (length iPts) 0)
                (setq Pts  (cons (list (car iPts) (cadr iPts) (caddr iPts))
                         Pts
                   )
                  iPts (cdddr iPts)
                )
              )
            )
          )
          (setq i (1+ i))
            )
            (entdel (entlast))
            (setq Pts (reverse Pts))                          ;Tous les points
          )
        )

        (setq sLayr  nil
          sLayr1 nil
          iLwt   nil
          iCol   nil
          lLays  nil
          iCnt   0
          iCnt4  0
          istop  (length Pts)
        )
        (while (< iCnt istop)                             ;Tant compteur < nombre de pts
          (setq ssFinals
             (SSWindow
               (nth iCnt Pts)
               5
               "_F"
               (list
                 (cons
               0
               "LINE,LWPOLYLINE,POLYLINE,ARC,CIRCLE,ELLIPSE,SPLINE"
                 )
               )
             )
          )
          (if (< (sslength ssFinals) 2)
            (progn
          ;;       (princ " Un")
          (setq oQuel (ssname (ssget (nth iCnt Pts)) 0)
                sLayr (vla-get-layer (vlax-ename->vla-object oQuel))      ;Nom du calque pour l'objet
                iLwt  (vla-get-lineweight (vlax-ename->vla-object oQuel))
                iCol  (vla-get-color (vlax-ename->vla-object oQuel))
          )
            )
            (progn
          ;;       (princ " +d'Un")
          (while (< iCnt4 (sslength ssFinals))
            (setq oQuel1 (entget (ssname ssfinals iCnt4))
              sLayr1 (vla-get-layer (vlax-ename->vla-object oQuel1))      ;Nom du calque pour l'objet
              iLwt1  (vla-get-lineweight (vlax-ename->vla-object oQuel1))
              iCol1  (vla-get-color (vlax-ename->vla-object oQuel1))
              lLays  (append lLays (list sLayr1))
            )

            (Cassure "lLays")
            (setq iCnt4 (1+ iCnt4))
          )
            )
          )

          (cond ((= iLwt -1) (setq sLwt "ByLayer"))
            ((= iLwt -2) (setq sLwt "ByBlock"))
            ((= iLwt -3) (setq sLwt "Default"))
            (T (setq sLwt (rtos (* 0.01 iLwt) 2 2)))
          )

                          ;    (setq what_col (dxf 62 oQuel1))
                          ;      (if (= what_col nil)
                          ;           (progn
                          ;             (setq layinfo  (tblsearch "LAYER" sLayr)
                          ;                   what_col (abs (dxf 62 layinfo)))
                          ;           )
                          ;      )
                          ;    (princ what_col)

          (if (= iCnt 0)
            (progn
          (setq rDP2p  0.0
                lDesPuntos (append lDesPuntos (list rDP2p sLayr iCol sLwt))
                lDesPunt   lDesPuntos
          )
            )
            (progn
          (Cassure "iCnt")
          (if (< iCnt istop)
            (setq rDP2p      (distance (nth (1- iCnt) Pts) (nth iCnt Pts))    ;Dist entre les points
              lDesPuntos (append lDesPuntos (list rDP2p sLayr iCol sLwt))
              lDesPunt2  lDesPuntos
            )
          )
            )
          )
          (setq iCnt (1+ iCnt))
        )

        (setq pDe2    (upoint 1 "" "Specify first point" nil nil)
          pA2 (upoint 1 "" "Specify next point" nil pDe2)
          rAngg   (angle pDe2 pA2)
      ;;;  lDesPuntos (reverse lDesPuntos)
      ;;;  lDesPuntos (remove_n 0 lDesPuntos)
          iStop   (length lDesPuntos)
          iCnt2   0
          rOCumul 0
        )

        (Cassure "lDesPuntos iStop")

      ;;; Stud Wall 75->NSW;DoubleW;110;50;75;BrickWork1;Red;0.35;BrickWork2;White;0.25|Timber2;200;0.35;Timber2;200;0.25
      ;;; (0.0 "Layer1" 256 "ByLayer" 55.0 "Layer2" 256 "ByLayer" 123.0 "Layer3" 256 "ByLayer" 299.0 "Layer4" 256 "ByLayer")

        (while (< iCnt2 iStop)
          (setq rOffs   (nth iCnt2 lDesPuntos)
            sCalq   (nth (1+ iCnt2) lDesPuntos)
            rOCumul (+ rOffs rOCumul)
            pNewS   (polar pDe2 (- rAngg (dtr 90)) rOCumul)
            pNewE   (polar pA2 (- rAngg (dtr 90)) rOCumul)
          )

          (LineMake pNewS pNewE sCalq)

          (setq iCnt2 (+ iCnt2 4))
        )

        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;;;(setq oname (car sel_obj)
      ;;;      edata (entget oname)
      ;;;      what_is  (dxf 0 edata)
      ;;;      what_ltp (dxf 6 edata)
      ;;;      what_lay (dxf 8 edata)
      ;;;      what_col (dxf 62 edata) )

      ;to add: (vla-add *layers-Object* "NewLayerName")
      ;to delete: (vla-delete (vla-item *layers-Object* "NewLayerName"))

      ;----------------------------
      ;---AA Match Opening Angle.
      ;----------------------------
      (defun GetSwing (/ eObListe eObjName)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
        )
        (WsFlashMess2
          (strcat "\nMatch Door Angle." copyright)
          (strcat "\nMatch Door Angle." copyright)
          (strcat "\nMatch Door Angle." copyright)
          (strcat "\nMatch Door Angle." copyright)
        )
        (GetDrINI)
        (setq eObjName (objsel "INSERT" "Select source object" nil))
        (if (/= nil (AAXDataGet (car eObjName) sAppNameAA))
          (setq sPorteOuiNon
             (cdr (nth 0 (AAXDataGet (car eObjName) sAppNameAA)))    )                    ;Une porte? Vue en plan?
        )

        (if (= sPorteOuiNon "DOORPL")
          (progn
            (setq ODeNom     (car eObjName)
              rJambWidth (read (cdr (nth 1 (AAXDataGet ODeNom sAppNameAA))))              ;Jamb Width
              sDrHan2    (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))                 ;Database handle
              LDR        (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))                 ;Door type
              jambprefix (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA)))                 ;Jamb type
              OFFDIST    (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))))              ;Door width
              p1         (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))                 ;Hinge
              pMCote     (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))                 ;meme cote
              pCoteO     (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))                 ;cote oppose
              PercOpen   (cdr (nth 9 (AAXDataGet ODeNom sAppNameAA)))                 ;opening angle
              rMurThk    (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))                    ;Wall thk
              bHeader    (read (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA))))             ;Header present
              bThreshold (read (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA))))             ;Threshold/Sill present?
              aA3        (angle p1 pMCote)
              ODeNom     (entget ODeNom)
              lDoor      ODeNom
              ODrNam     (dxf -1 lDoor)
              pCnr1      (list 0 0 0)
              oBlNam     (dxf 2 lDoor)
              pHinge     (dxf 10 lDoor)
              xFactor    (dxf 41 lDoor)
              yFactor    (dxf 42 lDoor)
              aRot       (dxf 50 lDoor)
              sTr2       LDR
              sTr3a      (rtos Offdist)
              sDrHand    sDrHan2
              p2         pMCote
              p3         pCoteO
              sJambTyp   jambprefix
              sJx        jambprefix
              iCnt       0
            )

            ;; :L Last visible object added to the database.
            ;; :E Everything within the cursor's object selection pickbox.
            ;; :S Allow single selection only.
            (princ "\nSelect destination door(s): ")
            (setq bCont (ssget "+.:E" (list (cons 0 "INSERT")))
              ssDRM bCont
              nIdx  (sslength ssDRM)
            )

            (while (< iCnt nIdx)
          (if
            (= (cdr (nth 0 (AAXDataGet (ssname ssDRM iCnt) sAppNameAA)))
               "DOORPL"     )                                     ;Une porte? Vue en plan?
             (progn
               (setq ODeNom     (ssname ssDRM iCnt)
                 rJambWidth (read
                      (cdr (nth 1 (AAXDataGet ODeNom sAppNameAA)))              )     ;Jamb Width
                 sDrHan2    (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))              ;Database handle
                 LDR        (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))              ;Door type
                 jambprefix (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA)))              ;Jamb type
                 OFFDIST    (read (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA)))          )     ;Door width
                 p1         (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))              ;Hinge
                 pMCote     (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))              ;meme cote
                 pCoteO     (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))              ;cote oppose
                 ;;             PercOpen   (cdr (nth 9 (AAXDataGet ODeNom sAppNameAA)))       ;opening angle
                 rMurThk    (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))             ;Wall thk
                 bHeader    (read (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA)))         )     ;Header present
                 bThreshold (read
                      (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA)))             )     ;Threshold/Sill present?

                 aA3        (angle p1 pMCote)
                 ODeNom     (entget ODeNom)
                 lDoor      ODeNom
                 ODrNam     (dxf -1 lDoor)
                 pCnr1      (list 0 0 0)
                 oBlNam     (dxf 2 lDoor)
                 pHinge     (dxf 10 lDoor)
                 xFactor    (dxf 41 lDoor)
                 yFactor    (dxf 42 lDoor)
                 aRot       (dxf 50 lDoor)
                 sTr2       LDR
                 sTr3a      (rtos Offdist)
                 sDrHand    sDrHan2
                 p2         pMCote
                 p3         pCoteO
                 bEfface    nil
                 sJambTyp   jambprefix
                 sOpenAng   PercOpen
               )

               (entdel (ssname ssDRM iCnt))

               (cond ((= LDR "S")     (SingleMk))
                 ((= LDR "D")     (DoubleMk))
                 ((= LDR "A")     (AccessMk))
                 ((= LDR "UD")    (UnDblMk))
                 ((= LDR "BD")    (BifoldDMk))
                 ((= LDR "BS")    (BifoldSMk))
      ;;             ((= LDR "W")     (OpenMk))
      ;;             ((= LDR "SFd")   (SldFixMkD))            ;AABuild108
      ;;             ((= LDR "FSFd")  (FixSldFixMk))          ;AA1.3b100
      ;;             ((= LDR "FSSFd") (FixSldSldFixMk))           ;AA1.3b100
                 ((= LDR "SSS")   (SurfSlidSMk))
                 ((= LDR "SSD")   (SurfSlidDMk))          ;AA1.3b100
                 ((= LDR "CSD")   (CavSlidDMk))           ;AA1.3b100
                     ((= LDR "CSS")   (CavSlidSMk))
               )
               (Hshset_lay (dxf 8 lDoor) nil "CONTINUOUS" nil)
               (entmake (list (cons 0 "INSERT")
                      (cons 2 BlockNameD)
                      (cons 10 pHinge)
                      (cons 41 xFactor)
                      (cons 42 YFactor)
                      (cons 50 aRot)
                    )
               )

               (SetDoorInfoTag sAppNameAA (entlast) nil)

               (setq iCnt (1+ iCnt))
             )
             (progn
               (setq iCnt (1+ iCnt))
               (princ "\nCannot modify this object. ")
             )
          )
            )
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;------------------------
      ;---AA GRADE and RATIO.
      ;------------------------
      (defun DoAgain ()
        (setq elist (objsel "LINE" "Select line object to list" nil))
        (setq nelist (entget (car elist))
          pt1    (dxf 10 nelist)
          pt2    (dxf 11 nelist)
          tst    (rtd (angle pt1 pt2))
        )
        (if (or (= tst 0)
            (= tst 90)
            (= tst 180)
            (= tst 270)
            )
          (setq tst 0.001)
        )
        (setq len  (distance pt1 pt2)
          opp  (abs (* len (cal (strcat "sin(" (rtos tst) ")"))))
          adj  (abs (* len (cal (strcat "cos(" (rtos tst) ")"))))
          vert (rtos (* opp (/ 100 adj)) 2 3)
          ris1 (rtos (* opp (/ 1 adj)) 2 3)
          run2 (rtos (* adj (/ 1 opp)) 2 3)
        )
        (if (= tst 0.001)
          (setq vert "Nil"
            ris1 "Nil"
            run2 "Nil"
          )
        )
        (setq what_haps 3)
      )

      (defun C:DGR (/ adj vert ris1 run2 opp pt1 ang pt2 f len tst)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (TraceIsOn "C:AAGRA.")
        (setvar "ORTHOMODE" 0)
        (command "_.UCS" "_W")

        (princ (strcat "\nDraw by GRade or RAtio." copyright))

        (if (= grrad nil)
          (setq grrad 5.0)
        )
        (if (= rRun nil)
          (setq rRun 10.0)
        )
        (if (= rRise nil)
          (setq rRise 1.0)
        )

        (setq choix (ukword 1
                    "Grade Ratio"
                    "Draw line using a [Grade/Ratio] "
                    "Grade"
                )
        )
        (if (/= choix "List")
          (progn
            (setq pt1 (upoint 1 "" "Specify start point" nil nil)
              pt2 (upoint 1 "" "Indicate general direction" nil pt1)
            )
            (if (= choix "Grade")
          (progn
            (setq grrad (ureal 1 "" "Enter percentage for grade" grrad)
              len   (distance pt1 pt2)
              ang   (atan (/ grrad 100.0))
              tst   (angle pt1 pt2)
            )
            (if (< tst (dtr 360))
              (setq f (- (dtr 360) ang))
            )
            (if (< tst (dtr 270))
              (setq f (+ (dtr 180) ang))
            )
            (if (< tst (dtr 180))
              (setq f (- (dtr 180) ang))
            )
            (if (< tst (dtr 90))
              (setq f ang)
            )
            (command "_.line" pt1 (polar pt1 f len) "")
          )
          (progn
            (setq rRun  (ureal 1 "" "Enter Run" rRun)
              rRise (ureal 1 "" "Enter Rise" rRise)
              len   (distance pt1 pt2)
              ang   (atan (/ rRise rRun))
              tst   (angle pt1 pt2)
            )
            (if (< tst (dtr 360))
              (setq f (- (dtr 360) ang))
            )
            (if (< tst (dtr 270))
              (setq f (+ (dtr 180) ang))
            )
            (if (< tst (dtr 180))
              (setq f (- (dtr 180) ang))
            )
            (if (< tst (dtr 90))
              (setq f ang)
            )
            (command "_.line" pt1 (polar pt1 f len) "")
          )
            )
          )
        )
        (if (= choix "List")
          (DoAgain)
        )
        (if (= choix "List")
          (progn
            (Odcl_LoadProject "XQuery" T)
            (Odcl_Form_Show XQuery_GraList)
            (cond
          ((= (getvar "AUNITS") 0)
           (setq tell_ang_unit "Decimal Degrees")
          )
          ((= (getvar "AUNITS") 1) (setq tell_ang_unit "Deg/Min/Sec"))
          ((= (getvar "AUNITS") 2) (setq tell_ang_unit "Gradians"))
          ((= (getvar "AUNITS") 3) (setq tell_ang_unit "Radians"))
          ((= (getvar "AUNITS") 4) (setq tell_ang_unit "Surveyor"))
            )
            (cond
          ((= (getvar "LUNITS") 1) (setq tell_lin_unit "Scientific"))
          ((= (getvar "LUNITS") 2) (setq tell_lin_unit "Decimal"))
          ((= (getvar "LUNITS") 3) (setq tell_lin_unit "Engineering"))
          ((= (getvar "LUNITS") 4) (setq tell_lin_unit "Architural"))
          ((= (getvar "LUNITS") 5) (setq tell_lin_unit "Fractional"))
            )
            (if (= (getvar "ANGDIR") 1)
          (setq tell_ang "CW")
          (setq tell_ang "CCW")
            )
            (cond
          ((= (getvar "ANGBASE") (dtr 0)) (setq tell_base "East"))
          ((= (getvar "ANGBASE") (dtr 90)) (setq tell_base "North"))
          ((= (getvar "ANGBASE") (dtr 180)) (setq tell_base "West"))
          ((= (getvar "ANGBASE") (dtr 270)) (setq tell_base "South"))
            )

                          ; add an item that displays simple text.
            (Setq nRow (Odcl_Grid_AddString
                 XQuery_GraList_Grid
                 (strcat "Percentage\t" vert " %")
                 "\t"
               )
            )
                          ; set the new grid cell style.
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 1 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Horizontal to vertical ratio\t1:" ris1)
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 2 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Vertical to horizontal ratio\t1:" run2)
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 3 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Layer\t" (dxf 8 nelist))
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 4 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Color\t"
              (if (dxf 62 nelist)
                (itoa (dxf 62 nelist))
                "BYLAYER"
              )
          )
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 5 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "From point\t" (gc_pts pt1))
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 6 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "To point\t" (gc_pts pt2))
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 7 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Angle in XY plane\t" (angtos (angle pt1 pt2)))
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 8 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "3D length\t" (rtos (distance pt1 pt2)))
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 9 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Angle base\t" tell_base)
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 10 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Angle direction\t" tell_ang)
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 11 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Angular units\t" tell_ang_unit)
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 12 1 6)

            (Odcl_Grid_AddString
          XQuery_GraList_Grid
          (strcat "Linear units\t" tell_lin_unit)
          "\t"
            )
            (Odcl_Grid_SetItemStyle XQuery_GraList_Grid 13 1 6)
          )
        )

        (command "_.UCS" "_P")
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;-------------------
      ;---AA Add To Block.
      ;-------------------
      ;;; Utility routine to convert a selection set to an ActiveX array
      (defun selectionset->array (ss / c r)
        (setq c -1)
        (repeat (sslength ss)
          (setq r (cons (ssname ss (setq c (1+ c))) r))
        )
        (setq r (reverse r))
        (vlax-safearray-fill
          (vlax-make-safearray
            vlax-vbObject
            (cons 0 (1- (length r)))
          )
          (mapcar 'vlax-ename->vla-object r)
        )
      )

      ;;; Adds the specified item to a given block definition
      ;;; Arguments: the entity name of a block reference
      ;;;            a selection set containing the objects to add
      ;;; Returns: nil
      ;;; The drawing must be regenerated for the change to become visible
      (defun AXAddToBlock (blk ss / doc blkref blkdef inspt refpt)
        (setq doc (vla-get-ActiveDocument (vlax-get-acad-object))
          blkref  (vlax-ename->vla-object blk)
          blkdef  (vla-Item (vla-get-Blocks doc) (vla-get-Name blkref))
          inspt   (vlax-variant-value (vla-get-InsertionPoint blkref))
          ssarray (selectionset->array ss)
          refpt   (vlax-3d-point '(0 0 0))
        )
        (foreach ent (vlax-safearray->list ssarray)
          (vla-Move ent inspt refpt)
        )
        (vla-CopyObjects doc ssarray blkdef)
        (foreach ent (vlax-safearray->list ssarray)
          (vla-Delete ent)
        )
        (princ)
      )

      (defun nAjout (/)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nAdd Objects to Block." copyright))
        (setq ent1  (objsel "INSERT" "Select a block" T)
          rXScl (dxf 41 (entget ent1))
          rYScl (dxf 42 (entget ent1))
          rZScl (dxf 43 (entget ent1))
          rAngl (dxf 50 (entget ent1))
        )
        (if (or (/= rAngl 0) (/= rXScl 1) (/= rYScl 1) (/= rZScl 1))
          (progn
            (cond
          ((= langue "Franais")
           (setq bOK
              (dos_msgbox
                "Objects will be added to master block in database.\nUnpredictable results occur happen when selected block has been scaled, distorted, rotated, mirrored... \nContinue?"
                "Message ArchAssist"
                4
                1
              )
           )
          )
          ((= langue "Italiano")
           (setq bOK
              (dos_msgbox
                "Objects will be added to master block in database.\nUnpredictable results occur happen when selected block has been scaled, distorted, rotated, mirrored... \nContinue?"
                "ArchAssist  Message"
                4
                1
              )
           )
          )
          ((= langue "Espanol")
           (setq bOK
              (dos_msgbox
                "Objects will be added to master block in database.\nUnpredictable results will occur when selected block has bbeen scaled, distorted, rotated, mirrored... \nContinue?"
                "Mensaje ArchAssist"
                4
                1
              )
           )
          )
          (T
           (setq bOK
              (dos_msgbox
                "Objects will be added to master block in database.\nUnpredictable results will occur when selected block has been scaled, distorted, rotated, mirrored... \nContinue?"
                "ArchAssist  Message"
                4
                1
              )
           )
          )
            )
          )
         (setq bOK 6)
        )
        (if (= bOK 6)
          (progn
            (princ "\nSelect objects to add to block:")
            (AXAddToBlock ent1 (ssget))
          )
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;----------
      ;---Legend.
      ;----------
      (defun C:LG  () (Legend))
      (defun C:LLG () (Legend))
      (defun C:TAG () (attatt))

      ;;; a list of all block names not just inserted ones!
      ;;; return example ("*D5" "A$C263E5435" "b2" "b1")
      (defun AXBlocksGet (/ b bn tl)
        (TraceIsOn "AXBlocksGet.")
        (vlax-for b (vla-get-blocks
              (vla-get-ActiveDocument (vlax-get-acad-object))
                )
          (if (= (vla-get-islayout b) :vlax-false)
            (setq tl (cons (vla-get-name b) tl))
          )
        )
        (reverse tl)
      )

      ;;; Returns a list with references to a given block
      ;;; (IsBlockIns <block name>)
      ;;; example: (IsBlockIns "b1")
      ;;; return: (<Entity name: 2ea6290> <Entity name: 2ea6288>)
      ;;; tip: if return is nil it's not inserted
      (defun IsBlockIns (bn / lst ed)
        (TraceIsOn "IsBlockIns.")
        (if (setq ed (tblobjname "block" bn))
          (setq
            lst (entget
              (cdr (assoc 330 (entget ed)))
            )
          )
        )
        (apply
          'append
          (mapcar '(lambda (x)
                 (list (cdr x))
               )
              (cdr (reverse (cdr (member (assoc 102 lst) lst))))
          )
        )
      )

      ;; ! Function : Find the attribute in a block given the tag value
      ;; ! Argument : 'bn' - entity name of the block
      ;; !            'sTagname'   - tag value to search for
      ;; ! Returns  : The attribute entity name, if it exists otherwise nil is returned
      ;; (AXGetAttribute *sDoc "11" "AALEGEND")  ;Celui la m'arrange!
      (defun AXGetAttribute (doc bn sTagname / layout i atts sTag str)
        (TraceIsOn "AXGetAttribute.")
        (vlax-for layout (vla-get-layouts doc)
          (vlax-for i (vla-get-block layout)                  ;Collection des blocs appartenant au layout
            (if (and
              (or (= (vla-get-objectname i) "AcDbBlockReference")
                  (= (vla-get-objectname i) "AcDbMInsertBlock"))
              (= (strcase (vla-get-name i)) (strcase bn))
            )
          (if (and
                (= (vla-get-hasattributes i) :vlax-true)
                (safearray-value
              (setq atts
                     (vlax-variant-value
                   (vla-getattributes i)
                     )
              )
                )
              )
            (foreach sTag (vlax-safearray->list atts)
              (if (= (strcase sTagname) (strcase (vla-get-tagstring sTag)))
                (setq str (vla-get-TextString sTag))
              )
            )
          )
            )
          )
        )
        str
      )

      ;;; Arguments: The object name of an attributed block
      ;;; Example: (axGetAttributes (car (entsel)))
      ;;; Returns a list of attribute tags and associated values.
      (defun AXGetAttributes (ent / blkref lst)
        (if (= (vla-Get-ObjectName
             (setq blkref (vlax-Ename->vla-Object ent))
           )
           "AcDbBlockReference"
            )
          (if (vla-Get-HasAttributes blkref)
            (mapcar
          '(lambda (x)
             (setq
               lst (cons
                 (cons (vla-Get-TagString x) (vla-Get-TextString x))
                 lst
               )
             )
           )
          (vlax-safearray->list
            (vlax-variant-value (vla-GetAttributes blkref))
          )
            )                 ; mapcar
          )
        )
        (reverse lst)
      )

      ;;; (AXReplaceAttrVal *sDoc "11" "AALEGEND" "NewVal") change valeur
      (defun AXReplaceAttrVal (doc bn tagname textstring / layout i atts tag)
        (vlax-for layout (vla-get-layouts doc)
          (vlax-for i (vla-get-block layout)
            (if (and
              (= (vla-get-objectname i) "AcDbBlockReference")
              (= (strcase (vla-get-name i)) (strcase bn))
            )
          (if (and
                (= (vla-get-hasattributes i) :vlax-true)
                (safearray-value
              (setq atts
                     (vlax-variant-value
                   (vla-getattributes i)
                     )
              )
                )
              )
            (foreach tag (vlax-safearray->list atts)
              (if (= (strcase tagname) (strcase (vla-get-tagstring tag)))
                (vla-put-TextString tag textstring)
              )
            )
            (vla-update i)
          )
            )
          )
        )
      )

      ;;;---------------
      (defun LEGEND (/)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setvar "ORTHOMODE" 0)
        (TraceIsOn "LEGEND.")
        (princ (strcat "\nLegend Harvester." copyright))

        (setq *sDoc (vla-get-activedocument (vlax-get-acad-object)))
        (setq lBonBlocs nil
          lDesBlocs nil
          lDesBlocs (AXBlocksGet)
        )
        (foreach item lDesBlocs
          (if (or (wcmatch item "*U*")            ;Pas d'anonymes
              (wcmatch item "*D*")            ;Ni de cotations
              (= nil (IsBlockIns item))
          )                       ;Seulement les blocs inseres dans le dessin
            (princ)
            (setq lBonBlocs (append lBonBlocs (list item)))
          )
        )

        (setq lLegBlocs nil  lDescs nil)
        (foreach item lBonBlocs
          (if (AXGetAttribute *sDoc item "AALEGEND")      ;Ceux qui ont une legende
            (setq lLegBlocs
               (append
                 lLegBlocs
                 (list (strcat item ": " (AXGetAttribute *sDoc item "AALEGEND"))
                 )
               )
            )                         ;Nom du bloc et champ de l'attrib.
          )
        )

        (setq lLegBlocs (dos_editlist "Legend Editor" "Add, Remove or Move Up/Down descriptions. " lLegBlocs)  lBlocsFinaux nil)

        (foreach item lLegBlocs
            (setq lBlocsFinaux
               (append
                 lBlocsFinaux
                 (list (strfind item ": "))
                   )
            )
        )

        (LegendOut lBlocsFinaux)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun LegendOut (lListe / pIns1)
        (TraceIsOn "LegendOut.")
        (PointMake (list 0 0 0) "0")
        (setq oMarque (entlast)
          pIns1   '(0 0 0)
        )
        (setvar "ATTDIA" 0)
        (TextMake pIns1                           ;Placer la legende
              (rtos (* (getvar "DIMSCALE") 5))
              0
              7
              0
              "%%ULEGEND"
              nil
        )
        (foreach item lListe
          (setq pIns1 (polar pIns1 (dtr 270) (* (getvar "DIMSCALE") 10))) ;5mm*2*scale
          (command "_.INSERT" (car item) pIns1 "" "" "")          ;Placer le bloc
          (while (wcmatch (getvar "cmdnames") "*INSERT*")
            (command "")
          )   ;while
          (GetBoundBox (entlast))
          (setq rEchel (/ rRefLg (* (getvar "DIMSCALE") 15.0)))
          (command "_.SCALE" (entlast) "" pIns1 "_R" rEchel 1.0)
          (setq bpt (polar pIns1 (dtr 0) (* (getvar "DIMSCALE") 20)))     ;5mm*4
          (TextMake bpt                           ;Placer la description
                (rtos (* (getvar "DIMSCALE") 2.5))
                0
                7
                0
                (cadr item)
                nil
          )
        )
        (setq ssObj (SSAfterObj oMarque))
        (princ "\rPlace legend...")
        (command "_.MOVE" ssObj "" pIns1 pause)
        (entdel oMarque)
      )

      ;;;---------------
      (defun AddAttrib (eInsert height mode prompt insPt tag value / )
        (TraceIsOn "AddAttrib.")
        (vla-addAttribute
          (vla-item (vla-get-blocks *sDoc)(vla-get-name (vlax-ename->vla-object eInsert)))
           height mode prompt (vlax-3d-point insPt) tag value
        )
        (command "_.ATTSYNC" "_s" (car ent1) "_y")
      )

      ;;;Attach attribute
      (defun AttAtt (/)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (princ (strcat "\nLegend Harvester." copyright))
        (setq ent1    (objsel "INSERT"
                "Select a block to attach description to"
                nil
              )
          pWs (cadr ent1)
        )
        (if (= sBlurb nil)
          (setq sBlurb "PARK BENCH")
        )
        (SETQ sBlurb (ustr 6 "Enter a description" sBlurb T))
        (AddAttrib (car ent1) 2.5 acAttributeModeInvisible "Enter a description:" pWs "AALEGEND" sBlurb)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun GetBoundBox (object / p1 p2)
        (TraceIsOn "GetBoundBox.")
        (vla-getboundingbox (vlax-ename->vla-object object) 'p1 'p2)
        (list (vlax-safearray->list p1) (vlax-safearray->list p2))
        (setq rRefLg (distance (vlax-safearray->list p1)
                   (vlax-safearray->list p2)
                 )
        )
      )

      ;-----------------
      ;---Column Layout.
      ;-----------------
      (defun C:CL () (PlaceColumns))

      (defun BlocksToListBox2 (/ BlockInfo)
        (TraceIsOn "BlocksToListBox2.")
        (Odcl_ListBox_Clear Walls_ColLayout_BlockNameList)
        (setq BlockInfo (ai_table "block" 62))
        (setq
          BlockInfo (remove_n (positionIt "AADetails" blockinfo) blockinfo)
        )
        (foreach blok BlockInfo
          (Odcl_ListBox_AddString Walls_ColLayout_BlockNameList blok)
        )
      )

      (defun LayersToListBox2 (/ LayInfo)
        (TraceIsOn "LayersToListBox2.")
        (Odcl_ListBox_Clear Walls_ColLayout_LayerComboBox)
        (setq LayInfo (acad_strlsort (ai_table "layer" 4)))
        (foreach calq LayInfo
          (Odcl_ComboBox_AddString Walls_ColLayout_LayerComboBox calq)
        )
      )

      (defun PlaceColumns ()
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )

        (WsFlashMess2
          (strcat "\nColumn Layout." copyright)
          (strcat "\nColumn Layout." copyright)
          (strcat "\nColumn Layout." copyright)
          (strcat "\nColumn Layout." copyright)
        )

        ;;(command "_.INSERT" "AADetails" ^C) ;;### TM WRONG CODE !!!
        (command "_.INSERT" "AADetails")(command)(command)

        (if (null sLeLayr)
          (Setq sLeLayr "0")
        )
        (if (= sNomDuBloc nil)
          (Setq sNomDuBloc "AARecCol")
        )

        (defun c:ColLayout_OnInitialize ()
          (if (= bGlobal nil)
            (progn (Odcl_ListBox_SetCurSel Walls_ColLayout_AreaOptionList 0)
               (Odcl_Control_SetProperty
                 Walls_ColLayout_SelAreaTextButton
                 "Enabled"
                 nil
               )
            )
            (progn (Odcl_ListBox_SetCurSel Walls_ColLayout_AreaOptionList 1)
               (Odcl_Control_SetProperty
                 Walls_ColLayout_SelAreaTextButton
                 "Enabled"
                 T
               )
            )
          )

          (LayersToListBox2)
          (BlocksToListBox2)
          (Odcl_ListBox_SelectString
            Walls_ColLayout_BlockNameList
            sNomDuBloc
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_BlockView
            "BlockName"
            sNomDuBloc
          )

          (cond
            ((= sNomDuBloc "AARecCol")
             (Odcl_Control_SetProperty
           Walls_ColLayout_WidthTextBox
           "Visible"
           T
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_LengthTextBox
           "Visible"
           T
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label2
           "Visible"
           T
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label3
           "Visible"
           T
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_DiamTextBox
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label4
           "Visible"
           nil
             )
            )
            ((= sNomDuBloc "AACirCol")
             (Odcl_Control_SetProperty
           Walls_ColLayout_WidthTextBox
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_LengthTextBox
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label2
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label3
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_DiamTextBox
           "Visible"
           T
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label4
           "Visible"
           T
             )
            )
            (T
             (Odcl_Control_SetProperty
           Walls_ColLayout_WidthTextBox
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_LengthTextBox
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label2
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label3
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_DiamTextBox
           "Visible"
           nil
             )
             (Odcl_Control_SetProperty
           Walls_ColLayout_Label4
           "Visible"
           nil
             )
            )
          )

          (Odcl_ComboBox_SelectString
            Walls_ColLayout_LayerComboBox
            sLeLayr
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_LineCheckBox
            "Value"
            T
          )

        )

        (defun c:ColLayout_AreaOptionList_OnSelChanged
           (nSelection sSelText /)
          ;;    (Odcl_MessageBox "To Do: code must be added to AreaOptionList" (etos nSelection))
          (if (= nSelection 0)
            (progn (setq bGlobal T)
               (Odcl_Control_SetProperty
                 Walls_ColLayout_SelAreaTextButton
                 "Enabled"
                 nil
               )
            )
            (progn (setq bGlobal nil)
               (Odcl_Control_SetProperty
                 Walls_ColLayout_SelAreaTextButton
                 "Enabled"
                 T
               )
            )
          )
        )

        (defun c:ColLayout_SelAreaTextButton_OnClicked ()
          (setq bDispForm nil)
          (Setq bWin (Odcl_ListBox_GetCurSel Walls_ColLayout_AreaOptionList))
          (Setq sLeLayr (Odcl_Control_GetProperty
                  Walls_ColLayout_LayerComboBox
                  "Text"
                )
          )
          (Odcl_Form_Close Walls_ColLayout)
          (setq pC1 (upoint 1 "" "Specify first corner" nil nil)
            pC2 (GetCorner pC1 "\nSpecify opposite corner:")
          )
          (if (= bWin 0)
            (setq bGlobal nil)
            (setq bGlobal T)
          )
          (setq bDispForm T)
        )

        (defun c:ColLayout_BlockNameList_OnSelChanged (nSelection sSelText /)
          ;;    (Odcl_MessageBox "To Do: code must be added to BlockNameList" "To do")
          (setq sNomDuBloc sSelText)
          (if (= sNomDuBloc "AARecCol")
            (progn
          (Odcl_Control_SetProperty
            Walls_ColLayout_WidthTextBox
            "Visible"
            T
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_LengthTextBox
            "Visible"
            T
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_Label2
            "Visible"
            T
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_Label3
            "Visible"
            T
          )
            )
            (progn
          (Odcl_Control_SetProperty
            Walls_ColLayout_WidthTextBox
            "Visible"
            nil
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_LengthTextBox
            "Visible"
            nil
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_Label2
            "Visible"
            nil
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_Label3
            "Visible"
            nil
          )
            )
          )
          (if (= sNomDuBloc "AACirCol")
            (progn
          (Odcl_Control_SetProperty
            Walls_ColLayout_DiamTextBox
            "Visible"
            T
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_Label4
            "Visible"
            T
          )
            )
            (progn
          (Odcl_Control_SetProperty
            Walls_ColLayout_DiamTextBox
            "Visible"
            nil
          )
          (Odcl_Control_SetProperty
            Walls_ColLayout_Label4
            "Visible"
            nil
          )
            )
          )

          (Odcl_Control_SetProperty
            Walls_ColLayout_BlockView
            "BlockName"
            sNomDuBloc
          )
        )

        (defun c:ColLayout_CancelTextButton_OnClicked ()
          (setq bDispForm nil)
          (Setq sLeLayr (Odcl_Control_GetProperty
                  Walls_ColLayout_LayerComboBox
                  "Text"
                )
          )
          (Odcl_Form_Close Walls_ColLayout)
        )

        (defun c:ColLayout_Ok_OnClicked ()
          (setq bDispForm nil)        ;same for cancel
          (Setq sLeLayr (Odcl_Control_GetProperty
                  Walls_ColLayout_LayerComboBox
                  "Text"
                )
          )
          (Setq bWin (Odcl_ListBox_GetCurSel Walls_ColLayout_AreaOptionList))
          (Setq bLine (Odcl_Control_GetProperty
                Walls_ColLayout_LineCheckBox
                "Value"
              )
          )                   ;nil or T
          (Setq bPline (Odcl_Control_GetProperty
                 Walls_ColLayout_PlineCheckBox
                 "Value"
               )
          )                   ;nil or T
          (Setq rWidt (Odcl_Control_GetProperty
                Walls_ColLayout_WidthTextBox
                "Text"
              )
          )
          (Setq rLengt (Odcl_Control_GetProperty
                 Walls_ColLayout_LengthTextBox
                 "Text"
               )
          )
          (Setq rDiam (Odcl_Control_GetProperty
                Walls_ColLayout_DiamTextBox
                "Text"
              )
          )
          (Odcl_Form_Close Walls_ColLayout)

          (if (= sLeLayr "")
            (Setq sLeLayr "0")
          )

          (cond
            ((= sNomDuBloc "AARecCol")
             (setq rValX rLengt
               rValY rWidt
             )
            )
            ((= sNomDuBloc "AACirCol")
             (setq rValX rDiam
               rValY rDiam
             )
            )
            (T
             (setq rValX 1
               rValY 1
             )
            )
          )

          (setq ssObj2 (GETssObj)
            PTS    (TrueIntersGetter ssObj2)
          )

          (foreach PT PTS         ;Loop through list of points
            (command "_.INSERT" sNomDuBloc (trans Pt 0 1) rValX rValY 0)
          )
        )

        (if (= (Odcl_LoadProject ProjName2 nil) -10)
          (progn
            (WsFlashMess1
          "Echec chargement bote de dialogue. Fichier Walls.ods introuvable.\nSi la location a change, la mise  jour de MetricArchAssist2004.ini est indispensable."
          "Archivio Walls.ods non trovato. Se il percorso modificasse da ultima installazione,\nl'aggiornamento di MetricArchAssist2004.ini  indispensabile."
          "Archivo Walls.ods no ha sido encontrado. Si la ubicacion es modificada\ndesde la ultima instalacion, archivo MetricArchAssist2004.ini tendra que ser actualizado con su nueva ubicacion."
          "File Walls.ods not found. If Path modified since last\ninstallation, update MetricArchAssist2004.ini file."
          1 5)
            (exit)
          )
          (setq bDispForm T)
        )

        (while bDispForm
          (setq bDispForm nil)
          (Odcl_Form_Show Walls_ColLayout)
        )

        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;---------------------------------
      ;---Add customised short-cut menu.
      ;---------------------------------
      (defun vlaxx-string-p (arg)
        (and (equal (type arg) 'str))
      )

      (defun vlaxx-integer-p (arg)
        (and (equal (type arg) 'int))
      )

      (defun vlaxx-real-p (arg)
        (and (equal (type arg) 'real))
      )

      (defun AAOn ()
        (if (not *mouse*)
          (progn
            (setq *mouse* (aa:mouse nil))
            (princ "\nMouse reactor enabled!\n")
          )
        )
        (if (or (null *Custom-Popup*)
            (null (MenuItemsGet *Custom-Popup*))
            )
          (progn
            (aa:CustomPopMenuAdd)
            (princ "\nCustom menu popup enabled!\n")
          )
        )
        (princ)
      )

      (defun AAOff ()
        (if *Custom-Popup*
          (progn
            (mapcar 'vla-delete (MenuItemsGet *Custom-Popup*))
            (menucmd "p0=acad.pop0")
            (princ
          (strcat
            "\nAll menu items have been removed from: \n; menugroup = "
            (vla-get-name
              (vla-get-parent (vla-get-parent *Custom-Popup*))
            )
            "\n; Name = "
            (vla-get-name *Custom-Popup*)
            "\n"
          )
            )
          )
        )
        (princ)
      )

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      (pragma '((unprotect-assign *MenuGroupname*)))
      (setq *MenuGroupname* "acad")
      (pragma '((protect-assign *MenuGroupname*)))
      (setq *enableColor* T)
      ;;(setvar "cmdecho" 0)
      (vl-load-reactors)

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      (defun aa:CleanMouseReactor ()
        (setq *mouse* nil)
        ;;LCA - COMMENT: (vlr-remove-all) returns a list of lists
        (vlr-remove-all :VLR-Mouse-Reactor)
        T
      )

      (defun aa:mouse (data)
        (VLR-Mouse-Reactor
          data
          '(
            (:VLR-beginDoubleClick . aa:StartDoubleClick)
            (:VLR-beginRightClick . aa:StartRightClick)
           )
        )
      )

      (defun aa:StartDoubleClick (reactorObject Listofsomething)
        (TraceIsOn "aa:StartDoubleClick.")
        (alert (strcat
             "Begin Double Click"
             "\nReactor Object: "
             (vl-princ-to-string reactorObject)
             "\nClick Point:"
             (vl-princ-to-string Listofsomething)
           )
        )
        (princ)
      )

      (defun aa:StartRightClick (reactorObject Listofsomething / menuPopName)
        (TraceIsOn "aa:StartRightClick.")
        (setq *objectUnderCursor*
           (if (setq data (nentselp "" (car Listofsomething)))
             (vlax-ename->vla-object (car data))
           )
        )
        (if (and *objectUnderCursor*
             (equal (getvar "cmdnames") "")
            )
          (progn

            (princ (strcat "\nObject \""
                   (vla-get-ObjectName *objectUnderCursor*)
                   "\"\ under Cursor!\n"
               )
            )

            (if (setq menuPopName (aa:FindPopMenuPosition "Custom-Menu"))
          (progn
            (menucmd (strcat "p0=" *MenuGroupname* "." menuPopName))
            (menucmd "p0=*")
          )
          (progn
            (alert "Can not find \"CustomMenu\" pop up")
            (menucmd "p0=acad.pop0")
          )
            )
          )
          (progn
            (menucmd "p0=acad.pop0")
          )
        )
        (princ)
      )

      (defun aa:FindPopMenuPosition (PopName / dl cnt)
        (TraceIsOn "aa:FindPopMenuPosition.")
        (setq cnt 0)
        (vlax-for menu
              (vla-get-Menus
                (vla-item (vla-get-MenuGroups (vlax-get-acad-object))
                  "acad"
                )
              )
          (if (/= (vla-get-TagString menu) "")
            (progn
          (setq cnt (1+ cnt)
                dl  (cons
                  (cons (strcase (vla-get-name menu))
                    (strcat "POP" (vl-princ-to-string cnt))
                  )
                  dl
                )
          )
            )
          )
        )
        (cdr (assoc (strcase PopName) dl))
      )

      (defun aa:Menus (/ acad menuGroups dl)
        (setq acad         (vlax-get-acad-object)
          *acadmenuGroups* (vla-get-MenuGroups acad)
        )
        (vlax-for Item *acadmenuGroups* (setq dl (cons item dl)))
        (reverse dl)
      )

      ;;; returns a menuObject. The parameter must be a valid menugroup
      (defun aa:ReturnMenuObject (MenuName / dl)
        (TraceIsOn "aa:ReturnMenuObject.")
        (if (menugroup MenuName)
          (progn
            (vlax-for Item (vla-get-MenuGroups (vlax-get-acad-object))
          (setq dl (cons (list (strcase (vla-get-name item)) item) dl))
            )
            (cadr (assoc (strcase MenuName) dl))
          )
        )
      )

      ;;; adds a menu item to a popupobject.
      ;;; This function is identical to vla-AddMenuItem except that
      ;;; error checking is performed. Returns the menuitem
      ;;; object if successful. If an error is encountered an error
      ;;; message is printed and the function returns nil.
      (defun aa:MenuItemAdd (ParentMenuObject Index Label Macro / res)
        (TraceIsOn "aa:MenuItemAdd.")
        (print (list ParentMenuObject Index Label Macro))
        (if (and (vlaxx-string-p Label)
             (or (vlaxx-integer-p Index)
                 (vlaxx-string-p Index)
                 (equal (vlax-variant-type Index) 2)
                 (equal (vlax-variant-type Index) 8)
             )
             (vlaxx-string-p Macro)
            )
          (if
            (and (equal (type ParentMenuObject) 'vla-object)
             (vlax-property-available-p ParentMenuObject "ShortcutMenu")
            )
             (progn
           (setq res (vla-AddMenuItem ParentMenuObject Index Label Macro))
             )
             (princ "\nError: ParentMenuObject is not a valid pop up menu object")
          )
          (princ "\nError: Index, Label or Macro are not correct")
        )
        res
      )

      (defun aa:DropMenuItem (item)
        (TraceIsOn "aa:DropMenuItem.")
        (princ
          (apply 'strcat
             (mapcar
               (function
                 (lambda (x)
               (strcat
                 "\n; "
                 (vl-princ-to-string x)
                 " = "
                 (vl-prin1-to-string
                   (eval
                     (list (read
                         (strcat "vla-get-" (vl-princ-to-string x))
                       )
                       item
                     )
                   )
                 )
               )
                 )
               )
               '(Application       Caption    Check      Enable
                 HelpString Index      Label  Macro      Parent
                 Tagstring  Type
                )
             )
          )
        )
        (terpri)
        T
      )

      ;;; searches for a popup label in a popupmenu object returns the popupmenu object is it exists.
      (defun aa:PopMenuExists (MenuGroupObject popupLabel / dl)
        (vlax-for item MenuGroupObject
          (setq dl (cons
                 (list (strcase (vla-get-NameNoMnemonic item))
                   item
                 )
                 dl
               )
          )
        )
        (cadr (assoc (strcase popuplabel) dl))
      )

      (defun aa:MenuItemLabelExist (MenuPopupObject itemLabel / dl)
        (vlax-for item MenuPopupObject
          (setq dl (cons
                 (list (strcase (vla-get-TagString item))
                   item
                 )
                 dl
               )
          )
        )
        (cadr (assoc (strcase itemLabel) dl))
      )

      ;;; returns a list of all menuitem objects contained within a popupmenu object.
      (defun MenuItemsGet (popup / dl)
        (if popup
          (progn
            (vlax-for MenuItem POpup
          (setq dl (cons MenuItem dl))
            )
            (reverse dl)
          )
        )
      )

      ;;; Adds a specific custom popup menu to AutoCAD. Returns the newly created popupmenu Object.
      (defun aa:CustomPopMenuAdd ()
        (TraceIsOn "aa:CustomPopMenuAdd.")
        (if (or (null *Custom-Popup*)
            (null (MenuItemsGet *Custom-Popup*))
            )
          (progn
            (setq acadMenuObject     (aa:ReturnMenuObject *MenuGroupname*)
              acadPopupMenuGroup (vla-get-Menus acadMenuObject)
            )
            (if
          (not (aa:PopMenuExists acadPopupMenuGroup "Custom-Menu"))
           (setq *Custom-Popup*
              (vla-add acadPopupMenuGroup "Custom-Menu")
           )
            )
          )
        )


        (setq
          item0         (aa:MenuItemAdd *Custom-Popup* 0 "&Enter" "\n")
          item1         (vla-AddSeparator *Custom-Popup* 1)

      ;;; item2 (aa:MenuItemAdd *Custom-Popup* 2 "&Cancel" "(command) ")
      ;;;    item2          (aa:MenuItemAdd *Custom-Popup* 2 "&Redraw" "_Redraw ")

          item2         (aa:MenuItemAdd *Custom-Popup* 2 "&Erase opening" "_EO ")
          item3         (aa:MenuItemAdd *Custom-Popup* 3 "&Erase" "_Erase ")
          item4         (aa:MenuItemAdd *Custom-Popup* 4 "&Cop&y" "_Copy ")
          item5         (vla-AddSeparator *Custom-Popup* 5)
          *ColorSubMenu*    (vla-AddSubMenu
                  *Custom-Popup*
                  8
                  "Wood Studs"
                    )
          *BylayerMenuItem* (aa:MenuItemAdd
                  *ColorSubMenu*
                  0
                  "2x4"
                  "(aa:QuelleValue 0) "
                    )
          *RedMenuItem*     (aa:MenuItemAdd
                  *ColorSubMenu*
                  1
                  "2x6"
                  "(aa:QuelleValue 1) "
                    )
          *YellowMenuItem*  (aa:MenuItemAdd
                  *ColorSubMenu*
                  2
                  "2x8"
                  "(aa:QuelleValue 2) "
                    )

          *GreenMenuItem*   (aa:MenuItemAdd
                  *ColorSubMenu*
                  3
                  "2x10"
                  "(aa:QuelleValue 3) "
                    )
          *CyanMenuItem*    (aa:MenuItemAdd
                  *ColorSubMenu*
                  4
                  "2x12"
                  "(aa:QuelleValue 4) "
                    )
        )


        *Custom-Popup*
      )

      (defun aa:QuelleValue (Key /)
        (TraceIsOn "aa:QuelleValue.")
        (if *objectUnderCursor*
          (progn
            (cond ((equal Key 0) (setq Width 3.5))
              ((equal Key 1) (setq Width 5.5))
              ((equal Key 2) (setq Width 7.25))
              ((equal Key 3) (setq Width 9.25))
              ((equal Key 4) (setq Width 11.25))
            )
          )
        )
        Width
      )

      ;----------------------
      ;---CIRCLE CENTERLINES.
      ;----------------------
      (defun C:CCL (/ e1 e2 ss cen rad n)
        (TraceIsOn ".CCL")
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nAdd Circle Centrelines." copyright))
        (setvar "OSMODE" 1024)
        (command "_.UCS" "_W")
        (princ "\nSelect circle objects...")
        (setq ss (getall)
          n  (sslength ss)
          i  0
        )
        (repeat n
          (setq ename (ssname ss i)
            ent   (entget ename)
            i (1+ i)
          )
          (if (= "CIRCLE" (cdr (assoc 0 ent)))
            (progn
          (setq cen (dxf 10 ent)
                rad (dxf 40 ent)
                le  (* rad 1.3334)          ;Hangs out by 1/3 of radius
                pl  (polar cen (dtr 180) le)
                pt  (polar cen Hshpi90 le)
                pr  (polar cen (dtr 0) le)
                pb  (polar cen Hshpi270 le)
          )
          (Hshset_lay "CLines" "252" "Acad_Iso08w100" "0.25")
          (command "_.LINE" pl pr "")
          (command "_.LINE" pt pb "")
            )
          )
        )
        (command "_.UCS" "_P")
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;--------------------------
      ;---ArchAssist Flatten 'm.
      ;--------------------------
      (defun C:SROLL () (SRoll))

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      (setq FrPrompt065 "1er coin"
            ItPrompt065 "Specify first corner point"
            EsPrompt065 "Elija una esquina"
            EnPrompt065 "Specify first corner point"
      )
      (setq FrPrompt066 "Autre coin: "
            ItPrompt066 "Altro angolo: "
            EsPrompt066 "Otra esquina: "
            EnPrompt066 "Specify opposite corner: "
      )

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

      (defun ArchAssistemarque (/)
        (WsFlashMess1
          "Echec chargement bote de dialogue. Fichier ArchAssist.DCL introuvable.\nSi la location a change, la mise  jour de ArchAssist.INI est indispensable."
          "File ArchAssist.DCL not found. If Path modified since last\ninstallation, update ArchAssist.INI file."
          "Archivo ArchAssist.DCL no ha sido encontrado. Si la ubicacion es modificada\ndesde la ultima instalacion, archivo ArchAssist.INI tendra que ser actualizado con su nueva ubicacion."
          "File ArchAssist.DCL not found. If Path modified since last\ninstallation, update ArchAssist.INI file."
          1 5)
        (exit)
      )

      (defun NAM (c1 e1) (cdr (assoc c1 e1))) ;GETS ASSOC NAME

      (defun dopoly (en / r d x y e1 pname DD DD1 E2 e3 e)
        (setq pname en)           ;pname is the polyline name
        (setq r 1)
        (while r
          (setq e (entget pname))

          (if (/= (cdr (assoc 0 e)) "SEQEND")
            (progn
          (setq d (assoc 10 e))
          (setq d (cdr d))
          (if (= (dxf 0 e) "VERTEX")
            (setq d (trans d (cdr (assoc -1 e)) 1))
          )
          (setq x (car d))
          (setq y (cadr d))
          (setq e1 (list 10 x y newelev))
          (setq e2 (subst e1 (assoc 10 e) e))
          (setq dd (dxf 70 e))
          (if (assoc 210 e2)
            (setq e3 (subst (list 210 0.0 0.0 1.0) (assoc 210 e2) e2))
            (setq e3 e2)
          )
          (if (= (dxf 0 e) "POLYLINE")
            (if (<= (logior 8 dd) dd)
              (setq dd (- dd 8))
            )
          )
          (if (= (dxf 0 e) "POLYLINE")
            (if (<= (logior 16 dd) dd)
              (setq dd (- dd 16))
            )
          )
          (if (= (dxf 0 e) "POLYLINE")
            (if (<= (logior 32 dd) dd)
              (setq dd (- dd 32))
            )
          )
          (if (= (dxf 0 e) "POLYLINE")
            (if (<= (logior 64 dd) dd)
              (setq dd (- dd 64))
            )
          )
          (if (= (dxf 0 e) "VERTEX")
            (if (<= (logior 32 dd) dd)
              (setq dd (- dd 32))
            )
          )
          (if (= (dxf 0 e) "VERTEX")
            (if (<= (logior 64 dd) dd)
              (setq dd (- dd 64))
            )
          )

          (if (= (dxf 0 e) "VERTEX")
            (if (<= (logior 128 dd) dd)
              (setq dd (- dd 128))
            )
          )
          (setq dd1 (cons 70 dd))
          (setq e4 (subst dd1 (assoc 70 e3) e3))
          (entmake e4)
            )                 ;progn
          )                   ;if
          (if (= (dxf 0 e) "SEQEND")
            (progn
          (setq r nil)
          (entmake e)
            )
            (setq pname (entnext pname))
          )
        )                 ;while
        (entdel en)
        (entupd (entlast))
      )                   ;dopoly

      ;;; Flatten other entities
      (defun doents (en / d1 d2 e1 e2 e3 e4 x1 x2 y1 y2 pname)
        (setq pname en)           ;pname is the entity name, i is the index
        (setq e (entget pname))
        (if (= (cdr (assoc 0 e)) "LWPOLYLINE")
          (progn
            (setq elev (assoc 38 e))
            (setq e1 (cons 38 0.0))
            (setq e4 (subst e1 elev e))
            (entmod e4)
          )                   ;progn
          (progn
            (setq d1 (assoc 10 e))
            (setq x1 (cadr d1))
            (setq y1 (caddr d1))
            (setq e1 (list 10 x1 y1 newelev))
            (setq e2 (subst e1 d1 e))
            (setq d2 (assoc 11 e2))
            (setq x2 (cadr d2))
            (setq y2 (caddr d2))
            (setq e3 (list 11 x2 y2 newelev))
            (setq e4 (subst e3 d2 e2))
            (entmod e4)
          )                   ;progn
        )                 ;if
      )                   ;doents

      (defun audit? ()
        (if (= incaudi "1")
          (progn
            (rs_error)
            (setq incaudi "0")
          )
          (progn
            (set_tile "error"
              "Audit maintains drawing database integrity."
            )
            (setq incaudi "1")
          )
        )
      )

      ;;; Flatten text (and the extra groups 11-18)
      (defun do-text (en  /   x0  x1  x2  x3  x4  x5  x6  x7  x8  y0  y1  y2
              y3  y4  y5  y6  y7  y8  d   d1  d2  d3  d4  d5  d6  d7
              d8  e   e1  e2  e3  e4  e5  e6  e7  e8  e9
                 )
        (setq pname en)           ;pname is the entity name, i is the index
        (setq e (entget pname))
        (progn
          (setq d (assoc 10 e))
          (setq x (cadr d))
          (setq y (caddr d))
          (setq d1 (assoc 11 e))
          (setq x1 (cadr d1))
          (setq y1 (caddr d1))
          (setq d2 (assoc 12 e))
          (setq x2 (cadr d2))
          (setq y2 (caddr d2))
          (setq d3 (assoc 13 e))
          (setq x3 (cadr d3))
          (setq y3 (caddr d3))
          (setq d4 (assoc 14 e))
          (setq x4 (cadr d4))
          (setq y4 (caddr d4))
          (setq d5 (assoc 15 e))
          (setq x5 (cadr d5))
          (setq y5 (caddr d5))
          (setq d6 (assoc 16 e))
          (setq x6 (cadr d6))
          (setq y6 (caddr d6))
          (setq d7 (assoc 17 e))
          (setq x7 (cadr d7))
          (setq y7 (caddr d7))
          (setq d8 (assoc 18 e))
          (setq x8 (cadr d8))
          (setq y8 (caddr d8))
          (setq e0 (list 10 x y newelev))
          (setq e1 (list 11 x1 y1 newelev))
          (setq e2 (list 12 x2 y2 newelev))
          (setq e3 (list 13 x3 y3 newelev))
          (setq e4 (list 14 x4 y4 newelev))
          (setq e5 (list 15 x5 y5 newelev))
          (setq e6 (list 16 x6 y6 newelev))
          (setq e7 (list 17 x7 y7 newelev))
          (setq e8 (list 18 x8 y8 newelev))
          (setq e9 (subst e0 d e))
          (setq e9 (subst e1 d1 e9))
          (setq e9 (subst e2 d2 e9))
          (setq e9 (subst e3 d3 e9))
          (setq e9 (subst e4 d4 e9))
          (setq e9 (subst e5 d5 e9))
          (setq e9 (subst e6 d6 e9))
          (setq e9 (subst e7 d7 e9))
          (setq e9 (subst e8 d8 e9))

          (entmod e9)
        )                 ;progn
      )                   ;dotext

      (defun set_all ()
        (mode_tile "inc_all" 1)       ; disable all button
        (mode_tile "inc_cler" 0)      ; enable clear button
        (set_tile "inc_attr" "1")
        (set_tile "inc_line" "1")
        (set_tile "inc_arc" "1")
        (set_tile "inc_circ" "1")
        (set_tile "inc_dims" "1")
        (set_tile "inc_text" "1")
        (set_tile "inc_pont" "1")
        (set_tile "inc_insrt" "1")
        (set_tile "inc_poly" "1")
        (set_tile "inc_sold" "1")
        (set_tile "inc_shpe" "1")
        (set_tile "inc_wipe" "1")
        (setq inclear nil)
      )

      (defun clear_all ()
        (mode_tile "inc_all" 0)
        (mode_tile "inc_cler" 1)
        (set_tile "inc_attr" "0")
        (set_tile "inc_line" "0")
        (set_tile "inc_arc" "0")
        (set_tile "inc_circ" "0")
        (set_tile "inc_dims" "0")
        (set_tile "inc_text" "0")
        (set_tile "inc_pont" "0")
        (set_tile "inc_insrt" "0")
        (set_tile "inc_poly" "0")
        (set_tile "inc_sold" "0")
        (set_tile "inc_shpe" "0")
        (set_tile "inc_wipe" "0")
        (setq inclear T)
      )

      (defun chk_all_tiles ()
        (cond
          ((= "1" (get_tile "inc_line"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_attr"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_arc"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_circ"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_dims"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_text"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_pont"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_insrt"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_poly"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_sold"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_shpe"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          ((= "1" (get_tile "inc_wipe"))
           (progn (mode_tile "inc_all" 0) (mode_tile "inc_cler" 0))
          )
          (t (mode_tile "inc_cler" 1))
        )
        (if (and
          (= "1" (get_tile "inc_line"))
          (= "1" (get_tile "inc_attr"))
          (= "1" (get_tile "inc_arc"))
          (= "1" (get_tile "inc_circ"))
          (= "1" (get_tile "inc_dims"))
          (= "1" (get_tile "inc_text"))
          (= "1" (get_tile "inc_pont"))
          (= "1" (get_tile "inc_insrt"))
          (= "1" (get_tile "inc_poly"))
          (= "1" (get_tile "inc_sold"))
          (= "1" (get_tile "inc_shpe"))
          (= "1" (get_tile "inc_wipe"))
            )
          (mode_tile "inc_all" 1)
        )
      )

      (defun save_inctiles ()
        (setq tile-list nil)
        (setq tile-list (STORE_TILES "inc_attr" tile-list))
        (setq tile-list (STORE_TILES "inc_line" tile-list))
        (setq tile-list (STORE_TILES "inc_arc" tile-list))
        (setq tile-list (STORE_TILES "inc_circ" tile-list))
        (setq tile-list (STORE_TILES "inc_dims" tile-list))
        (setq tile-list (STORE_TILES "inc_text" tile-list))
        (setq tile-list (STORE_TILES "inc_pont" tile-list))
        (setq tile-list (STORE_TILES "inc_insrt" tile-list))
        (setq tile-list (STORE_TILES "inc_poly" tile-list))
        (setq tile-list (STORE_TILES "inc_sold" tile-list))
        (setq tile-list (STORE_TILES "inc_shpe" tile-list))
        (setq tile-list (STORE_TILES "inc_wipe" tile-list))
        (setq tile-list (STORE_TILES "inc_audi" tile-list))
        (setq tile-list (STORE_TILES "newelev" tile-list))
      )

      (defun store_tiles (tile tile-list)
        (cons (list tile (get_tile tile)) tile-list)
      )

      (defun restore_tiles (tile-list / list-cnt)
        (setq list-cnt 0)
        (repeat (length tile-list)
          (set_tile (nth 0 (nth list-cnt tile-list))
                (nth 1 (nth list-cnt tile-list))
          )
          (setq list-cnt (1+ list-cnt))
        )
        (princ)
      )

      (defun get_corners (/)
        (cond
          ((= langue "Franais")
           (setq pt1 (upoint 8 "" FrPrompt065 nil nil)
             pt2 (getcorner pt1 FrPrompt066)
             cnr T
           )
          )
          ((= langue "Italiano")
           (setq pt1 (upoint 8 "" ItPrompt065 nil nil)
             pt2 (getcorner pt1 ItPrompt066)
             cnr T
           )
          )
          ((= langue "Espanol")
           (setq pt1 (upoint 8 "" EsPrompt065 nil nil)
             pt2 (getcorner pt1 EsPrompt066)
             cnr T
           )
          )
          (T
           (setq pt1 (upoint 8 "" EnPrompt065 nil nil)
             pt2 (getcorner pt1 EnPrompt066)
             cnr T
           )
          )
        )
      )

      (defun transfer_pts ()
        (set_tile "x_pt1" (rtos (nth 0 pt1)))
        (set_tile "y_pt1" (rtos (nth 1 pt1)))
        (set_tile "z_pt1" (rtos (nth 2 pt1)))
        (set_tile "x_pt2" (rtos (nth 0 pt2)))
        (set_tile "y_pt2" (rtos (nth 1 pt2)))
        (set_tile "z_pt2" (rtos (nth 2 pt2)))
      )

      (defun presets ()
        (mode_tile "clear" 1)
        (set_tile "inc_attr" "0")
        (set_tile "inc_arc" "0")
        (set_tile "inc_circ" "0")
        (set_tile "inc_dims" "0")
        (set_tile "inc_insrt" "0")
        (set_tile "inc_text" "1")
        (set_tile "inc_pont" "1")
        (set_tile "inc_poly" "1")
        (set_tile "inc_line" "1")
        (set_tile "inc_sold" "0")
        (set_tile "inc_shpe" "0")
        (set_tile "inc_wipe" "1")
        (set_tile "inc_all" "0")
        (set_tile "inc_cler" "0")
        (set_tile "inc_audi" "0")
        (set_tile "newelev" (rtos newelev))
        (set_tile "x_pt1" (rtos (nth 0 (getvar "vsmin"))))
        (set_tile "y_pt1" (rtos (nth 1 (getvar "vsmin"))))
        (set_tile "z_pt1" (rtos (nth 2 (getvar "vsmin"))))
        (set_tile "x_pt2" (rtos (nth 0 (getvar "vsmax"))))
        (set_tile "y_pt2" (rtos (nth 1 (getvar "vsmax"))))
        (set_tile "z_pt2" (rtos (nth 2 (getvar "vsmax"))))
      )

      (defun SRoll (/       fl1     dia_id  inctext incline incpoly incaudi
                incall  inclear flat_tot        ss      ed      en
                sl      n       il      ip      ia      il      pt1
               )
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (command "_.UCS" "_W")
        (setvar "ORTHOMODE" 0)
        (setvar "OSMODE" 0)
        (setvar "REGENMODE" 0)
        (command "_.VIEW" "_S" "SRoll")
        (WsFlashMess2
          (strcat "\nLe Rouleau Compresseur." copyright)
          (strcat "\nSteamRoll Objects to a new Z coordinate value."
              copyright
          )
          (strcat "\nAplanar a un nuevo valor de Z." copyright)
          (strcat "\nSteamRoll Objects to a new Z coordinate value."
              copyright
          )
        )
        (cond
          ((= langue "Franais")
           (setq bakup
              (dos_msgbox
                "Rouleau Compresseur 'dtruit' certains lements du dessin.\nServez vous d'une copie, pas de l'original. On continu?"
                "ArchAssist  Message"
                4
                1
              )
           )
          )
          ((= langue "Italiano")
           (setq bakup
              (dos_msgbox
                "SteamRoll 'destroys' some drawing parts.\nWork on a copy, not original. Continue?"
                "ArchAssist  Message"
                4
                1
              )
           )
          )
          ((= langue "Espanol")
           (setq bakup
              (dos_msgbox
                "SteamRoll 'destroys' some drawing parts.\nWork on a copy, not original. Continue?"
                "ArchAssist  Message"
                4
                1
              )
           )
          )
          (T
           (setq bakup
              (dos_msgbox
                "SteamRoll 'destroys' some drawing parts.\nWork on a copy, not original. Continue?"
                "ArchAssist  Message"
                4
                1
              )
           )
          )
        )
        (if (= bakup 6)
          (progn
            (setq c   "|"
              cnr nil
              ss  nil
            )
            (if (= newelev nil)
          (setq newelev 0.0)
            )
            (setq dcl_id (load_dialog (strcat Chemin "ArchAssist.DCL")))
            (setq status 4)
            (while (< 1 status)
          (if (not
                (cond
              ((= langue "Franais") (new_dialog "Fr_Smash" dcl_id))
              ((= langue "Italiano") (new_dialog "It_Smash" dcl_id))
              ((= langue "Espanol") (new_dialog "Es_Smash" dcl_id))
              (T (new_dialog "Smash" dcl_id))
                )
              )
            (progn
              (ArchAssistemarque)
            )
          )
          (if pt1
            (progn
              (transfer_pts)
              (restore_tiles tile-list)
            )
            (presets)
          )
          (cond
            ((= langue "Franais")
             (set_tile "Smash1" "ArchAssist  Rouleau Compresseur")
            )
            ((= langue "Italiano")
             (set_tile "Smash1" "ArchAssist  SteamRoller")
            )
            ((= langue "Espanol")
             (set_tile "Smash1" "ArchAssist  SteamRoller")
            )
            (T (set_tile "Smash1" "ArchAssist  SteamRoller"))
          )
          (fill_image_color "wht_col" -15)
          (action_tile "sel-pts" "(save_inctiles) (done_dialog 3)")
          (action_tile "x_pt1" "(rs_error)(setq x_pt1 $value)")
          (action_tile "y_pt1" "(rs_error)(setq y_pt1 $value)")
          (action_tile "z_pt1" "(rs_error)(setq z_pt1 $value)")
          (action_tile "x_pt2" "(rs_error)(setq x_pt2 $value)")
          (action_tile "y_pt2" "(rs_error)(setq y_pt2 $value)")
          (action_tile "z_pt2" "(rs_error)(setq z_pt2 $value)")
          (action_tile "inc_attr" "(chk_all_tiles)")
          (action_tile "inc_arc" "(chk_all_tiles)")
          (action_tile "inc_circ" "(chk_all_tiles)")
          (action_tile "inc_dims" "(chk_all_tiles)")
          (action_tile "inc_insrt" "(chk_all_tiles)")
          (action_tile "inc_text" "(chk_all_tiles)")
          (action_tile "inc_pont" "(chk_all_tiles)")
          (action_tile "inc_poly" "(chk_all_tiles)")
          (action_tile "inc_line" "(chk_all_tiles)")
          (action_tile "inc_sold" "(chk_all_tiles)")
          (action_tile "inc_shpe" "(chk_all_tiles)")
          (action_tile "inc_wipe" "(chk_all_tiles)")
          (action_tile "inc_all" "(set_all)")
          (action_tile "inc_cler" "(clear_all)")
          (action_tile "inc_audi" "(audit?)")
          (action_tile "accept" "(SR_chk)")
          (action_tile "about" "(_about)")
          (action_tile "cancel" "(done_dialog 0)")
          (setq status (start_dialog))
          (cond
            ((= status 0) (done_dialog 0))
            ((>= status 4) (setq status (start_dialog)))
            ((= status 1)
             (if cnr
               (setq ss (ssget "_C" pt1-lst pt2-lst))
               (progn
                 (setq ss (ssget "_X"
                         (list (cons -4 "<OR")
                           (cons -4 "<AND")
                           (cons 0 "POLYLINE")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "LWPOLYLINE")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "CIRCLE")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "WIPEOUT")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "LINE")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "POINT")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "TEXT")
                           (cons -4 "AND>")
                           (cons -4 "<AND")
                           (cons 0 "ARC")
                           (cons -4 "AND>")
                           (cons -4 "OR>")
                         )
                  )
                 )
               )
             )
             (if (= inclear nil)
               (progn
                 (if ss
               (progn
                 (setq n 0)

                 (command "_.VPOINT" "-1,-1,0.4")

                 (WsFlashMess2
                   "\nObjets restant  traiter:\n"
                   "\nRemaining objects to process:\n"
                   "\nRemaining objects to process:\n"
                   "\nRemaining objects to process:\n"
                 )
                 (while (< n (sslength ss))
                   (princ (strcat "\r" (rtos (- (sslength ss) n) 2 0))
                   )
                   (setq en (ssname ss n))
                   (setq ed (entget en))
                   (if (and (= incattr "1") (= (dxf 0 ed) "ATTRIB"))
                     (doents en)
                   )
                   (if (and (= incarc "1") (= (dxf 0 ed) "ARC"))
                     (doents en)
                   )
                   (if (and (= incirc "1") (= (dxf 0 ed) "CIRCLE"))
                     (doents en)
                   )
                   (if
                     (and (= incdims "1") (= (dxf 0 ed) "DIMENSION"))
                  (doents en)
                   )
                   (if (and (= incinsrt "1") (= (dxf 0 ed) "INSERT"))
                     (doents en)
                   )
                   (if (and (= incpont "1") (= (dxf 0 ed) "POINT"))
                     (doents en)
                   )
                   (if
                     (and (= incpoly "1") (= (dxf 0 ed) "POLYLINE"))
                  (dopoly en)
                   )
                   (if
                     (and (= incpoly "1") (= (dxf 0 ed) "LWPOLYLINE"))
                  (doents en)
                   )
                   (if (and (= incline "1") (= (dxf 0 ed) "LINE"))
                     (doents en)
                   )
                   (if (and (= inctext "1") (= (dxf 0 ed) "TEXT"))
                     (do-text en)
                   )
                   (if (and (= incsold "1") (= (dxf 0 ed) "SOLID"))
                     (doents en)
                   )
                   (if (and (= incshpe "1") (= (dxf 0 ed) "SHAPE"))
                     (doents en)
                   )
                   (if (and (= incwipe "1") (= (dxf 0 ed) "WIPEOUT"))
                     (doents en)
                   )
                   (setq n (1+ n))
                 )
                 (if (= incaudi "1")
                   (command "_.AUDIT" "_y")
                 )
               )
                 )
               )
             )
            )
            ((= status 3) (get_corners))
          )
            )
            (unload_dialog dcl_id)
          )
        )
        (command "_.VIEW" "_R" "SRoll")
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun SR_chk ()
        (cond
          ((= (type (read (get_tile "x_pt1"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid X co-ordinate")
           (mode_tile "x_pt1" 2)
          )
          ((= (type (read (get_tile "y_pt1"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid Y co-ordinate")
           (mode_tile "y_pt1" 2)
          )
          ((= (type (read (get_tile "z_pt1"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid Z co-ordinate")
           (mode_tile "z_pt1" 2)
          )
          ((= (type (read (get_tile "x_pt2"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid X co-ordinate")
           (mode_tile "x_pt2" 2)
          )
          ((= (type (read (get_tile "y_pt2"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid Y co-ordinate")
           (mode_tile "y_pt2" 2)
          )
          ((= (type (read (get_tile "z_pt2"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid Z co-ordinate")
           (mode_tile "z_pt2" 2)
          )
          ((= (type (read (get_tile "newelev"))) 'SYM)
           (fill_image_color "wht_col" 1)
           (set_tile "error" "Invalid value")
           (mode_tile "newelev" 2)
          )
          (T
           (setq
             incattr  (get_tile "inc_attr")
             incarc   (get_tile "inc_arc")
             incirc   (get_tile "inc_circ")
             incdims  (get_tile "inc_dims")
             incinsrt (get_tile "inc_insrt")
             inctext  (get_tile "inc_text")
             incpont  (get_tile "inc_pont")
             incpoly  (get_tile "inc_poly")
             incline  (get_tile "inc_line")
             incsold  (get_tile "inc_sold")
             incshpe  (get_tile "inc_shpe")
             incwipe  (get_tile "inc_wipe")
             incall   (get_tile "inc_all")
             x_pt1    (distof (get_tile "x_pt1"))
             y_pt1    (distof (get_tile "y_pt1"))
             z_pt1    (distof (get_tile "z_pt1"))
             x_pt2    (distof (get_tile "x_pt2"))
             y_pt2    (distof (get_tile "y_pt2"))
             z_pt2    (distof (get_tile "z_pt2"))
             newelev  (distof (get_tile "newelev"))
             pt1-lst  (list x_pt1 y_pt1 z_pt1)
             pt2-lst  (list x_pt2 y_pt2 z_pt2)
           )
           (done_dialog 1)
          )                   ;T
        )
      )

      ;;------------------------------------
      ;;Function : Returns a list of all segment start and end points from a list of points
      ;;Arguments:
      ;;          'vlist'  - List of points
      ;;          'Closed' - Flag to indicate if the polygon is closed
      ;;Action   : Returns a list of segment points
      (defun GetLesVertices (vlist Closed / SegPtLst cnt len pt1 pt2)
        (setq SegPtLst '())
        (if vlist
          (progn
            (setq
          len (length vlist)
          cnt 0
            )
            (repeat (1- len)
          (setq
            pt1      (nth cnt vlist)
            pt2      (nth (1+ cnt) vlist)
            SegPtLst (cons (list pt1 pt2) SegPtLst)
            cnt      (1+ cnt)
          )
            )
            (if Closed
          (setq SegPtLst (cons (list (last vlist) (car vlist)) SegPtLst))
            )
            (setq SegPtLst (reverse SegPtLst))
          )
        )
        SegPtLst
      )

      ;;Function : Draws a series of vectors using grdraw and the specified color
      ;;Arguments: 'vlist'  - List of points to draw
      ;;           'Closed' - Flag to indicate if the polygon is closed or not
      ;;           'Color'  - Indicates the color to be used for drawing the vectors
      (defun GrDrawVecs (vlist Closed Color / Lst)
        (setq Lst (GetLesVertices vlist Closed))
        (foreach tmp Lst
          (grdraw (car tmp) (cadr tmp) Color)
        )
      )

      ;;-----------------  ***********Appears to conflict with short-cut list routine AAOn!
      ;;(setq *Acad-Object* nil
      ;;      *active-document*    nil
      ;;)
      ;;(setq *acad-object* (vlax-Get-Acad-Object))
      ;;(setq *active-document* (vla-Get-ActiveDocument *Acad-Object*))
      ;;
      ;;(defun ReacSouris (sAppNameAA)
      ;;  (setq    MouseReac (vlr-mouse-reactor                ;Constructs an editor reactor object that notifies of mouse event.
      ;;           sAppNameAA
      ;;           '(;;(:vlr-beginDoubleClick . Mlc_DblClick)      ;The user has double-clicked.
      ;;             (:vlr-beginRightClick . Mlc_RightClick)       ;The user has right-clicked.
      ;;            )
      ;;         )
      ;;  )
      ;;  (vlr-set-notification MouseReac 'active-document-only)
      ;;  (princ)
      ;;)
      ;;
      ;;(defun Mlc_RightClick (obj lst / ssObj typ sLay lLayers    )
      ;;  (setq    ssObj nil
      ;;   typ nil
      ;;  )
      ;;  (if lst
      ;;    (progn
      ;;      (setq ssObj (ssget "i"))
      ;;      (if (and ssObj (= 'PICKSET (type ssObj)) (= (sslength ssObj) 1))
      ;;   (progn
      ;;     (setq lLayers (vla-get-layers *active-document*))
      ;;     (cond
      ;;       ((setq sLay (vla-get-layer
      ;;            (vlax-ename->vla-object (ssname ssObj 0))
      ;;              )
      ;;        )
      ;;             (vla-sendcommand *active-document* "_.Point ")
      ;;        (vla-put-activelayer
      ;;          *active-document*
      ;;          (vla-item lLayers sLay)
      ;;        )
      ;;       )
      ;;     )
      ;;     (vlax-release-object lLayers)
      ;;   )
      ;;      )
      ;;    )
      ;;  )
      ;;  (princ)
      ;;)
      ;;
      ;;(defun RemReacSouris ()
      ;;  (if (vlr-reactors :vlr-mouse-reactor)
      ;;    (progn (vlr-remove-all :vlr-mouse-reactor) (setq MouseReac nil))
      ;;  )
      ;;  (princ)
      ;;)
      ;;
      ;;(defun CargaReacSouris ()
      ;;  (if ReacSouris
      ;;    (ReacSouris nil)
      ;;  )
      ;;  (princ)
      ;;)
      ;;
      ;;(RemReacSouris)
      ;;(CargaReacSouris)

      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ;;(defun c:plp () (ChainIt))

      ;;;  ;; creates a selection set of objects which meet end to end
      ;;;  ;; only selects objects on the same layer as picked object
      ;;;  ;; pass selection set to pedit join...

      ;;  Return (ename Startpt Endpt)
      (defun AtArc (ent / e rp r ba ea p1 p2)
        (setq e  (cdr (assoc -1 ent))
          rp (cdr (assoc 10 ent))
          r  (cdr (assoc 40 ent))
          ba (cdr (assoc 50 ent))
          ea (cdr (assoc 51 ent))
          p1 (trans (polar rp ba r) e 0)
          p2 (trans (polar rp ea r) e 0)
        )
        (list e p1 p2)
      )

      ;;  Return (ename Startpt Endpt)
      (defun AtDroite (ent)
        (list
          (cdr (assoc -1 ent))
          (cdr (assoc 10 ent))
          (cdr (assoc 11 ent))
        )
      )

      ;;  Return (ename Startpt Endpt)
      (defun AtPLigne (ent / e)
        (setq e (cdr (assoc -1 ent)))
        (list
          e
          (car (Extremite e))
          (cadr (Extremite e))
        )
      )

      ;;  Add ent-> (ename Startpt Endpt) to list
      (defun AtListe (e / ent)
        (setq ent (entget e))
        (cond
          ((= (cdr (assoc 0 ent)) "LINE")
           (setq sslist (cons (AtDroite ent) sslist))
          )
          ((= (cdr (assoc 0 ent)) "ARC")
           (setq sslist (cons (AtArc ent) sslist))
          )
          ((= (cdr (assoc 0 ent)) "POLYLINE")
           (setq sslist (cons (AtPLigne ent) sslist))
          )
          ((= (cdr (assoc 0 ent)) "LWPOLYLINE")
           (setq sslist (cons (AtPLigne ent) sslist))
          )
        )
      )

      ;;argument: an ename - returns: Start and End points as a list
      (defun Extremite (vobj / name stpt endpt)
        (if (= (type vobj) 'ename)
          (setq vobj (vlax-ename->vla-object vobj))
        )
        (and
          (setq name (vla-get-objectname vobj))
          (cond
            ((vl-position
           name
           '("AcDbArc"           "AcDbLine"      "AcDbEllipse"
             "AcDbSpline"        "AcDbPolyline"      "AcDb2dPolyline"
             "AcDb3dPolyline"
            )
             )
             (setq stpt (vlax-curve-getstartpoint vobj))
             (setq endpt (vlax-curve-getendpoint vobj))
            )
          )                   ;cond
        )                 ;and
        (list stpt endpt)
      )

      ;; get list of (ename startpt endpt) for picked ent
      (defun ObtListe (ent)
        (cond
          ((= (cdr (assoc 0 ent)) "ARC")
           (setq ent (AtArc ent))
          )
          ((= (cdr (assoc 0 ent)) "LINE")
           (setq ent (AtDroite ent))
          )
          ((= (cdr (assoc 0 ent)) "POLYLINE")
           (setq ent (AtPLigne ent))
          )
          ((= (cdr (assoc 0 ent)) "LWPOLYLINE")
           (setq ent (AtPLigne ent))
          )
        )
        ent
      )

      (defun ChainIt (/      sslist elist  ss i      e
                  e2     found  ent    ent2   ok     start  end
                  start2 end2   fuzz   layer  ssex   typlst
                 )
        ;; Get the start object
        (if                   ;
          (and
            (cadr (ssgetfirst))       ;objects are selected
            ;;at least one arc, line or pline
            (setq ssex (ssget "i" (list (cons 0 "LINE,ARC,LWPOLYLINE,POLYLINE"))))
          )                   ;and

           ;; ======  then  =============
           (setq e (ssname ssex 0))
           ;; ======  else  =============
           (progn
             (sssetfirst)
             (setq typlst '("LINE" "ARC" "POLYLINE" "LWPOLYLINE"))
             (while
           (or
             (not (setq e
                     (car (entsel "\nSelect start object (arc, line or pline) for chain: "))
              )
             )
             (not (member (cdr (assoc 0 (entget e))) typlst))
           )
            (princ "\nTry again...")
             )                ;while
           )                  ;progn
        )                 ;if

        (setq typlst '("LINE" "ARC" "POLYLINE" "LWPOLYLINE"))
        (while
          (or
            (not
          (setq e2 (car (entsel "\nSelect end object (arc, line or pline) for chain: ")))
            )
            (not (member (cdr (assoc 0 (entget e2))) typlst))
          )
           (princ "\nTry again...")
        )                 ;while

        (and
          (setq ok   1
            fuzz 1e-8         ; 0.00000001
          )
          (setq ent (entget e))       ; first or picked ent
          (setq ent2 (entget e2))     ; second picked ent,
          (setq layer (cdr (assoc 8 ent)))    ; layer to match
          (= layer (cdr (assoc 8 ent2)))  ; layers match
          (setq ent    (ObtListe ent)
            elist  '()
            start  (cadr ent)
            end    (caddr ent)
            ent2   (ObtListe ent2)    ;
            start2 (cadr ent2)
            end2   (caddr ent2)
          )
          (setq ss                ; get all objects that matched picked
             (ssget "X"
                (list '(0 . "LINE,ARC,*POLYLINE") (cons 8 layer))
             )
          )
          (ssdel e ss)            ; remove picked start from selection set

          ;;  make a list of all from ss  ((ename startpt endpt) ....)
          (setq i 0)
          (repeat (sslength ss)
            (AtListe (ssname ss i))
            (setq i (1+ i))
          )                   ;repeat
          ;;  find attached items, does not test all branches
          (PtContigu start ent sslist)
          (if (not found)
            (PtContigu end ent sslist)
          )
        )                 ;and
        (if found
          (progn
            (setq elist (cons ent elist))
            (setq ssres (ssadd))
            (foreach x elist          ;create a selection set of the list
          (ssadd (car x) ssres)
            )
      ;;    (cadr (sssetfirst ssres ssres))   ;display the selected items

             (while
                (progn (princ "\nSelect objects to add or hold down Shift key to deselect...\n")
                   (command "._SELECT" ssres pause)
                   (not (setq ssres (ssget "_P" (list (cons 0 "LINE,ARC,POLYLINE,LWPOLYLINE") )))
                   )
                )
               (print "No object found...")
             )

          (setq 1ss (ak_ssget ssres))

          (command "_.PEDIT" "_M" ssres "" "_Y" "_J" 0 "")
              (setq ew1 (entlast))

              (SetWallInfoTag sAppNameAA ew1)
          (setq spt (upoint 1 "" "Specify point on side to offset" nil nil))
      ;;  (command "_.CHPROP" ew1 "" "_LA" sLAy1 "")
          (vla-put-layer (vlax-Ename->vla-Object ew1) sLAy1)

          (command "_.offset" (read sWidth1) ew1 spt "")              ;run the offset command
              (setq ew2 (entlast))
              (SetWallInfoTag sAppNameAA ew2)
          (vla-put-layer (vlax-Ename->vla-Object ew2) sLAy2)

          (if (= sTypes "DoubleW")
            (progn (command "_.offset"
                    (+ (read sWidth1) (read sCavite))
                    ew1
                    spt
                    ""
               )
                   (setq ew3 (entlast))
                   (vla-put-layer (vlax-Ename->vla-Object ew3) sLAy4)
                       (SetWallInfoTag sAppNameAA ew3)

               (command "_.offset" (read sOverall) ew1 spt "")
                   (setq ew4 (entlast))
                       (SetWallInfoTag sAppNameAA ew4)
                   (vla-put-layer (vlax-Ename->vla-Object ew4) sLAy3)

                   (command "_.EXPLODE" ew3)
                   (command "_.EXPLODE" ew4)
            )
          )
             (command "_.EXPLODE" ew1)
             (command "_.EXPLODE" ew2)
          )                   ; progn
          (prompt "\n* Unable to join. No contiguous objects found *")
        )
      )

      ;;  check the list for matching points
      ;;  p point to match
      ;;  elst (ename startpt endpt) of pt
      ;;  |List list pf remaining elst
      ;;  ### TM : using '|' for symbol names was critical to LTE, FIXED in LTE now !!
      ;;  ### TM : but better use only _ $ ^ # chars for special purpose !
      (defun PtContigu    (p elst |list / entx ex p1 p2 idx res)
        (setq idx (length |List))
        (while (and (not found) (>= (setq idx (1- idx)) 0))
          (setq entx (nth idx |List)
                ex   (car entx)
                p1   (cadr entx)
                p2   (caddr entx)
          )
          (cond               ; test point match with fuzz factor
            ((equal p start2 fuzz)        ; text for target
             (setq found 1)
             (setq elist (cons ent2 elist))
            )
            ((equal p end2 fuzz)      ; text for target
             (setq found 1)
             (setq elist (cons ent2 elist))
            )
            ((equal p p1 fuzz)        ; test for next branch
             (setq res (PtContigu p2 entx (vl-remove entx |List)))
             (if found            ; we are backing out collecting the path
               (setq elist (cons entx elist))
             )
            )
            ((equal p p2 fuzz)        ; test for next branch
             (setq res (PtContigu p1 entx (vl-remove entx |List)))
             (if found            ; we are backing out collecting the path
               (setq elist (cons entx elist))
             )
            )
          )
        )
        T
      )

      ;---------------------------
      ;---ArchAssist Door Modify.
      ;---------------------------
      (defun LayersForDoorsM (/)
        (TraceIsOn ".LayersForDoorsM")
        (Odcl_ComboBox_Clear Walls_ModOpens_HeaderComboBox)
        (Odcl_ComboBox_Clear Walls_ModOpens_ThresholdComboBox)

        (setq LayrInfo (ai_table "Layer" 4)
          lLay1    (Remove_Dup (cons sHeaderLyr LayrInfo))
          lLay5    (Remove_Dup (cons sThreshLyr LayrInfo))
          lLay1    (acad_strlsort lLay1)
          lLay5    (acad_strlsort lLay5)
        )

        (foreach item lLay1 (Odcl_ComboBox_AddString Walls_ModOpens_HeaderComboBox item))
        (Odcl_ComboBox_SelectString Walls_ModOpens_HeaderComboBox sHeaderLyr)             ;sLay1 23
        (foreach item lLay5 (Odcl_ComboBox_AddString Walls_ModOpens_ThresholdComboBox item))
        (Odcl_ComboBox_SelectString Walls_ModOpens_ThresholdComboBox sThreshLyr)          ;23
      )

      (defun LayersForWindsM (/)
        (TraceIsOn ".LayersForWindsM")
        (Odcl_ComboBox_Clear Walls_ModOpens_WnHeaderComboBox)
        (Odcl_ComboBox_Clear Walls_ModOpens_WnSillComboBox)

        (setq LayrInfo (ai_table "Layer" 4)
          lLays1   (Remove_Dup (cons sWHeaderLyr LayrInfo))
          lLays5   (Remove_Dup (cons sSillLyr LayrInfo))
          lLays1   (acad_strlsort lLays1)
          lLays5   (acad_strlsort lLays5)
        )

        (foreach item lLays1 (Odcl_ComboBox_AddString Walls_ModOpens_WnHeaderComboBox item))
        (Odcl_ComboBox_SelectString Walls_ModOpens_WnHeaderComboBox sWHeaderLyr)          ;41
        (foreach item lLays5 (Odcl_ComboBox_AddString Walls_ModOpens_WnSillComboBox item))
        (Odcl_ComboBox_SelectString Walls_ModOpens_WnSillComboBox sSillLyr)               ;23
        (TraceIsOn "LayersForWindsM.")
      )

      ;;;----
      (defun ModOuvert (/ ssdoor A0a A1a A2 A3)
        (TraceIsOn ".ModOuvert")
        (setq sOpenAng1 nil
          leafwidt1 nil
          bBypass T
        )
        (setq ssdoor nil
          ssWind nil
        )
        (setq bPorte nil
          bFeinster nil
          bThreSil nil
          rMoveInsPt
           0.0
        )

        (if (= (Odcl_Tab_GetCurSel Walls_ModOpens_TabControl1) 0)
          (progn

            (if (= iHeadr nil)                                ;For Doors
          (Setq iHeadr (atoi sHeaderCol))
            )
            (if (= iThres nil)
          (Setq iThres (atoi sThreshCol))
            )
            (if (= iJamb nil)
          (Setq iJamb (atoi sJambCol))
            )
            (if (= iLeaf nil)
          (Setq iLeaf (atoi sLeafCol))
            )
            (if (= iSwing nil)
          (Setq iSwing (atoi sSwingCol))
            )

            (LayersForDoorsM)

            (setq sManuf1  (car (strfind lSngM1 "-"))                 ;"Single Doors-Manuf1"
              lWid1    (cadr (strfind lSngM1 "-"))
              lDoorWidth1  nil
              lDoorWidth1  (DeStrAListe lWid1 "|")

              lDoorWidth1s nil
              lDoorWidth1s (DeStrAListe (cadr (strfind lSldM1 "-")) "|")          ;"Sliding Doors-Manuf1"

              sManuf2  (car (strfind lSngM2 "-"))                 ;"Single Doors-Manuf2"
              sManuf3  (car (strfind lSngM3 "-"))                 ;"Single Doors-Manuf3"

              lDFSFWidth1  nil
              lDFSFWidth1  (DeStrAListe (cadr (strfind lFSFM1 "-")) "|")          ;"Sliding Doors FSF-Manuf1"
              lDFSSFWidth1 nil
              lDFSSFWidth1 (DeStrAListe (cadr (strfind lFSSFM1 "-")) "|")         ;"Sliding Doors FSSF-Manuf1"

              lDoorWidth2  nil
              lDoorWidth2  (DeStrAListe lWid2 "|")                    ;"Double Doors"
              lDoorWidth3  nil
              lDoorWidth3  (DeStrAListe lWid3 "|")                    ;"Folding Doors"


              lDoorWidth1  (Remove_Dup (cons sTr3a lDoorWidth1))              ;"Single Doors-Manuf1"
              lDFSFWidth1  (Remove_Dup (cons sTr3a lDFSFWidth1))              ;"Sliding Doors FSF-Manuf1"
              lDFSSFWidth1 (Remove_Dup (cons sTr3a lDFSSFWidth1))             ;"Sliding Doors FSSF-Manuf1"

              lDoorWidth2  (Remove_Dup (cons sTr3a lDoorWidth2))              ;"Double Doors"
              lDoorWidth3  (Remove_Dup (cons sTr3a lDoorWidth3))              ;"Folding Doors"
            )

            (if (or (= sDrTyp "Sliding [Sliding-Fixed]")
                (= sDrTyp "Sliding [F-S-F]")
                (= sDrTyp "Sliding [F-S-S-F]")
            )
          (progn

            (LayersForWindsM)

            (if (= iHead nil)
              (Setq iHead (atoi sWHeaderCol))
            )
            (if (= iSill nil)
              (Setq iSill (atoi sSillCol))
            )
            (if (= iMull nil)
              (Setq iMull (atoi sFrameCol))
            )
            (if (= iSash nil)
              (Setq iSash (atoi sSashCol))
            )
            (if (= iGlass nil)
              (Setq iGlass (atoi sGlazeCol))
            )

            (setq sTr3a sTrW3a
              lDoorWidth1s
               (Remove_Dup (cons sTr3a lDoorWidth1s))                 ;"Sliding Doors-Manuf1"
            )
          )
            )

            (Odcl_ComboBox_Clear Walls_ModOpens_DrWidthComboBox)

            (cond ((= sDrTyp "Single hung")
               (foreach item lDoorWidth1
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
              )
              ((= sDrTyp "Double door")
               (foreach item lDoorWidth2
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )
              ((= sDrTyp "Double door unequal")
               (foreach item lDoorWidth2
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )

              ((= sDrTyp "Bifold double")
               (foreach item lDoorWidth2
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )

              ((= sDrTyp "Bifold single")
               (foreach item lDoorWidth3
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )
              ((= sDrTyp "Surface sliding single")
               (foreach item lDoorWidth1
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )
              ((= sDrTyp "Cavity sliding single")
               (foreach item lDoorWidth1
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )
              ((= sDrTyp "Doorway")                           ;AABuild108
               (foreach item lDoorWidth2
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )

              ((= sDrTyp "Sliding [Sliding-Fixed]")                   ;AABuild108
               (foreach item lDoorWidth1s
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )

              ((= sDrTyp "Sliding [F-S-F]")                       ;AA1.3b100
               (foreach item lDFSFWidth1
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )
              ((= sDrTyp "Sliding [F-S-S-F]")                         ;AA1.3b100
               (foreach item lDFSSFWidth1
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
                 )
               )
               (ShowDoorM)
              )
            )

            (if (/= nil sWDTest)
          (progn
            (Odcl_ComboBox_AddString
              Walls_ModOpens_DrWidthComboBox
              sWDTest
            )
            (Odcl_ComboBox_SelectString
              Walls_ModOpens_DrWidthComboBox
              sWDTest
            )
          )
          (Odcl_ComboBox_SelectString
            Walls_ModOpens_DrWidthComboBox
            sTr3a
          )
            )
          )

          (progn

            (if (= iHead nil)                                 ;For Windows
          (Setq iHead (atoi sWHeaderCol))
            )
            (if (= iSill nil)
          (Setq iSill (atoi sSillCol))
            )
            (if (= iMull nil)
          (Setq iMull (atoi sFrameCol))
            )
            (if (= iSash nil)
          (Setq iSash (atoi sSashCol))
            )
            (if (= iGlass nil)
          (Setq iGlass (atoi sGlazeCol))
            )

            (LayersForWindsM)

            (setq sSeuil4 (last (DeStrAListe lTFxdM1 "-")))
            (setq lSeuils (DeStrAListe (car (DeStrAListe lTFxdM1 "-")) "|"))
            (setq sSeuil1 (car lSeuils))
            (setq sSeuil2 (cadr lSeuils))
            (setq sSeuil3 (caddr lSeuils))

            (setq sManuf1 (car (strfind lWFxdM1 "-"))
              lFWin1W (cadr (strfind lWFxdM1 "-"))                    ;Fixed Windows
              lFxdWWidth  nil
              lFxdWWidth  (DeStrAListe lFWin1W "|")
              lFWin1H (cadr (strfind lHFxdM1 "-"))
              lFxdWHeight nil
              lFxdWHeight (DeStrAListe lFWin1H "|")

              lSWin1W (cadr (strfind lWSLdM1 "-"))                    ;Sliding Windows
              lSLdWWidth  nil
              lSLdWWidth  (DeStrAListe lSWin1W "|")
              lSWin1H (cadr (strfind lHSLdM1 "-"))
              lSLdWHeight nil
              lSLdWHeight (DeStrAListe lSWin1H "|")
              lSLdWWidt3  nil

              lDWin1W (cadr (strfind lWDHgM1 "-"))                    ;Double Hung Windows
              lDHgWWidth  nil
              lDHgWWidth  (DeStrAListe lDWin1W "|")
              lDWin1H (cadr (strfind lHDHgM1 "-"))
              lDHgWHeight nil
              lDHgWHeight (DeStrAListe lDWin1H "|")
              lDHgWWidt3  nil

                          ;   sManuf2 (car (strfind lWFxdM2 "-"))
                          ;   sManuf3 (car (strfind lWFxdM3 "-"))
            )

            (setq lFxdWWidth (Remove_Dup (cons sTrW3a lFxdWWidth)))
            (setq lFxdWHeight (Remove_Dup (cons sTr4b lFxdWHeight)))

            (setq lSLdWWidth (Remove_Dup (cons sTrW3a lSLdWWidth)))               ;2 panneaux
            (setq iCnnt (1- (length lSLdWWidth)))
            (while (> (atoi (nth iCnnt lSLdWWidth)) (atoi sSeuil2))
          (setq lSLdWWidt3
                 (append (list (nth iCnnt lSLdWWidth)) lSLdWWidt3)
          )                                       ;3 panneaux
          (setq iCnnt (1- iCnnt))
            )
            (setq lSLdWWidt3 (Remove_Dup (cons sTrW3a lSLdWWidt3)))
            (setq lSLdWHeight (Remove_Dup (cons sTr4b lSLdWHeight)))

            (setq lDHgWWidth (Remove_Dup (cons sTrW3a lDHgWWidth)))
            (setq lDHgWHeight (Remove_Dup (cons sTr4b lDHgWHeight)))

            (Odcl_ComboBox_Clear Walls_ModOpens_WnWidthComboBox)
            (Odcl_ComboBox_Clear Walls_ModOpens_WnHeightComboBox)

            (cond ((= sWnTyp "Fixed Panels")
               (foreach item lFxdWWidth
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
                 )
               )
               (foreach item lFxdWHeight
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
                 )
               )
               (ShowWindM)
              )
              ((= sWnTyp "Sliding [Sliding-Fixed]")
               (foreach item lSLdWWidth
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
                 )
               )
               (foreach item lSLdWHeight
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
                 )
               )
               (ShowWindM)
              )
              ((= sWnTyp "Sliding [S-F-S]")
               (foreach item lSLdWWidt3
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
                 )
               )
               (foreach item lSLdWHeight
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
                 )
               )
               (ShowWindM)
              )
              ((= sWnTyp "Double Hung")
               (foreach item lDHgWWidth
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
                 )
               )
               (foreach item lDHgWHeight
                 (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
                 )
               )
               (ShowWindM)
              )
            )

            (if (/= nil sWTest)
          (progn
            (Odcl_ComboBox_AddString
              Walls_ModOpens_WnWidthComboBox
              sWTest
            )
            (Odcl_ComboBox_SelectString
              Walls_ModOpens_WnWidthComboBox
              sWTest
            )
          )
          (Odcl_ComboBox_SelectString
            Walls_ModOpens_WnWidthComboBox
            sTrW3a
          )
            )

            (if (/= nil sHTest)
          (progn
            (Odcl_ComboBox_AddString
              Walls_ModOpens_WnHeightComboBox
              sHTest
            )
            (Odcl_ComboBox_SelectString
              Walls_ModOpens_WnHeightComboBox
              sHTest
            )
          )
          (Odcl_ComboBox_SelectString
            Walls_ModOpens_WnHeightComboBox
            sTr4b
          )
            )
          )
        )
      )

      (defun ShowDoorM    ( / bDrawJamb)
        (TraceIsOn ".ShowDoorM")
        (Setq bTimb
           (Odcl_Control_GetProperty Walls_ModOpens_TimberOption "Value")
        )
        (Setq bJbThk (Odcl_Control_GetProperty
               Walls_ModOpens_JambThkCheckBox
               "Value"
                 )
        )
        (Setq bCenter (Odcl_Control_GetProperty
                Walls_ModOpens_CenteredCheckBox
                "Value"
              )
        )

        (Odcl_PictureBox_Clear Walls_ModOpens_DoorPictureBox)
        (cond
          ((= sDrTyp "Single hung")
           (setq iLg       100
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 8 16 3 94 iLeaf)             ;Porte
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_ModOpens_DoorPictureBox                ;Swing
             (list (list 8 110 (- iLg 6) 25 110 11 10 iSwing))
           )
          )

          ((= sDrTyp "Double door")
           (setq iLg       140
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 8 44 3 (fix (/ (- iLg 7) 2)) iLeaf)      ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_ModOpens_DoorPictureBox                ;Swing gauche
             (list (list 8 110 (fix (/ (- iLg 7) 2)) 25 110 11 10 iSwing)
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 139 44 3 (fix (/ (- iLg 7) 2)) iLeaf)    ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_ModOpens_DoorPictureBox                ;Swing droit
             (list
           (list 139 110 (fix (/ (- iLg 7) 2)) 139 10 8 110 iSwing)
             )
           )
          )

          ((= sDrTyp "Double door unequal")
           (setq iLg       140
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 8 20 3 90 iLeaf)             ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_ModOpens_DoorPictureBox                ;Swing gauche
             (list (list 8 110 90 25 110 11 10 iSwing))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 139 67 3 43 iLeaf)               ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawArc
             Walls_ModOpens_DoorPictureBox                ;Swing droit
             (list (list 139 110 43 139 10 8 110 iSwing))
           )
          )

          ((= sDrTyp "Bifold double")
           (setq iLg       140
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;gauche
             (list (list 5 110 8 113 iLeaf)
               (list 5 110 30 85 iLeaf)
               (list 33 88 30 85 iLeaf)
               (list 33 88 8 113 iLeaf)
               (list 33 88 36 85 iLeaf)
               (list 36 85 60 110 iLeaf)
               (list 60 110 57 113 iLeaf)
               (list 57 113 33 88 iLeaf)
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;droite
             (list (list 145 110 142 113 iLeaf)
               (list 145 110 120 85 iLeaf)
               (list 142 113 117 88 iLeaf)
               (list 120 85 117 88 iLeaf)
               (list 117 88 114 85 iLeaf)
               (list 114 85 90 110 iLeaf)
               (list 90 110 93 113 iLeaf)
               (list 93 113 117 88 iLeaf)
             )
           )
          )

          ((= sDrTyp "Bifold single")
           (setq iLg       100
             bDrawJamb T
             sPortW    (etos (- (atoi str3a) (* 2 35)))
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox
             (list (list 5 110 8 113 iLeaf)
               (list 5 110 38 77 iLeaf)
               (list 38 77 41 80 iLeaf)
               (list 41 80 8 113 iLeaf)
               (list 41 80 44 77 iLeaf)
               (list 41 80 74 113 iLeaf)
               (list 74 113 77 110 iLeaf)
               (list 44 77 77 110 iLeaf)
             )
           )
          )

          ((= sDrTyp "Surface sliding double")
           (setq iLg 140
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list -30 104 70 3 iLeaf)              ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_ModOpens_DoorPictureBox
             (list (list -2 99 13 99 iLeaf)
               (list 13 99 8 97 iLeaf)
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 110 104 70 3 iLeaf)              ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_ModOpens_DoorPictureBox
             (list (list 152 99 137 99 iLeaf)
               (list 137 99 142 97 iLeaf)
             )
           )
          )

          ((= sDrTyp "Surface sliding single")
           (setq iLg 100
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 55 104 95 3 iLeaf)               ;Porte
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_ModOpens_DoorPictureBox
             (list (list 112 99 97 99 iLeaf)
               (list 97 99 102 97 iLeaf)
             )
           )
          )

          ((= sDrTyp "Cavity sliding double")
           (setq iLg 140
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list -30 122 70 3 iLeaf)              ;Porte gauche
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_ModOpens_DoorPictureBox
             (list (list -2 99 13 99 iLeaf)
               (list 13 99 8 97 iLeaf)
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 110 122 70 3 iLeaf)              ;Porte droite
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_ModOpens_DoorPictureBox
             (list (list 152 99 137 99 iLeaf)
               (list 137 99 142 97 iLeaf)
             )
           )
          )

          ((= sDrTyp "Cavity sliding single")
           (setq iLg 100
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 50 122 95 3 iLeaf)               ;Porte
             )
           )
           (Odcl_PictureBox_DrawLine                  ;Fleche
             Walls_ModOpens_DoorPictureBox
             (list (list 112 99 97 99 iLeaf)
               (list 97 99 102 97 iLeaf)
             )
           )
          )

          ((= sDrTyp "Doorway")                   ;AABuild108
           (setq iLg 100
             bDrawJamb nil
             sPortW "None"
           )
          )


          ((= sDrTyp "Sliding [Sliding-Fixed]")           ;AABuild108
           (setq iLg 140
             iHt 16
             bDrawJamb nil
             sPortW sTr3a
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list
           (list 5 121 iLg iHt iMull)             ;cadre ext. 5 89
           (list 5 121 4 iHt iMull)               ;cadre gauche
           (list 141 121 4 iHt iMull)             ;cadre droit    141 89 4
             )
           )

           (Setq bThreshold
              (Odcl_Control_GetProperty
                Walls_ModOpens_ThresholdCheckBox
                "Value"
              )
           )
           (if (= bThreshold T)
             (progn
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 5 108 iLg 13 iSill)          ;Sill
             )
           )
           (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_ModOpens_DoorPictureBox
               (list (list 6 109 (- iLg 2) 11 iSill 1)        ;hatch
               )
             )
           )
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 9 129 4 4 iMull)             ;montant gauche 1   9 97 4 4
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 13 129 60 4 iSash)               ;sash gauche        13 129 60 4
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing gauche     13 99 73 99
             (list (list 13 131 73 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 73 129 4 4 iMull)                ;montant gauche 2   73 129 4 4
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 73 125 4 4 iMull)                ;montant droit 1    73 93 4 4
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 77 125 60 4 iSash)               ;sash droit         77 125 60 4
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing droit      77 95 136 95
             (list (list 77 127 137 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 137 125 4 4 iMull)               ;montant droit 2    136 93 4 4
             )
           )
          )

          ((= sDrTyp "Sliding [F-S-F]")
           (setq iLg 140
             iHt 16
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list
           (list 5 121 iLg iHt iMull)             ;cadre ext. 5 89
           (list 5 121 4 iHt iMull)               ;cadre gauche
           (list 141 121 4 iHt iMull)             ;cadre droit    141 89 4
             )
           )

           (Setq bThreshold
              (Odcl_Control_GetProperty
                Walls_ModOpens_ThresholdCheckBox
                "Value"
              )
           )
           (if (= bThreshold T)
             (progn
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 5 108 iLg 13 iSill)          ;Sill
             )
           )
           (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_ModOpens_DoorPictureBox
               (list (list 6 109 (- iLg 2) 11 iSill 1)        ;hatch
               )
             )
           )
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 9 125 4 4 iMull)             ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 13 125 37 4 iSash)               ;sash gauche
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing gauche
             (list (list 13 127 50 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 50 125 4 4 iMull)                ;montant droit
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 50 129 4 4 iMull)                ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 54 129 43 4 iSash)               ;sash milieu
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing milieu
             (list (list 54 131 96 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 96 129 4 4 iMull)                ;montant droit
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 96 125 4 4 iMull)                ;montant gauche 1
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 100 125 37 4 iSash)              ;sash droit
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing droit
             (list (list 100 127 137 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 137 125 4 4 iMull)               ;montant droit
             )
           )
          )

          ((= sDrTyp "Sliding [F-S-S-F]")
           (setq iLg 140
             iHt 16
             bDrawJamb nil
             sPortW sTr3a
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list
           (list 5 121 iLg iHt iMull)             ;cadre ext.
           (list 5 121 4 iHt iMull)               ;cadre gauche
           (list 141 121 4 iHt iMull)             ;cadre droit
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 9 125 4 4 iMull)             ;montant gauche fixe
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 13 125 25 4 iSash)               ;sash gauche
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing gauche
             (list (list 13 127 38 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 38 125 4 4 iMull)                ;montant droit fixe
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 38 129 4 4 iMull)                ;montant gauche coulissant
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 42 129 29 4 iSash)               ;sash milieu
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing milieu
             (list (list 42 131 71 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 71 129 4 4 iMull)                ;montant droit coulissant
             )
           )
      ;;;---
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 75 129 4 4 iMull)                ;montant gauche coulissant
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 79 129 29 4 iSash)               ;sash milieu
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing milieu
             (list (list 79 131 108 131 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 108 129 4 4 iMull)               ;montant droit coulissant
             )
           )

           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 108 125 4 4 iMull)               ;montant droit fixe
             )
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 112 125 25 4 iSash)              ;sash droit
             )
           )
           (Odcl_PictureBox_DrawLine
             Walls_ModOpens_DoorPictureBox                ;glazing droit
             (list (list 112 127 137 127 iGlass))
           )
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 137 125 4 4 iMull)               ;montant droit fixe
             )
           )

           (Setq bThreshold
              (Odcl_Control_GetProperty
                Walls_ModOpens_ThresholdCheckBox
                "Value"
              )
           )
           (if (= bThreshold T)
             (progn
           (Odcl_PictureBox_DrawRect
             Walls_ModOpens_DoorPictureBox
             (list (list 5 108 iLg 13 iSill)          ;Sill
             )
           )
           (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_ModOpens_DoorPictureBox
               (list (list 6 109 (- iLg 2) 11 iSill 1)        ;hatch
               )
             )
           )
             )
           )
          )

        )

        (Odcl_PictureBox_DrawText
          Walls_ModOpens_DoorPictureBox
          (list (list 5 160 34 (strcat "Opening: " sTr3a) "BL" 0)
            (list 90 160 34 (strcat "Door: " sPortW) "BL" 0)  ;AABuild108
          )
        )

        (Setq bHeader (Odcl_Control_GetProperty
                Walls_ModOpens_HeaderCheckBox
                "Value"
              )
        )
        (if (= bHeader T)
          (if (= iLg 140)
            (Odcl_PictureBox_DrawLine
          Walls_ModOpens_DoorPictureBox               ;header
          (list (list 5 110 25 110 iHeadr)            ;Top
                (list 35 110 55 110 iHeadr)
                (list 65 110 85 110 iHeadr)
                (list 95 110 115 110 iHeadr)
                (list 125 110 145 110 iHeadr)
                (list 5 137 25 137 iHeadr)            ;Bottom
                (list 35 137 55 137 iHeadr)
                (list 65 137 85 137 iHeadr)
                (list 95 137 115 137 iHeadr)
                (list 125 137 145 137 iHeadr)
          )
            )
            (Odcl_PictureBox_DrawLine                 ;100 iLg
          Walls_ModOpens_DoorPictureBox               ;header
          (list (list 5 110 24 110 iHeadr)            ;Top
                (list 32 110 51 110 iHeadr)
                (list 59 110 78 110 iHeadr)
                (list 86 110 105 110 iHeadr)
                (list 5 137 24 137 iHeadr)            ;Bottom
                (list 32 137 51 137 iHeadr)
                (list 59 137 78 137 iHeadr)
                (list 86 137 105 137 iHeadr)
          )
            )
          )
        )

        (Setq bThreshold
           (Odcl_Control_GetProperty
             Walls_ModOpens_ThresholdCheckBox
             "Value"
           )
        )
        (if (and (= bThreshold T)
             (/= sDrTyp "Sliding [Sliding-Fixed]")
             (/= sDrTyp "Sliding [F-S-F]")
             (/= sDrTyp "Sliding [F-S-S-F]")
            )                             ;AABuild108
          (progn
            (if (= bCenter nil)
          (progn                          ;not centered
               (Odcl_PictureBox_DrawLine
                 Walls_ModOpens_DoorPictureBox            ;Sill
                 (list (list 5 117 (+ 5 iLg) 117 iSill)
               (list 5 117 5 140 iSill)
               (list (+ 5 iLg) 117 (+ 5 iLg) 140 iSill)
               (list 5 140 (+ 5 iLg) 140 iSill)
             )
               )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
              Walls_ModOpens_DoorPictureBox
              (list (list 5 111 iLg 23 iThres)            ;Sill
              )
            )
          )
            )
          )
        )

        (if (= bCenter nil)
          (progn                          ;not centered
            (Odcl_PictureBox_DrawRect
          Walls_ModOpens_DoorPictureBox
          (list (list -10 110 15 11 24)               ;Mur haut gauche
                (list -10 126 15 11 24)               ;Mur bas gauche
          )
            )
            (Odcl_PictureBox_DrawHatchRect
          Walls_ModOpens_DoorPictureBox
          (list (list -10 111 13 9 252 3)             ;Top hatch
                (list -10 127 13 9 252 3)             ;Bot. hatch
          )
            )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
          Walls_ModOpens_DoorPictureBox
          (list (list -10 104 15 11 24)               ;Mur haut gauche
                (list -10 120 15 11 24)               ;Mur bas gauche
          )
            )
            (Odcl_PictureBox_DrawHatchRect
          Walls_ModOpens_DoorPictureBox
          (list (list -10 105 13 9 252 3)             ;Top hatch
                (list -10 121 13 9 252 3)             ;Bot. hatch
          )
            )
          )
        )

        (if (= iLg 100)
          (progn
            (if (= bCenter nil)
          (progn                          ;not centered
            (Odcl_PictureBox_DrawRect
              Walls_ModOpens_DoorPictureBox
              (list (list 105 110 95 11 24)           ;Mur haut droit
                (list 105 126 95 11 24)           ;Mur bas droit
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_ModOpens_DoorPictureBox
              (list (list 106 111 93 9 252 3)             ;Top hatch
                (list 106 127 93 9 252 3)             ;Bot. hatch
              )
            )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
              Walls_ModOpens_DoorPictureBox
              (list (list 105 104 95 11 24)           ;Mur haut
                (list 105 120 95 11 24)           ;Mur bas
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_ModOpens_DoorPictureBox
              (list (list 106 105 93 9 252 3)             ;Top hatch
                (list 106 121 93 9 252 3)             ;Bot. hatch
              )
            )
          )
            )
          )
        )

        (if (= iLg 140)
          (progn
            (if (= bCenter nil)
          (progn                          ;not centered
            (Odcl_PictureBox_DrawRect
              Walls_ModOpens_DoorPictureBox
              (list (list 145 110 70 11 24)           ;Mur haut droit
                (list 145 126 70 11 24)           ;Mur bas droit
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_ModOpens_DoorPictureBox
              (list (list 146 111 68 9 252 3)             ;Top hatch
                (list 146 127 68 9 252 3)             ;Bot. hatch
              )
            )
          )
          (progn                          ;centered
            (Odcl_PictureBox_DrawRect
              Walls_ModOpens_DoorPictureBox
              (list (list 145 104 70 11 24)           ;Mur haut
                (list 145 120 70 11 24)           ;Mur bas
              )
            )
            (Odcl_PictureBox_DrawHatchRect
              Walls_ModOpens_DoorPictureBox
              (list (list 146 105 68 9 252 3)             ;Top hatch
                (list 146 121 68 9 252 3)             ;Bot. hatch
              )
            )
          )
            )
          )
        )

        (if (= bJbThk nil)
          (setq bThkOn 0)
          (setq bThkOn 1)
        )

        (if (= bDrawJamb T)
          (progn
            (if (= bTimb T)
          (progn
            (if (= bThkOn 0)
              (setq iDepth 126)
              (setq iDepth 137)
            )
            (Odcl_PictureBox_DrawLine             ;timber jamb
              Walls_ModOpens_DoorPictureBox           ;gauche
              (list (list 5 110 8 110 iJamb)
                (list 8 110 8 113 iJamb)
                (list 8 113 10 113 iJamb)
                (list 10 113 10 iDepth iJamb)
                (list 10 iDepth 5 iDepth iJamb)
                (list 5 iDepth 5 110 iJamb)
                (list (+ iLg 5) 110 (+ iLg 2) 110 iJamb)  ;droit
                (list (+ iLg 2) 110 (+ iLg 2) 113 iJamb)
                (list (+ iLg 2) 113 iLg 113 iJamb)
                (list iLg 113 iLg iDepth iJamb)
                (list iLg iDepth (+ iLg 5) iDepth iJamb)
                (list (+ iLg 5) iDepth (+ iLg 5) 110 iJamb)
              )
            )
          )
          (progn
            (if (= bThkOn 0)
              (setq iDepth 125)
              (setq iDepth 136)
            )
            (Odcl_PictureBox_DrawLine             ;steel jamb
              Walls_ModOpens_DoorPictureBox           ;gauche
              (list
                (list 5 110 8 110 iJamb)
                (list 8 110 8 113 iJamb)
                (list 8 113 10 113 iJamb)
                (list 10 113 10 iDepth iJamb)
                (list 10 iDepth 8 iDepth iJamb)
                (list 8 iDepth 8 (+ iDepth 3) iJamb)
                (list 8 (+ iDepth 3) 5 (+ iDepth 3) iJamb)
                (list (+ iLg 5) 110 (+ iLg 2) 110 iJamb)      ;droit
                (list (+ iLg 2) 110 (+ iLg 2) 113 iJamb)
                (list (+ iLg 2) 113 iLg 113 iJamb)
                (list iLg 113 iLg iDepth iJamb)
                (list iLg iDepth (+ iLg 2) iDepth iJamb)
                (list (+ iLg 2) iDepth (+ iLg 2) (+ iDepth 3) iJamb)
                (list (+ iLg 2) (+ iDepth 3) (+ iLg 5) (+ iDepth 3) iJamb)
              )
            )
          )
            )
          )
        )
        (Odcl_PictureBox_StoreImage Walls_ModOpens_DoorPictureBox)
      )

      ;;;-------------
      (defun c:ModOpens_DrWidthComboBox_OnSelChanged (nSelection sSelText /)
        (TraceIsOn ".c:ModOpens_DrWidthComboBox_OnSelChanged")
        (Setq sTr3a sSelText)
        (ShowDoorM)
      )
      (defun c:ModOpens_DrWidthComboBox_OnEditChanged (sText /)
        (TraceIsOn ".c:ModOpens_DrWidthComboBox_OnEditChanged")
        (Setq sTr3a sText
          sWDTest sText
        )
        (ShowDoorM)
      )

      (defun c:ModOpens_HeaderComboBox_OnSelChanged (nSelection sSelText /)
        (TraceIsOn ".c:ModOpens_HeaderComboBox_OnSelChanged")
        (Setq layinfo (tblsearch "LAYER" sSelText)
          iHeadr  (abs (cdr (assoc 62 layinfo)))          ;get color
        )
        (ShowDoorM)
      )
      (defun c:ModOpens_ThresholdComboBox_OnSelChanged (nSelection sSelText /)
        (TraceIsOn ".c:ModOpens_ThresholdComboBox_OnSelChanged")
        (Setq layinfo (tblsearch "LAYER" sSelText)
          iThres  (abs (cdr (assoc 62 layinfo)))          ;get color
        )
        (ShowDoorM)
      )

      ;;;-------------
      (defun ShowWindM (/)
        (TraceIsOn ".ShowWindM")
        (Setq bJbThk nil
          bCenter nil
        )
        (Setq bSillOut (Odcl_Control_GetProperty
                 Walls_ModOpens_OutlineOption
                 "Value"
               )
        )
        (Setq bSillHat (Odcl_Control_GetProperty
                 Walls_ModOpens_HatchSillOption
                 "Value"
               )
        )
        (setq iLg 180                         ;1800mm at 1/10eme
          iHt 16
        )

        (Odcl_PictureBox_Clear Walls_ModOpens_WindowPictureBox)

         (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
         (list (list 5 140 34 (strcat "Width:  " sTrW3a) "BL" 0)  ;AABuild108
           (list 5 155 34 (strcat "Height: " sTr4b) "BL" 0)   ;AABuild108
         )
         )

            (Odcl_PictureBox_DrawRect
               Walls_ModOpens_WindowPictureBox
               (list
                 (list   5 89 iLg iHt iMull)          ;cadre ext.
                 (list   5 89   4 iHt iMull)          ;cadre gauche
                 (list 181 89   4 iHt iMull)          ;cadre droit
               )
             )

        (Setq bHeaderW (Odcl_Control_GetProperty Walls_ModOpens_WnHeaderCheckBox "Value"))
        (if (= bHeaderW T)
          (Odcl_PictureBox_DrawLine                   ;(StartX StartY EndX EndY LineColor)
            Walls_ModOpens_WindowPictureBox               ;header
            (list
              (list   5  78  23  78 iHead)            ;18 et 9
              (list  32  78  50  78 iHead)
              (list  59  78  77  78 iHead)
              (list  86  78 104  78 iHead)
              (list 113  78 131  78 iHead)
              (list 140  78 158  78 iHead)
              (list 167  78 185  78 iHead)
              (list   5  105  23  105 iHead)
              (list  32  105  50  105 iHead)
              (list  59  105  77  105 iHead)
              (list  86  105 104  105 iHead)
              (list 113  105 131  105 iHead)
              (list 140  105 158  105 iHead)
              (list 167  105 185  105 iHead)
            )
          )
        )

        (Setq bTransom (Odcl_Control_GetProperty Walls_ModOpens_TransomCheckBox "Value"))
        (if (= bTransom T)
           (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
           (list (list 140 30 34 "+ Transom" "BL" 0))
           )
        )

        (Setq bSill (Odcl_Control_GetProperty Walls_ModOpens_WnSillCheckBox "Value"))
        (if (= bSill T)
          (progn
             (Odcl_PictureBox_DrawRect
               Walls_ModOpens_WindowPictureBox
               (list (list 5 76 iLg 13 iSill)             ;Sill
               )
             )
             (if (= bSillHat T)
             (Odcl_PictureBox_DrawHatchRect
               Walls_ModOpens_WindowPictureBox
               (list (list 6 77 (- iLg 2) 11 iSill 1)         ;hatch
               )
             )
             )
          )
        )

        (cond
          ((= sWnTyp "Fixed Panels")
           (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
           (list (list 5 30 34 "Fixed type" "BL" 0))
           )
           (if (<= (atoi sTrW3a) (atof sSeuil1))
             (progn (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 95 4 4 iMull)             ;gauche
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 95 164 4 iSash)          ;sash
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing
              (list (list 13 97 177 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 95 4 4 iMull)           ;droit
              )
                )
             )
           )
           (if (and (> (atoi sTrW3a)(atof sSeuil1))(<= (atoi sTrW3a)(atof sSeuil2)))
            (progn
                   (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
                    (list (list 5 30 34 "Fixed type" "BL" 0))
                   )
               (Odcl_PictureBox_DrawRect              ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 95 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 95 76 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing gauche
              (list (list 13 97 89 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 89 95 4 4 iMull)            ;montant gauche 2
              )
                )
              (Odcl_PictureBox_DrawRect
                Walls_ModOpens_WindowPictureBox
                (list (list 93 89 4 iHt iMull)        ;cadre du milieu
                )
               )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 97 95 4 4 iMull)            ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 101 95 76 4 iSash)          ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_ModOpens_WindowPictureBox         ;glazing droit
              (list (list 101 97 177 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 95 4 4 iMull)           ;montant droit 2
              )
                )
            )
           )
           (if (> (atoi sTrW3a) (atof sSeuil2))
             (progn
                   (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
                    (list (list 5 30 34 "Fixed type" "BL" 0))
                   )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
                  Walls_ModOpens_WindowPictureBox
              (list (list 9 95 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 95 47 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing gauche
              (list (list 13 97 60 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 60 95 4 4 iMull)            ;montant gauche 2
              )
                )
              (Odcl_PictureBox_DrawRect
                Walls_ModOpens_WindowPictureBox
                (list (list 64 89 4 iHt iMull)        ;cadre
                )
               )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 68 95 4 4 iMull)            ;montant milieu 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 72 95 46 4 iSash)           ;sash milieu
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_ModOpens_WindowPictureBox         ;glazing milieu
              (list (list 72 97 118 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 118 95 4 4 iMull)           ;montant milieu 2
              )
                )
              (Odcl_PictureBox_DrawRect
                Walls_ModOpens_WindowPictureBox
                (list (list 122 89 4 iHt iMull)       ;cadre
                )
               )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 126 95 4 4 iMull)           ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 130 95 47 4 iSash)          ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_ModOpens_WindowPictureBox         ;glazing droit
              (list (list 130 97 177 97 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 95 4 4 iMull)           ;montant droit 2
              )
                )
            )
           )
          )
          ((= sWnTyp "Sliding [Sliding-Fixed]")
                   (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
                    (list (list 5 30 34 "Sliding-Fixed type" "BL" 0))
                   )
               (Odcl_PictureBox_DrawRect              ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 97 80 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing gauche
              (list (list 13 99 93 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 93 97 4 4 iMull)            ;montant gauche 2
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 93 93 4 4 iMull)            ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 97 93 80 4 iSash)           ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_ModOpens_WindowPictureBox         ;glazing droit
              (list (list 97 95 177 95 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;montant droit 2
              )
                )
          )
          ((= sWnTyp "Sliding [S-F-S]")
                   (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
                    (list (list 5 30 34 "Sliding-Fixed-Sliding type" "BL" 0))
                   )
               (Odcl_PictureBox_DrawRect              ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;montant gauche 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 97 37 4 iSash)           ;sash gauche
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing gauche
              (list (list 13 99 50 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 50 97 4 4 iMull)            ;montant gauche 2
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 50 93 4 4 iMull)            ;montant milieu 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 54 93 82 4 iSash)           ;sash milieu
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_ModOpens_WindowPictureBox         ;glazing milieu
              (list (list 54 95 136 95 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 136 93 4 4 iMull)           ;montant milieu 2
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 136 97 4 4 iMull)           ;montant droit 1
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 140 97 37 4 iSash)          ;sash droit
              )
                )
                (Odcl_PictureBox_DrawLine
              Walls_ModOpens_WindowPictureBox         ;glazing droit
              (list (list 140 99 177 99 iGlass))
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 97 4 4 iMull)           ;montant droit 2
              )
                )
          )

          ((= sWnTyp "Double Hung")
           (Odcl_PictureBox_DrawText Walls_ModOpens_WindowPictureBox
           (list (list 5 30 34 "Double Hung type" "BL" 0))
           )
           (if (<= (atoi sTrW3a) (atof sSeuil1))
             (progn (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;gauche, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 93 4 4 iMull)             ;gauche, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 97 164 4 iSash)          ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 93 164 4 iSash)          ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, bas
              (list (list 13 99 177 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, haut
              (list (list 13 95 177 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;droit, haut
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 97 4 4 iMull)           ;droit, bas
              )
                )
             )
           )
           (if (and (> (atoi sTrW3a)(atof sSeuil1))(<= (atoi sTrW3a)(atof sSeuil2)))
             (progn (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;gauche, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 93 4 4 iMull)             ;gauche, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 97 80 4 iSash)           ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 93 80 4 iSash)           ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, bas
              (list (list 13 99 93 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, haut
              (list (list 13 95 93 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 93 97 4 4 iMull)            ;milieu, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 93 93 4 4 iMull)            ;milieu, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 97 93 80 4 iSash)           ;sash, haut fixe
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, haut fixe
              (list (list 97 95 177 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;droit, haut
              )
                )
             )
           )
           (if (> (atoi sTrW3a) (atof sSeuil2))
             (progn
                    (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 97 4 4 iMull)             ;gauche, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 9 93 4 4 iMull)             ;gauche, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 97 52 4 iSash)           ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 13 93 52 4 iSash)           ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, bas
              (list (list 13 99 65 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, haut
              (list (list 13 95 65 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 65 97 4 4 iMull)            ;1, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 65 93 4 4 iMull)            ;1, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 69 93 52 4 iSash)           ;sash, haut fixe
              )
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, haut fixe
              (list (list 69 95 121 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 121 97 4 4 iMull)           ;2, bas
              )
                )
                (Odcl_PictureBox_DrawRect             ;(UpperLeftX UpperLeftY Width Height Color)
              Walls_ModOpens_WindowPictureBox
              (list (list 121 93 4 4 iMull)           ;2, haut
              )
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 125 97 52 4 iSash)          ;sash, bas
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 125 93 52 4 iSash)          ;sash, haut
              )
                )

                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, bas
              (list (list 125 99 177 99 iGlass))
                )
                (Odcl_PictureBox_DrawLine             ;(StartX StartY EndX EndY LineColor)
              Walls_ModOpens_WindowPictureBox         ;glazing, haut
              (list (list 125 95 177 95 iGlass))
                )

                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 93 4 4 iMull)           ;droit, haut
              )
                )
                (Odcl_PictureBox_DrawRect
              Walls_ModOpens_WindowPictureBox
              (list (list 177 97 4 4 iMull)           ;droit, bas
              )
                )
             )
           )
          )

        )

                (Odcl_PictureBox_DrawRect
               Walls_ModOpens_WindowPictureBox
               (list (list -10 78 15 11 24)           ;Mur haut gauche
                 (list -10 94 15 11 24)           ;Mur bas gauche
                 (list 185 78 10 11 24)           ;Mur haut droit
                 (list 185 94 10 11 24)           ;Mur bas droit
               )
             )
             (Odcl_PictureBox_DrawHatchRect
               Walls_ModOpens_WindowPictureBox
               (list (list -10 79 13 9 252 3)             ;Top hatch
                 (list -10 95 13 9 252 3)             ;Bot. hatch
                 (list 186 79  9 9 252 3)
                 (list 186 95  9 9 252 3)
               )
             )
         (Odcl_PictureBox_StoreImage Walls_ModOpens_WindowPictureBox)
      )

      ;;;-------------
      (defun c:ModOpens_WnWidthComboBox_OnSelChanged (nSelection sSelText /)
        (Setq sTrW3a sSelText)
        (ShowWindM)
      )
      (defun c:ModOpens_WnWidthComboBox_OnEditChanged (sText /)
        (Setq sTrW3a sText
          sWTest  sText
        )
        (ShowWindM)
      )

      (defun c:ModOpens_WnHeightComboBox_OnSelChanged (nSelection sSelText /)
        (Setq sTr4b sSelText)
        (ShowWindM)
      )
      (defun c:ModOpens_WnHeightComboBox_OnEditChanged (sText /)
        (Setq sTr4b sText
          sHTest  sText
        )
        (ShowWindM)
      )

      (defun c:ModOpens_WnHeaderComboBox_OnSelChanged (nSelection sSelText /)
        (TraceIsOn ".c:ModOpens_WnHeaderComboBox_OnSelChanged")
        (Setq layinfo (tblsearch "LAYER" sSelText)
          iHead   (abs (cdr (assoc 62 layinfo))) ;get color
        )
        (ShowWindM)
      )
      (defun c:ModOpens_WnSillComboBox_OnSelChanged (nSelection sSelText /)
        (TraceIsOn ".c:ModOpens_WnSillComboBox_OnSelChanged")
        (Setq layinfo (tblsearch "LAYER" sSelText)
          iSill   (abs (cdr (assoc 62 layinfo))) ;get color
        )
        (ShowWindM)
      )

      (defun c:ModOpens_TabControl1_OnChanged (nSelIndex /)
        (TraceIsOn ".c:ModOpens_TabControl1_OnChanged")
        (ModOuvert)
      )

      (defun c:ModOpens_OK_OnClicked ()
        (TraceIsOn ".c:ModOpens_OK_OnClicked")
        (setq bGood T)
        (if
          (= (Odcl_Tab_GetCurSel Walls_ModOpens_TabControl1)
             0
          )
           (progn
             (Setq bSteel (Odcl_Control_GetProperty
                    Walls_ModOpens_SteelOption
                    "Value"
                  )
             )
             (if (= bSteel T)
           (setq jambprefix "S"
                 ajambtype  "Steel Jamb"
                 sJambTyp   "S"
           )
             )
             (Setq bTimb (Odcl_Control_GetProperty
                   Walls_ModOpens_TimberOption
                   "Value"
                 )
             )
             (if (= bTimb T)
           (setq jambprefix "T"
                 ajambtype  "Timber Jamb"
                 sJambTyp   "T"
           )
             )
             (Setq bJbThk (Odcl_Control_GetProperty
                    Walls_ModOpens_JambThkCheckBox
                    "Value"
                  )
             )
             (Setq sTr3a (Odcl_ComboBox_GetTBText             ;get displayed width
                   Walls_ModOpens_DrWidthComboBox
                 )
             )

             (Setq
           bCenter (Odcl_Control_GetProperty
                 Walls_ModOpens_CenteredCheckBox
                 "Value"
               )
             )

             (Setq bHeader (Odcl_Control_GetProperty
                     Walls_ModOpens_HeaderCheckBox
                     "Value"
                   )
             )
             (if (= bHeader T)
           (progn (Setq bThreshold nil
                    sHeaderLyr (Odcl_ComboBox_GetTBText
                         Walls_ModOpens_HeaderComboBox
                       )
              )
              (Hshset_lay
                sHeaderLyr
                sHeaderCol
                "ACAD_ISO02W100"
                sHeaderLwt
              )
           )
             )

             (Setq bThreshold
                (Odcl_Control_GetProperty
              Walls_ModOpens_ThresholdCheckBox
              "Value"
                )
             )
             (if (= bThreshold T)
           (progn (Setq bHeader nil
                    sThreshLyr
                     (Odcl_ComboBox_GetTBText
                   Walls_ModOpens_ThresholdComboBox
                     )
              )
              (Hshset_lay
                sThreshLyr
                sThreshCol
                nil
                sThreshLwt
              )
           )
             )

             ;;       (Odcl_PictureBox_StoreImage Walls_ModOpens_WindowPictureBox)
             ;;   (setq bDisplay T)
             (Odcl_Form_Close Walls_ModOpens)

             (cond ((and (= bJbThk T) (= bCenter nil))
                (setq rJambWidth rMurThk)
               )
               ((and (= bJbThk nil) (= bCenter T))
                (if (< (distance pHinge pCoteO) 130)
              (setq rJambWidth rMurThk
                    rMoveInsPt 0.0
              )
              (setq rJambWidth 130
                    rMoveInsPt (* 0.5 (- rMurThk 130))
              )
                )
               )
               ((and (= bJbThk nil) (= bCenter nil))
                (if (< rMurThk 130)
              (setq rJambWidth rMurThk
                    rMoveInsPt 0.0
              )
              (setq rJambWidth 160
                    rMoveInsPt 0.0
              )
                )
               )
             )

             (setq sOpenAng_c PercOpen
               sOpenAng   PercOpen
               leafwidt_c "30"
               leafwidt   "30"
               OFFDIST    (atoi sTr3a)
             )

            (Setq rPropWid OFFDIST                        ;Proposed width
              rWidDiff (- rCurrWid rPropWid))             ;Current width and difference

            (if (/= rWidDiff 0)
             (progn
              (setq pFar (polar pHinge aA3 rCurrWid)
                pMi  (polar pHinge aA3 (* 0.5 rCurrWid))
          )
          (CrossIt pHinge 150 0.013)                  ;Charniere en bleu
          (CrossIt pMi 22 0.013)                      ;milieu ou rouge
          (CrossIt pFar 61 0.013)                     ;oppose ou jaune
          (setq sChoix (ukword 1 "Blue Red Yellow" "Change of width noted! Spread from the Blue cross (current hinge pt)\nor from [Red/Yellow] cross? " "Blue"))
             )
            )

            (cond
             ((= sChoix "Red")
          (setq pNCharn (polar pHinge aA3 (* 0.5 rWidDiff))
                p1      pNCharn
                pTmpOpp (polar p3 aA3 (* 0.5 rWidDiff))
                p3      pTmpOpp
          )
             )
             ((= sChoix "Yellow")
          (setq pNCharn (polar pHinge aA3 rWidDiff)
                p1      pNCharn
                pTmpOpp (polar p3 aA3 rWidDiff)
                p3      pTmpOpp
          )
             )
            )

             (setq p2      (polar p1 aA3 rPropWid)
               OLine3  (ssname (ssget pHinge) 0)
               LayerIs (vla-get-layer (vlax-ename->vla-object OLine3))    ;Nom du calque pour l'objet
               lname   LayerIs
               sRHing  nil
             )

             (Emprise)                            ;Remove door then build a new one

             (setq OLine3  (ssname (ssget pHinge) 0)
               LayerIs (vla-get-layer (vlax-ename->vla-object OLine3))    ;Nom du calque pour l'objet
               lname   LayerIs
               sRHing  nil
             )

             (LineMake p1 p3 LayerIs)
             (setq oCutEdg1 (entlast))
             (SetJambInfoTag sAppNameAA oCutEdg1)

             (setq pP3 (Polar p2 aA1 rMurThk))
             (LineMake p2 pP3 LayerIs)
             (setq oCutEdg2 (entlast))
             (SetJambInfoTag sAppNameAA oCutEdg2)

             (setq ssTrim1 (ssadd))
             (ssadd oCutEdg1 ssTrim1)
             (ssadd oCutEdg2 ssTrim1)
             (setq trimpt1 (polar p1 aA3 (* 0.5 rPropWid))            ;rPropWid=OFFSET
               trimpt2 (polar p3 aA3 (* 0.5 rPropWid))
               pTr1    (polar trimpt1 (+ Pi aA1) 2.0)
               pTr2    (polar trimpt2 aA1 2.0)
             )
             (command "_.trim" ssTrim1 "" "_F" pTr1 pTr2 "" "")

      ;;(Hshset_lay "DOOR" nil "CONTINUOUS" nil)
      ;;       (setq pDoorIns (polar p1 aA1 rMoveInsPt))
             (setq pDoorIns p1)

      ;;;Portes
             (cond ((= sTr2 "S")     (SingleMk))              ;Single
               ((= sTr2 "D")     (DoubleMk))              ;Double
               ((= sTr2 "UD")    (UnDblMk))               ;Unequal double
               ((= sTr2 "BD")    (BifoldDMk))             ;AA1.3b100
               ((= sTr2 "BS")    (BifoldSMk))             ;Concertina
               ((= sTr2 "W")     (OpenMk))                ;Doorway
               ((= sTr2 "SFd")   (SldFixMkD))
               ((= sTr2 "FSFd")  (FixSldFixMk))               ;AA1.3b100
               ((= sTr2 "FSSFd") (FixSldSldFixMk))            ;AA1.3b100
               ((= sTr2 "SSS")   (SurfSlidSMk))
               ((= sTr2 "SSD")   (SurfSlidDMk))               ;AA1.3b100
               ((= sTr2 "CSD")   (CavSlidDMk))                ;AA1.3b100
               ((= sTr2 "CSS")   (CavSlidSMk))
             )

             (Hshset_lay
           (dxf 8 lODeLyr)
           nil
           "CONTINUOUS"
           nil
             )

             (entmake (list (cons 0 "INSERT")
                    (cons 2 BlockNameD)
                    (cons 10 pDoorIns)
                    (cons 41 xFactor)
                    (cons 42 YFactor)
                    (cons 50 aRot)
              )
             )

             (SetDoorInfoTag sAppNameAA (entlast) nil)

             (if (< rMurThk 230)
           (setq bThreshold nil)
             )
             (if (= bThreshold T)
           (progn (ThresholdMk)
              (entmake (list (cons 0 "INSERT")
                         (cons 2 BlockNameD2)
                         (cons 10 pHinge)
                         (cons 41 XFactor)
                         (cons 42 YFactor)
                         (cons 50 aA3)
                   )
              )
              ;;                 (setq rSpace2 (/ OffDist (fix (/ OffDist 100)) 1.0))
              ;;       (setvar "snapbase" (list (car pDoorIns) (cadr pDoorIns)))
              ;;       (Hshset_lay sThreshLyr "23" nil "0.25")
              ;;       (command ".-BHATCH" "_P" "_U" (rtd aA1) rSpace2 "_N" "_S" (entlast) "" "")
           )
             )
           )

           (progn
             (Setq sTrW3a
                (Odcl_ComboBox_GetTBText Walls_ModOpens_WnWidthComboBox))         ;get displayed width
             (Setq sTr4b
                (Odcl_ComboBox_GetTBText Walls_ModOpens_WnHeightComboBox))        ;get displayed height

             (Setq bJbThk nil)
             (Setq bCenter nil)

             (Setq bSillOut (Odcl_Control_GetProperty
                  Walls_ModOpens_OutlineOption
                  "Value"
                    )
             )
             (Setq bSillHat (Odcl_Control_GetProperty
                  Walls_ModOpens_HatchSillOption
                  "Value"
                    )
             )

          (Hshset_lay sGlazeLyr sGlazeCol "CONTINUOUS" sGlazeLwt)
          (Hshset_lay sFrameLyr sFrameCol "CONTINUOUS" sFrameLwt)
          (Hshset_lay sSashLyr sSashCol "CONTINUOUS" sSashLwt)

             (Setq bHeaderW (Odcl_Control_GetProperty
                  Walls_ModOpens_WnHeaderCheckBox
                  "Value"
                    )
             )
             (if (= bHeaderW T)
           (progn (Setq bSill nil
                    sWHeaderLyr
                     (Odcl_ComboBox_GetTBText
                   Walls_ModOpens_WnHeaderComboBox
                     )
              )
              (Hshset_lay
                sWHeaderLyr
                sWHeaderCol
                "ACAD_ISO02W100"
                sWHeaderLwt
              )
           )
             )

             (Setq bSill (Odcl_Control_GetProperty
                   Walls_ModOpens_WnSillCheckBox
                   "Value"
                 )
             )
             (if (= bSill T)
           (progn (Setq bHeaderW nil
                    sSillLyr (Odcl_ComboBox_GetTBText
                       Walls_ModOpens_WnSillComboBox
                         )
              )
              (Hshset_lay sSillLyr sSillCol nil sSillLwt)
           )
             )

                          ;if height less than 1600 and transom asked, deny.
             (Setq bTransom (Odcl_Control_GetProperty
                  Walls_ModOpens_TransomCheckBox
                  "Value"
                    )
             )
             (if (and (= bTransom T) (< (atoi sTr4b) (atof sSeuil4)))
           (setq bTransom nil)
             )

             (setq PercOpen "90")

             ;;       (Odcl_PictureBox_StoreImage Walls_ModOpens_WindowPictureBox)
             ;;   (setq bDisplay T)
             (Odcl_Form_Close Walls_ModOpens)

             (cond ((and (= bJbThk T) (= bCenter nil))
                (setq rJambWidth rMurThk)
               )
               ((and (= bJbThk nil) (= bCenter T))
                (if (< (distance pHinge pCoteO) 130)
              (setq rJambWidth rMurThk
                    rMoveInsPt 0.0
              )
              (setq rJambWidth 130
                    rMoveInsPt (* 0.5 (- rMurThk 130))
              )
                )
               )
               ((and (= bJbThk nil) (= bCenter nil))
                (if (< rMurThk 130)
              (setq rJambWidth rMurThk
                    rMoveInsPt 0.0
                    bSill      nil
              )
              (setq rJambWidth 160
                    rMoveInsPt 0.0
              )
                )
               )
             )

             (setq sOpenAng_c PercOpen
               sOpenAng   PercOpen
               leafwidt_c "30"
               leafwidt   "30"
               LWn    sTr2W
               OFFDIST    (atoi sTrW3a)
             )

            (Setq rPropWid OFFDIST                        ;Proposed width
              rWidDiff (- rCurrWid rPropWid))             ;Current width and difference

            (if (/= rWidDiff 0)
             (progn
              (setq pFar (polar pHinge aA3 rCurrWid)
                pMi  (polar pHinge aA3 (* 0.5 rCurrWid))
          )
          (CrossIt pHinge 150 0.013)                  ;Charniere en bleu
          (CrossIt pMi 22 0.013)                      ;milieu ou rouge
          (CrossIt pFar 61 0.013)                     ;oppose ou jaune
          (setq sChoix (ukword 1 "Blue Red Yellow" "Change of width noted! Spread from the Blue cross (current insert pt)\nor from [Red/Yellow] cross? " "Blue"))
             )
            )

            (cond
             ((= sChoix "Red")
          (setq pNCharn (polar pHinge aA3 (* 0.5 rWidDiff))
                p1      pNCharn
                pTmpOpp (polar p3 aA3 (* 0.5 rWidDiff))
                p3      pTmpOpp
          )
             )
             ((= sChoix "Yellow")
          (setq pNCharn (polar pHinge aA3 rWidDiff)
                p1      pNCharn
                pTmpOpp (polar p3 aA3 rWidDiff)
                p3      pTmpOpp
          )
             )
            )

             (setq p2      (polar p1 aA3 rPropWid)
               OLine3  (ssname (ssget p3) 0)
               LayerIs (vla-get-layer (vlax-ename->vla-object OLine3))    ;Nom du calque pour l'objet
               lname   LayerIs
               sRHing  nil
             )

             (Emprise)                            ;Remove door then build a new one

             (setq OLine3  (ssname (ssget p3) 0)
               LayerIs (vla-get-layer (vlax-ename->vla-object OLine3))    ;Nom du calque pour l'objet
               lname   LayerIs
               sRHing  nil
             )

             (LineMake p1 p3 LayerIs)
             (setq oCutEdg1 (entlast))
             (SetJambInfoTag sAppNameAA oCutEdg1)

             (setq pP3 (Polar p2 aA1 rMurThk))
             (LineMake p2 pP3 LayerIs)
             (setq oCutEdg2 (entlast))
             (SetJambInfoTag sAppNameAA oCutEdg2)

             (setq ssTrim1 (ssadd))
             (ssadd oCutEdg1 ssTrim1)
             (ssadd oCutEdg2 ssTrim1)
             (setq trimpt1 (polar p1 aA3 (* 0.5 rPropWid))        ;rPropWid=OFFSET
               trimpt2 (polar p3 aA3 (* 0.5 rPropWid))
               pTr1    (polar trimpt1 (+ Pi aA1) 2.0)
               pTr2    (polar trimpt2 aA1 2.0)
             )
             (command "_.trim" ssTrim1 "" "_F" pTr1 pTr2 "" "")

      ;;       (setq pWindIns (polar pHinge aA1 rMoveInsPt))
             (setq pWindIns p1)

             (cond ((= LWn "FW")  (FixedMk))              ;Fenetres
               ((= LWn "SF")  (SldFixMk))
               ((= LWn "SFS") (SldFixSldMk))
               ((= LWn "DH")  (DblHungMk))
             )

             ;;   (if bCurvWall
             ;;    (setq A0 (angle p1 p2))
             ;;        )

             (entmake (list (cons 0 "INSERT")
                    (cons 2 BlockNameW)
                    (cons 10 pWindIns)
                    (cons 41 XFactor)
                    (cons 42 YFactor)
                    (cons 50 aRot)
              )
             )

             (SetWindInfoTag sAppNameAA (entlast) nil)

           )
        )
      )

      (defun c:ModOpens_DoorListBox_OnSelChanged (nSelection sSelText /)
        ;;      (Odcl_MessageBox "To Do: code must be added to DoorListBox" (etos sSelText))
        (cond ((= sSelText "Single hung")
           (setq sDrTyp "Single hung"
                 str2   "S"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             T
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth1
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
      ;;         (if (= (member sTr3a lDoorWidth1) nil)
      ;;       (setq sTr3a (nth 1 lDoorWidth1))
      ;;   )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Double door")
           (setq sDrTyp "Double door"
                 str2   "D"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             T
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth2
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Double door unequal")
           (setq sDrTyp "Double door unequal"
                 str2   "UD"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             T
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth2
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Bifold double")
           (setq sDrTyp "Bifold double"
                 str2   "BD"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             T
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth2
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Bifold single")
           (setq sDrTyp "Bifold single"
                 str2   "BS"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             T
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             T
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth3
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Surface sliding double")
           (setq sDrTyp "Surface sliding double"
                 str2   "SSD"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth2
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Surface sliding single")
           (setq sDrTyp "Surface sliding single"
                 str2   "SSS"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth1
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Cavity sliding double")
           (setq sDrTyp "Cavity sliding double"
                 str2   "CSD"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth2
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Cavity sliding single")
           (setq sDrTyp "Cavity sliding single"
                 str2   "CSS"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth1
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Doorway")             ;AABuild108
           (setq sDrTyp "Doorway"
                 str2   "W"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth2
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Sliding [Sliding-Fixed]")
           (setq sDrTyp "Sliding [Sliding-Fixed]"
                 str2   "SFd"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDoorWidth1s
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTrW3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Sliding [F-S-F]")
           (setq sDrTyp "Sliding [F-S-F]"
                 str2   "FSFd"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDFSFWidth1
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
      ;;         (if (= (member sTr3a lDFSFWidth1) nil)   ;;;>>>***
      ;;       (setq sTr3a (nth 0 lDFSFWidth1))
      ;;   )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

          ((= sSelText "Sliding [F-S-S-F]")
           (setq sDrTyp "Sliding [F-S-S-F]"
                 str2   "FSSFd"
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_SteelOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_TimberOption
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Enabled"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_JambThkCheckBox
             "Value"
             nil
           )
           (Odcl_Control_SetProperty
             Walls_ModOpens_CenteredCheckBox
             "Value"
             nil
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_DrWidthComboBox
           )
           (foreach item lDFSSFWidth1
             (Odcl_ComboBox_AddString
               Walls_ModOpens_DrWidthComboBox
               item
             )
           )
      ;;         (if (= (member sTr3a lDFSSFWidth1) nil)  ;;;>>>***
      ;;       (setq sTr3a (nth 0 lDFSSFWidth1))
      ;;   )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_DrWidthComboBox
             sTr3a
           )
           (ShowDoorM)
          )

        )
      )

      (defun c:ModOpens_HeaderCheckBox_OnClicked (nValue /)
        (if (= nValue 1)
          (progn ;;(Odcl_Control_SetProperty
             ;; Walls_ModOpens_HeaderComboBox
             ;;  "Enabled"
             ;;  T
             ;;)
             (Odcl_Control_SetProperty
               Walls_ModOpens_ThresholdCheckBox
               "Value"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_ThresholdComboBox
               "Enabled"
               nil
             )
          )
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_HeaderComboBox
               "Enabled"
               nil
             )
          )
        )
        (ShowDoorM)
      )

      (defun c:ModOpens_ThresholdCheckBox_OnClicked (nValue /)
        (if (= nValue 1)
          (progn ;;(Odcl_Control_SetProperty
             ;;  Walls_ModOpens_ThresholdComboBox
             ;;  "Enabled"
             ;;  T
             ;;)
             (Odcl_Control_SetProperty
               Walls_ModOpens_HeaderCheckBox
               "Value"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_HeaderComboBox
               "Enabled"
               nil
             )
          )
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_ThresholdComboBox
               "Enabled"
               nil
             )
          )
        )
        (ShowDoorM)
      )

      (defun c:ModOpens_SteelOption_OnClicked (nValue /)
        (ShowDoorM)
      )

      (defun c:ModOpens_TimberOption_OnClicked (nValue /)
        (ShowDoorM)
      )

      (defun c:ModOpens_JambThkCheckBox_OnClicked (nValue /)
        (Setq bThkOn nValue)
        (if (= bThkOn 1)
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_CenteredCheckBox
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_CenteredCheckBox
               "Value"
               nil
             )
          )
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_CenteredCheckBox
               "Enabled"
               T
             )
          )
        )
        (ShowDoorM)
      )

      (defun c:ModOpens_CenteredCheckBox_OnClicked (nValue /)
        (Setq bCenter nValue)
        (if (= bCenter 1)
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_JambThkCheckBox
               "Value"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_JambThkCheckBox
               "Enabled"
               nil
             )
          )
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_JambThkCheckBox
               "Enabled"
               T
             )
          )
        )
        (ShowDoorM)
      )

      (defun c:ModOpens_WindListBox_OnSelChanged (nSelection sSelText /)
        (TraceIsOn ".Openings_WindListBox_OnSelChanged")
        (cond ((= sSelText "Fixed Panels")
           (setq sWnTyp "Fixed Panels"
                 sTr2W  "FW"
           )              ;Single
           ;;(Odcl_Control_SetProperty Walls_ModOpens_WnJambThkCheckBox "Enabled" T)
           ;;(Odcl_Control_SetProperty Walls_ModOpens_WnCenteredCheckBox "Enabled" T)
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnWidthComboBox
           )
           (Odcl_ListBox_SelectString
             Walls_ModOpens_WindListBox
             sWnTyp
           )
           (foreach item lFxdWWidth
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnWidthComboBox
             sTrW3a
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnHeightComboBox
           )
           (foreach item lFxdWHeight
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnHeightComboBox
             sTr4b
           )
           (ShowWindM)
          )
          ((= sSelText "Sliding [Sliding-Fixed]")
           (setq sWnTyp "Sliding [Sliding-Fixed]"
                 sTr2W  "SF"
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnWidthComboBox
           )
           (Odcl_ListBox_SelectString
             Walls_ModOpens_WindListBox
             sWnTyp
           )
           (foreach item lSLdWWidth
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnWidthComboBox
             sTrW3a
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnHeightComboBox
           )
           (foreach item lSLdWHeight
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnHeightComboBox
             sTr4b
           )
           (ShowWindM)
          )
          ((= sSelText "Sliding [S-F-S]")
           (setq sWnTyp "Sliding [S-F-S]"
                 sTr2W  "SFS"
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnWidthComboBox
           )
           (Odcl_ListBox_SelectString
             Walls_ModOpens_WindListBox
             sWnTyp
           )
           (foreach item lSLdWWidt3
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnWidthComboBox
             sTrW3a
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnHeightComboBox
           )
           (foreach item lSLdWHeight
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnHeightComboBox
             sTr4b
           )
           (ShowWindM)
          )

          ((= sSelText "Double Hung")
           (setq sWnTyp "Double Hung"
                 sTr2W  "DH"
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnWidthComboBox
           )
           (Odcl_ListBox_SelectString
             Walls_ModOpens_WindListBox
             sWnTyp
           )
           (foreach item lDHgWWidth
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnWidthComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnWidthComboBox
             sTrW3a
           )
           (Odcl_ComboBox_Clear
             Walls_ModOpens_WnHeightComboBox
           )
           (foreach item lDHgWHeight
             (Odcl_ComboBox_AddString
               Walls_ModOpens_WnHeightComboBox
               item
             )
           )
           (Odcl_ComboBox_SelectString
             Walls_ModOpens_WnHeightComboBox
             sTr4b
           )
           (ShowWindM)
          )

        )
      )

      (defun c:ModOpens_WnHeaderCheckBox_OnClicked (nValue /)
        (if (= nValue 1)
          (progn ;;(Odcl_Control_SetProperty
             ;; Walls_ModOpens_WnHeaderComboBox
             ;;  "Enabled"
             ;;  T
             ;;)
             (Odcl_Control_SetProperty
               Walls_ModOpens_WnSillCheckBox
               "Value"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_WnSillComboBox
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_OutlineOption
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_HatchSillOption
               "Enabled"
               nil
             )
          )
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_WnHeaderComboBox
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_OutlineOption
               "Enabled"
               T
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_HatchSillOption
               "Enabled"
               T
             )
          )
        )
        (ShowWindM)
      )

      (defun c:ModOpens_TransomCheckBox_OnClicked (nValue /)
        (if (= nValue 1)
          (setq bTransom T)
          (setq bTransom nil)
        )
        (ShowWindM)
      )

      (defun c:ModOpens_WnSillCheckBox_OnClicked (nValue /)
        (if (= nValue 1)
          (progn ;;(Odcl_Control_SetProperty
             ;;  Walls_ModOpens_WnSillComboBox
             ;;  "Enabled"
             ;;  T
             ;;)
             (Odcl_Control_SetProperty
               Walls_ModOpens_WnHeaderCheckBox
               "Value"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_WnHeaderComboBox
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_OutlineOption
               "Enabled"
               T
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_HatchSillOption
               "Enabled"
               T
             )
          )
          (progn (Odcl_Control_SetProperty
               Walls_ModOpens_WnSillComboBox
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_OutlineOption
               "Enabled"
               nil
             )
             (Odcl_Control_SetProperty
               Walls_ModOpens_HatchSillOption
               "Enabled"
               nil
             )
          )
        )
        (ShowWindM)
      )

      (defun c:ModOpens_OutlineOption_OnClicked (nValue /)
        (ShowWindM)
      )

      (defun c:ModOpens_HatchSillOption_OnClicked (nValue /)
        (ShowWindM)
      )

      (defun c:COP () (ChangeOuvert) (princ))

      (defun ChangeOuvert (/ eObListe eObjName)
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (setq OLDERROR *error*
          *error*  Wallserror
          bGood    nil
        )
        (WsFlashMess2
          (strcat "\nChange Opening Properties." copyright)
          (strcat "\nChange Opening Properties." copyright)
          (strcat "\nChange Opening Properties." copyright)
          (strcat "\nChange Opening Properties." copyright)
        )

        (GetLayers)
        (GetDrINI)
        (GetWnINI)

        (setvar "ATTDIA" 0)
      ;;;(MontreAA3)

        (princ "\nSelect a door or window to modify: ")
        (setq eObjName (ssget "+.:S" '((0 . "INSERT"))))

        (if (/= nil eObjName)
          (progn
            (if (/= nil (AAXDataGet (ssname eObjName 0) sAppNameAA))
          (progn
            (setq sPorteOuiNon
               (cdr
                 (nth
                   0
                   (AAXDataGet (ssname eObjName 0) sAppNameAA)
                 )
               )
            )             ;Une porte? Vue en plan?

            (if (= sPorteOuiNon "DOORPL")
              (progn
                (setq ODeNom     (ssname eObjName 0)          ;;(car eObjName)
                  rJambWidth (read
                       (cdr (nth 1 (AAXDataGet ODeNom sAppNameAA))
                       )
                         )    ;Jamb Width
                  sDrHan2    (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))             ;Database handle
                  LDR        (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))             ;Door type
                  jambprefix (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA)))             ;Jamb type
                  OFFDIST    (read
                       (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA)))      )            ;Door width

                      rCurrWid   OFFDIST

                  p1         (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))             ;Hinge
                  pMCote     (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))             ;meme cote
                  pCoteO     (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))             ;cote oppose
                  PercOpen   (cdr (nth 9 (AAXDataGet ODeNom sAppNameAA)))             ;opening angle
                  rMurThk    (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))                ;Wall thk

                  Headr      (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA)))                ;Header present
                  Threshld   (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA)))                ;Threshold/Sill present?

                  sTofIns    (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))                ;Inseree a quelle heure?
                  aA1        (angle p1 pCoteO)
                  aA3        (angle p1 pMCote)
                  A1         aA1
                  A0         aA3
                  ODrNam     (dxf -1 (entget ODeNom))
                  pCnr1      (list 0 0 0)
                  oBlNam     (dxf 2 (entget ODeNom))
                  pHinge     (dxf 10 (entget ODeNom))
                  xFactor    (dxf 41 (entget ODeNom))
                  yFactor    (dxf 42 (entget ODeNom))
                  aRot       (dxf 50 (entget ODeNom))
                  sTr2       LDR
                  sTr3a      (rtos Offdist)
                  sDrHand    sDrHan2
                  p2         pMCote
                  p3         pCoteO
                  bEfface    nil
                  sJambTyp   jambprefix
                  sJx        jambprefix
                )

                (defun c:ModOpens_OnInitialize ()
                (TraceIsOn ".c:ModOpens_OnInitializeDoors")
                (Odcl_Tab_SetCurSel Walls_ModOpens_TabControl1 0)
                  (cond
                    ((= sTr2 "S")     (setq sDrTyp "Single hung"))
                    ((= sTr2 "D")     (setq sDrTyp "Double door"))
                    ((= sTr2 "W")     (setq sDrTyp "Doorway"))
                    ((= sTr2 "UD")    (setq sDrTyp "Double door unequal"))
                    ((= sTr2 "BD")    (setq sDrTyp "Bifold double"))              ;AA1.3b100
                    ((= sTr2 "BS")    (setq sDrTyp "Bifold single"))
                        ((= sTr2 "SFd")   (setq sDrTyp "Sliding [S-F]"))              ;AABuild108
                        ((= sTr2 "FSFd")  (setq sDrTyp "Sliding [F-S-F]"))            ;AA1.3b100
                        ((= sTr2 "FSSFd") (setq sDrTyp "Sliding [F-S-S-F]"))          ;AA1.3b100
                            ((= sTr2 "SSD")   (setq sDrTyp "Surface sliding double"))
                    ((= sTr2 "SSS")   (setq sDrTyp "Surface sliding single"))
                            ((= sTr2 "CSD")   (setq sDrTyp "Cavity sliding double"))
                    ((= sTr2 "CSS")   (setq sDrTyp "Cavity sliding single"))
                  )
                  (Setq iOpIndx (Odcl_ListBox_SelectString
                          Walls_ModOpens_DoorListBox
                          sDrTyp
                        )
                  )
                  (Odcl_ListBox_SetCurSel
                    Walls_ModOpens_DoorListBox
                    iOpIndx
                  )

                  (ModOuvert)

                  (if (= Headr "T")
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_HeaderCheckBox
                    "Enabled"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_HeaderCheckBox
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_ThresholdCheckBox
                    "Value"
                    nil
                  )
                  ;;      (Odcl_Control_SetProperty
                  ;;        Walls_ModOpens_ThresholdComboBox
                  ;;        "Enabled"
                  ;;        nil
                  ;;      )
                    )
                    (progn
                  ;;      (Odcl_Control_SetProperty
                  ;;        Walls_ModOpens_HeaderComboBox
                  ;;        "Enabled"
                  ;;        nil
                  ;;      )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_HeaderCheckBox
                    "Value"
                    nil
                  )
                    )
                  )
                  (setq bHeader (read Headr))

                  (if (= Threshld "T")
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_ThresholdCheckBox
                    "Enabled"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_ThresholdCheckBox
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_HeaderCheckBox
                    "Value"
                    nil
                  )
                  ;;      (Odcl_Control_SetProperty
                  ;;        Walls_ModOpens_HeaderComboBox
                  ;;        "Enabled"
                  ;;        nil
                  ;;      )
                    )
                    (progn
                  ;;      (Odcl_Control_SetProperty
                  ;;        Walls_ModOpens_ThresholdComboBox
                  ;;        "Enabled"
                  ;;        nil
                  ;;      )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_ThresholdCheckBox
                    "Value"
                    nil
                  )
                    )
                  )
                  (setq bThreshold (read Threshld))

                  (if (= jambprefix "S")
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_SteelOption
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_TimberOption
                    "Value"
                    nil
                  )
                    )
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_SteelOption
                    "Value"
                    nil
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_TimberOption
                    "Value"
                    T
                  )
                    )
                  )

                  (if (= rMurThk rJambWidth)
                    (Odcl_Control_SetProperty
                  Walls_ModOpens_JambThkCheckBox
                  "Value"
                  T
                    )         ;Use wall thk
                    (Odcl_Control_SetProperty
                  Walls_ModOpens_JambThkCheckBox
                  "Value"
                  nil
                    )         ;
                  )

                  (Odcl_Control_SetProperty
                    Walls_ModOpens_CenteredCheckBox
                    "Value"
                    nil
                  )           ;Jamb centered

                )

                (Odcl_Form_Show Walls_ModOpens)
              )

              (progn
                (setq ODeNom     (ssname eObjName 0)          ;;(car eObjName)
                  rJambWidth (read
                       (cdr (nth 1 (AAXDataGet ODeNom sAppNameAA))
                       )
                         )    ;Jamb Width
                  sWnHand    (cdr (nth 2 (AAXDataGet ODeNom sAppNameAA)))             ;Database handle
                  LDR        (cdr (nth 3 (AAXDataGet ODeNom sAppNameAA)))             ;Type de fenetre
                  WinWid     (read (cdr (nth 4 (AAXDataGet ODeNom sAppNameAA))) )         ;Win height
                  OFFDIST    (read
                       (cdr (nth 5 (AAXDataGet ODeNom sAppNameAA))) )             ;Win width

                      rCurrWid   OFFDIST

                  p1         (cdr (nth 6 (AAXDataGet ODeNom sAppNameAA)))             ;Hinge
                  pMCote     (cdr (nth 7 (AAXDataGet ODeNom sAppNameAA)))             ;meme cote
                  pCoteO     (cdr (nth 8 (AAXDataGet ODeNom sAppNameAA)))             ;cote oppose
                  PercOpen   (cdr (nth 9 (AAXDataGet ODeNom sAppNameAA)))             ;opening angle
                  rMurThk    (cdr (nth 10 (AAXDataGet ODeNom sAppNameAA)))                ;Wall thk

                  HeadrW     (cdr (nth 11 (AAXDataGet ODeNom sAppNameAA)))                ;Header present
                  Sill       (cdr (nth 12 (AAXDataGet ODeNom sAppNameAA)))                ;Sill present? Hatch or outline?

                  bTransom   (cdr (nth 13 (AAXDataGet ODeNom sAppNameAA)))                ;Transom present?
                  sSeuil4    (cdr (nth 14 (AAXDataGet ODeNom sAppNameAA)))                ;Add Transom if height is

                  sTofIns    (cdr (nth 15 (AAXDataGet ODeNom sAppNameAA)))                ;Inseree a quelle heure?
                  aA1        (angle p1 pCoteO)
                  aA3        (angle p1 pMCote)
                  A1         aA1
                  A0         aA3
                  ODrNam     (dxf -1 (entget ODeNom))
                  pCnr1      (list 0 0 0)
                  oBlNam     (dxf 2 (entget ODeNom))
                  pHinge     (dxf 10 (entget ODeNom))
                  xFactor    (dxf 41 (entget ODeNom))
                  yFactor    (dxf 42 (entget ODeNom))
                  aRot       (dxf 50 (entget ODeNom))
                  sTr2W      LDR
                  sTrW3a     (rtos Offdist)
                  sTr4b      (rtos WinWid)
                  sSb        sTr4b
                  sDrHand    sDrHan2
                  p2         pMCote
                  p3         pCoteO
                  bEfface    nil
      ;;          sJambTyp   jambprefix
                  TmpHead    HeadrW
                  TmpSill    (car (strfind Sill "/"))
                  TmpSllHa   (cadr (strfind Sill "/"))
                  TmpTrnsom  bTransom
                  sDrHan     sWnHand
                )

                (defun c:ModOpens_OnInitialize ()
                (TraceIsOn ".c:ModOpens_OnInitializeWindows")

                  (cond
                    ((= sTr2W "FW")  (Odcl_Tab_SetCurSel Walls_ModOpens_TabControl1 1) (setq sWnTyp "Fixed Panels"))
                    ((= sTr2W "SF")  (Odcl_Tab_SetCurSel Walls_ModOpens_TabControl1 1) (setq sWnTyp "Sliding [Sliding-Fixed]")) ;Fenetres
                    ((= sTr2W "SFd") (Odcl_Tab_SetCurSel Walls_ModOpens_TabControl1 0) (setq sDrTyp "Sliding [Sliding-Fixed]")) ;Portes
                    ((= sTr2W "SFS") (Odcl_Tab_SetCurSel Walls_ModOpens_TabControl1 1) (setq sWnTyp "Sliding [S-F-S]"))
                    ((= sTr2W "DH")  (Odcl_Tab_SetCurSel Walls_ModOpens_TabControl1 1) (setq sWnTyp "Double Hung"))

                  )

                  (if (= sTr2W "SFd")
                   (progn
                    (Setq iOpIndx (Odcl_ListBox_SelectString
                            Walls_ModOpens_DoorListBox
                            sDrTyp
                          )
                    )
                    (Odcl_ListBox_SetCurSel
                      Walls_ModOpens_DoorListBox
                      iOpIndx
                    )
                   )
                   (progn
                    (Setq iOpIndx (Odcl_ListBox_SelectString
                            Walls_ModOpens_WindListBox
                            sWnTyp
                          )
                    )
                    (Odcl_ListBox_SetCurSel
                      Walls_ModOpens_WindListBox
                      iOpIndx
                    )
                   )
                  )

                  (ModOuvert)

                  (if (= TmpTrnsom "T")
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_TransomCheckBox
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_TransomCheckBox
                    "Value"
                    nil
                  )
                  )
                  (if (= TmpTrnsom "T")
                    (setq bTransom (read TmpTrnsom))
                  )

                  (if (= TmpHead "T")
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnHeaderCheckBox
                    "Enabled"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnHeaderCheckBox
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnSillCheckBox
                    "Value"
                    nil
                  )
                    )
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnHeaderCheckBox
                    "Value"
                    nil
                  )
                    )
                  )
                  (if (= TmpHead "T")
                    (setq bHeaderW (read TmpHead)  bHeader (read TmpHead))
                  )

                  (if (= TmpSill "T")
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnSillCheckBox
                    "Enabled"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnSillCheckBox
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnHeaderCheckBox
                    "Value"
                    nil
                  )
                    )
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_WnSillCheckBox
                    "Value"
                    nil
                  )
                    )
                  )
                  (if (= TmpSill "T")
                    (setq bSill (read TmpSill))
                  )

                  (if (= TmpSllHa "T")                ;
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_OutlineOption
                    "Value"
                    nil
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_HatchSillOption
                    "Value"
                    T
                  )
                    )
                    (progn
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_OutlineOption
                    "Value"
                    T
                  )
                  (Odcl_Control_SetProperty
                    Walls_ModOpens_HatchSillOption
                    "Value"
                    nil
                  )
                    )
                  )
                  (if (= TmpSllHa "T")
                    (setq bSillHat (read TmpSllHa))
                  )

                )

                (Odcl_Form_Show Walls_ModOpens)
              )
            )
          )
            )
          )
        )

        (if (= bGood nil) (princ "\nSelected object not a door/window object drawn in plan view. "))

        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      ;--------------------------
      ;---Emprise de l'ouverture.                     sDrHan2 changed to sDrHan
      ;--------------------------
      (defun Emprise ( / pC1 pC1a pC2a pC2 )
        (TraceIsOn ".Emprise")
        (command "_.UCS" "_W")
        (setvar "osmode" 0)

        (setq aDiff1  (- aA1 aA3)
          pYAxis  (polar pHinge (+ Pi aA1) rMurThk)
        )
        (command "_.UCS" "_3POINT" pHinge pMCote pYAxis)

        (if (minusp yFactor)
          (if (minusp xFactor)
            (setq sOuvreTo "R")
            (setq sOuvreTo "L")
          )
          (if (minusp xFactor)
            (setq sOuvreTo "L")
            (setq sOuvreTo "R")
          )
        )

        (if (= sOuvreTo "L")
          (setq aCross1 135
            aCross2 0
            aCross3 HshPi270
          )
          (setq aCross1 135
            aCross2 0
            aCross3 HshPi270
          )
        )
        (setq pC1  (polar pCnr1 (dtr aCross1) 70)
          pC1a     (polar pC1 aCross3 (+ rMurThk 99))
          pC2  (polar pC1 aCross2 (+ rCurrWid 99))
          pC2a     (polar pC2 aCross3 (+ rMurThk 99))
          bCont    nil
          bACont   nil
          bPCont   nil
          bLwCont  nil
          eObject1 nil
          eObject2 nil
          lObj1    nil
          lObj2    nil
          BonSens1 nil
          BonSens2 nil
          VxList1  nil
          VxList2  nil
          iCount   0
          lODeLyr  (entget ODeNom)
        )

        (entdel oDeNom)

        (command "_.ERASE" (ssget "_WP" (list pC1 pC1a pC2a pC2)) "")
      ;;  (command "_.PLINE" pC1 pC1a pC2a pC2 "")
        (Cassure "pC1 pC1a pC2a pC2")
                                ;(PointMake (trans pC1 1 0) "1")
                                ;(PointMake (trans pC1a 1 0) "1a")
                                ;(PointMake (trans pC2a 1 0) "2a")
                                ;(PointMake (trans pC2 1 0) "2")

        (setq pBL  (trans pC1a 1 0)             ;0=World (WCS), 1=User (current UCS)
          pTR  (trans pC2 1 0)
          llur (CornersGet pBL pTR)
          pBL  (car llur)
          pTR  (cadr llur)
        )

        (setq ssGardArc1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "ARC")))
          ssGardArc3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "ARC")))
          ssGardLin1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "LINE")))
          ssGardLin3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "LINE")))
          ssGardPln1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "POLYLINE")))
          ssGardPln3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "POLYLINE")))
          ssGardLwP1 (ssget "_F" (list pC1 pC1a) (list (cons 0 "LWPOLYLINE")))
          ssGardLwP3 (ssget "_F" (list pC2 pC2a) (list (cons 0 "LWPOLYLINE")))
        )

        (if (/= ssGardLin1 nil)             ;Pour les lignes
          (setq bCont    T
            iNum4G1  (sslength ssGardLin1)
            iNum     iNum4G1
            ss1Garde ssGardLin1
          )
        )
        (if (/= ssGardArc1 nil)             ;Pour les arcs
          (setq bACont   T
            iNum4G1  (sslength ssGardArc1)
            iNumA    iNum4G1
            ss1Guard ssGardArc1
          )
        )
        (if (/= ssGardPln1 nil)             ;Pour les polylignes
          (setq bPCont   T
            iNum4G1  (sslength ssGardPln1)
            iNumP    iNum4G1
            ss1GardP ssGardPln1
          )
        )
        (if (/= ssGardLwp1 nil)             ;Pour les polylignes legeres
          (setq bLwCont   T
            iNum4G1   (sslength ssGardLwp1)
            iNumLw    iNum4G1
            ss1GardLw ssGardLwp1
          )
        )
        (if (/= ssGardLin3 nil)
          (setq bCont    T
            iNum4G3  (sslength ssGardLin3)
            iNum     iNum4G3
            ss2Garde ssGardLin3
          )
        )
        (if (/= ssGardArc3 nil)             ;Pour les arcs
          (setq bACont   T
            iNum4G3  (sslength ssGardArc3)
            iNumA    iNum4G3
            ss2Guard ssGardArc3
          )
          (setq bACont nil)
        )
        (if (/= ssGardPln3 nil)             ;Pour les polylignes
          (setq bPCont   T
            iNum4G3  (sslength ssGardPln3)
            iNumP    iNum4G3
            ss2GardP ssGardPln3
          )
        )
        (if (/= ssGardLwp3 nil)             ;Pour les polylignes legeres
          (setq bLwCont   T
            iNum4G3   (sslength ssGardLwp3)
            iNumLw    iNum4G3
            ss2GardLw ssGardLwp3
          )
        )
        (if (equal iNum4G1 iNum4G3)
          (setq bCont1 T
            bACont1 T
            bPCont1 T
            bLwCont1 T
            bCont3 T
            bACont3 T
            bPCont3 T
            bLwCont3 T
          )
          (progn (princ "\rError: Walls with unequal number of objects.")
             (setq bCont1 nil
               bACont1 nil
               bPCont1 nil
               bLwCont1 nil
               bCont3 nil
               bACont3 nil
               bPCont3 nil
               bLwCont3 nil
             )
          )
        )
        (command "_.UCS" "_P")
        (if (and (= bCont T) (or (= bCont1 T) (= bCont3 T)))
          (progn
            (While (< iCount iNum)
          (setq eObject1 (ssname ss1Garde iCount)
                eObject2 (ssname ss2Garde iCount)
                lObj1    (entget eObject1)
                lObj2    (entget eObject2)
                pStLn1   (dxf 10 lObj1)         ;depart ligne 1
                pStLn2   (dxf 10 lObj2)         ;depart ligne 2
                pEnLn2   (dxf 11 lObj2)         ;fin ligne 2
          )
          (if (= (Colineaire pStLn2 pStLn1 pEnLn2 nil 5) T)
            (progn
              (entdel eObject2)
              (setvar "ORTHOMODE" 0)
              (if (> (distance pStLn1 pStLn2) (distance pStLn1 pEnLn2))
                (command "_.change" eObject1 "" (trans pStLn2 0 1))
                (command "_.change" eObject1 "" (trans pEnLn2 0 1))
              )
            )
            (princ "\rError: Non-colinear walls.")
          )
          (setq iCount (1+ iCount))
            )
          )
        )
        (if (and (= bACont T) (or (= bACont1 T) (= bACont3 T)))
          (progn (While (< iCount iNumA)
               (setq eObject1 (ssname ss1Guard iCount)
                 eObject2 (ssname ss2Guard iCount)
                 lObj1    (entget eObject1)
                 lObj2    (entget eObject2)
                 pCen1    (dxf 10 lObj1)
                 pCen2    (dxf 10 lObj2)
                 sLtyp    (dxf 6 lObj1)
                 sLyr     (dxf 8 lObj1)
                 iClr     (dxf 62 lObj1)
               )
               (if (equal (distance pCen1 pCen2) 0.0 0.001)
                 (progn (setq lpt1 (polar pCen1 (dxf 50 lObj1) (dxf 40 lObj1))
                      lpt2 (polar pCen1 (dxf 51 lObj1) (dxf 40 lObj1))
                      lpt3 (polar pCen2 (dxf 50 lObj2) (dxf 40 lObj2))
                      lpt4 (polar pCen2 (dxf 51 lObj2) (dxf 40 lObj2))
                       ;;thick (dxf 39 lObj1)
                    )
                    (if (< (distance lpt1 lpt4) (distance lpt2 lpt3))
                  (command "_.arc" lpt3 lpt1 lpt2)
                  (command "_.arc" lpt1 lpt2 lpt4)
                    )
                    (entdel eObject1)
                    (entdel eObject2)
                    (if (= iClr nil)
                  (setq iClr "BYLAYER")
                    )
                    (if (= sLtyp nil)
                  (setq sLtyp "BYLAYER")
                    )
                    (vla-put-layer (vlax-Ename->vla-Object (entlast)) sLyr)
                    (vla-put-color (vlax-Ename->vla-Object (entlast)) iClr)
                    (vla-put-linetype (vlax-Ename->vla-Object (entlast)) sLtyp)
                 )
                 (progn (if eObject1
                  (redraw eObject1)
                    )
                    (prompt "\nArcs centres must coincide. ")
                 )
               )
               (setq iCount (1+ iCount))
             )
          )
        )
        (if (and (= bPCont T) (or (= bPCont1 T) (= bPCont3 T)))
          (progn
            (While (< iCount iNumP)
          (setq eObject1 (ssname ss1GardP iCount)
                eObject2 (ssname ss2GardP iCount)
                lObj1    (entget eObject1)
                lObj2    (entget eObject2)
                sLyr     (dxf 8 lObj1)
                VxList1  (Liste4Pline eObject1)
                VxList2  (Liste4Pline eObject2)
          )
          (Hshset_lay sLyr nil nil nil)
          (if (equal eObject1 eObject2)         ;si ce sont 2 differentes plines
            (progn                  ;sinon
              (setq VxList1
                 (remove_n (what_nth (first_Vtx eObject1) VxList1)
                       VxList1
                 )
              )
              (setq
                VxList1 (remove_n (what_nth (lastVtx eObject1) VxList1)
                      VxList1
                    )
              )
                                ;            (plinemake VxList1 0 0 nil)
              (command "_.PLINE")
              (foreach pnt VxList1 (command pnt))
              (command "_C")
              (entdel eObject1)
            )
            (progn
              (if (= T (PtInRect (first_Vtx eObject1) pBL pTR))
                (progn
              (setq BonSens1 nil
                    VxList1  (remove_n
                       (what_nth (first_Vtx eObject1) VxList1)
                       VxList1
                         )
              )
              (cassure "1er:BonSens1 (first_Vtx eObject1) VxList1")
                )
                (progn
              (setq BonSens1 T
                    VxList1
                     (remove_n (what_nth (lastVtx eObject1) VxList1)
                       VxList1
                     )
              )
              (cassure "dernier:BonSens1 (lastVtx eObject1) VxList1")
                )
              )
              (if (= T (PtInRect (first_Vtx eObject2) pBL pTR))
                (progn
              (setq BonSens2 T
                    VxList2  (remove_n
                       (what_nth (first_Vtx eObject2) VxList2)
                       VxList2
                         )
              )
              (cassure "1er:BonSens2 (first_Vtx eObject2) VxList2")
                )
                (progn
              (setq BonSens2 nil
                    VxList2
                     (remove_n (what_nth (lastVtx eObject2) VxList2)
                       VxList2
                     )
              )
              (cassure "dernier:BonSens2 (lastVtx eObject2) VxList2")
                )
              )
              (if (= BonSens1 nil)
                (setq VxList1 (reverse VxList1))
              )
              (if (= BonSens2 nil)
                (setq VxList2 (reverse VxList2))
              )
              (plinemake (append VxList1 VxList2) 0 0 nil)
              (entdel eObject1)
              (entdel eObject2)
            )
          )
          (setq iCount (1+ iCount))
            )
          )
        )
        (if (and (= bLwCont T) (or (= bLwCont1 T) (= bLwCont3 T)))
          (progn
            (While (< iCount iNumLw)
          (setq eObject1 (ssname ss1GardLw iCount)
                eObject2 (ssname ss2GardLw iCount)
                lObj1    (entget eObject1)
                lObj2    (entget eObject2)
                sLyr     (dxf 8 lObj1)
                pVSt1    (dxf 10 lObj1)
                pVSt1    (list (car pVSt1) (cadr pVSt1) 0.0)
                pVEn1    (dxf 10 (reverse lObj1))
                pVEn1    (list (car pVEn1) (cadr pVEn1) 0.0)
                pVSt2    (dxf 10 lObj2)
                pVSt2    (list (car pVSt2) (cadr pVSt2) 0.0)
                pVEn2    (dxf 10 (reverse lObj2))
                pVEn2    (list (car pVEn2) (cadr pVEn2) 0.0)
                VxList1  (Liste4LwPl eObject1)
                VxList2  (Liste4LwPl eObject2)
          )
          (Hshset_lay sLyr nil nil nil)
          (if (equal eObject1 eObject2)         ;si ce sont 2 differentes plines
            (progn                  ;sinon
              (setq VxList1 (remove_n (what_nth pVSt1 VxList1) VxList1))
              (setq VxList1 (remove_n (what_nth pVEn1 VxList1) VxList1))
              (command "_.PLINE")
              (foreach pnt VxList1 (command pnt))
              (command "_C")
              (entdel eObject1)
            )
            (progn
              (if (= T (PtInRect pVSt1 pBL pTR))
                (progn
              (setq BonSens1 nil
                    VxList1  (remove_n (what_nth pVSt1 VxList1) VxList1)
              )
              (cassure "1er:BonSens1 pVSt1 VxList1")
                )
                (progn
              (setq BonSens1 T
                    VxList1  (remove_n (what_nth pVEn1 VxList1) VxList1)
              )
              (cassure "dernier:BonSens1 pVEn1 VxList1")
                )
              )
              (if (= T (PtInRect pVSt2 pBL pTR))
                (progn
              (setq BonSens2 T
                    VxList2  (remove_n (what_nth pVSt2 VxList2) VxList2)
              )
              (cassure "1er:BonSens2 pVSt2 VxList2")
                )
                (progn
              (setq BonSens2 nil
                    VxList2  (remove_n (what_nth pVEn2 VxList2) VxList2)
              )
              (cassure "dernier:BonSens2 pVEn2 VxList2")
                )
              )
              (if (= BonSens1 nil)
                (setq VxList1 (reverse VxList1))
              )
              (if (= BonSens2 nil)
                (setq VxList2 (reverse VxList2))
              )
              (plinemake (append VxList1 VxList2) 0 0 T) ;T for LW
              (entdel eObject1)
              (entdel eObject2)
            )
          )
          (setq iCount (1+ iCount))
            )
          )
        )
        (if (= sPorteOuiNon "DOORPL")
          (progn
            (setq ssToKill (SSgetXD (list (cons 0 "INSERT"))
                        (quote (1 "=" "DOOREL"))
                        1
                        "AA_OPENING"
                   )
            )
            (foreach item ssToKill
          (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan)
                   (= (cdr (nth 4 (AAXDataGet (car item) sAppNameAA))) sTofIns))      ;le 4eme champ des ELEV
                (entdel (car item))
              )
            )
          )
        )
        (if (= sPorteOuiNon "WINDPL")
          (progn
            (setq
          ssToKill2 (SSgetXD (list (cons 0 "INSERT"))
                     (quote (1 "=" "WINDEL"))
                     1
                     "AA_OPENING"
                )
            )
            (foreach item ssToKill2
          (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan)
                   (= (cdr (nth 8 (AAXDataGet (car item) sAppNameAA))) sTofIns))      ;le 8eme champ des ELEV
                 (entdel (car item))
               )
            )
            (setq
          ssToKill2 (SSgetXD (list (cons 0 "INSERT"))
                     (quote (1 "=" "WINDSC"))
                     1
                     "AA_OPENING"
                )
            )
            (foreach item ssToKill2
          (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan)
                   (= (cdr (nth 6 (AAXDataGet (car item) sAppNameAA))) sTofIns))      ;le 6eme champ des SECT
                 (entdel (car item))
               )
            )
            (setq
          ssToKill2 (SSgetXD (list (cons 0 "INSERT"))
                     (quote (1 "=" "WINDBR"))
                     1
                     "AA_OPENING"
                )
            )
            (foreach item ssToKill2
          (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan)
                   (= (cdr (nth 2 (AAXDataGet (car item) sAppNameAA))) sTofIns))      ;le 2eme champ des Fed Bars
                 (entdel (car item))
               )
            )
            (setq
          ssToKill2 (SSgetXD (list (cons 0 "INSERT"))
                     (quote (1 "=" "WALLSC"))
                     1
                     "AA_OPENING"
                )
            )
            (foreach item ssToKill2
          (if (and (= (cdr (nth 1 (AAXDataGet (car item) sAppNameAA))) sDrHan)
                   (= (cdr (nth 5 (AAXDataGet (car item) sAppNameAA))) sTofIns))      ;le 5eme champ des SECT mur
                 (entdel (car item))
               )
            )
          )
        )
      )

      ;---------------------------------------------
      ;---Finds full path of any file on hard drive.
      ;---------------------------------------------
      ;Usage:    (FindFilePath "filename.ext")
      ;Examlpe:  Run on command prompt: (FindFilePath "acad.exe")
      ;Return:   "C:\\Program Files\\AutoCAD 2000i\\acad.exe"
      ;Note:     Does not search network drives

      (defun FindFilePath (InputFile     /         PCDrives
                   PCDrive       PCDriveChoice PCDriveLetter
                   PCFolder      PCFolderPath  i
                  )
        (if (/= (vl-string-trim " " InputFile) "")
          (progn
            (setq ReturnFolder nil)
            (setq ContProc T)
            (setq FlSysObj (vlax-create-object "scripting.filesystemobject"))
            (setq PCDrives (vlax-get-property FlSysObj 'Drives))
            (vlax-for PCDrive PCDrives
          (setq PCDriveChoice (vlax-get-property PCDrive 'DriveLetter))
          (if (/= PCDriveChoice "A")
            (setq PCDriveLetter (cons PCDriveChoice PCDriveLetter))
          )               ;if
            )                 ;vlax
            (setq i 0)
            (while (and (<= i (1- (length PCDriveLetter))) ContProc)
          (setq PCDrive (vlax-get-property
                  PCDrives
                  'Item
                  (nth i PCDriveLetter)
                    )
          )
          (if (and
                (= (vlax-get-property PCDrive 'DriveType) 2)
                (= (vlax-get-property PCDrive 'IsReady) :vlax-true)
              )               ;and
            (progn
              (setq PCFolder (vlax-get-property PCDrive 'RootFolder))
              (setq PCFolderPath (vlax-get-property PCFolder 'Path))
              (MainProc InputFile PCFolderPath)
            )             ;progn
          )               ;if
          (setq i (1+ i))
            )                 ;while
            ReturnFolder
          )                   ;progn
        )                 ;if
      )                   ;defun

      (defun MainProc (SearchFile  SearchFolder    /
               PCFolder    PCSubFolders    PCLoopFolder
               PCFolderPath    FullPath
              )
        (if (/= (substr SearchFolder (strlen SearchFolder) 1) "\\")
          (setq SearchFolder (strcat SearchFolder "\\"))
        )                 ;if
        (setq FullPath (strcat SearchFolder SearchFile))
        (if (= (vlax-invoke-method FlSysObj 'FileExists FullPath)
           :vlax-true
            )
          (progn
            (setq ReturnFolder FullPath)
            (setq ContProc nil)
          )                   ;progn
          (progn
            (if
          (not (vl-catch-all-error-p
                 (setq PCFolder (vl-catch-all-apply
                      'vlax-invoke-method
                      (list FlSysObj 'GetFolder SearchFolder)
                        )
                 )
               )
          )
           (progn
             (setq PCSubFolders (vlax-get-property PCFolder 'SubFolders))
             (vlax-for PCLoopFolder PCSubFolders
               (if ContProc
                 (progn
               (setq PCFolderPath
                  (vlax-get-property PCLoopFolder 'Path)
               )
               (MainProc SearchFile PCFolderPath)
                 )            ;progn
               )              ;if
             )                ;vlax
           )              ;progn
            )                 ;if
          )                   ;progn
        )                 ;if
      )                   ;defun

      ;------------------------------
      ;---Increments/decrements text.
      ;------------------------------
      ;;; Replace Mode is under development, intended to replace the existing text
      ;;;   with the number only.
      ;;;   Start value >0, replace the text with the start value, Start = 1
      ;;;     6ABC -> 1   4ABC -> 2  1ABC -> 3
      ;;;   Start value 0, use the number found in the first text as the start value
      ;;;     6ABC -> 6   4ABC -> 7  1ABC -> 8
      ;;;   Example of use, Place a text '1' at the first tread on stairs, use the
      ;;;   array command to populate the remaining treads, then use the Replace
      ;;;   mode to change the ''s to 1 2 3 4  etc.
      ;;;
      ;;;  Limitations
      ;;;  In mtext, if there is a change in the formatting within a number such as
      ;;;   number 123456 with a color change ie 123 in red and 456 in ByLayer the
      ;;;   456 will be ignored and only the 123 altered, the actual string looks
      ;;;   something like this "{\\C1;123}456\\PNew Line"
      ;;;  Also commas break the number into two items 123,456 will be treated as
      ;;;  123 and the 456 ignored
      ;;;
      ;;;  Negative numbers in text are processed only when the increment
      ;;;   value is negative and you approve it at a prompt
      ;;;   If No
      ;;;     "A-1" -2 returns "A-3",  "A1" -2 returns "A-1"
      ;;;   If Yes return numbers will not go below zero
      ;;;     "A-5" -2 returns "A-3",  "A1" -2 returns "A0"
      ;;;
      ;;;  No support for reals or fractions

      (defun c:inkDek (/     ent       lastval   lastent   ss1
               disppoint ignore    nocr      newpos    addmode
               bLoop     newtxt
              )

        (setq olderror *error*
          *error*  INKerror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1) )
         (progn
          (FlashMess3
            "ArchAssist maintenant Actif."
            "ArchAssist now Active."
            "ArchAssist now Active."
            "ArchAssist now Active."
          )
         )
        )
        (princ (strcat "\nIncrement/Decrement text." copyright))

      (defun INKerror (msg#)
        (setq *error* OLDERROR)
        (command)             ;Cancel any pending command and end any UNDO group
        (command)
        (command)
        (command "._UNDO" "_E")
          (if (and lastent
               (/= ent lastent)
          )
            (entdel lastent)          ; remove the last entity
          )
        (dos_waitcursor)
        (reset nil)
        (resset varlst)
        (if (member msg# '("console break" "Function cancelled" "Error: quit / exit abort"))
          (princ " ")
          (princ (strcat "\nError " (itoa (getvar "ERRNO")) ": " msg# "\n"))
                          ;Otherwise, print an error indication
        )
        (princ)
      )

        (setq
          mode (ukword
             1
             "Exist Copy"
             "Increment existing text or copy increment? [Copy/Exist] "
             "Copy"
           )
        )

        (if (= incvalue nil)
          (setq incvalue 1)
        )
        (setq incvalue (uint 2 "" "Enter increment" incvalue))

        (if (< incvalue 0)
          (progn
            (setq negnum
               (ukword
                 1
                 "Yes No"
                 "If negative numbers found in text, ignore minus sign? [Yes/No] "
                 "No"
               )
            )
            (if (= negnum "No")
          (setq negnum T)
          (setq negnum nil)
            )
          )
        )

        (cond
          ((= mode "Exist")
           (command "_.undo" "_b")
           (setq bLoop T)
           (while bLoop           ;Loop until ESCAPE or Enter is pressed
             (princ "\nSelect annotation object(s)...")
             (setq ss1 (ai_aselect))
             (setq ss1 (ssget "_P" (list (cons 0 "TEXT,MTEXT"))))
             (if (/= ss1 nil)
           (progn
             (setq idx (sslength ss1))
             (while (>= (setq idx (1- idx)) 0)
               (setq ent (ssname ss1 idx))
               (if (NbAccroi ent nil)
                 (princ " ")
                 (princ "\nNo number found!")
               )
             )
             (setq bLoop nil)
           )
             )
           )
          )

          ((= mode "Copy")
           (setq bLoop T
             newtxt nil
           )
           (while bLoop           ;Loop until ESCAPE or Enter is pressed
             (if (or newtxt
                 (setq
               ent (objsel "TEXT or MTEXT" "Select annotation object to clone" T)
                 )
             )                ; verify selection was TEXT or MTEXT
           (progn             ;  got an entity
             (if (and (null newtxt) (dxf 3 (entget ent)))
               ;; mtext has > 250 characters
               (princ "text too large!")
               (progn
                 (command "_.undo" "_b")
                 (if newtxt
               (progn
                 (if
                   (> (dxf 40
                       (tblsearch "STYLE" (getvar "TEXTSTYLE"))
                  )
                  0
                   )
                    (command "_.TEXT" "0,0" "" newtxt)
                    (command "_.TEXT" "0,0" "" "" newtxt)
                 )
                 (setq ent    (entlast)
                   newtxt ent ; save the entity name
                 )
               )          ; progn
                 )
                 (setq bLoop T)       ; exit control
                 (while bLoop     ; Loop until Enter or Escape
               (if (setq lastval (NbAccroi ent lastval))
                 ;; get new text entity
                 (progn
                   (if newtxt
                     (entdel newtxt)
                   )
                   (setq lastent (entlast)
                     newtxt  nil
                   )
                   (setq disppoint (dxf 10 (entget lastent))
                     nocr      t
                   )
                   (prompt "\nSpecify insertion point of text")
                   (command "._MOVE" lastent "" disppoint pause)
                   (setq newpos (dxf 10 (entget lastent)))
                   (if (and (= (car newpos) (* (car disppoint) 2))
                        (= (cadr newpos) (* (cadr disppoint) 2))
                   )
                     (progn       ;ENTER was pressed
                   (entdel (entlast)) ; remove last copy
                   (setq bLoop nil) ; exit routine
                     )
                   )
                   (setq ent (list lastent disppoint))
                          ;get last object pasted
                 )
                 (progn
                   (princ "\nNo number found!")
                   (if newtxt
                     (entdel newtxt)
                   )
                  (setq bLoop nil)        ;exit loop
                 )
               )
                 )
               )
             )
             (setq bLoop nil)     ;exit loop
           )

      ;    ;;   User to enter the original text
      ;    (while
      ;      (null (setq newtxt
      ;           (getstring t "\nEnter text:")
      ;        )
      ;      )
      ;       (prompt "\nNo text try again.")
      ;    )
             )
           )
          )

                          ;    ((= mode "Replace")            ; Replace mode

                          ;    (command "undo" "begin")
                          ;    (setq bLoop t)
                          ;    (while bLoop           ; Loop until ESCAPE or Enter is pressed
                          ;      (setvar "errno" 0)
                          ;      (setq lastval nil)       ; prevent cumulative increment
                          ;      (cond
                          ;        ((and (setq ent (entsel "\nSelect Text to Replace: "))
                          ;          (member (cdr (assoc 0 (entget (car ent))))
                          ;              '("TEXT" "MTEXT")
                          ;          )
                          ;         )             ; verify selection was TEXT or MTEXT
                          ;         (if (cdr (assoc 3 (entget (car ent))))
                          ;       (alert "MTEXT is too large, not supported!")
                          ;       (if (NbAccroi ent lastval)
                          ;         (prompt "\nText Updated.")
                          ;         (alert "No number in text.")
                          ;       )
                          ;         )
                          ;        )
                          ;
                          ;        ((= 7 (getvar "errno"))    ; user Picked Nothing
                          ;         (princ "\n*-*  Miss Pick, Try Again  *-*")
                          ;        )
                          ;
                          ;        ((setq bLoop nil)      ; Enter Pressed
                          ;        )
                          ;
                          ;      )
                          ;    )
                          ;    )

                          ;    (t (prompt "\nNo text selected."))
        )
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;; ==========================================================================
      ;;                          Local  Functions                                ;
      ;; ==========================================================================

      ;;  Process the number within the text
      (defun NbAccroi (ent lastval / numstr elist entstr ndx newstring code bLoop bLoop2
                     nextchr)

        (setq elist (entget (if (= (type ent) 'list)(car ent) ent)))
        (setq entstr (cdr (assoc 1 elist)))
        ;; MText with < 250 characters
        (setq ndx     1
              numbers (list "0" "1" "2" "3" "4" "5" "6" "7" "8" "9")
              ;"ignore" is a list of code flags that will end with ;
              ignore  (list "\\C" "\\F" "\\H" "\\S" "\\T" "\\Q" "\\W" "\\A")
        )
        (if (/= entstr nil) ; skip if empty text
          (progn
            (setq strlength (strlen entstr)
                  newstring ""
                  numstr    ""
            )
            (while (<= ndx strlength)             ;step through text and find first number
              (setq nextchr (substr entstr ndx 1))        ;Get next character

              (cond
                ((member nextchr numbers)             ;number found, so get next numbers
                 (if (and (> ndx 1) (< incvalue 0) negnum)    ;negnum=Y/N
                                          ;Check for negative number
                   (if (= (substr entstr (- ndx 1) 1) "-")
                     (setq newstring (substr newstring 1 (1- (strlen newstring)))
                           numstr    "-"
                     )
                   )
                 )
                 ;;  collect all numeric characters until non-numeric found
                 ;;  this is the number to increment
                 (while (and (<= ndx strlength) (member nextchr numbers))
                   (setq numstr  (strcat numstr nextchr)
                         ndx     (1+ ndx)
                         nextchr (substr entstr ndx 1)
                   )
                 )
                 ;
                 (if lastval
                   (setq lastval (+ incvalue lastval))    ;convert to number & add increment value
                   (setq lastval (+ incvalue (atoi numstr)))
                 )
                 (if (and (not negnum) (< lastval 0))     ;prevent negative numbers in text
                   (setq lastval 0)
                 )
                 ;Reassemble string
                 (setq newstring
                        (strcat newstring
                                (rtos lastval 2 0)
                                (substr entstr ndx (- strlength ndx -1))
                        )
                 )
                 (if (= mode "Exist")             ;update text else create new text
                   (entmod (subst (cons 1 newstring) (assoc 1 elist) elist))
                   (entmake (subst (cons 1 newstring) (assoc 1 elist) elist))
                 )
                 (if (= mode "Replace")           ;remove the old object
                   (if (= (type ent) 'list)
                     (entdel (car ent))
                     (entdel ent)
                   )
                 )
                 (setq ndx (1+ strlength))            ;force exit
                )

                ((or (= nextchr "\\") (= nextchr "%"))
                 ;; process ctrl codes
                 ;;  This section processes groups codes so
                 ;;  that the numbers in them are ignored

                 (setq code (strcase (substr entstr ndx 2)))
                 ;; get next 2 characters
                 ;; set to 2 because \\ is counted as one

                 (cond                    ;Process Coded information
                       ((= (strlen code) 1) ; true if \ or % is last char in text
                        (ChrAdd 1)
                       )

                       ((member code ignore) ; this code will end with ";"
                        (while (/= (setq nextchr (substr entstr ndx 1)) ";")
                          (ChrAdd 1)
                        )
                        (ChrAdd 1)
                       )

                       ((= nextchr "%")
                        (if (= code "%%") ; code found
                          (if (distof (substr entstr (+ ndx 2) 1))
                            ;;number found so ignore %%nnn
                            (ChrAdd 5)
                            ;; else letter code, so ignore %%p
                            (ChrAdd 3)
                          )
                          ;; else no %% codes so add the % char to list
                          (ChrAdd 1)
                        )
                       )

                       ;; found \\U then get 7 character group
                       ((= code "\\U") (ChrAdd 7))

                       ;; found \\M then get 8 character group
                       ((= code "\\M") (ChrAdd 8))

                       ;; found \\ then get 2 character group
                       ((= nextchr "\\") (ChrAdd 2))
                 )
                 ;
                )

                ;;no ctrl codes or number so add the char to list
                (t (ChrAdd 1))

              )
            )
          )
        )
        lastval                       ;return the incrementd value
      )

      (defun ChrAdd (cnt) ; add cnt characters to newstring
        (setq newstring (strcat newstring (substr entstr ndx cnt)))
        (setq ndx (+ ndx cnt))
      )

      ;-------------------------------------
      ;---Returns centroid for closed plines.
      ;-------------------------------------
      (defun PlCentroid (str / vent space array newvent cent)
        (setq vent (vlax-ename->vla-object str))              ;Assigns to the variable VENT the Vla-Object of the polyline
        (if (or (= "AcDbPolyline" (vla-get-objectname vent))
            (= "AcDb2dPolyline" (vla-get-objectname vent)) )      ;Checks to make sure it is a polyline
          (progn
            (if (= (vla-get-closed vent) :vlax-true)              ;Checks to make sure it is a closed
                                          ;polyline. You cannot create a region without a closed polyline
          (progn
            (if
              (= (vla-get-activespace
               (vla-get-activedocument (vlax-get-acad-object))
                 )
                 1
              )
               (progn (setq space (vla-get-modelspace
                        (vla-get-activedocument
                          (vlax-get-acad-object)
                        )
                      )
                  )
               )
               (progn (if (= (vla-get-mspace
                       (vla-get-activedocument
                         (vlax-get-acad-object)
                       )
                     )
                     :vlax-true
                  )
                    (setq space (vla-get-modelspace
                          (vla-get-activedocument
                            (vlax-get-acad-object)
                          )
                        )
                    )
                    (setq space (vla-get-paperspace
                          (vla-get-activedocument
                            (vlax-get-acad-object)
                          )
                        )
                    )
                  )
               )
            )                             ;Gets the active space and assigns the Vla-"Space"-Object to the variable SPACE
            (setq array (vlax-make-safearray vlax-vbobject '(0 . 0))) ;Creates an array;
            (setq newvent (vlax-safearray-fill array (list vent)))    ;Fills array with the Vla-Object assigned to VENT
            (setq newvent (vlax-safearray-get-element
                    (vlax-variant-value
                      (vla-addregion space newvent)
                    )
                    0
                  )
            )                             ;Create a region from the array
            (setq cent (vlax-safearray->list
                     (vlax-variant-value
                   (vla-get-centroid newvent)
                     )
                   )
            )                             ;Gets the centroid
            (vla-delete newvent)
          )
            )
          )
        )
        cent
      )

      ;-------------------------------
      ;---zoom to extents of a selset.
      ;-------------------------------
      ;; Routine to create a selection set and zoom to the extents of the ss.   miff Jun 2003
      (defun c:zfo (/ count ent ll ll-ent ll-ss ur ur-ent ur-ss llx lly urx ury)
        (setq olderror *error*
          *error*  Wallserror
        )
        (Wallsgather_info)
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1))
          (progn
            (FlashMess3
              "ArchAssist maintenant Actif."
              "ArchAssist now Active."
              "ArchAssist now Active."
              "ArchAssist now Active."
            )
          )
        )
        (princ (strcat "\nZoom to fit selected objects." copyright))
        (if (or (setq zz_ss (cadr (ssgetfirst))) (setq zz_ss (ssget)))
          (progn (setq count -1)
             (while (< (setq count (1+ count)) (sslength zz_ss))
               (setq ent (vlax-ename->vla-object (ssname zz_ss count)))
               (vla-getboundingbox ent 'll 'ur)
               ;;      (vla-highlight ent :vlax-true)
               (setq ll-ent (vlax-safearray->list ll)
                 ur-ent (vlax-safearray->list ur)
               )
               (if (not ll-ss)
                 (setq ll-ss ll-ent
                   ur-ss ur-ent
                 )
                 (progn
               (if (< (car ll-ent)
                  (car ll-ss)
                   )
                 (if (< (cadr ll-ent)
                    (cadr ll-ss)
                     )
                   (setq ll-ss ll-ent)
                   (setq ll-ss (list (car ll-ent) (cadr ll-ss)))
                 )
                 (if (< (cadr ll-ent) (cadr ll-ss))
                   (setq ll-ss (list (car ll-ss) (cadr ll-ent)))
                 )
               )
               (if (> (car ur-ent) (car ur-ss))
                 (if (> (cadr ur-ent) (cadr ur-ss))
                   (setq ur-ss ur-ent)
                   (setq ur-ss (list (car ur-ent) (cadr ur-ss)))
                 )
                 (if (> (cadr ur-ent) (cadr ur-ss))
                   (setq ur-ss (list (car ur-ss)
                             (cadr ur-ent)
                       )
                   )
                 )            ;if
               )          ;if
                 )            ;progn
               )              ;if

             )                ;while
             (setq ll  (vlax-3d-point ll-ss)
               ur  (vlax-3d-point ur-ss)
               llx (car ll-ss)
               lly (cadr ll-ss)
               urx (car ur-ss)
               ury (cadr ur-ss)
             )                ;setq
             (vla-zoomwindow (vlax-get-acad-object) ll ur)
             (vla-zoomscaled
               (vlax-get-acad-object)
               0.98
               acZoomScaledRelative
             )
                          ;      (grvecs (list -7
                          ;            (list llx lly)
                          ;            (list llx ury)
                          ;            (list llx ury)
                          ;            (list urx ury)
                          ;            (list urx ury)
                          ;            (list urx lly)
                          ;            (list urx lly)
                          ;            (list llx lly)
                          ;          )
                          ;      )
          )                   ;progn
        )                 ;if
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      ;-------------------
      ;;; (AXLayoutImport "ltsc2.dwg" "Serge" T)                  ;;<<--**
      (defun AXLayoutImport (source name current)
          (cond
              ( (setq fn (findfile source))
                (command "-.layout" "_t" fn name)
                (if current (command "-.layout" "_s" name))
              )
              ( T (princ (strcat "\nUnable to attach layout from " source )) )
          ); cond
      )

      (defun AXLayoutPlot (name)
        (setvar "cmdecho" 0)
        (command "-.plot" "N" name "" "" "N" "N" "Y")
        (princ)
      )

      (defun AXLayoutsGet ()
        (vla-get-Layouts (VLxx_ActiveDocument))
      )

      (defun AXLayoutsListAll ()                          ;Ordre alphabetique
        (vlax-for each (AXLayoutsGet)
          (setq out (cons (vla-get-Name each) out))
        )
        (if out (reverse out))
      )

      ;Purpose: Activates the rightmost layout tab.
      (defun ax:ActivateLastLayout (/ tabs i tmp)
        (setq i -1)
        (vlax-for layout
         (vla-Get-Layouts (vla-Get-ActiveDocument (vlax-Get-Acad-Object)))
          (if (> (setq tmp (vla-get-taborder layout)) i)
            (setq i      tmp
              retval layout
            )
          )
        )
        retval
      )

      ;;;;;;;;;;;-----------------**************
      (defun AXRemoveXRecord (obXDico sNomDico strKeepWC / obDico jCompt)
        (TraceIsOn ".AXRemoveXRecord")
        ;; This function performs the chore of deleting each filer that doesn't match the wildcard
        (vl-catch-all-apply                           ;trap errors
          (function
            (lambda ()
          (setq obDico (vla-Item obXDico sNomDico))
                                          ;get layer filters dictionary
          (vlax-for obXRec obDico                     ;loop thru all XRecords in the dictionary
            (cond ((not (and strKeepWC
                     (wcmatch (vla-Get-Name obXRec) strKeepWC)
                    )
               )
                                          ;if deleting all filters, or current doesn't match wildcard
               (setq jCompt (1+ (cond (jCompt)            ;increment counter
                              (0)
                            )
                        )
               )                          ;initialisation du compteur
               (vla-Delete obXRec)
              )
            )
          )
            )
          )
        )                                 ;Borrar filtro
        (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" (strcat (itoa jCompt) " layer filters removed."))
      )

      (defun AXLayerFiltersDel ( / obXDico)
        (TraceIsOn ".AXLayerFiltersDel")
        ;; This function insures that an Extension Dictionary exists, and works on both locations for layer filters
        (vl-catch-all-apply                           ;trappe les erreurs si dictionaire pas trouve
          (function
            (lambda ()
          (setq
            obXDico (vla-GetExtensionDictionary               ;bind dictionary to variable
                   (vla-get-Layers
                     (vla-get-ActiveDocument (vlax-get-acad-object))
                   )
                 )
          )
            )
          )
        )
        (cond (obXDico                            ;Si dictionaire existe
           (or                                ;use OR to return T for success
             (AXRemoveXRecord obXDico "ACAD_LAYERFILTERS" nil)        ;Avant 2005 layer filters
             (AXRemoveXRecord obXDico "AcLyDictionary" nil)       ;strKeepWC
           )
          )
        )
      )

      (defun LayerFiltersList (/ lLayerFilt lstTemp)              ;Used
        (TraceIsOn ".LayerFiltersList")
      ;;LCA - WARNING: Using entity data from table objects may have new requirements
        (setq bQlqChose (assoc 360 (entget (dxf 330 (entget (tblobjname "LAYER" "0"))))))
        (if (/= bQlqChose nil)
          (progn (setq
               lLayerFilt (entget
                    (dxf 360
                         (entget (dxf 360 (entget (dxf 330 (entget (tblobjname "LAYER" "0"))))) ;_
                         )
                    )
                  )
             )
             (foreach lstTemp lLayerFilt
               (if (= (car lstTemp) 3)
                 (progn (setq lFilters (append lFilters (list (cdr lstTemp)))))
               )
             )
          )
        )
        (princ)
      )

      ;Purpose: Closes all open documents without saving.
      (defun VLxx_CloseAllDocs (/ item cur)
        (vlax-For item (vla-Get-Documents (VLxx_AcadObject))
          (if (= (vla-Get-Active item) :vlax-False)
            (vla-Close item :vlax-False)
            (setq cur item)
          )
        )
        (vla-SendCommand cur "_.CLOSE")
      )

      (defun AXPurgeAll ( / item cur)                 ;Used
        (vlax-for item (vla-get-Documents (VLxx_AcadObject))
          (vla-PurgeAll item)
          ;;(vla-save item)
        )
      )

      (defun AXAuditAll (/ docs)
        (vlax-for item (vla-get-Documents (VLxx_AcadObject))
          (vla-auditinfo item T)                  ;T denotes fix errors = Yes
        )
      )

      ;; re-position layout tabs into alpha/numeric order briefly tested, could be improved to handle
      ;; "proper" counting, ie: 1,2,3,10 not 1,10,2,3
      (defun AXLayoutsSort (/ tabs order name)            ;Used
        (setq tabs   (vla-get-layouts *active-document*)
          order  (layoutlist)
        )
        (foreach x order (vla-put-taborder (vla-item tabs x) (1+ (vl-position x order))))
        (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" "Sorting complete.")
        (princ)
      )

      ;Purpose: Returns the next "default" name for new layout tabs.
      ;Example: (AXGetNewLayoutName)                 ;Used
      (defun AXGetNewLayoutName ( / i layouts name)
        (setq i 0)
        (setq layouts
           (vla-get-layouts
             (vla-get-activedocument
               (vlax-get-acad-object)
             )
           )
        )
        (vl-catch-all-apply
          '(lambda ()
             (while
           (vla-item layouts
                 (setq name
                    (strcat
                      "Layout"
                      (itoa (setq i (1+ i)))
                    )
                 )
           )
             )
           )
        )
        name
      )

      ;Purpose: Adds and activates a new layout.
      (defun AXAddLayout ( / doc layout)              ;Used
        (setq layout
           (vla-add (vla-get-layouts
                  (setq doc (vla-get-activedocument (vlax-get-acad-object))
                  )
                )
                (AXGetNewLayoutName)
           )
        )
        (vla-put-activelayout doc layout)
        layout
      )

      (TraceIsOn ".10")

      ;;   Mde = Mode, Values:
      ;;         - Nil (GetDispCmdLines) [SYM] in get mode with nil
      ;;         - Number of command lines (SetDispCmdLines) [INT]
      ;; Return [Typ]:
      ;;   > Number of command lines (Mde = Nil) [INT]
      ;;   > Nil (Mde = Number) [SYM]
      ;; Notes:
      ;; - If Mde is numeric, it must be > 0.
      ;; - Use a DocManagerReactor with a 'vlr-documentToBeDestroyed'-event to release the Cm:AcDObj object at
      ;; the end of a AutoCAD session otherwise AutoCAD maybe crashes...
      (defun AXDispCmdLines (Mde / DspObj)                ;Used
        (TraceIsOn "AXDispCmdLines.")
        (setq Cm:AcDObj (cond (Cm:AcDObj)
                   ((vlax-get-acad-object))
                 )
          DspObj (vla-get-Display (vla-get-Preferences Cm:AcDObj))
        )
        (if (numberp Mde)
          (vla-put-DockedVisibleLines DspObj Mde)         ;Set mode
          (vla-get-DockedVisibleLines DspObj)             ;Get mode
        )
      )

      (defun DeleteLayerState () ;;(layerStateName / acadApp activeDoc layerStateManagerApp)
        (setq acadApp   (vlax-get-acad-object)
          activeDoc (vla-get-activeDocument acadApp)
        )
        (setq layerStateManagerApp (vla-getInterfaceObject acadApp "AutoCAD.AcadLayerStateManager"))
        (vla-setDatabase layerStateManagerApp (vla-get-database activeDoc))
        (vl-catch-all-apply 'vla-delete (list layerStateManagerApp layerStateName))
        (vlax-release-object layerStateManagerApp)
        (princ)
      )

      (defun AXRemAppids (/ iCount)                   ;Used
        (TraceIsOn ".AXRemAppids")
        (setq iCount 0)
        ;;(or *acad* (setq *acad* (vlax-get-acad-object)))        ;Define Global Reference to the ACAD application/session object
        ;;(or *doc* (setq *doc* (vla-get-activedocument *acad*))) ;Define Global Reference to the ACTIVE DOCUMENT object
        (vlax-for Appid (vla-get-registeredapplications *active-document*)
          (if (vl-catch-all-error-p (vl-catch-all-apply 'vla-delete (list Appid)))
           (princ (strcat "\nCannot delete " (vla-get-name Appid)))
           (progn
            (setq iCount (1+ iCount))
           )
          )
        )
        (setq sMess1 (strcat "Registered application(s) deleted " (itoa iCount) "."))
        (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" sMess1)
      )

      (defun AXRemApid (/ Appid)                  ;Used
        (TraceIsOn ".AXRemApid")
        (vlax-for Appid (vla-get-registeredapplications *active-document*)
          (cond ((= sMot (vla-get-name Appid))
             (if
               (progn
                    (setq bRADisp nil)
                (vl-catch-all-error-p (vl-catch-all-apply 'vla-delete (list Appid)))
               )
               (progn
                (setq bRADisp T)
                (setq sMess1 (strcat "Cannot delete " (vla-get-name Appid) "."))
                (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" sMess1)

               )
             )
            )
          )
        )
      )

      ;;function to delete a single page setup
      (defun AXRemPageSetup (PageSetupNam / PageSU)
        (vlax-for EachPS (vla-get-plotconfigurations (vla-get-activedocument (vlax-get-acad-object)))
          (if (equal PageSetupNam (vla-get-name EachPS))
            (vl-catch-all-apply 'vla-delete (list EachPS))
          )
        )
      )

      ;;function to delete all page setups
      (defun AXRemPageSetups (/ PageSU)
        (vlax-for EachPS (vla-get-plotconfigurations (vla-get-activedocument (vlax-get-acad-object)))
          (vl-catch-all-apply 'vla-delete (list EachPS))
        )
      )

      (TraceIsOn ".10a")

      (defun AXRemDimStyle (/ sItem )                 ;Used
        (TraceIsOn ".AXRemDimStyle")
        (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" " ")
        (vlax-for sItem (vla-get-DimStyles *active-document*)
          (cond ((= sMot (vla-get-name sItem))
             (if (progn
              (setq bDSDisp nil)
                  (vl-catch-all-error-p (vl-catch-all-apply 'vla-delete (list sItem)))
                 )
                 (progn
                  (setq bDSDisp T)
                  (Setq sMess2 (strcat "Style " (vla-get-name sItem) " is in use and cannot be deleted."))
                  (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" sMess2)
                 )
             )
            )
          )
        )
      )

      ;;(vlax-for sItem (vla-get-DimStyles *active-document*) (princ (vla-get-Name sItem))(princ "--")(list sItem))

      (defun AXListDimStyles (/ out each)             ;Used
        (vlax-For each (vla-get-DimStyles (VLxx_ActiveDocument))
          (setq out (cons (vla-get-Name each) out))
        )
        (if out
          ;;(mapcar 'strcase (reverse out))
          (reverse out)
        )
      )

      ;Get DimStyle object by name.
      ;EXAMPLE: (AXGetDimStyles "Standard")
      (defun AXGetDimStyles (name / itm)
        (if (setq itm (vl-Position (strcase name) (AXListDimStyles)))
          (vla-Item (vla-get-DimStyles (VLxx_ActiveDocument)) itm)
        )
      )

      ;; Returns a list of all open Drawing names including path.
      ;; Return [Typ]: > Drawing names [LIST]
      ;; Notes:
      ;; - Use a DocManagerReactor with a 'vlr-documentToBeDestroyed'-event to release the Cm:AcDObj object at
      ;; the end of a AutoCAD session otherwise AutoCAD maybe crashes...
      (defun AXGetOpenDwg (/ DwgLst DwgNme)               ;Used
        (TraceIsOn ".AXGetOpenDwg")
        (setq Cm:AcDObj (cond (Cm:AcDObj)
                  (T (vlax-get-acad-object))
                )
        )
        (vlax-for Doc (vla-get-Documents Cm:AcDObj)
          (if (/= (setq DwgNme (vla-get-Fullname Doc)) "")
            (setq DwgLst (cons DwgNme DwgLst))
          )
        )
        (reverse DwgLst)
      )

      ;Purpose: Returns a list containing all layouts in the current document.
      ;Notes: AXLayoutlist returns a list of layout names sorted by tab order, not by name like LAYOUTLIST.
      (defun AXLayoutlist (/ layouts c lst lay)           ;Used
        (setq layouts (vla-get-layouts
                (vla-get-activedocument (vlax-get-acad-object))
              )
          c   -1
        )
        (repeat (vla-get-count layouts)
          (setq lst (cons (setq c (1+ c)) lst))
        )
        (vlax-for lay layouts
          (setq lst
             (subst
               (vla-get-name lay)
               (vla-get-taborder lay)
               lst
             )
          )
        )
        (reverse lst)
      )

      ;Purpose: Renames an existing layout.
      ;Arguments: A string containing the name of the layout to rename and a string containing the new name for it.
      ;Example: (AXRenLayout "Layout1" "myLayout")
      (defun AXRenLayout (oldName newName)                ;Used           <<--**
        (vla-put-name
          (vla-item (vla-get-layouts
              (vla-get-activedocument (vlax-get-acad-object))
                )
                oldName
          )
          newName
        )
        (princ)
      )

      ;;;on vla method of purging empty groups
      ;;(defun c:purgegroups ()
      ;;  (vlax-for g (vla-get-groups (vla-get-activedocument (vlax-get-acad-object)))
      ;;    (if   (< (vla-get-count g) 2)
      ;;      (vla-delete g)
      ;;    )
      ;;  )
      ;;  (princ)
      ;;)

      (defun AXPurgeGroups (/ VlaObj)
        (or *AcadApp* (setq *AcadApp* (vlax-get-acad-object)))
        (or *ActiveDwg*
            (setq *ActiveDwg* (vla-get-ActiveDocument *AcadApp*))
        )
        (vlax-for ForElm (setq VlaObj (vla-get-groups *ActiveDwg*))
          (or (< 1 (vla-get-count ForElm)) (vla-delete ForElm))
        )
        (vlax-release-object VlaObj)
      )

      ;;;Two routines to add layer filters back in.  MAKELAYERFILTER and MAKEFLTR.
      ;;;First one puts the filters into the drawing, second one accepts input to a global
      ;;;variable to input to the first one.  Could all be rolled into one routine.  Also, a text file
      ;;;could be created to store a set in.
      ;;; Function can define a named layer filter
      (defun dict:MlayerF (NAME   FILTER     /          LayTbl
                   ExtDict    LayColl    ExtDictData
                   GrpList    NewDict    Temp       XdLst
                   XrecData   NewXRec
                  )
        (vl-load-com)
        (setq LayTbl (cdr (assoc 330 (entget (tblobjname "LAYER" "0")))))
        (setq ExtDict (cdr (assoc 360 (entget LayTbl))))
        (if (not ExtDict)
          (progn (vl-load-com)
             (setq LayColl (vla-get-layers
                     (vla-get-activedocument (vlax-get-acad-object))
                   )
             )
             (setq ExtDict (vla-GetExtensionDictionary LayColl))
             (setq ExtDict (vlax-vla-object->ename ExtDict))
          )
        )
        (setq ExtDictData (entget ExtDict))
        (while ExtDictData
          (setq Temp        (if (= (caar ExtDictData) 3)
                  (list
                    (cons (cdr (car ExtDictData)) (cdr (cadr ExtDictData)))
                  )
                    )
            ExtDictData (cdr ExtDictData)
          )
          (setq GrpList (append GrpList Temp))
        )
        (if (or (not GrpList)
            (not (cdr (assoc "ACAD_LAYERFILTERS" GrpList)))
            )
          (progn (setq DictList (list (cons 0 "DICTIONARY")
                      (cons 100 "AcDbDictionary")
                    )
             )
             (setq NewDict (entmakex DictList))
             (setq NewDict (dictadd ExtDict "ACAD_LAYERFILTERS" NewDict))
          )
          (setq NewDict (cdr (assoc "ACAD_LAYERFILTERS" GrpList)))
        )
        (setq ExtDictData
           nil
          Temp nil
          GrpList nil
        )
        (setq ExtDictData (entget NewDict))
        (while ExtDictData
          (setq Temp        (if (= (caar ExtDictData) 3)
                  (list
                    (cons (cdr (car ExtDictData)) (cdr (cadr ExtDictData)))
                  )
                    )
            ExtDictData (cdr ExtDictData)
          )
          (setq GrpList (append GrpList Temp))
        )
        (if (or (not GrpList) (not (cdr (assoc FILTER GrpList))))
          (progn (setq XdLst (list (cons 0 "XRECORD") (cons 100 "AcDbXrecord")))
             (setq XrecData (append XdLst
                        (list (cons 1 NAME)
                          (cons 1 FILTER)
                          (cons 1 "*")
                          (cons 1 "*")
                          (cons 70 0)
                          (cons 1 "*")
                          (cons 1 "*")
                        )
                    )
             )
             (setq NewXRec (entmakex XRecData))
             (dictadd NewDict Name NewXRec)
          )
          nil
        )
      )

      ;Purpose: Converts a list of points to a coordinate list.
      ;Example: (VLxx_List->VariantArray (list '(2.0 3.0 0.0) '(5.0 7.5 0.0) '(10.0 4.25 0.0)))
      ;Notes: Use this for converting AutoLISP points into an ActiveX coordinate array
      ;like the one expected by AddPolyline.
      (defun List->VariantArray (nList / ArraySpace sArray)

        ; allocate space for an array of 2d points stored as doubles
        (setq ArraySpace
          (vlax-Make-SafeArray
            vlax-vbDouble ; element type
            (cons 0
              (- (length nList) 1)
            ) ; array dimension
          )
        )
        (setq sArray (vlax-SafeArray-Fill ArraySpace nList))

        ; return array variant
        (vlax-Make-Variant sArray)
      )

      ;Purpose: Returns the centroid of a closed polyline
      ;Arguments: The object name of a closed, planar polyline
      ;Example: (VLxx_PolyCentroid (car (entsel)))
      (defun VLxx_PolyCentroid (poly / pl ms va reg cen)
        (setq pl (vlax-Ename->vla-Object poly)
          ms (VLxx_ModelSpace)
          va (vlax-Make-SafeArray vlax-vbObject '(0 . 0))
        )
        (vlax-SafeArray-Put-Element va 0 pl)
        (setq reg (car
                (vlax-SafeArray->list
              (vlax-Variant-Value (vla-AddRegion ms va))
                )
              )
          cen (vla-Get-Centroid reg)
        )
        (vla-Delete reg)
        (vlax-SafeArray->list (vlax-Variant-Value cen))
      )

      (defun VLxx_Saved-p ()
        (= (vla-get-saved (VLxx_ActiveDocument)) :vlax-True)
      )

      (defun VLxx_SaveAs2000 (name)
        (vla-saveas (VLxx_ActiveDocument) name acR15_DWG)
      )

      (defun VLxx_SaveAsR14 (name)
        (vla-saveas (VLxx_ActiveDocument) name acR14_DWG)
      )

      (TraceIsOn ".11")

      ;Purpose: Draws a rectangle representing the area displayed by a paper space viewport
      ;Arguments: The entity name of a paper space view port
      ; (ax:DrawVpBorder (car (entsel)))
      (defun ax:DrawVpBorder (vp / ll ur coords pl)
        (setq vp (vlax-ename->vla-object vp))
        (vla-GetBoundingBox vp 'll 'ur)
        (setq ll     (trans (vlax-safearray->list ll) 3 2)
          ur     (trans (vlax-safearray->list ur) 3 2)
          coords (vlax-safearray-fill
               (vlax-make-safearray vlax-vbDouble (cons 0 7))
               (list (nth 0 ll)
                     (nth 1 ll)
                     (nth 0 ur)
                     (nth 1 ll)
                     (nth 0 ur)
                     (nth 1 ur)
                     (nth 0 ll)
                     (nth 1 ur)
               )
                 )
        )
        (vla-put-closed
          (setq pl (vla-AddLightWeightPolyline
                 (vla-get-ModelSpace (vla-get-Document vp))
                 coords
               )
          )
          :vlax-true
        )
        (vlax-vla-object->ename pl)
      )

      (TraceIsOn ".12")

      ;;; Arguments: A list containing one atom and one or more dotted pairs.
      ;;; The atom is the object name of the block to change. The dotted pairs
      ;;; consist of the attribute tag and the new value for that attribute.
      ;;; Example: (ax:ChangeAttributes (list ename '("myAttribute" . "NewValue")))
      ;;; Notes: Modifies the specified attribute in the specified block reference.
      (defun VLxx_ChangeAttributes (lst / blk itm atts)
        (setq blk (vlax-Ename->vla-Object (car lst))
          lst (cdr lst)
        )
        (if (= (vla-Get-HasAttributes blk) :vlax-true)
          (progn
            (setq atts
               (vlax-SafeArray->list
                 (vlax-Variant-Value (vla-GetAttributes blk))
               )
            )                 ; setq

            (foreach item lst
          (mapcar
            '(lambda (x)
               (if
                 (= (strcase (car item)) (strcase (vla-Get-TagString x)))
              (vla-Put-TextString x (cdr item))
               )
             )
            atts
          )               ; mapcar
            )                 ; foreach
            (vla-Update blk)
          )
        )
      )

      ;;; Arguments: The object name of an attributed block
      ;;; Example: (ax:GetAttributes (car (entsel)))
      ;;; Returns a list of attribute tags and associated values.
      (defun VLxx_GetAttributes (ent / blkref lst)
        (if (= (vla-Get-ObjectName
             (setq blkref (vlax-Ename->vla-Object ent))
           )
           "AcDbBlockReference"
            )
          (if (vla-Get-HasAttributes blkref)
            (mapcar
          '(lambda (x)
             (setq
               lst (cons
                 (cons (vla-Get-TagString x) (vla-Get-TextString x))
                 lst
               )
             )
           )
          (vlax-safearray->list
            (vlax-variant-value (vla-GetAttributes blkref))
          )
            )                 ; mapcar
          )
        )
        (reverse lst)
      )

      (defun AXExportProfil (strName strFilename BooleReplace)
        (if (AXProfileExist-p strName)
          (if (not (findfile strFilename))
            (progn
              (vlax-Invoke-Method
                (vlax-get-Property (VLxx_AcadPrefs) "Profiles")
                'ExportProfile strName strFilename
              )
              T ;; return TRUE
            ); progn
            (if BooleReplace
              (progn
                (vl-file-delete (findfile strFilename))
                (if (not (findfile strFilename))
                  (progn
                    (vlax-Invoke-Method
                      (vlax-get-Property (VLxx_AcadPrefs) "Profiles")
                      'ExportProfile strName strFilename
                    )
                    T ;; return TRUE
                  ); progn
                  (princ "\nCannot replace ARG file, aborted.")
                )
              ); progn
              (princ (strcat "\n" strFilename " already exists, aborted."))
            )
          )
        )
      )

      ;;; DESCRIPTION: Imports ARG file as new profile
      ;;; EXAMPLE: (AXImportProfil "c:/stuff/test.arg" "MyProfile")
      (defun AXImportProfil (ARGfile pName)
        (cond
          ((findfile ARGfile)
           (vlax-invoke-method
             (vlax-get-property (VLxx_AcadPrefs) "Profiles")
             'ImportProfile
             pName
             ARGFile
             (vlax-make-variant 1 :vlax-vbBoolean) ;T
             ;; == TRUE
           )
          )
          (T (princ "\nFile not found.\nPlease verify the correct filename was given."))
        )
      )

      ;;; DESCRIPTION: Deletes a profile from the AcadApplication object
      ;;; EXAMPLE: (AXEffaceProfile "MyProfile")
      (defun AXEffaceProfile (pName)
        (cond
          ((= bCProf T)
            (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" "Cannot delete a profile that is in use.")
          )
          ((= bUnnamed T)
            (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_OnError2 "Caption" "Cannot delete <<Unnamed Profile>>.")
          )
          (T (vlax-invoke-method
              (vlax-get-property (VLxx_AcadPrefs) "Profiles")
              'DeleteProfile
              pName
             )
          )
        )
      )

      ;;; DESCRIPTION: Boolean test for profile existence
      ;;; EXAMPLE: (if (AXProfileExist-p "MyProfile") ...)
      (defun AXProfileExist-p (pName)
        ;; Search for CAPS profile-name in CAPS list of profiles
        (not
          (not
            (member
          (strcase pName)
          (mapcar 'strcase (AXListeProfils))
            )
          )
        )
      )

      (defun AXListeProfils   ( / hold)
        (vlax-invoke-method
          (vlax-get-property (VLxx_AcadPrefs) "Profiles")
          'GetAllProfileNames
          'hold
        )
        (if Hold
          (vlax-safearray->list hold)
        )
      )

      ;;;---------------
      ;;; Drawing SetUp.
      ;;;---------------
      ;;; Base Conversion
      (defun NumVaHEX (i / s a)     ;a est entier ou integre
          (setq s "")         ;s est portion ou restant
          (while (> i 0)
            (setq a (rem i 16)
              i (lsh i -4)
            )
            (setq s (strcat (if (< a 10)
                      (chr (+ 48 a)) ;48: (ascii "0")
                      (chr (+ 55 a))
                    )
                    s
                )         ;55: (ascii "A") - 10
            )
          )
      )

      (defun Lay2HTML (/      acad   doc    dwg    layers name   col
                     ltp    lwt    pst    onoff  frz    dat    path
                     olist  outfile       output
                    )
          (setq acad   (vlax-get-acad-object)
                doc    (vla-get-activedocument acad)
                dwg    (vla-get-name doc)
                path   (vla-get-path doc)
                layers (vla-get-layers doc)
          )
          (vlax-for each layers
            (setq name (vla-get-name each)
              col  (itoa (vla-get-color each))
              ltp  (vla-get-linetype each)
              lwt  (itoa (vla-get-lineweight each))
            )
            (cond ((= lwt "-1") (setq lwt "ByLayer"))
              ((= lwt "-2") (setq lwt "ByBlock"))
              ((= lwt "-3") (setq lwt "Default"))
              (T (setq lwt (rtos (* 0.01 (atoi lwt)) 2 2)))
            )
            (setq pst   (vla-get-plotstylename each)
              onoff (if (= :vlax-true (vla-get-layeron each))
                  "On"
                  "Off"
                    )
              frz   (if (= :vlax-true (vla-get-freeze each))
                  "Freeze"
                  "Thaw"
                    )
              lok   (if (= :vlax-true (vla-get-Lock each))
                  "Lock"
                  "Unlock"
                    )
              plt   (if (= :vlax-true (vla-get-Plottable each))
                  "Plottable"
                  "Unplottable"
                    )
              dat   (list name onoff frz lok col ltp lwt pst plt)
              olist (cons dat olist)
            )
          )               ;vlax-for
          (vlax-release-object layers)
          (vlax-release-object doc)
          (vlax-release-object acad)
          (cond
            (olist
             (setq outfile (strcat (vl-filename-base dwg) ".html"))
             (setq outfile (strcat path "/" outfile))
             (cond
               ((setq output (open outfile "w"))
                (write-line "<html>" output)
                (write-line "<head><title>" output)
                (write-line (strcat "Layer list to HTML: " dwg) output)
                (write-line "</title></head><body>" output)
                (write-line
              (strcat "<p><b>Layer listing for " dwg "</b><br>")
              output
                )
                (write-line "</p>" output) ;rajoute une ligne
                (write-line "<table border=1>" output)
                (write-line "<tr bgcolor=\"#FFCCFF\">" output)
                (write-line
              "<td><b>Name&nbsp;&nbsp;&nbsp;&nbsp;</b></td>"
              output
                )
                (write-line "<td><b>On</b></td>" output)
                (write-line "<td><b>Freeze</b></td>" output)
                (write-line "<td><b>Lock</b></td>" output)
                (write-line "<td><b>Colour</b></td>" output)
                (write-line "<td><b>Linetype</b></td>" output)
                (write-line "<td><b>Lineweight</b></td>" output)
                (write-line "<td><b>PlotStyle</b></td>" output)
                (write-line "<td><b>Plottable</b></td>" output)
                (foreach layset olist
              (write-line "<tr>" output)
              (write-line
                (strcat "<td>" (nth 0 layset) "</td>")
                output
              )           ;print name
              (write-line
                (strcat "<td>" (nth 1 layset) "</td>")
                output
              )           ;print onoff
              (write-line
                (strcat "<td>" (nth 2 layset) "</td>")
                output
              )           ;print frz
              (write-line
                (strcat "<td>" (nth 3 layset) "</td>")
                output
              )           ;print lok
              (setq RGBLay (dos_acitorgb (atoi (nth 4 layset)))
                    RLay   (car RGBLay)
                    GLay   (cadr RGBLay)
                    BLay   (caddr RGBLay)
                    Hex1   (NumVaHEX RLay)
                    Hex2   (NumVaHEX GLay)
                    Hex3   (NumVaHEX BLay)
              )
              (if (= Hex1 nil)
                (setq Hex1 "00")
              )
              (if (= Hex2 nil)
                (setq Hex2 "00")
              )
              (if (= Hex3 nil)
                (setq Hex3 "00")
              )
              (if (< (ascii Hex1) 10)
                (setq Hex1 (strcat "0" Hex1))
              )
              (if (< (ascii Hex2) 10)
                (setq Hex2 (strcat "0" Hex2))
              )
              (if (< (ascii Hex3) 10)
                (setq Hex3 (strcat "0" Hex3))
              )
              (setq HexN (strcat Hex1 Hex2 Hex3))
              (write-line
                (strcat "<td bgcolor=#"
                    HexN
                    ">"
                    (nth 4 layset)
                    "</td>"
                )
                output
              )
                          ;print col
              (write-line
                (strcat "<td>" (nth 5 layset) "</td>")
                output
              )           ;print ltp
              (write-line
                (strcat "<td>" (nth 6 layset) "</td>")
                output
              )           ;print lwt
              (write-line
                (strcat "<td>" (nth 7 layset) "</td>")
                output
              )           ;print pst
              (write-line
                (strcat "<td>" (nth 8 layset) "</td>")
                output
              )           ;print plt
              (write-line "</tr>" output)
                )             ;foreach layer set
                (write-line "</table></body></html>" output)
                (close output)
                (setq output nil)
                (princ "\nHTML completed. Now opening...\n")
                (vl-cmdf "_.browser" outfile)
               )
               (T (princ "\nUnable to open HTML file."))
             )
            )
            (T (princ "\nUnable to get layer table info."))
          )
      )

      (defun C:FlushDSU (/)
          (if (/= nil (getdictname "DSU_Cfg"))
            (deldict "DSU_Cfg")
          )
          (princ
            "\n** Stored info regarding DwgSetUp Automatic Layering deleted **\n"
          )
          (princ)
      )

      (TraceIsOn ".20c")

      (defun DSUCalquesRead (/)
          (TraceIsOn "DSUCalquesRead.")
          (princ "\t")
          (setq laylist (acad_strlsort (ai_table "layer" 4))
                laynum  (length laylist)
                count   0
                count1  0
                layinf  nil
          )
          (dos_waitcursor T)
          (while (< count laynum)
            (setq sLayNom (nth count laylist)
              what    (DSUlayer_state sLayNom)
              what    (ins_before 1 sLayNom what)
              layinf  (append layinf (list what))
            )
            (setq count (1+ count))
          )
          (dos_waitcursor)
      )

      ; Passes back a list of states on/off thaw/freeze unlock/lock color linetype
      (defun DSUlayer_state (layer-name / tmp result)
          (TraceIsOn "DSUlayer_state.")
          (if (= 'STR (type layer-name))
            (progn            ; (princ tmp)
              (if (setq tmp (tblsearch "layer" layer-name))
                (setq result (list (cdr (assoc 6 tmp)))
                  result (cons (itoa (abs (cdr (assoc 62 tmp)))) result)
                          ;couleur
                  result (cond ((= 4 (logand (cdr (assoc 70 tmp)) 4))
                          ;lok
                        (cons "LOCK" result)
                       )
                       ((= 0 (logand (cdr (assoc 70 tmp)) 4))
                          ;unlok
                        (cons "UNLOCK" result)
                       )
                     )
                  result (cond ((= 1 (logand (cdr (assoc 70 tmp)) 1))
                          ;frz
                        (cons "FREEZE" result)
                       )
                       ((= 0 (logand (cdr (assoc 70 tmp)) 1))
                          ;thw
                        (cons "THAW" result)
                       )
                     )
                  result (cond ((< (cdr (assoc 62 tmp)) 0)
                        (cons "OFF" result)
                       )
                       ((> (cdr (assoc 62 tmp)) 0)
                        (cons "ON" result)
                       )
                     )
                )
              )
              result
            )
            (progn (princ "\nThe layer name must be a string type.")
               nil
            )
          )
      )

      (defun C:DSU () (DSetUp))

      (defun DSetUp (/ sBlkName fName InsF sizel sizeh)
        (TraceIsOn "DSetUp.")
        (setq OLDERROR *error*
              *error*  Wallserror
              inMS     1
              oTL      (getvar "TILEMODE")
              bReadLF  T
              bReadRA  T
        )
        (WallsMod)                          ;Parceque on n'utilise pas
        (if (or (mod nutime 1.7) (mod nutime 4.0) (mod nutime 6.1)) ;Walls_GatherInfo
          (progn
            (FlashMess3
              "DwgSetUp maintenant Actif."
              "DwgSetUp now Active."
              "DwgSetUp now Active."
              "DwgSetUp now Active."
            )
          )
        )

        (setvar "CMDECHO" 0)
        (command "_.UNDO" "_MARK")
        (princ (strcat "\nDrawing SetUp." copyright))
        (DSUCalquesRead)
        (PropsRead)

        (defun AddBlocksToListBox (/ BlockInfo)
          (TraceIsOn "AddBlocksToListBox.")
          (Odcl_ListBox_Clear NomDuProj "DwgSetUp" "BlockNameList")
          (setq BlockInfo (ai_table "block" 62))
          (foreach item BlockInfo
            (Odcl_ListBox_AddString
              NomDuProj
              "DwgSetUp"
              "BlockNameList"
              item
            )
          )
        )

        (defun c:DwgSetUp_OnInitialize ()
          (TraceIsOn "DwgSetUp_OnInitialize.")
          (AddBlocksToListBox)      ;recherche et montre les blocs
          (setq iIndex 0)

          (if (= bMetUnit T)        ;Section metrique
            (progn
              (setq lScales (list "1"      "2"      "5"      "10"
                                  "20"     "25"     "50"     "100"
                                  "200"    "250"    "500"    "750"
                                  "1000"   "1250"   "2000"   "2500"
                                  "4000"   "5000"   "8000"   "10000"
                            )
              )
              (if (= data nil)
                (setq data "100"
                      new_scale 100
                )
              )

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_Unit
                "Visible"
                T
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_unitmilli
                "Visible"
                T
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_unitcenti
                "Visible"
                T
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_unitmeter
                "Visible"
                T
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_GradLabel
                "Visible"
                nil
              )

              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "unitmilli" "Value" 1)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "unitcenti" "Value" 0)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "unitmeter" "Value" 0)

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_MetricFrame
                "Visible"
                T
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_ImpFrame
                "Visible"
                nil
              )

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea0
                "Caption"
                "A0: 1189x841"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb1
                "Caption"
                "B1: 1000x707"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea1
                "Caption"
                "A1: 841x594"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb2
                "Caption"
                "B2: 707x500"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea2
                "Caption"
                "A2: 594x420"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb3
                "Caption"
                "B3: 500x353"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea3
                "Caption"
                "A3: 420x297"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea4
                "Caption"
                "A4: 210x297"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea4r
                "Caption"
                "A4R: 297x210"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea0
                "ToolTipText"
                "Close to ANSI E"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb1
                "ToolTipText"
                "Close to ARCH D"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea1
                "ToolTipText"
                "Close to ANSI D"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea2
                "ToolTipText"
                "Close to ANSI C"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea3
                "ToolTipText"
                "Close to ANSI B"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea4
                "ToolTipText"
                "Close to ANSI A"
              )

              (if (= new_size nil)
                (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "sizea1" "Value" 1)
              )

            )
            (progn          ;Section imperiale
              (setq lScales (list "1"     "4"     "8"     "12"
                                  "16"    "24"    "32"    "48"
                                  "60"    "64"    "96"    "120"
                                  "128"   "192"   "240"   "360"
                                  "480"   "600"   "720"   "1200"
                                  "2400"  "6000"
                            )
              )
              (if (= data nil)
                 (setq data "48"
                       sGrad "1/4\"=1'"
                       new_scale 48
                 )
              )

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_Unit
                "Visible"
                nil
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_unitmilli
                "Visible"
                nil
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_unitcenti
                "Visible"
                nil
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_unitmeter
                "Visible"
                nil
              )

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_GradLabel
                "Visible"
                T
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_GradLabel
                "Caption"
                sGrad
              )

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_MetricFrame
                "Visible"
                nil
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_ImpFrame
                "Visible"
                T
              )

              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea0
                "Caption"
                "ANSI E: 44x34"
              )             ;A0
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb1
                "Caption"
                "ARCH D: 36x24"
              )             ;B1
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea1
                "Caption"
                "ANSI D: 34x22"
              )             ;A1
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb2
                "Caption"
                "ARCH C: 24x18"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea2
                "Caption"
                "ANSI C: 22x17"
              )             ;A2
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb3
                "Caption"
                ": ??"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea3
                "Caption"
                "ANSI B: 17x11"
              )             ;A3
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea4
                "Caption"
                "ANSI A: 8.5x11"
              )             ;A4
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea4r
                "Caption"
                ": ??"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea0
                "ToolTipText"
                "Close to ISO A0"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizeb1
                "ToolTipText"
                "Close to ISO B1"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea1
                "ToolTipText"
                "Close to ISO A1"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea2
                "ToolTipText"
                "Close to ISO A2"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea3
                "ToolTipText"
                "Close to ISO A3"
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_sizea4
                "ToolTipText"
                "Close to ISO A4"
              )

            )
          )

          (Odcl_ComboBox_AddList
            DSetUpForm_DwgSetUp_ScaleListComboBox
            lScales
          )
          (Odcl_ComboBox_SelectString
            NomDuProj
            "DwgSetUp"
            "ScaleListComboBox"
            data
          )

          (setq iComLine (AXDispCmdLines nil))

          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CommandTextBox
            "Text"
            (itoa iComLine)
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CommandSliderBar
            "Value"
            iComLine
          )

          (foreach lItem LayInf
            (Odcl_AxControl_DoMethod
              DSetUpForm_DwgSetUp_AcadLayerListCtrl
              "AddLayer"
              (cadr lItem)
            )
            (if (= (car lItem) "OFF")
              (Odcl_AxControl_DoMethod
                DSetUpForm_DwgSetUp_AcadLayerListCtrl
                "SetLayerOn"
                iIndex          ;nil=Off T=ON
                nil
              )
            )
            (if (= (nth 2 lItem) "FREEZE")
              (Odcl_AxControl_DoMethod
                DSetUpForm_DwgSetUp_AcadLayerListCtrl
                "SetLayerThaw"
                iIndex
                nil         ;nil=frz T=thw
              )
            )
            (if (= (nth 3 lItem) "LOCK")
              (Odcl_AxControl_DoMethod
                DSetUpForm_DwgSetUp_AcadLayerListCtrl
                "SetLayerLock"
                iIndex
                nil         ;nil=lock T=unlock
              )
            )
            (Odcl_AxControl_DoMethod
              DSetUpForm_DwgSetUp_AcadLayerListCtrl
              "SetLayerColor"
              iIndex
              (read (nth 4 lItem))
            )
            (setq iIndex (1+ iIndex))
          )
          (setq iLayCount (VLxx_CollectionCount (VLxx_GetLayers)))
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_NumLayersLabel
            "Caption"
            (itoa iLayCount)
          )

          (defun c:DwgSetUp_AcadLayerListCtrl_SelChanged
            (currentLayer / SSObj NumOb)
            ;;(Odcl_MessageBox "To Do: code must be added to SelChanged" (etos currentLayer))
            (Setq sCalque currentLayer)
            (Setq
              iNIndx (Odcl_AxControl_DoMethod
                   DSetUpForm_DwgSetUp_AcadLayerListCtrl
                   "FindLayer"
                   sCalque
                 )
            )               ;ca marche
            (setq SSObj (ssget "_X" (list (cons 8 sCalque))))
            (if SSobj
              (setq NumOb (rtos (sslength SSObj) 2 0))
              (setq NumOb "Nil")
            )
            (Odcl_Control_SetProperty
              DSetUpForm_DwgSetUp_NumObjLabel
              "Caption"
              NumOb
            )
          )

          (defun c:DwgSetUp_AcadLayerListCtrl_OnOff (layerName state /)
            ;;(Odcl_MessageBox "To Do: code must be added to OnOff" (Strcat (etos LayerName) "-" (etos state)))
            (Setq sCalque layerName)
            (Setq
              iNIndx (Odcl_AxControl_DoMethod
                   DSetUpForm_DwgSetUp_AcadLayerListCtrl
                   "FindLayer"
                   sCalque
                 )
            )               ;ca marche
            (setq SSObj (ssget "_X" (list (cons 8 sCalque))))
            (if SSobj
              (setq NumOb (rtos (sslength SSObj) 2 0))
              (setq NumOb "Nil")
            )
            (Odcl_Control_SetProperty
              DSetUpForm_DwgSetUp_NumObjLabel
              "Caption"
              NumOb
            )
            (if (= (car (nth iNIndx layinf)) "OFF") ;Si c'est off
              (progn (command "_.LAYER" "_On" sCalque "")
                 (setq NewInf (replace_n 0 "ON" (nth iNIndx layinf))
                   layinf (replace_n iNIndx NewInf layinf)
                 )
              )
              (progn (command "_.LAYER" "_Off" sCalque "")
                 (setq NewInf (replace_n 0 "OFF" (nth iNIndx layinf))
                   layinf (replace_n iNIndx NewInf layinf)
                 )
              )
            )
          )

          (defun c:DwgSetUp_AcadLayerListCtrl_ThawFreeze
            (layerName state /)    ;state seems to be nil only!
            ;;(Odcl_MessageBox "To Do: code must be added to ThawFreeze" (Strcat (etos LayerName) "-" (etos state)))
            (Setq sCalque layerName)
            (Setq
              iNIndx (Odcl_AxControl_DoMethod
                   DSetUpForm_DwgSetUp_AcadLayerListCtrl
                   "FindLayer"
                   sCalque
                 )
            )               ;ca marche
            (setq SSObj (ssget "_X" (list (cons 8 sCalque))))
            (if SSobj
              (setq NumOb (rtos (sslength SSObj) 2 0))
              (setq NumOb "Nil")
            )
            (Odcl_Control_SetProperty
              DSetUpForm_DwgSetUp_NumObjLabel
              "Caption"
              NumOb
            )
            (if (= (caddr (nth iNIndx layinf)) "FREEZE") ;Si c'est frz
              (progn (command "_.LAYER" "_T" sCalque "")
                 (setq NewInf (replace_n 2 "THAW" (nth iNIndx layinf))
                   layinf (replace_n iNIndx NewInf layinf)
                 )
              )
              (progn (command "_.LAYER" "_F" sCalque "")
                 (setq NewInf (replace_n 2 "FREEZE" (nth iNIndx layinf))
                   layinf (replace_n iNIndx NewInf layinf)
                 )
              )
            )
          )

          (TraceIsOn ".21")

          (defun c:DwgSetUp_AcadLayerListCtrl_LockUnlock
            (layerName state /)
            ;;(Odcl_MessageBox "To Do: code must be added to LockUnlock" (Strcat (etos LayerName) "-" (etos state)))
            ;;(Setq iIndx2 (Odcl_AxControl_DoMethod DSetUpForm_DwgSetUp_AcadLayerListCtrl "GetSelLayer"))
            (Setq sCalque layerName)
            (Setq iNIndx (Odcl_AxControl_DoMethod
                   DSetUpForm_DwgSetUp_AcadLayerListCtrl
                   "FindLayer"
                   sCalque
                 )
            )               ;ca marche
            (setq SSObj (ssget "_X" (list (cons 8 sCalque))))
            (if SSobj
              (setq NumOb (rtos (sslength SSObj) 2 0))
              (setq NumOb "Nil")
            )
            (Odcl_Control_SetProperty
              DSetUpForm_DwgSetUp_NumObjLabel
              "Caption"
              NumOb
            )
            (if (= (cadddr (nth iNIndx layinf)) "LOCK") ;Si c'est frz
              (progn (command "_.LAYER" "_U" sCalque "")
                 (setq NewInf (replace_n 3 "UNLOCK" (nth iNIndx layinf))
                   layinf (replace_n iNIndx NewInf layinf)
                 )
              )
              (progn (command "_.LAYER" "_LO" sCalque "")
                 (setq NewInf (replace_n 3 "LOCK" (nth iNIndx layinf))
                   layinf (replace_n iNIndx NewInf layinf)
                 )
              )
            )
          )

          (defun c:DwgSetUp_DeleteTextButton_OnClicked ()
            (Setq iNIndx (Odcl_AxControl_DoMethod
                   DSetUpForm_DwgSetUp_AcadLayerListCtrl
                   "FindLayer"
                   sCalque
                 )
            )               ;ca marche
            (cond
              ((or (/= sCalque "0")
               (/= (strcase sCalque nil) "DEFPOINTS")
               )
               (if (= (ssget "_X" (list (cons 8 sCalque))) nil)
             (progn
               (Odcl_AxControl_DoMethod
                 DSetUpForm_DwgSetUp_AcadLayerListCtrl
                 "RemoveLayer"
                 iNIndx
               )
               (command "_.PURGE" "_LA" sCalque "_N")
             )
               )
              )
            )
          )

          (defun c:DwgSetUp_CurrentTextButton_OnClicked ()
            (if (/= (caddr (nth iNIndx layinf)) "FREEZE")
                        ;Si c'est frz
              (progn (command "_.LAYER" "_S" sCalque "")
                        ;(setvar "clayer" sCalque)
              )
            )
          )

          (defun c:DwgSetUp_HTMLTextButton_OnClicked () (Lay2HTML))

          (defun c:DwgSetUp_BlockNameList_SelectionChanged
            (Selection sSelText /)
            (setq sBlkName sSelText)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "BlockView" "BlockName" sBlkName)
            (Odcl_BlockView_DisplayBlock
              NomDuProj "DwgSetUp" "BlockView" sBlkName 0)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "TopGraphicButton" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "FrontGraphicButton" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "RightGraphicButton" "BackColor" -16)
          )
          (defun c:DwgSetUp_TopGraphicButton_Clicked ()
            (Odcl_BlockView_DisplayBlock
              NomDuProj "DwgSetUp" "BlockView" sBlkName 0)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "TopGraphicButton" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "FrontGraphicButton" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "RightGraphicButton" "BackColor" -16)
          )
          (defun c:DwgSetUp_FrontGraphicButton_Clicked ()
            (Odcl_BlockView_DisplayBlock
              NomDuProj "DwgSetUp" "BlockView" sBlkName 4)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "TopGraphicButton" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "FrontGraphicButton" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "RightGraphicButton" "BackColor" -16)
          )
          (defun c:DwgSetUp_RightGraphicButton_Clicked ()
            (Odcl_BlockView_DisplayBlock
              NomDuProj "DwgSetUp" "BlockView" sBlkName 2)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "TopGraphicButton" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "FrontGraphicButton" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "RightGraphicButton" "BackColor" -18)
          )
          (defun c:DwgSetUp_niente_Clicked (/)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "BlockView" "RenderMode" 2)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "niente" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "cache1" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "ombre" "BackColor" -16)
          )
          (defun c:DwgSetUp_cache1_Clicked (/)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "BlockView" "RenderMode" 3)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "niente" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "cache1" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "ombre" "BackColor" -16)
          )
          (defun c:DwgSetUp_ombre_Clicked (/)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "BlockView" "RenderMode" 5)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "niente" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "cache1" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "ombre" "BackColor" -18)
          )
          (defun c:DwgSetUp_orbiting_Clicked (/)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "BlockView" "InterfaceMode" 1)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "orbiting" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "panning" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "zooming" "BackColor" -16)
          )
          (defun c:DwgSetUp_panning_Clicked (/)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "BlockView" "InterfaceMode" 2)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "orbiting" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "panning" "BackColor" -18)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "zooming" "BackColor" -16)
          )
          (defun c:DwgSetUp_zooming_Clicked (/)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "BlockView" "InterfaceMode" 3)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "orbiting" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "panning" "BackColor" -16)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "zooming" "BackColor" -18)
          )

          ;The vlax-get-acad-object function returns a pointer to the AutoCAD Application object.
          (setq EcranQuest
             (VLxx_GetPrefKey "Display" "DisplayScreenMenu")
          )
          (if (= EcranQuest :vlax-true)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_smnu" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_smnu" "Value" 0)
          )

          (setq ScrollQuest
             (VLxx_GetPrefKey "Display" "DisplayScrollBars")
          )

          (if (= ScrollQuest :vlax-true)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_scrb" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_scrb" "Value" 0)
          )

          (setq LTabsQuest
             (VLxx_GetPrefKey "Display" "DisplayLayoutTabs")
          )
          (if (= LTabsQuest :vlax-true)
            (progn
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_LTab" "Value" 1)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "NLTab" "Enabled" 1)
            )
            (progn
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_LTab" "Value" 0)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "NLTab" "Enabled" 0)
            )
          )

          (if (= (Odcl_Control_GetProperty
                   NomDuProj
                   "DwgSetUp"
                   "tog_LTab"
                   "Value"
                 )
                 1
              )
            (progn
              (VLxx_SetPrefKey "Display" "DisplayLayoutTabs" :vlax-true)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "NLTab" "Enabled" 1)
            )
          )
          (if (= (Odcl_Control_GetProperty
                   NomDuProj
                   "DwgSetUp"
                   "tog_LTab"
                   "Value"
                 )
                 0
              )
            (progn
              (VLxx_SetPrefKey
                 "Display"
                 "DisplayLayoutTabs"
                 :vlax-false
              )
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "NLTab" "Enabled" 0)
            )
          )

          (if (= (Odcl_Control_GetProperty
                   NomDuProj
                   "DwgSetUp"
                   "InsF"
                   "Value"
                 )
                 1
              )
            (progn
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "InsF" "Value" 1)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "Load" "Enabled" 1)
            )
            (progn
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "InsF" "Value" 0)
              (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "Load" "Enabled" 0)
            )
          )

          (if (or (= (getvar "UCSICON") 1) (= (getvar "UCSICON") 3))
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_ucs" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_ucs" "Value" 0)
          )
          (if (= (getvar "GRIDMODE") 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_grid" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_grid" "Value" 0)
          )

          (if (= (getvar "LWDISPLAY") 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_lwt" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_lwt" "Value" 0)
          )
          (if (= (getvar "ORTHOMODE") 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_ortho" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_ortho" "Value" 0)
          )

          (if (= (getvar "SNAPMODE") 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_snap" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_snap" "Value" 0)
          )
          (if (= (getvar "BLIPMODE") 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_blip" "Value" 1)
            (Odcl_Control_SetProperty NomDuProj "DwgSetUp" "tog_blip" "Value" 0)
          )

          (while (< count1 laynum)
            (setq sLayNom1 (nth count1 laylist)
                  what1    (DSUlayer_state sLayNom1)
            )
            (if (and (= (car what1) "ON") (= (cadr what1) "THAW"))
                        ;Tant qu'ils sont visibles
              (progn (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxB
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxC
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxDim
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxH
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxI
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxO
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxT
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxV
                   sLayNom1
                 )
                 (Odcl_ComboBox_AddString
                   DSetUpForm_DwgSetUp_ComboBoxPlt
                   sLayNom1
                 )
              )
            )
            (setq count1 (1+ count1))
          )

          (if (= nil (getdictname "DSU_Cfg")) ;S'il n'existe pas deja
            (progn (createdict "DSU_Cfg") ;creer un dico
               (addrecord "DSU_Cfg" "BlkID" (list (cons 300 "0")))
                        ;avec les defauts suivants
               (addrecord "DSU_Cfg" "XLiID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "DimID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "HatID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "ImgID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "OLEID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "TxtID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "VptID" (list (cons 300 "0")))
               (addrecord "DSU_Cfg" "PltID" (list (cons 300 "0")))

               (addrecord "DSU_Cfg" "BlkTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxB
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "XLiTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxC
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "DimTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxDim
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "HatTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxH
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "ImaTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxI
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "OleTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxO
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "TxtTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxT
                 "Enabled"
                 0
               )
               (addrecord "DSU_Cfg" "VptTg" (list (cons 300 "0")))
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxV
                 "Enabled"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_ComboBoxPlt
                 "Enabled"
                 1
               )
               (addrecord "DSU_Cfg" "ZSvTg" (list (cons 300 "0")))

               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxB
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxC
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxDim
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxH
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxI
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxO
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxT
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxV
                 "Value"
                 0
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxPlt
                 "Value"
                 1
               )
               (Odcl_Control_SetProperty
                 DSetUpForm_DwgSetUp_CheckBoxZ
                 "Value"
                 0
               )
            )
            (progn
              (setq bBlk (read (dxf 300 (getrecord "DSU_Cfg" "BlkTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxB
            "Value"
            bBlk
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxB
            "Enabled"
            bBlk
              )
              (setq bXLi (read (dxf 300 (getrecord "DSU_Cfg" "XLiTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxC
            "Value"
            bXLi
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxC
            "Enabled"
            bXLi
              )
              (setq bDim (read (dxf 300 (getrecord "DSU_Cfg" "DimTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxDim
            "Value"
            bDim
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxDim
            "Enabled"
            bDim
              )
              (setq bHat (read (dxf 300 (getrecord "DSU_Cfg" "HatTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxH
            "Value"
            bHat
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxH
            "Enabled"
            bHat
              )
              (setq bImg (read (dxf 300 (getrecord "DSU_Cfg" "ImaTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxI
            "Value"
            bImg
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxI
            "Enabled"
            bImg
              )
              (setq bOLE (read (dxf 300 (getrecord "DSU_Cfg" "OleTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxO
            "Value"
            bOLE
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxO
            "Enabled"
            bOLE
              )
              (setq bTxt (read (dxf 300 (getrecord "DSU_Cfg" "TxtTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxT
            "Value"
            bTxt
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxT
            "Enabled"
            bTxt
              )
              (setq bVpt (read (dxf 300 (getrecord "DSU_Cfg" "VptTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxV
            "Value"
            bVpt
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxV
            "Enabled"
            bVpt
              )
              (setq bZSv (read (dxf 300 (getrecord "DSU_Cfg" "ZSvTg"))))
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CheckBoxZ
            "Value"
            bZSv
              )
            )
          )

          (setq sBlk11 (dxf 300 (getrecord "DSU_Cfg" "BlkID")))
          (setq sXLi11 (dxf 300 (getrecord "DSU_Cfg" "XLiID")))
          (setq sDim11 (dxf 300 (getrecord "DSU_Cfg" "DimID")))
          (setq sHat11 (dxf 300 (getrecord "DSU_Cfg" "HatID")))
          (setq sImg11 (dxf 300 (getrecord "DSU_Cfg" "ImgID")))
          (setq sOLE11 (dxf 300 (getrecord "DSU_Cfg" "OLEID")))
          (setq sTxt11 (dxf 300 (getrecord "DSU_Cfg" "TxtID")))
          (setq sVpt11 (dxf 300 (getrecord "DSU_Cfg" "VptID")))
          (setq sPlt11 (dxf 300 (getrecord "DSU_Cfg" "PltID")))

          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxB
            sBlk11
          )             ;affiche calque
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxC
            sXLi11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxDim
            sDim11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxH
            sHat11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxI
            sImg11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxO
            sOLE11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxT
            sTxt11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxV
            sVpt11
          )
          (Odcl_ComboBox_SelectString
            DSetUpForm_DwgSetUp_ComboBoxPlt
            sPlt11
          )

          (defun c:DwgSetUp_CheckBoxB_OnClicked (nValue /)
            (setq BoxBEna nValue)
            (if (= BoxBEna 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxB
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxB
            "Enabled"
            0
              )
            )
          )
          (defun c:DwgSetUp_CheckBoxC_OnClicked (nValue /)
            (setq BoxCEna nValue)
            (if (= BoxCEna 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxC
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxC
            "Enabled"
            0
              )
            )
          )

          (defun c:DwgSetUp_CheckBoxDim_OnClicked (nValue /)
            (setq BoxDimEna nValue)
            (if (= BoxDimEna 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxDim
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxDim
            "Enabled"
            0
              )
            )
          )
          (defun c:DwgSetUp_CheckBoxH_OnClicked (nValue /)
            (setq BoxHEna nValue)
            (if (= BoxHEna 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxH
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxH
            "Enabled"
            0
              )
            )
          )

          (defun c:DwgSetUp_CheckBoxI_OnClicked (nValue /)
            (if (= nValue 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxI
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxI
            "Enabled"
            0
              )
            )
          )
          (defun c:DwgSetUp_CheckBoxO_OnClicked (nValue /)
            (if (= nValue 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxO
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxO
            "Enabled"
            0
              )
            )
          )

          (defun c:DwgSetUp_CheckBoxT_OnClicked (nValue /)
            (if (= nValue 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxT
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxT
            "Enabled"
            0
              )
            )
          )
          (defun c:DwgSetUp_CheckBoxV_OnClicked (nValue /)
            (if (= nValue 1)
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxV
            "Enabled"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ComboBoxV
            "Enabled"
            0
              )
            )
          )

          (defun c:DwgSetUp_CheckBoxPlt_OnClicked (nValue /)
            (if (= nValue 1)
              (progn (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_inMS
                   "Enabled"
                   1
                 )
                 (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_inPS
                   "Enabled"
                   1
                 )
                 (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_ComboBoxPlt
                   "Enabled"
                   1
                 )
              )
              (progn (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_inMS
                   "Enabled"
                   0
                 )
                 (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_inPS
                   "Enabled"
                   0
                 )
                 (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_ComboBoxPlt
                   "Enabled"
                   0
                 )
              )
            )
          )
          (defun c:DwgSetUp_CheckBoxZ_OnClicked (nValue /)
            (setq bZSv nValue)
          )

          (defun c:DwgSetUp_inMS_OnClicked (nValue /)
            (if (= nValue 1)
              (setq inMS 1
                inPS 0
              )
            )
          )
          (defun c:DwgSetUp_inPS_OnClicked (nValue /)
            (if (= nValue 1)
              (setq inMS 0
                inPS 1
              )
            )
          )

          (defun c:DwgSetUp_Bak_OnClicked () (BakFolder))
          (defun c:DwgSetUp_MouseMove_OnMouseMove (nFlags nX nY /)
            (Odcl_Control_SetProperty
              DSetUpForm_DwgSetUp_MouseMove
              "ToolTipText"
              sNewpath
            )
          )

          (if (/= rLTS4PS nil)
            (progn
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_LTSTextBox
                "Visible"
                1
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_LTSSpinButton
                "Visible"
                1
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_MSTextBox
                "Visible"
                1
              )
            )
            (progn
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_LTSTextBox
                "Visible"
                0
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_LTSSpinButton
                "Visible"
                0
              )
              (Odcl_Control_SetProperty
                DSetUpForm_DwgSetUp_MSTextBox
                "Visible"
                0
              )
            )
          )

          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_MSTextBox
            "Text"
            (rtos (getvar "USERR2"))
          )

          (LesFonctions)

          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_TitleTextBox
            "Text"
            sTitle
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_SubjectTextBox
            "Text"
            sSubject
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_AuthorTextBox
            "Text"
            sAuthor
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_KeywordTextBox
            "Text"
            sKeywords
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CommentTextBox
            "Text"
            sComments
          )
          ;;(Odcl_ComboBox_SetCurSel DSetUpForm_DwgSetUp_ComboBox1 0)

        )

        ;;;-Layer States

        (defun LesFonctions ()
          (TraceIsOn ".LesFonctions")
          (if (= bReadLF T)
            (progn
              (LayerFiltersList)
              (setq iLF (length lFilters))
            )
          )
          (if (= bReadRA T)
            (setq lRegApps (AXListRegApps)
                  iNRA     (length lRegApps)
            )
          )
          (setq lLayoutts (AXLayoutlist)
                 lDimSt   (AXListDimStyles)
                 iDS      (length lDimSt)
                 lOpDwgs  (AXGetOpenDwg)
                 lProf    (AXListeProfils)
                 iProf    (length lProf)
          )

          (Odcl_Tree_Clear DSetUpForm_DwgSetUp_PurgeTreeControl)

          ;;;-Dimensions Styles
          (setq iCnt 1
                sItem nil
                lNew nil
          )
          (Odcl_Tree_AddParent
            DSetUpForm_DwgSetUp_PurgeTreeControl
            (strcat "Dimensions Styles  [" (itoa iDS) "]")
            "ds0"
            0
            0
          )
          (foreach item lDimSt
            (setq sItem (list "ds0" item (strcat "ds" (itoa iCnt)) 2 3))
            (setq lNew (append lNew (list sItem)))
            (setq iCnt (1+ iCnt))
          )
          (Odcl_Tree_AddChild
            DSetUpForm_DwgSetUp_PurgeTreeControl
            lNew
          )

          ;;;-Layer Filters
          (setq iCnt 1
                sItem nil
                lNew nil
          )
          (Odcl_Tree_AddParent
            DSetUpForm_DwgSetUp_PurgeTreeControl
            (strcat "Layer Filters  [" (itoa iLF) "]")
            "lf0"
            0
            0
          )
          (Odcl_Tree_SetExpandedImage
            DSetUpForm_DwgSetUp_PurgeTreeControl
            "lf0"
            1
          )

          (foreach item lFilters
            (setq sItem (list "lf0" item (strcat "lf" (itoa iCnt)) 2 3))
            (setq lNew (append lNew (list sItem)))
            (setq iCnt (1+ iCnt))
          )
          ;;(Odcl_Tree_AddChild DSetUpForm_DwgSetUp_PurgeTreeControl lNew)
          (setq lFilters nil)

          ;;;-Layouts
          (setq iCnt 1
                sItem nil
                lNew nil
          )
          (Odcl_Tree_AddParent
            DSetUpForm_DwgSetUp_PurgeTreeControl
            "Layouts "           "la0"
            0
            0
          )
          ;;(Odcl_Tree_SetExpandedImage DSetUpForm_DwgSetUp_PurgeTreeControl "la0" 1)
          (foreach item lLayoutts
            (setq sItem (list "la0" item (strcat "la" (itoa iCnt)) 2 3))
            (setq lNew (append lNew (list sItem)))
            (setq iCnt (1+ iCnt))
          )
          (Odcl_Tree_AddChild
            DSetUpForm_DwgSetUp_PurgeTreeControl
            lNew
          )

          ;;;-Open drawings
          (setq iCnt 1
                sItem nil
                lNew nil
          )
          (Odcl_Tree_AddParent
            DSetUpForm_DwgSetUp_PurgeTreeControl
            (strcat "Open drawings  [" (itoa (AXDocCount)) "]")
            "od0"
            0
            0
          )
          ;;(Odcl_Tree_SetExpandedImage DSetUpForm_DwgSetUp_PurgeTreeControl "od0" 1)
          (foreach item lOpDwgs
            (setq sItem (list "od0" item (strcat "od" (itoa iCnt)) 2 3))
            (setq lNew (append lNew (list sItem)))
            (setq iCnt (1+ iCnt))
          )
          (Odcl_Tree_AddChild
            DSetUpForm_DwgSetUp_PurgeTreeControl
            lNew
          )

          ;;;-Profiles
          (setq iCnt 1
                sItem nil
                lNew nil
          )
          (Odcl_Tree_AddParent
            DSetUpForm_DwgSetUp_PurgeTreeControl
            (strcat "Profiles  [" (itoa iProf) "]")
            "pr0"
            0
            0
          )
          (foreach item lProf
            (setq sItem (list "pr0" item (strcat "pr" (itoa iCnt)) 2 3))
            (setq lNew (append lNew (list sItem)))
            (setq iCnt (1+ iCnt))
          )
          (Odcl_Tree_AddChild
            DSetUpForm_DwgSetUp_PurgeTreeControl
            lNew
          )

          ;;;-Reg Applications
          (setq iCnt 1
                sItem nil
                lNew nil
          )
          (Odcl_Tree_AddParent
            DSetUpForm_DwgSetUp_PurgeTreeControl
            (strcat "Registered applications  [" (itoa iNRA) "]")
            "ra0"
            0
            0
          )
          ;;(Odcl_Tree_SetExpandedImage DSetUpForm_DwgSetUp_PurgeTreeControl "ra0" 1)
          (foreach item lRegApps
            (setq sItem (list "ra0" item (strcat "ra" (itoa iCnt)) 2 3))
            (setq lNew (append lNew (list sItem)))
            (setq iCnt (1+ iCnt))
          )
          (Odcl_Tree_AddChild
            DSetUpForm_DwgSetUp_PurgeTreeControl
            lNew
          )

          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LATextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_PurgeAllTextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_DelRATextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_DelAllRATextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ExportTextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ImportTextButton
            "Enabled"
            0
          )
        )

        (defun c:DwgSetUp_ComboBox1_OnSelChanged
               (nSelection sSelText /)
          (TraceIsOn "DwgSetUp_ComboBox1_OnSelChanged.")
          (setq sSite sSelText)
        )

        (defun c:DwgSetUp_QueryTextBox_OnReturnPressed ()
          ;;(Odcl_MessageBox "To Do: code must be added to QueryTextBox" "To do")
          ;;(Setq sSite (Odcl_Control_GetProperty DSetUpForm_DwgSetUp_ComboBox1 "Text"))
          (Setq ListeMots nil
            sQuery    (Odcl_Control_GetProperty
                    DSetUpForm_DwgSetUp_QueryTextBox
                    "Text"
                  )
          )
          (if (/= "" sQuery)
            (progn (setq ListeMots (strfind sQuery "+"))
               (if (= (cadr ListeMots) nil)
                 (setq ListeMots (strcat sQuery "+"))
               )
               (ForumSearch)
            )
          )
        )

        ;;;-----------------------------------
        (defun ForumSearch (/ sURL Url)
          (TraceIsOn "ForumSearch.")
          (or *acad* (setq *acad* (vlax-get-acad-object)))
          (or *doc* (setq *doc* (vla-get-activedocument *acad*)))
          (or *utility* (setq *utility* (vla-get-utility *doc*)))
          ;;  (setq sURL (strcat "http://groups.google.com/groups?q=" "group%3Aautocad.general"))   ;marche pas
          (setq sURL
             (strcat
               "http://groups.google.com/groups?q=group%3Aautodesk."
               sSite
             )
          )
          (foreach Mot ListeMots
            (setq sURL (strcat sURL "+%22" (RajoutDePlus Mot) "%22"))
          )
          (setq sURL (strcat sURL "&ie=UTF-8&oe=UTF-8&hl=en"))
          (vla-launchbrowserdialog
            *utility* 'Url "Title" "Open" sURL "" :vlax-true)
          Url
        )

        (defun RajoutDePlus (sString / n)
          (TraceIsOn "RajoutDePlus.")
          (while (setq n (vl-string-search " " sString))
            (setq sString (vl-string-subst "+" " " sString n))
          )
          sString
        )

        ;;;------------
        (defun c:DwgSetUp_PurgeTreeControl_OnSelChanged (sSelText SelKey /)
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LATextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_PurgeAllTextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_DelRATextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_DelAllRATextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ExportTextButton
            "Enabled"
            0
          )
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_ImportTextButton
            "Enabled"
            0
          )
          ;; (Odcl_MessageBox "To Do: code must be added to PurgeTreeControl" (strcat sSelText "-" SelKey))
          (setq sMot sSelText
            sCle SelKey
          )
          (cond ((and (wcmatch sCle "la*") (= sCle "la0"))
             (setq bELabel T)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_LATextButton
               "Enabled"
               1
             )
            )
            ((= sCle "ds0")
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelRATextButton
               "Enabled"
               0
             )
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelAllRATextButton
               "Enabled"
               1
             )
            )
            ((and (wcmatch sCle "ds*") (/= sCle "ds0"))
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelRATextButton
               "Enabled"
               1
             )
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelAllRATextButton
               "Enabled"
               0
             )
            )

            ((= sCle "pr0")
             (setq bELabel nil)
             (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_ImportTextButton
                   "Enabled"
                   1
             )
            )

            ((and (wcmatch sCle "pr*") (/= sCle "pr0"))
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelRATextButton
               "Enabled"
               1
             )
             (Odcl_Control_SetProperty
                   DSetUpForm_DwgSetUp_ExportTextButton
                   "Enabled"
                   1
             )
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelAllRATextButton
               "Enabled"
               0
             )
            )

            ((and (= sCle "lf0") (> iLF 0))
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelAllRATextButton
               "Enabled"
               1
             )
            )
            ((= sCle "od0")
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_PurgeAllTextButton
               "Enabled"
               1
             )
            )
            ((= sCle "ra0")
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelRATextButton
               "Enabled"
               0
             )
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelAllRATextButton
               "Enabled"
               1
             )
            )
            ((and (wcmatch sCle "ra*") (/= sCle "ra0"))
             (setq bELabel nil)
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelRATextButton
               "Enabled"
               1
             )
             (Odcl_Control_SetProperty
               DSetUpForm_DwgSetUp_DelAllRATextButton
               "Enabled"
               0
             )
            )
          )

          ;;  (if (= nil bELabel)
          ;;    (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_PurgeTreeControl "EditLabel" 0)
          ;;    (Odcl_Control_SetProperty DSetUpForm_DwgSetUp_PurgeTreeControl "EditLabel" 1)
          ;;  )

          (Setq itm (Odcl_Tree_GetItem
                  DSetUpForm_DwgSetUp_PurgeTreeControl
                  sCle
                )
          )             ;("DRC_DORATT" 2 3 "ra5")
          (Setq idt (Odcl_Tree_GetItemData
                  DSetUpForm_DwgSetUp_PurgeTreeControl
                  sCle
                )
          )             ;0
          (Setq itx (Odcl_Tree_GetItemText
                  DSetUpForm_DwgSetUp_PurgeTreeControl
                  sCle
                )
          )             ;"DRC_DORATT"
          (Setq sit (Odcl_Tree_GetSelectedItem
                  DSetUpForm_DwgSetUp_PurgeTreeControl
                )
          )             ;"ra5"
          (Setq rVa (Odcl_Tree_GetVisibleCount
                  DSetUpForm_DwgSetUp_PurgeTreeControl
                )
          )             ;visible
          (Setq rVb (Odcl_Tree_CountItems
                  DSetUpForm_DwgSetUp_PurgeTreeControl
                )
          )             ;total
          ;;(Odcl_Tree_SelectItem DSetUpForm_DwgSetUp_PurgeTreeControl Key [as String or Long])
        )

        (defun c:DwgSetUp_LATextButton_OnClicked ()
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_OnError2
            "Caption"
            " "
          )
          (AXLayoutsSort)
          (LesFonctions)
          (princ)
        )

        (defun c:DwgSetUp_PurgeTreeControl_OnEditLabelItem (sItemText Key /)
          ;;(Odcl_MessageBox "To Do: code must be added to PurgeTreeControl" (strcat sItemText "-" Key))
          (cond ((and (wcmatch sCle "la*") (/= sCle "la0"))
             (setq sLNiuNam sItemText)
             (AXRenLayout sMot sItemText)
            )
          )
          (princ)
        )

        (defun c:DwgSetUp_ExportTextButton_OnClicked ()
          (setq sARGFolder (dos_getdir "Sefect folder for ARG file"))
          (if (/= sARGFolder nil)
            (AXExportProfil sMot (strcat sARGFolder sMot ".arg") T)
          )
          (LesFonctions)
          (princ)
        )

        (defun c:DwgSetUp_ImportTextButton_OnClicked ()
          (setq sARGFolder (getfiled "Import Profile" sMot "arg" 2))
          (if (/= sARGFolder nil)
               (progn
            (setq sARGFile (last (destraliste sARGFolder "\\")))
            (AXImportProfil sARGFolder (car (strfind sARGFile ".")))
           )
          )
          (LesFonctions)
          (princ)
        )

        (defun c:DwgSetUp_PurgeAllTextButton_OnClicked ()
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_OnError2
            "Caption"
            " "
          )
          (TraceIsOn "DwgSetUp_PurgeAllTextButton_OnClicked.")
          (AXPurgeAll)
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_OnError2
            "Caption"
            "Purge complete."
          )
          ;;  (princ "\nPurge complete!")
        )

        (defun c:DwgSetUp_DelRATextButton_OnClicked ()
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_OnError2
            "Caption"
            " "
          )
          (TraceIsOn "DwgSetUp_DelRATextButton_OnClicked.")
          (cond ((wcmatch sCle "ds*")
             (AXRemDimStyle)
             (if (= bDSDisp nil)
               (progn
                 (Odcl_Tree_DeleteItem
                   DSetUpForm_DwgSetUp_PurgeTreeControl
                   sCle
                 )
               )
             )
            )
            ((wcmatch sCle "pr*")
                     (if (= sMot "<<Unnamed Profile>>")
                         (setq bUnnamed T)
                         (setq bUnnamed nil)
             )
                     (if (= sMot (getvar 'CPROFILE))
                         (setq bCProf T)
                         (setq bCProf nil)
             )
             (AXEffaceProfile sMot)
             (LesFonctions)
             (princ)
            )
            ((wcmatch sCle "ra*")
             (AXRemApid)
             (if (= bRADisp nil)
               (Odcl_Tree_DeleteItem
                 DSetUpForm_DwgSetUp_PurgeTreeControl
                 sCle
               )
             )
            )
          )
          (LesFonctions)
          (princ)
        )

        (TraceIsOn ".22")

        (defun c:DwgSetUp_DelAllRATextButton_OnClicked ()
          (TraceIsOn "DwgSetUp_DelAllRATextButton_OnClicked.")
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_OnError2
            "Caption"
            " "
          )
          (cond ((= sCle "lf0")     ;Layer Filters
             (AXLayerFiltersDel)
             (setq bReadLF T
                   bReadRA nil
             )
             (Odcl_Form_Close DSetUpForm_DwgSetUp)

                        ;  (Odcl_Tree_AddParent DSetUpForm_DwgSetUp_PurgeTreeControl
                        ;    (strcat "Layer Filters  [" (itoa 0) "]")
                        ;    "lf0" 0 0)
             ;;      (LesFonctions)
                        ;         (Odcl_Form_Show DSetUpForm_DwgSetUp)
                        ;         (Odcl_Tab_SetCurSel DSetUpForm_DwgSetUp_TabControl1 4)
            )
            ((= sCle "ra0")     ;All Registered Apps.
             (AXRemAppids)
             (setq bReadLF nil
                   bReadRA T
             )
             (LesFonctions)
            )
          )
          (princ "")
        )

        (defun c:DwgSetUp_CheckBoxLTS_OnClicked (nValue /)
          (if (= nValue 1)
            (progn
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LTSTextBox
            "Visible"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LTSSpinButton
            "Visible"
            1
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_MSTextBox
            "Visible"
            1
              )
            )
            (progn
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LTSTextBox
            "Visible"
            0
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LTSSpinButton
            "Visible"
            0
              )
              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_MSTextBox
            "Visible"
            0
              )
            )
          )
        )

        (defun c:DwgSetUp_LTSSpinButton_OnChanged (nValue nDelta /)
          ;; (Odcl_MessageBox "To Do: code must be added to SpinButton1" (strcat (etos nValue) "-" (etos nDelta)))
          (setq rLTS4PS (/ nValue 10.0))
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_LTSTextBox
            "Text"
            (rtos rLTS4PS 2 1)
          )
        )

        (defun c:DwgSetUp_ComboBoxB_OnSelChanged (nSelection sSelText /)
          (setq sBlkLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxC_OnSelChanged (nSelection sSelText /)
          (setq sXLiLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxDim_OnSelChanged (nSelection sSelText /)
          (setq sDimLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxH_OnSelChanged (nSelection sSelText /)
          (setq sHatLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxI_OnSelChanged (nSelection sSelText /)
          (setq sImgLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxO_OnSelChanged (nSelection sSelText /)
          (setq sOLELay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxT_OnSelChanged (nSelection sSelText /)
          (setq sTxtLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxV_OnSelChanged (nSelection sSelText /)
          (setq sVptLay sSelText)
        )
        (defun c:DwgSetUp_ComboBoxPlt_OnSelChanged (nSelection sSelText /)
          (setq sPltLay sSelText)
        )

        (defun c:DwgSetUp_sizea0_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizea0")
          )
        )
        (defun c:DwgSetUp_sizeb1_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizeb1")
          )
        )
        (defun c:DwgSetUp_sizea1_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizea1")
          )
        )
        (defun c:DwgSetUp_sizeb2_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizeb2")
          )
        )
        (defun c:DwgSetUp_sizea2_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizea2")
          )
        )
        (defun c:DwgSetUp_sizeb3_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizeb3")
          )
        )
        (defun c:DwgSetUp_sizea3_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizea3")
          )
        )
        (defun c:DwgSetUp_sizea4_Clicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizea4")
          )
        )
        (defun c:DwgSetUp_sizea4r_OnClicked (nValue /)
          (if (= nValue 1)
            (setq new_size "sizea4r")
          )
        )
        (defun c:DwgSetUp_unitmilli_Clicked (nValue /)
          (if (= nValue 1)
            (setq units1 "unitmilli"
              bmmUn  1
            )
            (setq bmmUn 0)
          )
        )
        (defun c:DwgSetUp_unitcenti_Clicked (nValue /)
          (if (= nValue 1)
            (setq units1 "unitcenti"
              bcmUn  1
            )
            (setq bcmUn 0)
          )
        )
        (defun c:DwgSetUp_unitmeter_Clicked (nValue /)
          (if (= nValue 1)
            (setq units1 "unitmeter"
              bmUn   1
            )
            (setq bmUn 0)
          )
        )

        (defun c:DwgSetUp_tog_blip_Clicked (nValue /)
          (TraceIsOn "blip.")
          (if (= nValue 1)
            (setvar "BLIPMODE" 1)
            (setvar "BLIPMODE" 0)
          )
        )

        (defun c:DwgSetUp_tog_grid_Clicked (nValue /)
          (TraceIsOn "grid.")
          (if (= nValue 1)
            (setvar "GRIDMODE" 1)
            (setvar "GRIDMODE" 0)
          )
          (command "_.regen")
        )

        (defun c:DwgSetUp_tog_ucs_Clicked (nValue /)
          (TraceIsOn "ucs.")
          (if (= nValue 1)
            (setvar "UCSICON" 1)
            (setvar "UCSICON" 0)
          )
          (command "_.regen")
        )

        (defun c:DwgSetUp_tog_lwt_OnClicked (nValue /)
          (TraceIsOn "toglwt.")
          (if (= nValue 1)
            (setvar "LWDISPLAY" 1)
            (setvar "LWDISPLAY" 0)
          )
          (command "_.regen")
        )

        (defun c:DwgSetUp_tog_smnu_Clicked (nValue /)
          (TraceIsOn "togsmnu.")
          (if (= nValue 1)
            (VLxx_SetPrefKey "Display" "DisplayScreenMenu" :vlax-true)
            (VLxx_SetPrefKey "Display" "DisplayScreenMenu" :vlax-false)
          )
        )

        (defun c:DwgSetUp_tog_ortho_Clicked (nValue /)
          (TraceIsOn "togortho.")
          (if (= nValue 1)
            (setvar "ORTHOMODE" 1)
            (setvar "ORTHOMODE" 0)
          )
        )

        (defun c:DwgSetUp_tog_snap_Clicked (nValue /)
          (TraceIsOn "togsnap.")
          (if (= nValue 1)
            (setvar "SNAPMODE" 1)
            (setvar "SNAPMODE" 0)
          )
        )

        (defun c:DwgSetUp_tog_scrb_Clicked (nValue /)
          (TraceIsOn "togscrb.")
          (if (= nValue 1)
            (VLxx_SetPrefKey "Display" "DisplayScrollBars" :vlax-true)
            (VLxx_SetPrefKey "Display" "DisplayScrollBars" :vlax-false)
          )
        )

        (defun c:DwgSetUp_tog_LTab_Clicked (nValue /)
          (TraceIsOn "togLTab.")
          ;;  (Odcl_MessageBox "To Do: code must be added to c:DwgSetUp_tog_LTab_Clicked" (etos nValue) )
          (if (= nValue 1)
            (progn
              (VLxx_SetPrefKey "Display" "DisplayLayoutTabs" :vlax-true)
              (Odcl_Control_SetProperty
            NomDuProj "DwgSetUp" "NLTab" "Enabled" 1)
            )
            (progn (VLxx_SetPrefKey
                 "Display"
                 "DisplayLayoutTabs"
                 :vlax-false
               )
               (Odcl_Control_SetProperty
                 NomDuProj "DwgSetUp" "NLTab" "Enabled" 0)
               (Odcl_Control_SetProperty
                 NomDuProj "DwgSetUp" "NLTab" "Value" 0)
            )
          )
        )

        (defun c:DwgSetUp_NLTab_Clicked (nValue /)
          (TraceIsOn "NLTab.")
          ;;  (Odcl_MessageBox "To Do: code must be added to c:DwgSetUp_NLTab_Clicked" (etos nValue))
          (if (= nValue 1)
            (progn (Odcl_Control_SetProperty
                 NomDuProj "DwgSetUp" "OnError" "Caption"
                 "Please, wait...")
               (AXAddLayout)
               (dos_pause 1)
               (Odcl_Control_SetProperty
                 NomDuProj
                 "DwgSetUp"
                 "OnError"
                 "Caption"
                 "To add more tabs click Add layout."
                )
               (dos_pause 2)
               (Odcl_Control_SetProperty
                 NomDuProj "DwgSetUp" "NLTab" "Value" 0)
               (Odcl_Control_SetProperty
                 NomDuProj "DwgSetUp" "OnError" "Caption" " ")
            )
          )
          (if (= nValue 0)
            (progn (Odcl_Control_SetProperty
                 NomDuProj "DwgSetUp" "OnError" "Caption" " ")
               (rs_error)
            )
          )
        )

        (defun c:DwgSetUp_CommandSliderBar_OnReleasedCapture (nValue /)
          ;;   (Odcl_MessageBox "To Do: code must be added to CommandSliderBar" (etos nValue))
          (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_CommandTextBox
            "Text"
            nValue
          )
          (AXDispCmdLines nValue)
        )

        (TraceIsOn ".23")

        (defun c:DwgSetUp_InsF_Clicked (nValue /)
          (TraceIsOn "InsF_Clicked.")
          (setq InsF nValue)
          (if (= InsF 1)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "Load" "Enabled" 1)
            (Odcl_Control_SetProperty
              NomDuProj "DwgSetUp" "Load" "Enabled" 0)
          )
        )

        (defun c:DwgSetUp_Load_Clicked ()
          (TraceIsOn "Load_Clicked.")
          (setq fname (GetFiled "Select Drawing File" "" "dwg" 4))
          (if (/= fname nil)
            (Odcl_BlockView_LoadDwg
              NomDuProj
              "DwgSetUp"
              "BlockView"
              fname
            )
          )
        )

        (defun c:DwgSetUp_BlockView_OnDblClicked ()
          (if (/= sBlkName nil)     ;if the currently selected item is nil this means nothing is selected and we should do nothing
            (progn          ;otherwise we go ahead, close the dialog box and insert the block.
              (Odcl_Form_Close NomDuProj "DwgSetUp")
                        ;get the point to insert from the user
              (setq
            intPt (upoint 7 "" "Specify insertion point" nil nil)
              )             ;insert the block
              (command "_.INSERT" sBlkName intPt 1 1 pause)
            )
          )
        )

        (defun c:DwgSetUp_BlockView_OnRightDblClick ()
          (if (/= sBlkName nil)     ;if the currently selected item is nil this means nothing is selected and we should do nothing
            (progn          ;otherwise we go ahead, close the dialog box and insert the block.
              (Odcl_Form_Close NomDuProj "DwgSetUp")
                        ;get the point to insert from the user
              (setq
            intPt (upoint 7 "" "Specify insertion point" nil nil)
              )             ;insert the block
              (command "_.INSERT"
                   sBlkName
                   intPt
                   (/ new_scale new_units)
                   (/ new_scale new_units)
                   pause
              )
            )
          )
        )

        ;;;-------------------------
        (defun c:DwgSetUp_about_Clicked () (_about) (princ))

        (defun c:DwgSetUp_Cancel_Clicked ()
          ;;;(Setq bBoxLTS (Odcl_Control_GetProperty DSetUpForm_DwgSetUp_CheckBoxLTS "Value"))
          ;;;(setq rLTS4PS (atof (Odcl_Control_GetProperty DSetUpForm_DwgSetUp_LTSTextBox "Text")))
          ;;;(Setq sBoxMS  (Odcl_Control_GetProperty DSetUpForm_DwgSetUp_MSTextBox "Text"))
          ;;;(setvar "USERR2" (atof sBoxMS))
          (Odcl_Form_Close NomDuProj "DwgSetUp")
          (princ)
        )

        (defun c:DwgSetUp_Ok_Clicked ()
          (Setq sTitle (Odcl_Control_GetProperty
                 DSetUpForm_DwgSetUp_TitleTextBox
                 "Text"
                   )
          )
          (Setq sSubject (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_SubjectTextBox
                   "Text"
                 )
          )
          (Setq sAuthor (Odcl_Control_GetProperty
                  DSetUpForm_DwgSetUp_AuthorTextBox
                  "Text"
                )
          )
          (Setq sKeywords (Odcl_Control_GetProperty
                    DSetUpForm_DwgSetUp_KeywordTextBox
                    "Text"
                  )
          )
          (Setq sComments (Odcl_Control_GetProperty
                    DSetUpForm_DwgSetUp_CommentTextBox
                    "Text"
                  )
          )
          (PropsWrite)
          (Setq bBoxLTS (Odcl_Control_GetProperty
                  DSetUpForm_DwgSetUp_CheckBoxLTS
                  "Value"
                )
          )
          (setq rLTS4PS (atof (Odcl_Control_GetProperty
                    DSetUpForm_DwgSetUp_LTSTextBox
                    "Text"
                      )
                )
          )
          (Setq sBoxMS (Odcl_Control_GetProperty
                 DSetUpForm_DwgSetUp_MSTextBox
                 "Text"
                   )
          )
          (setvar "USERR2" (atof sBoxMS))
          (Odcl_Form_Close NomDuProj "DwgSetUp")

          (if (/= fname nil)
            (setq fname (RepAllStrings fname "\\" "/"))
          )

          (if
            (= (Odcl_Tab_GetCurSel DSetUpForm_DwgSetUp_TabControl1) 0)
             (progn
               (Do_Dwg_SetUp)
               (command "_.RECTANG" (list 0 0) URC)
               (command "_.ZOOM" "_A")
             )
          )
          (if
            (= (Odcl_Tab_GetCurSel DSetUpForm_DwgSetUp_TabControl1) 1)
             (progn
               (Do_Dwg_SetUp)
               (if (or (/= 1 InsF) (= fname nil))
             (princ "")
             (progn
               (command "_.INSERT"
                    (car (strfind fname ".dwg"))
                    "0,0"
                    (/ new_scale new_units)
                    (/ new_scale new_units)
                    "0"
               )
               (command "_.ZOOM" "_A")
             )
               )
             )
          )
          (TraceIsOn ".24")
          (command "_.regenall")
          (setq bBlk (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxB
                   "Value"
                 )
          )             ;ticked on/off
          (setq bXLi (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxC
                   "Value"
                 )
          )             ;ticked on/off
          (setq bDim (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxDim
                   "Value"
                 )
          )
          (setq bHat (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxH
                   "Value"
                 )
          )
          (setq bImg (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxI
                   "Value"
                 )
          )
          (setq bOLE (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxO
                   "Value"
                 )
          )
          (setq bTxt (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxT
                   "Value"
                 )
          )
          (setq bVpt (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxV
                   "Value"
                 )
          )
          (setq bPlt (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxPlt
                   "Value"
                 )
          )
          (setq bZSv (Odcl_Control_GetProperty
                   DSetUpForm_DwgSetUp_CheckBoxZ
                   "Value"
                 )
          )

          (Setq sBlkLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxB
                )
          )
          (Setq sXLiLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxC
                )
          )
          (Setq sDimLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxDim
                )
          )
          (Setq sHatLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxH
                )
          )
          (Setq sImgLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxI
                )
          )
          (Setq sOLELay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxO
                )
          )
          (Setq sTxtLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxT
                )
          )
          (Setq sVptLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxV
                )
          )
          (Setq sPltLay (Odcl_ComboBox_GetTBText
                  DSetUpForm_DwgSetUp_ComboBoxPlt
                )
          )
          (if (= bBlk T)
            (progn
              (changerecord "DSU_Cfg" "BlkID" (list (cons 300 sBlkLay)))
              (changerecord "DSU_Cfg" "BlkTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "BlkTg" (list (cons 300 "0")))
          )
          (if (= bXLi T)
            (progn
              (changerecord "DSU_Cfg" "XLiID" (list (cons 300 sXLiLay)))
              (changerecord "DSU_Cfg" "XLiTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "XLiTg" (list (cons 300 "0")))
          )
          (if (= bDim T)
            (progn
              (changerecord "DSU_Cfg" "DimID" (list (cons 300 sDimLay)))
              (changerecord "DSU_Cfg" "DimTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "DimTg" (list (cons 300 "0")))
          )
          (if (= bHat T)
            (progn
              (changerecord "DSU_Cfg" "HatID" (list (cons 300 sHatLay)))
              (changerecord "DSU_Cfg" "HatTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "HatTg" (list (cons 300 "0")))
          )
          (if (= bImg T)
            (progn
              (changerecord "DSU_Cfg" "ImgID" (list (cons 300 sImgLay)))
              (changerecord "DSU_Cfg" "ImaTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "ImaTg" (list (cons 300 "0")))
          )
          (if (= bOLE T)
            (progn
              (changerecord "DSU_Cfg" "OLEID" (list (cons 300 sOLELay)))
              (changerecord "DSU_Cfg" "OleTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "OleTg" (list (cons 300 "0")))
          )
          (if (= bTxt T)
            (progn
              (changerecord "DSU_Cfg" "TxtID" (list (cons 300 sTxtLay)))
              (changerecord "DSU_Cfg" "TxtTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "TxtTg" (list (cons 300 "0")))
          )
          (if (= bVpt T)
            (progn
              (changerecord "DSU_Cfg" "VptID" (list (cons 300 sVptLay)))
              (changerecord "DSU_Cfg" "VptTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "VptTg" (list (cons 300 "0")))
          )
          (if (= bPlt T)
            (changerecord "DSU_Cfg" "PltID" (list (cons 300 sPltLay)))
          )
          (if (= bZSv T)
            (progn
              (changerecord "DSU_Cfg" "ZSvTg" (list (cons 300 "1")))
            )
            (changerecord "DSU_Cfg" "ZSvTg" (list (cons 300 "0")))
          )
        )

        (defun c:DwgSetUp_ScaleListComboBox_SelectionChanged
               (Selection sSelText /)
                        ;  (Odcl_MessageBox "To Do: code must be added to c:DwgSetUp_ScaleList_SelectionChanged" (etos sSelText))
          (TraceIsOn "ScaleListComboBox_SelectionChanged.")
          (setq data sSelText)

          (if (= bMetUnit T)        ;Section metrique
            (progn
              (cond ((= data "1") (setq new_scale 1))
                ((= data "2") (setq new_scale 2))
                ((= data "5") (setq new_scale 5))
                ((= data "10") (setq new_scale 10))
                ((= data "20") (setq new_scale 20))
                ((= data "25") (setq new_scale 25))
                ((= data "50") (setq new_scale 50))
                ((= data "100") (setq new_scale 100))
                ((= data "200") (setq new_scale 200))
                ((= data "250") (setq new_scale 250))
                ((= data "500") (setq new_scale 500))
                ((= data "750") (setq new_scale 750))
                ((= data "1000") (setq new_scale 1000))
                ((= data "1250") (setq new_scale 1250))
                ((= data "2000") (setq new_scale 2000))
                ((= data "2500") (setq new_scale 2500))
                ((= data "4000") (setq new_scale 4000))
                ((= data "5000") (setq new_scale 5000))
                ((= data "8000") (setq new_scale 8000))
                ((= data "10000") (setq new_scale 10000))
              )

            )
            (progn
              (cond ((= data "1")
                 (setq new_scale 1
                   sGrad "12\"=1'"
                 )
                )
                ((= data "4")
                 (setq new_scale 4
                   sGrad "3\"=1'"
                 )
                )
                ((= data "8")
                 (setq new_scale 8
                   sGrad     "1 1/2\"=1'"
                 )
                )
                ((= data "12")
                 (setq new_scale 12
                   sGrad "1\"=1'"
                 )
                )
                ((= data "16")
                 (setq new_scale 16
                   sGrad "3/4\"=1'"
                 )
                )
                ((= data "24")
                 (setq new_scale 24
                   sGrad "1/2\"=1'"
                 )
                )
                ((= data "32")
                 (setq new_scale 32
                   sGrad "3/8\"=1'"
                 )
                )
                ((= data "48")
                 (setq new_scale 48
                   sGrad "1/4\"=1'"
                 )
                )
                ((= data "60")
                 (setq new_scale 60
                   sGrad "1\"=5'"
                 )
                )
                ((= data "64")
                 (setq new_scale 64
                   sGrad "3/16\"=1'"
                 )
                )
                ((= data "96")
                 (setq new_scale 96
                   sGrad "1/8\"=1'"
                 )
                )
                ((= data "120")
                 (setq new_scale 120
                   sGrad "1\"=10'"
                 )
                )
                ((= data "128")
                 (setq new_scale 128
                   sGrad "3/32\"=1'"
                 )
                )
                ((= data "192")
                 (setq new_scale 192
                   sGrad "1/16\"=1'"
                 )
                )
                ((= data "240")
                 (setq new_scale 240
                   sGrad "1\"=20'"
                 )
                )
                ((= data "360")
                 (setq new_scale 360
                   sGrad "1\"=30'"
                 )
                )
                ((= data "480")
                 (setq new_scale 480
                   sGrad "1\"=40'"
                 )
                )
                ((= data "600")
                 (setq new_scale 600
                   sGrad "1\"=50'"
                 )
                )
                ((= data "720")
                 (setq new_scale 720
                   sGrad "1\"=60'"
                 )
                )
                ((= data "1200")
                 (setq new_scale 1200
                   sGrad "1\"=100'"
                 )
                )
                ((= data "2400")
                 (setq new_scale 2400
                   sGrad "1\"=200'"
                 )
                )
                ((= data "6000")
                 (setq new_scale 6000
                   sGrad "1\"=500'"
                 )
                )
              )

              (Odcl_Control_SetProperty
            DSetUpForm_DwgSetUp_GradLabel
            "Caption"
            sGrad
              )

            )
          )
        )

        (TraceIsOn ".25")

        (defun Do_Dwg_SetUp ()
          (TraceIsOn ".Do_Dwg_SetUp")
          (setvar "TILEMODE" 1)
          (if (= bMetUnit T)        ;Section metrique
            (progn
              (if (= new_size nil)
            (setq new_size "sizea1")
              )
              (cond ((= new_size "sizea2")
                 (setq sizel 594.0
                   sizeh 420.0
                 )
                )
                ((= new_size "sizea1")
                 (setq sizel 841.0
                   sizeh 594.0
                 )
                )
                ((= new_size "sizea0")
                 (setq sizel 1189.0
                   sizeh 841.0
                 )
                )
                ((= new_size "sizea4")
                 (setq sizel 210.0
                   sizeh 297.0
                 )
                )
                ((= new_size "sizea4r")
                 (setq sizel 297.0
                   sizeh 210.0
                 )
                )
                ((= new_size "sizea3")
                 (setq sizel 420.0
                   sizeh 297.0
                 )
                )
                ((= new_size "sizeb1")
                 (setq sizel 1000.0
                   sizeh 707.0
                 )
                )
                ((= new_size "sizeb2")
                 (setq sizel 707.0
                   sizeh 500.0
                 )
                )
                ((= new_size "sizeb3")
                 (setq sizel 500.0
                   sizeh 353.0
                 )
                )
              )
              (if (= units1 nil)
            (setq units1 "unitmilli")
              )
              (if (= new_scale nil)
            (setq new_scale 1)
              )
              (cond ((= units1 "unitmilli") ;si unite et mm
                 (setvar "lunits" 2)
                 (setvar "luprec" 0)
                 (setq sx 5
                   sy 5
                   gx 10
                   gy 10
                 )
                )
                ((= units1 "unitcenti") ;si unite et cm
                 (setvar "lunits" 2)
                 (setvar "luprec" 0)
                 (setq sx 0.5
                   sy 0.5
                   gx 1
                   gy 1
                 )
                )
                (T          ;sinon unite et m
                 (setvar "lunits" 2)
                 (setvar "luprec" 2)
                 (setq sx 0.05
                   sy 0.05
                   gx 0.10
                   gy 0.10
                 )
                )
              )
              (setq new_units (cond ((= units1 "unitmilli") 1)
                        ((= units1 "unitmeter") 1000)
                        ((= units1 "unitcenti") 10)
                      )
              )
            )               ;progn
            (progn          ;Section imperiale
              (if (= new_size nil)
            (setq new_size "sizeb1")
              )
              (cond ((= new_size "sizea2")
                 (setq sizel 22.0
                   sizeh 17.0
                 )
                )
                ((= new_size "sizea1")
                 (setq sizel 34.0
                   sizeh 22.0
                 )
                )
                ((= new_size "sizea0")
                 (setq sizel 44.0
                   sizeh 34.0
                 )
                )
                ((= new_size "sizea4")
                 (setq sizel 8.5
                   sizeh 11.0
                 )
                )
                ((= new_size "sizea3")
                 (setq sizel 17.0
                   sizeh 11.0
                 )
                )
                ((= new_size "sizeb1")
                 (setq sizel 36.0
                   sizeh 24.0
                 )
                )
                ((= new_size "sizeb2")
                 (setq sizel 24.0
                   sizeh 18.0
                 )
                )
              )
              (setvar "lunits" 4)
              (setvar "luprec" 4)
              (if (= new_scale nil)
            (setq new_scale 1)
              )
              (setq new_units 1)
              (setq sx 0.5
                sy 0.5
                gx 1
                gy 1
              )
            )               ;progn
          )
          (setq URC (list (/ (* sizel new_scale) new_units)
                      (/ (* sizeh new_scale) new_units)
                )
            echelle (FIX new_scale)
          )
          (setvar "limmin" (list 0 0))
          (setvar "limmax" URC)
          (setvar "snapunit" (list (* new_scale sx) (* new_scale sy)))
          (setvar "gridunit" (list (* new_scale gx) (* new_scale gy)))
          (setvar "DIMSCALE" echelle)
          (if (= bMetUnit T)        ;Section metrique
            (setvar "LTSCALE" (* echelle 0.25))
            (setvar "LTSCALE" (* echelle 0.80))
          )
          (setvar "USERR2" (getvar "LTSCALE"))
          (if (= inMS 1)
            (setq cdms echelle)
            (setq cdms 1)
          )
          (setq *VLxx_AcadObject*
             nil
            *VLxx_activedocument*
             nil
            *VLxx_modelspace*
             nil
            cpnt (list (* 2 cdms) (* 2 cdms) 0.0) ;2mm from origin
            chgt (* 2.0 cdms)   ;Text height twice DimScale
          )
          (VLxx_AcadObject)
          (VLxx_activedocument)
          (VLxx_modelspace)
          (princ)
        )

        (princ "\n")
        (DSUDisp)
        (princ "\n")
        (Odcl_Form_Show DSetUpForm_DwgSetUp)
        (setvar "TILEMODE" oTL)
        (setq *error* OLDERROR)
        (princ)
      )

      (TraceIsOn ".20d")

      ;;;----------------
      ;;;---Auto Layering.
      ;;;----------------
      (if *MesReacteurs*
        (vlr-remove *MesReacteurs*)
      )

      (if *MesReacteurs* (vlr-remove *MesReacteurs*)) ;; ### TM : better remove !!
      (setq *MesReacteurs*
        (vlr-command-reactor
           nil
          '((:vlr-commandWillStart . DebutCommand4DHT)
            (:vlr-commandEnded .     FinCommand4DHT)
            (:vlr-commandCancelled . AnnuleCommand4DHT)
            (:vlr-commandFailed .    RateCommand4DHT)
           )
        )
      )

      (defun DebutCommand4DHT (Calling_Reactor commandInfo / commStart)
        (TraceIsOn ".DebutCommand4DHT")
        (setq sCurLayer (getvar 'CLAYER))
        (setq commStart (nth 0 commandInfo))
        (cond
          ((= commStart "DIMLINEAR")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMALIGNED")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMORDINATE")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMRADIUS")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMDIAMETER")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMANGULAR")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "QDIM")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((wcmatch commStart "*LEADER")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMBASELINE")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "DIMCONTINUE")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "TOLERANCE")
           (if (= bDim T)
             (setvar "CLAYER" sDimLay)
           )
          )
          ((= commStart "RAY")
           (if (= bXLi T)
             (setvar "CLAYER" sXLiLay)
           )
          )
          ((= commStart "XLINE")
           (if (= bXLi T)
             (setvar "CLAYER" sXLiLay)
           )
          )
          ((wcmatch commStart "*HATCH")
           (if (= bHat T)
             (setvar "CLAYER" sHatLay)
           )
          )
          ((= commStart "INSERT")
           (if (= bBlk T)
             (setvar "CLAYER" sBlkLay)
           )
          )
          ((= commStart "IMAGE")
           (if (= bImg T)
             (setvar "CLAYER" sImgLay)
           )
          )
          ((= commStart "INSERTOBJ")
           (if (= bOLE T)
             (setvar "CLAYER" sOLELay)
           )
          )
          ((wcmatch commStart "*TEXT")
           (if (= bTxt T)
             (setvar "CLAYER" sTxtLay)
           )
          )
          ((= commStart "MVIEW")
           (if (= bVpt T)
             (setvar "CLAYER" sVptLay)
           )
          )
          ((= commStart "VPORTS")
           (if (= bVpt T)
             (setvar "CLAYER" sVptLay)
           )
          )
        )
        (princ)
      )

      (defun FinCommand4DHT (Calling_Reactor endcommandInfo / commFin)
        (TraceIsOn ".FinCommand4DHT")
        (setq commFin (nth 0 endcommandInfo))
        (cond
          ((= commFin "DIMLINEAR") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMALIGNED") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMORDINATE") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMRADIUS") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMDIAMETER") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMANGULAR") (setvar "CLAYER" sCurLayer))
          ((= commFin "QDIM") (setvar "CLAYER" sCurLayer))
          ((wcmatch commFin "*LEADER") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMBASELINE") (setvar "CLAYER" sCurLayer))
          ((= commFin "DIMCONTINUE") (setvar "CLAYER" sCurLayer))
          ((= commFin "TOLERANCE") (setvar "CLAYER" sCurLayer))
          ((= commFin "RAY") (setvar "CLAYER" sCurLayer))
          ((= commFin "XLINE") (setvar "CLAYER" sCurLayer))
          ((wcmatch commFin "*HATCH") (setvar "CLAYER" sCurLayer))
          ((= commFin "INSERT") (setvar "CLAYER" sCurLayer))
          ((= commFin "IMAGE") (setvar "CLAYER" sCurLayer))
          ((= commFin "INSERTOBJ") (setvar "CLAYER" sCurLayer))
          ((wcmatch commFin "*TEXT") (setvar "CLAYER" sCurLayer))
          ((= commFin "MVIEW") (setvar "CLAYER" sCurLayer))
          ((= commFin "VPORTS") (setvar "CLAYER" sCurLayer))
        )
        (princ)
      )

      (defun AnnuleCommand4DHT (Calling_Reactor cancelcommandInfo / commAnnule)
        (setq commAnnule (nth 0 cancelcommandInfo))
        (cond
          ((= commAnnule "DIMLINEAR") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMALIGNED") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMORDINATE") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMRADIUS") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMDIAMETER") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMANGULAR") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "QDIM") (setvar "CLAYER" sCurLayer))
          ((wcmatch commAnnule "*LEADER") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMBASELINE") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "DIMCONTINUE") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "TOLERANCE") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "RAY") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "XLINE") (setvar "CLAYER" sCurLayer))
          ((wcmatch commAnnule "*HATCH") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "INSERT") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "IMAGE") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "INSERTOBJ") (setvar "CLAYER" sCurLayer))
          ((wcmatch commAnnule "*TEXT") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "MVIEW") (setvar "CLAYER" sCurLayer))
          ((= commAnnule "VPORTS") (setvar "CLAYER" sCurLayer))
        )
        (princ)
      )

      (defun RateCommand4DHT (Calling_Reactor cancelcommandInfo / commRate)
        (setq commRate (nth 0 cancelcommandInfo))
        (cond
          ((= commRate "DIMLINEAR") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMALIGNED") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMORDINATE") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMRADIUS") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMDIAMETER") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMANGULAR") (setvar "CLAYER" sCurLayer))
          ((= commRate "QDIM") (setvar "CLAYER" sCurLayer))
          ((wcmatch commRate "*LEADER") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMBASELINE") (setvar "CLAYER" sCurLayer))
          ((= commRate "DIMCONTINUE") (setvar "CLAYER" sCurLayer))
          ((= commRate "TOLERANCE") (setvar "CLAYER" sCurLayer))
          ((= commRate "RAY") (setvar "CLAYER" sCurLayer))
          ((= commRate "XLINE") (setvar "CLAYER" sCurLayer))
          ((wcmatch commRate "*HATCH") (setvar "CLAYER" sCurLayer))
          ((= commRate "INSERT") (setvar "CLAYER" sCurLayer))
          ((= commRate "IMAGE") (setvar "CLAYER" sCurLayer))
          ((= commRate "INSERTOBJ") (setvar "CLAYER" sCurLayer))
          ((wcmatch commRate "*TEXT") (setvar "CLAYER" sCurLayer))
          ((= commRate "MVIEW") (setvar "CLAYER" sCurLayer))
          ((= commRate "VPORTS") (setvar "CLAYER" sCurLayer))
        )
        (princ)
      )

      (TraceIsOn ".26")

      ;;;----------------
      ;;;---Move BackUps.
      ;;;----------------
      (defun BakFolder (/)
        (TraceIsOn ".BakFolder")
        (setq sNewpath (dos_getdir "Select folder for backup files"))
        (if (= sNewpath nil)
          (setq sNewpath "nil")
        )
        (dos_setini "CustomSettings" "BAKFolder" sNewpath Hshfile3)
        (princ)
      )

      (defun BakRelocate (calling-reactor commandInfo / dwgname filesize)
        (TraceIsOn ".BakRelocate")
        (setq
          sRelFold (dos_getini "CustomSettings" "BAKFolder" Hshfile3)
        )
        (if (and (/= sRelFold nil)
                 (/= sRelFold "nil")
                 (/= sRelFold "Nil")
            )
          (progn
            (setq dwgname    (cadr commandInfo)
              Dessinname (getvar "DWGNAME")
              Dessinpfix (getvar "DWGPREFIX")
              Dessinpath (strcat Dessinpfix Dessinname)
              Dessinpath (RepAllStrings Dessinpath "\\" "/")
              sFile2Move (strrep Dessinname ".dwg" ".bak")
            )
            (setq sNewpath sRelFold)
            (dos_move (strcat Dessinpfix sFile2Move)
                  (strcat sNewpath sFile2Move)
            )
            (princ)
          )
          (progn
            (setq dwgname    (cadr commandInfo)
              Dessinname (getvar "DWGNAME")
              Dessinpfix (getvar "DWGPREFIX")
              Dessinpath (strcat Dessinpfix Dessinname)
              Dessinpath (RepAllStrings Dessinpath "\\" "/")
              sNewpath   Dessinpath
            )
          )
        )
      )

      (if *BAKObjReacteur* (vlr-remove *BAKObjReacteur*))
      (setq *BAKObjReacteur*
        (vlr-dwg-Reactor nil '((:vlr-saveComplete . BakRelocate)))
      )

      ;;;----------------
      ;;; Dwg Properties.
      ;;;----------------
      (defun PropsRead (/ ListeX)

        (defun Dxf999 (gc999) (cdr (assoc gc999 ListeX)))

        (setq ListeX (dictsearch (namedobjdict) "DWGPROPS"))
        (setq sTitl    (Dxf999 2)
              sSubj    (Dxf999 3)
              sAuth    (Dxf999 4)
              sComm    (Dxf999 6)
              sKeyw    (Dxf999 7)
              sLastSavedBy (Dxf999 8)
              sRevisionNo  (Dxf999 9)
              sCust0       (Dxf999 300)
              sCust1       (Dxf999 301)
              sCust2       (Dxf999 302)
              sCust3       (Dxf999 303)
              sCust4       (Dxf999 304)
              sCust5       (Dxf999 305)
              sCust6       (Dxf999 306)
              sCust7       (Dxf999 307)
              sCust8       (Dxf999 308)
              sCust9       (Dxf999 309)
        )
        ListeX
        (if (= (Dxf 2 ListeX) nil)
          (setq sTitle "")
          (setq sTitle (Dxf 2 ListeX))
        )
        (if (= (Dxf 3 ListeX) nil)
          (setq sSubject "")
          (setq sSubject (Dxf 3 ListeX))
        )
        (if (= (Dxf 4 ListeX) nil)
          (setq sAuthor "")
          (setq sAuthor (Dxf 4 ListeX))
        )
        (if (= (Dxf 7 ListeX) nil)
          (setq sKeywords "")
          (setq sKeywords (Dxf 7 ListeX))
        )
        (if (= (Dxf 6 ListeX) nil)
          (setq sComments "")
          (setq sComments (Dxf 6 ListeX))
        )
      )

      (defun PropsWrite (/ ListeX)
        (TraceIsOn ".PropsWrite")
        ;;  remove any existing Properties
        (dictremove (namedobjdict) "DWGPROPS")
        (if (= sLastSavedBy nil)
          (setq sLastSavedBy (getvar "LOGINNAME"))
        )
        (if (= sRevisionNo nil)
          (setq sRevisionNo "")
        )
        (if (= sCust0 nil)
          (setq sCust0 "")
        )
        (if (= sCust1 nil)
          (setq sCust1 "")
        )
        (if (= sCust2 nil)
          (setq sCust2 "")
        )
        (if (= sCust3 nil)
          (setq sCust3 "")
        )
        (if (= sCust4 nil)
          (setq sCust4 "")
        )
        (if (= sCust5 nil)
          (setq sCust5 "")
        )
        (if (= sCust6 nil)
          (setq sCust6 "")
        )
        (if (= sCust7 nil)
          (setq sCust7 "")
        )
        (if (= sCust8 nil)
          (setq sCust8 "")
        )
        (if (= sCust9 nil)
          (setq sCust9 "")
        )
        ;; make data list
        (setq ListeX (list '(0 . "XRECORD")
                   '(100 . "AcDbXrecord")
                   '(1 . "DWGPROPS COOKIE")
                   (cons 2 sTitle)
                   (cons 3 sSubject)
                   (cons 4 sAuthor)
                   (cons 6 sComments)
                   (cons 7 sKeywords)
                   (cons 8 sLastSavedBy)
                   (cons 9 sRevisionNo)
                   (cons 300 sCust0)
                   (cons 301 sCust1)
                   (cons 302 sCust2)
                   (cons 303 sCust3)
                   (cons 304 sCust4)
                   (cons 305 sCust5)
                   (cons 306 sCust6)
                   (cons 307 sCust7)
                   (cons 308 sCust8)
                   (cons 309 sCust9)
                   (cons 40 (getvar "TDINDWG"))
                   (cons 41 (getvar "TDCREATE"))
                   (cons 42 (getvar "TDUPDATE"))
                 )
        )
        ;;  make Xrecord and add to NOD
        (dictadd (namedobjdict) "DWGPROPS" (entmakex ListeX))
      )

      ;;;---
      ;Remove reactors by name.
      (defun Reactor_Remove (Nam / ReactorsInDwg ReactorGroup ReactorObject)
        (TraceIsOn ".Reactor_Remove")
        (setq ReactorsInDwg (VLR-Reactors))
        (foreach ReactorGroup ReactorsInDwg
          (foreach ReactorObject (cdr ReactorGroup)
            (if (= (VLR-Data ReactorObject) Nam)
              ;Returns application-specific data associated with a reactor.
              (VLR-Remove ReactorObject) ;Disables a reactor, does not delete it.
            )
          )
        )
      )

      (TraceIsOn ".27")

      ;;;------------------------------
      ;;; LTScale switch with Reactors.
      ;;;------------------------------
      ;; set on 1 Viewport scaling governs linetype scaling. If TILEMODE is set to 0, dash lengths are
      ;; based on paper space drawing units, even for objects in model space. In this mode, viewports
      ;; can have varying magnifications, yet display linetypes identically.
      (setq *VLxx_AcadObject*      nil
            *VLxx_activedocument*  nil
            *VLxx_modelspace*      nil
            cdms                   (getvar "DIMSCALE")
            cpnt                   (list cdms cdms 0.0)
            chgt                   (* 2.0 cdms)
      )

      (VLxx_AcadObject)
      (VLxx_activedocument)
      (VLxx_modelspace)

      (defun Rajouter (/ acadapp acaddoc mspace)
        (vla-addText
          (VLxx_modelspace)
          "PSLtscale"
          (vlax-3d-point cpnt)
          chgt
        )
      )

      (defun MiseAJour4LTS (txtobj / clog cpfx cdwg)
        (TraceIsOn ".MiseAJour4LTS")
        ;;(Reactor_Remove "DSU4Lts")                  ;Not a good idea here!
        (setq CLOG  (menucmd "M=$(getvar,loginname)")
              cdate ""
        )
        (if (= 0 (logand 1 (getvar "tilemode")))
                        ;if 0 then PSpace environment
          (progn        ;donc si espace papier uniquement
            (cond ((= (getvar "PSLTSCALE") 1)
               ;;et si psltscale = 1 [traits tires egaux]
               ;;(setvar "LTSCALE" rLTS4PS)
               ;;(vla-setvariable *active-document* "PSLTSCALE" 1)
               (vla-setvariable *active-document* "LTSCALE" rLTS4PS)
              )
              ((= (getvar "PSLTSCALE") 0) ;ou si psltscale = 0
               (vla-setvariable
                 *active-document*
                 "LTSCALE"
                 (getvar "USERR2")
               )
              )
            )
          )
          (vla-setvariable
            *active-document*
            "LTSCALE"
            (getvar "USERR2")
          )             ;if 1 then MSpace environment
        )
        (cond ((or (not txtobj) (vlax-erased-p txtobj))
               (setq txtobj (Rajouter))
              )
        )
        (vla-put-TextString txtobj cdate)
        txtobj
      )

      (defun ToDict (txtobj)
        (vlax-ldata-put "PlotScaling" "TXTOBJ" txtobj)
      )

      (defun FromDict () (vlax-ldata-get "PlotScaling" "TXTOBJ"))

      (defun OnDemandforSpace (Call_Reactor CommandList / txtobject)
        (if (= bBoxLTS T)
          (progn
            (cond ((or (member "PSPACE" CommandList)
                        ;si commande espace papier est demandee
                   (member "MSPACE" CommandList)
                        ;si commande espace modele est demandee
                   (member "ZOOM" CommandList)
                        ;si commande est demandee
                   (member "PAN" CommandList)
                        ;si commande est demandee
               )
               (VLxx_ActiveSpace)
               (setq txtobject (FromDict)
                        ;chercher dans dictionnaire
                 txtobject (MiseAJour4LTS txtobject)
                        ;faire la mise a jour.
               )
               (ToDict txtobject)   ;mettre dans dictionnaire.
              )
            )
          )
        )
      )

      (if *TxtObjReac* (vlr-remove *TxtObjReac*))
      (if *MiscReac*   (vlr-remove *MiscReac*))
      (if *LTSReac*    (vlr-remove *LTSReac*))

      (setq *TxtObjReac* (vlr-Editor-reactor
                           nil
                           '((:vlr-commandEnded . OnDemandforSpace)
                             (:vlr-beginsave    . zall)
                             (:vlr-savecomplete . zp)
                            )
                         )
            *MiscReac*   (vlr-miscellaneous-reactor
                            nil
                           '((:vlr-layoutSwitched . SwitchLayouts))
                         )
            *LTSReac*    (vlr-Sysvar-Reactor
                            nil
                           '((:vlr-sysVarChanged . LtsGet))
                         )
      )

      ;;;------------------------
      ;;;---Zoom All before save.
      ;;;------------------------
      (defun ZALL (Reactor Info)
        (if (= bZSv T)
          (vla-zoomall (vlax-get-acad-object))
        )
      )

      (defun ZP (Reactor Info)
        (if (= bZSv T)
          (vla-zoomprevious (vlax-get-acad-object))
        )
      )

      (defun LtsGet (Reactor Info)
        (if (= (car Info) "LTSCALE")
          (progn (if (and (= bBoxLTS T) (eq (getvar "CTAB") "Model"))
               (progn (setvar "USERR2" (getvar "LTSCALE")))
             )
          )
        )
      )

      (defun SwitchLayouts (Reactor Info)
        (if (= bBoxLTS T)
          (progn (VLxx_ActiveSpace)
             (setq txtobject (FromDict)             ;chercher dans dictionnaire
                   txtobject (MiseAJour4LTS txtobject)  ;faire la mise a jour.
             )
             (ToDict txtobject)             ;mettre dans dictionnaire.
             (vla-sendcommand *active-document* "_.REGENALL ")
          )
        )
        (princ)
      )

      (TraceIsOn ".28")

      ;----------------
      ; Smart Hatching.
      ;----------------
      (defun C:HBK () (BkHatch))
      (defun C:HBL () (BlHatch))
      (defun C:HCC () (CnHatch))
      (defun C:HA  () (AlHatch))
      (defun C:HSO () (AsHatch))
      (defun C:HPT () (PHatch))
      (defun C:HTM () (THatch))

      (if (= (wcmatch (getvar "ACADVER") "*15*") T)
        (progn                            ;For AutoCAD 2000-2002

          (defun LineTAdd ()
            (setq Hshfil2 (findfile "acad.lin")
              fil1    nil
            )
            (setq Hshfil3 (car (strfind Hshfil2 "acad.lin")))
            (setq Hshfil4 (dos_getfiled
                    "Select Linetype File"
                    Hshfil3
                    "Linetype (*.lin)|*.lin||"
                  )
            )
            (setq file1  (strcat chemin "MetricArchAssist.ini")
              sOrigLType (strrep Hshfil4 "lin" "olt")
            )
            (dos_setini "Locations" "OrigLType" sOrigLType file1)
            (dos_copy Hshfil4 sOrigLType)             ;Garde original
            (dos_setini "Locations" "NewLType" Hshfil4 file1)
            (setq fil5 (open Hshfil4 "a"))

            (if (/= fil5 nil)
              (progn
                (if (/= (setq fil1 (open (strcat Chemin "AALType.lin") "r"))
                        nil
                    )
                  (progn (while (setq ligne (read-line fil1)) (wlf ligne fil5))
                    (close fil1)
                  )
                )
                (setq bLT T)
              )
            )

            (close fil5)
            (Odcl_Control_ForceUpdateNow Walls_AACfg_Label2)
          )

          (defun HatchPAdd ()
            (setq Hshfil2 (findfile "acad.pat")
              fil1    nil
            )
            (setq Hshfil3 (car (strfind Hshfil2 "acad.pat")))     ;Hshfil4 Quel nom si different du defaut?
            (setq Hshfil4 (dos_getfiled
                    "Select Hatch file"
                    Hshfil3
                    "Hatch Patterns (*.pat)|*.pat||"
                  )
            )
            (setq file1      (strcat chemin "MetricArchAssist.ini")
              sOrigHatch (strrep Hshfil4 "pat" "olh")
            )
            (dos_setini "Locations" "OrigHatch" sOrigHatch file1)
            (dos_copy Hshfil4 sOrigHatch)             ;Garde original
            (dos_setini "Locations" "NewHatch" Hshfil4 file1)
            (setq fil5 (open Hshfil4 "a"))

            (if (/= fil5 nil)
              (progn
                (if (/= (setq fil1 (open (strcat Chemin "AAHatch.pat") "r"))
                        nil
                    )
                  (progn (while (setq ligne (read-line fil1)) (wlf ligne fil5))
                     (close fil1)
                  )
                )
                (setq bHP T)
              )
            )

            (close fil5)
          )

          (defun c:AACfg_LTTextButton_OnClicked ()
            (setq bLT nil)
            (LineTAdd)
            (princ)
          )

          (defun c:AACfg_ResetHTextButton_OnClicked ()
            (setq file1    (strcat chemin "MetricArchAssist.ini")
                  sPar     (dos_getini "Locations" "OrigHatch" file1)
                  sReplace (dos_getini "Locations" "NewHatch" file1)
            )
            (dos_delete sReplace)
            (dos_rename sPar (strrep sPar "olh" "pat"))
            (dos_setini "Locations" "OrigHatch" "Nil" file1)
            (dos_setini "Locations" "NewHatch" "Nil" file1)
          )

          (defun c:AACfg_ResetLTTextButton_OnClicked ()
            (setq file1    (strcat chemin "MetricArchAssist.ini")
              sPar     (dos_getini "Locations" "OrigLType" file1)
              sReplace (dos_getini "Locations" "NewLType" file1)
            )
            (dos_delete sReplace)
            (dos_rename sPar (strrep sPar "olt" "lin"))
            (dos_setini "Locations" "OrigLType" "Nil" file1)
            (dos_setini "Locations" "NewLType" "Nil" file1)
          )

          (defun c:AACfg_HatchTextButton_OnClicked ()
            (setq bHP nil)
            (HatchPAdd)
            (princ)
          )

          (defun c:AACfg_OnInitialize ()
            (if (= (getvar "MeasureInit") 0)              ;si dans le registre = 0
              (progn
                (if (and (= bImpUnit nil) (= bMetUnit nil))
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_AACfg_ImperialOption
                      "Value"
                      T
                    )                         ;c'est en imp.
                    (Odcl_Control_SetProperty
                      Walls_AACfg_MetricOption
                      "Value"
                      nil
                    )
                  )
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_AACfg_ImperialOption
                      "Value"
                      bImpUnit
                    )
                    (Odcl_Control_SetProperty
                      Walls_AACfg_MetricOption
                      "Value"
                      bMetUnit
                    )
                  )
                )
              )
              (progn
                (if (and (= bImpUnit nil) (= bMetUnit nil))
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_AACfg_ImperialOption
                      "Value"
                      nil
                    )
                    (Odcl_Control_SetProperty
                      Walls_AACfg_MetricOption
                      "Value"
                      T
                    )                         ;sinon c'est du metrique
                  )
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_AACfg_ImperialOption
                      "Value"
                      bImpUnit
                    )
                    (Odcl_Control_SetProperty
                      Walls_AACfg_MetricOption
                      "Value"
                      bMetUnit
                    )
                  )
                )
              )
            )

            (if bHP
              (Odcl_Control_SetProperty Walls_AACfg_Label2 "Visible" T)
              (Odcl_Control_SetProperty Walls_AACfg_Label2 "Visible" nil)
            )
            (if bLT
              (Odcl_Control_SetProperty Walls_AACfg_Label3 "Visible" T)
              (Odcl_Control_SetProperty Walls_AACfg_Label3 "Visible" nil)
            )
          )

          (defun c:AACfg_OnOK ()
            (Odcl_Form_Close Walls_AACfg)

            (if bHP
              (Odcl_Control_SetProperty
                Walls_AACfg_HatchTextButton
                "Enabled"
                nil
              )
            )
            (if bLT
              (Odcl_Control_SetProperty
                Walls_AACfg_LTTextButton
                "Enabled"
                nil
              )
            )
            (Setq bMetUnit (Odcl_Control_GetProperty Walls_AACfg_MetricOption "Value"))  ;T ou nil
            (Setq bImpUnit
                  (Odcl_Control_GetProperty
                     Walls_AACfg_ImperialOption
                    "Value"
                  )
            )
            (if bMetUnit
              (setq bImpUnit nil)
            )
            (if bImpUnit
              (setq bMetUnit nil)
            )
          )

        )
      )

      (if (= (wcmatch (getvar "ACADVER") "16*") T)
        (progn                            ;For AutoCAD 2004-2006

          (defun LineTAdd ()
            (setq Hshfil2 (findfile "acad.lin")
                  fil1    nil
            )
            (setq Hshfil3 (car (strfind Hshfil2 "acad.lin")))
            (setq Hshfil4 (dos_getfiled
                            "Select Linetype File"
                            Hshfil3
                            "Linetype (*.lin)|*.lin||"
                          )
            )
            (setq file1      (strcat chemin "MetricArchAssist2004.ini")
                  sOrigLType (strrep Hshfil4 "lin" "olt")
            )
            (dos_setini "Locations" "OrigLType" sOrigLType file1)
            (dos_copy Hshfil4 sOrigLType)             ;Garde original
            (dos_setini "Locations" "NewLType" Hshfil4 file1)
            (setq fil5 (open Hshfil4 "a"))

            (if (/= fil5 nil)
              (progn
                (if (/= (setq fil1 (open (strcat Chemin "AALType.lin") "r"))
                        nil
                    )
                  (progn
                    (while (setq ligne (read-line fil1)) (wlf ligne fil5))
                    (close fil1)
                  )
                )
                (setq bLT T)
              )
            )

            (close fil5)
            (Odcl_Control_ForceUpdateNow Walls_RelSixteen_Label2)
          )

          (defun HatchPAdd ()
            (setq Hshfil2 (findfile "acad.pat")
                  fil1    nil
            )
            (setq Hshfil3 (car (strfind Hshfil2 "acad.pat")))     ;Hshfil4 Quel nom si different du defaut?
            (setq Hshfil4 (dos_getfiled
                            "Select Hatch file"
                            Hshfil3
                            "Hatch Patterns (*.pat)|*.pat||"
                          )
            )
            (setq file1      (strcat chemin "MetricArchAssist2004.ini")
                  sOrigHatch (strrep Hshfil4 "pat" "olh")
            )
            (dos_setini "Locations" "OrigHatch" sOrigHatch file1)
            (dos_copy Hshfil4 sOrigHatch)             ;Garde original
            (dos_setini "Locations" "NewHatch" Hshfil4 file1)
            (setq fil5 (open Hshfil4 "a"))

            (if (/= fil5 nil)
              (progn
                (if (/= (setq fil1 (open (strcat Chemin "AAHatch.pat") "r"))
                        nil
                    )
                  (progn
                    (while (setq ligne (read-line fil1)) (wlf ligne fil5))
                    (close fil1)
                  )
                )
                (setq bHP T)
              )
            )

            (close fil5)
          )

          (defun c:RelSixteen_LTTextButton_OnClicked ()
            (setq bLT nil)
            (LineTAdd)
            (princ)
          )

          (defun c:RelSixteen_ResetHTextButton_OnClicked ()
            (setq file1    (strcat chemin "MetricArchAssist2004.ini")
                  sPar     (dos_getini "Locations" "OrigHatch" file1)
                  sReplace (dos_getini "Locations" "NewHatch" file1)
            )
            (dos_delete sReplace)
            (dos_rename sPar (strrep sPar "olh" "pat"))
            (dos_setini "Locations" "OrigHatch" "Nil" file1)
            (dos_setini "Locations" "NewHatch" "Nil" file1)
          )

          (defun c:RelSixteen_ResetLTTextButton_OnClicked ()
            (setq file1    (strcat chemin "MetricArchAssist2004.ini")
                  sPar     (dos_getini "Locations" "OrigLType" file1)
                  sReplace (dos_getini "Locations" "NewLType" file1)
            )
            (dos_delete sReplace)
            (dos_rename sPar (strrep sPar "olt" "lin"))
            (dos_setini "Locations" "OrigLType" "Nil" file1)
            (dos_setini "Locations" "NewLType" "Nil" file1)
          )

          (defun c:RelSixteen_HatchTextButton_OnClicked ()
            (setq bHP nil)
            (HatchPAdd)
            (princ)
          )

          (defun c:RelSixteen_OnInitialize ()
            (if (= (getvar 'MeasureInit) 0)               ;si dans le registre = 0
              (progn
                (if (and (= bImpUnit nil) (= bMetUnit nil))
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_ImperialOption
                      "Value"
                      T
                    )                         ;c'est en imp.
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_MetricOption
                      "Value"
                      nil
                    )
                  )
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_ImperialOption
                      "Value"
                      bImpUnit
                    )
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_MetricOption
                      "Value"
                      bMetUnit
                    )
                  )
                )
              )
              (progn
                (if (and (= bImpUnit nil) (= bMetUnit nil))
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_ImperialOption
                      "Value"
                      nil
                    )
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_MetricOption
                      "Value"
                      T
                    )                         ;sinon c'est du metrique
                  )
                  (progn
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_ImperialOption
                      "Value"
                      bImpUnit
                    )
                    (Odcl_Control_SetProperty
                      Walls_RelSixteen_MetricOption
                      "Value"
                      bMetUnit
                    )
                  )
                )
              )
            )

            (if bHP
              (Odcl_Control_SetProperty Walls_RelSixteen_Label2 "Visible" T)
              (Odcl_Control_SetProperty Walls_RelSixteen_Label2 "Visible" nil)
            )
            (if bLT
              (Odcl_Control_SetProperty Walls_RelSixteen_Label3 "Visible" T)
              (Odcl_Control_SetProperty Walls_RelSixteen_Label3 "Visible" nil)
            )
          )

          (defun c:RelSixteen_Ok_OnClicked ()
            (Odcl_Form_Close Walls_RelSixteen)

            (if bHP
              (Odcl_Control_SetProperty
                Walls_RelSixteen_HatchTextButton
                "Enabled"
                nil
              )
            )
            (if bLT
              (Odcl_Control_SetProperty
                Walls_RelSixteen_LTTextButton
                "Enabled"
                nil
              )
            )
            (Setq bMetUnit (Odcl_Control_GetProperty Walls_RelSixteen_MetricOption "Value") ) ;T ou nil
            (Setq bImpUnit
                    (Odcl_Control_GetProperty
                      Walls_RelSixteen_ImperialOption
                      "Value"
                    )
            )
            (if bMetUnit
              (setq bImpUnit nil)
            )
            (if bImpUnit
              (setq bMetUnit nil)
            )
          )
        )
      )

      (defun AsHatch (/ )
        (setq olderror *error*  *error* Wallserror  ss1 nil)
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nSolid Hatching." copyright)
          (strcat "\nSolid Hatching." copyright)
          (strcat "\nSolid Hatching." copyright)
          (strcat "\nSolid Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (command "_.SELECT")
        (while (= (logand (getvar "cmdactive") 1) 1) (command pause))
        (setq bPHat bHat  bHat nil)
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq ss1 (ssget "_P" (list (cons -4 "<OR")
                                    (cons -4 "<AND")
                                    (cons 8 (strcat (substr sBrkWall1Lyr 1 5) "*"))
                                    (cons -4 "AND>")
                                    (cons -4 "<AND")
                                    (cons 8 (strcat (substr sBlkWall1Lyr 1 5) "*"))
                                    (cons -4 "AND>")
                                    (cons -4 "<AND")
                                    (cons 8 (strcat (substr sConWall1Lyr 1 5) "*"))
                                    (cons -4 "AND>")
                                    (cons -4 "<AND")
                                    (cons 8 (strcat (substr sParWall1Lyr 1 5) "*"))
                                    (cons -4 "AND>")
                                    (cons -4 "<AND")
                                    (cons 8 (strcat (substr sTmbWall1Lyr 1 5) "*"))
                                    (cons -4 "AND>")
                                    (cons -4 "OR>")
                              )
                  )
        )
        (if ss1
          (progn
           ;;;(setq 1ss (ak_ssget ss1))
           (command ".-BHATCH" "_P" "SOLID" "1" "0" "_S" ss1 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil) (resset varlst)
      )

      (defun AlHatch (/)
        (setq olderror *error*
              *error*  Wallserror
              ss1  nil
              ss2  nil
              ss3  nil
        )
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nSmart Hatching." copyright)
          (strcat "\nSmart Hatching." copyright)
          (strcat "\nSmart Hatching." copyright)
          (strcat "\nSmart Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (setq pC1   (upoint 1 "" "Specify first corner" nil nil)
          pC2   (GetCorner pC1 "\nSpecify opposite corner:")
          bPHat bHat
          bHat  nil
        )
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq
          ss1 (ssget "_C"
                 pC1
                 pC2
                 (list (cons 8 (strcat (substr sBrkWall1Lyr 1 5) "*")))
          )
        )
        (if ss1
          (progn
            (command ".-BHATCH"    "_A"     "_S"     "_I"     ""
                 "_P" "AA-BRICK"    "1.5"    "5"      "_S"
                 ss1  ""   ""
                )
          )
        )
        (setq
          ss2 (ssget "_C"
                 pC1
                 pC2
                 (list (cons 8 (strcat (substr sBlkWall1Lyr 1 5) "*")))
          )
        )
        (if ss2
          (progn
            (command ".-BHATCH" "_P" "_U" "48" "250" "_N" "_S" ss2 "" "")
          )
        )
        (setq
          ss3 (ssget "_C"
                 pC1
                 pC2
                 (list (cons 8 (strcat (substr sConWall1Lyr 1 5) "*")))
          )
        )
        (if ss3
          (progn
            (command ".-BHATCH" "_P" "AR-CONC" "1.3" "6" "_S" ss3 "" "")
          )
        )
        (setq
          ss4 (ssget "_C"
                 pC1
                 pC2
                 (list (cons 8 (strcat (substr sParWall1Lyr 1 5) "*")))
          )
        )
        (if ss4
          (progn
            (command ".-BHATCH" "_A" "_S" "_I" "" "_P" "NET" "15" "45" "_S"
                 ss4 "" "")
          )
        )
        (setq
          ss5 (ssget "_C"
                 pC1
                 pC2
                 (list (cons 8 (strcat (substr sTmbWall1Lyr 1 5) "*"))
                   (cons 8 (strcat (substr sTmbWall2Lyr 1 5) "*"))
                 )
          )
        )
        (if ss5
          (progn
            (command ".-BHATCH" "_A" "_S" "_I" "" "_P" "DOTS" "18" "45" "_S"
                 ss5 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun BkHatch (/)                  ;Brick
        (setq olderror *error*
              *error*  Wallserror
              ss1  nil
        )
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nBrick Hatching." copyright)
          (strcat "\nBrick Hatching." copyright)
          (strcat "\nBrick Hatching." copyright)
          (strcat "\nBrick Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (command "_.SELECT")
        (while (= (logand (getvar "cmdactive") 1) 1)
          (command pause)
        )
        (setq bPHat bHat
          bHat  nil
        )
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq
          ss1 (ssget "_P"
                 (list (cons 8 (strcat (substr sBrkWall1Lyr 1 5) "*")))
          )
        )
        (if ss1
          (progn
                          ;     (setq 1ss (ak_ssget ss1))
            (command ".-BHATCH" "_A" "_S" "_I" "" "_P" "AA-BRICK" "1.5" "5" "_S" ss1 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun BlHatch (/ ss1)          ;Block
        (setq olderror *error*
              *error*  Wallserror
              ss1  nil
        )
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nBlock Hatching." copyright)
          (strcat "\nBlock Hatching." copyright)
          (strcat "\nBlock Hatching." copyright)
          (strcat "\nBlock Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (command "_.SELECT")
        (while (= (logand (getvar "cmdactive") 1) 1)
          (command pause)
        )
        (setq bPHat bHat
          bHat  nil
        )
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq
          ss1 (ssget "_P"
                 (list (cons 8 (strcat (substr sBlkWall1Lyr 1 5) "*")))
          )
        )
        (if ss1
          (progn
                          ;     (setq 1ss (ak_ssget ss1))
            (command ".-BHATCH" "_P" "_U" "48" "250" "_N" "_S" ss1 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun CnHatch (/)          ;Concrete
        (setq olderror *error*
              *error*  Wallserror
              ss1  nil
        )
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nConcrete Hatching." copyright)
          (strcat "\nConcrete Hatching." copyright)
          (strcat "\nConcrete Hatching." copyright)
          (strcat "\nConcrete Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (command "_.SELECT")
        (while (= (logand (getvar "cmdactive") 1) 1)
          (command pause)
        )
        (setq bPHat bHat
          bHat  nil
        )
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq
          ss1 (ssget "_P"
                 (list (cons 8 (strcat (substr sConWall1Lyr 1 5) "*")))
          )
        )
        (if ss1
          (progn
            (command ".-BHATCH" "_P" "AR-CONC" "1.3" "6" "_S" ss1 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun PHatch (/)           ;Partition
        (setq olderror *error*
              *error*  Wallserror
              ss1  nil
        )
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nPartition Hatching." copyright)
          (strcat "\nPartition Hatching." copyright)
          (strcat "\nPartition Hatching." copyright)
          (strcat "\nPartition Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (command "_.SELECT")
        (while (= (logand (getvar "cmdactive") 1) 1)
          (command pause)
        )
        (setq bPHat bHat
          bHat  nil
        )
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq
          ss1 (ssget "_P"
                 (list (cons 8 (strcat (substr sParWall1Lyr 1 5) "*")))
          )
        )
        (if ss1
          (progn
            (command ".-BHATCH" "_A" "_S" "_I" "" "_P" "NET" "15" "45" "_S"
                 ss1 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun THatch (/)           ;Timber
        (setq olderror *error*
             *error*  Wallserror
             ss1  nil
             ss2  nil
        )
        (Wallsgather_info)
        (WsFlashMess2
          (strcat "\nTimber Hatching." copyright)
          (strcat "\nTimber Hatching." copyright)
          (strcat "\nTimber Hatching." copyright)
          (strcat "\nTimber Hatching." copyright)
        )
        (princ "\nSelect objects: ")
        (command "_.SELECT")
        (while (= (logand (getvar "cmdactive") 1) 1)
          (command pause)
        )
        (setq bPHat bHat
          bHat  nil
        )
        (Hshset_lay sHatch1Lyr sHatch1Col "CONTINUOUS" sHatch1Lwt)
        (setq
          ss1 (ssget "_P"
                 (list (cons 8 (strcat (substr sTmbWall1Lyr 1 5) "*"))
                   (cons 8 (strcat (substr sTmbWall2Lyr 1 5) "*"))
                 )
          )
        )
        (if ss1
          (progn
            (command ".-BHATCH" "_A" "_S" "_I" "" "_P" "DOTS" "18" "45" "_S"
                 ss1 "" "")
          )
        )
        (setq bHat bPHat)
        (setq *error* OLDERROR)
        (reset nil)
        (resset varlst)
      )

      (defun C:AAHist ()
        (Odcl_Form_Show Walls_History)
        (princ)
      )

    )

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    (Beurre)

    ;;(AAMenu "Unload")
    (AAMenu "Reload")

    (if bOpt
      (progn
        (vl-registry-write
          (strcat "HKEY_CURRENT_USER\\"
                  (vlax-product-key)
                  "\\RestoreAALib"
          )
          "Value"
          "Cache"
        )
        (vl-registry-write
          (strcat "HKEY_CURRENT_USER\\"
                  (vlax-product-key)
                  "\\RestoreLView"
          )
          "Value"
          "Cache"
        )
        (if (= (wcmatch (getvar "ACADVER") "*15*") T)
          (progn
            (dos_msgbox
              (strcat
                "After this message, the Options dialogue box will display. Select ArchAssist tab [far right]"
                "\nthen specify location for requested files. These crucial settings to the proper operating"
                "\nof the software need be done at first Installation only. This message will not re-appear."
              )
              "ArchAssist Message"
              1
              4
            )
            (command "_.OPTIONS")
          )
        )
        (if (= (wcmatch (getvar "ACADVER") "16*") T)
          (progn
            (dos_msgbox
              (strcat
                "After this message, a dialogue box will display. Click on each button and then"
                "\nspecify location for requested files. These crucial settings to the proper operating"
                "\nof the software need be done at first Installation only. This message will not re-appear."
              )
              "ArchAssist Message"
              1
              4
            )
            (Odcl_Form_Show Walls_RelSixteen)
          )
        )
      )
    )

    (FlashMess3
      "ArchAssist maintenant Actif."
      "ArchAssist now Active."
      "ArchAssist now Active."
      "ArchAssist now Active."
    )

)

(princ)

